//========================================================================
// GLFW - An OpenGL framework
// File:     internal.h
// Platform: X11 (Unix)
// Version:  2.3
// Date:     2002.11.23
// Author:   Marcus Geelnard (marcus.geelnard@home.se)
// WWW:      http://hem.passagen.se/opengl/glfw/
//------------------------------------------------------------------------
// Copyright (c) 2002 Marcus Geelnard
//
// GLFW is provided "as-is", without any express or implied warranty,
// without even the implied warranty of fitness for a particular purpose.
// In no event will the author be held liable for any damage araising from
// the use of GLFW.
//
// Permission is granted to anyone to use GLFW for any purpose, including
// commercial applications, and to alter it and redistribute it freely,
// subject to the following restrictions:
//
// 1. You must not claim that you wrote GLFW.
// 2. If you modify any parts of GLFW, it must be clearly stated which
//    modifications have been made, and that those modifications are not
//    the work of the author of GLFW.
// 3. If you use GLFW in a product, an acknowledgment in the product
//    documentation would be appreciated, but is not required.
// 4. This notice may not be removed or altered from any source
//    distribution of GLFW.
//
// Marcus Geelnard
// marcus.geelnard@home.se
//========================================================================


// Include files
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <X11/Xlib.h>
#include <X11/keysym.h>
#include <GL/glx.h>
#include <GL/gl.h>
#include <sys/time.h>
#include <signal.h>
#include <unistd.h>
#include "../glfw.h"
#include "glfw_dyn.h"

// With XFree86, we can use the XF86VidMode extension
#if defined( GLFW_USE_XF86VIDMODE )
#include <X11/extensions/xf86vmode.h>
#endif

// IRIX hardware counter support
#ifdef __sgi
#include <time.h>
#include <sys/syssgi.h>
#endif // __sgi

// Use a single constant for querying number of online processors
// (e.g. SGI defines _SC_NPROC_ONLN instead of _SC_NPROCESSORS_ONLN)
#ifndef _SC_NPROCESSORS_ONLN
#ifdef  _SC_NPROC_ONLN
#define _SC_NPROCESSORS_ONLN _SC_NPROC_ONLN
#else
#error POSIX constant _SC_NPROCESSORS_ONLN not defined!
#endif
#endif // _SC_NPROCESSORS_ONLN

// Under my Linux installation, GLX_ARB_get_proc_address is not defined,
// but glXGetProcAddress() works. Therefor GLX_ARB_get_proc_address is
// defined here. If this gives you compiler errors, uncomment the
// following lines:
#ifndef GLX_ARB_get_proc_address
#define GLX_ARB_get_proc_address 1
#endif // GLX_ARB_get_proc_address



//========================================================================
// Global variables (GLFW internals)
//========================================================================

#if defined( _init_c_ )
#define GLFWGLOBAL
#else
#define GLFWGLOBAL extern
#endif


// Flag indicating if glfw has been initialised
#if defined( _init_c_ )
GLFWGLOBAL int _glfwInitialised = GL_FALSE;
#else
GLFWGLOBAL int _glfwInitialised;
#endif


//------------------------------------------------------------------------
// Window structure
//------------------------------------------------------------------------
typedef struct _GLFWwin_struct _GLFWwin;

struct _GLFWwin_struct {
    // Window states
    int         Opened;          // True if window is opened
    int         Iconified;       // True if window is iconified
    int         Active;          // True if window is active (in focus)

    // User selected window settings
    int         Fullscreen;      // Fullscreen flag
    int         MouseLock;       // Mouse-lock flag

    // Window resources
    Window      Win;             // Window
    Display     *Dpy;            // Display
    XVisualInfo *VI;             // Visual
    GLXContext  CX;              // OpenGL rendering context
    Atom        WMDeleteWindow;  // For WM close detection

    // Window status
    int         Width, Height;   // Window width and heigth

    // Various internal variables
    int         KeyboardGrabbed; // True if keyboard is currently grabbed
    int         PointerGrabbed;  // True if pointer is currently grabbed
    int         PointerHidden;   // True if pointer is currently hidden
};

GLFWGLOBAL _GLFWwin _glfwWin;


//------------------------------------------------------------------------
// User input status (should go in _GLFWwin)
//------------------------------------------------------------------------
GLFWGLOBAL struct {
    // Mouse status
    int  MousePosX, MousePosY;
    int  WheelPos;
    char MouseButton[ GLFW_MOUSE_BUTTON_LAST+1 ];

    // Keyboard status
    char Key[ GLFW_KEY_LAST+1 ];

    // User selected settings
    int  StickyKeys;
    int  StickyMouseButtons;
    int  KeyRepeat;

    // Internal variables
    int  MouseMoved, OldMouseX, OldMouseY;
} _glfwInput;


//------------------------------------------------------------------------
// User callback functions (should go in _GLFWwin)
//------------------------------------------------------------------------
GLFWGLOBAL GLFWwindowsizefun  _glfwWindowSizeCallback;
GLFWGLOBAL GLFWmousebuttonfun _glfwMouseButtonCallback;
GLFWGLOBAL GLFWmouseposfun    _glfwMousePosCallback;
GLFWGLOBAL GLFWmousewheelfun  _glfwMouseWheelCallback;
GLFWGLOBAL GLFWkeyfun         _glfwKeyCallback;


//------------------------------------------------------------------------
// X11 buffer-swap interval control (should go in _GLFWwin)
//------------------------------------------------------------------------
typedef int ( * GLXSWAPINTERVALSGI_T) (int interval);
GLFWGLOBAL GLXSWAPINTERVALSGI_T _glXSwapIntervalSGI;


//------------------------------------------------------------------------
// Window hints (set by glfwOpenWindowHint)
//------------------------------------------------------------------------
GLFWGLOBAL struct {
    int          RefreshRate;
    int          AccumRedBits;
    int          AccumGreenBits;
    int          AccumBlueBits;
    int          AccumAlphaBits;
    int          AuxBuffers;
    int          Stereo;
} _glfwWinHints;


//------------------------------------------------------------------------
// Fullscreen info
//------------------------------------------------------------------------
GLFWGLOBAL struct {
    int         ModeChanged;
#if defined( GLFW_USE_XF86VIDMODE )
    XF86VidModeModeInfo OldMode;
#endif
} _glfwFS;


//------------------------------------------------------------------------
// Screen saver info
//------------------------------------------------------------------------
GLFWGLOBAL struct {
    int         SaverChanged;
    int         Timeout;
    int         Interval;
    int         Blanking;
    int         Exposure;
} _glfwSaver;


//------------------------------------------------------------------------
// Joystick information & state
//------------------------------------------------------------------------
GLFWGLOBAL struct {
    int           Present;
    int           fd;
    int           NumAxes;
    int           NumButtons;
    float         *Axis;
    unsigned char *Button;
} _glfwJoy[ GLFW_JOYSTICK_LAST + 1 ];


//========================================================================
// Prototypes for GLFW internal functions
//========================================================================

// Fullscreen support
int  _glfwGetClosestVideoMode( int *w, int *h, int *r, int *g, int *b );
void _glfwSetVideoModeMODE( int mode );
void _glfwSetVideoMode( int *w, int *h, int r, int g, int b );

// Cursor handling
Cursor _glfwCreateNULLCursor( Display *display, Window root );

// Joystick input
void _glfwInitJoysticks( void );
void _glfwTerminateJoysticks( void );
