#include <caml/mlvalues.h>
#include <caml/memory.h>
#include <caml/alloc.h>
#include <caml/bigarray.h>
#include <caml/fail.h>
#define gl__ba__(a) (Bigarray_val(a)->dim[0]?Data_bigarray_val(a):0)

#ifdef _WIN32
#include <windows.h>
static char* libname=0;
static HMODULE lib=0;
static void init_lib()
{
	lib = LoadLibrary(libname?libname:"opengl32.dll");
	if(lib == 0) failwith("error loading gl dll");
}

extern void* (APIENTRY *getprocaddress)(char* name);
static void* APIENTRY init_getfunc(char* name)
{
	void* f=0;
	if(lib == 0) init_lib();
	f=GetProcAddress(lib,"wglGetProcAddress");
	if(f==0) failwith("wglGetProcAddress: not found");
	getprocaddress=f;
	return getprocaddress(name);
}
void* (APIENTRY *getprocaddress)(char* name)=init_getfunc;

static void* getfunc(char* name)
{
	void* f;
	f=getprocaddress(name);
	if(f==0) f=GetProcAddress(lib,name);
	if(f==0) { static char buf[1024]; sprintf(buf,"%s: not found",name); failwith(buf); }
	return f;
}

static int testfunc(char* name)
{
	void* f=0;
	f=getprocaddress(name);
	if(f==0) f=GetProcAddress(lib,name);
	return f?1:0;
}
#endif
#ifdef __linux__
#ifndef APIENTRY
#define APIENTRY
#endif
#include <dlfcn.h>
#include <stdio.h>
static void* lib=0;
static char* libname=0;

static void init_lib()
{
	lib = dlopen(libname?libname:"libGL.so.1",RTLD_LAZY);
	if(lib == 0) failwith("error loading gl dll");
}

extern void* (APIENTRY *getprocaddress)(char* name);
static void* init_getfunc(char* name)
{
	void* f=0;
	if(lib == 0) init_lib();
	f=dlsym(lib,"glXGetProcAddressARB");
	if(f==0) f=dlsym(lib,"glXGetProcAdress");
	if(f==0) failwith("glXGetProcAddress: not found");
	getprocaddress=f;
	return getprocaddress(name);
}
void* (*getprocaddress)(char* name)=init_getfunc;

static void* getfunc(char* name)
{
	void* f=0;
	f=getprocaddress(name);
	if(f==0) f=dlsym(lib,name);
	if(f==0) { static char buf[1024]; sprintf(buf,"%s: not found",name); failwith(buf); }
	return f;
}

static int testfunc(char* name)
{
	void* f;
	f=getprocaddress(name);
	if(f==0) f=dlsym(lib,name);
	return f?1:0;
}
#endif

#include <string.h>

value ml_overload_test_function(value vs)
{
	CAMLparam1(vs);
	char* s=String_val(vs);
	value r;
	r=Val_bool(testfunc(s));
	CAMLreturn(r);
}

void ml_overload_set_lib(value vs)
{
	CAMLparam1(vs);
	char* s=String_val(vs);
	if(strlen(s)==0) failwith("set_lib: empty libname");
	if(lib) failwith("set_lib: library already loaded (fix me)");
	libname=strdup(s);
	CAMLreturn0;
}

value ml_overload_get_lib(value unit)
{
	CAMLparam0();
	CAMLlocal1(r);
#ifdef _WIN32
	r=copy_string(libname?libname:"opengl32.dll");
#elif __linux__
	r=copy_string(libname?libname:"libGL.so.1");
#endif
	CAMLreturn(r);
}
extern void (APIENTRY *ov_glTbufferMask3DFX)(unsigned int param0);

static void APIENTRY init_ov_glTbufferMask3DFX(unsigned int param0)
{
	void *f;
	f=(void*)getfunc("glTbufferMask3DFX");
	ov_glTbufferMask3DFX=f;
	ov_glTbufferMask3DFX(param0);
}
void (APIENTRY *ov_glTbufferMask3DFX)(unsigned int param0)=init_ov_glTbufferMask3DFX;

void __gl_ml_glTbufferMask3DFX(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glTbufferMask3DFX(l0);
}
extern void (APIENTRY *ov_glGetImageTransformParameterfvHP)(unsigned int param0,unsigned int param1,float* param2);

static void APIENTRY init_ov_glGetImageTransformParameterfvHP(unsigned int param0,unsigned int param1,float* param2)
{
	void *f;
	f=(void*)getfunc("glGetImageTransformParameterfvHP");
	ov_glGetImageTransformParameterfvHP=f;
	ov_glGetImageTransformParameterfvHP(param0,param1,param2);
}
void (APIENTRY *ov_glGetImageTransformParameterfvHP)(unsigned int param0,unsigned int param1,float* param2)=init_ov_glGetImageTransformParameterfvHP;

void __gl_ml_glGetImageTransformParameterfvHP(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glGetImageTransformParameterfvHP(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetImageTransformParameterivHP)(unsigned int param0,unsigned int param1,signed int* param2);

static void APIENTRY init_ov_glGetImageTransformParameterivHP(unsigned int param0,unsigned int param1,signed int* param2)
{
	void *f;
	f=(void*)getfunc("glGetImageTransformParameterivHP");
	ov_glGetImageTransformParameterivHP=f;
	ov_glGetImageTransformParameterivHP(param0,param1,param2);
}
void (APIENTRY *ov_glGetImageTransformParameterivHP)(unsigned int param0,unsigned int param1,signed int* param2)=init_ov_glGetImageTransformParameterivHP;

void __gl_ml_glGetImageTransformParameterivHP(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int* l2=(signed int*)(gl__ba__(v2));
	ov_glGetImageTransformParameterivHP(l0,l1,l2);
}
extern void (APIENTRY *ov_glImageTransformParameterfvHP)(unsigned int param0,unsigned int param1,float* param2);

static void APIENTRY init_ov_glImageTransformParameterfvHP(unsigned int param0,unsigned int param1,float* param2)
{
	void *f;
	f=(void*)getfunc("glImageTransformParameterfvHP");
	ov_glImageTransformParameterfvHP=f;
	ov_glImageTransformParameterfvHP(param0,param1,param2);
}
void (APIENTRY *ov_glImageTransformParameterfvHP)(unsigned int param0,unsigned int param1,float* param2)=init_ov_glImageTransformParameterfvHP;

void __gl_ml_glImageTransformParameterfvHP(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glImageTransformParameterfvHP(l0,l1,l2);
}
extern void (APIENTRY *ov_glImageTransformParameterivHP)(unsigned int param0,unsigned int param1,signed int* param2);

static void APIENTRY init_ov_glImageTransformParameterivHP(unsigned int param0,unsigned int param1,signed int* param2)
{
	void *f;
	f=(void*)getfunc("glImageTransformParameterivHP");
	ov_glImageTransformParameterivHP=f;
	ov_glImageTransformParameterivHP(param0,param1,param2);
}
void (APIENTRY *ov_glImageTransformParameterivHP)(unsigned int param0,unsigned int param1,signed int* param2)=init_ov_glImageTransformParameterivHP;

void __gl_ml_glImageTransformParameterivHP(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int* l2=(signed int*)(gl__ba__(v2));
	ov_glImageTransformParameterivHP(l0,l1,l2);
}
extern void (APIENTRY *ov_glImageTransformParameterfHP)(unsigned int param0,unsigned int param1,float param2);

static void APIENTRY init_ov_glImageTransformParameterfHP(unsigned int param0,unsigned int param1,float param2)
{
	void *f;
	f=(void*)getfunc("glImageTransformParameterfHP");
	ov_glImageTransformParameterfHP=f;
	ov_glImageTransformParameterfHP(param0,param1,param2);
}
void (APIENTRY *ov_glImageTransformParameterfHP)(unsigned int param0,unsigned int param1,float param2)=init_ov_glImageTransformParameterfHP;

void __gl_ml_glImageTransformParameterfHP(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float l2=Double_val(v2);
	ov_glImageTransformParameterfHP(l0,l1,l2);
}
extern void (APIENTRY *ov_glImageTransformParameteriHP)(unsigned int param0,unsigned int param1,signed int param2);

static void APIENTRY init_ov_glImageTransformParameteriHP(unsigned int param0,unsigned int param1,signed int param2)
{
	void *f;
	f=(void*)getfunc("glImageTransformParameteriHP");
	ov_glImageTransformParameteriHP=f;
	ov_glImageTransformParameteriHP(param0,param1,param2);
}
void (APIENTRY *ov_glImageTransformParameteriHP)(unsigned int param0,unsigned int param1,signed int param2)=init_ov_glImageTransformParameteriHP;

void __gl_ml_glImageTransformParameteriHP(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	ov_glImageTransformParameteriHP(l0,l1,l2);
}
extern void (APIENTRY *ov_glMultiModeDrawElementsIBM)(unsigned int* param0,signed int* param1,unsigned int param2,void** param3,signed int param4,signed int param5);

static void APIENTRY init_ov_glMultiModeDrawElementsIBM(unsigned int* param0,signed int* param1,unsigned int param2,void** param3,signed int param4,signed int param5)
{
	void *f;
	f=(void*)getfunc("glMultiModeDrawElementsIBM");
	ov_glMultiModeDrawElementsIBM=f;
	ov_glMultiModeDrawElementsIBM(param0,param1,param2,param3,param4,param5);
}
void (APIENTRY *ov_glMultiModeDrawElementsIBM)(unsigned int* param0,signed int* param1,unsigned int param2,void** param3,signed int param4,signed int param5)=init_ov_glMultiModeDrawElementsIBM;
extern void (APIENTRY *ov_glMultiModeDrawArraysIBM)(unsigned int param0,signed int* param1,signed int* param2,signed int param3,signed int param4);

static void APIENTRY init_ov_glMultiModeDrawArraysIBM(unsigned int param0,signed int* param1,signed int* param2,signed int param3,signed int param4)
{
	void *f;
	f=(void*)getfunc("glMultiModeDrawArraysIBM");
	ov_glMultiModeDrawArraysIBM=f;
	ov_glMultiModeDrawArraysIBM(param0,param1,param2,param3,param4);
}
void (APIENTRY *ov_glMultiModeDrawArraysIBM)(unsigned int param0,signed int* param1,signed int* param2,signed int param3,signed int param4)=init_ov_glMultiModeDrawArraysIBM;

void __gl_ml_glMultiModeDrawArraysIBM(value v0,value v1,value v2,value v3,value v4)
{
	unsigned int l0=Int_val(v0);
	signed int* l1=(signed int*)(gl__ba__(v1));
	signed int* l2=(signed int*)(gl__ba__(v2));
	signed int l3=Int_val(v3);
	signed int l4=Int_val(v4);
	ov_glMultiModeDrawArraysIBM(l0,l1,l2,l3,l4);
}
extern void (APIENTRY *ov_glVertexPointerListIBM)(signed int param0,unsigned int param1,signed int param2,void** param3,signed int param4);

static void APIENTRY init_ov_glVertexPointerListIBM(signed int param0,unsigned int param1,signed int param2,void** param3,signed int param4)
{
	void *f;
	f=(void*)getfunc("glVertexPointerListIBM");
	ov_glVertexPointerListIBM=f;
	ov_glVertexPointerListIBM(param0,param1,param2,param3,param4);
}
void (APIENTRY *ov_glVertexPointerListIBM)(signed int param0,unsigned int param1,signed int param2,void** param3,signed int param4)=init_ov_glVertexPointerListIBM;
extern void (APIENTRY *ov_glTexCoordPointerListIBM)(signed int param0,unsigned int param1,signed int param2,void** param3,signed int param4);

static void APIENTRY init_ov_glTexCoordPointerListIBM(signed int param0,unsigned int param1,signed int param2,void** param3,signed int param4)
{
	void *f;
	f=(void*)getfunc("glTexCoordPointerListIBM");
	ov_glTexCoordPointerListIBM=f;
	ov_glTexCoordPointerListIBM(param0,param1,param2,param3,param4);
}
void (APIENTRY *ov_glTexCoordPointerListIBM)(signed int param0,unsigned int param1,signed int param2,void** param3,signed int param4)=init_ov_glTexCoordPointerListIBM;
extern void (APIENTRY *ov_glNormalPointerListIBM)(unsigned int param0,signed int param1,void** param2,signed int param3);

static void APIENTRY init_ov_glNormalPointerListIBM(unsigned int param0,signed int param1,void** param2,signed int param3)
{
	void *f;
	f=(void*)getfunc("glNormalPointerListIBM");
	ov_glNormalPointerListIBM=f;
	ov_glNormalPointerListIBM(param0,param1,param2,param3);
}
void (APIENTRY *ov_glNormalPointerListIBM)(unsigned int param0,signed int param1,void** param2,signed int param3)=init_ov_glNormalPointerListIBM;
extern void (APIENTRY *ov_glIndexPointerListIBM)(unsigned int param0,signed int param1,void** param2,signed int param3);

static void APIENTRY init_ov_glIndexPointerListIBM(unsigned int param0,signed int param1,void** param2,signed int param3)
{
	void *f;
	f=(void*)getfunc("glIndexPointerListIBM");
	ov_glIndexPointerListIBM=f;
	ov_glIndexPointerListIBM(param0,param1,param2,param3);
}
void (APIENTRY *ov_glIndexPointerListIBM)(unsigned int param0,signed int param1,void** param2,signed int param3)=init_ov_glIndexPointerListIBM;
extern void (APIENTRY *ov_glFogCoordPointerListIBM)(unsigned int param0,signed int param1,void** param2,signed int param3);

static void APIENTRY init_ov_glFogCoordPointerListIBM(unsigned int param0,signed int param1,void** param2,signed int param3)
{
	void *f;
	f=(void*)getfunc("glFogCoordPointerListIBM");
	ov_glFogCoordPointerListIBM=f;
	ov_glFogCoordPointerListIBM(param0,param1,param2,param3);
}
void (APIENTRY *ov_glFogCoordPointerListIBM)(unsigned int param0,signed int param1,void** param2,signed int param3)=init_ov_glFogCoordPointerListIBM;
extern void (APIENTRY *ov_glEdgeFlagPointerListIBM)(signed int param0,unsigned char** param1,signed int param2);

static void APIENTRY init_ov_glEdgeFlagPointerListIBM(signed int param0,unsigned char** param1,signed int param2)
{
	void *f;
	f=(void*)getfunc("glEdgeFlagPointerListIBM");
	ov_glEdgeFlagPointerListIBM=f;
	ov_glEdgeFlagPointerListIBM(param0,param1,param2);
}
void (APIENTRY *ov_glEdgeFlagPointerListIBM)(signed int param0,unsigned char** param1,signed int param2)=init_ov_glEdgeFlagPointerListIBM;
extern void (APIENTRY *ov_glSecondaryColorPointerListIBM)(signed int param0,unsigned int param1,signed int param2,void** param3,signed int param4);

static void APIENTRY init_ov_glSecondaryColorPointerListIBM(signed int param0,unsigned int param1,signed int param2,void** param3,signed int param4)
{
	void *f;
	f=(void*)getfunc("glSecondaryColorPointerListIBM");
	ov_glSecondaryColorPointerListIBM=f;
	ov_glSecondaryColorPointerListIBM(param0,param1,param2,param3,param4);
}
void (APIENTRY *ov_glSecondaryColorPointerListIBM)(signed int param0,unsigned int param1,signed int param2,void** param3,signed int param4)=init_ov_glSecondaryColorPointerListIBM;
extern void (APIENTRY *ov_glColorPointerListIBM)(signed int param0,unsigned int param1,signed int param2,void** param3,signed int param4);

static void APIENTRY init_ov_glColorPointerListIBM(signed int param0,unsigned int param1,signed int param2,void** param3,signed int param4)
{
	void *f;
	f=(void*)getfunc("glColorPointerListIBM");
	ov_glColorPointerListIBM=f;
	ov_glColorPointerListIBM(param0,param1,param2,param3,param4);
}
void (APIENTRY *ov_glColorPointerListIBM)(signed int param0,unsigned int param1,signed int param2,void** param3,signed int param4)=init_ov_glColorPointerListIBM;
extern void (APIENTRY *ov_glBlendFuncSeparateINGR)(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3);

static void APIENTRY init_ov_glBlendFuncSeparateINGR(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3)
{
	void *f;
	f=(void*)getfunc("glBlendFuncSeparateINGR");
	ov_glBlendFuncSeparateINGR=f;
	ov_glBlendFuncSeparateINGR(param0,param1,param2,param3);
}
void (APIENTRY *ov_glBlendFuncSeparateINGR)(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3)=init_ov_glBlendFuncSeparateINGR;

void __gl_ml_glBlendFuncSeparateINGR(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	unsigned int l3=Int_val(v3);
	ov_glBlendFuncSeparateINGR(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glTexCoordPointervINTEL)(signed int param0,unsigned int param1,void** param2);

static void APIENTRY init_ov_glTexCoordPointervINTEL(signed int param0,unsigned int param1,void** param2)
{
	void *f;
	f=(void*)getfunc("glTexCoordPointervINTEL");
	ov_glTexCoordPointervINTEL=f;
	ov_glTexCoordPointervINTEL(param0,param1,param2);
}
void (APIENTRY *ov_glTexCoordPointervINTEL)(signed int param0,unsigned int param1,void** param2)=init_ov_glTexCoordPointervINTEL;
extern void (APIENTRY *ov_glColorPointervINTEL)(signed int param0,unsigned int param1,void** param2);

static void APIENTRY init_ov_glColorPointervINTEL(signed int param0,unsigned int param1,void** param2)
{
	void *f;
	f=(void*)getfunc("glColorPointervINTEL");
	ov_glColorPointervINTEL=f;
	ov_glColorPointervINTEL(param0,param1,param2);
}
void (APIENTRY *ov_glColorPointervINTEL)(signed int param0,unsigned int param1,void** param2)=init_ov_glColorPointervINTEL;
extern void (APIENTRY *ov_glNormalPointervINTEL)(unsigned int param0,void** param1);

static void APIENTRY init_ov_glNormalPointervINTEL(unsigned int param0,void** param1)
{
	void *f;
	f=(void*)getfunc("glNormalPointervINTEL");
	ov_glNormalPointervINTEL=f;
	ov_glNormalPointervINTEL(param0,param1);
}
void (APIENTRY *ov_glNormalPointervINTEL)(unsigned int param0,void** param1)=init_ov_glNormalPointervINTEL;
extern void (APIENTRY *ov_glVertexPointervINTEL)(signed int param0,unsigned int param1,void** param2);

static void APIENTRY init_ov_glVertexPointervINTEL(signed int param0,unsigned int param1,void** param2)
{
	void *f;
	f=(void*)getfunc("glVertexPointervINTEL");
	ov_glVertexPointervINTEL=f;
	ov_glVertexPointervINTEL(param0,param1,param2);
}
void (APIENTRY *ov_glVertexPointervINTEL)(signed int param0,unsigned int param1,void** param2)=init_ov_glVertexPointervINTEL;
extern void (APIENTRY *ov_glResizeBuffersMESA)();

static void APIENTRY init_ov_glResizeBuffersMESA()
{
	void *f;
	f=(void*)getfunc("glResizeBuffersMESA");
	ov_glResizeBuffersMESA=f;
	ov_glResizeBuffersMESA();
}
void (APIENTRY *ov_glResizeBuffersMESA)()=init_ov_glResizeBuffersMESA;

void __gl_ml_glResizeBuffersMESA()
{
	ov_glResizeBuffersMESA();
}
extern void (APIENTRY *ov_glWindowPos4svMESA)(signed short* param0);

static void APIENTRY init_ov_glWindowPos4svMESA(signed short* param0)
{
	void *f;
	f=(void*)getfunc("glWindowPos4svMESA");
	ov_glWindowPos4svMESA=f;
	ov_glWindowPos4svMESA(param0);
}
void (APIENTRY *ov_glWindowPos4svMESA)(signed short* param0)=init_ov_glWindowPos4svMESA;

void __gl_ml_glWindowPos4svMESA(value v0)
{
	signed short* l0=(signed short*)(gl__ba__(v0));
	ov_glWindowPos4svMESA(l0);
}
extern void (APIENTRY *ov_glWindowPos4sMESA)(signed short param0,signed short param1,signed short param2,signed short param3);

static void APIENTRY init_ov_glWindowPos4sMESA(signed short param0,signed short param1,signed short param2,signed short param3)
{
	void *f;
	f=(void*)getfunc("glWindowPos4sMESA");
	ov_glWindowPos4sMESA=f;
	ov_glWindowPos4sMESA(param0,param1,param2,param3);
}
void (APIENTRY *ov_glWindowPos4sMESA)(signed short param0,signed short param1,signed short param2,signed short param3)=init_ov_glWindowPos4sMESA;

void __gl_ml_glWindowPos4sMESA(value v0,value v1,value v2,value v3)
{
	signed short l0=Int_val(v0);
	signed short l1=Int_val(v1);
	signed short l2=Int_val(v2);
	signed short l3=Int_val(v3);
	ov_glWindowPos4sMESA(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glWindowPos4ivMESA)(signed int* param0);

static void APIENTRY init_ov_glWindowPos4ivMESA(signed int* param0)
{
	void *f;
	f=(void*)getfunc("glWindowPos4ivMESA");
	ov_glWindowPos4ivMESA=f;
	ov_glWindowPos4ivMESA(param0);
}
void (APIENTRY *ov_glWindowPos4ivMESA)(signed int* param0)=init_ov_glWindowPos4ivMESA;

void __gl_ml_glWindowPos4ivMESA(value v0)
{
	signed int* l0=(signed int*)(gl__ba__(v0));
	ov_glWindowPos4ivMESA(l0);
}
extern void (APIENTRY *ov_glWindowPos4iMESA)(signed int param0,signed int param1,signed int param2,signed int param3);

static void APIENTRY init_ov_glWindowPos4iMESA(signed int param0,signed int param1,signed int param2,signed int param3)
{
	void *f;
	f=(void*)getfunc("glWindowPos4iMESA");
	ov_glWindowPos4iMESA=f;
	ov_glWindowPos4iMESA(param0,param1,param2,param3);
}
void (APIENTRY *ov_glWindowPos4iMESA)(signed int param0,signed int param1,signed int param2,signed int param3)=init_ov_glWindowPos4iMESA;

void __gl_ml_glWindowPos4iMESA(value v0,value v1,value v2,value v3)
{
	signed int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	ov_glWindowPos4iMESA(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glWindowPos4fvMESA)(float* param0);

static void APIENTRY init_ov_glWindowPos4fvMESA(float* param0)
{
	void *f;
	f=(void*)getfunc("glWindowPos4fvMESA");
	ov_glWindowPos4fvMESA=f;
	ov_glWindowPos4fvMESA(param0);
}
void (APIENTRY *ov_glWindowPos4fvMESA)(float* param0)=init_ov_glWindowPos4fvMESA;

void __gl_ml_glWindowPos4fvMESA(value v0)
{
	float* l0=(float*)(gl__ba__(v0));
	ov_glWindowPos4fvMESA(l0);
}
extern void (APIENTRY *ov_glWindowPos4fMESA)(float param0,float param1,float param2,float param3);

static void APIENTRY init_ov_glWindowPos4fMESA(float param0,float param1,float param2,float param3)
{
	void *f;
	f=(void*)getfunc("glWindowPos4fMESA");
	ov_glWindowPos4fMESA=f;
	ov_glWindowPos4fMESA(param0,param1,param2,param3);
}
void (APIENTRY *ov_glWindowPos4fMESA)(float param0,float param1,float param2,float param3)=init_ov_glWindowPos4fMESA;

void __gl_ml_glWindowPos4fMESA(value v0,value v1,value v2,value v3)
{
	float l0=Double_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	float l3=Double_val(v3);
	ov_glWindowPos4fMESA(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glWindowPos4dvMESA)(double* param0);

static void APIENTRY init_ov_glWindowPos4dvMESA(double* param0)
{
	void *f;
	f=(void*)getfunc("glWindowPos4dvMESA");
	ov_glWindowPos4dvMESA=f;
	ov_glWindowPos4dvMESA(param0);
}
void (APIENTRY *ov_glWindowPos4dvMESA)(double* param0)=init_ov_glWindowPos4dvMESA;

void __gl_ml_glWindowPos4dvMESA(value v0)
{
	double* l0=(double*)(gl__ba__(v0));
	ov_glWindowPos4dvMESA(l0);
}
extern void (APIENTRY *ov_glWindowPos4dMESA)(double param0,double param1,double param2,double param3);

static void APIENTRY init_ov_glWindowPos4dMESA(double param0,double param1,double param2,double param3)
{
	void *f;
	f=(void*)getfunc("glWindowPos4dMESA");
	ov_glWindowPos4dMESA=f;
	ov_glWindowPos4dMESA(param0,param1,param2,param3);
}
void (APIENTRY *ov_glWindowPos4dMESA)(double param0,double param1,double param2,double param3)=init_ov_glWindowPos4dMESA;

void __gl_ml_glWindowPos4dMESA(value v0,value v1,value v2,value v3)
{
	double l0=Double_val(v0);
	double l1=Double_val(v1);
	double l2=Double_val(v2);
	double l3=Double_val(v3);
	ov_glWindowPos4dMESA(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glWindowPos3svMESA)(signed short* param0);

static void APIENTRY init_ov_glWindowPos3svMESA(signed short* param0)
{
	void *f;
	f=(void*)getfunc("glWindowPos3svMESA");
	ov_glWindowPos3svMESA=f;
	ov_glWindowPos3svMESA(param0);
}
void (APIENTRY *ov_glWindowPos3svMESA)(signed short* param0)=init_ov_glWindowPos3svMESA;

void __gl_ml_glWindowPos3svMESA(value v0)
{
	signed short* l0=(signed short*)(gl__ba__(v0));
	ov_glWindowPos3svMESA(l0);
}
extern void (APIENTRY *ov_glWindowPos3sMESA)(signed short param0,signed short param1,signed short param2);

static void APIENTRY init_ov_glWindowPos3sMESA(signed short param0,signed short param1,signed short param2)
{
	void *f;
	f=(void*)getfunc("glWindowPos3sMESA");
	ov_glWindowPos3sMESA=f;
	ov_glWindowPos3sMESA(param0,param1,param2);
}
void (APIENTRY *ov_glWindowPos3sMESA)(signed short param0,signed short param1,signed short param2)=init_ov_glWindowPos3sMESA;

void __gl_ml_glWindowPos3sMESA(value v0,value v1,value v2)
{
	signed short l0=Int_val(v0);
	signed short l1=Int_val(v1);
	signed short l2=Int_val(v2);
	ov_glWindowPos3sMESA(l0,l1,l2);
}
extern void (APIENTRY *ov_glWindowPos3ivMESA)(signed int* param0);

static void APIENTRY init_ov_glWindowPos3ivMESA(signed int* param0)
{
	void *f;
	f=(void*)getfunc("glWindowPos3ivMESA");
	ov_glWindowPos3ivMESA=f;
	ov_glWindowPos3ivMESA(param0);
}
void (APIENTRY *ov_glWindowPos3ivMESA)(signed int* param0)=init_ov_glWindowPos3ivMESA;

void __gl_ml_glWindowPos3ivMESA(value v0)
{
	signed int* l0=(signed int*)(gl__ba__(v0));
	ov_glWindowPos3ivMESA(l0);
}
extern void (APIENTRY *ov_glWindowPos3iMESA)(signed int param0,signed int param1,signed int param2);

static void APIENTRY init_ov_glWindowPos3iMESA(signed int param0,signed int param1,signed int param2)
{
	void *f;
	f=(void*)getfunc("glWindowPos3iMESA");
	ov_glWindowPos3iMESA=f;
	ov_glWindowPos3iMESA(param0,param1,param2);
}
void (APIENTRY *ov_glWindowPos3iMESA)(signed int param0,signed int param1,signed int param2)=init_ov_glWindowPos3iMESA;

void __gl_ml_glWindowPos3iMESA(value v0,value v1,value v2)
{
	signed int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	ov_glWindowPos3iMESA(l0,l1,l2);
}
extern void (APIENTRY *ov_glWindowPos3fvMESA)(float* param0);

static void APIENTRY init_ov_glWindowPos3fvMESA(float* param0)
{
	void *f;
	f=(void*)getfunc("glWindowPos3fvMESA");
	ov_glWindowPos3fvMESA=f;
	ov_glWindowPos3fvMESA(param0);
}
void (APIENTRY *ov_glWindowPos3fvMESA)(float* param0)=init_ov_glWindowPos3fvMESA;

void __gl_ml_glWindowPos3fvMESA(value v0)
{
	float* l0=(float*)(gl__ba__(v0));
	ov_glWindowPos3fvMESA(l0);
}
extern void (APIENTRY *ov_glWindowPos3fMESA)(float param0,float param1,float param2);

static void APIENTRY init_ov_glWindowPos3fMESA(float param0,float param1,float param2)
{
	void *f;
	f=(void*)getfunc("glWindowPos3fMESA");
	ov_glWindowPos3fMESA=f;
	ov_glWindowPos3fMESA(param0,param1,param2);
}
void (APIENTRY *ov_glWindowPos3fMESA)(float param0,float param1,float param2)=init_ov_glWindowPos3fMESA;

void __gl_ml_glWindowPos3fMESA(value v0,value v1,value v2)
{
	float l0=Double_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	ov_glWindowPos3fMESA(l0,l1,l2);
}
extern void (APIENTRY *ov_glWindowPos3dvMESA)(double* param0);

static void APIENTRY init_ov_glWindowPos3dvMESA(double* param0)
{
	void *f;
	f=(void*)getfunc("glWindowPos3dvMESA");
	ov_glWindowPos3dvMESA=f;
	ov_glWindowPos3dvMESA(param0);
}
void (APIENTRY *ov_glWindowPos3dvMESA)(double* param0)=init_ov_glWindowPos3dvMESA;

void __gl_ml_glWindowPos3dvMESA(value v0)
{
	double* l0=(double*)(gl__ba__(v0));
	ov_glWindowPos3dvMESA(l0);
}
extern void (APIENTRY *ov_glWindowPos3dMESA)(double param0,double param1,double param2);

static void APIENTRY init_ov_glWindowPos3dMESA(double param0,double param1,double param2)
{
	void *f;
	f=(void*)getfunc("glWindowPos3dMESA");
	ov_glWindowPos3dMESA=f;
	ov_glWindowPos3dMESA(param0,param1,param2);
}
void (APIENTRY *ov_glWindowPos3dMESA)(double param0,double param1,double param2)=init_ov_glWindowPos3dMESA;

void __gl_ml_glWindowPos3dMESA(value v0,value v1,value v2)
{
	double l0=Double_val(v0);
	double l1=Double_val(v1);
	double l2=Double_val(v2);
	ov_glWindowPos3dMESA(l0,l1,l2);
}
extern void (APIENTRY *ov_glWindowPos2svMESA)(signed short* param0);

static void APIENTRY init_ov_glWindowPos2svMESA(signed short* param0)
{
	void *f;
	f=(void*)getfunc("glWindowPos2svMESA");
	ov_glWindowPos2svMESA=f;
	ov_glWindowPos2svMESA(param0);
}
void (APIENTRY *ov_glWindowPos2svMESA)(signed short* param0)=init_ov_glWindowPos2svMESA;

void __gl_ml_glWindowPos2svMESA(value v0)
{
	signed short* l0=(signed short*)(gl__ba__(v0));
	ov_glWindowPos2svMESA(l0);
}
extern void (APIENTRY *ov_glWindowPos2sMESA)(signed short param0,signed short param1);

static void APIENTRY init_ov_glWindowPos2sMESA(signed short param0,signed short param1)
{
	void *f;
	f=(void*)getfunc("glWindowPos2sMESA");
	ov_glWindowPos2sMESA=f;
	ov_glWindowPos2sMESA(param0,param1);
}
void (APIENTRY *ov_glWindowPos2sMESA)(signed short param0,signed short param1)=init_ov_glWindowPos2sMESA;

void __gl_ml_glWindowPos2sMESA(value v0,value v1)
{
	signed short l0=Int_val(v0);
	signed short l1=Int_val(v1);
	ov_glWindowPos2sMESA(l0,l1);
}
extern void (APIENTRY *ov_glWindowPos2ivMESA)(signed int* param0);

static void APIENTRY init_ov_glWindowPos2ivMESA(signed int* param0)
{
	void *f;
	f=(void*)getfunc("glWindowPos2ivMESA");
	ov_glWindowPos2ivMESA=f;
	ov_glWindowPos2ivMESA(param0);
}
void (APIENTRY *ov_glWindowPos2ivMESA)(signed int* param0)=init_ov_glWindowPos2ivMESA;

void __gl_ml_glWindowPos2ivMESA(value v0)
{
	signed int* l0=(signed int*)(gl__ba__(v0));
	ov_glWindowPos2ivMESA(l0);
}
extern void (APIENTRY *ov_glWindowPos2iMESA)(signed int param0,signed int param1);

static void APIENTRY init_ov_glWindowPos2iMESA(signed int param0,signed int param1)
{
	void *f;
	f=(void*)getfunc("glWindowPos2iMESA");
	ov_glWindowPos2iMESA=f;
	ov_glWindowPos2iMESA(param0,param1);
}
void (APIENTRY *ov_glWindowPos2iMESA)(signed int param0,signed int param1)=init_ov_glWindowPos2iMESA;

void __gl_ml_glWindowPos2iMESA(value v0,value v1)
{
	signed int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	ov_glWindowPos2iMESA(l0,l1);
}
extern void (APIENTRY *ov_glWindowPos2fvMESA)(float* param0);

static void APIENTRY init_ov_glWindowPos2fvMESA(float* param0)
{
	void *f;
	f=(void*)getfunc("glWindowPos2fvMESA");
	ov_glWindowPos2fvMESA=f;
	ov_glWindowPos2fvMESA(param0);
}
void (APIENTRY *ov_glWindowPos2fvMESA)(float* param0)=init_ov_glWindowPos2fvMESA;

void __gl_ml_glWindowPos2fvMESA(value v0)
{
	float* l0=(float*)(gl__ba__(v0));
	ov_glWindowPos2fvMESA(l0);
}
extern void (APIENTRY *ov_glWindowPos2fMESA)(float param0,float param1);

static void APIENTRY init_ov_glWindowPos2fMESA(float param0,float param1)
{
	void *f;
	f=(void*)getfunc("glWindowPos2fMESA");
	ov_glWindowPos2fMESA=f;
	ov_glWindowPos2fMESA(param0,param1);
}
void (APIENTRY *ov_glWindowPos2fMESA)(float param0,float param1)=init_ov_glWindowPos2fMESA;

void __gl_ml_glWindowPos2fMESA(value v0,value v1)
{
	float l0=Double_val(v0);
	float l1=Double_val(v1);
	ov_glWindowPos2fMESA(l0,l1);
}
extern void (APIENTRY *ov_glWindowPos2dvMESA)(double* param0);

static void APIENTRY init_ov_glWindowPos2dvMESA(double* param0)
{
	void *f;
	f=(void*)getfunc("glWindowPos2dvMESA");
	ov_glWindowPos2dvMESA=f;
	ov_glWindowPos2dvMESA(param0);
}
void (APIENTRY *ov_glWindowPos2dvMESA)(double* param0)=init_ov_glWindowPos2dvMESA;

void __gl_ml_glWindowPos2dvMESA(value v0)
{
	double* l0=(double*)(gl__ba__(v0));
	ov_glWindowPos2dvMESA(l0);
}
extern void (APIENTRY *ov_glWindowPos2dMESA)(double param0,double param1);

static void APIENTRY init_ov_glWindowPos2dMESA(double param0,double param1)
{
	void *f;
	f=(void*)getfunc("glWindowPos2dMESA");
	ov_glWindowPos2dMESA=f;
	ov_glWindowPos2dMESA(param0,param1);
}
void (APIENTRY *ov_glWindowPos2dMESA)(double param0,double param1)=init_ov_glWindowPos2dMESA;

void __gl_ml_glWindowPos2dMESA(value v0,value v1)
{
	double l0=Double_val(v0);
	double l1=Double_val(v1);
	ov_glWindowPos2dMESA(l0,l1);
}
extern void (APIENTRY *ov_glHintPGI)(unsigned int param0,signed int param1);

static void APIENTRY init_ov_glHintPGI(unsigned int param0,signed int param1)
{
	void *f;
	f=(void*)getfunc("glHintPGI");
	ov_glHintPGI=f;
	ov_glHintPGI(param0,param1);
}
void (APIENTRY *ov_glHintPGI)(unsigned int param0,signed int param1)=init_ov_glHintPGI;

void __gl_ml_glHintPGI(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	ov_glHintPGI(l0,l1);
}
extern void (APIENTRY *ov_glGlobalAlphaFactoruiSUN)(unsigned int param0);

static void APIENTRY init_ov_glGlobalAlphaFactoruiSUN(unsigned int param0)
{
	void *f;
	f=(void*)getfunc("glGlobalAlphaFactoruiSUN");
	ov_glGlobalAlphaFactoruiSUN=f;
	ov_glGlobalAlphaFactoruiSUN(param0);
}
void (APIENTRY *ov_glGlobalAlphaFactoruiSUN)(unsigned int param0)=init_ov_glGlobalAlphaFactoruiSUN;

void __gl_ml_glGlobalAlphaFactoruiSUN(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glGlobalAlphaFactoruiSUN(l0);
}
extern void (APIENTRY *ov_glGlobalAlphaFactorusSUN)(unsigned short param0);

static void APIENTRY init_ov_glGlobalAlphaFactorusSUN(unsigned short param0)
{
	void *f;
	f=(void*)getfunc("glGlobalAlphaFactorusSUN");
	ov_glGlobalAlphaFactorusSUN=f;
	ov_glGlobalAlphaFactorusSUN(param0);
}
void (APIENTRY *ov_glGlobalAlphaFactorusSUN)(unsigned short param0)=init_ov_glGlobalAlphaFactorusSUN;

void __gl_ml_glGlobalAlphaFactorusSUN(value v0)
{
	unsigned short l0=Int_val(v0);
	ov_glGlobalAlphaFactorusSUN(l0);
}
extern void (APIENTRY *ov_glGlobalAlphaFactorubSUN)(unsigned char param0);

static void APIENTRY init_ov_glGlobalAlphaFactorubSUN(unsigned char param0)
{
	void *f;
	f=(void*)getfunc("glGlobalAlphaFactorubSUN");
	ov_glGlobalAlphaFactorubSUN=f;
	ov_glGlobalAlphaFactorubSUN(param0);
}
void (APIENTRY *ov_glGlobalAlphaFactorubSUN)(unsigned char param0)=init_ov_glGlobalAlphaFactorubSUN;

void __gl_ml_glGlobalAlphaFactorubSUN(value v0)
{
	unsigned char l0=Int_val(v0);
	ov_glGlobalAlphaFactorubSUN(l0);
}
extern void (APIENTRY *ov_glGlobalAlphaFactordSUN)(double param0);

static void APIENTRY init_ov_glGlobalAlphaFactordSUN(double param0)
{
	void *f;
	f=(void*)getfunc("glGlobalAlphaFactordSUN");
	ov_glGlobalAlphaFactordSUN=f;
	ov_glGlobalAlphaFactordSUN(param0);
}
void (APIENTRY *ov_glGlobalAlphaFactordSUN)(double param0)=init_ov_glGlobalAlphaFactordSUN;

void __gl_ml_glGlobalAlphaFactordSUN(value v0)
{
	double l0=Double_val(v0);
	ov_glGlobalAlphaFactordSUN(l0);
}
extern void (APIENTRY *ov_glGlobalAlphaFactorfSUN)(float param0);

static void APIENTRY init_ov_glGlobalAlphaFactorfSUN(float param0)
{
	void *f;
	f=(void*)getfunc("glGlobalAlphaFactorfSUN");
	ov_glGlobalAlphaFactorfSUN=f;
	ov_glGlobalAlphaFactorfSUN(param0);
}
void (APIENTRY *ov_glGlobalAlphaFactorfSUN)(float param0)=init_ov_glGlobalAlphaFactorfSUN;

void __gl_ml_glGlobalAlphaFactorfSUN(value v0)
{
	float l0=Double_val(v0);
	ov_glGlobalAlphaFactorfSUN(l0);
}
extern void (APIENTRY *ov_glGlobalAlphaFactoriSUN)(signed int param0);

static void APIENTRY init_ov_glGlobalAlphaFactoriSUN(signed int param0)
{
	void *f;
	f=(void*)getfunc("glGlobalAlphaFactoriSUN");
	ov_glGlobalAlphaFactoriSUN=f;
	ov_glGlobalAlphaFactoriSUN(param0);
}
void (APIENTRY *ov_glGlobalAlphaFactoriSUN)(signed int param0)=init_ov_glGlobalAlphaFactoriSUN;

void __gl_ml_glGlobalAlphaFactoriSUN(value v0)
{
	signed int l0=Int_val(v0);
	ov_glGlobalAlphaFactoriSUN(l0);
}
extern void (APIENTRY *ov_glGlobalAlphaFactorsSUN)(signed short param0);

static void APIENTRY init_ov_glGlobalAlphaFactorsSUN(signed short param0)
{
	void *f;
	f=(void*)getfunc("glGlobalAlphaFactorsSUN");
	ov_glGlobalAlphaFactorsSUN=f;
	ov_glGlobalAlphaFactorsSUN(param0);
}
void (APIENTRY *ov_glGlobalAlphaFactorsSUN)(signed short param0)=init_ov_glGlobalAlphaFactorsSUN;

void __gl_ml_glGlobalAlphaFactorsSUN(value v0)
{
	signed short l0=Int_val(v0);
	ov_glGlobalAlphaFactorsSUN(l0);
}
extern void (APIENTRY *ov_glGlobalAlphaFactorbSUN)(signed char param0);

static void APIENTRY init_ov_glGlobalAlphaFactorbSUN(signed char param0)
{
	void *f;
	f=(void*)getfunc("glGlobalAlphaFactorbSUN");
	ov_glGlobalAlphaFactorbSUN=f;
	ov_glGlobalAlphaFactorbSUN(param0);
}
void (APIENTRY *ov_glGlobalAlphaFactorbSUN)(signed char param0)=init_ov_glGlobalAlphaFactorbSUN;

void __gl_ml_glGlobalAlphaFactorbSUN(value v0)
{
	signed char l0=Int_val(v0);
	ov_glGlobalAlphaFactorbSUN(l0);
}
extern void (APIENTRY *ov_glDrawMeshArraysSUN)(unsigned int param0,signed int param1,signed int param2,signed int param3);

static void APIENTRY init_ov_glDrawMeshArraysSUN(unsigned int param0,signed int param1,signed int param2,signed int param3)
{
	void *f;
	f=(void*)getfunc("glDrawMeshArraysSUN");
	ov_glDrawMeshArraysSUN=f;
	ov_glDrawMeshArraysSUN(param0,param1,param2,param3);
}
void (APIENTRY *ov_glDrawMeshArraysSUN)(unsigned int param0,signed int param1,signed int param2,signed int param3)=init_ov_glDrawMeshArraysSUN;

void __gl_ml_glDrawMeshArraysSUN(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	ov_glDrawMeshArraysSUN(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glReplacementCodePointerSUN)(unsigned int param0,signed int param1,void** param2);

static void APIENTRY init_ov_glReplacementCodePointerSUN(unsigned int param0,signed int param1,void** param2)
{
	void *f;
	f=(void*)getfunc("glReplacementCodePointerSUN");
	ov_glReplacementCodePointerSUN=f;
	ov_glReplacementCodePointerSUN(param0,param1,param2);
}
void (APIENTRY *ov_glReplacementCodePointerSUN)(unsigned int param0,signed int param1,void** param2)=init_ov_glReplacementCodePointerSUN;
extern void (APIENTRY *ov_glReplacementCodeubvSUN)(unsigned char* param0);

static void APIENTRY init_ov_glReplacementCodeubvSUN(unsigned char* param0)
{
	void *f;
	f=(void*)getfunc("glReplacementCodeubvSUN");
	ov_glReplacementCodeubvSUN=f;
	ov_glReplacementCodeubvSUN(param0);
}
void (APIENTRY *ov_glReplacementCodeubvSUN)(unsigned char* param0)=init_ov_glReplacementCodeubvSUN;

void __gl_ml_glReplacementCodeubvSUN(value v0)
{
	unsigned char* l0=(unsigned char*)(gl__ba__(v0));
	ov_glReplacementCodeubvSUN(l0);
}
extern void (APIENTRY *ov_glReplacementCodeusvSUN)(unsigned short* param0);

static void APIENTRY init_ov_glReplacementCodeusvSUN(unsigned short* param0)
{
	void *f;
	f=(void*)getfunc("glReplacementCodeusvSUN");
	ov_glReplacementCodeusvSUN=f;
	ov_glReplacementCodeusvSUN(param0);
}
void (APIENTRY *ov_glReplacementCodeusvSUN)(unsigned short* param0)=init_ov_glReplacementCodeusvSUN;

void __gl_ml_glReplacementCodeusvSUN(value v0)
{
	unsigned short* l0=(unsigned short*)(gl__ba__(v0));
	ov_glReplacementCodeusvSUN(l0);
}
extern void (APIENTRY *ov_glReplacementCodeuivSUN)(unsigned int* param0);

static void APIENTRY init_ov_glReplacementCodeuivSUN(unsigned int* param0)
{
	void *f;
	f=(void*)getfunc("glReplacementCodeuivSUN");
	ov_glReplacementCodeuivSUN=f;
	ov_glReplacementCodeuivSUN(param0);
}
void (APIENTRY *ov_glReplacementCodeuivSUN)(unsigned int* param0)=init_ov_glReplacementCodeuivSUN;

void __gl_ml_glReplacementCodeuivSUN(value v0)
{
	unsigned int* l0=(unsigned int*)(gl__ba__(v0));
	ov_glReplacementCodeuivSUN(l0);
}
extern void (APIENTRY *ov_glReplacementCodeubSUN)(unsigned char param0);

static void APIENTRY init_ov_glReplacementCodeubSUN(unsigned char param0)
{
	void *f;
	f=(void*)getfunc("glReplacementCodeubSUN");
	ov_glReplacementCodeubSUN=f;
	ov_glReplacementCodeubSUN(param0);
}
void (APIENTRY *ov_glReplacementCodeubSUN)(unsigned char param0)=init_ov_glReplacementCodeubSUN;

void __gl_ml_glReplacementCodeubSUN(value v0)
{
	unsigned char l0=Int_val(v0);
	ov_glReplacementCodeubSUN(l0);
}
extern void (APIENTRY *ov_glReplacementCodeusSUN)(unsigned short param0);

static void APIENTRY init_ov_glReplacementCodeusSUN(unsigned short param0)
{
	void *f;
	f=(void*)getfunc("glReplacementCodeusSUN");
	ov_glReplacementCodeusSUN=f;
	ov_glReplacementCodeusSUN(param0);
}
void (APIENTRY *ov_glReplacementCodeusSUN)(unsigned short param0)=init_ov_glReplacementCodeusSUN;

void __gl_ml_glReplacementCodeusSUN(value v0)
{
	unsigned short l0=Int_val(v0);
	ov_glReplacementCodeusSUN(l0);
}
extern void (APIENTRY *ov_glReplacementCodeuiSUN)(unsigned int param0);

static void APIENTRY init_ov_glReplacementCodeuiSUN(unsigned int param0)
{
	void *f;
	f=(void*)getfunc("glReplacementCodeuiSUN");
	ov_glReplacementCodeuiSUN=f;
	ov_glReplacementCodeuiSUN(param0);
}
void (APIENTRY *ov_glReplacementCodeuiSUN)(unsigned int param0)=init_ov_glReplacementCodeuiSUN;

void __gl_ml_glReplacementCodeuiSUN(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glReplacementCodeuiSUN(l0);
}
extern void (APIENTRY *ov_glReplacementCodeuiTexCoord2fColor4fNormal3fVertex3fvSUN)(unsigned int* param0,float* param1,float* param2,float* param3,float* param4);

static void APIENTRY init_ov_glReplacementCodeuiTexCoord2fColor4fNormal3fVertex3fvSUN(unsigned int* param0,float* param1,float* param2,float* param3,float* param4)
{
	void *f;
	f=(void*)getfunc("glReplacementCodeuiTexCoord2fColor4fNormal3fVertex3fvSUN");
	ov_glReplacementCodeuiTexCoord2fColor4fNormal3fVertex3fvSUN=f;
	ov_glReplacementCodeuiTexCoord2fColor4fNormal3fVertex3fvSUN(param0,param1,param2,param3,param4);
}
void (APIENTRY *ov_glReplacementCodeuiTexCoord2fColor4fNormal3fVertex3fvSUN)(unsigned int* param0,float* param1,float* param2,float* param3,float* param4)=init_ov_glReplacementCodeuiTexCoord2fColor4fNormal3fVertex3fvSUN;

void __gl_ml_glReplacementCodeuiTexCoord2fColor4fNormal3fVertex3fvSUN(value v0,value v1,value v2,value v3,value v4)
{
	unsigned int* l0=(unsigned int*)(gl__ba__(v0));
	float* l1=(float*)(gl__ba__(v1));
	float* l2=(float*)(gl__ba__(v2));
	float* l3=(float*)(gl__ba__(v3));
	float* l4=(float*)(gl__ba__(v4));
	ov_glReplacementCodeuiTexCoord2fColor4fNormal3fVertex3fvSUN(l0,l1,l2,l3,l4);
}
extern void (APIENTRY *ov_glReplacementCodeuiTexCoord2fColor4fNormal3fVertex3fSUN)(unsigned int param0,float param1,float param2,float param3,float param4,float param5,float param6,float param7,float param8,float param9,float param10,float param11,float param12);

static void APIENTRY init_ov_glReplacementCodeuiTexCoord2fColor4fNormal3fVertex3fSUN(unsigned int param0,float param1,float param2,float param3,float param4,float param5,float param6,float param7,float param8,float param9,float param10,float param11,float param12)
{
	void *f;
	f=(void*)getfunc("glReplacementCodeuiTexCoord2fColor4fNormal3fVertex3fSUN");
	ov_glReplacementCodeuiTexCoord2fColor4fNormal3fVertex3fSUN=f;
	ov_glReplacementCodeuiTexCoord2fColor4fNormal3fVertex3fSUN(param0,param1,param2,param3,param4,param5,param6,param7,param8,param9,param10,param11,param12);
}
void (APIENTRY *ov_glReplacementCodeuiTexCoord2fColor4fNormal3fVertex3fSUN)(unsigned int param0,float param1,float param2,float param3,float param4,float param5,float param6,float param7,float param8,float param9,float param10,float param11,float param12)=init_ov_glReplacementCodeuiTexCoord2fColor4fNormal3fVertex3fSUN;

void __gl_ml_glReplacementCodeuiTexCoord2fColor4fNormal3fVertex3fSUN(value v0,value v1,value v2,value v3,value v4,value v5,value v6,value v7,value v8,value v9,value v10,value v11,value v12)
{
	unsigned int l0=Int_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	float l3=Double_val(v3);
	float l4=Double_val(v4);
	float l5=Double_val(v5);
	float l6=Double_val(v6);
	float l7=Double_val(v7);
	float l8=Double_val(v8);
	float l9=Double_val(v9);
	float l10=Double_val(v10);
	float l11=Double_val(v11);
	float l12=Double_val(v12);
	ov_glReplacementCodeuiTexCoord2fColor4fNormal3fVertex3fSUN(l0,l1,l2,l3,l4,l5,l6,l7,l8,l9,l10,l11,l12);
}

void byte___gl_ml_glReplacementCodeuiTexCoord2fColor4fNormal3fVertex3fSUN(value * argv,int argn)
{
	__gl_ml_glReplacementCodeuiTexCoord2fColor4fNormal3fVertex3fSUN(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6],argv[7],argv[8],argv[9],argv[10],argv[11],argv[12]);
}
extern void (APIENTRY *ov_glReplacementCodeuiTexCoord2fNormal3fVertex3fvSUN)(unsigned int* param0,float* param1,float* param2,float* param3);

static void APIENTRY init_ov_glReplacementCodeuiTexCoord2fNormal3fVertex3fvSUN(unsigned int* param0,float* param1,float* param2,float* param3)
{
	void *f;
	f=(void*)getfunc("glReplacementCodeuiTexCoord2fNormal3fVertex3fvSUN");
	ov_glReplacementCodeuiTexCoord2fNormal3fVertex3fvSUN=f;
	ov_glReplacementCodeuiTexCoord2fNormal3fVertex3fvSUN(param0,param1,param2,param3);
}
void (APIENTRY *ov_glReplacementCodeuiTexCoord2fNormal3fVertex3fvSUN)(unsigned int* param0,float* param1,float* param2,float* param3)=init_ov_glReplacementCodeuiTexCoord2fNormal3fVertex3fvSUN;

void __gl_ml_glReplacementCodeuiTexCoord2fNormal3fVertex3fvSUN(value v0,value v1,value v2,value v3)
{
	unsigned int* l0=(unsigned int*)(gl__ba__(v0));
	float* l1=(float*)(gl__ba__(v1));
	float* l2=(float*)(gl__ba__(v2));
	float* l3=(float*)(gl__ba__(v3));
	ov_glReplacementCodeuiTexCoord2fNormal3fVertex3fvSUN(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glReplacementCodeuiTexCoord2fNormal3fVertex3fSUN)(unsigned int param0,float param1,float param2,float param3,float param4,float param5,float param6,float param7,float param8);

static void APIENTRY init_ov_glReplacementCodeuiTexCoord2fNormal3fVertex3fSUN(unsigned int param0,float param1,float param2,float param3,float param4,float param5,float param6,float param7,float param8)
{
	void *f;
	f=(void*)getfunc("glReplacementCodeuiTexCoord2fNormal3fVertex3fSUN");
	ov_glReplacementCodeuiTexCoord2fNormal3fVertex3fSUN=f;
	ov_glReplacementCodeuiTexCoord2fNormal3fVertex3fSUN(param0,param1,param2,param3,param4,param5,param6,param7,param8);
}
void (APIENTRY *ov_glReplacementCodeuiTexCoord2fNormal3fVertex3fSUN)(unsigned int param0,float param1,float param2,float param3,float param4,float param5,float param6,float param7,float param8)=init_ov_glReplacementCodeuiTexCoord2fNormal3fVertex3fSUN;

void __gl_ml_glReplacementCodeuiTexCoord2fNormal3fVertex3fSUN(value v0,value v1,value v2,value v3,value v4,value v5,value v6,value v7,value v8)
{
	unsigned int l0=Int_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	float l3=Double_val(v3);
	float l4=Double_val(v4);
	float l5=Double_val(v5);
	float l6=Double_val(v6);
	float l7=Double_val(v7);
	float l8=Double_val(v8);
	ov_glReplacementCodeuiTexCoord2fNormal3fVertex3fSUN(l0,l1,l2,l3,l4,l5,l6,l7,l8);
}

void byte___gl_ml_glReplacementCodeuiTexCoord2fNormal3fVertex3fSUN(value * argv,int argn)
{
	__gl_ml_glReplacementCodeuiTexCoord2fNormal3fVertex3fSUN(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6],argv[7],argv[8]);
}
extern void (APIENTRY *ov_glReplacementCodeuiTexCoord2fVertex3fvSUN)(unsigned int* param0,float* param1,float* param2);

static void APIENTRY init_ov_glReplacementCodeuiTexCoord2fVertex3fvSUN(unsigned int* param0,float* param1,float* param2)
{
	void *f;
	f=(void*)getfunc("glReplacementCodeuiTexCoord2fVertex3fvSUN");
	ov_glReplacementCodeuiTexCoord2fVertex3fvSUN=f;
	ov_glReplacementCodeuiTexCoord2fVertex3fvSUN(param0,param1,param2);
}
void (APIENTRY *ov_glReplacementCodeuiTexCoord2fVertex3fvSUN)(unsigned int* param0,float* param1,float* param2)=init_ov_glReplacementCodeuiTexCoord2fVertex3fvSUN;

void __gl_ml_glReplacementCodeuiTexCoord2fVertex3fvSUN(value v0,value v1,value v2)
{
	unsigned int* l0=(unsigned int*)(gl__ba__(v0));
	float* l1=(float*)(gl__ba__(v1));
	float* l2=(float*)(gl__ba__(v2));
	ov_glReplacementCodeuiTexCoord2fVertex3fvSUN(l0,l1,l2);
}
extern void (APIENTRY *ov_glReplacementCodeuiTexCoord2fVertex3fSUN)(unsigned int param0,float param1,float param2,float param3,float param4,float param5);

static void APIENTRY init_ov_glReplacementCodeuiTexCoord2fVertex3fSUN(unsigned int param0,float param1,float param2,float param3,float param4,float param5)
{
	void *f;
	f=(void*)getfunc("glReplacementCodeuiTexCoord2fVertex3fSUN");
	ov_glReplacementCodeuiTexCoord2fVertex3fSUN=f;
	ov_glReplacementCodeuiTexCoord2fVertex3fSUN(param0,param1,param2,param3,param4,param5);
}
void (APIENTRY *ov_glReplacementCodeuiTexCoord2fVertex3fSUN)(unsigned int param0,float param1,float param2,float param3,float param4,float param5)=init_ov_glReplacementCodeuiTexCoord2fVertex3fSUN;

void __gl_ml_glReplacementCodeuiTexCoord2fVertex3fSUN(value v0,value v1,value v2,value v3,value v4,value v5)
{
	unsigned int l0=Int_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	float l3=Double_val(v3);
	float l4=Double_val(v4);
	float l5=Double_val(v5);
	ov_glReplacementCodeuiTexCoord2fVertex3fSUN(l0,l1,l2,l3,l4,l5);
}

void byte___gl_ml_glReplacementCodeuiTexCoord2fVertex3fSUN(value * argv,int argn)
{
	__gl_ml_glReplacementCodeuiTexCoord2fVertex3fSUN(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5]);
}
extern void (APIENTRY *ov_glReplacementCodeuiColor4fNormal3fVertex3fvSUN)(unsigned int* param0,float* param1,float* param2,float* param3);

static void APIENTRY init_ov_glReplacementCodeuiColor4fNormal3fVertex3fvSUN(unsigned int* param0,float* param1,float* param2,float* param3)
{
	void *f;
	f=(void*)getfunc("glReplacementCodeuiColor4fNormal3fVertex3fvSUN");
	ov_glReplacementCodeuiColor4fNormal3fVertex3fvSUN=f;
	ov_glReplacementCodeuiColor4fNormal3fVertex3fvSUN(param0,param1,param2,param3);
}
void (APIENTRY *ov_glReplacementCodeuiColor4fNormal3fVertex3fvSUN)(unsigned int* param0,float* param1,float* param2,float* param3)=init_ov_glReplacementCodeuiColor4fNormal3fVertex3fvSUN;

void __gl_ml_glReplacementCodeuiColor4fNormal3fVertex3fvSUN(value v0,value v1,value v2,value v3)
{
	unsigned int* l0=(unsigned int*)(gl__ba__(v0));
	float* l1=(float*)(gl__ba__(v1));
	float* l2=(float*)(gl__ba__(v2));
	float* l3=(float*)(gl__ba__(v3));
	ov_glReplacementCodeuiColor4fNormal3fVertex3fvSUN(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glReplacementCodeuiColor4fNormal3fVertex3fSUN)(unsigned int param0,float param1,float param2,float param3,float param4,float param5,float param6,float param7,float param8,float param9,float param10);

static void APIENTRY init_ov_glReplacementCodeuiColor4fNormal3fVertex3fSUN(unsigned int param0,float param1,float param2,float param3,float param4,float param5,float param6,float param7,float param8,float param9,float param10)
{
	void *f;
	f=(void*)getfunc("glReplacementCodeuiColor4fNormal3fVertex3fSUN");
	ov_glReplacementCodeuiColor4fNormal3fVertex3fSUN=f;
	ov_glReplacementCodeuiColor4fNormal3fVertex3fSUN(param0,param1,param2,param3,param4,param5,param6,param7,param8,param9,param10);
}
void (APIENTRY *ov_glReplacementCodeuiColor4fNormal3fVertex3fSUN)(unsigned int param0,float param1,float param2,float param3,float param4,float param5,float param6,float param7,float param8,float param9,float param10)=init_ov_glReplacementCodeuiColor4fNormal3fVertex3fSUN;

void __gl_ml_glReplacementCodeuiColor4fNormal3fVertex3fSUN(value v0,value v1,value v2,value v3,value v4,value v5,value v6,value v7,value v8,value v9,value v10)
{
	unsigned int l0=Int_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	float l3=Double_val(v3);
	float l4=Double_val(v4);
	float l5=Double_val(v5);
	float l6=Double_val(v6);
	float l7=Double_val(v7);
	float l8=Double_val(v8);
	float l9=Double_val(v9);
	float l10=Double_val(v10);
	ov_glReplacementCodeuiColor4fNormal3fVertex3fSUN(l0,l1,l2,l3,l4,l5,l6,l7,l8,l9,l10);
}

void byte___gl_ml_glReplacementCodeuiColor4fNormal3fVertex3fSUN(value * argv,int argn)
{
	__gl_ml_glReplacementCodeuiColor4fNormal3fVertex3fSUN(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6],argv[7],argv[8],argv[9],argv[10]);
}
extern void (APIENTRY *ov_glReplacementCodeuiNormal3fVertex3fvSUN)(unsigned int* param0,float* param1,float* param2);

static void APIENTRY init_ov_glReplacementCodeuiNormal3fVertex3fvSUN(unsigned int* param0,float* param1,float* param2)
{
	void *f;
	f=(void*)getfunc("glReplacementCodeuiNormal3fVertex3fvSUN");
	ov_glReplacementCodeuiNormal3fVertex3fvSUN=f;
	ov_glReplacementCodeuiNormal3fVertex3fvSUN(param0,param1,param2);
}
void (APIENTRY *ov_glReplacementCodeuiNormal3fVertex3fvSUN)(unsigned int* param0,float* param1,float* param2)=init_ov_glReplacementCodeuiNormal3fVertex3fvSUN;

void __gl_ml_glReplacementCodeuiNormal3fVertex3fvSUN(value v0,value v1,value v2)
{
	unsigned int* l0=(unsigned int*)(gl__ba__(v0));
	float* l1=(float*)(gl__ba__(v1));
	float* l2=(float*)(gl__ba__(v2));
	ov_glReplacementCodeuiNormal3fVertex3fvSUN(l0,l1,l2);
}
extern void (APIENTRY *ov_glReplacementCodeuiNormal3fVertex3fSUN)(unsigned int param0,float param1,float param2,float param3,float param4,float param5,float param6);

static void APIENTRY init_ov_glReplacementCodeuiNormal3fVertex3fSUN(unsigned int param0,float param1,float param2,float param3,float param4,float param5,float param6)
{
	void *f;
	f=(void*)getfunc("glReplacementCodeuiNormal3fVertex3fSUN");
	ov_glReplacementCodeuiNormal3fVertex3fSUN=f;
	ov_glReplacementCodeuiNormal3fVertex3fSUN(param0,param1,param2,param3,param4,param5,param6);
}
void (APIENTRY *ov_glReplacementCodeuiNormal3fVertex3fSUN)(unsigned int param0,float param1,float param2,float param3,float param4,float param5,float param6)=init_ov_glReplacementCodeuiNormal3fVertex3fSUN;

void __gl_ml_glReplacementCodeuiNormal3fVertex3fSUN(value v0,value v1,value v2,value v3,value v4,value v5,value v6)
{
	unsigned int l0=Int_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	float l3=Double_val(v3);
	float l4=Double_val(v4);
	float l5=Double_val(v5);
	float l6=Double_val(v6);
	ov_glReplacementCodeuiNormal3fVertex3fSUN(l0,l1,l2,l3,l4,l5,l6);
}

void byte___gl_ml_glReplacementCodeuiNormal3fVertex3fSUN(value * argv,int argn)
{
	__gl_ml_glReplacementCodeuiNormal3fVertex3fSUN(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6]);
}
extern void (APIENTRY *ov_glReplacementCodeuiColor3fVertex3fvSUN)(unsigned int* param0,float* param1,float* param2);

static void APIENTRY init_ov_glReplacementCodeuiColor3fVertex3fvSUN(unsigned int* param0,float* param1,float* param2)
{
	void *f;
	f=(void*)getfunc("glReplacementCodeuiColor3fVertex3fvSUN");
	ov_glReplacementCodeuiColor3fVertex3fvSUN=f;
	ov_glReplacementCodeuiColor3fVertex3fvSUN(param0,param1,param2);
}
void (APIENTRY *ov_glReplacementCodeuiColor3fVertex3fvSUN)(unsigned int* param0,float* param1,float* param2)=init_ov_glReplacementCodeuiColor3fVertex3fvSUN;

void __gl_ml_glReplacementCodeuiColor3fVertex3fvSUN(value v0,value v1,value v2)
{
	unsigned int* l0=(unsigned int*)(gl__ba__(v0));
	float* l1=(float*)(gl__ba__(v1));
	float* l2=(float*)(gl__ba__(v2));
	ov_glReplacementCodeuiColor3fVertex3fvSUN(l0,l1,l2);
}
extern void (APIENTRY *ov_glReplacementCodeuiColor3fVertex3fSUN)(unsigned int param0,float param1,float param2,float param3,float param4,float param5,float param6);

static void APIENTRY init_ov_glReplacementCodeuiColor3fVertex3fSUN(unsigned int param0,float param1,float param2,float param3,float param4,float param5,float param6)
{
	void *f;
	f=(void*)getfunc("glReplacementCodeuiColor3fVertex3fSUN");
	ov_glReplacementCodeuiColor3fVertex3fSUN=f;
	ov_glReplacementCodeuiColor3fVertex3fSUN(param0,param1,param2,param3,param4,param5,param6);
}
void (APIENTRY *ov_glReplacementCodeuiColor3fVertex3fSUN)(unsigned int param0,float param1,float param2,float param3,float param4,float param5,float param6)=init_ov_glReplacementCodeuiColor3fVertex3fSUN;

void __gl_ml_glReplacementCodeuiColor3fVertex3fSUN(value v0,value v1,value v2,value v3,value v4,value v5,value v6)
{
	unsigned int l0=Int_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	float l3=Double_val(v3);
	float l4=Double_val(v4);
	float l5=Double_val(v5);
	float l6=Double_val(v6);
	ov_glReplacementCodeuiColor3fVertex3fSUN(l0,l1,l2,l3,l4,l5,l6);
}

void byte___gl_ml_glReplacementCodeuiColor3fVertex3fSUN(value * argv,int argn)
{
	__gl_ml_glReplacementCodeuiColor3fVertex3fSUN(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6]);
}
extern void (APIENTRY *ov_glReplacementCodeuiColor4ubVertex3fvSUN)(unsigned int* param0,unsigned char* param1,float* param2);

static void APIENTRY init_ov_glReplacementCodeuiColor4ubVertex3fvSUN(unsigned int* param0,unsigned char* param1,float* param2)
{
	void *f;
	f=(void*)getfunc("glReplacementCodeuiColor4ubVertex3fvSUN");
	ov_glReplacementCodeuiColor4ubVertex3fvSUN=f;
	ov_glReplacementCodeuiColor4ubVertex3fvSUN(param0,param1,param2);
}
void (APIENTRY *ov_glReplacementCodeuiColor4ubVertex3fvSUN)(unsigned int* param0,unsigned char* param1,float* param2)=init_ov_glReplacementCodeuiColor4ubVertex3fvSUN;

void __gl_ml_glReplacementCodeuiColor4ubVertex3fvSUN(value v0,value v1,value v2)
{
	unsigned int* l0=(unsigned int*)(gl__ba__(v0));
	unsigned char* l1=(unsigned char*)(gl__ba__(v1));
	float* l2=(float*)(gl__ba__(v2));
	ov_glReplacementCodeuiColor4ubVertex3fvSUN(l0,l1,l2);
}
extern void (APIENTRY *ov_glReplacementCodeuiColor4ubVertex3fSUN)(unsigned int param0,unsigned char param1,unsigned char param2,unsigned char param3,unsigned char param4,float param5,float param6,float param7);

static void APIENTRY init_ov_glReplacementCodeuiColor4ubVertex3fSUN(unsigned int param0,unsigned char param1,unsigned char param2,unsigned char param3,unsigned char param4,float param5,float param6,float param7)
{
	void *f;
	f=(void*)getfunc("glReplacementCodeuiColor4ubVertex3fSUN");
	ov_glReplacementCodeuiColor4ubVertex3fSUN=f;
	ov_glReplacementCodeuiColor4ubVertex3fSUN(param0,param1,param2,param3,param4,param5,param6,param7);
}
void (APIENTRY *ov_glReplacementCodeuiColor4ubVertex3fSUN)(unsigned int param0,unsigned char param1,unsigned char param2,unsigned char param3,unsigned char param4,float param5,float param6,float param7)=init_ov_glReplacementCodeuiColor4ubVertex3fSUN;

void __gl_ml_glReplacementCodeuiColor4ubVertex3fSUN(value v0,value v1,value v2,value v3,value v4,value v5,value v6,value v7)
{
	unsigned int l0=Int_val(v0);
	unsigned char l1=Int_val(v1);
	unsigned char l2=Int_val(v2);
	unsigned char l3=Int_val(v3);
	unsigned char l4=Int_val(v4);
	float l5=Double_val(v5);
	float l6=Double_val(v6);
	float l7=Double_val(v7);
	ov_glReplacementCodeuiColor4ubVertex3fSUN(l0,l1,l2,l3,l4,l5,l6,l7);
}

void byte___gl_ml_glReplacementCodeuiColor4ubVertex3fSUN(value * argv,int argn)
{
	__gl_ml_glReplacementCodeuiColor4ubVertex3fSUN(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6],argv[7]);
}
extern void (APIENTRY *ov_glReplacementCodeuiVertex3fvSUN)(unsigned int* param0,float* param1);

static void APIENTRY init_ov_glReplacementCodeuiVertex3fvSUN(unsigned int* param0,float* param1)
{
	void *f;
	f=(void*)getfunc("glReplacementCodeuiVertex3fvSUN");
	ov_glReplacementCodeuiVertex3fvSUN=f;
	ov_glReplacementCodeuiVertex3fvSUN(param0,param1);
}
void (APIENTRY *ov_glReplacementCodeuiVertex3fvSUN)(unsigned int* param0,float* param1)=init_ov_glReplacementCodeuiVertex3fvSUN;

void __gl_ml_glReplacementCodeuiVertex3fvSUN(value v0,value v1)
{
	unsigned int* l0=(unsigned int*)(gl__ba__(v0));
	float* l1=(float*)(gl__ba__(v1));
	ov_glReplacementCodeuiVertex3fvSUN(l0,l1);
}
extern void (APIENTRY *ov_glReplacementCodeuiVertex3fSUN)(unsigned int param0,float param1,float param2,float param3);

static void APIENTRY init_ov_glReplacementCodeuiVertex3fSUN(unsigned int param0,float param1,float param2,float param3)
{
	void *f;
	f=(void*)getfunc("glReplacementCodeuiVertex3fSUN");
	ov_glReplacementCodeuiVertex3fSUN=f;
	ov_glReplacementCodeuiVertex3fSUN(param0,param1,param2,param3);
}
void (APIENTRY *ov_glReplacementCodeuiVertex3fSUN)(unsigned int param0,float param1,float param2,float param3)=init_ov_glReplacementCodeuiVertex3fSUN;

void __gl_ml_glReplacementCodeuiVertex3fSUN(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	float l3=Double_val(v3);
	ov_glReplacementCodeuiVertex3fSUN(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glTexCoord4fColor4fNormal3fVertex4fvSUN)(float* param0,float* param1,float* param2,float* param3);

static void APIENTRY init_ov_glTexCoord4fColor4fNormal3fVertex4fvSUN(float* param0,float* param1,float* param2,float* param3)
{
	void *f;
	f=(void*)getfunc("glTexCoord4fColor4fNormal3fVertex4fvSUN");
	ov_glTexCoord4fColor4fNormal3fVertex4fvSUN=f;
	ov_glTexCoord4fColor4fNormal3fVertex4fvSUN(param0,param1,param2,param3);
}
void (APIENTRY *ov_glTexCoord4fColor4fNormal3fVertex4fvSUN)(float* param0,float* param1,float* param2,float* param3)=init_ov_glTexCoord4fColor4fNormal3fVertex4fvSUN;

void __gl_ml_glTexCoord4fColor4fNormal3fVertex4fvSUN(value v0,value v1,value v2,value v3)
{
	float* l0=(float*)(gl__ba__(v0));
	float* l1=(float*)(gl__ba__(v1));
	float* l2=(float*)(gl__ba__(v2));
	float* l3=(float*)(gl__ba__(v3));
	ov_glTexCoord4fColor4fNormal3fVertex4fvSUN(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glTexCoord4fColor4fNormal3fVertex4fSUN)(float param0,float param1,float param2,float param3,float param4,float param5,float param6,float param7,float param8,float param9,float param10,float param11,float param12,float param13,float param14);

static void APIENTRY init_ov_glTexCoord4fColor4fNormal3fVertex4fSUN(float param0,float param1,float param2,float param3,float param4,float param5,float param6,float param7,float param8,float param9,float param10,float param11,float param12,float param13,float param14)
{
	void *f;
	f=(void*)getfunc("glTexCoord4fColor4fNormal3fVertex4fSUN");
	ov_glTexCoord4fColor4fNormal3fVertex4fSUN=f;
	ov_glTexCoord4fColor4fNormal3fVertex4fSUN(param0,param1,param2,param3,param4,param5,param6,param7,param8,param9,param10,param11,param12,param13,param14);
}
void (APIENTRY *ov_glTexCoord4fColor4fNormal3fVertex4fSUN)(float param0,float param1,float param2,float param3,float param4,float param5,float param6,float param7,float param8,float param9,float param10,float param11,float param12,float param13,float param14)=init_ov_glTexCoord4fColor4fNormal3fVertex4fSUN;

void __gl_ml_glTexCoord4fColor4fNormal3fVertex4fSUN(value v0,value v1,value v2,value v3,value v4,value v5,value v6,value v7,value v8,value v9,value v10,value v11,value v12,value v13,value v14)
{
	float l0=Double_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	float l3=Double_val(v3);
	float l4=Double_val(v4);
	float l5=Double_val(v5);
	float l6=Double_val(v6);
	float l7=Double_val(v7);
	float l8=Double_val(v8);
	float l9=Double_val(v9);
	float l10=Double_val(v10);
	float l11=Double_val(v11);
	float l12=Double_val(v12);
	float l13=Double_val(v13);
	float l14=Double_val(v14);
	ov_glTexCoord4fColor4fNormal3fVertex4fSUN(l0,l1,l2,l3,l4,l5,l6,l7,l8,l9,l10,l11,l12,l13,l14);
}

void byte___gl_ml_glTexCoord4fColor4fNormal3fVertex4fSUN(value * argv,int argn)
{
	__gl_ml_glTexCoord4fColor4fNormal3fVertex4fSUN(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6],argv[7],argv[8],argv[9],argv[10],argv[11],argv[12],argv[13],argv[14]);
}
extern void (APIENTRY *ov_glTexCoord2fColor4fNormal3fVertex3fvSUN)(float* param0,float* param1,float* param2,float* param3);

static void APIENTRY init_ov_glTexCoord2fColor4fNormal3fVertex3fvSUN(float* param0,float* param1,float* param2,float* param3)
{
	void *f;
	f=(void*)getfunc("glTexCoord2fColor4fNormal3fVertex3fvSUN");
	ov_glTexCoord2fColor4fNormal3fVertex3fvSUN=f;
	ov_glTexCoord2fColor4fNormal3fVertex3fvSUN(param0,param1,param2,param3);
}
void (APIENTRY *ov_glTexCoord2fColor4fNormal3fVertex3fvSUN)(float* param0,float* param1,float* param2,float* param3)=init_ov_glTexCoord2fColor4fNormal3fVertex3fvSUN;

void __gl_ml_glTexCoord2fColor4fNormal3fVertex3fvSUN(value v0,value v1,value v2,value v3)
{
	float* l0=(float*)(gl__ba__(v0));
	float* l1=(float*)(gl__ba__(v1));
	float* l2=(float*)(gl__ba__(v2));
	float* l3=(float*)(gl__ba__(v3));
	ov_glTexCoord2fColor4fNormal3fVertex3fvSUN(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glTexCoord2fColor4fNormal3fVertex3fSUN)(float param0,float param1,float param2,float param3,float param4,float param5,float param6,float param7,float param8,float param9,float param10,float param11);

static void APIENTRY init_ov_glTexCoord2fColor4fNormal3fVertex3fSUN(float param0,float param1,float param2,float param3,float param4,float param5,float param6,float param7,float param8,float param9,float param10,float param11)
{
	void *f;
	f=(void*)getfunc("glTexCoord2fColor4fNormal3fVertex3fSUN");
	ov_glTexCoord2fColor4fNormal3fVertex3fSUN=f;
	ov_glTexCoord2fColor4fNormal3fVertex3fSUN(param0,param1,param2,param3,param4,param5,param6,param7,param8,param9,param10,param11);
}
void (APIENTRY *ov_glTexCoord2fColor4fNormal3fVertex3fSUN)(float param0,float param1,float param2,float param3,float param4,float param5,float param6,float param7,float param8,float param9,float param10,float param11)=init_ov_glTexCoord2fColor4fNormal3fVertex3fSUN;

void __gl_ml_glTexCoord2fColor4fNormal3fVertex3fSUN(value v0,value v1,value v2,value v3,value v4,value v5,value v6,value v7,value v8,value v9,value v10,value v11)
{
	float l0=Double_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	float l3=Double_val(v3);
	float l4=Double_val(v4);
	float l5=Double_val(v5);
	float l6=Double_val(v6);
	float l7=Double_val(v7);
	float l8=Double_val(v8);
	float l9=Double_val(v9);
	float l10=Double_val(v10);
	float l11=Double_val(v11);
	ov_glTexCoord2fColor4fNormal3fVertex3fSUN(l0,l1,l2,l3,l4,l5,l6,l7,l8,l9,l10,l11);
}

void byte___gl_ml_glTexCoord2fColor4fNormal3fVertex3fSUN(value * argv,int argn)
{
	__gl_ml_glTexCoord2fColor4fNormal3fVertex3fSUN(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6],argv[7],argv[8],argv[9],argv[10],argv[11]);
}
extern void (APIENTRY *ov_glTexCoord2fNormal3fVertex3fvSUN)(float* param0,float* param1,float* param2);

static void APIENTRY init_ov_glTexCoord2fNormal3fVertex3fvSUN(float* param0,float* param1,float* param2)
{
	void *f;
	f=(void*)getfunc("glTexCoord2fNormal3fVertex3fvSUN");
	ov_glTexCoord2fNormal3fVertex3fvSUN=f;
	ov_glTexCoord2fNormal3fVertex3fvSUN(param0,param1,param2);
}
void (APIENTRY *ov_glTexCoord2fNormal3fVertex3fvSUN)(float* param0,float* param1,float* param2)=init_ov_glTexCoord2fNormal3fVertex3fvSUN;

void __gl_ml_glTexCoord2fNormal3fVertex3fvSUN(value v0,value v1,value v2)
{
	float* l0=(float*)(gl__ba__(v0));
	float* l1=(float*)(gl__ba__(v1));
	float* l2=(float*)(gl__ba__(v2));
	ov_glTexCoord2fNormal3fVertex3fvSUN(l0,l1,l2);
}
extern void (APIENTRY *ov_glTexCoord2fNormal3fVertex3fSUN)(float param0,float param1,float param2,float param3,float param4,float param5,float param6,float param7);

static void APIENTRY init_ov_glTexCoord2fNormal3fVertex3fSUN(float param0,float param1,float param2,float param3,float param4,float param5,float param6,float param7)
{
	void *f;
	f=(void*)getfunc("glTexCoord2fNormal3fVertex3fSUN");
	ov_glTexCoord2fNormal3fVertex3fSUN=f;
	ov_glTexCoord2fNormal3fVertex3fSUN(param0,param1,param2,param3,param4,param5,param6,param7);
}
void (APIENTRY *ov_glTexCoord2fNormal3fVertex3fSUN)(float param0,float param1,float param2,float param3,float param4,float param5,float param6,float param7)=init_ov_glTexCoord2fNormal3fVertex3fSUN;

void __gl_ml_glTexCoord2fNormal3fVertex3fSUN(value v0,value v1,value v2,value v3,value v4,value v5,value v6,value v7)
{
	float l0=Double_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	float l3=Double_val(v3);
	float l4=Double_val(v4);
	float l5=Double_val(v5);
	float l6=Double_val(v6);
	float l7=Double_val(v7);
	ov_glTexCoord2fNormal3fVertex3fSUN(l0,l1,l2,l3,l4,l5,l6,l7);
}

void byte___gl_ml_glTexCoord2fNormal3fVertex3fSUN(value * argv,int argn)
{
	__gl_ml_glTexCoord2fNormal3fVertex3fSUN(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6],argv[7]);
}
extern void (APIENTRY *ov_glTexCoord2fColor3fVertex3fvSUN)(float* param0,float* param1,float* param2);

static void APIENTRY init_ov_glTexCoord2fColor3fVertex3fvSUN(float* param0,float* param1,float* param2)
{
	void *f;
	f=(void*)getfunc("glTexCoord2fColor3fVertex3fvSUN");
	ov_glTexCoord2fColor3fVertex3fvSUN=f;
	ov_glTexCoord2fColor3fVertex3fvSUN(param0,param1,param2);
}
void (APIENTRY *ov_glTexCoord2fColor3fVertex3fvSUN)(float* param0,float* param1,float* param2)=init_ov_glTexCoord2fColor3fVertex3fvSUN;

void __gl_ml_glTexCoord2fColor3fVertex3fvSUN(value v0,value v1,value v2)
{
	float* l0=(float*)(gl__ba__(v0));
	float* l1=(float*)(gl__ba__(v1));
	float* l2=(float*)(gl__ba__(v2));
	ov_glTexCoord2fColor3fVertex3fvSUN(l0,l1,l2);
}
extern void (APIENTRY *ov_glTexCoord2fColor3fVertex3fSUN)(float param0,float param1,float param2,float param3,float param4,float param5,float param6,float param7);

static void APIENTRY init_ov_glTexCoord2fColor3fVertex3fSUN(float param0,float param1,float param2,float param3,float param4,float param5,float param6,float param7)
{
	void *f;
	f=(void*)getfunc("glTexCoord2fColor3fVertex3fSUN");
	ov_glTexCoord2fColor3fVertex3fSUN=f;
	ov_glTexCoord2fColor3fVertex3fSUN(param0,param1,param2,param3,param4,param5,param6,param7);
}
void (APIENTRY *ov_glTexCoord2fColor3fVertex3fSUN)(float param0,float param1,float param2,float param3,float param4,float param5,float param6,float param7)=init_ov_glTexCoord2fColor3fVertex3fSUN;

void __gl_ml_glTexCoord2fColor3fVertex3fSUN(value v0,value v1,value v2,value v3,value v4,value v5,value v6,value v7)
{
	float l0=Double_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	float l3=Double_val(v3);
	float l4=Double_val(v4);
	float l5=Double_val(v5);
	float l6=Double_val(v6);
	float l7=Double_val(v7);
	ov_glTexCoord2fColor3fVertex3fSUN(l0,l1,l2,l3,l4,l5,l6,l7);
}

void byte___gl_ml_glTexCoord2fColor3fVertex3fSUN(value * argv,int argn)
{
	__gl_ml_glTexCoord2fColor3fVertex3fSUN(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6],argv[7]);
}
extern void (APIENTRY *ov_glTexCoord2fColor4ubVertex3fvSUN)(float* param0,unsigned char* param1,float* param2);

static void APIENTRY init_ov_glTexCoord2fColor4ubVertex3fvSUN(float* param0,unsigned char* param1,float* param2)
{
	void *f;
	f=(void*)getfunc("glTexCoord2fColor4ubVertex3fvSUN");
	ov_glTexCoord2fColor4ubVertex3fvSUN=f;
	ov_glTexCoord2fColor4ubVertex3fvSUN(param0,param1,param2);
}
void (APIENTRY *ov_glTexCoord2fColor4ubVertex3fvSUN)(float* param0,unsigned char* param1,float* param2)=init_ov_glTexCoord2fColor4ubVertex3fvSUN;

void __gl_ml_glTexCoord2fColor4ubVertex3fvSUN(value v0,value v1,value v2)
{
	float* l0=(float*)(gl__ba__(v0));
	unsigned char* l1=(unsigned char*)(gl__ba__(v1));
	float* l2=(float*)(gl__ba__(v2));
	ov_glTexCoord2fColor4ubVertex3fvSUN(l0,l1,l2);
}
extern void (APIENTRY *ov_glTexCoord2fColor4ubVertex3fSUN)(float param0,float param1,unsigned char param2,unsigned char param3,unsigned char param4,unsigned char param5,float param6,float param7,float param8);

static void APIENTRY init_ov_glTexCoord2fColor4ubVertex3fSUN(float param0,float param1,unsigned char param2,unsigned char param3,unsigned char param4,unsigned char param5,float param6,float param7,float param8)
{
	void *f;
	f=(void*)getfunc("glTexCoord2fColor4ubVertex3fSUN");
	ov_glTexCoord2fColor4ubVertex3fSUN=f;
	ov_glTexCoord2fColor4ubVertex3fSUN(param0,param1,param2,param3,param4,param5,param6,param7,param8);
}
void (APIENTRY *ov_glTexCoord2fColor4ubVertex3fSUN)(float param0,float param1,unsigned char param2,unsigned char param3,unsigned char param4,unsigned char param5,float param6,float param7,float param8)=init_ov_glTexCoord2fColor4ubVertex3fSUN;

void __gl_ml_glTexCoord2fColor4ubVertex3fSUN(value v0,value v1,value v2,value v3,value v4,value v5,value v6,value v7,value v8)
{
	float l0=Double_val(v0);
	float l1=Double_val(v1);
	unsigned char l2=Int_val(v2);
	unsigned char l3=Int_val(v3);
	unsigned char l4=Int_val(v4);
	unsigned char l5=Int_val(v5);
	float l6=Double_val(v6);
	float l7=Double_val(v7);
	float l8=Double_val(v8);
	ov_glTexCoord2fColor4ubVertex3fSUN(l0,l1,l2,l3,l4,l5,l6,l7,l8);
}

void byte___gl_ml_glTexCoord2fColor4ubVertex3fSUN(value * argv,int argn)
{
	__gl_ml_glTexCoord2fColor4ubVertex3fSUN(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6],argv[7],argv[8]);
}
extern void (APIENTRY *ov_glTexCoord4fVertex4fvSUN)(float* param0,float* param1);

static void APIENTRY init_ov_glTexCoord4fVertex4fvSUN(float* param0,float* param1)
{
	void *f;
	f=(void*)getfunc("glTexCoord4fVertex4fvSUN");
	ov_glTexCoord4fVertex4fvSUN=f;
	ov_glTexCoord4fVertex4fvSUN(param0,param1);
}
void (APIENTRY *ov_glTexCoord4fVertex4fvSUN)(float* param0,float* param1)=init_ov_glTexCoord4fVertex4fvSUN;

void __gl_ml_glTexCoord4fVertex4fvSUN(value v0,value v1)
{
	float* l0=(float*)(gl__ba__(v0));
	float* l1=(float*)(gl__ba__(v1));
	ov_glTexCoord4fVertex4fvSUN(l0,l1);
}
extern void (APIENTRY *ov_glTexCoord4fVertex4fSUN)(float param0,float param1,float param2,float param3,float param4,float param5,float param6,float param7);

static void APIENTRY init_ov_glTexCoord4fVertex4fSUN(float param0,float param1,float param2,float param3,float param4,float param5,float param6,float param7)
{
	void *f;
	f=(void*)getfunc("glTexCoord4fVertex4fSUN");
	ov_glTexCoord4fVertex4fSUN=f;
	ov_glTexCoord4fVertex4fSUN(param0,param1,param2,param3,param4,param5,param6,param7);
}
void (APIENTRY *ov_glTexCoord4fVertex4fSUN)(float param0,float param1,float param2,float param3,float param4,float param5,float param6,float param7)=init_ov_glTexCoord4fVertex4fSUN;

void __gl_ml_glTexCoord4fVertex4fSUN(value v0,value v1,value v2,value v3,value v4,value v5,value v6,value v7)
{
	float l0=Double_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	float l3=Double_val(v3);
	float l4=Double_val(v4);
	float l5=Double_val(v5);
	float l6=Double_val(v6);
	float l7=Double_val(v7);
	ov_glTexCoord4fVertex4fSUN(l0,l1,l2,l3,l4,l5,l6,l7);
}

void byte___gl_ml_glTexCoord4fVertex4fSUN(value * argv,int argn)
{
	__gl_ml_glTexCoord4fVertex4fSUN(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6],argv[7]);
}
extern void (APIENTRY *ov_glTexCoord2fVertex3fvSUN)(float* param0,float* param1);

static void APIENTRY init_ov_glTexCoord2fVertex3fvSUN(float* param0,float* param1)
{
	void *f;
	f=(void*)getfunc("glTexCoord2fVertex3fvSUN");
	ov_glTexCoord2fVertex3fvSUN=f;
	ov_glTexCoord2fVertex3fvSUN(param0,param1);
}
void (APIENTRY *ov_glTexCoord2fVertex3fvSUN)(float* param0,float* param1)=init_ov_glTexCoord2fVertex3fvSUN;

void __gl_ml_glTexCoord2fVertex3fvSUN(value v0,value v1)
{
	float* l0=(float*)(gl__ba__(v0));
	float* l1=(float*)(gl__ba__(v1));
	ov_glTexCoord2fVertex3fvSUN(l0,l1);
}
extern void (APIENTRY *ov_glTexCoord2fVertex3fSUN)(float param0,float param1,float param2,float param3,float param4);

static void APIENTRY init_ov_glTexCoord2fVertex3fSUN(float param0,float param1,float param2,float param3,float param4)
{
	void *f;
	f=(void*)getfunc("glTexCoord2fVertex3fSUN");
	ov_glTexCoord2fVertex3fSUN=f;
	ov_glTexCoord2fVertex3fSUN(param0,param1,param2,param3,param4);
}
void (APIENTRY *ov_glTexCoord2fVertex3fSUN)(float param0,float param1,float param2,float param3,float param4)=init_ov_glTexCoord2fVertex3fSUN;

void __gl_ml_glTexCoord2fVertex3fSUN(value v0,value v1,value v2,value v3,value v4)
{
	float l0=Double_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	float l3=Double_val(v3);
	float l4=Double_val(v4);
	ov_glTexCoord2fVertex3fSUN(l0,l1,l2,l3,l4);
}
extern void (APIENTRY *ov_glColor4fNormal3fVertex3fvSUN)(float* param0,float* param1,float* param2);

static void APIENTRY init_ov_glColor4fNormal3fVertex3fvSUN(float* param0,float* param1,float* param2)
{
	void *f;
	f=(void*)getfunc("glColor4fNormal3fVertex3fvSUN");
	ov_glColor4fNormal3fVertex3fvSUN=f;
	ov_glColor4fNormal3fVertex3fvSUN(param0,param1,param2);
}
void (APIENTRY *ov_glColor4fNormal3fVertex3fvSUN)(float* param0,float* param1,float* param2)=init_ov_glColor4fNormal3fVertex3fvSUN;

void __gl_ml_glColor4fNormal3fVertex3fvSUN(value v0,value v1,value v2)
{
	float* l0=(float*)(gl__ba__(v0));
	float* l1=(float*)(gl__ba__(v1));
	float* l2=(float*)(gl__ba__(v2));
	ov_glColor4fNormal3fVertex3fvSUN(l0,l1,l2);
}
extern void (APIENTRY *ov_glColor4fNormal3fVertex3fSUN)(float param0,float param1,float param2,float param3,float param4,float param5,float param6,float param7,float param8,float param9);

static void APIENTRY init_ov_glColor4fNormal3fVertex3fSUN(float param0,float param1,float param2,float param3,float param4,float param5,float param6,float param7,float param8,float param9)
{
	void *f;
	f=(void*)getfunc("glColor4fNormal3fVertex3fSUN");
	ov_glColor4fNormal3fVertex3fSUN=f;
	ov_glColor4fNormal3fVertex3fSUN(param0,param1,param2,param3,param4,param5,param6,param7,param8,param9);
}
void (APIENTRY *ov_glColor4fNormal3fVertex3fSUN)(float param0,float param1,float param2,float param3,float param4,float param5,float param6,float param7,float param8,float param9)=init_ov_glColor4fNormal3fVertex3fSUN;

void __gl_ml_glColor4fNormal3fVertex3fSUN(value v0,value v1,value v2,value v3,value v4,value v5,value v6,value v7,value v8,value v9)
{
	float l0=Double_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	float l3=Double_val(v3);
	float l4=Double_val(v4);
	float l5=Double_val(v5);
	float l6=Double_val(v6);
	float l7=Double_val(v7);
	float l8=Double_val(v8);
	float l9=Double_val(v9);
	ov_glColor4fNormal3fVertex3fSUN(l0,l1,l2,l3,l4,l5,l6,l7,l8,l9);
}

void byte___gl_ml_glColor4fNormal3fVertex3fSUN(value * argv,int argn)
{
	__gl_ml_glColor4fNormal3fVertex3fSUN(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6],argv[7],argv[8],argv[9]);
}
extern void (APIENTRY *ov_glNormal3fVertex3fvSUN)(float* param0,float* param1);

static void APIENTRY init_ov_glNormal3fVertex3fvSUN(float* param0,float* param1)
{
	void *f;
	f=(void*)getfunc("glNormal3fVertex3fvSUN");
	ov_glNormal3fVertex3fvSUN=f;
	ov_glNormal3fVertex3fvSUN(param0,param1);
}
void (APIENTRY *ov_glNormal3fVertex3fvSUN)(float* param0,float* param1)=init_ov_glNormal3fVertex3fvSUN;

void __gl_ml_glNormal3fVertex3fvSUN(value v0,value v1)
{
	float* l0=(float*)(gl__ba__(v0));
	float* l1=(float*)(gl__ba__(v1));
	ov_glNormal3fVertex3fvSUN(l0,l1);
}
extern void (APIENTRY *ov_glNormal3fVertex3fSUN)(float param0,float param1,float param2,float param3,float param4,float param5);

static void APIENTRY init_ov_glNormal3fVertex3fSUN(float param0,float param1,float param2,float param3,float param4,float param5)
{
	void *f;
	f=(void*)getfunc("glNormal3fVertex3fSUN");
	ov_glNormal3fVertex3fSUN=f;
	ov_glNormal3fVertex3fSUN(param0,param1,param2,param3,param4,param5);
}
void (APIENTRY *ov_glNormal3fVertex3fSUN)(float param0,float param1,float param2,float param3,float param4,float param5)=init_ov_glNormal3fVertex3fSUN;

void __gl_ml_glNormal3fVertex3fSUN(value v0,value v1,value v2,value v3,value v4,value v5)
{
	float l0=Double_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	float l3=Double_val(v3);
	float l4=Double_val(v4);
	float l5=Double_val(v5);
	ov_glNormal3fVertex3fSUN(l0,l1,l2,l3,l4,l5);
}

void byte___gl_ml_glNormal3fVertex3fSUN(value * argv,int argn)
{
	__gl_ml_glNormal3fVertex3fSUN(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5]);
}
extern void (APIENTRY *ov_glColor3fVertex3fvSUN)(float* param0,float* param1);

static void APIENTRY init_ov_glColor3fVertex3fvSUN(float* param0,float* param1)
{
	void *f;
	f=(void*)getfunc("glColor3fVertex3fvSUN");
	ov_glColor3fVertex3fvSUN=f;
	ov_glColor3fVertex3fvSUN(param0,param1);
}
void (APIENTRY *ov_glColor3fVertex3fvSUN)(float* param0,float* param1)=init_ov_glColor3fVertex3fvSUN;

void __gl_ml_glColor3fVertex3fvSUN(value v0,value v1)
{
	float* l0=(float*)(gl__ba__(v0));
	float* l1=(float*)(gl__ba__(v1));
	ov_glColor3fVertex3fvSUN(l0,l1);
}
extern void (APIENTRY *ov_glColor3fVertex3fSUN)(float param0,float param1,float param2,float param3,float param4,float param5);

static void APIENTRY init_ov_glColor3fVertex3fSUN(float param0,float param1,float param2,float param3,float param4,float param5)
{
	void *f;
	f=(void*)getfunc("glColor3fVertex3fSUN");
	ov_glColor3fVertex3fSUN=f;
	ov_glColor3fVertex3fSUN(param0,param1,param2,param3,param4,param5);
}
void (APIENTRY *ov_glColor3fVertex3fSUN)(float param0,float param1,float param2,float param3,float param4,float param5)=init_ov_glColor3fVertex3fSUN;

void __gl_ml_glColor3fVertex3fSUN(value v0,value v1,value v2,value v3,value v4,value v5)
{
	float l0=Double_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	float l3=Double_val(v3);
	float l4=Double_val(v4);
	float l5=Double_val(v5);
	ov_glColor3fVertex3fSUN(l0,l1,l2,l3,l4,l5);
}

void byte___gl_ml_glColor3fVertex3fSUN(value * argv,int argn)
{
	__gl_ml_glColor3fVertex3fSUN(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5]);
}
extern void (APIENTRY *ov_glColor4ubVertex3fvSUN)(unsigned char* param0,float* param1);

static void APIENTRY init_ov_glColor4ubVertex3fvSUN(unsigned char* param0,float* param1)
{
	void *f;
	f=(void*)getfunc("glColor4ubVertex3fvSUN");
	ov_glColor4ubVertex3fvSUN=f;
	ov_glColor4ubVertex3fvSUN(param0,param1);
}
void (APIENTRY *ov_glColor4ubVertex3fvSUN)(unsigned char* param0,float* param1)=init_ov_glColor4ubVertex3fvSUN;

void __gl_ml_glColor4ubVertex3fvSUN(value v0,value v1)
{
	unsigned char* l0=(unsigned char*)(gl__ba__(v0));
	float* l1=(float*)(gl__ba__(v1));
	ov_glColor4ubVertex3fvSUN(l0,l1);
}
extern void (APIENTRY *ov_glColor4ubVertex3fSUN)(unsigned char param0,unsigned char param1,unsigned char param2,unsigned char param3,float param4,float param5,float param6);

static void APIENTRY init_ov_glColor4ubVertex3fSUN(unsigned char param0,unsigned char param1,unsigned char param2,unsigned char param3,float param4,float param5,float param6)
{
	void *f;
	f=(void*)getfunc("glColor4ubVertex3fSUN");
	ov_glColor4ubVertex3fSUN=f;
	ov_glColor4ubVertex3fSUN(param0,param1,param2,param3,param4,param5,param6);
}
void (APIENTRY *ov_glColor4ubVertex3fSUN)(unsigned char param0,unsigned char param1,unsigned char param2,unsigned char param3,float param4,float param5,float param6)=init_ov_glColor4ubVertex3fSUN;

void __gl_ml_glColor4ubVertex3fSUN(value v0,value v1,value v2,value v3,value v4,value v5,value v6)
{
	unsigned char l0=Int_val(v0);
	unsigned char l1=Int_val(v1);
	unsigned char l2=Int_val(v2);
	unsigned char l3=Int_val(v3);
	float l4=Double_val(v4);
	float l5=Double_val(v5);
	float l6=Double_val(v6);
	ov_glColor4ubVertex3fSUN(l0,l1,l2,l3,l4,l5,l6);
}

void byte___gl_ml_glColor4ubVertex3fSUN(value * argv,int argn)
{
	__gl_ml_glColor4ubVertex3fSUN(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6]);
}
extern void (APIENTRY *ov_glColor4ubVertex2fvSUN)(unsigned char* param0,float* param1);

static void APIENTRY init_ov_glColor4ubVertex2fvSUN(unsigned char* param0,float* param1)
{
	void *f;
	f=(void*)getfunc("glColor4ubVertex2fvSUN");
	ov_glColor4ubVertex2fvSUN=f;
	ov_glColor4ubVertex2fvSUN(param0,param1);
}
void (APIENTRY *ov_glColor4ubVertex2fvSUN)(unsigned char* param0,float* param1)=init_ov_glColor4ubVertex2fvSUN;

void __gl_ml_glColor4ubVertex2fvSUN(value v0,value v1)
{
	unsigned char* l0=(unsigned char*)(gl__ba__(v0));
	float* l1=(float*)(gl__ba__(v1));
	ov_glColor4ubVertex2fvSUN(l0,l1);
}
extern void (APIENTRY *ov_glColor4ubVertex2fSUN)(unsigned char param0,unsigned char param1,unsigned char param2,unsigned char param3,float param4,float param5);

static void APIENTRY init_ov_glColor4ubVertex2fSUN(unsigned char param0,unsigned char param1,unsigned char param2,unsigned char param3,float param4,float param5)
{
	void *f;
	f=(void*)getfunc("glColor4ubVertex2fSUN");
	ov_glColor4ubVertex2fSUN=f;
	ov_glColor4ubVertex2fSUN(param0,param1,param2,param3,param4,param5);
}
void (APIENTRY *ov_glColor4ubVertex2fSUN)(unsigned char param0,unsigned char param1,unsigned char param2,unsigned char param3,float param4,float param5)=init_ov_glColor4ubVertex2fSUN;

void __gl_ml_glColor4ubVertex2fSUN(value v0,value v1,value v2,value v3,value v4,value v5)
{
	unsigned char l0=Int_val(v0);
	unsigned char l1=Int_val(v1);
	unsigned char l2=Int_val(v2);
	unsigned char l3=Int_val(v3);
	float l4=Double_val(v4);
	float l5=Double_val(v5);
	ov_glColor4ubVertex2fSUN(l0,l1,l2,l3,l4,l5);
}

void byte___gl_ml_glColor4ubVertex2fSUN(value * argv,int argn)
{
	__gl_ml_glColor4ubVertex2fSUN(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5]);
}
extern void (APIENTRY *ov_glFinishTextureSUNX)();

static void APIENTRY init_ov_glFinishTextureSUNX()
{
	void *f;
	f=(void*)getfunc("glFinishTextureSUNX");
	ov_glFinishTextureSUNX=f;
	ov_glFinishTextureSUNX();
}
void (APIENTRY *ov_glFinishTextureSUNX)()=init_ov_glFinishTextureSUNX;

void __gl_ml_glFinishTextureSUNX()
{
	ov_glFinishTextureSUNX();
}
extern void (APIENTRY *ov_glCopyTexSubImage3D)(unsigned int param0,signed int param1,signed int param2,signed int param3,signed int param4,signed int param5,signed int param6,signed int param7,signed int param8);

static void APIENTRY init_ov_glCopyTexSubImage3D(unsigned int param0,signed int param1,signed int param2,signed int param3,signed int param4,signed int param5,signed int param6,signed int param7,signed int param8)
{
	void *f;
	f=(void*)getfunc("glCopyTexSubImage3D");
	ov_glCopyTexSubImage3D=f;
	ov_glCopyTexSubImage3D(param0,param1,param2,param3,param4,param5,param6,param7,param8);
}
void (APIENTRY *ov_glCopyTexSubImage3D)(unsigned int param0,signed int param1,signed int param2,signed int param3,signed int param4,signed int param5,signed int param6,signed int param7,signed int param8)=init_ov_glCopyTexSubImage3D;

void __gl_ml_glCopyTexSubImage3D(value v0,value v1,value v2,value v3,value v4,value v5,value v6,value v7,value v8)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	signed int l4=Int_val(v4);
	signed int l5=Int_val(v5);
	signed int l6=Int_val(v6);
	signed int l7=Int_val(v7);
	signed int l8=Int_val(v8);
	ov_glCopyTexSubImage3D(l0,l1,l2,l3,l4,l5,l6,l7,l8);
}

void byte___gl_ml_glCopyTexSubImage3D(value * argv,int argn)
{
	__gl_ml_glCopyTexSubImage3D(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6],argv[7],argv[8]);
}
extern void (APIENTRY *ov_glTexSubImage3D)(unsigned int param0,signed int param1,signed int param2,signed int param3,signed int param4,signed int param5,signed int param6,signed int param7,unsigned int param8,unsigned int param9,void* param10);

static void APIENTRY init_ov_glTexSubImage3D(unsigned int param0,signed int param1,signed int param2,signed int param3,signed int param4,signed int param5,signed int param6,signed int param7,unsigned int param8,unsigned int param9,void* param10)
{
	void *f;
	f=(void*)getfunc("glTexSubImage3D");
	ov_glTexSubImage3D=f;
	ov_glTexSubImage3D(param0,param1,param2,param3,param4,param5,param6,param7,param8,param9,param10);
}
void (APIENTRY *ov_glTexSubImage3D)(unsigned int param0,signed int param1,signed int param2,signed int param3,signed int param4,signed int param5,signed int param6,signed int param7,unsigned int param8,unsigned int param9,void* param10)=init_ov_glTexSubImage3D;

void __gl_ml_glTexSubImage3D(value v0,value v1,value v2,value v3,value v4,value v5,value v6,value v7,value v8,value v9,value v10)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	signed int l4=Int_val(v4);
	signed int l5=Int_val(v5);
	signed int l6=Int_val(v6);
	signed int l7=Int_val(v7);
	unsigned int l8=Int_val(v8);
	unsigned int l9=Int_val(v9);
	void* l10=gl__ba__(v10);
	ov_glTexSubImage3D(l0,l1,l2,l3,l4,l5,l6,l7,l8,l9,l10);
}

void byte___gl_ml_glTexSubImage3D(value * argv,int argn)
{
	__gl_ml_glTexSubImage3D(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6],argv[7],argv[8],argv[9],argv[10]);
}
extern void (APIENTRY *ov_glTexImage3D)(unsigned int param0,signed int param1,signed int param2,signed int param3,signed int param4,signed int param5,signed int param6,unsigned int param7,unsigned int param8,void* param9);

static void APIENTRY init_ov_glTexImage3D(unsigned int param0,signed int param1,signed int param2,signed int param3,signed int param4,signed int param5,signed int param6,unsigned int param7,unsigned int param8,void* param9)
{
	void *f;
	f=(void*)getfunc("glTexImage3D");
	ov_glTexImage3D=f;
	ov_glTexImage3D(param0,param1,param2,param3,param4,param5,param6,param7,param8,param9);
}
void (APIENTRY *ov_glTexImage3D)(unsigned int param0,signed int param1,signed int param2,signed int param3,signed int param4,signed int param5,signed int param6,unsigned int param7,unsigned int param8,void* param9)=init_ov_glTexImage3D;

void __gl_ml_glTexImage3D(value v0,value v1,value v2,value v3,value v4,value v5,value v6,value v7,value v8,value v9)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	signed int l4=Int_val(v4);
	signed int l5=Int_val(v5);
	signed int l6=Int_val(v6);
	unsigned int l7=Int_val(v7);
	unsigned int l8=Int_val(v8);
	void* l9=gl__ba__(v9);
	ov_glTexImage3D(l0,l1,l2,l3,l4,l5,l6,l7,l8,l9);
}

void byte___gl_ml_glTexImage3D(value * argv,int argn)
{
	__gl_ml_glTexImage3D(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6],argv[7],argv[8],argv[9]);
}
extern void (APIENTRY *ov_glResetMinmax)(unsigned int param0);

static void APIENTRY init_ov_glResetMinmax(unsigned int param0)
{
	void *f;
	f=(void*)getfunc("glResetMinmax");
	ov_glResetMinmax=f;
	ov_glResetMinmax(param0);
}
void (APIENTRY *ov_glResetMinmax)(unsigned int param0)=init_ov_glResetMinmax;

void __gl_ml_glResetMinmax(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glResetMinmax(l0);
}
extern void (APIENTRY *ov_glResetHistogram)(unsigned int param0);

static void APIENTRY init_ov_glResetHistogram(unsigned int param0)
{
	void *f;
	f=(void*)getfunc("glResetHistogram");
	ov_glResetHistogram=f;
	ov_glResetHistogram(param0);
}
void (APIENTRY *ov_glResetHistogram)(unsigned int param0)=init_ov_glResetHistogram;

void __gl_ml_glResetHistogram(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glResetHistogram(l0);
}
extern void (APIENTRY *ov_glMinmax)(unsigned int param0,unsigned int param1,unsigned char param2);

static void APIENTRY init_ov_glMinmax(unsigned int param0,unsigned int param1,unsigned char param2)
{
	void *f;
	f=(void*)getfunc("glMinmax");
	ov_glMinmax=f;
	ov_glMinmax(param0,param1,param2);
}
void (APIENTRY *ov_glMinmax)(unsigned int param0,unsigned int param1,unsigned char param2)=init_ov_glMinmax;

void __gl_ml_glMinmax(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned char l2=Int_val(v2);
	ov_glMinmax(l0,l1,l2);
}
extern void (APIENTRY *ov_glHistogram)(unsigned int param0,signed int param1,unsigned int param2,unsigned char param3);

static void APIENTRY init_ov_glHistogram(unsigned int param0,signed int param1,unsigned int param2,unsigned char param3)
{
	void *f;
	f=(void*)getfunc("glHistogram");
	ov_glHistogram=f;
	ov_glHistogram(param0,param1,param2,param3);
}
void (APIENTRY *ov_glHistogram)(unsigned int param0,signed int param1,unsigned int param2,unsigned char param3)=init_ov_glHistogram;

void __gl_ml_glHistogram(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	unsigned char l3=Int_val(v3);
	ov_glHistogram(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glGetMinmaxParameteriv)(unsigned int param0,unsigned int param1,signed int* param2);

static void APIENTRY init_ov_glGetMinmaxParameteriv(unsigned int param0,unsigned int param1,signed int* param2)
{
	void *f;
	f=(void*)getfunc("glGetMinmaxParameteriv");
	ov_glGetMinmaxParameteriv=f;
	ov_glGetMinmaxParameteriv(param0,param1,param2);
}
void (APIENTRY *ov_glGetMinmaxParameteriv)(unsigned int param0,unsigned int param1,signed int* param2)=init_ov_glGetMinmaxParameteriv;

void __gl_ml_glGetMinmaxParameteriv(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int* l2=(signed int*)(gl__ba__(v2));
	ov_glGetMinmaxParameteriv(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetMinmaxParameterfv)(unsigned int param0,unsigned int param1,float* param2);

static void APIENTRY init_ov_glGetMinmaxParameterfv(unsigned int param0,unsigned int param1,float* param2)
{
	void *f;
	f=(void*)getfunc("glGetMinmaxParameterfv");
	ov_glGetMinmaxParameterfv=f;
	ov_glGetMinmaxParameterfv(param0,param1,param2);
}
void (APIENTRY *ov_glGetMinmaxParameterfv)(unsigned int param0,unsigned int param1,float* param2)=init_ov_glGetMinmaxParameterfv;

void __gl_ml_glGetMinmaxParameterfv(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glGetMinmaxParameterfv(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetMinmax)(unsigned int param0,unsigned char param1,unsigned int param2,unsigned int param3,void* param4);

static void APIENTRY init_ov_glGetMinmax(unsigned int param0,unsigned char param1,unsigned int param2,unsigned int param3,void* param4)
{
	void *f;
	f=(void*)getfunc("glGetMinmax");
	ov_glGetMinmax=f;
	ov_glGetMinmax(param0,param1,param2,param3,param4);
}
void (APIENTRY *ov_glGetMinmax)(unsigned int param0,unsigned char param1,unsigned int param2,unsigned int param3,void* param4)=init_ov_glGetMinmax;

void __gl_ml_glGetMinmax(value v0,value v1,value v2,value v3,value v4)
{
	unsigned int l0=Int_val(v0);
	unsigned char l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	unsigned int l3=Int_val(v3);
	void* l4=gl__ba__(v4);
	ov_glGetMinmax(l0,l1,l2,l3,l4);
}
extern void (APIENTRY *ov_glGetHistogramParameteriv)(unsigned int param0,unsigned int param1,signed int* param2);

static void APIENTRY init_ov_glGetHistogramParameteriv(unsigned int param0,unsigned int param1,signed int* param2)
{
	void *f;
	f=(void*)getfunc("glGetHistogramParameteriv");
	ov_glGetHistogramParameteriv=f;
	ov_glGetHistogramParameteriv(param0,param1,param2);
}
void (APIENTRY *ov_glGetHistogramParameteriv)(unsigned int param0,unsigned int param1,signed int* param2)=init_ov_glGetHistogramParameteriv;

void __gl_ml_glGetHistogramParameteriv(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int* l2=(signed int*)(gl__ba__(v2));
	ov_glGetHistogramParameteriv(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetHistogramParameterfv)(unsigned int param0,unsigned int param1,float* param2);

static void APIENTRY init_ov_glGetHistogramParameterfv(unsigned int param0,unsigned int param1,float* param2)
{
	void *f;
	f=(void*)getfunc("glGetHistogramParameterfv");
	ov_glGetHistogramParameterfv=f;
	ov_glGetHistogramParameterfv(param0,param1,param2);
}
void (APIENTRY *ov_glGetHistogramParameterfv)(unsigned int param0,unsigned int param1,float* param2)=init_ov_glGetHistogramParameterfv;

void __gl_ml_glGetHistogramParameterfv(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glGetHistogramParameterfv(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetHistogram)(unsigned int param0,unsigned char param1,unsigned int param2,unsigned int param3,void* param4);

static void APIENTRY init_ov_glGetHistogram(unsigned int param0,unsigned char param1,unsigned int param2,unsigned int param3,void* param4)
{
	void *f;
	f=(void*)getfunc("glGetHistogram");
	ov_glGetHistogram=f;
	ov_glGetHistogram(param0,param1,param2,param3,param4);
}
void (APIENTRY *ov_glGetHistogram)(unsigned int param0,unsigned char param1,unsigned int param2,unsigned int param3,void* param4)=init_ov_glGetHistogram;

void __gl_ml_glGetHistogram(value v0,value v1,value v2,value v3,value v4)
{
	unsigned int l0=Int_val(v0);
	unsigned char l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	unsigned int l3=Int_val(v3);
	void* l4=gl__ba__(v4);
	ov_glGetHistogram(l0,l1,l2,l3,l4);
}
extern void (APIENTRY *ov_glSeparableFilter2D)(unsigned int param0,unsigned int param1,signed int param2,signed int param3,unsigned int param4,unsigned int param5,void* param6,void* param7);

static void APIENTRY init_ov_glSeparableFilter2D(unsigned int param0,unsigned int param1,signed int param2,signed int param3,unsigned int param4,unsigned int param5,void* param6,void* param7)
{
	void *f;
	f=(void*)getfunc("glSeparableFilter2D");
	ov_glSeparableFilter2D=f;
	ov_glSeparableFilter2D(param0,param1,param2,param3,param4,param5,param6,param7);
}
void (APIENTRY *ov_glSeparableFilter2D)(unsigned int param0,unsigned int param1,signed int param2,signed int param3,unsigned int param4,unsigned int param5,void* param6,void* param7)=init_ov_glSeparableFilter2D;

void __gl_ml_glSeparableFilter2D(value v0,value v1,value v2,value v3,value v4,value v5,value v6,value v7)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	unsigned int l4=Int_val(v4);
	unsigned int l5=Int_val(v5);
	void* l6=gl__ba__(v6);
	void* l7=gl__ba__(v7);
	ov_glSeparableFilter2D(l0,l1,l2,l3,l4,l5,l6,l7);
}

void byte___gl_ml_glSeparableFilter2D(value * argv,int argn)
{
	__gl_ml_glSeparableFilter2D(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6],argv[7]);
}
extern void (APIENTRY *ov_glGetSeparableFilter)(unsigned int param0,unsigned int param1,unsigned int param2,void* param3,void* param4,void* param5);

static void APIENTRY init_ov_glGetSeparableFilter(unsigned int param0,unsigned int param1,unsigned int param2,void* param3,void* param4,void* param5)
{
	void *f;
	f=(void*)getfunc("glGetSeparableFilter");
	ov_glGetSeparableFilter=f;
	ov_glGetSeparableFilter(param0,param1,param2,param3,param4,param5);
}
void (APIENTRY *ov_glGetSeparableFilter)(unsigned int param0,unsigned int param1,unsigned int param2,void* param3,void* param4,void* param5)=init_ov_glGetSeparableFilter;

void __gl_ml_glGetSeparableFilter(value v0,value v1,value v2,value v3,value v4,value v5)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	void* l3=gl__ba__(v3);
	void* l4=gl__ba__(v4);
	void* l5=gl__ba__(v5);
	ov_glGetSeparableFilter(l0,l1,l2,l3,l4,l5);
}

void byte___gl_ml_glGetSeparableFilter(value * argv,int argn)
{
	__gl_ml_glGetSeparableFilter(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5]);
}
extern void (APIENTRY *ov_glGetConvolutionParameteriv)(unsigned int param0,unsigned int param1,signed int* param2);

static void APIENTRY init_ov_glGetConvolutionParameteriv(unsigned int param0,unsigned int param1,signed int* param2)
{
	void *f;
	f=(void*)getfunc("glGetConvolutionParameteriv");
	ov_glGetConvolutionParameteriv=f;
	ov_glGetConvolutionParameteriv(param0,param1,param2);
}
void (APIENTRY *ov_glGetConvolutionParameteriv)(unsigned int param0,unsigned int param1,signed int* param2)=init_ov_glGetConvolutionParameteriv;

void __gl_ml_glGetConvolutionParameteriv(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int* l2=(signed int*)(gl__ba__(v2));
	ov_glGetConvolutionParameteriv(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetConvolutionParameterfv)(unsigned int param0,unsigned int param1,float* param2);

static void APIENTRY init_ov_glGetConvolutionParameterfv(unsigned int param0,unsigned int param1,float* param2)
{
	void *f;
	f=(void*)getfunc("glGetConvolutionParameterfv");
	ov_glGetConvolutionParameterfv=f;
	ov_glGetConvolutionParameterfv(param0,param1,param2);
}
void (APIENTRY *ov_glGetConvolutionParameterfv)(unsigned int param0,unsigned int param1,float* param2)=init_ov_glGetConvolutionParameterfv;

void __gl_ml_glGetConvolutionParameterfv(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glGetConvolutionParameterfv(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetConvolutionFilter)(unsigned int param0,unsigned int param1,unsigned int param2,void* param3);

static void APIENTRY init_ov_glGetConvolutionFilter(unsigned int param0,unsigned int param1,unsigned int param2,void* param3)
{
	void *f;
	f=(void*)getfunc("glGetConvolutionFilter");
	ov_glGetConvolutionFilter=f;
	ov_glGetConvolutionFilter(param0,param1,param2,param3);
}
void (APIENTRY *ov_glGetConvolutionFilter)(unsigned int param0,unsigned int param1,unsigned int param2,void* param3)=init_ov_glGetConvolutionFilter;

void __gl_ml_glGetConvolutionFilter(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	void* l3=gl__ba__(v3);
	ov_glGetConvolutionFilter(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glCopyConvolutionFilter2D)(unsigned int param0,unsigned int param1,signed int param2,signed int param3,signed int param4,signed int param5);

static void APIENTRY init_ov_glCopyConvolutionFilter2D(unsigned int param0,unsigned int param1,signed int param2,signed int param3,signed int param4,signed int param5)
{
	void *f;
	f=(void*)getfunc("glCopyConvolutionFilter2D");
	ov_glCopyConvolutionFilter2D=f;
	ov_glCopyConvolutionFilter2D(param0,param1,param2,param3,param4,param5);
}
void (APIENTRY *ov_glCopyConvolutionFilter2D)(unsigned int param0,unsigned int param1,signed int param2,signed int param3,signed int param4,signed int param5)=init_ov_glCopyConvolutionFilter2D;

void __gl_ml_glCopyConvolutionFilter2D(value v0,value v1,value v2,value v3,value v4,value v5)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	signed int l4=Int_val(v4);
	signed int l5=Int_val(v5);
	ov_glCopyConvolutionFilter2D(l0,l1,l2,l3,l4,l5);
}

void byte___gl_ml_glCopyConvolutionFilter2D(value * argv,int argn)
{
	__gl_ml_glCopyConvolutionFilter2D(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5]);
}
extern void (APIENTRY *ov_glCopyConvolutionFilter1D)(unsigned int param0,unsigned int param1,signed int param2,signed int param3,signed int param4);

static void APIENTRY init_ov_glCopyConvolutionFilter1D(unsigned int param0,unsigned int param1,signed int param2,signed int param3,signed int param4)
{
	void *f;
	f=(void*)getfunc("glCopyConvolutionFilter1D");
	ov_glCopyConvolutionFilter1D=f;
	ov_glCopyConvolutionFilter1D(param0,param1,param2,param3,param4);
}
void (APIENTRY *ov_glCopyConvolutionFilter1D)(unsigned int param0,unsigned int param1,signed int param2,signed int param3,signed int param4)=init_ov_glCopyConvolutionFilter1D;

void __gl_ml_glCopyConvolutionFilter1D(value v0,value v1,value v2,value v3,value v4)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	signed int l4=Int_val(v4);
	ov_glCopyConvolutionFilter1D(l0,l1,l2,l3,l4);
}
extern void (APIENTRY *ov_glConvolutionParameteriv)(unsigned int param0,unsigned int param1,signed int* param2);

static void APIENTRY init_ov_glConvolutionParameteriv(unsigned int param0,unsigned int param1,signed int* param2)
{
	void *f;
	f=(void*)getfunc("glConvolutionParameteriv");
	ov_glConvolutionParameteriv=f;
	ov_glConvolutionParameteriv(param0,param1,param2);
}
void (APIENTRY *ov_glConvolutionParameteriv)(unsigned int param0,unsigned int param1,signed int* param2)=init_ov_glConvolutionParameteriv;

void __gl_ml_glConvolutionParameteriv(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int* l2=(signed int*)(gl__ba__(v2));
	ov_glConvolutionParameteriv(l0,l1,l2);
}
extern void (APIENTRY *ov_glConvolutionParameteri)(unsigned int param0,unsigned int param1,signed int param2);

static void APIENTRY init_ov_glConvolutionParameteri(unsigned int param0,unsigned int param1,signed int param2)
{
	void *f;
	f=(void*)getfunc("glConvolutionParameteri");
	ov_glConvolutionParameteri=f;
	ov_glConvolutionParameteri(param0,param1,param2);
}
void (APIENTRY *ov_glConvolutionParameteri)(unsigned int param0,unsigned int param1,signed int param2)=init_ov_glConvolutionParameteri;

void __gl_ml_glConvolutionParameteri(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	ov_glConvolutionParameteri(l0,l1,l2);
}
extern void (APIENTRY *ov_glConvolutionParameterfv)(unsigned int param0,unsigned int param1,float* param2);

static void APIENTRY init_ov_glConvolutionParameterfv(unsigned int param0,unsigned int param1,float* param2)
{
	void *f;
	f=(void*)getfunc("glConvolutionParameterfv");
	ov_glConvolutionParameterfv=f;
	ov_glConvolutionParameterfv(param0,param1,param2);
}
void (APIENTRY *ov_glConvolutionParameterfv)(unsigned int param0,unsigned int param1,float* param2)=init_ov_glConvolutionParameterfv;

void __gl_ml_glConvolutionParameterfv(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glConvolutionParameterfv(l0,l1,l2);
}
extern void (APIENTRY *ov_glConvolutionParameterf)(unsigned int param0,unsigned int param1,float param2);

static void APIENTRY init_ov_glConvolutionParameterf(unsigned int param0,unsigned int param1,float param2)
{
	void *f;
	f=(void*)getfunc("glConvolutionParameterf");
	ov_glConvolutionParameterf=f;
	ov_glConvolutionParameterf(param0,param1,param2);
}
void (APIENTRY *ov_glConvolutionParameterf)(unsigned int param0,unsigned int param1,float param2)=init_ov_glConvolutionParameterf;

void __gl_ml_glConvolutionParameterf(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float l2=Double_val(v2);
	ov_glConvolutionParameterf(l0,l1,l2);
}
extern void (APIENTRY *ov_glConvolutionFilter2D)(unsigned int param0,unsigned int param1,signed int param2,signed int param3,unsigned int param4,unsigned int param5,void* param6);

static void APIENTRY init_ov_glConvolutionFilter2D(unsigned int param0,unsigned int param1,signed int param2,signed int param3,unsigned int param4,unsigned int param5,void* param6)
{
	void *f;
	f=(void*)getfunc("glConvolutionFilter2D");
	ov_glConvolutionFilter2D=f;
	ov_glConvolutionFilter2D(param0,param1,param2,param3,param4,param5,param6);
}
void (APIENTRY *ov_glConvolutionFilter2D)(unsigned int param0,unsigned int param1,signed int param2,signed int param3,unsigned int param4,unsigned int param5,void* param6)=init_ov_glConvolutionFilter2D;

void __gl_ml_glConvolutionFilter2D(value v0,value v1,value v2,value v3,value v4,value v5,value v6)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	unsigned int l4=Int_val(v4);
	unsigned int l5=Int_val(v5);
	void* l6=gl__ba__(v6);
	ov_glConvolutionFilter2D(l0,l1,l2,l3,l4,l5,l6);
}

void byte___gl_ml_glConvolutionFilter2D(value * argv,int argn)
{
	__gl_ml_glConvolutionFilter2D(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6]);
}
extern void (APIENTRY *ov_glConvolutionFilter1D)(unsigned int param0,unsigned int param1,signed int param2,unsigned int param3,unsigned int param4,void* param5);

static void APIENTRY init_ov_glConvolutionFilter1D(unsigned int param0,unsigned int param1,signed int param2,unsigned int param3,unsigned int param4,void* param5)
{
	void *f;
	f=(void*)getfunc("glConvolutionFilter1D");
	ov_glConvolutionFilter1D=f;
	ov_glConvolutionFilter1D(param0,param1,param2,param3,param4,param5);
}
void (APIENTRY *ov_glConvolutionFilter1D)(unsigned int param0,unsigned int param1,signed int param2,unsigned int param3,unsigned int param4,void* param5)=init_ov_glConvolutionFilter1D;

void __gl_ml_glConvolutionFilter1D(value v0,value v1,value v2,value v3,value v4,value v5)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	unsigned int l3=Int_val(v3);
	unsigned int l4=Int_val(v4);
	void* l5=gl__ba__(v5);
	ov_glConvolutionFilter1D(l0,l1,l2,l3,l4,l5);
}

void byte___gl_ml_glConvolutionFilter1D(value * argv,int argn)
{
	__gl_ml_glConvolutionFilter1D(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5]);
}
extern void (APIENTRY *ov_glCopyColorSubTable)(unsigned int param0,signed int param1,signed int param2,signed int param3,signed int param4);

static void APIENTRY init_ov_glCopyColorSubTable(unsigned int param0,signed int param1,signed int param2,signed int param3,signed int param4)
{
	void *f;
	f=(void*)getfunc("glCopyColorSubTable");
	ov_glCopyColorSubTable=f;
	ov_glCopyColorSubTable(param0,param1,param2,param3,param4);
}
void (APIENTRY *ov_glCopyColorSubTable)(unsigned int param0,signed int param1,signed int param2,signed int param3,signed int param4)=init_ov_glCopyColorSubTable;

void __gl_ml_glCopyColorSubTable(value v0,value v1,value v2,value v3,value v4)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	signed int l4=Int_val(v4);
	ov_glCopyColorSubTable(l0,l1,l2,l3,l4);
}
extern void (APIENTRY *ov_glColorSubTable)(unsigned int param0,signed int param1,signed int param2,unsigned int param3,unsigned int param4,void* param5);

static void APIENTRY init_ov_glColorSubTable(unsigned int param0,signed int param1,signed int param2,unsigned int param3,unsigned int param4,void* param5)
{
	void *f;
	f=(void*)getfunc("glColorSubTable");
	ov_glColorSubTable=f;
	ov_glColorSubTable(param0,param1,param2,param3,param4,param5);
}
void (APIENTRY *ov_glColorSubTable)(unsigned int param0,signed int param1,signed int param2,unsigned int param3,unsigned int param4,void* param5)=init_ov_glColorSubTable;

void __gl_ml_glColorSubTable(value v0,value v1,value v2,value v3,value v4,value v5)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	unsigned int l3=Int_val(v3);
	unsigned int l4=Int_val(v4);
	void* l5=gl__ba__(v5);
	ov_glColorSubTable(l0,l1,l2,l3,l4,l5);
}

void byte___gl_ml_glColorSubTable(value * argv,int argn)
{
	__gl_ml_glColorSubTable(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5]);
}
extern void (APIENTRY *ov_glGetColorTableParameteriv)(unsigned int param0,unsigned int param1,signed int* param2);

static void APIENTRY init_ov_glGetColorTableParameteriv(unsigned int param0,unsigned int param1,signed int* param2)
{
	void *f;
	f=(void*)getfunc("glGetColorTableParameteriv");
	ov_glGetColorTableParameteriv=f;
	ov_glGetColorTableParameteriv(param0,param1,param2);
}
void (APIENTRY *ov_glGetColorTableParameteriv)(unsigned int param0,unsigned int param1,signed int* param2)=init_ov_glGetColorTableParameteriv;

void __gl_ml_glGetColorTableParameteriv(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int* l2=(signed int*)(gl__ba__(v2));
	ov_glGetColorTableParameteriv(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetColorTableParameterfv)(unsigned int param0,unsigned int param1,float* param2);

static void APIENTRY init_ov_glGetColorTableParameterfv(unsigned int param0,unsigned int param1,float* param2)
{
	void *f;
	f=(void*)getfunc("glGetColorTableParameterfv");
	ov_glGetColorTableParameterfv=f;
	ov_glGetColorTableParameterfv(param0,param1,param2);
}
void (APIENTRY *ov_glGetColorTableParameterfv)(unsigned int param0,unsigned int param1,float* param2)=init_ov_glGetColorTableParameterfv;

void __gl_ml_glGetColorTableParameterfv(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glGetColorTableParameterfv(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetColorTable)(unsigned int param0,unsigned int param1,unsigned int param2,void* param3);

static void APIENTRY init_ov_glGetColorTable(unsigned int param0,unsigned int param1,unsigned int param2,void* param3)
{
	void *f;
	f=(void*)getfunc("glGetColorTable");
	ov_glGetColorTable=f;
	ov_glGetColorTable(param0,param1,param2,param3);
}
void (APIENTRY *ov_glGetColorTable)(unsigned int param0,unsigned int param1,unsigned int param2,void* param3)=init_ov_glGetColorTable;

void __gl_ml_glGetColorTable(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	void* l3=gl__ba__(v3);
	ov_glGetColorTable(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glCopyColorTable)(unsigned int param0,unsigned int param1,signed int param2,signed int param3,signed int param4);

static void APIENTRY init_ov_glCopyColorTable(unsigned int param0,unsigned int param1,signed int param2,signed int param3,signed int param4)
{
	void *f;
	f=(void*)getfunc("glCopyColorTable");
	ov_glCopyColorTable=f;
	ov_glCopyColorTable(param0,param1,param2,param3,param4);
}
void (APIENTRY *ov_glCopyColorTable)(unsigned int param0,unsigned int param1,signed int param2,signed int param3,signed int param4)=init_ov_glCopyColorTable;

void __gl_ml_glCopyColorTable(value v0,value v1,value v2,value v3,value v4)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	signed int l4=Int_val(v4);
	ov_glCopyColorTable(l0,l1,l2,l3,l4);
}
extern void (APIENTRY *ov_glColorTableParameteriv)(unsigned int param0,unsigned int param1,signed int* param2);

static void APIENTRY init_ov_glColorTableParameteriv(unsigned int param0,unsigned int param1,signed int* param2)
{
	void *f;
	f=(void*)getfunc("glColorTableParameteriv");
	ov_glColorTableParameteriv=f;
	ov_glColorTableParameteriv(param0,param1,param2);
}
void (APIENTRY *ov_glColorTableParameteriv)(unsigned int param0,unsigned int param1,signed int* param2)=init_ov_glColorTableParameteriv;

void __gl_ml_glColorTableParameteriv(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int* l2=(signed int*)(gl__ba__(v2));
	ov_glColorTableParameteriv(l0,l1,l2);
}
extern void (APIENTRY *ov_glColorTableParameterfv)(unsigned int param0,unsigned int param1,float* param2);

static void APIENTRY init_ov_glColorTableParameterfv(unsigned int param0,unsigned int param1,float* param2)
{
	void *f;
	f=(void*)getfunc("glColorTableParameterfv");
	ov_glColorTableParameterfv=f;
	ov_glColorTableParameterfv(param0,param1,param2);
}
void (APIENTRY *ov_glColorTableParameterfv)(unsigned int param0,unsigned int param1,float* param2)=init_ov_glColorTableParameterfv;

void __gl_ml_glColorTableParameterfv(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glColorTableParameterfv(l0,l1,l2);
}
extern void (APIENTRY *ov_glColorTable)(unsigned int param0,unsigned int param1,signed int param2,unsigned int param3,unsigned int param4,void* param5);

static void APIENTRY init_ov_glColorTable(unsigned int param0,unsigned int param1,signed int param2,unsigned int param3,unsigned int param4,void* param5)
{
	void *f;
	f=(void*)getfunc("glColorTable");
	ov_glColorTable=f;
	ov_glColorTable(param0,param1,param2,param3,param4,param5);
}
void (APIENTRY *ov_glColorTable)(unsigned int param0,unsigned int param1,signed int param2,unsigned int param3,unsigned int param4,void* param5)=init_ov_glColorTable;

void __gl_ml_glColorTable(value v0,value v1,value v2,value v3,value v4,value v5)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	unsigned int l3=Int_val(v3);
	unsigned int l4=Int_val(v4);
	void* l5=gl__ba__(v5);
	ov_glColorTable(l0,l1,l2,l3,l4,l5);
}

void byte___gl_ml_glColorTable(value * argv,int argn)
{
	__gl_ml_glColorTable(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5]);
}
extern void (APIENTRY *ov_glDrawRangeElements)(unsigned int param0,unsigned int param1,unsigned int param2,signed int param3,unsigned int param4,void* param5);

static void APIENTRY init_ov_glDrawRangeElements(unsigned int param0,unsigned int param1,unsigned int param2,signed int param3,unsigned int param4,void* param5)
{
	void *f;
	f=(void*)getfunc("glDrawRangeElements");
	ov_glDrawRangeElements=f;
	ov_glDrawRangeElements(param0,param1,param2,param3,param4,param5);
}
void (APIENTRY *ov_glDrawRangeElements)(unsigned int param0,unsigned int param1,unsigned int param2,signed int param3,unsigned int param4,void* param5)=init_ov_glDrawRangeElements;

void __gl_ml_glDrawRangeElements(value v0,value v1,value v2,value v3,value v4,value v5)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	unsigned int l4=Int_val(v4);
	void* l5=gl__ba__(v5);
	ov_glDrawRangeElements(l0,l1,l2,l3,l4,l5);
}

void byte___gl_ml_glDrawRangeElements(value * argv,int argn)
{
	__gl_ml_glDrawRangeElements(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5]);
}
extern void (APIENTRY *ov_glBlendEquation)(unsigned int param0);

static void APIENTRY init_ov_glBlendEquation(unsigned int param0)
{
	void *f;
	f=(void*)getfunc("glBlendEquation");
	ov_glBlendEquation=f;
	ov_glBlendEquation(param0);
}
void (APIENTRY *ov_glBlendEquation)(unsigned int param0)=init_ov_glBlendEquation;

void __gl_ml_glBlendEquation(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glBlendEquation(l0);
}
extern void (APIENTRY *ov_glBlendColor)(float param0,float param1,float param2,float param3);

static void APIENTRY init_ov_glBlendColor(float param0,float param1,float param2,float param3)
{
	void *f;
	f=(void*)getfunc("glBlendColor");
	ov_glBlendColor=f;
	ov_glBlendColor(param0,param1,param2,param3);
}
void (APIENTRY *ov_glBlendColor)(float param0,float param1,float param2,float param3)=init_ov_glBlendColor;

void __gl_ml_glBlendColor(value v0,value v1,value v2,value v3)
{
	float l0=Double_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	float l3=Double_val(v3);
	ov_glBlendColor(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glPopName)();

static void APIENTRY init_ov_glPopName()
{
	void *f;
	f=(void*)getfunc("glPopName");
	ov_glPopName=f;
	ov_glPopName();
}
void (APIENTRY *ov_glPopName)()=init_ov_glPopName;

void __gl_ml_glPopName()
{
	ov_glPopName();
}
extern void (APIENTRY *ov_glPushName)(unsigned int name);

static void APIENTRY init_ov_glPushName(unsigned int name)
{
	void *f;
	f=(void*)getfunc("glPushName");
	ov_glPushName=f;
	ov_glPushName(name);
}
void (APIENTRY *ov_glPushName)(unsigned int name)=init_ov_glPushName;

void __gl_ml_glPushName(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glPushName(l0);
}
extern void (APIENTRY *ov_glLoadName)(unsigned int name);

static void APIENTRY init_ov_glLoadName(unsigned int name)
{
	void *f;
	f=(void*)getfunc("glLoadName");
	ov_glLoadName=f;
	ov_glLoadName(name);
}
void (APIENTRY *ov_glLoadName)(unsigned int name)=init_ov_glLoadName;

void __gl_ml_glLoadName(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glLoadName(l0);
}
extern void (APIENTRY *ov_glInitNames)();

static void APIENTRY init_ov_glInitNames()
{
	void *f;
	f=(void*)getfunc("glInitNames");
	ov_glInitNames=f;
	ov_glInitNames();
}
void (APIENTRY *ov_glInitNames)()=init_ov_glInitNames;

void __gl_ml_glInitNames()
{
	ov_glInitNames();
}
extern void (APIENTRY *ov_glSelectBuffer)(signed int size,unsigned int* buffer);

static void APIENTRY init_ov_glSelectBuffer(signed int size,unsigned int* buffer)
{
	void *f;
	f=(void*)getfunc("glSelectBuffer");
	ov_glSelectBuffer=f;
	ov_glSelectBuffer(size,buffer);
}
void (APIENTRY *ov_glSelectBuffer)(signed int size,unsigned int* buffer)=init_ov_glSelectBuffer;

void __gl_ml_glSelectBuffer(value v0,value v1)
{
	signed int l0=Int_val(v0);
	unsigned int* l1=(unsigned int*)(gl__ba__(v1));
	ov_glSelectBuffer(l0,l1);
}
extern void (APIENTRY *ov_glPassThrough)(float token);

static void APIENTRY init_ov_glPassThrough(float token)
{
	void *f;
	f=(void*)getfunc("glPassThrough");
	ov_glPassThrough=f;
	ov_glPassThrough(token);
}
void (APIENTRY *ov_glPassThrough)(float token)=init_ov_glPassThrough;

void __gl_ml_glPassThrough(value v0)
{
	float l0=Double_val(v0);
	ov_glPassThrough(l0);
}
extern void (APIENTRY *ov_glFeedbackBuffer)(signed int size,unsigned int type,float* buffer);

static void APIENTRY init_ov_glFeedbackBuffer(signed int size,unsigned int type,float* buffer)
{
	void *f;
	f=(void*)getfunc("glFeedbackBuffer");
	ov_glFeedbackBuffer=f;
	ov_glFeedbackBuffer(size,type,buffer);
}
void (APIENTRY *ov_glFeedbackBuffer)(signed int size,unsigned int type,float* buffer)=init_ov_glFeedbackBuffer;

void __gl_ml_glFeedbackBuffer(value v0,value v1,value v2)
{
	signed int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glFeedbackBuffer(l0,l1,l2);
}
extern void (APIENTRY *ov_glFogiv)(unsigned int pname,signed int* params);

static void APIENTRY init_ov_glFogiv(unsigned int pname,signed int* params)
{
	void *f;
	f=(void*)getfunc("glFogiv");
	ov_glFogiv=f;
	ov_glFogiv(pname,params);
}
void (APIENTRY *ov_glFogiv)(unsigned int pname,signed int* params)=init_ov_glFogiv;

void __gl_ml_glFogiv(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed int* l1=(signed int*)(gl__ba__(v1));
	ov_glFogiv(l0,l1);
}
extern void (APIENTRY *ov_glFogfv)(unsigned int pname,float* params);

static void APIENTRY init_ov_glFogfv(unsigned int pname,float* params)
{
	void *f;
	f=(void*)getfunc("glFogfv");
	ov_glFogfv=f;
	ov_glFogfv(pname,params);
}
void (APIENTRY *ov_glFogfv)(unsigned int pname,float* params)=init_ov_glFogfv;

void __gl_ml_glFogfv(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float* l1=(float*)(gl__ba__(v1));
	ov_glFogfv(l0,l1);
}
extern void (APIENTRY *ov_glFogi)(unsigned int pname,signed int param);

static void APIENTRY init_ov_glFogi(unsigned int pname,signed int param)
{
	void *f;
	f=(void*)getfunc("glFogi");
	ov_glFogi=f;
	ov_glFogi(pname,param);
}
void (APIENTRY *ov_glFogi)(unsigned int pname,signed int param)=init_ov_glFogi;

void __gl_ml_glFogi(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	ov_glFogi(l0,l1);
}
extern void (APIENTRY *ov_glFogf)(unsigned int pname,float param);

static void APIENTRY init_ov_glFogf(unsigned int pname,float param)
{
	void *f;
	f=(void*)getfunc("glFogf");
	ov_glFogf=f;
	ov_glFogf(pname,param);
}
void (APIENTRY *ov_glFogf)(unsigned int pname,float param)=init_ov_glFogf;

void __gl_ml_glFogf(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float l1=Double_val(v1);
	ov_glFogf(l0,l1);
}
extern void (APIENTRY *ov_glEvalMesh2)(unsigned int mode,signed int i1,signed int i2,signed int j1,signed int j2);

static void APIENTRY init_ov_glEvalMesh2(unsigned int mode,signed int i1,signed int i2,signed int j1,signed int j2)
{
	void *f;
	f=(void*)getfunc("glEvalMesh2");
	ov_glEvalMesh2=f;
	ov_glEvalMesh2(mode,i1,i2,j1,j2);
}
void (APIENTRY *ov_glEvalMesh2)(unsigned int mode,signed int i1,signed int i2,signed int j1,signed int j2)=init_ov_glEvalMesh2;

void __gl_ml_glEvalMesh2(value v0,value v1,value v2,value v3,value v4)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	signed int l4=Int_val(v4);
	ov_glEvalMesh2(l0,l1,l2,l3,l4);
}
extern void (APIENTRY *ov_glEvalMesh1)(unsigned int mode,signed int i1,signed int i2);

static void APIENTRY init_ov_glEvalMesh1(unsigned int mode,signed int i1,signed int i2)
{
	void *f;
	f=(void*)getfunc("glEvalMesh1");
	ov_glEvalMesh1=f;
	ov_glEvalMesh1(mode,i1,i2);
}
void (APIENTRY *ov_glEvalMesh1)(unsigned int mode,signed int i1,signed int i2)=init_ov_glEvalMesh1;

void __gl_ml_glEvalMesh1(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	ov_glEvalMesh1(l0,l1,l2);
}
extern void (APIENTRY *ov_glEvalPoint2)(signed int i,signed int j);

static void APIENTRY init_ov_glEvalPoint2(signed int i,signed int j)
{
	void *f;
	f=(void*)getfunc("glEvalPoint2");
	ov_glEvalPoint2=f;
	ov_glEvalPoint2(i,j);
}
void (APIENTRY *ov_glEvalPoint2)(signed int i,signed int j)=init_ov_glEvalPoint2;

void __gl_ml_glEvalPoint2(value v0,value v1)
{
	signed int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	ov_glEvalPoint2(l0,l1);
}
extern void (APIENTRY *ov_glEvalPoint1)(signed int i);

static void APIENTRY init_ov_glEvalPoint1(signed int i)
{
	void *f;
	f=(void*)getfunc("glEvalPoint1");
	ov_glEvalPoint1=f;
	ov_glEvalPoint1(i);
}
void (APIENTRY *ov_glEvalPoint1)(signed int i)=init_ov_glEvalPoint1;

void __gl_ml_glEvalPoint1(value v0)
{
	signed int l0=Int_val(v0);
	ov_glEvalPoint1(l0);
}
extern void (APIENTRY *ov_glMapGrid2f)(signed int un,float u1,float u2,signed int vn,float v1,float v2);

static void APIENTRY init_ov_glMapGrid2f(signed int un,float u1,float u2,signed int vn,float v1,float v2)
{
	void *f;
	f=(void*)getfunc("glMapGrid2f");
	ov_glMapGrid2f=f;
	ov_glMapGrid2f(un,u1,u2,vn,v1,v2);
}
void (APIENTRY *ov_glMapGrid2f)(signed int un,float u1,float u2,signed int vn,float v1,float v2)=init_ov_glMapGrid2f;

void __gl_ml_glMapGrid2f(value v0,value v1,value v2,value v3,value v4,value v5)
{
	signed int l0=Int_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	signed int l3=Int_val(v3);
	float l4=Double_val(v4);
	float l5=Double_val(v5);
	ov_glMapGrid2f(l0,l1,l2,l3,l4,l5);
}

void byte___gl_ml_glMapGrid2f(value * argv,int argn)
{
	__gl_ml_glMapGrid2f(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5]);
}
extern void (APIENTRY *ov_glMapGrid2d)(signed int un,double u1,double u2,signed int vn,double v1,double v2);

static void APIENTRY init_ov_glMapGrid2d(signed int un,double u1,double u2,signed int vn,double v1,double v2)
{
	void *f;
	f=(void*)getfunc("glMapGrid2d");
	ov_glMapGrid2d=f;
	ov_glMapGrid2d(un,u1,u2,vn,v1,v2);
}
void (APIENTRY *ov_glMapGrid2d)(signed int un,double u1,double u2,signed int vn,double v1,double v2)=init_ov_glMapGrid2d;

void __gl_ml_glMapGrid2d(value v0,value v1,value v2,value v3,value v4,value v5)
{
	signed int l0=Int_val(v0);
	double l1=Double_val(v1);
	double l2=Double_val(v2);
	signed int l3=Int_val(v3);
	double l4=Double_val(v4);
	double l5=Double_val(v5);
	ov_glMapGrid2d(l0,l1,l2,l3,l4,l5);
}

void byte___gl_ml_glMapGrid2d(value * argv,int argn)
{
	__gl_ml_glMapGrid2d(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5]);
}
extern void (APIENTRY *ov_glMapGrid1f)(signed int un,float u1,float u2);

static void APIENTRY init_ov_glMapGrid1f(signed int un,float u1,float u2)
{
	void *f;
	f=(void*)getfunc("glMapGrid1f");
	ov_glMapGrid1f=f;
	ov_glMapGrid1f(un,u1,u2);
}
void (APIENTRY *ov_glMapGrid1f)(signed int un,float u1,float u2)=init_ov_glMapGrid1f;

void __gl_ml_glMapGrid1f(value v0,value v1,value v2)
{
	signed int l0=Int_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	ov_glMapGrid1f(l0,l1,l2);
}
extern void (APIENTRY *ov_glMapGrid1d)(signed int un,double u1,double u2);

static void APIENTRY init_ov_glMapGrid1d(signed int un,double u1,double u2)
{
	void *f;
	f=(void*)getfunc("glMapGrid1d");
	ov_glMapGrid1d=f;
	ov_glMapGrid1d(un,u1,u2);
}
void (APIENTRY *ov_glMapGrid1d)(signed int un,double u1,double u2)=init_ov_glMapGrid1d;

void __gl_ml_glMapGrid1d(value v0,value v1,value v2)
{
	signed int l0=Int_val(v0);
	double l1=Double_val(v1);
	double l2=Double_val(v2);
	ov_glMapGrid1d(l0,l1,l2);
}
extern void (APIENTRY *ov_glEvalCoord2fv)(float* u);

static void APIENTRY init_ov_glEvalCoord2fv(float* u)
{
	void *f;
	f=(void*)getfunc("glEvalCoord2fv");
	ov_glEvalCoord2fv=f;
	ov_glEvalCoord2fv(u);
}
void (APIENTRY *ov_glEvalCoord2fv)(float* u)=init_ov_glEvalCoord2fv;

void __gl_ml_glEvalCoord2fv(value v0)
{
	float* l0=(float*)(gl__ba__(v0));
	ov_glEvalCoord2fv(l0);
}
extern void (APIENTRY *ov_glEvalCoord2dv)(double* u);

static void APIENTRY init_ov_glEvalCoord2dv(double* u)
{
	void *f;
	f=(void*)getfunc("glEvalCoord2dv");
	ov_glEvalCoord2dv=f;
	ov_glEvalCoord2dv(u);
}
void (APIENTRY *ov_glEvalCoord2dv)(double* u)=init_ov_glEvalCoord2dv;

void __gl_ml_glEvalCoord2dv(value v0)
{
	double* l0=(double*)(gl__ba__(v0));
	ov_glEvalCoord2dv(l0);
}
extern void (APIENTRY *ov_glEvalCoord2f)(float u,float v);

static void APIENTRY init_ov_glEvalCoord2f(float u,float v)
{
	void *f;
	f=(void*)getfunc("glEvalCoord2f");
	ov_glEvalCoord2f=f;
	ov_glEvalCoord2f(u,v);
}
void (APIENTRY *ov_glEvalCoord2f)(float u,float v)=init_ov_glEvalCoord2f;

void __gl_ml_glEvalCoord2f(value v0,value v1)
{
	float l0=Double_val(v0);
	float l1=Double_val(v1);
	ov_glEvalCoord2f(l0,l1);
}
extern void (APIENTRY *ov_glEvalCoord2d)(double u,double v);

static void APIENTRY init_ov_glEvalCoord2d(double u,double v)
{
	void *f;
	f=(void*)getfunc("glEvalCoord2d");
	ov_glEvalCoord2d=f;
	ov_glEvalCoord2d(u,v);
}
void (APIENTRY *ov_glEvalCoord2d)(double u,double v)=init_ov_glEvalCoord2d;

void __gl_ml_glEvalCoord2d(value v0,value v1)
{
	double l0=Double_val(v0);
	double l1=Double_val(v1);
	ov_glEvalCoord2d(l0,l1);
}
extern void (APIENTRY *ov_glEvalCoord1fv)(float* u);

static void APIENTRY init_ov_glEvalCoord1fv(float* u)
{
	void *f;
	f=(void*)getfunc("glEvalCoord1fv");
	ov_glEvalCoord1fv=f;
	ov_glEvalCoord1fv(u);
}
void (APIENTRY *ov_glEvalCoord1fv)(float* u)=init_ov_glEvalCoord1fv;

void __gl_ml_glEvalCoord1fv(value v0)
{
	float* l0=(float*)(gl__ba__(v0));
	ov_glEvalCoord1fv(l0);
}
extern void (APIENTRY *ov_glEvalCoord1dv)(double* u);

static void APIENTRY init_ov_glEvalCoord1dv(double* u)
{
	void *f;
	f=(void*)getfunc("glEvalCoord1dv");
	ov_glEvalCoord1dv=f;
	ov_glEvalCoord1dv(u);
}
void (APIENTRY *ov_glEvalCoord1dv)(double* u)=init_ov_glEvalCoord1dv;

void __gl_ml_glEvalCoord1dv(value v0)
{
	double* l0=(double*)(gl__ba__(v0));
	ov_glEvalCoord1dv(l0);
}
extern void (APIENTRY *ov_glEvalCoord1f)(float u);

static void APIENTRY init_ov_glEvalCoord1f(float u)
{
	void *f;
	f=(void*)getfunc("glEvalCoord1f");
	ov_glEvalCoord1f=f;
	ov_glEvalCoord1f(u);
}
void (APIENTRY *ov_glEvalCoord1f)(float u)=init_ov_glEvalCoord1f;

void __gl_ml_glEvalCoord1f(value v0)
{
	float l0=Double_val(v0);
	ov_glEvalCoord1f(l0);
}
extern void (APIENTRY *ov_glEvalCoord1d)(double u);

static void APIENTRY init_ov_glEvalCoord1d(double u)
{
	void *f;
	f=(void*)getfunc("glEvalCoord1d");
	ov_glEvalCoord1d=f;
	ov_glEvalCoord1d(u);
}
void (APIENTRY *ov_glEvalCoord1d)(double u)=init_ov_glEvalCoord1d;

void __gl_ml_glEvalCoord1d(value v0)
{
	double l0=Double_val(v0);
	ov_glEvalCoord1d(l0);
}
extern void (APIENTRY *ov_glGetMapiv)(unsigned int target,unsigned int query,signed int* v);

static void APIENTRY init_ov_glGetMapiv(unsigned int target,unsigned int query,signed int* v)
{
	void *f;
	f=(void*)getfunc("glGetMapiv");
	ov_glGetMapiv=f;
	ov_glGetMapiv(target,query,v);
}
void (APIENTRY *ov_glGetMapiv)(unsigned int target,unsigned int query,signed int* v)=init_ov_glGetMapiv;

void __gl_ml_glGetMapiv(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int* l2=(signed int*)(gl__ba__(v2));
	ov_glGetMapiv(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetMapfv)(unsigned int target,unsigned int query,float* v);

static void APIENTRY init_ov_glGetMapfv(unsigned int target,unsigned int query,float* v)
{
	void *f;
	f=(void*)getfunc("glGetMapfv");
	ov_glGetMapfv=f;
	ov_glGetMapfv(target,query,v);
}
void (APIENTRY *ov_glGetMapfv)(unsigned int target,unsigned int query,float* v)=init_ov_glGetMapfv;

void __gl_ml_glGetMapfv(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glGetMapfv(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetMapdv)(unsigned int target,unsigned int query,double* v);

static void APIENTRY init_ov_glGetMapdv(unsigned int target,unsigned int query,double* v)
{
	void *f;
	f=(void*)getfunc("glGetMapdv");
	ov_glGetMapdv=f;
	ov_glGetMapdv(target,query,v);
}
void (APIENTRY *ov_glGetMapdv)(unsigned int target,unsigned int query,double* v)=init_ov_glGetMapdv;

void __gl_ml_glGetMapdv(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	double* l2=(double*)(gl__ba__(v2));
	ov_glGetMapdv(l0,l1,l2);
}
extern void (APIENTRY *ov_glMap2f)(unsigned int target,float u1,float u2,signed int ustride,signed int uorder,float v1,float v2,signed int vstride,signed int vorder,float* points);

static void APIENTRY init_ov_glMap2f(unsigned int target,float u1,float u2,signed int ustride,signed int uorder,float v1,float v2,signed int vstride,signed int vorder,float* points)
{
	void *f;
	f=(void*)getfunc("glMap2f");
	ov_glMap2f=f;
	ov_glMap2f(target,u1,u2,ustride,uorder,v1,v2,vstride,vorder,points);
}
void (APIENTRY *ov_glMap2f)(unsigned int target,float u1,float u2,signed int ustride,signed int uorder,float v1,float v2,signed int vstride,signed int vorder,float* points)=init_ov_glMap2f;

void __gl_ml_glMap2f(value v0,value v1,value v2,value v3,value v4,value v5,value v6,value v7,value v8,value v9)
{
	unsigned int l0=Int_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	signed int l3=Int_val(v3);
	signed int l4=Int_val(v4);
	float l5=Double_val(v5);
	float l6=Double_val(v6);
	signed int l7=Int_val(v7);
	signed int l8=Int_val(v8);
	float* l9=(float*)(gl__ba__(v9));
	ov_glMap2f(l0,l1,l2,l3,l4,l5,l6,l7,l8,l9);
}

void byte___gl_ml_glMap2f(value * argv,int argn)
{
	__gl_ml_glMap2f(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6],argv[7],argv[8],argv[9]);
}
extern void (APIENTRY *ov_glMap2d)(unsigned int target,double u1,double u2,signed int ustride,signed int uorder,double v1,double v2,signed int vstride,signed int vorder,double* points);

static void APIENTRY init_ov_glMap2d(unsigned int target,double u1,double u2,signed int ustride,signed int uorder,double v1,double v2,signed int vstride,signed int vorder,double* points)
{
	void *f;
	f=(void*)getfunc("glMap2d");
	ov_glMap2d=f;
	ov_glMap2d(target,u1,u2,ustride,uorder,v1,v2,vstride,vorder,points);
}
void (APIENTRY *ov_glMap2d)(unsigned int target,double u1,double u2,signed int ustride,signed int uorder,double v1,double v2,signed int vstride,signed int vorder,double* points)=init_ov_glMap2d;

void __gl_ml_glMap2d(value v0,value v1,value v2,value v3,value v4,value v5,value v6,value v7,value v8,value v9)
{
	unsigned int l0=Int_val(v0);
	double l1=Double_val(v1);
	double l2=Double_val(v2);
	signed int l3=Int_val(v3);
	signed int l4=Int_val(v4);
	double l5=Double_val(v5);
	double l6=Double_val(v6);
	signed int l7=Int_val(v7);
	signed int l8=Int_val(v8);
	double* l9=(double*)(gl__ba__(v9));
	ov_glMap2d(l0,l1,l2,l3,l4,l5,l6,l7,l8,l9);
}

void byte___gl_ml_glMap2d(value * argv,int argn)
{
	__gl_ml_glMap2d(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6],argv[7],argv[8],argv[9]);
}
extern void (APIENTRY *ov_glMap1f)(unsigned int target,float u1,float u2,signed int stride,signed int order,float* points);

static void APIENTRY init_ov_glMap1f(unsigned int target,float u1,float u2,signed int stride,signed int order,float* points)
{
	void *f;
	f=(void*)getfunc("glMap1f");
	ov_glMap1f=f;
	ov_glMap1f(target,u1,u2,stride,order,points);
}
void (APIENTRY *ov_glMap1f)(unsigned int target,float u1,float u2,signed int stride,signed int order,float* points)=init_ov_glMap1f;

void __gl_ml_glMap1f(value v0,value v1,value v2,value v3,value v4,value v5)
{
	unsigned int l0=Int_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	signed int l3=Int_val(v3);
	signed int l4=Int_val(v4);
	float* l5=(float*)(gl__ba__(v5));
	ov_glMap1f(l0,l1,l2,l3,l4,l5);
}

void byte___gl_ml_glMap1f(value * argv,int argn)
{
	__gl_ml_glMap1f(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5]);
}
extern void (APIENTRY *ov_glMap1d)(unsigned int target,double u1,double u2,signed int stride,signed int order,double* points);

static void APIENTRY init_ov_glMap1d(unsigned int target,double u1,double u2,signed int stride,signed int order,double* points)
{
	void *f;
	f=(void*)getfunc("glMap1d");
	ov_glMap1d=f;
	ov_glMap1d(target,u1,u2,stride,order,points);
}
void (APIENTRY *ov_glMap1d)(unsigned int target,double u1,double u2,signed int stride,signed int order,double* points)=init_ov_glMap1d;

void __gl_ml_glMap1d(value v0,value v1,value v2,value v3,value v4,value v5)
{
	unsigned int l0=Int_val(v0);
	double l1=Double_val(v1);
	double l2=Double_val(v2);
	signed int l3=Int_val(v3);
	signed int l4=Int_val(v4);
	double* l5=(double*)(gl__ba__(v5));
	ov_glMap1d(l0,l1,l2,l3,l4,l5);
}

void byte___gl_ml_glMap1d(value * argv,int argn)
{
	__gl_ml_glMap1d(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5]);
}
extern void (APIENTRY *ov_glGetTexImage)(unsigned int target,signed int level,unsigned int format,unsigned int type,void* pixels);

static void APIENTRY init_ov_glGetTexImage(unsigned int target,signed int level,unsigned int format,unsigned int type,void* pixels)
{
	void *f;
	f=(void*)getfunc("glGetTexImage");
	ov_glGetTexImage=f;
	ov_glGetTexImage(target,level,format,type,pixels);
}
void (APIENTRY *ov_glGetTexImage)(unsigned int target,signed int level,unsigned int format,unsigned int type,void* pixels)=init_ov_glGetTexImage;

void __gl_ml_glGetTexImage(value v0,value v1,value v2,value v3,value v4)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	unsigned int l3=Int_val(v3);
	void* l4=gl__ba__(v4);
	ov_glGetTexImage(l0,l1,l2,l3,l4);
}
extern void (APIENTRY *ov_glTexImage2D)(unsigned int target,signed int level,signed int internalFormat,signed int width,signed int height,signed int border,unsigned int format,unsigned int type,void* pixels);

static void APIENTRY init_ov_glTexImage2D(unsigned int target,signed int level,signed int internalFormat,signed int width,signed int height,signed int border,unsigned int format,unsigned int type,void* pixels)
{
	void *f;
	f=(void*)getfunc("glTexImage2D");
	ov_glTexImage2D=f;
	ov_glTexImage2D(target,level,internalFormat,width,height,border,format,type,pixels);
}
void (APIENTRY *ov_glTexImage2D)(unsigned int target,signed int level,signed int internalFormat,signed int width,signed int height,signed int border,unsigned int format,unsigned int type,void* pixels)=init_ov_glTexImage2D;

void __gl_ml_glTexImage2D(value v0,value v1,value v2,value v3,value v4,value v5,value v6,value v7,value v8)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	signed int l4=Int_val(v4);
	signed int l5=Int_val(v5);
	unsigned int l6=Int_val(v6);
	unsigned int l7=Int_val(v7);
	void* l8=gl__ba__(v8);
	ov_glTexImage2D(l0,l1,l2,l3,l4,l5,l6,l7,l8);
}

void byte___gl_ml_glTexImage2D(value * argv,int argn)
{
	__gl_ml_glTexImage2D(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6],argv[7],argv[8]);
}
extern void (APIENTRY *ov_glTexImage1D)(unsigned int target,signed int level,signed int internalFormat,signed int width,signed int border,unsigned int format,unsigned int type,void* pixels);

static void APIENTRY init_ov_glTexImage1D(unsigned int target,signed int level,signed int internalFormat,signed int width,signed int border,unsigned int format,unsigned int type,void* pixels)
{
	void *f;
	f=(void*)getfunc("glTexImage1D");
	ov_glTexImage1D=f;
	ov_glTexImage1D(target,level,internalFormat,width,border,format,type,pixels);
}
void (APIENTRY *ov_glTexImage1D)(unsigned int target,signed int level,signed int internalFormat,signed int width,signed int border,unsigned int format,unsigned int type,void* pixels)=init_ov_glTexImage1D;

void __gl_ml_glTexImage1D(value v0,value v1,value v2,value v3,value v4,value v5,value v6,value v7)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	signed int l4=Int_val(v4);
	unsigned int l5=Int_val(v5);
	unsigned int l6=Int_val(v6);
	void* l7=gl__ba__(v7);
	ov_glTexImage1D(l0,l1,l2,l3,l4,l5,l6,l7);
}

void byte___gl_ml_glTexImage1D(value * argv,int argn)
{
	__gl_ml_glTexImage1D(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6],argv[7]);
}
extern void (APIENTRY *ov_glGetTexLevelParameteriv)(unsigned int target,signed int level,unsigned int pname,signed int* params);

static void APIENTRY init_ov_glGetTexLevelParameteriv(unsigned int target,signed int level,unsigned int pname,signed int* params)
{
	void *f;
	f=(void*)getfunc("glGetTexLevelParameteriv");
	ov_glGetTexLevelParameteriv=f;
	ov_glGetTexLevelParameteriv(target,level,pname,params);
}
void (APIENTRY *ov_glGetTexLevelParameteriv)(unsigned int target,signed int level,unsigned int pname,signed int* params)=init_ov_glGetTexLevelParameteriv;

void __gl_ml_glGetTexLevelParameteriv(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	signed int* l3=(signed int*)(gl__ba__(v3));
	ov_glGetTexLevelParameteriv(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glGetTexLevelParameterfv)(unsigned int target,signed int level,unsigned int pname,float* params);

static void APIENTRY init_ov_glGetTexLevelParameterfv(unsigned int target,signed int level,unsigned int pname,float* params)
{
	void *f;
	f=(void*)getfunc("glGetTexLevelParameterfv");
	ov_glGetTexLevelParameterfv=f;
	ov_glGetTexLevelParameterfv(target,level,pname,params);
}
void (APIENTRY *ov_glGetTexLevelParameterfv)(unsigned int target,signed int level,unsigned int pname,float* params)=init_ov_glGetTexLevelParameterfv;

void __gl_ml_glGetTexLevelParameterfv(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	float* l3=(float*)(gl__ba__(v3));
	ov_glGetTexLevelParameterfv(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glGetTexParameteriv)(unsigned int target,unsigned int pname,signed int* params);

static void APIENTRY init_ov_glGetTexParameteriv(unsigned int target,unsigned int pname,signed int* params)
{
	void *f;
	f=(void*)getfunc("glGetTexParameteriv");
	ov_glGetTexParameteriv=f;
	ov_glGetTexParameteriv(target,pname,params);
}
void (APIENTRY *ov_glGetTexParameteriv)(unsigned int target,unsigned int pname,signed int* params)=init_ov_glGetTexParameteriv;

void __gl_ml_glGetTexParameteriv(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int* l2=(signed int*)(gl__ba__(v2));
	ov_glGetTexParameteriv(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetTexParameterfv)(unsigned int target,unsigned int pname,float* params);

static void APIENTRY init_ov_glGetTexParameterfv(unsigned int target,unsigned int pname,float* params)
{
	void *f;
	f=(void*)getfunc("glGetTexParameterfv");
	ov_glGetTexParameterfv=f;
	ov_glGetTexParameterfv(target,pname,params);
}
void (APIENTRY *ov_glGetTexParameterfv)(unsigned int target,unsigned int pname,float* params)=init_ov_glGetTexParameterfv;

void __gl_ml_glGetTexParameterfv(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glGetTexParameterfv(l0,l1,l2);
}
extern void (APIENTRY *ov_glTexParameteriv)(unsigned int target,unsigned int pname,signed int* params);

static void APIENTRY init_ov_glTexParameteriv(unsigned int target,unsigned int pname,signed int* params)
{
	void *f;
	f=(void*)getfunc("glTexParameteriv");
	ov_glTexParameteriv=f;
	ov_glTexParameteriv(target,pname,params);
}
void (APIENTRY *ov_glTexParameteriv)(unsigned int target,unsigned int pname,signed int* params)=init_ov_glTexParameteriv;

void __gl_ml_glTexParameteriv(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int* l2=(signed int*)(gl__ba__(v2));
	ov_glTexParameteriv(l0,l1,l2);
}
extern void (APIENTRY *ov_glTexParameterfv)(unsigned int target,unsigned int pname,float* params);

static void APIENTRY init_ov_glTexParameterfv(unsigned int target,unsigned int pname,float* params)
{
	void *f;
	f=(void*)getfunc("glTexParameterfv");
	ov_glTexParameterfv=f;
	ov_glTexParameterfv(target,pname,params);
}
void (APIENTRY *ov_glTexParameterfv)(unsigned int target,unsigned int pname,float* params)=init_ov_glTexParameterfv;

void __gl_ml_glTexParameterfv(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glTexParameterfv(l0,l1,l2);
}
extern void (APIENTRY *ov_glTexParameteri)(unsigned int target,unsigned int pname,signed int param);

static void APIENTRY init_ov_glTexParameteri(unsigned int target,unsigned int pname,signed int param)
{
	void *f;
	f=(void*)getfunc("glTexParameteri");
	ov_glTexParameteri=f;
	ov_glTexParameteri(target,pname,param);
}
void (APIENTRY *ov_glTexParameteri)(unsigned int target,unsigned int pname,signed int param)=init_ov_glTexParameteri;

void __gl_ml_glTexParameteri(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	ov_glTexParameteri(l0,l1,l2);
}
extern void (APIENTRY *ov_glTexParameterf)(unsigned int target,unsigned int pname,float param);

static void APIENTRY init_ov_glTexParameterf(unsigned int target,unsigned int pname,float param)
{
	void *f;
	f=(void*)getfunc("glTexParameterf");
	ov_glTexParameterf=f;
	ov_glTexParameterf(target,pname,param);
}
void (APIENTRY *ov_glTexParameterf)(unsigned int target,unsigned int pname,float param)=init_ov_glTexParameterf;

void __gl_ml_glTexParameterf(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float l2=Double_val(v2);
	ov_glTexParameterf(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetTexEnviv)(unsigned int target,unsigned int pname,signed int* params);

static void APIENTRY init_ov_glGetTexEnviv(unsigned int target,unsigned int pname,signed int* params)
{
	void *f;
	f=(void*)getfunc("glGetTexEnviv");
	ov_glGetTexEnviv=f;
	ov_glGetTexEnviv(target,pname,params);
}
void (APIENTRY *ov_glGetTexEnviv)(unsigned int target,unsigned int pname,signed int* params)=init_ov_glGetTexEnviv;

void __gl_ml_glGetTexEnviv(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int* l2=(signed int*)(gl__ba__(v2));
	ov_glGetTexEnviv(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetTexEnvfv)(unsigned int target,unsigned int pname,float* params);

static void APIENTRY init_ov_glGetTexEnvfv(unsigned int target,unsigned int pname,float* params)
{
	void *f;
	f=(void*)getfunc("glGetTexEnvfv");
	ov_glGetTexEnvfv=f;
	ov_glGetTexEnvfv(target,pname,params);
}
void (APIENTRY *ov_glGetTexEnvfv)(unsigned int target,unsigned int pname,float* params)=init_ov_glGetTexEnvfv;

void __gl_ml_glGetTexEnvfv(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glGetTexEnvfv(l0,l1,l2);
}
extern void (APIENTRY *ov_glTexEnviv)(unsigned int target,unsigned int pname,signed int* params);

static void APIENTRY init_ov_glTexEnviv(unsigned int target,unsigned int pname,signed int* params)
{
	void *f;
	f=(void*)getfunc("glTexEnviv");
	ov_glTexEnviv=f;
	ov_glTexEnviv(target,pname,params);
}
void (APIENTRY *ov_glTexEnviv)(unsigned int target,unsigned int pname,signed int* params)=init_ov_glTexEnviv;

void __gl_ml_glTexEnviv(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int* l2=(signed int*)(gl__ba__(v2));
	ov_glTexEnviv(l0,l1,l2);
}
extern void (APIENTRY *ov_glTexEnvfv)(unsigned int target,unsigned int pname,float* params);

static void APIENTRY init_ov_glTexEnvfv(unsigned int target,unsigned int pname,float* params)
{
	void *f;
	f=(void*)getfunc("glTexEnvfv");
	ov_glTexEnvfv=f;
	ov_glTexEnvfv(target,pname,params);
}
void (APIENTRY *ov_glTexEnvfv)(unsigned int target,unsigned int pname,float* params)=init_ov_glTexEnvfv;

void __gl_ml_glTexEnvfv(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glTexEnvfv(l0,l1,l2);
}
extern void (APIENTRY *ov_glTexEnvi)(unsigned int target,unsigned int pname,signed int param);

static void APIENTRY init_ov_glTexEnvi(unsigned int target,unsigned int pname,signed int param)
{
	void *f;
	f=(void*)getfunc("glTexEnvi");
	ov_glTexEnvi=f;
	ov_glTexEnvi(target,pname,param);
}
void (APIENTRY *ov_glTexEnvi)(unsigned int target,unsigned int pname,signed int param)=init_ov_glTexEnvi;

void __gl_ml_glTexEnvi(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	ov_glTexEnvi(l0,l1,l2);
}
extern void (APIENTRY *ov_glTexEnvf)(unsigned int target,unsigned int pname,float param);

static void APIENTRY init_ov_glTexEnvf(unsigned int target,unsigned int pname,float param)
{
	void *f;
	f=(void*)getfunc("glTexEnvf");
	ov_glTexEnvf=f;
	ov_glTexEnvf(target,pname,param);
}
void (APIENTRY *ov_glTexEnvf)(unsigned int target,unsigned int pname,float param)=init_ov_glTexEnvf;

void __gl_ml_glTexEnvf(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float l2=Double_val(v2);
	ov_glTexEnvf(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetTexGeniv)(unsigned int coord,unsigned int pname,signed int* params);

static void APIENTRY init_ov_glGetTexGeniv(unsigned int coord,unsigned int pname,signed int* params)
{
	void *f;
	f=(void*)getfunc("glGetTexGeniv");
	ov_glGetTexGeniv=f;
	ov_glGetTexGeniv(coord,pname,params);
}
void (APIENTRY *ov_glGetTexGeniv)(unsigned int coord,unsigned int pname,signed int* params)=init_ov_glGetTexGeniv;

void __gl_ml_glGetTexGeniv(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int* l2=(signed int*)(gl__ba__(v2));
	ov_glGetTexGeniv(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetTexGenfv)(unsigned int coord,unsigned int pname,float* params);

static void APIENTRY init_ov_glGetTexGenfv(unsigned int coord,unsigned int pname,float* params)
{
	void *f;
	f=(void*)getfunc("glGetTexGenfv");
	ov_glGetTexGenfv=f;
	ov_glGetTexGenfv(coord,pname,params);
}
void (APIENTRY *ov_glGetTexGenfv)(unsigned int coord,unsigned int pname,float* params)=init_ov_glGetTexGenfv;

void __gl_ml_glGetTexGenfv(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glGetTexGenfv(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetTexGendv)(unsigned int coord,unsigned int pname,double* params);

static void APIENTRY init_ov_glGetTexGendv(unsigned int coord,unsigned int pname,double* params)
{
	void *f;
	f=(void*)getfunc("glGetTexGendv");
	ov_glGetTexGendv=f;
	ov_glGetTexGendv(coord,pname,params);
}
void (APIENTRY *ov_glGetTexGendv)(unsigned int coord,unsigned int pname,double* params)=init_ov_glGetTexGendv;

void __gl_ml_glGetTexGendv(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	double* l2=(double*)(gl__ba__(v2));
	ov_glGetTexGendv(l0,l1,l2);
}
extern void (APIENTRY *ov_glTexGeniv)(unsigned int coord,unsigned int pname,signed int* params);

static void APIENTRY init_ov_glTexGeniv(unsigned int coord,unsigned int pname,signed int* params)
{
	void *f;
	f=(void*)getfunc("glTexGeniv");
	ov_glTexGeniv=f;
	ov_glTexGeniv(coord,pname,params);
}
void (APIENTRY *ov_glTexGeniv)(unsigned int coord,unsigned int pname,signed int* params)=init_ov_glTexGeniv;

void __gl_ml_glTexGeniv(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int* l2=(signed int*)(gl__ba__(v2));
	ov_glTexGeniv(l0,l1,l2);
}
extern void (APIENTRY *ov_glTexGenfv)(unsigned int coord,unsigned int pname,float* params);

static void APIENTRY init_ov_glTexGenfv(unsigned int coord,unsigned int pname,float* params)
{
	void *f;
	f=(void*)getfunc("glTexGenfv");
	ov_glTexGenfv=f;
	ov_glTexGenfv(coord,pname,params);
}
void (APIENTRY *ov_glTexGenfv)(unsigned int coord,unsigned int pname,float* params)=init_ov_glTexGenfv;

void __gl_ml_glTexGenfv(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glTexGenfv(l0,l1,l2);
}
extern void (APIENTRY *ov_glTexGendv)(unsigned int coord,unsigned int pname,double* params);

static void APIENTRY init_ov_glTexGendv(unsigned int coord,unsigned int pname,double* params)
{
	void *f;
	f=(void*)getfunc("glTexGendv");
	ov_glTexGendv=f;
	ov_glTexGendv(coord,pname,params);
}
void (APIENTRY *ov_glTexGendv)(unsigned int coord,unsigned int pname,double* params)=init_ov_glTexGendv;

void __gl_ml_glTexGendv(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	double* l2=(double*)(gl__ba__(v2));
	ov_glTexGendv(l0,l1,l2);
}
extern void (APIENTRY *ov_glTexGeni)(unsigned int coord,unsigned int pname,signed int param);

static void APIENTRY init_ov_glTexGeni(unsigned int coord,unsigned int pname,signed int param)
{
	void *f;
	f=(void*)getfunc("glTexGeni");
	ov_glTexGeni=f;
	ov_glTexGeni(coord,pname,param);
}
void (APIENTRY *ov_glTexGeni)(unsigned int coord,unsigned int pname,signed int param)=init_ov_glTexGeni;

void __gl_ml_glTexGeni(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	ov_glTexGeni(l0,l1,l2);
}
extern void (APIENTRY *ov_glTexGenf)(unsigned int coord,unsigned int pname,float param);

static void APIENTRY init_ov_glTexGenf(unsigned int coord,unsigned int pname,float param)
{
	void *f;
	f=(void*)getfunc("glTexGenf");
	ov_glTexGenf=f;
	ov_glTexGenf(coord,pname,param);
}
void (APIENTRY *ov_glTexGenf)(unsigned int coord,unsigned int pname,float param)=init_ov_glTexGenf;

void __gl_ml_glTexGenf(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float l2=Double_val(v2);
	ov_glTexGenf(l0,l1,l2);
}
extern void (APIENTRY *ov_glTexGend)(unsigned int coord,unsigned int pname,double param);

static void APIENTRY init_ov_glTexGend(unsigned int coord,unsigned int pname,double param)
{
	void *f;
	f=(void*)getfunc("glTexGend");
	ov_glTexGend=f;
	ov_glTexGend(coord,pname,param);
}
void (APIENTRY *ov_glTexGend)(unsigned int coord,unsigned int pname,double param)=init_ov_glTexGend;

void __gl_ml_glTexGend(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	double l2=Double_val(v2);
	ov_glTexGend(l0,l1,l2);
}
extern void (APIENTRY *ov_glClearStencil)(signed int s);

static void APIENTRY init_ov_glClearStencil(signed int s)
{
	void *f;
	f=(void*)getfunc("glClearStencil");
	ov_glClearStencil=f;
	ov_glClearStencil(s);
}
void (APIENTRY *ov_glClearStencil)(signed int s)=init_ov_glClearStencil;

void __gl_ml_glClearStencil(value v0)
{
	signed int l0=Int_val(v0);
	ov_glClearStencil(l0);
}
extern void (APIENTRY *ov_glStencilOp)(unsigned int fail,unsigned int zfail,unsigned int zpass);

static void APIENTRY init_ov_glStencilOp(unsigned int fail,unsigned int zfail,unsigned int zpass)
{
	void *f;
	f=(void*)getfunc("glStencilOp");
	ov_glStencilOp=f;
	ov_glStencilOp(fail,zfail,zpass);
}
void (APIENTRY *ov_glStencilOp)(unsigned int fail,unsigned int zfail,unsigned int zpass)=init_ov_glStencilOp;

void __gl_ml_glStencilOp(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	ov_glStencilOp(l0,l1,l2);
}
extern void (APIENTRY *ov_glStencilMask)(unsigned int mask);

static void APIENTRY init_ov_glStencilMask(unsigned int mask)
{
	void *f;
	f=(void*)getfunc("glStencilMask");
	ov_glStencilMask=f;
	ov_glStencilMask(mask);
}
void (APIENTRY *ov_glStencilMask)(unsigned int mask)=init_ov_glStencilMask;

void __gl_ml_glStencilMask(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glStencilMask(l0);
}
extern void (APIENTRY *ov_glStencilFunc)(unsigned int func,signed int ref,unsigned int mask);

static void APIENTRY init_ov_glStencilFunc(unsigned int func,signed int ref,unsigned int mask)
{
	void *f;
	f=(void*)getfunc("glStencilFunc");
	ov_glStencilFunc=f;
	ov_glStencilFunc(func,ref,mask);
}
void (APIENTRY *ov_glStencilFunc)(unsigned int func,signed int ref,unsigned int mask)=init_ov_glStencilFunc;

void __gl_ml_glStencilFunc(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	ov_glStencilFunc(l0,l1,l2);
}
extern void (APIENTRY *ov_glCopyPixels)(signed int x,signed int y,signed int width,signed int height,unsigned int type);

static void APIENTRY init_ov_glCopyPixels(signed int x,signed int y,signed int width,signed int height,unsigned int type)
{
	void *f;
	f=(void*)getfunc("glCopyPixels");
	ov_glCopyPixels=f;
	ov_glCopyPixels(x,y,width,height,type);
}
void (APIENTRY *ov_glCopyPixels)(signed int x,signed int y,signed int width,signed int height,unsigned int type)=init_ov_glCopyPixels;

void __gl_ml_glCopyPixels(value v0,value v1,value v2,value v3,value v4)
{
	signed int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	unsigned int l4=Int_val(v4);
	ov_glCopyPixels(l0,l1,l2,l3,l4);
}
extern void (APIENTRY *ov_glDrawPixels)(signed int width,signed int height,unsigned int format,unsigned int type,void* pixels);

static void APIENTRY init_ov_glDrawPixels(signed int width,signed int height,unsigned int format,unsigned int type,void* pixels)
{
	void *f;
	f=(void*)getfunc("glDrawPixels");
	ov_glDrawPixels=f;
	ov_glDrawPixels(width,height,format,type,pixels);
}
void (APIENTRY *ov_glDrawPixels)(signed int width,signed int height,unsigned int format,unsigned int type,void* pixels)=init_ov_glDrawPixels;

void __gl_ml_glDrawPixels(value v0,value v1,value v2,value v3,value v4)
{
	signed int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	unsigned int l3=Int_val(v3);
	void* l4=gl__ba__(v4);
	ov_glDrawPixels(l0,l1,l2,l3,l4);
}
extern void (APIENTRY *ov_glReadPixels)(signed int x,signed int y,signed int width,signed int height,unsigned int format,unsigned int type,void* pixels);

static void APIENTRY init_ov_glReadPixels(signed int x,signed int y,signed int width,signed int height,unsigned int format,unsigned int type,void* pixels)
{
	void *f;
	f=(void*)getfunc("glReadPixels");
	ov_glReadPixels=f;
	ov_glReadPixels(x,y,width,height,format,type,pixels);
}
void (APIENTRY *ov_glReadPixels)(signed int x,signed int y,signed int width,signed int height,unsigned int format,unsigned int type,void* pixels)=init_ov_glReadPixels;

void __gl_ml_glReadPixels(value v0,value v1,value v2,value v3,value v4,value v5,value v6)
{
	signed int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	unsigned int l4=Int_val(v4);
	unsigned int l5=Int_val(v5);
	void* l6=gl__ba__(v6);
	ov_glReadPixels(l0,l1,l2,l3,l4,l5,l6);
}

void byte___gl_ml_glReadPixels(value * argv,int argn)
{
	__gl_ml_glReadPixels(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6]);
}
extern void (APIENTRY *ov_glBitmap)(signed int width,signed int height,float xorig,float yorig,float xmove,float ymove,unsigned char* bitmap);

static void APIENTRY init_ov_glBitmap(signed int width,signed int height,float xorig,float yorig,float xmove,float ymove,unsigned char* bitmap)
{
	void *f;
	f=(void*)getfunc("glBitmap");
	ov_glBitmap=f;
	ov_glBitmap(width,height,xorig,yorig,xmove,ymove,bitmap);
}
void (APIENTRY *ov_glBitmap)(signed int width,signed int height,float xorig,float yorig,float xmove,float ymove,unsigned char* bitmap)=init_ov_glBitmap;

void __gl_ml_glBitmap(value v0,value v1,value v2,value v3,value v4,value v5,value v6)
{
	signed int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	float l2=Double_val(v2);
	float l3=Double_val(v3);
	float l4=Double_val(v4);
	float l5=Double_val(v5);
	unsigned char* l6=(unsigned char*)(gl__ba__(v6));
	ov_glBitmap(l0,l1,l2,l3,l4,l5,l6);
}

void byte___gl_ml_glBitmap(value * argv,int argn)
{
	__gl_ml_glBitmap(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6]);
}
extern void (APIENTRY *ov_glGetPixelMapusv)(unsigned int map,unsigned short* values);

static void APIENTRY init_ov_glGetPixelMapusv(unsigned int map,unsigned short* values)
{
	void *f;
	f=(void*)getfunc("glGetPixelMapusv");
	ov_glGetPixelMapusv=f;
	ov_glGetPixelMapusv(map,values);
}
void (APIENTRY *ov_glGetPixelMapusv)(unsigned int map,unsigned short* values)=init_ov_glGetPixelMapusv;

void __gl_ml_glGetPixelMapusv(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	unsigned short* l1=(unsigned short*)(gl__ba__(v1));
	ov_glGetPixelMapusv(l0,l1);
}
extern void (APIENTRY *ov_glGetPixelMapuiv)(unsigned int map,unsigned int* values);

static void APIENTRY init_ov_glGetPixelMapuiv(unsigned int map,unsigned int* values)
{
	void *f;
	f=(void*)getfunc("glGetPixelMapuiv");
	ov_glGetPixelMapuiv=f;
	ov_glGetPixelMapuiv(map,values);
}
void (APIENTRY *ov_glGetPixelMapuiv)(unsigned int map,unsigned int* values)=init_ov_glGetPixelMapuiv;

void __gl_ml_glGetPixelMapuiv(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	unsigned int* l1=(unsigned int*)(gl__ba__(v1));
	ov_glGetPixelMapuiv(l0,l1);
}
extern void (APIENTRY *ov_glGetPixelMapfv)(unsigned int map,float* values);

static void APIENTRY init_ov_glGetPixelMapfv(unsigned int map,float* values)
{
	void *f;
	f=(void*)getfunc("glGetPixelMapfv");
	ov_glGetPixelMapfv=f;
	ov_glGetPixelMapfv(map,values);
}
void (APIENTRY *ov_glGetPixelMapfv)(unsigned int map,float* values)=init_ov_glGetPixelMapfv;

void __gl_ml_glGetPixelMapfv(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float* l1=(float*)(gl__ba__(v1));
	ov_glGetPixelMapfv(l0,l1);
}
extern void (APIENTRY *ov_glPixelMapusv)(unsigned int map,signed int mapsize,unsigned short* values);

static void APIENTRY init_ov_glPixelMapusv(unsigned int map,signed int mapsize,unsigned short* values)
{
	void *f;
	f=(void*)getfunc("glPixelMapusv");
	ov_glPixelMapusv=f;
	ov_glPixelMapusv(map,mapsize,values);
}
void (APIENTRY *ov_glPixelMapusv)(unsigned int map,signed int mapsize,unsigned short* values)=init_ov_glPixelMapusv;

void __gl_ml_glPixelMapusv(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	unsigned short* l2=(unsigned short*)(gl__ba__(v2));
	ov_glPixelMapusv(l0,l1,l2);
}
extern void (APIENTRY *ov_glPixelMapuiv)(unsigned int map,signed int mapsize,unsigned int* values);

static void APIENTRY init_ov_glPixelMapuiv(unsigned int map,signed int mapsize,unsigned int* values)
{
	void *f;
	f=(void*)getfunc("glPixelMapuiv");
	ov_glPixelMapuiv=f;
	ov_glPixelMapuiv(map,mapsize,values);
}
void (APIENTRY *ov_glPixelMapuiv)(unsigned int map,signed int mapsize,unsigned int* values)=init_ov_glPixelMapuiv;

void __gl_ml_glPixelMapuiv(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	unsigned int* l2=(unsigned int*)(gl__ba__(v2));
	ov_glPixelMapuiv(l0,l1,l2);
}
extern void (APIENTRY *ov_glPixelMapfv)(unsigned int map,signed int mapsize,float* values);

static void APIENTRY init_ov_glPixelMapfv(unsigned int map,signed int mapsize,float* values)
{
	void *f;
	f=(void*)getfunc("glPixelMapfv");
	ov_glPixelMapfv=f;
	ov_glPixelMapfv(map,mapsize,values);
}
void (APIENTRY *ov_glPixelMapfv)(unsigned int map,signed int mapsize,float* values)=init_ov_glPixelMapfv;

void __gl_ml_glPixelMapfv(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glPixelMapfv(l0,l1,l2);
}
extern void (APIENTRY *ov_glPixelTransferi)(unsigned int pname,signed int param);

static void APIENTRY init_ov_glPixelTransferi(unsigned int pname,signed int param)
{
	void *f;
	f=(void*)getfunc("glPixelTransferi");
	ov_glPixelTransferi=f;
	ov_glPixelTransferi(pname,param);
}
void (APIENTRY *ov_glPixelTransferi)(unsigned int pname,signed int param)=init_ov_glPixelTransferi;

void __gl_ml_glPixelTransferi(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	ov_glPixelTransferi(l0,l1);
}
extern void (APIENTRY *ov_glPixelTransferf)(unsigned int pname,float param);

static void APIENTRY init_ov_glPixelTransferf(unsigned int pname,float param)
{
	void *f;
	f=(void*)getfunc("glPixelTransferf");
	ov_glPixelTransferf=f;
	ov_glPixelTransferf(pname,param);
}
void (APIENTRY *ov_glPixelTransferf)(unsigned int pname,float param)=init_ov_glPixelTransferf;

void __gl_ml_glPixelTransferf(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float l1=Double_val(v1);
	ov_glPixelTransferf(l0,l1);
}
extern void (APIENTRY *ov_glPixelStorei)(unsigned int pname,signed int param);

static void APIENTRY init_ov_glPixelStorei(unsigned int pname,signed int param)
{
	void *f;
	f=(void*)getfunc("glPixelStorei");
	ov_glPixelStorei=f;
	ov_glPixelStorei(pname,param);
}
void (APIENTRY *ov_glPixelStorei)(unsigned int pname,signed int param)=init_ov_glPixelStorei;

void __gl_ml_glPixelStorei(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	ov_glPixelStorei(l0,l1);
}
extern void (APIENTRY *ov_glPixelStoref)(unsigned int pname,float param);

static void APIENTRY init_ov_glPixelStoref(unsigned int pname,float param)
{
	void *f;
	f=(void*)getfunc("glPixelStoref");
	ov_glPixelStoref=f;
	ov_glPixelStoref(pname,param);
}
void (APIENTRY *ov_glPixelStoref)(unsigned int pname,float param)=init_ov_glPixelStoref;

void __gl_ml_glPixelStoref(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float l1=Double_val(v1);
	ov_glPixelStoref(l0,l1);
}
extern void (APIENTRY *ov_glPixelZoom)(float xfactor,float yfactor);

static void APIENTRY init_ov_glPixelZoom(float xfactor,float yfactor)
{
	void *f;
	f=(void*)getfunc("glPixelZoom");
	ov_glPixelZoom=f;
	ov_glPixelZoom(xfactor,yfactor);
}
void (APIENTRY *ov_glPixelZoom)(float xfactor,float yfactor)=init_ov_glPixelZoom;

void __gl_ml_glPixelZoom(value v0,value v1)
{
	float l0=Double_val(v0);
	float l1=Double_val(v1);
	ov_glPixelZoom(l0,l1);
}
extern void (APIENTRY *ov_glColorMaterial)(unsigned int face,unsigned int mode);

static void APIENTRY init_ov_glColorMaterial(unsigned int face,unsigned int mode)
{
	void *f;
	f=(void*)getfunc("glColorMaterial");
	ov_glColorMaterial=f;
	ov_glColorMaterial(face,mode);
}
void (APIENTRY *ov_glColorMaterial)(unsigned int face,unsigned int mode)=init_ov_glColorMaterial;

void __gl_ml_glColorMaterial(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	ov_glColorMaterial(l0,l1);
}
extern void (APIENTRY *ov_glGetMaterialiv)(unsigned int face,unsigned int pname,signed int* params);

static void APIENTRY init_ov_glGetMaterialiv(unsigned int face,unsigned int pname,signed int* params)
{
	void *f;
	f=(void*)getfunc("glGetMaterialiv");
	ov_glGetMaterialiv=f;
	ov_glGetMaterialiv(face,pname,params);
}
void (APIENTRY *ov_glGetMaterialiv)(unsigned int face,unsigned int pname,signed int* params)=init_ov_glGetMaterialiv;

void __gl_ml_glGetMaterialiv(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int* l2=(signed int*)(gl__ba__(v2));
	ov_glGetMaterialiv(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetMaterialfv)(unsigned int face,unsigned int pname,float* params);

static void APIENTRY init_ov_glGetMaterialfv(unsigned int face,unsigned int pname,float* params)
{
	void *f;
	f=(void*)getfunc("glGetMaterialfv");
	ov_glGetMaterialfv=f;
	ov_glGetMaterialfv(face,pname,params);
}
void (APIENTRY *ov_glGetMaterialfv)(unsigned int face,unsigned int pname,float* params)=init_ov_glGetMaterialfv;

void __gl_ml_glGetMaterialfv(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glGetMaterialfv(l0,l1,l2);
}
extern void (APIENTRY *ov_glMaterialiv)(unsigned int face,unsigned int pname,signed int* params);

static void APIENTRY init_ov_glMaterialiv(unsigned int face,unsigned int pname,signed int* params)
{
	void *f;
	f=(void*)getfunc("glMaterialiv");
	ov_glMaterialiv=f;
	ov_glMaterialiv(face,pname,params);
}
void (APIENTRY *ov_glMaterialiv)(unsigned int face,unsigned int pname,signed int* params)=init_ov_glMaterialiv;

void __gl_ml_glMaterialiv(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int* l2=(signed int*)(gl__ba__(v2));
	ov_glMaterialiv(l0,l1,l2);
}
extern void (APIENTRY *ov_glMaterialfv)(unsigned int face,unsigned int pname,float* params);

static void APIENTRY init_ov_glMaterialfv(unsigned int face,unsigned int pname,float* params)
{
	void *f;
	f=(void*)getfunc("glMaterialfv");
	ov_glMaterialfv=f;
	ov_glMaterialfv(face,pname,params);
}
void (APIENTRY *ov_glMaterialfv)(unsigned int face,unsigned int pname,float* params)=init_ov_glMaterialfv;

void __gl_ml_glMaterialfv(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glMaterialfv(l0,l1,l2);
}
extern void (APIENTRY *ov_glMateriali)(unsigned int face,unsigned int pname,signed int param);

static void APIENTRY init_ov_glMateriali(unsigned int face,unsigned int pname,signed int param)
{
	void *f;
	f=(void*)getfunc("glMateriali");
	ov_glMateriali=f;
	ov_glMateriali(face,pname,param);
}
void (APIENTRY *ov_glMateriali)(unsigned int face,unsigned int pname,signed int param)=init_ov_glMateriali;

void __gl_ml_glMateriali(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	ov_glMateriali(l0,l1,l2);
}
extern void (APIENTRY *ov_glMaterialf)(unsigned int face,unsigned int pname,float param);

static void APIENTRY init_ov_glMaterialf(unsigned int face,unsigned int pname,float param)
{
	void *f;
	f=(void*)getfunc("glMaterialf");
	ov_glMaterialf=f;
	ov_glMaterialf(face,pname,param);
}
void (APIENTRY *ov_glMaterialf)(unsigned int face,unsigned int pname,float param)=init_ov_glMaterialf;

void __gl_ml_glMaterialf(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float l2=Double_val(v2);
	ov_glMaterialf(l0,l1,l2);
}
extern void (APIENTRY *ov_glLightModeliv)(unsigned int pname,signed int* params);

static void APIENTRY init_ov_glLightModeliv(unsigned int pname,signed int* params)
{
	void *f;
	f=(void*)getfunc("glLightModeliv");
	ov_glLightModeliv=f;
	ov_glLightModeliv(pname,params);
}
void (APIENTRY *ov_glLightModeliv)(unsigned int pname,signed int* params)=init_ov_glLightModeliv;

void __gl_ml_glLightModeliv(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed int* l1=(signed int*)(gl__ba__(v1));
	ov_glLightModeliv(l0,l1);
}
extern void (APIENTRY *ov_glLightModelfv)(unsigned int pname,float* params);

static void APIENTRY init_ov_glLightModelfv(unsigned int pname,float* params)
{
	void *f;
	f=(void*)getfunc("glLightModelfv");
	ov_glLightModelfv=f;
	ov_glLightModelfv(pname,params);
}
void (APIENTRY *ov_glLightModelfv)(unsigned int pname,float* params)=init_ov_glLightModelfv;

void __gl_ml_glLightModelfv(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float* l1=(float*)(gl__ba__(v1));
	ov_glLightModelfv(l0,l1);
}
extern void (APIENTRY *ov_glLightModeli)(unsigned int pname,signed int param);

static void APIENTRY init_ov_glLightModeli(unsigned int pname,signed int param)
{
	void *f;
	f=(void*)getfunc("glLightModeli");
	ov_glLightModeli=f;
	ov_glLightModeli(pname,param);
}
void (APIENTRY *ov_glLightModeli)(unsigned int pname,signed int param)=init_ov_glLightModeli;

void __gl_ml_glLightModeli(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	ov_glLightModeli(l0,l1);
}
extern void (APIENTRY *ov_glLightModelf)(unsigned int pname,float param);

static void APIENTRY init_ov_glLightModelf(unsigned int pname,float param)
{
	void *f;
	f=(void*)getfunc("glLightModelf");
	ov_glLightModelf=f;
	ov_glLightModelf(pname,param);
}
void (APIENTRY *ov_glLightModelf)(unsigned int pname,float param)=init_ov_glLightModelf;

void __gl_ml_glLightModelf(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float l1=Double_val(v1);
	ov_glLightModelf(l0,l1);
}
extern void (APIENTRY *ov_glGetLightiv)(unsigned int light,unsigned int pname,signed int* params);

static void APIENTRY init_ov_glGetLightiv(unsigned int light,unsigned int pname,signed int* params)
{
	void *f;
	f=(void*)getfunc("glGetLightiv");
	ov_glGetLightiv=f;
	ov_glGetLightiv(light,pname,params);
}
void (APIENTRY *ov_glGetLightiv)(unsigned int light,unsigned int pname,signed int* params)=init_ov_glGetLightiv;

void __gl_ml_glGetLightiv(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int* l2=(signed int*)(gl__ba__(v2));
	ov_glGetLightiv(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetLightfv)(unsigned int light,unsigned int pname,float* params);

static void APIENTRY init_ov_glGetLightfv(unsigned int light,unsigned int pname,float* params)
{
	void *f;
	f=(void*)getfunc("glGetLightfv");
	ov_glGetLightfv=f;
	ov_glGetLightfv(light,pname,params);
}
void (APIENTRY *ov_glGetLightfv)(unsigned int light,unsigned int pname,float* params)=init_ov_glGetLightfv;

void __gl_ml_glGetLightfv(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glGetLightfv(l0,l1,l2);
}
extern void (APIENTRY *ov_glLightiv)(unsigned int light,unsigned int pname,signed int* params);

static void APIENTRY init_ov_glLightiv(unsigned int light,unsigned int pname,signed int* params)
{
	void *f;
	f=(void*)getfunc("glLightiv");
	ov_glLightiv=f;
	ov_glLightiv(light,pname,params);
}
void (APIENTRY *ov_glLightiv)(unsigned int light,unsigned int pname,signed int* params)=init_ov_glLightiv;

void __gl_ml_glLightiv(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int* l2=(signed int*)(gl__ba__(v2));
	ov_glLightiv(l0,l1,l2);
}
extern void (APIENTRY *ov_glLightfv)(unsigned int light,unsigned int pname,float* params);

static void APIENTRY init_ov_glLightfv(unsigned int light,unsigned int pname,float* params)
{
	void *f;
	f=(void*)getfunc("glLightfv");
	ov_glLightfv=f;
	ov_glLightfv(light,pname,params);
}
void (APIENTRY *ov_glLightfv)(unsigned int light,unsigned int pname,float* params)=init_ov_glLightfv;

void __gl_ml_glLightfv(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glLightfv(l0,l1,l2);
}
extern void (APIENTRY *ov_glLighti)(unsigned int light,unsigned int pname,signed int param);

static void APIENTRY init_ov_glLighti(unsigned int light,unsigned int pname,signed int param)
{
	void *f;
	f=(void*)getfunc("glLighti");
	ov_glLighti=f;
	ov_glLighti(light,pname,param);
}
void (APIENTRY *ov_glLighti)(unsigned int light,unsigned int pname,signed int param)=init_ov_glLighti;

void __gl_ml_glLighti(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	ov_glLighti(l0,l1,l2);
}
extern void (APIENTRY *ov_glLightf)(unsigned int light,unsigned int pname,float param);

static void APIENTRY init_ov_glLightf(unsigned int light,unsigned int pname,float param)
{
	void *f;
	f=(void*)getfunc("glLightf");
	ov_glLightf=f;
	ov_glLightf(light,pname,param);
}
void (APIENTRY *ov_glLightf)(unsigned int light,unsigned int pname,float param)=init_ov_glLightf;

void __gl_ml_glLightf(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float l2=Double_val(v2);
	ov_glLightf(l0,l1,l2);
}
extern void (APIENTRY *ov_glShadeModel)(unsigned int mode);

static void APIENTRY init_ov_glShadeModel(unsigned int mode)
{
	void *f;
	f=(void*)getfunc("glShadeModel");
	ov_glShadeModel=f;
	ov_glShadeModel(mode);
}
void (APIENTRY *ov_glShadeModel)(unsigned int mode)=init_ov_glShadeModel;

void __gl_ml_glShadeModel(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glShadeModel(l0);
}
extern void (APIENTRY *ov_glRectsv)(signed short* v1,signed short* v2);

static void APIENTRY init_ov_glRectsv(signed short* v1,signed short* v2)
{
	void *f;
	f=(void*)getfunc("glRectsv");
	ov_glRectsv=f;
	ov_glRectsv(v1,v2);
}
void (APIENTRY *ov_glRectsv)(signed short* v1,signed short* v2)=init_ov_glRectsv;

void __gl_ml_glRectsv(value v0,value v1)
{
	signed short* l0=(signed short*)(gl__ba__(v0));
	signed short* l1=(signed short*)(gl__ba__(v1));
	ov_glRectsv(l0,l1);
}
extern void (APIENTRY *ov_glRectiv)(signed int* v1,signed int* v2);

static void APIENTRY init_ov_glRectiv(signed int* v1,signed int* v2)
{
	void *f;
	f=(void*)getfunc("glRectiv");
	ov_glRectiv=f;
	ov_glRectiv(v1,v2);
}
void (APIENTRY *ov_glRectiv)(signed int* v1,signed int* v2)=init_ov_glRectiv;

void __gl_ml_glRectiv(value v0,value v1)
{
	signed int* l0=(signed int*)(gl__ba__(v0));
	signed int* l1=(signed int*)(gl__ba__(v1));
	ov_glRectiv(l0,l1);
}
extern void (APIENTRY *ov_glRectfv)(float* v1,float* v2);

static void APIENTRY init_ov_glRectfv(float* v1,float* v2)
{
	void *f;
	f=(void*)getfunc("glRectfv");
	ov_glRectfv=f;
	ov_glRectfv(v1,v2);
}
void (APIENTRY *ov_glRectfv)(float* v1,float* v2)=init_ov_glRectfv;

void __gl_ml_glRectfv(value v0,value v1)
{
	float* l0=(float*)(gl__ba__(v0));
	float* l1=(float*)(gl__ba__(v1));
	ov_glRectfv(l0,l1);
}
extern void (APIENTRY *ov_glRectdv)(double* v1,double* v2);

static void APIENTRY init_ov_glRectdv(double* v1,double* v2)
{
	void *f;
	f=(void*)getfunc("glRectdv");
	ov_glRectdv=f;
	ov_glRectdv(v1,v2);
}
void (APIENTRY *ov_glRectdv)(double* v1,double* v2)=init_ov_glRectdv;

void __gl_ml_glRectdv(value v0,value v1)
{
	double* l0=(double*)(gl__ba__(v0));
	double* l1=(double*)(gl__ba__(v1));
	ov_glRectdv(l0,l1);
}
extern void (APIENTRY *ov_glRects)(signed short x1,signed short y1,signed short x2,signed short y2);

static void APIENTRY init_ov_glRects(signed short x1,signed short y1,signed short x2,signed short y2)
{
	void *f;
	f=(void*)getfunc("glRects");
	ov_glRects=f;
	ov_glRects(x1,y1,x2,y2);
}
void (APIENTRY *ov_glRects)(signed short x1,signed short y1,signed short x2,signed short y2)=init_ov_glRects;

void __gl_ml_glRects(value v0,value v1,value v2,value v3)
{
	signed short l0=Int_val(v0);
	signed short l1=Int_val(v1);
	signed short l2=Int_val(v2);
	signed short l3=Int_val(v3);
	ov_glRects(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glRecti)(signed int x1,signed int y1,signed int x2,signed int y2);

static void APIENTRY init_ov_glRecti(signed int x1,signed int y1,signed int x2,signed int y2)
{
	void *f;
	f=(void*)getfunc("glRecti");
	ov_glRecti=f;
	ov_glRecti(x1,y1,x2,y2);
}
void (APIENTRY *ov_glRecti)(signed int x1,signed int y1,signed int x2,signed int y2)=init_ov_glRecti;

void __gl_ml_glRecti(value v0,value v1,value v2,value v3)
{
	signed int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	ov_glRecti(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glRectf)(float x1,float y1,float x2,float y2);

static void APIENTRY init_ov_glRectf(float x1,float y1,float x2,float y2)
{
	void *f;
	f=(void*)getfunc("glRectf");
	ov_glRectf=f;
	ov_glRectf(x1,y1,x2,y2);
}
void (APIENTRY *ov_glRectf)(float x1,float y1,float x2,float y2)=init_ov_glRectf;

void __gl_ml_glRectf(value v0,value v1,value v2,value v3)
{
	float l0=Double_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	float l3=Double_val(v3);
	ov_glRectf(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glRectd)(double x1,double y1,double x2,double y2);

static void APIENTRY init_ov_glRectd(double x1,double y1,double x2,double y2)
{
	void *f;
	f=(void*)getfunc("glRectd");
	ov_glRectd=f;
	ov_glRectd(x1,y1,x2,y2);
}
void (APIENTRY *ov_glRectd)(double x1,double y1,double x2,double y2)=init_ov_glRectd;

void __gl_ml_glRectd(value v0,value v1,value v2,value v3)
{
	double l0=Double_val(v0);
	double l1=Double_val(v1);
	double l2=Double_val(v2);
	double l3=Double_val(v3);
	ov_glRectd(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glRasterPos4sv)(signed short* v);

static void APIENTRY init_ov_glRasterPos4sv(signed short* v)
{
	void *f;
	f=(void*)getfunc("glRasterPos4sv");
	ov_glRasterPos4sv=f;
	ov_glRasterPos4sv(v);
}
void (APIENTRY *ov_glRasterPos4sv)(signed short* v)=init_ov_glRasterPos4sv;

void __gl_ml_glRasterPos4sv(value v0)
{
	signed short* l0=(signed short*)(gl__ba__(v0));
	ov_glRasterPos4sv(l0);
}
extern void (APIENTRY *ov_glRasterPos4iv)(signed int* v);

static void APIENTRY init_ov_glRasterPos4iv(signed int* v)
{
	void *f;
	f=(void*)getfunc("glRasterPos4iv");
	ov_glRasterPos4iv=f;
	ov_glRasterPos4iv(v);
}
void (APIENTRY *ov_glRasterPos4iv)(signed int* v)=init_ov_glRasterPos4iv;

void __gl_ml_glRasterPos4iv(value v0)
{
	signed int* l0=(signed int*)(gl__ba__(v0));
	ov_glRasterPos4iv(l0);
}
extern void (APIENTRY *ov_glRasterPos4fv)(float* v);

static void APIENTRY init_ov_glRasterPos4fv(float* v)
{
	void *f;
	f=(void*)getfunc("glRasterPos4fv");
	ov_glRasterPos4fv=f;
	ov_glRasterPos4fv(v);
}
void (APIENTRY *ov_glRasterPos4fv)(float* v)=init_ov_glRasterPos4fv;

void __gl_ml_glRasterPos4fv(value v0)
{
	float* l0=(float*)(gl__ba__(v0));
	ov_glRasterPos4fv(l0);
}
extern void (APIENTRY *ov_glRasterPos4dv)(double* v);

static void APIENTRY init_ov_glRasterPos4dv(double* v)
{
	void *f;
	f=(void*)getfunc("glRasterPos4dv");
	ov_glRasterPos4dv=f;
	ov_glRasterPos4dv(v);
}
void (APIENTRY *ov_glRasterPos4dv)(double* v)=init_ov_glRasterPos4dv;

void __gl_ml_glRasterPos4dv(value v0)
{
	double* l0=(double*)(gl__ba__(v0));
	ov_glRasterPos4dv(l0);
}
extern void (APIENTRY *ov_glRasterPos3sv)(signed short* v);

static void APIENTRY init_ov_glRasterPos3sv(signed short* v)
{
	void *f;
	f=(void*)getfunc("glRasterPos3sv");
	ov_glRasterPos3sv=f;
	ov_glRasterPos3sv(v);
}
void (APIENTRY *ov_glRasterPos3sv)(signed short* v)=init_ov_glRasterPos3sv;

void __gl_ml_glRasterPos3sv(value v0)
{
	signed short* l0=(signed short*)(gl__ba__(v0));
	ov_glRasterPos3sv(l0);
}
extern void (APIENTRY *ov_glRasterPos3iv)(signed int* v);

static void APIENTRY init_ov_glRasterPos3iv(signed int* v)
{
	void *f;
	f=(void*)getfunc("glRasterPos3iv");
	ov_glRasterPos3iv=f;
	ov_glRasterPos3iv(v);
}
void (APIENTRY *ov_glRasterPos3iv)(signed int* v)=init_ov_glRasterPos3iv;

void __gl_ml_glRasterPos3iv(value v0)
{
	signed int* l0=(signed int*)(gl__ba__(v0));
	ov_glRasterPos3iv(l0);
}
extern void (APIENTRY *ov_glRasterPos3fv)(float* v);

static void APIENTRY init_ov_glRasterPos3fv(float* v)
{
	void *f;
	f=(void*)getfunc("glRasterPos3fv");
	ov_glRasterPos3fv=f;
	ov_glRasterPos3fv(v);
}
void (APIENTRY *ov_glRasterPos3fv)(float* v)=init_ov_glRasterPos3fv;

void __gl_ml_glRasterPos3fv(value v0)
{
	float* l0=(float*)(gl__ba__(v0));
	ov_glRasterPos3fv(l0);
}
extern void (APIENTRY *ov_glRasterPos3dv)(double* v);

static void APIENTRY init_ov_glRasterPos3dv(double* v)
{
	void *f;
	f=(void*)getfunc("glRasterPos3dv");
	ov_glRasterPos3dv=f;
	ov_glRasterPos3dv(v);
}
void (APIENTRY *ov_glRasterPos3dv)(double* v)=init_ov_glRasterPos3dv;

void __gl_ml_glRasterPos3dv(value v0)
{
	double* l0=(double*)(gl__ba__(v0));
	ov_glRasterPos3dv(l0);
}
extern void (APIENTRY *ov_glRasterPos2sv)(signed short* v);

static void APIENTRY init_ov_glRasterPos2sv(signed short* v)
{
	void *f;
	f=(void*)getfunc("glRasterPos2sv");
	ov_glRasterPos2sv=f;
	ov_glRasterPos2sv(v);
}
void (APIENTRY *ov_glRasterPos2sv)(signed short* v)=init_ov_glRasterPos2sv;

void __gl_ml_glRasterPos2sv(value v0)
{
	signed short* l0=(signed short*)(gl__ba__(v0));
	ov_glRasterPos2sv(l0);
}
extern void (APIENTRY *ov_glRasterPos2iv)(signed int* v);

static void APIENTRY init_ov_glRasterPos2iv(signed int* v)
{
	void *f;
	f=(void*)getfunc("glRasterPos2iv");
	ov_glRasterPos2iv=f;
	ov_glRasterPos2iv(v);
}
void (APIENTRY *ov_glRasterPos2iv)(signed int* v)=init_ov_glRasterPos2iv;

void __gl_ml_glRasterPos2iv(value v0)
{
	signed int* l0=(signed int*)(gl__ba__(v0));
	ov_glRasterPos2iv(l0);
}
extern void (APIENTRY *ov_glRasterPos2fv)(float* v);

static void APIENTRY init_ov_glRasterPos2fv(float* v)
{
	void *f;
	f=(void*)getfunc("glRasterPos2fv");
	ov_glRasterPos2fv=f;
	ov_glRasterPos2fv(v);
}
void (APIENTRY *ov_glRasterPos2fv)(float* v)=init_ov_glRasterPos2fv;

void __gl_ml_glRasterPos2fv(value v0)
{
	float* l0=(float*)(gl__ba__(v0));
	ov_glRasterPos2fv(l0);
}
extern void (APIENTRY *ov_glRasterPos2dv)(double* v);

static void APIENTRY init_ov_glRasterPos2dv(double* v)
{
	void *f;
	f=(void*)getfunc("glRasterPos2dv");
	ov_glRasterPos2dv=f;
	ov_glRasterPos2dv(v);
}
void (APIENTRY *ov_glRasterPos2dv)(double* v)=init_ov_glRasterPos2dv;

void __gl_ml_glRasterPos2dv(value v0)
{
	double* l0=(double*)(gl__ba__(v0));
	ov_glRasterPos2dv(l0);
}
extern void (APIENTRY *ov_glRasterPos4s)(signed short x,signed short y,signed short z,signed short w);

static void APIENTRY init_ov_glRasterPos4s(signed short x,signed short y,signed short z,signed short w)
{
	void *f;
	f=(void*)getfunc("glRasterPos4s");
	ov_glRasterPos4s=f;
	ov_glRasterPos4s(x,y,z,w);
}
void (APIENTRY *ov_glRasterPos4s)(signed short x,signed short y,signed short z,signed short w)=init_ov_glRasterPos4s;

void __gl_ml_glRasterPos4s(value v0,value v1,value v2,value v3)
{
	signed short l0=Int_val(v0);
	signed short l1=Int_val(v1);
	signed short l2=Int_val(v2);
	signed short l3=Int_val(v3);
	ov_glRasterPos4s(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glRasterPos4i)(signed int x,signed int y,signed int z,signed int w);

static void APIENTRY init_ov_glRasterPos4i(signed int x,signed int y,signed int z,signed int w)
{
	void *f;
	f=(void*)getfunc("glRasterPos4i");
	ov_glRasterPos4i=f;
	ov_glRasterPos4i(x,y,z,w);
}
void (APIENTRY *ov_glRasterPos4i)(signed int x,signed int y,signed int z,signed int w)=init_ov_glRasterPos4i;

void __gl_ml_glRasterPos4i(value v0,value v1,value v2,value v3)
{
	signed int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	ov_glRasterPos4i(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glRasterPos4f)(float x,float y,float z,float w);

static void APIENTRY init_ov_glRasterPos4f(float x,float y,float z,float w)
{
	void *f;
	f=(void*)getfunc("glRasterPos4f");
	ov_glRasterPos4f=f;
	ov_glRasterPos4f(x,y,z,w);
}
void (APIENTRY *ov_glRasterPos4f)(float x,float y,float z,float w)=init_ov_glRasterPos4f;

void __gl_ml_glRasterPos4f(value v0,value v1,value v2,value v3)
{
	float l0=Double_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	float l3=Double_val(v3);
	ov_glRasterPos4f(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glRasterPos4d)(double x,double y,double z,double w);

static void APIENTRY init_ov_glRasterPos4d(double x,double y,double z,double w)
{
	void *f;
	f=(void*)getfunc("glRasterPos4d");
	ov_glRasterPos4d=f;
	ov_glRasterPos4d(x,y,z,w);
}
void (APIENTRY *ov_glRasterPos4d)(double x,double y,double z,double w)=init_ov_glRasterPos4d;

void __gl_ml_glRasterPos4d(value v0,value v1,value v2,value v3)
{
	double l0=Double_val(v0);
	double l1=Double_val(v1);
	double l2=Double_val(v2);
	double l3=Double_val(v3);
	ov_glRasterPos4d(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glRasterPos3s)(signed short x,signed short y,signed short z);

static void APIENTRY init_ov_glRasterPos3s(signed short x,signed short y,signed short z)
{
	void *f;
	f=(void*)getfunc("glRasterPos3s");
	ov_glRasterPos3s=f;
	ov_glRasterPos3s(x,y,z);
}
void (APIENTRY *ov_glRasterPos3s)(signed short x,signed short y,signed short z)=init_ov_glRasterPos3s;

void __gl_ml_glRasterPos3s(value v0,value v1,value v2)
{
	signed short l0=Int_val(v0);
	signed short l1=Int_val(v1);
	signed short l2=Int_val(v2);
	ov_glRasterPos3s(l0,l1,l2);
}
extern void (APIENTRY *ov_glRasterPos3i)(signed int x,signed int y,signed int z);

static void APIENTRY init_ov_glRasterPos3i(signed int x,signed int y,signed int z)
{
	void *f;
	f=(void*)getfunc("glRasterPos3i");
	ov_glRasterPos3i=f;
	ov_glRasterPos3i(x,y,z);
}
void (APIENTRY *ov_glRasterPos3i)(signed int x,signed int y,signed int z)=init_ov_glRasterPos3i;

void __gl_ml_glRasterPos3i(value v0,value v1,value v2)
{
	signed int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	ov_glRasterPos3i(l0,l1,l2);
}
extern void (APIENTRY *ov_glRasterPos3f)(float x,float y,float z);

static void APIENTRY init_ov_glRasterPos3f(float x,float y,float z)
{
	void *f;
	f=(void*)getfunc("glRasterPos3f");
	ov_glRasterPos3f=f;
	ov_glRasterPos3f(x,y,z);
}
void (APIENTRY *ov_glRasterPos3f)(float x,float y,float z)=init_ov_glRasterPos3f;

void __gl_ml_glRasterPos3f(value v0,value v1,value v2)
{
	float l0=Double_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	ov_glRasterPos3f(l0,l1,l2);
}
extern void (APIENTRY *ov_glRasterPos3d)(double x,double y,double z);

static void APIENTRY init_ov_glRasterPos3d(double x,double y,double z)
{
	void *f;
	f=(void*)getfunc("glRasterPos3d");
	ov_glRasterPos3d=f;
	ov_glRasterPos3d(x,y,z);
}
void (APIENTRY *ov_glRasterPos3d)(double x,double y,double z)=init_ov_glRasterPos3d;

void __gl_ml_glRasterPos3d(value v0,value v1,value v2)
{
	double l0=Double_val(v0);
	double l1=Double_val(v1);
	double l2=Double_val(v2);
	ov_glRasterPos3d(l0,l1,l2);
}
extern void (APIENTRY *ov_glRasterPos2s)(signed short x,signed short y);

static void APIENTRY init_ov_glRasterPos2s(signed short x,signed short y)
{
	void *f;
	f=(void*)getfunc("glRasterPos2s");
	ov_glRasterPos2s=f;
	ov_glRasterPos2s(x,y);
}
void (APIENTRY *ov_glRasterPos2s)(signed short x,signed short y)=init_ov_glRasterPos2s;

void __gl_ml_glRasterPos2s(value v0,value v1)
{
	signed short l0=Int_val(v0);
	signed short l1=Int_val(v1);
	ov_glRasterPos2s(l0,l1);
}
extern void (APIENTRY *ov_glRasterPos2i)(signed int x,signed int y);

static void APIENTRY init_ov_glRasterPos2i(signed int x,signed int y)
{
	void *f;
	f=(void*)getfunc("glRasterPos2i");
	ov_glRasterPos2i=f;
	ov_glRasterPos2i(x,y);
}
void (APIENTRY *ov_glRasterPos2i)(signed int x,signed int y)=init_ov_glRasterPos2i;

void __gl_ml_glRasterPos2i(value v0,value v1)
{
	signed int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	ov_glRasterPos2i(l0,l1);
}
extern void (APIENTRY *ov_glRasterPos2f)(float x,float y);

static void APIENTRY init_ov_glRasterPos2f(float x,float y)
{
	void *f;
	f=(void*)getfunc("glRasterPos2f");
	ov_glRasterPos2f=f;
	ov_glRasterPos2f(x,y);
}
void (APIENTRY *ov_glRasterPos2f)(float x,float y)=init_ov_glRasterPos2f;

void __gl_ml_glRasterPos2f(value v0,value v1)
{
	float l0=Double_val(v0);
	float l1=Double_val(v1);
	ov_glRasterPos2f(l0,l1);
}
extern void (APIENTRY *ov_glRasterPos2d)(double x,double y);

static void APIENTRY init_ov_glRasterPos2d(double x,double y)
{
	void *f;
	f=(void*)getfunc("glRasterPos2d");
	ov_glRasterPos2d=f;
	ov_glRasterPos2d(x,y);
}
void (APIENTRY *ov_glRasterPos2d)(double x,double y)=init_ov_glRasterPos2d;

void __gl_ml_glRasterPos2d(value v0,value v1)
{
	double l0=Double_val(v0);
	double l1=Double_val(v1);
	ov_glRasterPos2d(l0,l1);
}
extern void (APIENTRY *ov_glTexCoord4sv)(signed short* v);

static void APIENTRY init_ov_glTexCoord4sv(signed short* v)
{
	void *f;
	f=(void*)getfunc("glTexCoord4sv");
	ov_glTexCoord4sv=f;
	ov_glTexCoord4sv(v);
}
void (APIENTRY *ov_glTexCoord4sv)(signed short* v)=init_ov_glTexCoord4sv;

void __gl_ml_glTexCoord4sv(value v0)
{
	signed short* l0=(signed short*)(gl__ba__(v0));
	ov_glTexCoord4sv(l0);
}
extern void (APIENTRY *ov_glTexCoord4iv)(signed int* v);

static void APIENTRY init_ov_glTexCoord4iv(signed int* v)
{
	void *f;
	f=(void*)getfunc("glTexCoord4iv");
	ov_glTexCoord4iv=f;
	ov_glTexCoord4iv(v);
}
void (APIENTRY *ov_glTexCoord4iv)(signed int* v)=init_ov_glTexCoord4iv;

void __gl_ml_glTexCoord4iv(value v0)
{
	signed int* l0=(signed int*)(gl__ba__(v0));
	ov_glTexCoord4iv(l0);
}
extern void (APIENTRY *ov_glTexCoord4fv)(float* v);

static void APIENTRY init_ov_glTexCoord4fv(float* v)
{
	void *f;
	f=(void*)getfunc("glTexCoord4fv");
	ov_glTexCoord4fv=f;
	ov_glTexCoord4fv(v);
}
void (APIENTRY *ov_glTexCoord4fv)(float* v)=init_ov_glTexCoord4fv;

void __gl_ml_glTexCoord4fv(value v0)
{
	float* l0=(float*)(gl__ba__(v0));
	ov_glTexCoord4fv(l0);
}
extern void (APIENTRY *ov_glTexCoord4dv)(double* v);

static void APIENTRY init_ov_glTexCoord4dv(double* v)
{
	void *f;
	f=(void*)getfunc("glTexCoord4dv");
	ov_glTexCoord4dv=f;
	ov_glTexCoord4dv(v);
}
void (APIENTRY *ov_glTexCoord4dv)(double* v)=init_ov_glTexCoord4dv;

void __gl_ml_glTexCoord4dv(value v0)
{
	double* l0=(double*)(gl__ba__(v0));
	ov_glTexCoord4dv(l0);
}
extern void (APIENTRY *ov_glTexCoord3sv)(signed short* v);

static void APIENTRY init_ov_glTexCoord3sv(signed short* v)
{
	void *f;
	f=(void*)getfunc("glTexCoord3sv");
	ov_glTexCoord3sv=f;
	ov_glTexCoord3sv(v);
}
void (APIENTRY *ov_glTexCoord3sv)(signed short* v)=init_ov_glTexCoord3sv;

void __gl_ml_glTexCoord3sv(value v0)
{
	signed short* l0=(signed short*)(gl__ba__(v0));
	ov_glTexCoord3sv(l0);
}
extern void (APIENTRY *ov_glTexCoord3iv)(signed int* v);

static void APIENTRY init_ov_glTexCoord3iv(signed int* v)
{
	void *f;
	f=(void*)getfunc("glTexCoord3iv");
	ov_glTexCoord3iv=f;
	ov_glTexCoord3iv(v);
}
void (APIENTRY *ov_glTexCoord3iv)(signed int* v)=init_ov_glTexCoord3iv;

void __gl_ml_glTexCoord3iv(value v0)
{
	signed int* l0=(signed int*)(gl__ba__(v0));
	ov_glTexCoord3iv(l0);
}
extern void (APIENTRY *ov_glTexCoord3fv)(float* v);

static void APIENTRY init_ov_glTexCoord3fv(float* v)
{
	void *f;
	f=(void*)getfunc("glTexCoord3fv");
	ov_glTexCoord3fv=f;
	ov_glTexCoord3fv(v);
}
void (APIENTRY *ov_glTexCoord3fv)(float* v)=init_ov_glTexCoord3fv;

void __gl_ml_glTexCoord3fv(value v0)
{
	float* l0=(float*)(gl__ba__(v0));
	ov_glTexCoord3fv(l0);
}
extern void (APIENTRY *ov_glTexCoord3dv)(double* v);

static void APIENTRY init_ov_glTexCoord3dv(double* v)
{
	void *f;
	f=(void*)getfunc("glTexCoord3dv");
	ov_glTexCoord3dv=f;
	ov_glTexCoord3dv(v);
}
void (APIENTRY *ov_glTexCoord3dv)(double* v)=init_ov_glTexCoord3dv;

void __gl_ml_glTexCoord3dv(value v0)
{
	double* l0=(double*)(gl__ba__(v0));
	ov_glTexCoord3dv(l0);
}
extern void (APIENTRY *ov_glTexCoord2sv)(signed short* v);

static void APIENTRY init_ov_glTexCoord2sv(signed short* v)
{
	void *f;
	f=(void*)getfunc("glTexCoord2sv");
	ov_glTexCoord2sv=f;
	ov_glTexCoord2sv(v);
}
void (APIENTRY *ov_glTexCoord2sv)(signed short* v)=init_ov_glTexCoord2sv;

void __gl_ml_glTexCoord2sv(value v0)
{
	signed short* l0=(signed short*)(gl__ba__(v0));
	ov_glTexCoord2sv(l0);
}
extern void (APIENTRY *ov_glTexCoord2iv)(signed int* v);

static void APIENTRY init_ov_glTexCoord2iv(signed int* v)
{
	void *f;
	f=(void*)getfunc("glTexCoord2iv");
	ov_glTexCoord2iv=f;
	ov_glTexCoord2iv(v);
}
void (APIENTRY *ov_glTexCoord2iv)(signed int* v)=init_ov_glTexCoord2iv;

void __gl_ml_glTexCoord2iv(value v0)
{
	signed int* l0=(signed int*)(gl__ba__(v0));
	ov_glTexCoord2iv(l0);
}
extern void (APIENTRY *ov_glTexCoord2fv)(float* v);

static void APIENTRY init_ov_glTexCoord2fv(float* v)
{
	void *f;
	f=(void*)getfunc("glTexCoord2fv");
	ov_glTexCoord2fv=f;
	ov_glTexCoord2fv(v);
}
void (APIENTRY *ov_glTexCoord2fv)(float* v)=init_ov_glTexCoord2fv;

void __gl_ml_glTexCoord2fv(value v0)
{
	float* l0=(float*)(gl__ba__(v0));
	ov_glTexCoord2fv(l0);
}
extern void (APIENTRY *ov_glTexCoord2dv)(double* v);

static void APIENTRY init_ov_glTexCoord2dv(double* v)
{
	void *f;
	f=(void*)getfunc("glTexCoord2dv");
	ov_glTexCoord2dv=f;
	ov_glTexCoord2dv(v);
}
void (APIENTRY *ov_glTexCoord2dv)(double* v)=init_ov_glTexCoord2dv;

void __gl_ml_glTexCoord2dv(value v0)
{
	double* l0=(double*)(gl__ba__(v0));
	ov_glTexCoord2dv(l0);
}
extern void (APIENTRY *ov_glTexCoord1sv)(signed short* v);

static void APIENTRY init_ov_glTexCoord1sv(signed short* v)
{
	void *f;
	f=(void*)getfunc("glTexCoord1sv");
	ov_glTexCoord1sv=f;
	ov_glTexCoord1sv(v);
}
void (APIENTRY *ov_glTexCoord1sv)(signed short* v)=init_ov_glTexCoord1sv;

void __gl_ml_glTexCoord1sv(value v0)
{
	signed short* l0=(signed short*)(gl__ba__(v0));
	ov_glTexCoord1sv(l0);
}
extern void (APIENTRY *ov_glTexCoord1iv)(signed int* v);

static void APIENTRY init_ov_glTexCoord1iv(signed int* v)
{
	void *f;
	f=(void*)getfunc("glTexCoord1iv");
	ov_glTexCoord1iv=f;
	ov_glTexCoord1iv(v);
}
void (APIENTRY *ov_glTexCoord1iv)(signed int* v)=init_ov_glTexCoord1iv;

void __gl_ml_glTexCoord1iv(value v0)
{
	signed int* l0=(signed int*)(gl__ba__(v0));
	ov_glTexCoord1iv(l0);
}
extern void (APIENTRY *ov_glTexCoord1fv)(float* v);

static void APIENTRY init_ov_glTexCoord1fv(float* v)
{
	void *f;
	f=(void*)getfunc("glTexCoord1fv");
	ov_glTexCoord1fv=f;
	ov_glTexCoord1fv(v);
}
void (APIENTRY *ov_glTexCoord1fv)(float* v)=init_ov_glTexCoord1fv;

void __gl_ml_glTexCoord1fv(value v0)
{
	float* l0=(float*)(gl__ba__(v0));
	ov_glTexCoord1fv(l0);
}
extern void (APIENTRY *ov_glTexCoord1dv)(double* v);

static void APIENTRY init_ov_glTexCoord1dv(double* v)
{
	void *f;
	f=(void*)getfunc("glTexCoord1dv");
	ov_glTexCoord1dv=f;
	ov_glTexCoord1dv(v);
}
void (APIENTRY *ov_glTexCoord1dv)(double* v)=init_ov_glTexCoord1dv;

void __gl_ml_glTexCoord1dv(value v0)
{
	double* l0=(double*)(gl__ba__(v0));
	ov_glTexCoord1dv(l0);
}
extern void (APIENTRY *ov_glTexCoord4s)(signed short s,signed short t,signed short r,signed short q);

static void APIENTRY init_ov_glTexCoord4s(signed short s,signed short t,signed short r,signed short q)
{
	void *f;
	f=(void*)getfunc("glTexCoord4s");
	ov_glTexCoord4s=f;
	ov_glTexCoord4s(s,t,r,q);
}
void (APIENTRY *ov_glTexCoord4s)(signed short s,signed short t,signed short r,signed short q)=init_ov_glTexCoord4s;

void __gl_ml_glTexCoord4s(value v0,value v1,value v2,value v3)
{
	signed short l0=Int_val(v0);
	signed short l1=Int_val(v1);
	signed short l2=Int_val(v2);
	signed short l3=Int_val(v3);
	ov_glTexCoord4s(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glTexCoord4i)(signed int s,signed int t,signed int r,signed int q);

static void APIENTRY init_ov_glTexCoord4i(signed int s,signed int t,signed int r,signed int q)
{
	void *f;
	f=(void*)getfunc("glTexCoord4i");
	ov_glTexCoord4i=f;
	ov_glTexCoord4i(s,t,r,q);
}
void (APIENTRY *ov_glTexCoord4i)(signed int s,signed int t,signed int r,signed int q)=init_ov_glTexCoord4i;

void __gl_ml_glTexCoord4i(value v0,value v1,value v2,value v3)
{
	signed int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	ov_glTexCoord4i(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glTexCoord4f)(float s,float t,float r,float q);

static void APIENTRY init_ov_glTexCoord4f(float s,float t,float r,float q)
{
	void *f;
	f=(void*)getfunc("glTexCoord4f");
	ov_glTexCoord4f=f;
	ov_glTexCoord4f(s,t,r,q);
}
void (APIENTRY *ov_glTexCoord4f)(float s,float t,float r,float q)=init_ov_glTexCoord4f;

void __gl_ml_glTexCoord4f(value v0,value v1,value v2,value v3)
{
	float l0=Double_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	float l3=Double_val(v3);
	ov_glTexCoord4f(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glTexCoord4d)(double s,double t,double r,double q);

static void APIENTRY init_ov_glTexCoord4d(double s,double t,double r,double q)
{
	void *f;
	f=(void*)getfunc("glTexCoord4d");
	ov_glTexCoord4d=f;
	ov_glTexCoord4d(s,t,r,q);
}
void (APIENTRY *ov_glTexCoord4d)(double s,double t,double r,double q)=init_ov_glTexCoord4d;

void __gl_ml_glTexCoord4d(value v0,value v1,value v2,value v3)
{
	double l0=Double_val(v0);
	double l1=Double_val(v1);
	double l2=Double_val(v2);
	double l3=Double_val(v3);
	ov_glTexCoord4d(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glTexCoord3s)(signed short s,signed short t,signed short r);

static void APIENTRY init_ov_glTexCoord3s(signed short s,signed short t,signed short r)
{
	void *f;
	f=(void*)getfunc("glTexCoord3s");
	ov_glTexCoord3s=f;
	ov_glTexCoord3s(s,t,r);
}
void (APIENTRY *ov_glTexCoord3s)(signed short s,signed short t,signed short r)=init_ov_glTexCoord3s;

void __gl_ml_glTexCoord3s(value v0,value v1,value v2)
{
	signed short l0=Int_val(v0);
	signed short l1=Int_val(v1);
	signed short l2=Int_val(v2);
	ov_glTexCoord3s(l0,l1,l2);
}
extern void (APIENTRY *ov_glTexCoord3i)(signed int s,signed int t,signed int r);

static void APIENTRY init_ov_glTexCoord3i(signed int s,signed int t,signed int r)
{
	void *f;
	f=(void*)getfunc("glTexCoord3i");
	ov_glTexCoord3i=f;
	ov_glTexCoord3i(s,t,r);
}
void (APIENTRY *ov_glTexCoord3i)(signed int s,signed int t,signed int r)=init_ov_glTexCoord3i;

void __gl_ml_glTexCoord3i(value v0,value v1,value v2)
{
	signed int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	ov_glTexCoord3i(l0,l1,l2);
}
extern void (APIENTRY *ov_glTexCoord3f)(float s,float t,float r);

static void APIENTRY init_ov_glTexCoord3f(float s,float t,float r)
{
	void *f;
	f=(void*)getfunc("glTexCoord3f");
	ov_glTexCoord3f=f;
	ov_glTexCoord3f(s,t,r);
}
void (APIENTRY *ov_glTexCoord3f)(float s,float t,float r)=init_ov_glTexCoord3f;

void __gl_ml_glTexCoord3f(value v0,value v1,value v2)
{
	float l0=Double_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	ov_glTexCoord3f(l0,l1,l2);
}
extern void (APIENTRY *ov_glTexCoord3d)(double s,double t,double r);

static void APIENTRY init_ov_glTexCoord3d(double s,double t,double r)
{
	void *f;
	f=(void*)getfunc("glTexCoord3d");
	ov_glTexCoord3d=f;
	ov_glTexCoord3d(s,t,r);
}
void (APIENTRY *ov_glTexCoord3d)(double s,double t,double r)=init_ov_glTexCoord3d;

void __gl_ml_glTexCoord3d(value v0,value v1,value v2)
{
	double l0=Double_val(v0);
	double l1=Double_val(v1);
	double l2=Double_val(v2);
	ov_glTexCoord3d(l0,l1,l2);
}
extern void (APIENTRY *ov_glTexCoord2s)(signed short s,signed short t);

static void APIENTRY init_ov_glTexCoord2s(signed short s,signed short t)
{
	void *f;
	f=(void*)getfunc("glTexCoord2s");
	ov_glTexCoord2s=f;
	ov_glTexCoord2s(s,t);
}
void (APIENTRY *ov_glTexCoord2s)(signed short s,signed short t)=init_ov_glTexCoord2s;

void __gl_ml_glTexCoord2s(value v0,value v1)
{
	signed short l0=Int_val(v0);
	signed short l1=Int_val(v1);
	ov_glTexCoord2s(l0,l1);
}
extern void (APIENTRY *ov_glTexCoord2i)(signed int s,signed int t);

static void APIENTRY init_ov_glTexCoord2i(signed int s,signed int t)
{
	void *f;
	f=(void*)getfunc("glTexCoord2i");
	ov_glTexCoord2i=f;
	ov_glTexCoord2i(s,t);
}
void (APIENTRY *ov_glTexCoord2i)(signed int s,signed int t)=init_ov_glTexCoord2i;

void __gl_ml_glTexCoord2i(value v0,value v1)
{
	signed int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	ov_glTexCoord2i(l0,l1);
}
extern void (APIENTRY *ov_glTexCoord2f)(float s,float t);

static void APIENTRY init_ov_glTexCoord2f(float s,float t)
{
	void *f;
	f=(void*)getfunc("glTexCoord2f");
	ov_glTexCoord2f=f;
	ov_glTexCoord2f(s,t);
}
void (APIENTRY *ov_glTexCoord2f)(float s,float t)=init_ov_glTexCoord2f;

void __gl_ml_glTexCoord2f(value v0,value v1)
{
	float l0=Double_val(v0);
	float l1=Double_val(v1);
	ov_glTexCoord2f(l0,l1);
}
extern void (APIENTRY *ov_glTexCoord2d)(double s,double t);

static void APIENTRY init_ov_glTexCoord2d(double s,double t)
{
	void *f;
	f=(void*)getfunc("glTexCoord2d");
	ov_glTexCoord2d=f;
	ov_glTexCoord2d(s,t);
}
void (APIENTRY *ov_glTexCoord2d)(double s,double t)=init_ov_glTexCoord2d;

void __gl_ml_glTexCoord2d(value v0,value v1)
{
	double l0=Double_val(v0);
	double l1=Double_val(v1);
	ov_glTexCoord2d(l0,l1);
}
extern void (APIENTRY *ov_glTexCoord1s)(signed short s);

static void APIENTRY init_ov_glTexCoord1s(signed short s)
{
	void *f;
	f=(void*)getfunc("glTexCoord1s");
	ov_glTexCoord1s=f;
	ov_glTexCoord1s(s);
}
void (APIENTRY *ov_glTexCoord1s)(signed short s)=init_ov_glTexCoord1s;

void __gl_ml_glTexCoord1s(value v0)
{
	signed short l0=Int_val(v0);
	ov_glTexCoord1s(l0);
}
extern void (APIENTRY *ov_glTexCoord1i)(signed int s);

static void APIENTRY init_ov_glTexCoord1i(signed int s)
{
	void *f;
	f=(void*)getfunc("glTexCoord1i");
	ov_glTexCoord1i=f;
	ov_glTexCoord1i(s);
}
void (APIENTRY *ov_glTexCoord1i)(signed int s)=init_ov_glTexCoord1i;

void __gl_ml_glTexCoord1i(value v0)
{
	signed int l0=Int_val(v0);
	ov_glTexCoord1i(l0);
}
extern void (APIENTRY *ov_glTexCoord1f)(float s);

static void APIENTRY init_ov_glTexCoord1f(float s)
{
	void *f;
	f=(void*)getfunc("glTexCoord1f");
	ov_glTexCoord1f=f;
	ov_glTexCoord1f(s);
}
void (APIENTRY *ov_glTexCoord1f)(float s)=init_ov_glTexCoord1f;

void __gl_ml_glTexCoord1f(value v0)
{
	float l0=Double_val(v0);
	ov_glTexCoord1f(l0);
}
extern void (APIENTRY *ov_glTexCoord1d)(double s);

static void APIENTRY init_ov_glTexCoord1d(double s)
{
	void *f;
	f=(void*)getfunc("glTexCoord1d");
	ov_glTexCoord1d=f;
	ov_glTexCoord1d(s);
}
void (APIENTRY *ov_glTexCoord1d)(double s)=init_ov_glTexCoord1d;

void __gl_ml_glTexCoord1d(value v0)
{
	double l0=Double_val(v0);
	ov_glTexCoord1d(l0);
}
extern void (APIENTRY *ov_glColor4usv)(unsigned short* v);

static void APIENTRY init_ov_glColor4usv(unsigned short* v)
{
	void *f;
	f=(void*)getfunc("glColor4usv");
	ov_glColor4usv=f;
	ov_glColor4usv(v);
}
void (APIENTRY *ov_glColor4usv)(unsigned short* v)=init_ov_glColor4usv;

void __gl_ml_glColor4usv(value v0)
{
	unsigned short* l0=(unsigned short*)(gl__ba__(v0));
	ov_glColor4usv(l0);
}
extern void (APIENTRY *ov_glColor4uiv)(unsigned int* v);

static void APIENTRY init_ov_glColor4uiv(unsigned int* v)
{
	void *f;
	f=(void*)getfunc("glColor4uiv");
	ov_glColor4uiv=f;
	ov_glColor4uiv(v);
}
void (APIENTRY *ov_glColor4uiv)(unsigned int* v)=init_ov_glColor4uiv;

void __gl_ml_glColor4uiv(value v0)
{
	unsigned int* l0=(unsigned int*)(gl__ba__(v0));
	ov_glColor4uiv(l0);
}
extern void (APIENTRY *ov_glColor4ubv)(unsigned char* v);

static void APIENTRY init_ov_glColor4ubv(unsigned char* v)
{
	void *f;
	f=(void*)getfunc("glColor4ubv");
	ov_glColor4ubv=f;
	ov_glColor4ubv(v);
}
void (APIENTRY *ov_glColor4ubv)(unsigned char* v)=init_ov_glColor4ubv;

void __gl_ml_glColor4ubv(value v0)
{
	unsigned char* l0=(unsigned char*)(gl__ba__(v0));
	ov_glColor4ubv(l0);
}
extern void (APIENTRY *ov_glColor4sv)(signed short* v);

static void APIENTRY init_ov_glColor4sv(signed short* v)
{
	void *f;
	f=(void*)getfunc("glColor4sv");
	ov_glColor4sv=f;
	ov_glColor4sv(v);
}
void (APIENTRY *ov_glColor4sv)(signed short* v)=init_ov_glColor4sv;

void __gl_ml_glColor4sv(value v0)
{
	signed short* l0=(signed short*)(gl__ba__(v0));
	ov_glColor4sv(l0);
}
extern void (APIENTRY *ov_glColor4iv)(signed int* v);

static void APIENTRY init_ov_glColor4iv(signed int* v)
{
	void *f;
	f=(void*)getfunc("glColor4iv");
	ov_glColor4iv=f;
	ov_glColor4iv(v);
}
void (APIENTRY *ov_glColor4iv)(signed int* v)=init_ov_glColor4iv;

void __gl_ml_glColor4iv(value v0)
{
	signed int* l0=(signed int*)(gl__ba__(v0));
	ov_glColor4iv(l0);
}
extern void (APIENTRY *ov_glColor4fv)(float* v);

static void APIENTRY init_ov_glColor4fv(float* v)
{
	void *f;
	f=(void*)getfunc("glColor4fv");
	ov_glColor4fv=f;
	ov_glColor4fv(v);
}
void (APIENTRY *ov_glColor4fv)(float* v)=init_ov_glColor4fv;

void __gl_ml_glColor4fv(value v0)
{
	float* l0=(float*)(gl__ba__(v0));
	ov_glColor4fv(l0);
}
extern void (APIENTRY *ov_glColor4dv)(double* v);

static void APIENTRY init_ov_glColor4dv(double* v)
{
	void *f;
	f=(void*)getfunc("glColor4dv");
	ov_glColor4dv=f;
	ov_glColor4dv(v);
}
void (APIENTRY *ov_glColor4dv)(double* v)=init_ov_glColor4dv;

void __gl_ml_glColor4dv(value v0)
{
	double* l0=(double*)(gl__ba__(v0));
	ov_glColor4dv(l0);
}
extern void (APIENTRY *ov_glColor4bv)(signed char* v);

static void APIENTRY init_ov_glColor4bv(signed char* v)
{
	void *f;
	f=(void*)getfunc("glColor4bv");
	ov_glColor4bv=f;
	ov_glColor4bv(v);
}
void (APIENTRY *ov_glColor4bv)(signed char* v)=init_ov_glColor4bv;

void __gl_ml_glColor4bv(value v0)
{
	signed char* l0=(signed char*)(gl__ba__(v0));
	ov_glColor4bv(l0);
}
extern void (APIENTRY *ov_glColor3usv)(unsigned short* v);

static void APIENTRY init_ov_glColor3usv(unsigned short* v)
{
	void *f;
	f=(void*)getfunc("glColor3usv");
	ov_glColor3usv=f;
	ov_glColor3usv(v);
}
void (APIENTRY *ov_glColor3usv)(unsigned short* v)=init_ov_glColor3usv;

void __gl_ml_glColor3usv(value v0)
{
	unsigned short* l0=(unsigned short*)(gl__ba__(v0));
	ov_glColor3usv(l0);
}
extern void (APIENTRY *ov_glColor3uiv)(unsigned int* v);

static void APIENTRY init_ov_glColor3uiv(unsigned int* v)
{
	void *f;
	f=(void*)getfunc("glColor3uiv");
	ov_glColor3uiv=f;
	ov_glColor3uiv(v);
}
void (APIENTRY *ov_glColor3uiv)(unsigned int* v)=init_ov_glColor3uiv;

void __gl_ml_glColor3uiv(value v0)
{
	unsigned int* l0=(unsigned int*)(gl__ba__(v0));
	ov_glColor3uiv(l0);
}
extern void (APIENTRY *ov_glColor3ubv)(unsigned char* v);

static void APIENTRY init_ov_glColor3ubv(unsigned char* v)
{
	void *f;
	f=(void*)getfunc("glColor3ubv");
	ov_glColor3ubv=f;
	ov_glColor3ubv(v);
}
void (APIENTRY *ov_glColor3ubv)(unsigned char* v)=init_ov_glColor3ubv;

void __gl_ml_glColor3ubv(value v0)
{
	unsigned char* l0=(unsigned char*)(gl__ba__(v0));
	ov_glColor3ubv(l0);
}
extern void (APIENTRY *ov_glColor3sv)(signed short* v);

static void APIENTRY init_ov_glColor3sv(signed short* v)
{
	void *f;
	f=(void*)getfunc("glColor3sv");
	ov_glColor3sv=f;
	ov_glColor3sv(v);
}
void (APIENTRY *ov_glColor3sv)(signed short* v)=init_ov_glColor3sv;

void __gl_ml_glColor3sv(value v0)
{
	signed short* l0=(signed short*)(gl__ba__(v0));
	ov_glColor3sv(l0);
}
extern void (APIENTRY *ov_glColor3iv)(signed int* v);

static void APIENTRY init_ov_glColor3iv(signed int* v)
{
	void *f;
	f=(void*)getfunc("glColor3iv");
	ov_glColor3iv=f;
	ov_glColor3iv(v);
}
void (APIENTRY *ov_glColor3iv)(signed int* v)=init_ov_glColor3iv;

void __gl_ml_glColor3iv(value v0)
{
	signed int* l0=(signed int*)(gl__ba__(v0));
	ov_glColor3iv(l0);
}
extern void (APIENTRY *ov_glColor3fv)(float* v);

static void APIENTRY init_ov_glColor3fv(float* v)
{
	void *f;
	f=(void*)getfunc("glColor3fv");
	ov_glColor3fv=f;
	ov_glColor3fv(v);
}
void (APIENTRY *ov_glColor3fv)(float* v)=init_ov_glColor3fv;

void __gl_ml_glColor3fv(value v0)
{
	float* l0=(float*)(gl__ba__(v0));
	ov_glColor3fv(l0);
}
extern void (APIENTRY *ov_glColor3dv)(double* v);

static void APIENTRY init_ov_glColor3dv(double* v)
{
	void *f;
	f=(void*)getfunc("glColor3dv");
	ov_glColor3dv=f;
	ov_glColor3dv(v);
}
void (APIENTRY *ov_glColor3dv)(double* v)=init_ov_glColor3dv;

void __gl_ml_glColor3dv(value v0)
{
	double* l0=(double*)(gl__ba__(v0));
	ov_glColor3dv(l0);
}
extern void (APIENTRY *ov_glColor3bv)(signed char* v);

static void APIENTRY init_ov_glColor3bv(signed char* v)
{
	void *f;
	f=(void*)getfunc("glColor3bv");
	ov_glColor3bv=f;
	ov_glColor3bv(v);
}
void (APIENTRY *ov_glColor3bv)(signed char* v)=init_ov_glColor3bv;

void __gl_ml_glColor3bv(value v0)
{
	signed char* l0=(signed char*)(gl__ba__(v0));
	ov_glColor3bv(l0);
}
extern void (APIENTRY *ov_glColor4us)(unsigned short red,unsigned short green,unsigned short blue,unsigned short alpha);

static void APIENTRY init_ov_glColor4us(unsigned short red,unsigned short green,unsigned short blue,unsigned short alpha)
{
	void *f;
	f=(void*)getfunc("glColor4us");
	ov_glColor4us=f;
	ov_glColor4us(red,green,blue,alpha);
}
void (APIENTRY *ov_glColor4us)(unsigned short red,unsigned short green,unsigned short blue,unsigned short alpha)=init_ov_glColor4us;

void __gl_ml_glColor4us(value v0,value v1,value v2,value v3)
{
	unsigned short l0=Int_val(v0);
	unsigned short l1=Int_val(v1);
	unsigned short l2=Int_val(v2);
	unsigned short l3=Int_val(v3);
	ov_glColor4us(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glColor4ui)(unsigned int red,unsigned int green,unsigned int blue,unsigned int alpha);

static void APIENTRY init_ov_glColor4ui(unsigned int red,unsigned int green,unsigned int blue,unsigned int alpha)
{
	void *f;
	f=(void*)getfunc("glColor4ui");
	ov_glColor4ui=f;
	ov_glColor4ui(red,green,blue,alpha);
}
void (APIENTRY *ov_glColor4ui)(unsigned int red,unsigned int green,unsigned int blue,unsigned int alpha)=init_ov_glColor4ui;

void __gl_ml_glColor4ui(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	unsigned int l3=Int_val(v3);
	ov_glColor4ui(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glColor4ub)(unsigned char red,unsigned char green,unsigned char blue,unsigned char alpha);

static void APIENTRY init_ov_glColor4ub(unsigned char red,unsigned char green,unsigned char blue,unsigned char alpha)
{
	void *f;
	f=(void*)getfunc("glColor4ub");
	ov_glColor4ub=f;
	ov_glColor4ub(red,green,blue,alpha);
}
void (APIENTRY *ov_glColor4ub)(unsigned char red,unsigned char green,unsigned char blue,unsigned char alpha)=init_ov_glColor4ub;

void __gl_ml_glColor4ub(value v0,value v1,value v2,value v3)
{
	unsigned char l0=Int_val(v0);
	unsigned char l1=Int_val(v1);
	unsigned char l2=Int_val(v2);
	unsigned char l3=Int_val(v3);
	ov_glColor4ub(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glColor4s)(signed short red,signed short green,signed short blue,signed short alpha);

static void APIENTRY init_ov_glColor4s(signed short red,signed short green,signed short blue,signed short alpha)
{
	void *f;
	f=(void*)getfunc("glColor4s");
	ov_glColor4s=f;
	ov_glColor4s(red,green,blue,alpha);
}
void (APIENTRY *ov_glColor4s)(signed short red,signed short green,signed short blue,signed short alpha)=init_ov_glColor4s;

void __gl_ml_glColor4s(value v0,value v1,value v2,value v3)
{
	signed short l0=Int_val(v0);
	signed short l1=Int_val(v1);
	signed short l2=Int_val(v2);
	signed short l3=Int_val(v3);
	ov_glColor4s(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glColor4i)(signed int red,signed int green,signed int blue,signed int alpha);

static void APIENTRY init_ov_glColor4i(signed int red,signed int green,signed int blue,signed int alpha)
{
	void *f;
	f=(void*)getfunc("glColor4i");
	ov_glColor4i=f;
	ov_glColor4i(red,green,blue,alpha);
}
void (APIENTRY *ov_glColor4i)(signed int red,signed int green,signed int blue,signed int alpha)=init_ov_glColor4i;

void __gl_ml_glColor4i(value v0,value v1,value v2,value v3)
{
	signed int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	ov_glColor4i(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glColor4f)(float red,float green,float blue,float alpha);

static void APIENTRY init_ov_glColor4f(float red,float green,float blue,float alpha)
{
	void *f;
	f=(void*)getfunc("glColor4f");
	ov_glColor4f=f;
	ov_glColor4f(red,green,blue,alpha);
}
void (APIENTRY *ov_glColor4f)(float red,float green,float blue,float alpha)=init_ov_glColor4f;

void __gl_ml_glColor4f(value v0,value v1,value v2,value v3)
{
	float l0=Double_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	float l3=Double_val(v3);
	ov_glColor4f(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glColor4d)(double red,double green,double blue,double alpha);

static void APIENTRY init_ov_glColor4d(double red,double green,double blue,double alpha)
{
	void *f;
	f=(void*)getfunc("glColor4d");
	ov_glColor4d=f;
	ov_glColor4d(red,green,blue,alpha);
}
void (APIENTRY *ov_glColor4d)(double red,double green,double blue,double alpha)=init_ov_glColor4d;

void __gl_ml_glColor4d(value v0,value v1,value v2,value v3)
{
	double l0=Double_val(v0);
	double l1=Double_val(v1);
	double l2=Double_val(v2);
	double l3=Double_val(v3);
	ov_glColor4d(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glColor4b)(signed char red,signed char green,signed char blue,signed char alpha);

static void APIENTRY init_ov_glColor4b(signed char red,signed char green,signed char blue,signed char alpha)
{
	void *f;
	f=(void*)getfunc("glColor4b");
	ov_glColor4b=f;
	ov_glColor4b(red,green,blue,alpha);
}
void (APIENTRY *ov_glColor4b)(signed char red,signed char green,signed char blue,signed char alpha)=init_ov_glColor4b;

void __gl_ml_glColor4b(value v0,value v1,value v2,value v3)
{
	signed char l0=Int_val(v0);
	signed char l1=Int_val(v1);
	signed char l2=Int_val(v2);
	signed char l3=Int_val(v3);
	ov_glColor4b(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glColor3us)(unsigned short red,unsigned short green,unsigned short blue);

static void APIENTRY init_ov_glColor3us(unsigned short red,unsigned short green,unsigned short blue)
{
	void *f;
	f=(void*)getfunc("glColor3us");
	ov_glColor3us=f;
	ov_glColor3us(red,green,blue);
}
void (APIENTRY *ov_glColor3us)(unsigned short red,unsigned short green,unsigned short blue)=init_ov_glColor3us;

void __gl_ml_glColor3us(value v0,value v1,value v2)
{
	unsigned short l0=Int_val(v0);
	unsigned short l1=Int_val(v1);
	unsigned short l2=Int_val(v2);
	ov_glColor3us(l0,l1,l2);
}
extern void (APIENTRY *ov_glColor3ui)(unsigned int red,unsigned int green,unsigned int blue);

static void APIENTRY init_ov_glColor3ui(unsigned int red,unsigned int green,unsigned int blue)
{
	void *f;
	f=(void*)getfunc("glColor3ui");
	ov_glColor3ui=f;
	ov_glColor3ui(red,green,blue);
}
void (APIENTRY *ov_glColor3ui)(unsigned int red,unsigned int green,unsigned int blue)=init_ov_glColor3ui;

void __gl_ml_glColor3ui(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	ov_glColor3ui(l0,l1,l2);
}
extern void (APIENTRY *ov_glColor3ub)(unsigned char red,unsigned char green,unsigned char blue);

static void APIENTRY init_ov_glColor3ub(unsigned char red,unsigned char green,unsigned char blue)
{
	void *f;
	f=(void*)getfunc("glColor3ub");
	ov_glColor3ub=f;
	ov_glColor3ub(red,green,blue);
}
void (APIENTRY *ov_glColor3ub)(unsigned char red,unsigned char green,unsigned char blue)=init_ov_glColor3ub;

void __gl_ml_glColor3ub(value v0,value v1,value v2)
{
	unsigned char l0=Int_val(v0);
	unsigned char l1=Int_val(v1);
	unsigned char l2=Int_val(v2);
	ov_glColor3ub(l0,l1,l2);
}
extern void (APIENTRY *ov_glColor3s)(signed short red,signed short green,signed short blue);

static void APIENTRY init_ov_glColor3s(signed short red,signed short green,signed short blue)
{
	void *f;
	f=(void*)getfunc("glColor3s");
	ov_glColor3s=f;
	ov_glColor3s(red,green,blue);
}
void (APIENTRY *ov_glColor3s)(signed short red,signed short green,signed short blue)=init_ov_glColor3s;

void __gl_ml_glColor3s(value v0,value v1,value v2)
{
	signed short l0=Int_val(v0);
	signed short l1=Int_val(v1);
	signed short l2=Int_val(v2);
	ov_glColor3s(l0,l1,l2);
}
extern void (APIENTRY *ov_glColor3i)(signed int red,signed int green,signed int blue);

static void APIENTRY init_ov_glColor3i(signed int red,signed int green,signed int blue)
{
	void *f;
	f=(void*)getfunc("glColor3i");
	ov_glColor3i=f;
	ov_glColor3i(red,green,blue);
}
void (APIENTRY *ov_glColor3i)(signed int red,signed int green,signed int blue)=init_ov_glColor3i;

void __gl_ml_glColor3i(value v0,value v1,value v2)
{
	signed int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	ov_glColor3i(l0,l1,l2);
}
extern void (APIENTRY *ov_glColor3f)(float red,float green,float blue);

static void APIENTRY init_ov_glColor3f(float red,float green,float blue)
{
	void *f;
	f=(void*)getfunc("glColor3f");
	ov_glColor3f=f;
	ov_glColor3f(red,green,blue);
}
void (APIENTRY *ov_glColor3f)(float red,float green,float blue)=init_ov_glColor3f;

void __gl_ml_glColor3f(value v0,value v1,value v2)
{
	float l0=Double_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	ov_glColor3f(l0,l1,l2);
}
extern void (APIENTRY *ov_glColor3d)(double red,double green,double blue);

static void APIENTRY init_ov_glColor3d(double red,double green,double blue)
{
	void *f;
	f=(void*)getfunc("glColor3d");
	ov_glColor3d=f;
	ov_glColor3d(red,green,blue);
}
void (APIENTRY *ov_glColor3d)(double red,double green,double blue)=init_ov_glColor3d;

void __gl_ml_glColor3d(value v0,value v1,value v2)
{
	double l0=Double_val(v0);
	double l1=Double_val(v1);
	double l2=Double_val(v2);
	ov_glColor3d(l0,l1,l2);
}
extern void (APIENTRY *ov_glColor3b)(signed char red,signed char green,signed char blue);

static void APIENTRY init_ov_glColor3b(signed char red,signed char green,signed char blue)
{
	void *f;
	f=(void*)getfunc("glColor3b");
	ov_glColor3b=f;
	ov_glColor3b(red,green,blue);
}
void (APIENTRY *ov_glColor3b)(signed char red,signed char green,signed char blue)=init_ov_glColor3b;

void __gl_ml_glColor3b(value v0,value v1,value v2)
{
	signed char l0=Int_val(v0);
	signed char l1=Int_val(v1);
	signed char l2=Int_val(v2);
	ov_glColor3b(l0,l1,l2);
}
extern void (APIENTRY *ov_glIndexsv)(signed short* c);

static void APIENTRY init_ov_glIndexsv(signed short* c)
{
	void *f;
	f=(void*)getfunc("glIndexsv");
	ov_glIndexsv=f;
	ov_glIndexsv(c);
}
void (APIENTRY *ov_glIndexsv)(signed short* c)=init_ov_glIndexsv;

void __gl_ml_glIndexsv(value v0)
{
	signed short* l0=(signed short*)(gl__ba__(v0));
	ov_glIndexsv(l0);
}
extern void (APIENTRY *ov_glIndexiv)(signed int* c);

static void APIENTRY init_ov_glIndexiv(signed int* c)
{
	void *f;
	f=(void*)getfunc("glIndexiv");
	ov_glIndexiv=f;
	ov_glIndexiv(c);
}
void (APIENTRY *ov_glIndexiv)(signed int* c)=init_ov_glIndexiv;

void __gl_ml_glIndexiv(value v0)
{
	signed int* l0=(signed int*)(gl__ba__(v0));
	ov_glIndexiv(l0);
}
extern void (APIENTRY *ov_glIndexfv)(float* c);

static void APIENTRY init_ov_glIndexfv(float* c)
{
	void *f;
	f=(void*)getfunc("glIndexfv");
	ov_glIndexfv=f;
	ov_glIndexfv(c);
}
void (APIENTRY *ov_glIndexfv)(float* c)=init_ov_glIndexfv;

void __gl_ml_glIndexfv(value v0)
{
	float* l0=(float*)(gl__ba__(v0));
	ov_glIndexfv(l0);
}
extern void (APIENTRY *ov_glIndexdv)(double* c);

static void APIENTRY init_ov_glIndexdv(double* c)
{
	void *f;
	f=(void*)getfunc("glIndexdv");
	ov_glIndexdv=f;
	ov_glIndexdv(c);
}
void (APIENTRY *ov_glIndexdv)(double* c)=init_ov_glIndexdv;

void __gl_ml_glIndexdv(value v0)
{
	double* l0=(double*)(gl__ba__(v0));
	ov_glIndexdv(l0);
}
extern void (APIENTRY *ov_glIndexs)(signed short c);

static void APIENTRY init_ov_glIndexs(signed short c)
{
	void *f;
	f=(void*)getfunc("glIndexs");
	ov_glIndexs=f;
	ov_glIndexs(c);
}
void (APIENTRY *ov_glIndexs)(signed short c)=init_ov_glIndexs;

void __gl_ml_glIndexs(value v0)
{
	signed short l0=Int_val(v0);
	ov_glIndexs(l0);
}
extern void (APIENTRY *ov_glIndexi)(signed int c);

static void APIENTRY init_ov_glIndexi(signed int c)
{
	void *f;
	f=(void*)getfunc("glIndexi");
	ov_glIndexi=f;
	ov_glIndexi(c);
}
void (APIENTRY *ov_glIndexi)(signed int c)=init_ov_glIndexi;

void __gl_ml_glIndexi(value v0)
{
	signed int l0=Int_val(v0);
	ov_glIndexi(l0);
}
extern void (APIENTRY *ov_glIndexf)(float c);

static void APIENTRY init_ov_glIndexf(float c)
{
	void *f;
	f=(void*)getfunc("glIndexf");
	ov_glIndexf=f;
	ov_glIndexf(c);
}
void (APIENTRY *ov_glIndexf)(float c)=init_ov_glIndexf;

void __gl_ml_glIndexf(value v0)
{
	float l0=Double_val(v0);
	ov_glIndexf(l0);
}
extern void (APIENTRY *ov_glIndexd)(double c);

static void APIENTRY init_ov_glIndexd(double c)
{
	void *f;
	f=(void*)getfunc("glIndexd");
	ov_glIndexd=f;
	ov_glIndexd(c);
}
void (APIENTRY *ov_glIndexd)(double c)=init_ov_glIndexd;

void __gl_ml_glIndexd(value v0)
{
	double l0=Double_val(v0);
	ov_glIndexd(l0);
}
extern void (APIENTRY *ov_glNormal3sv)(signed short* v);

static void APIENTRY init_ov_glNormal3sv(signed short* v)
{
	void *f;
	f=(void*)getfunc("glNormal3sv");
	ov_glNormal3sv=f;
	ov_glNormal3sv(v);
}
void (APIENTRY *ov_glNormal3sv)(signed short* v)=init_ov_glNormal3sv;

void __gl_ml_glNormal3sv(value v0)
{
	signed short* l0=(signed short*)(gl__ba__(v0));
	ov_glNormal3sv(l0);
}
extern void (APIENTRY *ov_glNormal3iv)(signed int* v);

static void APIENTRY init_ov_glNormal3iv(signed int* v)
{
	void *f;
	f=(void*)getfunc("glNormal3iv");
	ov_glNormal3iv=f;
	ov_glNormal3iv(v);
}
void (APIENTRY *ov_glNormal3iv)(signed int* v)=init_ov_glNormal3iv;

void __gl_ml_glNormal3iv(value v0)
{
	signed int* l0=(signed int*)(gl__ba__(v0));
	ov_glNormal3iv(l0);
}
extern void (APIENTRY *ov_glNormal3fv)(float* v);

static void APIENTRY init_ov_glNormal3fv(float* v)
{
	void *f;
	f=(void*)getfunc("glNormal3fv");
	ov_glNormal3fv=f;
	ov_glNormal3fv(v);
}
void (APIENTRY *ov_glNormal3fv)(float* v)=init_ov_glNormal3fv;

void __gl_ml_glNormal3fv(value v0)
{
	float* l0=(float*)(gl__ba__(v0));
	ov_glNormal3fv(l0);
}
extern void (APIENTRY *ov_glNormal3dv)(double* v);

static void APIENTRY init_ov_glNormal3dv(double* v)
{
	void *f;
	f=(void*)getfunc("glNormal3dv");
	ov_glNormal3dv=f;
	ov_glNormal3dv(v);
}
void (APIENTRY *ov_glNormal3dv)(double* v)=init_ov_glNormal3dv;

void __gl_ml_glNormal3dv(value v0)
{
	double* l0=(double*)(gl__ba__(v0));
	ov_glNormal3dv(l0);
}
extern void (APIENTRY *ov_glNormal3bv)(signed char* v);

static void APIENTRY init_ov_glNormal3bv(signed char* v)
{
	void *f;
	f=(void*)getfunc("glNormal3bv");
	ov_glNormal3bv=f;
	ov_glNormal3bv(v);
}
void (APIENTRY *ov_glNormal3bv)(signed char* v)=init_ov_glNormal3bv;

void __gl_ml_glNormal3bv(value v0)
{
	signed char* l0=(signed char*)(gl__ba__(v0));
	ov_glNormal3bv(l0);
}
extern void (APIENTRY *ov_glNormal3s)(signed short nx,signed short ny,signed short nz);

static void APIENTRY init_ov_glNormal3s(signed short nx,signed short ny,signed short nz)
{
	void *f;
	f=(void*)getfunc("glNormal3s");
	ov_glNormal3s=f;
	ov_glNormal3s(nx,ny,nz);
}
void (APIENTRY *ov_glNormal3s)(signed short nx,signed short ny,signed short nz)=init_ov_glNormal3s;

void __gl_ml_glNormal3s(value v0,value v1,value v2)
{
	signed short l0=Int_val(v0);
	signed short l1=Int_val(v1);
	signed short l2=Int_val(v2);
	ov_glNormal3s(l0,l1,l2);
}
extern void (APIENTRY *ov_glNormal3i)(signed int nx,signed int ny,signed int nz);

static void APIENTRY init_ov_glNormal3i(signed int nx,signed int ny,signed int nz)
{
	void *f;
	f=(void*)getfunc("glNormal3i");
	ov_glNormal3i=f;
	ov_glNormal3i(nx,ny,nz);
}
void (APIENTRY *ov_glNormal3i)(signed int nx,signed int ny,signed int nz)=init_ov_glNormal3i;

void __gl_ml_glNormal3i(value v0,value v1,value v2)
{
	signed int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	ov_glNormal3i(l0,l1,l2);
}
extern void (APIENTRY *ov_glNormal3f)(float nx,float ny,float nz);

static void APIENTRY init_ov_glNormal3f(float nx,float ny,float nz)
{
	void *f;
	f=(void*)getfunc("glNormal3f");
	ov_glNormal3f=f;
	ov_glNormal3f(nx,ny,nz);
}
void (APIENTRY *ov_glNormal3f)(float nx,float ny,float nz)=init_ov_glNormal3f;

void __gl_ml_glNormal3f(value v0,value v1,value v2)
{
	float l0=Double_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	ov_glNormal3f(l0,l1,l2);
}
extern void (APIENTRY *ov_glNormal3d)(double nx,double ny,double nz);

static void APIENTRY init_ov_glNormal3d(double nx,double ny,double nz)
{
	void *f;
	f=(void*)getfunc("glNormal3d");
	ov_glNormal3d=f;
	ov_glNormal3d(nx,ny,nz);
}
void (APIENTRY *ov_glNormal3d)(double nx,double ny,double nz)=init_ov_glNormal3d;

void __gl_ml_glNormal3d(value v0,value v1,value v2)
{
	double l0=Double_val(v0);
	double l1=Double_val(v1);
	double l2=Double_val(v2);
	ov_glNormal3d(l0,l1,l2);
}
extern void (APIENTRY *ov_glNormal3b)(signed char nx,signed char ny,signed char nz);

static void APIENTRY init_ov_glNormal3b(signed char nx,signed char ny,signed char nz)
{
	void *f;
	f=(void*)getfunc("glNormal3b");
	ov_glNormal3b=f;
	ov_glNormal3b(nx,ny,nz);
}
void (APIENTRY *ov_glNormal3b)(signed char nx,signed char ny,signed char nz)=init_ov_glNormal3b;

void __gl_ml_glNormal3b(value v0,value v1,value v2)
{
	signed char l0=Int_val(v0);
	signed char l1=Int_val(v1);
	signed char l2=Int_val(v2);
	ov_glNormal3b(l0,l1,l2);
}
extern void (APIENTRY *ov_glVertex4sv)(signed short* v);

static void APIENTRY init_ov_glVertex4sv(signed short* v)
{
	void *f;
	f=(void*)getfunc("glVertex4sv");
	ov_glVertex4sv=f;
	ov_glVertex4sv(v);
}
void (APIENTRY *ov_glVertex4sv)(signed short* v)=init_ov_glVertex4sv;

void __gl_ml_glVertex4sv(value v0)
{
	signed short* l0=(signed short*)(gl__ba__(v0));
	ov_glVertex4sv(l0);
}
extern void (APIENTRY *ov_glVertex4iv)(signed int* v);

static void APIENTRY init_ov_glVertex4iv(signed int* v)
{
	void *f;
	f=(void*)getfunc("glVertex4iv");
	ov_glVertex4iv=f;
	ov_glVertex4iv(v);
}
void (APIENTRY *ov_glVertex4iv)(signed int* v)=init_ov_glVertex4iv;

void __gl_ml_glVertex4iv(value v0)
{
	signed int* l0=(signed int*)(gl__ba__(v0));
	ov_glVertex4iv(l0);
}
extern void (APIENTRY *ov_glVertex4fv)(float* v);

static void APIENTRY init_ov_glVertex4fv(float* v)
{
	void *f;
	f=(void*)getfunc("glVertex4fv");
	ov_glVertex4fv=f;
	ov_glVertex4fv(v);
}
void (APIENTRY *ov_glVertex4fv)(float* v)=init_ov_glVertex4fv;

void __gl_ml_glVertex4fv(value v0)
{
	float* l0=(float*)(gl__ba__(v0));
	ov_glVertex4fv(l0);
}
extern void (APIENTRY *ov_glVertex4dv)(double* v);

static void APIENTRY init_ov_glVertex4dv(double* v)
{
	void *f;
	f=(void*)getfunc("glVertex4dv");
	ov_glVertex4dv=f;
	ov_glVertex4dv(v);
}
void (APIENTRY *ov_glVertex4dv)(double* v)=init_ov_glVertex4dv;

void __gl_ml_glVertex4dv(value v0)
{
	double* l0=(double*)(gl__ba__(v0));
	ov_glVertex4dv(l0);
}
extern void (APIENTRY *ov_glVertex3sv)(signed short* v);

static void APIENTRY init_ov_glVertex3sv(signed short* v)
{
	void *f;
	f=(void*)getfunc("glVertex3sv");
	ov_glVertex3sv=f;
	ov_glVertex3sv(v);
}
void (APIENTRY *ov_glVertex3sv)(signed short* v)=init_ov_glVertex3sv;

void __gl_ml_glVertex3sv(value v0)
{
	signed short* l0=(signed short*)(gl__ba__(v0));
	ov_glVertex3sv(l0);
}
extern void (APIENTRY *ov_glVertex3iv)(signed int* v);

static void APIENTRY init_ov_glVertex3iv(signed int* v)
{
	void *f;
	f=(void*)getfunc("glVertex3iv");
	ov_glVertex3iv=f;
	ov_glVertex3iv(v);
}
void (APIENTRY *ov_glVertex3iv)(signed int* v)=init_ov_glVertex3iv;

void __gl_ml_glVertex3iv(value v0)
{
	signed int* l0=(signed int*)(gl__ba__(v0));
	ov_glVertex3iv(l0);
}
extern void (APIENTRY *ov_glVertex3fv)(float* v);

static void APIENTRY init_ov_glVertex3fv(float* v)
{
	void *f;
	f=(void*)getfunc("glVertex3fv");
	ov_glVertex3fv=f;
	ov_glVertex3fv(v);
}
void (APIENTRY *ov_glVertex3fv)(float* v)=init_ov_glVertex3fv;

void __gl_ml_glVertex3fv(value v0)
{
	float* l0=(float*)(gl__ba__(v0));
	ov_glVertex3fv(l0);
}
extern void (APIENTRY *ov_glVertex3dv)(double* v);

static void APIENTRY init_ov_glVertex3dv(double* v)
{
	void *f;
	f=(void*)getfunc("glVertex3dv");
	ov_glVertex3dv=f;
	ov_glVertex3dv(v);
}
void (APIENTRY *ov_glVertex3dv)(double* v)=init_ov_glVertex3dv;

void __gl_ml_glVertex3dv(value v0)
{
	double* l0=(double*)(gl__ba__(v0));
	ov_glVertex3dv(l0);
}
extern void (APIENTRY *ov_glVertex2sv)(signed short* v);

static void APIENTRY init_ov_glVertex2sv(signed short* v)
{
	void *f;
	f=(void*)getfunc("glVertex2sv");
	ov_glVertex2sv=f;
	ov_glVertex2sv(v);
}
void (APIENTRY *ov_glVertex2sv)(signed short* v)=init_ov_glVertex2sv;

void __gl_ml_glVertex2sv(value v0)
{
	signed short* l0=(signed short*)(gl__ba__(v0));
	ov_glVertex2sv(l0);
}
extern void (APIENTRY *ov_glVertex2iv)(signed int* v);

static void APIENTRY init_ov_glVertex2iv(signed int* v)
{
	void *f;
	f=(void*)getfunc("glVertex2iv");
	ov_glVertex2iv=f;
	ov_glVertex2iv(v);
}
void (APIENTRY *ov_glVertex2iv)(signed int* v)=init_ov_glVertex2iv;

void __gl_ml_glVertex2iv(value v0)
{
	signed int* l0=(signed int*)(gl__ba__(v0));
	ov_glVertex2iv(l0);
}
extern void (APIENTRY *ov_glVertex2fv)(float* v);

static void APIENTRY init_ov_glVertex2fv(float* v)
{
	void *f;
	f=(void*)getfunc("glVertex2fv");
	ov_glVertex2fv=f;
	ov_glVertex2fv(v);
}
void (APIENTRY *ov_glVertex2fv)(float* v)=init_ov_glVertex2fv;

void __gl_ml_glVertex2fv(value v0)
{
	float* l0=(float*)(gl__ba__(v0));
	ov_glVertex2fv(l0);
}
extern void (APIENTRY *ov_glVertex2dv)(double* v);

static void APIENTRY init_ov_glVertex2dv(double* v)
{
	void *f;
	f=(void*)getfunc("glVertex2dv");
	ov_glVertex2dv=f;
	ov_glVertex2dv(v);
}
void (APIENTRY *ov_glVertex2dv)(double* v)=init_ov_glVertex2dv;

void __gl_ml_glVertex2dv(value v0)
{
	double* l0=(double*)(gl__ba__(v0));
	ov_glVertex2dv(l0);
}
extern void (APIENTRY *ov_glVertex4s)(signed short x,signed short y,signed short z,signed short w);

static void APIENTRY init_ov_glVertex4s(signed short x,signed short y,signed short z,signed short w)
{
	void *f;
	f=(void*)getfunc("glVertex4s");
	ov_glVertex4s=f;
	ov_glVertex4s(x,y,z,w);
}
void (APIENTRY *ov_glVertex4s)(signed short x,signed short y,signed short z,signed short w)=init_ov_glVertex4s;

void __gl_ml_glVertex4s(value v0,value v1,value v2,value v3)
{
	signed short l0=Int_val(v0);
	signed short l1=Int_val(v1);
	signed short l2=Int_val(v2);
	signed short l3=Int_val(v3);
	ov_glVertex4s(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glVertex4i)(signed int x,signed int y,signed int z,signed int w);

static void APIENTRY init_ov_glVertex4i(signed int x,signed int y,signed int z,signed int w)
{
	void *f;
	f=(void*)getfunc("glVertex4i");
	ov_glVertex4i=f;
	ov_glVertex4i(x,y,z,w);
}
void (APIENTRY *ov_glVertex4i)(signed int x,signed int y,signed int z,signed int w)=init_ov_glVertex4i;

void __gl_ml_glVertex4i(value v0,value v1,value v2,value v3)
{
	signed int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	ov_glVertex4i(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glVertex4f)(float x,float y,float z,float w);

static void APIENTRY init_ov_glVertex4f(float x,float y,float z,float w)
{
	void *f;
	f=(void*)getfunc("glVertex4f");
	ov_glVertex4f=f;
	ov_glVertex4f(x,y,z,w);
}
void (APIENTRY *ov_glVertex4f)(float x,float y,float z,float w)=init_ov_glVertex4f;

void __gl_ml_glVertex4f(value v0,value v1,value v2,value v3)
{
	float l0=Double_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	float l3=Double_val(v3);
	ov_glVertex4f(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glVertex4d)(double x,double y,double z,double w);

static void APIENTRY init_ov_glVertex4d(double x,double y,double z,double w)
{
	void *f;
	f=(void*)getfunc("glVertex4d");
	ov_glVertex4d=f;
	ov_glVertex4d(x,y,z,w);
}
void (APIENTRY *ov_glVertex4d)(double x,double y,double z,double w)=init_ov_glVertex4d;

void __gl_ml_glVertex4d(value v0,value v1,value v2,value v3)
{
	double l0=Double_val(v0);
	double l1=Double_val(v1);
	double l2=Double_val(v2);
	double l3=Double_val(v3);
	ov_glVertex4d(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glVertex3s)(signed short x,signed short y,signed short z);

static void APIENTRY init_ov_glVertex3s(signed short x,signed short y,signed short z)
{
	void *f;
	f=(void*)getfunc("glVertex3s");
	ov_glVertex3s=f;
	ov_glVertex3s(x,y,z);
}
void (APIENTRY *ov_glVertex3s)(signed short x,signed short y,signed short z)=init_ov_glVertex3s;

void __gl_ml_glVertex3s(value v0,value v1,value v2)
{
	signed short l0=Int_val(v0);
	signed short l1=Int_val(v1);
	signed short l2=Int_val(v2);
	ov_glVertex3s(l0,l1,l2);
}
extern void (APIENTRY *ov_glVertex3i)(signed int x,signed int y,signed int z);

static void APIENTRY init_ov_glVertex3i(signed int x,signed int y,signed int z)
{
	void *f;
	f=(void*)getfunc("glVertex3i");
	ov_glVertex3i=f;
	ov_glVertex3i(x,y,z);
}
void (APIENTRY *ov_glVertex3i)(signed int x,signed int y,signed int z)=init_ov_glVertex3i;

void __gl_ml_glVertex3i(value v0,value v1,value v2)
{
	signed int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	ov_glVertex3i(l0,l1,l2);
}
extern void (APIENTRY *ov_glVertex3f)(float x,float y,float z);

static void APIENTRY init_ov_glVertex3f(float x,float y,float z)
{
	void *f;
	f=(void*)getfunc("glVertex3f");
	ov_glVertex3f=f;
	ov_glVertex3f(x,y,z);
}
void (APIENTRY *ov_glVertex3f)(float x,float y,float z)=init_ov_glVertex3f;

void __gl_ml_glVertex3f(value v0,value v1,value v2)
{
	float l0=Double_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	ov_glVertex3f(l0,l1,l2);
}
extern void (APIENTRY *ov_glVertex3d)(double x,double y,double z);

static void APIENTRY init_ov_glVertex3d(double x,double y,double z)
{
	void *f;
	f=(void*)getfunc("glVertex3d");
	ov_glVertex3d=f;
	ov_glVertex3d(x,y,z);
}
void (APIENTRY *ov_glVertex3d)(double x,double y,double z)=init_ov_glVertex3d;

void __gl_ml_glVertex3d(value v0,value v1,value v2)
{
	double l0=Double_val(v0);
	double l1=Double_val(v1);
	double l2=Double_val(v2);
	ov_glVertex3d(l0,l1,l2);
}
extern void (APIENTRY *ov_glVertex2s)(signed short x,signed short y);

static void APIENTRY init_ov_glVertex2s(signed short x,signed short y)
{
	void *f;
	f=(void*)getfunc("glVertex2s");
	ov_glVertex2s=f;
	ov_glVertex2s(x,y);
}
void (APIENTRY *ov_glVertex2s)(signed short x,signed short y)=init_ov_glVertex2s;

void __gl_ml_glVertex2s(value v0,value v1)
{
	signed short l0=Int_val(v0);
	signed short l1=Int_val(v1);
	ov_glVertex2s(l0,l1);
}
extern void (APIENTRY *ov_glVertex2i)(signed int x,signed int y);

static void APIENTRY init_ov_glVertex2i(signed int x,signed int y)
{
	void *f;
	f=(void*)getfunc("glVertex2i");
	ov_glVertex2i=f;
	ov_glVertex2i(x,y);
}
void (APIENTRY *ov_glVertex2i)(signed int x,signed int y)=init_ov_glVertex2i;

void __gl_ml_glVertex2i(value v0,value v1)
{
	signed int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	ov_glVertex2i(l0,l1);
}
extern void (APIENTRY *ov_glVertex2f)(float x,float y);

static void APIENTRY init_ov_glVertex2f(float x,float y)
{
	void *f;
	f=(void*)getfunc("glVertex2f");
	ov_glVertex2f=f;
	ov_glVertex2f(x,y);
}
void (APIENTRY *ov_glVertex2f)(float x,float y)=init_ov_glVertex2f;

void __gl_ml_glVertex2f(value v0,value v1)
{
	float l0=Double_val(v0);
	float l1=Double_val(v1);
	ov_glVertex2f(l0,l1);
}
extern void (APIENTRY *ov_glVertex2d)(double x,double y);

static void APIENTRY init_ov_glVertex2d(double x,double y)
{
	void *f;
	f=(void*)getfunc("glVertex2d");
	ov_glVertex2d=f;
	ov_glVertex2d(x,y);
}
void (APIENTRY *ov_glVertex2d)(double x,double y)=init_ov_glVertex2d;

void __gl_ml_glVertex2d(value v0,value v1)
{
	double l0=Double_val(v0);
	double l1=Double_val(v1);
	ov_glVertex2d(l0,l1);
}
extern void (APIENTRY *ov_glEnd)();

static void APIENTRY init_ov_glEnd()
{
	void *f;
	f=(void*)getfunc("glEnd");
	ov_glEnd=f;
	ov_glEnd();
}
void (APIENTRY *ov_glEnd)()=init_ov_glEnd;

void __gl_ml_glEnd()
{
	ov_glEnd();
}
extern void (APIENTRY *ov_glBegin)(unsigned int mode);

static void APIENTRY init_ov_glBegin(unsigned int mode)
{
	void *f;
	f=(void*)getfunc("glBegin");
	ov_glBegin=f;
	ov_glBegin(mode);
}
void (APIENTRY *ov_glBegin)(unsigned int mode)=init_ov_glBegin;

void __gl_ml_glBegin(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glBegin(l0);
}
extern void (APIENTRY *ov_glListBase)(unsigned int base);

static void APIENTRY init_ov_glListBase(unsigned int base)
{
	void *f;
	f=(void*)getfunc("glListBase");
	ov_glListBase=f;
	ov_glListBase(base);
}
void (APIENTRY *ov_glListBase)(unsigned int base)=init_ov_glListBase;

void __gl_ml_glListBase(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glListBase(l0);
}
extern void (APIENTRY *ov_glCallLists)(signed int n,unsigned int type,void* lists);

static void APIENTRY init_ov_glCallLists(signed int n,unsigned int type,void* lists)
{
	void *f;
	f=(void*)getfunc("glCallLists");
	ov_glCallLists=f;
	ov_glCallLists(n,type,lists);
}
void (APIENTRY *ov_glCallLists)(signed int n,unsigned int type,void* lists)=init_ov_glCallLists;

void __gl_ml_glCallLists(value v0,value v1,value v2)
{
	signed int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	void* l2=gl__ba__(v2);
	ov_glCallLists(l0,l1,l2);
}
extern void (APIENTRY *ov_glCallList)(unsigned int list);

static void APIENTRY init_ov_glCallList(unsigned int list)
{
	void *f;
	f=(void*)getfunc("glCallList");
	ov_glCallList=f;
	ov_glCallList(list);
}
void (APIENTRY *ov_glCallList)(unsigned int list)=init_ov_glCallList;

void __gl_ml_glCallList(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glCallList(l0);
}
extern void (APIENTRY *ov_glEndList)();

static void APIENTRY init_ov_glEndList()
{
	void *f;
	f=(void*)getfunc("glEndList");
	ov_glEndList=f;
	ov_glEndList();
}
void (APIENTRY *ov_glEndList)()=init_ov_glEndList;

void __gl_ml_glEndList()
{
	ov_glEndList();
}
extern void (APIENTRY *ov_glNewList)(unsigned int list,unsigned int mode);

static void APIENTRY init_ov_glNewList(unsigned int list,unsigned int mode)
{
	void *f;
	f=(void*)getfunc("glNewList");
	ov_glNewList=f;
	ov_glNewList(list,mode);
}
void (APIENTRY *ov_glNewList)(unsigned int list,unsigned int mode)=init_ov_glNewList;

void __gl_ml_glNewList(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	ov_glNewList(l0,l1);
}
extern unsigned int (APIENTRY *ov_glGenLists)(signed int range);

static unsigned int APIENTRY init_ov_glGenLists(signed int range)
{
	void *f;
	f=(void*)getfunc("glGenLists");
	ov_glGenLists=f;
	return ov_glGenLists(range);
}
unsigned int (APIENTRY *ov_glGenLists)(signed int range)=init_ov_glGenLists;

value __gl_ml_glGenLists(value v0)
{
	signed int l0=Int_val(v0);
	unsigned int r=ov_glGenLists(l0);
	if(((unsigned int)(r)) & 0x40000000) failwith("glGenLists int out of range");
	return(Val_int(r));
}
extern void (APIENTRY *ov_glDeleteLists)(unsigned int list,signed int range);

static void APIENTRY init_ov_glDeleteLists(unsigned int list,signed int range)
{
	void *f;
	f=(void*)getfunc("glDeleteLists");
	ov_glDeleteLists=f;
	ov_glDeleteLists(list,range);
}
void (APIENTRY *ov_glDeleteLists)(unsigned int list,signed int range)=init_ov_glDeleteLists;

void __gl_ml_glDeleteLists(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	ov_glDeleteLists(l0,l1);
}
extern unsigned char (APIENTRY *ov_glIsList)(unsigned int list);

static unsigned char APIENTRY init_ov_glIsList(unsigned int list)
{
	void *f;
	f=(void*)getfunc("glIsList");
	ov_glIsList=f;
	return ov_glIsList(list);
}
unsigned char (APIENTRY *ov_glIsList)(unsigned int list)=init_ov_glIsList;

value __gl_ml_glIsList(value v0)
{
	unsigned int l0=Int_val(v0);
	unsigned char r=ov_glIsList(l0);
	return(Val_int(r));
}
extern void (APIENTRY *ov_glTranslatef)(float x,float y,float z);

static void APIENTRY init_ov_glTranslatef(float x,float y,float z)
{
	void *f;
	f=(void*)getfunc("glTranslatef");
	ov_glTranslatef=f;
	ov_glTranslatef(x,y,z);
}
void (APIENTRY *ov_glTranslatef)(float x,float y,float z)=init_ov_glTranslatef;

void __gl_ml_glTranslatef(value v0,value v1,value v2)
{
	float l0=Double_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	ov_glTranslatef(l0,l1,l2);
}
extern void (APIENTRY *ov_glTranslated)(double x,double y,double z);

static void APIENTRY init_ov_glTranslated(double x,double y,double z)
{
	void *f;
	f=(void*)getfunc("glTranslated");
	ov_glTranslated=f;
	ov_glTranslated(x,y,z);
}
void (APIENTRY *ov_glTranslated)(double x,double y,double z)=init_ov_glTranslated;

void __gl_ml_glTranslated(value v0,value v1,value v2)
{
	double l0=Double_val(v0);
	double l1=Double_val(v1);
	double l2=Double_val(v2);
	ov_glTranslated(l0,l1,l2);
}
extern void (APIENTRY *ov_glScalef)(float x,float y,float z);

static void APIENTRY init_ov_glScalef(float x,float y,float z)
{
	void *f;
	f=(void*)getfunc("glScalef");
	ov_glScalef=f;
	ov_glScalef(x,y,z);
}
void (APIENTRY *ov_glScalef)(float x,float y,float z)=init_ov_glScalef;

void __gl_ml_glScalef(value v0,value v1,value v2)
{
	float l0=Double_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	ov_glScalef(l0,l1,l2);
}
extern void (APIENTRY *ov_glScaled)(double x,double y,double z);

static void APIENTRY init_ov_glScaled(double x,double y,double z)
{
	void *f;
	f=(void*)getfunc("glScaled");
	ov_glScaled=f;
	ov_glScaled(x,y,z);
}
void (APIENTRY *ov_glScaled)(double x,double y,double z)=init_ov_glScaled;

void __gl_ml_glScaled(value v0,value v1,value v2)
{
	double l0=Double_val(v0);
	double l1=Double_val(v1);
	double l2=Double_val(v2);
	ov_glScaled(l0,l1,l2);
}
extern void (APIENTRY *ov_glRotatef)(float angle,float x,float y,float z);

static void APIENTRY init_ov_glRotatef(float angle,float x,float y,float z)
{
	void *f;
	f=(void*)getfunc("glRotatef");
	ov_glRotatef=f;
	ov_glRotatef(angle,x,y,z);
}
void (APIENTRY *ov_glRotatef)(float angle,float x,float y,float z)=init_ov_glRotatef;

void __gl_ml_glRotatef(value v0,value v1,value v2,value v3)
{
	float l0=Double_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	float l3=Double_val(v3);
	ov_glRotatef(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glRotated)(double angle,double x,double y,double z);

static void APIENTRY init_ov_glRotated(double angle,double x,double y,double z)
{
	void *f;
	f=(void*)getfunc("glRotated");
	ov_glRotated=f;
	ov_glRotated(angle,x,y,z);
}
void (APIENTRY *ov_glRotated)(double angle,double x,double y,double z)=init_ov_glRotated;

void __gl_ml_glRotated(value v0,value v1,value v2,value v3)
{
	double l0=Double_val(v0);
	double l1=Double_val(v1);
	double l2=Double_val(v2);
	double l3=Double_val(v3);
	ov_glRotated(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glMultMatrixf)(float* m);

static void APIENTRY init_ov_glMultMatrixf(float* m)
{
	void *f;
	f=(void*)getfunc("glMultMatrixf");
	ov_glMultMatrixf=f;
	ov_glMultMatrixf(m);
}
void (APIENTRY *ov_glMultMatrixf)(float* m)=init_ov_glMultMatrixf;

void __gl_ml_glMultMatrixf(value v0)
{
	float* l0=(float*)(gl__ba__(v0));
	ov_glMultMatrixf(l0);
}
extern void (APIENTRY *ov_glMultMatrixd)(double* m);

static void APIENTRY init_ov_glMultMatrixd(double* m)
{
	void *f;
	f=(void*)getfunc("glMultMatrixd");
	ov_glMultMatrixd=f;
	ov_glMultMatrixd(m);
}
void (APIENTRY *ov_glMultMatrixd)(double* m)=init_ov_glMultMatrixd;

void __gl_ml_glMultMatrixd(value v0)
{
	double* l0=(double*)(gl__ba__(v0));
	ov_glMultMatrixd(l0);
}
extern void (APIENTRY *ov_glLoadMatrixf)(float* m);

static void APIENTRY init_ov_glLoadMatrixf(float* m)
{
	void *f;
	f=(void*)getfunc("glLoadMatrixf");
	ov_glLoadMatrixf=f;
	ov_glLoadMatrixf(m);
}
void (APIENTRY *ov_glLoadMatrixf)(float* m)=init_ov_glLoadMatrixf;

void __gl_ml_glLoadMatrixf(value v0)
{
	float* l0=(float*)(gl__ba__(v0));
	ov_glLoadMatrixf(l0);
}
extern void (APIENTRY *ov_glLoadMatrixd)(double* m);

static void APIENTRY init_ov_glLoadMatrixd(double* m)
{
	void *f;
	f=(void*)getfunc("glLoadMatrixd");
	ov_glLoadMatrixd=f;
	ov_glLoadMatrixd(m);
}
void (APIENTRY *ov_glLoadMatrixd)(double* m)=init_ov_glLoadMatrixd;

void __gl_ml_glLoadMatrixd(value v0)
{
	double* l0=(double*)(gl__ba__(v0));
	ov_glLoadMatrixd(l0);
}
extern void (APIENTRY *ov_glLoadIdentity)();

static void APIENTRY init_ov_glLoadIdentity()
{
	void *f;
	f=(void*)getfunc("glLoadIdentity");
	ov_glLoadIdentity=f;
	ov_glLoadIdentity();
}
void (APIENTRY *ov_glLoadIdentity)()=init_ov_glLoadIdentity;

void __gl_ml_glLoadIdentity()
{
	ov_glLoadIdentity();
}
extern void (APIENTRY *ov_glPopMatrix)();

static void APIENTRY init_ov_glPopMatrix()
{
	void *f;
	f=(void*)getfunc("glPopMatrix");
	ov_glPopMatrix=f;
	ov_glPopMatrix();
}
void (APIENTRY *ov_glPopMatrix)()=init_ov_glPopMatrix;

void __gl_ml_glPopMatrix()
{
	ov_glPopMatrix();
}
extern void (APIENTRY *ov_glPushMatrix)();

static void APIENTRY init_ov_glPushMatrix()
{
	void *f;
	f=(void*)getfunc("glPushMatrix");
	ov_glPushMatrix=f;
	ov_glPushMatrix();
}
void (APIENTRY *ov_glPushMatrix)()=init_ov_glPushMatrix;

void __gl_ml_glPushMatrix()
{
	ov_glPushMatrix();
}
extern void (APIENTRY *ov_glViewport)(signed int x,signed int y,signed int width,signed int height);

static void APIENTRY init_ov_glViewport(signed int x,signed int y,signed int width,signed int height)
{
	void *f;
	f=(void*)getfunc("glViewport");
	ov_glViewport=f;
	ov_glViewport(x,y,width,height);
}
void (APIENTRY *ov_glViewport)(signed int x,signed int y,signed int width,signed int height)=init_ov_glViewport;

void __gl_ml_glViewport(value v0,value v1,value v2,value v3)
{
	signed int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	ov_glViewport(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glFrustum)(double left,double right,double bottom,double top,double near_val,double far_val);

static void APIENTRY init_ov_glFrustum(double left,double right,double bottom,double top,double near_val,double far_val)
{
	void *f;
	f=(void*)getfunc("glFrustum");
	ov_glFrustum=f;
	ov_glFrustum(left,right,bottom,top,near_val,far_val);
}
void (APIENTRY *ov_glFrustum)(double left,double right,double bottom,double top,double near_val,double far_val)=init_ov_glFrustum;

void __gl_ml_glFrustum(value v0,value v1,value v2,value v3,value v4,value v5)
{
	double l0=Double_val(v0);
	double l1=Double_val(v1);
	double l2=Double_val(v2);
	double l3=Double_val(v3);
	double l4=Double_val(v4);
	double l5=Double_val(v5);
	ov_glFrustum(l0,l1,l2,l3,l4,l5);
}

void byte___gl_ml_glFrustum(value * argv,int argn)
{
	__gl_ml_glFrustum(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5]);
}
extern void (APIENTRY *ov_glOrtho)(double left,double right,double bottom,double top,double near_val,double far_val);

static void APIENTRY init_ov_glOrtho(double left,double right,double bottom,double top,double near_val,double far_val)
{
	void *f;
	f=(void*)getfunc("glOrtho");
	ov_glOrtho=f;
	ov_glOrtho(left,right,bottom,top,near_val,far_val);
}
void (APIENTRY *ov_glOrtho)(double left,double right,double bottom,double top,double near_val,double far_val)=init_ov_glOrtho;

void __gl_ml_glOrtho(value v0,value v1,value v2,value v3,value v4,value v5)
{
	double l0=Double_val(v0);
	double l1=Double_val(v1);
	double l2=Double_val(v2);
	double l3=Double_val(v3);
	double l4=Double_val(v4);
	double l5=Double_val(v5);
	ov_glOrtho(l0,l1,l2,l3,l4,l5);
}

void byte___gl_ml_glOrtho(value * argv,int argn)
{
	__gl_ml_glOrtho(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5]);
}
extern void (APIENTRY *ov_glMatrixMode)(unsigned int mode);

static void APIENTRY init_ov_glMatrixMode(unsigned int mode)
{
	void *f;
	f=(void*)getfunc("glMatrixMode");
	ov_glMatrixMode=f;
	ov_glMatrixMode(mode);
}
void (APIENTRY *ov_glMatrixMode)(unsigned int mode)=init_ov_glMatrixMode;

void __gl_ml_glMatrixMode(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glMatrixMode(l0);
}
extern void (APIENTRY *ov_glAccum)(unsigned int op,float value);

static void APIENTRY init_ov_glAccum(unsigned int op,float value)
{
	void *f;
	f=(void*)getfunc("glAccum");
	ov_glAccum=f;
	ov_glAccum(op,value);
}
void (APIENTRY *ov_glAccum)(unsigned int op,float value)=init_ov_glAccum;

void __gl_ml_glAccum(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float l1=Double_val(v1);
	ov_glAccum(l0,l1);
}
extern void (APIENTRY *ov_glClearAccum)(float red,float green,float blue,float alpha);

static void APIENTRY init_ov_glClearAccum(float red,float green,float blue,float alpha)
{
	void *f;
	f=(void*)getfunc("glClearAccum");
	ov_glClearAccum=f;
	ov_glClearAccum(red,green,blue,alpha);
}
void (APIENTRY *ov_glClearAccum)(float red,float green,float blue,float alpha)=init_ov_glClearAccum;

void __gl_ml_glClearAccum(value v0,value v1,value v2,value v3)
{
	float l0=Double_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	float l3=Double_val(v3);
	ov_glClearAccum(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glDepthRange)(double near_val,double far_val);

static void APIENTRY init_ov_glDepthRange(double near_val,double far_val)
{
	void *f;
	f=(void*)getfunc("glDepthRange");
	ov_glDepthRange=f;
	ov_glDepthRange(near_val,far_val);
}
void (APIENTRY *ov_glDepthRange)(double near_val,double far_val)=init_ov_glDepthRange;

void __gl_ml_glDepthRange(value v0,value v1)
{
	double l0=Double_val(v0);
	double l1=Double_val(v1);
	ov_glDepthRange(l0,l1);
}
extern void (APIENTRY *ov_glDepthMask)(unsigned char flag);

static void APIENTRY init_ov_glDepthMask(unsigned char flag)
{
	void *f;
	f=(void*)getfunc("glDepthMask");
	ov_glDepthMask=f;
	ov_glDepthMask(flag);
}
void (APIENTRY *ov_glDepthMask)(unsigned char flag)=init_ov_glDepthMask;

void __gl_ml_glDepthMask(value v0)
{
	unsigned char l0=Int_val(v0);
	ov_glDepthMask(l0);
}
extern void (APIENTRY *ov_glDepthFunc)(unsigned int func);

static void APIENTRY init_ov_glDepthFunc(unsigned int func)
{
	void *f;
	f=(void*)getfunc("glDepthFunc");
	ov_glDepthFunc=f;
	ov_glDepthFunc(func);
}
void (APIENTRY *ov_glDepthFunc)(unsigned int func)=init_ov_glDepthFunc;

void __gl_ml_glDepthFunc(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glDepthFunc(l0);
}
extern void (APIENTRY *ov_glClearDepth)(double depth);

static void APIENTRY init_ov_glClearDepth(double depth)
{
	void *f;
	f=(void*)getfunc("glClearDepth");
	ov_glClearDepth=f;
	ov_glClearDepth(depth);
}
void (APIENTRY *ov_glClearDepth)(double depth)=init_ov_glClearDepth;

void __gl_ml_glClearDepth(value v0)
{
	double l0=Double_val(v0);
	ov_glClearDepth(l0);
}
extern void (APIENTRY *ov_glHint)(unsigned int target,unsigned int mode);

static void APIENTRY init_ov_glHint(unsigned int target,unsigned int mode)
{
	void *f;
	f=(void*)getfunc("glHint");
	ov_glHint=f;
	ov_glHint(target,mode);
}
void (APIENTRY *ov_glHint)(unsigned int target,unsigned int mode)=init_ov_glHint;

void __gl_ml_glHint(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	ov_glHint(l0,l1);
}
extern void (APIENTRY *ov_glFlush)();

static void APIENTRY init_ov_glFlush()
{
	void *f;
	f=(void*)getfunc("glFlush");
	ov_glFlush=f;
	ov_glFlush();
}
void (APIENTRY *ov_glFlush)()=init_ov_glFlush;

void __gl_ml_glFlush()
{
	ov_glFlush();
}
extern void (APIENTRY *ov_glFinish)();

static void APIENTRY init_ov_glFinish()
{
	void *f;
	f=(void*)getfunc("glFinish");
	ov_glFinish=f;
	ov_glFinish();
}
void (APIENTRY *ov_glFinish)()=init_ov_glFinish;

void __gl_ml_glFinish()
{
	ov_glFinish();
}
extern char* (APIENTRY *ov_glGetString)(unsigned int name);

static char* APIENTRY init_ov_glGetString(unsigned int name)
{
	void *f;
	f=(void*)getfunc("glGetString");
	ov_glGetString=f;
	return ov_glGetString(name);
}
char* (APIENTRY *ov_glGetString)(unsigned int name)=init_ov_glGetString;

value __gl_ml_glGetString(value v0)
{
	CAMLparam0();
	CAMLxparam1(v0);
	unsigned int l0=Int_val(v0);
	char* r=ov_glGetString(l0);
	CAMLreturn(copy_string(r));
}
extern unsigned int (APIENTRY *ov_glGetError)();

static unsigned int APIENTRY init_ov_glGetError()
{
	void *f;
	f=(void*)getfunc("glGetError");
	ov_glGetError=f;
	return ov_glGetError();
}
unsigned int (APIENTRY *ov_glGetError)()=init_ov_glGetError;

value __gl_ml_glGetError()
{
	unsigned int r=ov_glGetError();
	if(((unsigned int)(r)) & 0x40000000) failwith("glGetError int out of range");
	return(Val_int(r));
}
extern signed int (APIENTRY *ov_glRenderMode)(unsigned int mode);

static signed int APIENTRY init_ov_glRenderMode(unsigned int mode)
{
	void *f;
	f=(void*)getfunc("glRenderMode");
	ov_glRenderMode=f;
	return ov_glRenderMode(mode);
}
signed int (APIENTRY *ov_glRenderMode)(unsigned int mode)=init_ov_glRenderMode;

value __gl_ml_glRenderMode(value v0)
{
	unsigned int l0=Int_val(v0);
	signed int r=ov_glRenderMode(l0);
	if(((unsigned int)(r)) & 0x40000000) failwith("glRenderMode int out of range");
	return(Val_int(r));
}
extern void (APIENTRY *ov_glPopAttrib)();

static void APIENTRY init_ov_glPopAttrib()
{
	void *f;
	f=(void*)getfunc("glPopAttrib");
	ov_glPopAttrib=f;
	ov_glPopAttrib();
}
void (APIENTRY *ov_glPopAttrib)()=init_ov_glPopAttrib;

void __gl_ml_glPopAttrib()
{
	ov_glPopAttrib();
}
extern void (APIENTRY *ov_glPushAttrib)(unsigned int mask);

static void APIENTRY init_ov_glPushAttrib(unsigned int mask)
{
	void *f;
	f=(void*)getfunc("glPushAttrib");
	ov_glPushAttrib=f;
	ov_glPushAttrib(mask);
}
void (APIENTRY *ov_glPushAttrib)(unsigned int mask)=init_ov_glPushAttrib;

void __gl_ml_glPushAttrib(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glPushAttrib(l0);
}
extern void (APIENTRY *ov_glGetIntegerv)(unsigned int pname,signed int* params);

static void APIENTRY init_ov_glGetIntegerv(unsigned int pname,signed int* params)
{
	void *f;
	f=(void*)getfunc("glGetIntegerv");
	ov_glGetIntegerv=f;
	ov_glGetIntegerv(pname,params);
}
void (APIENTRY *ov_glGetIntegerv)(unsigned int pname,signed int* params)=init_ov_glGetIntegerv;

void __gl_ml_glGetIntegerv(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed int* l1=(signed int*)(gl__ba__(v1));
	ov_glGetIntegerv(l0,l1);
}
extern void (APIENTRY *ov_glGetFloatv)(unsigned int pname,float* params);

static void APIENTRY init_ov_glGetFloatv(unsigned int pname,float* params)
{
	void *f;
	f=(void*)getfunc("glGetFloatv");
	ov_glGetFloatv=f;
	ov_glGetFloatv(pname,params);
}
void (APIENTRY *ov_glGetFloatv)(unsigned int pname,float* params)=init_ov_glGetFloatv;

void __gl_ml_glGetFloatv(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float* l1=(float*)(gl__ba__(v1));
	ov_glGetFloatv(l0,l1);
}
extern void (APIENTRY *ov_glGetDoublev)(unsigned int pname,double* params);

static void APIENTRY init_ov_glGetDoublev(unsigned int pname,double* params)
{
	void *f;
	f=(void*)getfunc("glGetDoublev");
	ov_glGetDoublev=f;
	ov_glGetDoublev(pname,params);
}
void (APIENTRY *ov_glGetDoublev)(unsigned int pname,double* params)=init_ov_glGetDoublev;

void __gl_ml_glGetDoublev(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	double* l1=(double*)(gl__ba__(v1));
	ov_glGetDoublev(l0,l1);
}
extern void (APIENTRY *ov_glGetBooleanv)(unsigned int pname,unsigned char* params);

static void APIENTRY init_ov_glGetBooleanv(unsigned int pname,unsigned char* params)
{
	void *f;
	f=(void*)getfunc("glGetBooleanv");
	ov_glGetBooleanv=f;
	ov_glGetBooleanv(pname,params);
}
void (APIENTRY *ov_glGetBooleanv)(unsigned int pname,unsigned char* params)=init_ov_glGetBooleanv;

void __gl_ml_glGetBooleanv(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	unsigned char* l1=(unsigned char*)(gl__ba__(v1));
	ov_glGetBooleanv(l0,l1);
}
extern unsigned char (APIENTRY *ov_glIsEnabled)(unsigned int cap);

static unsigned char APIENTRY init_ov_glIsEnabled(unsigned int cap)
{
	void *f;
	f=(void*)getfunc("glIsEnabled");
	ov_glIsEnabled=f;
	return ov_glIsEnabled(cap);
}
unsigned char (APIENTRY *ov_glIsEnabled)(unsigned int cap)=init_ov_glIsEnabled;

value __gl_ml_glIsEnabled(value v0)
{
	unsigned int l0=Int_val(v0);
	unsigned char r=ov_glIsEnabled(l0);
	return(Val_int(r));
}
extern void (APIENTRY *ov_glDisable)(unsigned int cap);

static void APIENTRY init_ov_glDisable(unsigned int cap)
{
	void *f;
	f=(void*)getfunc("glDisable");
	ov_glDisable=f;
	ov_glDisable(cap);
}
void (APIENTRY *ov_glDisable)(unsigned int cap)=init_ov_glDisable;

void __gl_ml_glDisable(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glDisable(l0);
}
extern void (APIENTRY *ov_glEnable)(unsigned int cap);

static void APIENTRY init_ov_glEnable(unsigned int cap)
{
	void *f;
	f=(void*)getfunc("glEnable");
	ov_glEnable=f;
	ov_glEnable(cap);
}
void (APIENTRY *ov_glEnable)(unsigned int cap)=init_ov_glEnable;

void __gl_ml_glEnable(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glEnable(l0);
}
extern void (APIENTRY *ov_glReadBuffer)(unsigned int mode);

static void APIENTRY init_ov_glReadBuffer(unsigned int mode)
{
	void *f;
	f=(void*)getfunc("glReadBuffer");
	ov_glReadBuffer=f;
	ov_glReadBuffer(mode);
}
void (APIENTRY *ov_glReadBuffer)(unsigned int mode)=init_ov_glReadBuffer;

void __gl_ml_glReadBuffer(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glReadBuffer(l0);
}
extern void (APIENTRY *ov_glDrawBuffer)(unsigned int mode);

static void APIENTRY init_ov_glDrawBuffer(unsigned int mode)
{
	void *f;
	f=(void*)getfunc("glDrawBuffer");
	ov_glDrawBuffer=f;
	ov_glDrawBuffer(mode);
}
void (APIENTRY *ov_glDrawBuffer)(unsigned int mode)=init_ov_glDrawBuffer;

void __gl_ml_glDrawBuffer(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glDrawBuffer(l0);
}
extern void (APIENTRY *ov_glGetClipPlane)(unsigned int plane,double* equation);

static void APIENTRY init_ov_glGetClipPlane(unsigned int plane,double* equation)
{
	void *f;
	f=(void*)getfunc("glGetClipPlane");
	ov_glGetClipPlane=f;
	ov_glGetClipPlane(plane,equation);
}
void (APIENTRY *ov_glGetClipPlane)(unsigned int plane,double* equation)=init_ov_glGetClipPlane;

void __gl_ml_glGetClipPlane(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	double* l1=(double*)(gl__ba__(v1));
	ov_glGetClipPlane(l0,l1);
}
extern void (APIENTRY *ov_glClipPlane)(unsigned int plane,double* equation);

static void APIENTRY init_ov_glClipPlane(unsigned int plane,double* equation)
{
	void *f;
	f=(void*)getfunc("glClipPlane");
	ov_glClipPlane=f;
	ov_glClipPlane(plane,equation);
}
void (APIENTRY *ov_glClipPlane)(unsigned int plane,double* equation)=init_ov_glClipPlane;

void __gl_ml_glClipPlane(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	double* l1=(double*)(gl__ba__(v1));
	ov_glClipPlane(l0,l1);
}
extern void (APIENTRY *ov_glScissor)(signed int x,signed int y,signed int width,signed int height);

static void APIENTRY init_ov_glScissor(signed int x,signed int y,signed int width,signed int height)
{
	void *f;
	f=(void*)getfunc("glScissor");
	ov_glScissor=f;
	ov_glScissor(x,y,width,height);
}
void (APIENTRY *ov_glScissor)(signed int x,signed int y,signed int width,signed int height)=init_ov_glScissor;

void __gl_ml_glScissor(value v0,value v1,value v2,value v3)
{
	signed int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	ov_glScissor(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glEdgeFlagv)(unsigned char* flag);

static void APIENTRY init_ov_glEdgeFlagv(unsigned char* flag)
{
	void *f;
	f=(void*)getfunc("glEdgeFlagv");
	ov_glEdgeFlagv=f;
	ov_glEdgeFlagv(flag);
}
void (APIENTRY *ov_glEdgeFlagv)(unsigned char* flag)=init_ov_glEdgeFlagv;

void __gl_ml_glEdgeFlagv(value v0)
{
	unsigned char* l0=(unsigned char*)(gl__ba__(v0));
	ov_glEdgeFlagv(l0);
}
extern void (APIENTRY *ov_glEdgeFlag)(unsigned char flag);

static void APIENTRY init_ov_glEdgeFlag(unsigned char flag)
{
	void *f;
	f=(void*)getfunc("glEdgeFlag");
	ov_glEdgeFlag=f;
	ov_glEdgeFlag(flag);
}
void (APIENTRY *ov_glEdgeFlag)(unsigned char flag)=init_ov_glEdgeFlag;

void __gl_ml_glEdgeFlag(value v0)
{
	unsigned char l0=Int_val(v0);
	ov_glEdgeFlag(l0);
}
extern void (APIENTRY *ov_glGetPolygonStipple)(unsigned char* mask);

static void APIENTRY init_ov_glGetPolygonStipple(unsigned char* mask)
{
	void *f;
	f=(void*)getfunc("glGetPolygonStipple");
	ov_glGetPolygonStipple=f;
	ov_glGetPolygonStipple(mask);
}
void (APIENTRY *ov_glGetPolygonStipple)(unsigned char* mask)=init_ov_glGetPolygonStipple;

void __gl_ml_glGetPolygonStipple(value v0)
{
	unsigned char* l0=(unsigned char*)(gl__ba__(v0));
	ov_glGetPolygonStipple(l0);
}
extern void (APIENTRY *ov_glPolygonStipple)(unsigned char* mask);

static void APIENTRY init_ov_glPolygonStipple(unsigned char* mask)
{
	void *f;
	f=(void*)getfunc("glPolygonStipple");
	ov_glPolygonStipple=f;
	ov_glPolygonStipple(mask);
}
void (APIENTRY *ov_glPolygonStipple)(unsigned char* mask)=init_ov_glPolygonStipple;

void __gl_ml_glPolygonStipple(value v0)
{
	unsigned char* l0=(unsigned char*)(gl__ba__(v0));
	ov_glPolygonStipple(l0);
}
extern void (APIENTRY *ov_glPolygonOffset)(float factor,float units);

static void APIENTRY init_ov_glPolygonOffset(float factor,float units)
{
	void *f;
	f=(void*)getfunc("glPolygonOffset");
	ov_glPolygonOffset=f;
	ov_glPolygonOffset(factor,units);
}
void (APIENTRY *ov_glPolygonOffset)(float factor,float units)=init_ov_glPolygonOffset;

void __gl_ml_glPolygonOffset(value v0,value v1)
{
	float l0=Double_val(v0);
	float l1=Double_val(v1);
	ov_glPolygonOffset(l0,l1);
}
extern void (APIENTRY *ov_glPolygonMode)(unsigned int face,unsigned int mode);

static void APIENTRY init_ov_glPolygonMode(unsigned int face,unsigned int mode)
{
	void *f;
	f=(void*)getfunc("glPolygonMode");
	ov_glPolygonMode=f;
	ov_glPolygonMode(face,mode);
}
void (APIENTRY *ov_glPolygonMode)(unsigned int face,unsigned int mode)=init_ov_glPolygonMode;

void __gl_ml_glPolygonMode(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	ov_glPolygonMode(l0,l1);
}
extern void (APIENTRY *ov_glLineStipple)(signed int factor,unsigned short pattern);

static void APIENTRY init_ov_glLineStipple(signed int factor,unsigned short pattern)
{
	void *f;
	f=(void*)getfunc("glLineStipple");
	ov_glLineStipple=f;
	ov_glLineStipple(factor,pattern);
}
void (APIENTRY *ov_glLineStipple)(signed int factor,unsigned short pattern)=init_ov_glLineStipple;

void __gl_ml_glLineStipple(value v0,value v1)
{
	signed int l0=Int_val(v0);
	unsigned short l1=Int_val(v1);
	ov_glLineStipple(l0,l1);
}
extern void (APIENTRY *ov_glLineWidth)(float width);

static void APIENTRY init_ov_glLineWidth(float width)
{
	void *f;
	f=(void*)getfunc("glLineWidth");
	ov_glLineWidth=f;
	ov_glLineWidth(width);
}
void (APIENTRY *ov_glLineWidth)(float width)=init_ov_glLineWidth;

void __gl_ml_glLineWidth(value v0)
{
	float l0=Double_val(v0);
	ov_glLineWidth(l0);
}
extern void (APIENTRY *ov_glPointSize)(float size);

static void APIENTRY init_ov_glPointSize(float size)
{
	void *f;
	f=(void*)getfunc("glPointSize");
	ov_glPointSize=f;
	ov_glPointSize(size);
}
void (APIENTRY *ov_glPointSize)(float size)=init_ov_glPointSize;

void __gl_ml_glPointSize(value v0)
{
	float l0=Double_val(v0);
	ov_glPointSize(l0);
}
extern void (APIENTRY *ov_glFrontFace)(unsigned int mode);

static void APIENTRY init_ov_glFrontFace(unsigned int mode)
{
	void *f;
	f=(void*)getfunc("glFrontFace");
	ov_glFrontFace=f;
	ov_glFrontFace(mode);
}
void (APIENTRY *ov_glFrontFace)(unsigned int mode)=init_ov_glFrontFace;

void __gl_ml_glFrontFace(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glFrontFace(l0);
}
extern void (APIENTRY *ov_glCullFace)(unsigned int mode);

static void APIENTRY init_ov_glCullFace(unsigned int mode)
{
	void *f;
	f=(void*)getfunc("glCullFace");
	ov_glCullFace=f;
	ov_glCullFace(mode);
}
void (APIENTRY *ov_glCullFace)(unsigned int mode)=init_ov_glCullFace;

void __gl_ml_glCullFace(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glCullFace(l0);
}
extern void (APIENTRY *ov_glLogicOp)(unsigned int opcode);

static void APIENTRY init_ov_glLogicOp(unsigned int opcode)
{
	void *f;
	f=(void*)getfunc("glLogicOp");
	ov_glLogicOp=f;
	ov_glLogicOp(opcode);
}
void (APIENTRY *ov_glLogicOp)(unsigned int opcode)=init_ov_glLogicOp;

void __gl_ml_glLogicOp(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glLogicOp(l0);
}
extern void (APIENTRY *ov_glBlendFunc)(unsigned int sfactor,unsigned int dfactor);

static void APIENTRY init_ov_glBlendFunc(unsigned int sfactor,unsigned int dfactor)
{
	void *f;
	f=(void*)getfunc("glBlendFunc");
	ov_glBlendFunc=f;
	ov_glBlendFunc(sfactor,dfactor);
}
void (APIENTRY *ov_glBlendFunc)(unsigned int sfactor,unsigned int dfactor)=init_ov_glBlendFunc;

void __gl_ml_glBlendFunc(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	ov_glBlendFunc(l0,l1);
}
extern void (APIENTRY *ov_glAlphaFunc)(unsigned int func,float ref);

static void APIENTRY init_ov_glAlphaFunc(unsigned int func,float ref)
{
	void *f;
	f=(void*)getfunc("glAlphaFunc");
	ov_glAlphaFunc=f;
	ov_glAlphaFunc(func,ref);
}
void (APIENTRY *ov_glAlphaFunc)(unsigned int func,float ref)=init_ov_glAlphaFunc;

void __gl_ml_glAlphaFunc(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float l1=Double_val(v1);
	ov_glAlphaFunc(l0,l1);
}
extern void (APIENTRY *ov_glColorMask)(unsigned char red,unsigned char green,unsigned char blue,unsigned char alpha);

static void APIENTRY init_ov_glColorMask(unsigned char red,unsigned char green,unsigned char blue,unsigned char alpha)
{
	void *f;
	f=(void*)getfunc("glColorMask");
	ov_glColorMask=f;
	ov_glColorMask(red,green,blue,alpha);
}
void (APIENTRY *ov_glColorMask)(unsigned char red,unsigned char green,unsigned char blue,unsigned char alpha)=init_ov_glColorMask;

void __gl_ml_glColorMask(value v0,value v1,value v2,value v3)
{
	unsigned char l0=Int_val(v0);
	unsigned char l1=Int_val(v1);
	unsigned char l2=Int_val(v2);
	unsigned char l3=Int_val(v3);
	ov_glColorMask(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glIndexMask)(unsigned int mask);

static void APIENTRY init_ov_glIndexMask(unsigned int mask)
{
	void *f;
	f=(void*)getfunc("glIndexMask");
	ov_glIndexMask=f;
	ov_glIndexMask(mask);
}
void (APIENTRY *ov_glIndexMask)(unsigned int mask)=init_ov_glIndexMask;

void __gl_ml_glIndexMask(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glIndexMask(l0);
}
extern void (APIENTRY *ov_glClear)(unsigned int mask);

static void APIENTRY init_ov_glClear(unsigned int mask)
{
	void *f;
	f=(void*)getfunc("glClear");
	ov_glClear=f;
	ov_glClear(mask);
}
void (APIENTRY *ov_glClear)(unsigned int mask)=init_ov_glClear;

void __gl_ml_glClear(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glClear(l0);
}
extern void (APIENTRY *ov_glClearColor)(float red,float green,float blue,float alpha);

static void APIENTRY init_ov_glClearColor(float red,float green,float blue,float alpha)
{
	void *f;
	f=(void*)getfunc("glClearColor");
	ov_glClearColor=f;
	ov_glClearColor(red,green,blue,alpha);
}
void (APIENTRY *ov_glClearColor)(float red,float green,float blue,float alpha)=init_ov_glClearColor;

void __gl_ml_glClearColor(value v0,value v1,value v2,value v3)
{
	float l0=Double_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	float l3=Double_val(v3);
	ov_glClearColor(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glClearIndex)(float c);

static void APIENTRY init_ov_glClearIndex(float c)
{
	void *f;
	f=(void*)getfunc("glClearIndex");
	ov_glClearIndex=f;
	ov_glClearIndex(c);
}
void (APIENTRY *ov_glClearIndex)(float c)=init_ov_glClearIndex;

void __gl_ml_glClearIndex(value v0)
{
	float l0=Double_val(v0);
	ov_glClearIndex(l0);
}
extern void (APIENTRY *ov_glIndexubv)(unsigned char* c);

static void APIENTRY init_ov_glIndexubv(unsigned char* c)
{
	void *f;
	f=(void*)getfunc("glIndexubv");
	ov_glIndexubv=f;
	ov_glIndexubv(c);
}
void (APIENTRY *ov_glIndexubv)(unsigned char* c)=init_ov_glIndexubv;

void __gl_ml_glIndexubv(value v0)
{
	unsigned char* l0=(unsigned char*)(gl__ba__(v0));
	ov_glIndexubv(l0);
}
extern void (APIENTRY *ov_glIndexub)(unsigned char c);

static void APIENTRY init_ov_glIndexub(unsigned char c)
{
	void *f;
	f=(void*)getfunc("glIndexub");
	ov_glIndexub=f;
	ov_glIndexub(c);
}
void (APIENTRY *ov_glIndexub)(unsigned char c)=init_ov_glIndexub;

void __gl_ml_glIndexub(value v0)
{
	unsigned char l0=Int_val(v0);
	ov_glIndexub(l0);
}
extern void (APIENTRY *ov_glPopClientAttrib)();

static void APIENTRY init_ov_glPopClientAttrib()
{
	void *f;
	f=(void*)getfunc("glPopClientAttrib");
	ov_glPopClientAttrib=f;
	ov_glPopClientAttrib();
}
void (APIENTRY *ov_glPopClientAttrib)()=init_ov_glPopClientAttrib;

void __gl_ml_glPopClientAttrib()
{
	ov_glPopClientAttrib();
}
extern void (APIENTRY *ov_glPushClientAttrib)(unsigned int mask);

static void APIENTRY init_ov_glPushClientAttrib(unsigned int mask)
{
	void *f;
	f=(void*)getfunc("glPushClientAttrib");
	ov_glPushClientAttrib=f;
	ov_glPushClientAttrib(mask);
}
void (APIENTRY *ov_glPushClientAttrib)(unsigned int mask)=init_ov_glPushClientAttrib;

void __gl_ml_glPushClientAttrib(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glPushClientAttrib(l0);
}
extern void (APIENTRY *ov_glDisableClientState)(unsigned int cap);

static void APIENTRY init_ov_glDisableClientState(unsigned int cap)
{
	void *f;
	f=(void*)getfunc("glDisableClientState");
	ov_glDisableClientState=f;
	ov_glDisableClientState(cap);
}
void (APIENTRY *ov_glDisableClientState)(unsigned int cap)=init_ov_glDisableClientState;

void __gl_ml_glDisableClientState(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glDisableClientState(l0);
}
extern void (APIENTRY *ov_glEnableClientState)(unsigned int cap);

static void APIENTRY init_ov_glEnableClientState(unsigned int cap)
{
	void *f;
	f=(void*)getfunc("glEnableClientState");
	ov_glEnableClientState=f;
	ov_glEnableClientState(cap);
}
void (APIENTRY *ov_glEnableClientState)(unsigned int cap)=init_ov_glEnableClientState;

void __gl_ml_glEnableClientState(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glEnableClientState(l0);
}
extern void (APIENTRY *ov_glCopyTexSubImage2D)(unsigned int target,signed int level,signed int xoffset,signed int yoffset,signed int x,signed int y,signed int width,signed int height);

static void APIENTRY init_ov_glCopyTexSubImage2D(unsigned int target,signed int level,signed int xoffset,signed int yoffset,signed int x,signed int y,signed int width,signed int height)
{
	void *f;
	f=(void*)getfunc("glCopyTexSubImage2D");
	ov_glCopyTexSubImage2D=f;
	ov_glCopyTexSubImage2D(target,level,xoffset,yoffset,x,y,width,height);
}
void (APIENTRY *ov_glCopyTexSubImage2D)(unsigned int target,signed int level,signed int xoffset,signed int yoffset,signed int x,signed int y,signed int width,signed int height)=init_ov_glCopyTexSubImage2D;

void __gl_ml_glCopyTexSubImage2D(value v0,value v1,value v2,value v3,value v4,value v5,value v6,value v7)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	signed int l4=Int_val(v4);
	signed int l5=Int_val(v5);
	signed int l6=Int_val(v6);
	signed int l7=Int_val(v7);
	ov_glCopyTexSubImage2D(l0,l1,l2,l3,l4,l5,l6,l7);
}

void byte___gl_ml_glCopyTexSubImage2D(value * argv,int argn)
{
	__gl_ml_glCopyTexSubImage2D(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6],argv[7]);
}
extern void (APIENTRY *ov_glCopyTexSubImage1D)(unsigned int target,signed int level,signed int xoffset,signed int x,signed int y,signed int width);

static void APIENTRY init_ov_glCopyTexSubImage1D(unsigned int target,signed int level,signed int xoffset,signed int x,signed int y,signed int width)
{
	void *f;
	f=(void*)getfunc("glCopyTexSubImage1D");
	ov_glCopyTexSubImage1D=f;
	ov_glCopyTexSubImage1D(target,level,xoffset,x,y,width);
}
void (APIENTRY *ov_glCopyTexSubImage1D)(unsigned int target,signed int level,signed int xoffset,signed int x,signed int y,signed int width)=init_ov_glCopyTexSubImage1D;

void __gl_ml_glCopyTexSubImage1D(value v0,value v1,value v2,value v3,value v4,value v5)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	signed int l4=Int_val(v4);
	signed int l5=Int_val(v5);
	ov_glCopyTexSubImage1D(l0,l1,l2,l3,l4,l5);
}

void byte___gl_ml_glCopyTexSubImage1D(value * argv,int argn)
{
	__gl_ml_glCopyTexSubImage1D(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5]);
}
extern void (APIENTRY *ov_glCopyTexImage2D)(unsigned int target,signed int level,unsigned int internalformat,signed int x,signed int y,signed int width,signed int height,signed int border);

static void APIENTRY init_ov_glCopyTexImage2D(unsigned int target,signed int level,unsigned int internalformat,signed int x,signed int y,signed int width,signed int height,signed int border)
{
	void *f;
	f=(void*)getfunc("glCopyTexImage2D");
	ov_glCopyTexImage2D=f;
	ov_glCopyTexImage2D(target,level,internalformat,x,y,width,height,border);
}
void (APIENTRY *ov_glCopyTexImage2D)(unsigned int target,signed int level,unsigned int internalformat,signed int x,signed int y,signed int width,signed int height,signed int border)=init_ov_glCopyTexImage2D;

void __gl_ml_glCopyTexImage2D(value v0,value v1,value v2,value v3,value v4,value v5,value v6,value v7)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	signed int l4=Int_val(v4);
	signed int l5=Int_val(v5);
	signed int l6=Int_val(v6);
	signed int l7=Int_val(v7);
	ov_glCopyTexImage2D(l0,l1,l2,l3,l4,l5,l6,l7);
}

void byte___gl_ml_glCopyTexImage2D(value * argv,int argn)
{
	__gl_ml_glCopyTexImage2D(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6],argv[7]);
}
extern void (APIENTRY *ov_glCopyTexImage1D)(unsigned int target,signed int level,unsigned int internalformat,signed int x,signed int y,signed int width,signed int border);

static void APIENTRY init_ov_glCopyTexImage1D(unsigned int target,signed int level,unsigned int internalformat,signed int x,signed int y,signed int width,signed int border)
{
	void *f;
	f=(void*)getfunc("glCopyTexImage1D");
	ov_glCopyTexImage1D=f;
	ov_glCopyTexImage1D(target,level,internalformat,x,y,width,border);
}
void (APIENTRY *ov_glCopyTexImage1D)(unsigned int target,signed int level,unsigned int internalformat,signed int x,signed int y,signed int width,signed int border)=init_ov_glCopyTexImage1D;

void __gl_ml_glCopyTexImage1D(value v0,value v1,value v2,value v3,value v4,value v5,value v6)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	signed int l4=Int_val(v4);
	signed int l5=Int_val(v5);
	signed int l6=Int_val(v6);
	ov_glCopyTexImage1D(l0,l1,l2,l3,l4,l5,l6);
}

void byte___gl_ml_glCopyTexImage1D(value * argv,int argn)
{
	__gl_ml_glCopyTexImage1D(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6]);
}
extern void (APIENTRY *ov_glTexSubImage2D)(unsigned int target,signed int level,signed int xoffset,signed int yoffset,signed int width,signed int height,unsigned int format,unsigned int type,void* pixels);

static void APIENTRY init_ov_glTexSubImage2D(unsigned int target,signed int level,signed int xoffset,signed int yoffset,signed int width,signed int height,unsigned int format,unsigned int type,void* pixels)
{
	void *f;
	f=(void*)getfunc("glTexSubImage2D");
	ov_glTexSubImage2D=f;
	ov_glTexSubImage2D(target,level,xoffset,yoffset,width,height,format,type,pixels);
}
void (APIENTRY *ov_glTexSubImage2D)(unsigned int target,signed int level,signed int xoffset,signed int yoffset,signed int width,signed int height,unsigned int format,unsigned int type,void* pixels)=init_ov_glTexSubImage2D;

void __gl_ml_glTexSubImage2D(value v0,value v1,value v2,value v3,value v4,value v5,value v6,value v7,value v8)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	signed int l4=Int_val(v4);
	signed int l5=Int_val(v5);
	unsigned int l6=Int_val(v6);
	unsigned int l7=Int_val(v7);
	void* l8=gl__ba__(v8);
	ov_glTexSubImage2D(l0,l1,l2,l3,l4,l5,l6,l7,l8);
}

void byte___gl_ml_glTexSubImage2D(value * argv,int argn)
{
	__gl_ml_glTexSubImage2D(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6],argv[7],argv[8]);
}
extern void (APIENTRY *ov_glTexSubImage1D)(unsigned int target,signed int level,signed int xoffset,signed int width,unsigned int format,unsigned int type,void* pixels);

static void APIENTRY init_ov_glTexSubImage1D(unsigned int target,signed int level,signed int xoffset,signed int width,unsigned int format,unsigned int type,void* pixels)
{
	void *f;
	f=(void*)getfunc("glTexSubImage1D");
	ov_glTexSubImage1D=f;
	ov_glTexSubImage1D(target,level,xoffset,width,format,type,pixels);
}
void (APIENTRY *ov_glTexSubImage1D)(unsigned int target,signed int level,signed int xoffset,signed int width,unsigned int format,unsigned int type,void* pixels)=init_ov_glTexSubImage1D;

void __gl_ml_glTexSubImage1D(value v0,value v1,value v2,value v3,value v4,value v5,value v6)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	unsigned int l4=Int_val(v4);
	unsigned int l5=Int_val(v5);
	void* l6=gl__ba__(v6);
	ov_glTexSubImage1D(l0,l1,l2,l3,l4,l5,l6);
}

void byte___gl_ml_glTexSubImage1D(value * argv,int argn)
{
	__gl_ml_glTexSubImage1D(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6]);
}
extern unsigned char (APIENTRY *ov_glIsTexture)(unsigned int texture);

static unsigned char APIENTRY init_ov_glIsTexture(unsigned int texture)
{
	void *f;
	f=(void*)getfunc("glIsTexture");
	ov_glIsTexture=f;
	return ov_glIsTexture(texture);
}
unsigned char (APIENTRY *ov_glIsTexture)(unsigned int texture)=init_ov_glIsTexture;

value __gl_ml_glIsTexture(value v0)
{
	unsigned int l0=Int_val(v0);
	unsigned char r=ov_glIsTexture(l0);
	return(Val_int(r));
}
extern unsigned char (APIENTRY *ov_glAreTexturesResident)(signed int n,unsigned int* textures,unsigned char* residences);

static unsigned char APIENTRY init_ov_glAreTexturesResident(signed int n,unsigned int* textures,unsigned char* residences)
{
	void *f;
	f=(void*)getfunc("glAreTexturesResident");
	ov_glAreTexturesResident=f;
	return ov_glAreTexturesResident(n,textures,residences);
}
unsigned char (APIENTRY *ov_glAreTexturesResident)(signed int n,unsigned int* textures,unsigned char* residences)=init_ov_glAreTexturesResident;

value __gl_ml_glAreTexturesResident(value v0,value v1,value v2)
{
	signed int l0=Int_val(v0);
	unsigned int* l1=(unsigned int*)(gl__ba__(v1));
	unsigned char* l2=(unsigned char*)(gl__ba__(v2));
	unsigned char r=ov_glAreTexturesResident(l0,l1,l2);
	return(Val_int(r));
}
extern void (APIENTRY *ov_glPrioritizeTextures)(signed int n,unsigned int* textures,float* priorities);

static void APIENTRY init_ov_glPrioritizeTextures(signed int n,unsigned int* textures,float* priorities)
{
	void *f;
	f=(void*)getfunc("glPrioritizeTextures");
	ov_glPrioritizeTextures=f;
	ov_glPrioritizeTextures(n,textures,priorities);
}
void (APIENTRY *ov_glPrioritizeTextures)(signed int n,unsigned int* textures,float* priorities)=init_ov_glPrioritizeTextures;

void __gl_ml_glPrioritizeTextures(value v0,value v1,value v2)
{
	signed int l0=Int_val(v0);
	unsigned int* l1=(unsigned int*)(gl__ba__(v1));
	float* l2=(float*)(gl__ba__(v2));
	ov_glPrioritizeTextures(l0,l1,l2);
}
extern void (APIENTRY *ov_glBindTexture)(unsigned int target,unsigned int texture);

static void APIENTRY init_ov_glBindTexture(unsigned int target,unsigned int texture)
{
	void *f;
	f=(void*)getfunc("glBindTexture");
	ov_glBindTexture=f;
	ov_glBindTexture(target,texture);
}
void (APIENTRY *ov_glBindTexture)(unsigned int target,unsigned int texture)=init_ov_glBindTexture;

void __gl_ml_glBindTexture(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	ov_glBindTexture(l0,l1);
}
extern void (APIENTRY *ov_glDeleteTextures)(signed int n,unsigned int* textures);

static void APIENTRY init_ov_glDeleteTextures(signed int n,unsigned int* textures)
{
	void *f;
	f=(void*)getfunc("glDeleteTextures");
	ov_glDeleteTextures=f;
	ov_glDeleteTextures(n,textures);
}
void (APIENTRY *ov_glDeleteTextures)(signed int n,unsigned int* textures)=init_ov_glDeleteTextures;

void __gl_ml_glDeleteTextures(value v0,value v1)
{
	signed int l0=Int_val(v0);
	unsigned int* l1=(unsigned int*)(gl__ba__(v1));
	ov_glDeleteTextures(l0,l1);
}
extern void (APIENTRY *ov_glGenTextures)(signed int n,unsigned int* textures);

static void APIENTRY init_ov_glGenTextures(signed int n,unsigned int* textures)
{
	void *f;
	f=(void*)getfunc("glGenTextures");
	ov_glGenTextures=f;
	ov_glGenTextures(n,textures);
}
void (APIENTRY *ov_glGenTextures)(signed int n,unsigned int* textures)=init_ov_glGenTextures;

void __gl_ml_glGenTextures(value v0,value v1)
{
	signed int l0=Int_val(v0);
	unsigned int* l1=(unsigned int*)(gl__ba__(v1));
	ov_glGenTextures(l0,l1);
}
extern void (APIENTRY *ov_glInterleavedArrays)(unsigned int format,signed int stride,void* pointer);

static void APIENTRY init_ov_glInterleavedArrays(unsigned int format,signed int stride,void* pointer)
{
	void *f;
	f=(void*)getfunc("glInterleavedArrays");
	ov_glInterleavedArrays=f;
	ov_glInterleavedArrays(format,stride,pointer);
}
void (APIENTRY *ov_glInterleavedArrays)(unsigned int format,signed int stride,void* pointer)=init_ov_glInterleavedArrays;

void __gl_ml_glInterleavedArrays(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	void* l2=gl__ba__(v2);
	ov_glInterleavedArrays(l0,l1,l2);
}
extern void (APIENTRY *ov_glDrawElements)(unsigned int mode,signed int count,unsigned int type,void* indices);

static void APIENTRY init_ov_glDrawElements(unsigned int mode,signed int count,unsigned int type,void* indices)
{
	void *f;
	f=(void*)getfunc("glDrawElements");
	ov_glDrawElements=f;
	ov_glDrawElements(mode,count,type,indices);
}
void (APIENTRY *ov_glDrawElements)(unsigned int mode,signed int count,unsigned int type,void* indices)=init_ov_glDrawElements;

void __gl_ml_glDrawElements(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	void* l3=gl__ba__(v3);
	ov_glDrawElements(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glDrawArrays)(unsigned int mode,signed int first,signed int count);

static void APIENTRY init_ov_glDrawArrays(unsigned int mode,signed int first,signed int count)
{
	void *f;
	f=(void*)getfunc("glDrawArrays");
	ov_glDrawArrays=f;
	ov_glDrawArrays(mode,first,count);
}
void (APIENTRY *ov_glDrawArrays)(unsigned int mode,signed int first,signed int count)=init_ov_glDrawArrays;

void __gl_ml_glDrawArrays(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	ov_glDrawArrays(l0,l1,l2);
}
extern void (APIENTRY *ov_glArrayElement)(signed int i);

static void APIENTRY init_ov_glArrayElement(signed int i)
{
	void *f;
	f=(void*)getfunc("glArrayElement");
	ov_glArrayElement=f;
	ov_glArrayElement(i);
}
void (APIENTRY *ov_glArrayElement)(signed int i)=init_ov_glArrayElement;

void __gl_ml_glArrayElement(value v0)
{
	signed int l0=Int_val(v0);
	ov_glArrayElement(l0);
}
extern void (APIENTRY *ov_glGetPointerv)(unsigned int pname,void** params);

static void APIENTRY init_ov_glGetPointerv(unsigned int pname,void** params)
{
	void *f;
	f=(void*)getfunc("glGetPointerv");
	ov_glGetPointerv=f;
	ov_glGetPointerv(pname,params);
}
void (APIENTRY *ov_glGetPointerv)(unsigned int pname,void** params)=init_ov_glGetPointerv;
extern void (APIENTRY *ov_glEdgeFlagPointer)(signed int stride,void* ptr);

static void APIENTRY init_ov_glEdgeFlagPointer(signed int stride,void* ptr)
{
	void *f;
	f=(void*)getfunc("glEdgeFlagPointer");
	ov_glEdgeFlagPointer=f;
	ov_glEdgeFlagPointer(stride,ptr);
}
void (APIENTRY *ov_glEdgeFlagPointer)(signed int stride,void* ptr)=init_ov_glEdgeFlagPointer;

void __gl_ml_glEdgeFlagPointer(value v0,value v1)
{
	signed int l0=Int_val(v0);
	void* l1=gl__ba__(v1);
	ov_glEdgeFlagPointer(l0,l1);
}
extern void (APIENTRY *ov_glTexCoordPointer)(signed int size,unsigned int type,signed int stride,void* ptr);

static void APIENTRY init_ov_glTexCoordPointer(signed int size,unsigned int type,signed int stride,void* ptr)
{
	void *f;
	f=(void*)getfunc("glTexCoordPointer");
	ov_glTexCoordPointer=f;
	ov_glTexCoordPointer(size,type,stride,ptr);
}
void (APIENTRY *ov_glTexCoordPointer)(signed int size,unsigned int type,signed int stride,void* ptr)=init_ov_glTexCoordPointer;

void __gl_ml_glTexCoordPointer(value v0,value v1,value v2,value v3)
{
	signed int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	void* l3=gl__ba__(v3);
	ov_glTexCoordPointer(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glIndexPointer)(unsigned int type,signed int stride,void* ptr);

static void APIENTRY init_ov_glIndexPointer(unsigned int type,signed int stride,void* ptr)
{
	void *f;
	f=(void*)getfunc("glIndexPointer");
	ov_glIndexPointer=f;
	ov_glIndexPointer(type,stride,ptr);
}
void (APIENTRY *ov_glIndexPointer)(unsigned int type,signed int stride,void* ptr)=init_ov_glIndexPointer;

void __gl_ml_glIndexPointer(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	void* l2=gl__ba__(v2);
	ov_glIndexPointer(l0,l1,l2);
}
extern void (APIENTRY *ov_glColorPointer)(signed int size,unsigned int type,signed int stride,void* ptr);

static void APIENTRY init_ov_glColorPointer(signed int size,unsigned int type,signed int stride,void* ptr)
{
	void *f;
	f=(void*)getfunc("glColorPointer");
	ov_glColorPointer=f;
	ov_glColorPointer(size,type,stride,ptr);
}
void (APIENTRY *ov_glColorPointer)(signed int size,unsigned int type,signed int stride,void* ptr)=init_ov_glColorPointer;

void __gl_ml_glColorPointer(value v0,value v1,value v2,value v3)
{
	signed int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	void* l3=gl__ba__(v3);
	ov_glColorPointer(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glNormalPointer)(unsigned int type,signed int stride,void* ptr);

static void APIENTRY init_ov_glNormalPointer(unsigned int type,signed int stride,void* ptr)
{
	void *f;
	f=(void*)getfunc("glNormalPointer");
	ov_glNormalPointer=f;
	ov_glNormalPointer(type,stride,ptr);
}
void (APIENTRY *ov_glNormalPointer)(unsigned int type,signed int stride,void* ptr)=init_ov_glNormalPointer;

void __gl_ml_glNormalPointer(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	void* l2=gl__ba__(v2);
	ov_glNormalPointer(l0,l1,l2);
}
extern void (APIENTRY *ov_glVertexPointer)(signed int size,unsigned int type,signed int stride,void* ptr);

static void APIENTRY init_ov_glVertexPointer(signed int size,unsigned int type,signed int stride,void* ptr)
{
	void *f;
	f=(void*)getfunc("glVertexPointer");
	ov_glVertexPointer=f;
	ov_glVertexPointer(size,type,stride,ptr);
}
void (APIENTRY *ov_glVertexPointer)(signed int size,unsigned int type,signed int stride,void* ptr)=init_ov_glVertexPointer;

void __gl_ml_glVertexPointer(value v0,value v1,value v2,value v3)
{
	signed int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	void* l3=gl__ba__(v3);
	ov_glVertexPointer(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glGetCompressedTexImage)(unsigned int param0,signed int param1,void* param2);

static void APIENTRY init_ov_glGetCompressedTexImage(unsigned int param0,signed int param1,void* param2)
{
	void *f;
	f=(void*)getfunc("glGetCompressedTexImage");
	ov_glGetCompressedTexImage=f;
	ov_glGetCompressedTexImage(param0,param1,param2);
}
void (APIENTRY *ov_glGetCompressedTexImage)(unsigned int param0,signed int param1,void* param2)=init_ov_glGetCompressedTexImage;

void __gl_ml_glGetCompressedTexImage(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	void* l2=gl__ba__(v2);
	ov_glGetCompressedTexImage(l0,l1,l2);
}
extern void (APIENTRY *ov_glCompressedTexSubImage1D)(unsigned int param0,signed int param1,signed int param2,signed int param3,unsigned int param4,signed int param5,void* param6);

static void APIENTRY init_ov_glCompressedTexSubImage1D(unsigned int param0,signed int param1,signed int param2,signed int param3,unsigned int param4,signed int param5,void* param6)
{
	void *f;
	f=(void*)getfunc("glCompressedTexSubImage1D");
	ov_glCompressedTexSubImage1D=f;
	ov_glCompressedTexSubImage1D(param0,param1,param2,param3,param4,param5,param6);
}
void (APIENTRY *ov_glCompressedTexSubImage1D)(unsigned int param0,signed int param1,signed int param2,signed int param3,unsigned int param4,signed int param5,void* param6)=init_ov_glCompressedTexSubImage1D;

void __gl_ml_glCompressedTexSubImage1D(value v0,value v1,value v2,value v3,value v4,value v5,value v6)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	unsigned int l4=Int_val(v4);
	signed int l5=Int_val(v5);
	void* l6=gl__ba__(v6);
	ov_glCompressedTexSubImage1D(l0,l1,l2,l3,l4,l5,l6);
}

void byte___gl_ml_glCompressedTexSubImage1D(value * argv,int argn)
{
	__gl_ml_glCompressedTexSubImage1D(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6]);
}
extern void (APIENTRY *ov_glCompressedTexSubImage2D)(unsigned int param0,signed int param1,signed int param2,signed int param3,signed int param4,signed int param5,unsigned int param6,signed int param7,void* param8);

static void APIENTRY init_ov_glCompressedTexSubImage2D(unsigned int param0,signed int param1,signed int param2,signed int param3,signed int param4,signed int param5,unsigned int param6,signed int param7,void* param8)
{
	void *f;
	f=(void*)getfunc("glCompressedTexSubImage2D");
	ov_glCompressedTexSubImage2D=f;
	ov_glCompressedTexSubImage2D(param0,param1,param2,param3,param4,param5,param6,param7,param8);
}
void (APIENTRY *ov_glCompressedTexSubImage2D)(unsigned int param0,signed int param1,signed int param2,signed int param3,signed int param4,signed int param5,unsigned int param6,signed int param7,void* param8)=init_ov_glCompressedTexSubImage2D;

void __gl_ml_glCompressedTexSubImage2D(value v0,value v1,value v2,value v3,value v4,value v5,value v6,value v7,value v8)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	signed int l4=Int_val(v4);
	signed int l5=Int_val(v5);
	unsigned int l6=Int_val(v6);
	signed int l7=Int_val(v7);
	void* l8=gl__ba__(v8);
	ov_glCompressedTexSubImage2D(l0,l1,l2,l3,l4,l5,l6,l7,l8);
}

void byte___gl_ml_glCompressedTexSubImage2D(value * argv,int argn)
{
	__gl_ml_glCompressedTexSubImage2D(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6],argv[7],argv[8]);
}
extern void (APIENTRY *ov_glCompressedTexSubImage3D)(unsigned int param0,signed int param1,signed int param2,signed int param3,signed int param4,signed int param5,signed int param6,signed int param7,unsigned int param8,signed int param9,void* param10);

static void APIENTRY init_ov_glCompressedTexSubImage3D(unsigned int param0,signed int param1,signed int param2,signed int param3,signed int param4,signed int param5,signed int param6,signed int param7,unsigned int param8,signed int param9,void* param10)
{
	void *f;
	f=(void*)getfunc("glCompressedTexSubImage3D");
	ov_glCompressedTexSubImage3D=f;
	ov_glCompressedTexSubImage3D(param0,param1,param2,param3,param4,param5,param6,param7,param8,param9,param10);
}
void (APIENTRY *ov_glCompressedTexSubImage3D)(unsigned int param0,signed int param1,signed int param2,signed int param3,signed int param4,signed int param5,signed int param6,signed int param7,unsigned int param8,signed int param9,void* param10)=init_ov_glCompressedTexSubImage3D;

void __gl_ml_glCompressedTexSubImage3D(value v0,value v1,value v2,value v3,value v4,value v5,value v6,value v7,value v8,value v9,value v10)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	signed int l4=Int_val(v4);
	signed int l5=Int_val(v5);
	signed int l6=Int_val(v6);
	signed int l7=Int_val(v7);
	unsigned int l8=Int_val(v8);
	signed int l9=Int_val(v9);
	void* l10=gl__ba__(v10);
	ov_glCompressedTexSubImage3D(l0,l1,l2,l3,l4,l5,l6,l7,l8,l9,l10);
}

void byte___gl_ml_glCompressedTexSubImage3D(value * argv,int argn)
{
	__gl_ml_glCompressedTexSubImage3D(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6],argv[7],argv[8],argv[9],argv[10]);
}
extern void (APIENTRY *ov_glCompressedTexImage1D)(unsigned int param0,signed int param1,unsigned int param2,signed int param3,signed int param4,signed int param5,void* param6);

static void APIENTRY init_ov_glCompressedTexImage1D(unsigned int param0,signed int param1,unsigned int param2,signed int param3,signed int param4,signed int param5,void* param6)
{
	void *f;
	f=(void*)getfunc("glCompressedTexImage1D");
	ov_glCompressedTexImage1D=f;
	ov_glCompressedTexImage1D(param0,param1,param2,param3,param4,param5,param6);
}
void (APIENTRY *ov_glCompressedTexImage1D)(unsigned int param0,signed int param1,unsigned int param2,signed int param3,signed int param4,signed int param5,void* param6)=init_ov_glCompressedTexImage1D;

void __gl_ml_glCompressedTexImage1D(value v0,value v1,value v2,value v3,value v4,value v5,value v6)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	signed int l4=Int_val(v4);
	signed int l5=Int_val(v5);
	void* l6=gl__ba__(v6);
	ov_glCompressedTexImage1D(l0,l1,l2,l3,l4,l5,l6);
}

void byte___gl_ml_glCompressedTexImage1D(value * argv,int argn)
{
	__gl_ml_glCompressedTexImage1D(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6]);
}
extern void (APIENTRY *ov_glCompressedTexImage2D)(unsigned int param0,signed int param1,unsigned int param2,signed int param3,signed int param4,signed int param5,signed int param6,void* param7);

static void APIENTRY init_ov_glCompressedTexImage2D(unsigned int param0,signed int param1,unsigned int param2,signed int param3,signed int param4,signed int param5,signed int param6,void* param7)
{
	void *f;
	f=(void*)getfunc("glCompressedTexImage2D");
	ov_glCompressedTexImage2D=f;
	ov_glCompressedTexImage2D(param0,param1,param2,param3,param4,param5,param6,param7);
}
void (APIENTRY *ov_glCompressedTexImage2D)(unsigned int param0,signed int param1,unsigned int param2,signed int param3,signed int param4,signed int param5,signed int param6,void* param7)=init_ov_glCompressedTexImage2D;

void __gl_ml_glCompressedTexImage2D(value v0,value v1,value v2,value v3,value v4,value v5,value v6,value v7)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	signed int l4=Int_val(v4);
	signed int l5=Int_val(v5);
	signed int l6=Int_val(v6);
	void* l7=gl__ba__(v7);
	ov_glCompressedTexImage2D(l0,l1,l2,l3,l4,l5,l6,l7);
}

void byte___gl_ml_glCompressedTexImage2D(value * argv,int argn)
{
	__gl_ml_glCompressedTexImage2D(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6],argv[7]);
}
extern void (APIENTRY *ov_glCompressedTexImage3D)(unsigned int param0,signed int param1,unsigned int param2,signed int param3,signed int param4,signed int param5,signed int param6,signed int param7,void* param8);

static void APIENTRY init_ov_glCompressedTexImage3D(unsigned int param0,signed int param1,unsigned int param2,signed int param3,signed int param4,signed int param5,signed int param6,signed int param7,void* param8)
{
	void *f;
	f=(void*)getfunc("glCompressedTexImage3D");
	ov_glCompressedTexImage3D=f;
	ov_glCompressedTexImage3D(param0,param1,param2,param3,param4,param5,param6,param7,param8);
}
void (APIENTRY *ov_glCompressedTexImage3D)(unsigned int param0,signed int param1,unsigned int param2,signed int param3,signed int param4,signed int param5,signed int param6,signed int param7,void* param8)=init_ov_glCompressedTexImage3D;

void __gl_ml_glCompressedTexImage3D(value v0,value v1,value v2,value v3,value v4,value v5,value v6,value v7,value v8)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	signed int l4=Int_val(v4);
	signed int l5=Int_val(v5);
	signed int l6=Int_val(v6);
	signed int l7=Int_val(v7);
	void* l8=gl__ba__(v8);
	ov_glCompressedTexImage3D(l0,l1,l2,l3,l4,l5,l6,l7,l8);
}

void byte___gl_ml_glCompressedTexImage3D(value * argv,int argn)
{
	__gl_ml_glCompressedTexImage3D(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6],argv[7],argv[8]);
}
extern void (APIENTRY *ov_glSampleCoverage)(float param0,unsigned char param1);

static void APIENTRY init_ov_glSampleCoverage(float param0,unsigned char param1)
{
	void *f;
	f=(void*)getfunc("glSampleCoverage");
	ov_glSampleCoverage=f;
	ov_glSampleCoverage(param0,param1);
}
void (APIENTRY *ov_glSampleCoverage)(float param0,unsigned char param1)=init_ov_glSampleCoverage;

void __gl_ml_glSampleCoverage(value v0,value v1)
{
	float l0=Double_val(v0);
	unsigned char l1=Int_val(v1);
	ov_glSampleCoverage(l0,l1);
}
extern void (APIENTRY *ov_glMultTransposeMatrixd)(double* param0);

static void APIENTRY init_ov_glMultTransposeMatrixd(double* param0)
{
	void *f;
	f=(void*)getfunc("glMultTransposeMatrixd");
	ov_glMultTransposeMatrixd=f;
	ov_glMultTransposeMatrixd(param0);
}
void (APIENTRY *ov_glMultTransposeMatrixd)(double* param0)=init_ov_glMultTransposeMatrixd;

void __gl_ml_glMultTransposeMatrixd(value v0)
{
	double* l0=(double*)(gl__ba__(v0));
	ov_glMultTransposeMatrixd(l0);
}
extern void (APIENTRY *ov_glMultTransposeMatrixf)(float* param0);

static void APIENTRY init_ov_glMultTransposeMatrixf(float* param0)
{
	void *f;
	f=(void*)getfunc("glMultTransposeMatrixf");
	ov_glMultTransposeMatrixf=f;
	ov_glMultTransposeMatrixf(param0);
}
void (APIENTRY *ov_glMultTransposeMatrixf)(float* param0)=init_ov_glMultTransposeMatrixf;

void __gl_ml_glMultTransposeMatrixf(value v0)
{
	float* l0=(float*)(gl__ba__(v0));
	ov_glMultTransposeMatrixf(l0);
}
extern void (APIENTRY *ov_glLoadTransposeMatrixd)(double* param0);

static void APIENTRY init_ov_glLoadTransposeMatrixd(double* param0)
{
	void *f;
	f=(void*)getfunc("glLoadTransposeMatrixd");
	ov_glLoadTransposeMatrixd=f;
	ov_glLoadTransposeMatrixd(param0);
}
void (APIENTRY *ov_glLoadTransposeMatrixd)(double* param0)=init_ov_glLoadTransposeMatrixd;

void __gl_ml_glLoadTransposeMatrixd(value v0)
{
	double* l0=(double*)(gl__ba__(v0));
	ov_glLoadTransposeMatrixd(l0);
}
extern void (APIENTRY *ov_glLoadTransposeMatrixf)(float* param0);

static void APIENTRY init_ov_glLoadTransposeMatrixf(float* param0)
{
	void *f;
	f=(void*)getfunc("glLoadTransposeMatrixf");
	ov_glLoadTransposeMatrixf=f;
	ov_glLoadTransposeMatrixf(param0);
}
void (APIENTRY *ov_glLoadTransposeMatrixf)(float* param0)=init_ov_glLoadTransposeMatrixf;

void __gl_ml_glLoadTransposeMatrixf(value v0)
{
	float* l0=(float*)(gl__ba__(v0));
	ov_glLoadTransposeMatrixf(l0);
}
extern void (APIENTRY *ov_glMultiTexCoord4sv)(unsigned int param0,signed short* param1);

static void APIENTRY init_ov_glMultiTexCoord4sv(unsigned int param0,signed short* param1)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord4sv");
	ov_glMultiTexCoord4sv=f;
	ov_glMultiTexCoord4sv(param0,param1);
}
void (APIENTRY *ov_glMultiTexCoord4sv)(unsigned int param0,signed short* param1)=init_ov_glMultiTexCoord4sv;

void __gl_ml_glMultiTexCoord4sv(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed short* l1=(signed short*)(gl__ba__(v1));
	ov_glMultiTexCoord4sv(l0,l1);
}
extern void (APIENTRY *ov_glMultiTexCoord4s)(unsigned int param0,signed short param1,signed short param2,signed short param3,signed short param4);

static void APIENTRY init_ov_glMultiTexCoord4s(unsigned int param0,signed short param1,signed short param2,signed short param3,signed short param4)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord4s");
	ov_glMultiTexCoord4s=f;
	ov_glMultiTexCoord4s(param0,param1,param2,param3,param4);
}
void (APIENTRY *ov_glMultiTexCoord4s)(unsigned int param0,signed short param1,signed short param2,signed short param3,signed short param4)=init_ov_glMultiTexCoord4s;

void __gl_ml_glMultiTexCoord4s(value v0,value v1,value v2,value v3,value v4)
{
	unsigned int l0=Int_val(v0);
	signed short l1=Int_val(v1);
	signed short l2=Int_val(v2);
	signed short l3=Int_val(v3);
	signed short l4=Int_val(v4);
	ov_glMultiTexCoord4s(l0,l1,l2,l3,l4);
}
extern void (APIENTRY *ov_glMultiTexCoord4iv)(unsigned int param0,signed int* param1);

static void APIENTRY init_ov_glMultiTexCoord4iv(unsigned int param0,signed int* param1)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord4iv");
	ov_glMultiTexCoord4iv=f;
	ov_glMultiTexCoord4iv(param0,param1);
}
void (APIENTRY *ov_glMultiTexCoord4iv)(unsigned int param0,signed int* param1)=init_ov_glMultiTexCoord4iv;

void __gl_ml_glMultiTexCoord4iv(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed int* l1=(signed int*)(gl__ba__(v1));
	ov_glMultiTexCoord4iv(l0,l1);
}
extern void (APIENTRY *ov_glMultiTexCoord4i)(unsigned int param0,signed int param1,signed int param2,signed int param3,signed int param4);

static void APIENTRY init_ov_glMultiTexCoord4i(unsigned int param0,signed int param1,signed int param2,signed int param3,signed int param4)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord4i");
	ov_glMultiTexCoord4i=f;
	ov_glMultiTexCoord4i(param0,param1,param2,param3,param4);
}
void (APIENTRY *ov_glMultiTexCoord4i)(unsigned int param0,signed int param1,signed int param2,signed int param3,signed int param4)=init_ov_glMultiTexCoord4i;

void __gl_ml_glMultiTexCoord4i(value v0,value v1,value v2,value v3,value v4)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	signed int l4=Int_val(v4);
	ov_glMultiTexCoord4i(l0,l1,l2,l3,l4);
}
extern void (APIENTRY *ov_glMultiTexCoord4fv)(unsigned int param0,float* param1);

static void APIENTRY init_ov_glMultiTexCoord4fv(unsigned int param0,float* param1)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord4fv");
	ov_glMultiTexCoord4fv=f;
	ov_glMultiTexCoord4fv(param0,param1);
}
void (APIENTRY *ov_glMultiTexCoord4fv)(unsigned int param0,float* param1)=init_ov_glMultiTexCoord4fv;

void __gl_ml_glMultiTexCoord4fv(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float* l1=(float*)(gl__ba__(v1));
	ov_glMultiTexCoord4fv(l0,l1);
}
extern void (APIENTRY *ov_glMultiTexCoord4f)(unsigned int param0,float param1,float param2,float param3,float param4);

static void APIENTRY init_ov_glMultiTexCoord4f(unsigned int param0,float param1,float param2,float param3,float param4)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord4f");
	ov_glMultiTexCoord4f=f;
	ov_glMultiTexCoord4f(param0,param1,param2,param3,param4);
}
void (APIENTRY *ov_glMultiTexCoord4f)(unsigned int param0,float param1,float param2,float param3,float param4)=init_ov_glMultiTexCoord4f;

void __gl_ml_glMultiTexCoord4f(value v0,value v1,value v2,value v3,value v4)
{
	unsigned int l0=Int_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	float l3=Double_val(v3);
	float l4=Double_val(v4);
	ov_glMultiTexCoord4f(l0,l1,l2,l3,l4);
}
extern void (APIENTRY *ov_glMultiTexCoord4dv)(unsigned int param0,double* param1);

static void APIENTRY init_ov_glMultiTexCoord4dv(unsigned int param0,double* param1)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord4dv");
	ov_glMultiTexCoord4dv=f;
	ov_glMultiTexCoord4dv(param0,param1);
}
void (APIENTRY *ov_glMultiTexCoord4dv)(unsigned int param0,double* param1)=init_ov_glMultiTexCoord4dv;

void __gl_ml_glMultiTexCoord4dv(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	double* l1=(double*)(gl__ba__(v1));
	ov_glMultiTexCoord4dv(l0,l1);
}
extern void (APIENTRY *ov_glMultiTexCoord4d)(unsigned int param0,double param1,double param2,double param3,double param4);

static void APIENTRY init_ov_glMultiTexCoord4d(unsigned int param0,double param1,double param2,double param3,double param4)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord4d");
	ov_glMultiTexCoord4d=f;
	ov_glMultiTexCoord4d(param0,param1,param2,param3,param4);
}
void (APIENTRY *ov_glMultiTexCoord4d)(unsigned int param0,double param1,double param2,double param3,double param4)=init_ov_glMultiTexCoord4d;

void __gl_ml_glMultiTexCoord4d(value v0,value v1,value v2,value v3,value v4)
{
	unsigned int l0=Int_val(v0);
	double l1=Double_val(v1);
	double l2=Double_val(v2);
	double l3=Double_val(v3);
	double l4=Double_val(v4);
	ov_glMultiTexCoord4d(l0,l1,l2,l3,l4);
}
extern void (APIENTRY *ov_glMultiTexCoord3sv)(unsigned int param0,signed short* param1);

static void APIENTRY init_ov_glMultiTexCoord3sv(unsigned int param0,signed short* param1)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord3sv");
	ov_glMultiTexCoord3sv=f;
	ov_glMultiTexCoord3sv(param0,param1);
}
void (APIENTRY *ov_glMultiTexCoord3sv)(unsigned int param0,signed short* param1)=init_ov_glMultiTexCoord3sv;

void __gl_ml_glMultiTexCoord3sv(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed short* l1=(signed short*)(gl__ba__(v1));
	ov_glMultiTexCoord3sv(l0,l1);
}
extern void (APIENTRY *ov_glMultiTexCoord3s)(unsigned int param0,signed short param1,signed short param2,signed short param3);

static void APIENTRY init_ov_glMultiTexCoord3s(unsigned int param0,signed short param1,signed short param2,signed short param3)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord3s");
	ov_glMultiTexCoord3s=f;
	ov_glMultiTexCoord3s(param0,param1,param2,param3);
}
void (APIENTRY *ov_glMultiTexCoord3s)(unsigned int param0,signed short param1,signed short param2,signed short param3)=init_ov_glMultiTexCoord3s;

void __gl_ml_glMultiTexCoord3s(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	signed short l1=Int_val(v1);
	signed short l2=Int_val(v2);
	signed short l3=Int_val(v3);
	ov_glMultiTexCoord3s(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glMultiTexCoord3iv)(unsigned int param0,signed int* param1);

static void APIENTRY init_ov_glMultiTexCoord3iv(unsigned int param0,signed int* param1)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord3iv");
	ov_glMultiTexCoord3iv=f;
	ov_glMultiTexCoord3iv(param0,param1);
}
void (APIENTRY *ov_glMultiTexCoord3iv)(unsigned int param0,signed int* param1)=init_ov_glMultiTexCoord3iv;

void __gl_ml_glMultiTexCoord3iv(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed int* l1=(signed int*)(gl__ba__(v1));
	ov_glMultiTexCoord3iv(l0,l1);
}
extern void (APIENTRY *ov_glMultiTexCoord3i)(unsigned int param0,signed int param1,signed int param2,signed int param3);

static void APIENTRY init_ov_glMultiTexCoord3i(unsigned int param0,signed int param1,signed int param2,signed int param3)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord3i");
	ov_glMultiTexCoord3i=f;
	ov_glMultiTexCoord3i(param0,param1,param2,param3);
}
void (APIENTRY *ov_glMultiTexCoord3i)(unsigned int param0,signed int param1,signed int param2,signed int param3)=init_ov_glMultiTexCoord3i;

void __gl_ml_glMultiTexCoord3i(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	ov_glMultiTexCoord3i(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glMultiTexCoord3fv)(unsigned int param0,float* param1);

static void APIENTRY init_ov_glMultiTexCoord3fv(unsigned int param0,float* param1)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord3fv");
	ov_glMultiTexCoord3fv=f;
	ov_glMultiTexCoord3fv(param0,param1);
}
void (APIENTRY *ov_glMultiTexCoord3fv)(unsigned int param0,float* param1)=init_ov_glMultiTexCoord3fv;

void __gl_ml_glMultiTexCoord3fv(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float* l1=(float*)(gl__ba__(v1));
	ov_glMultiTexCoord3fv(l0,l1);
}
extern void (APIENTRY *ov_glMultiTexCoord3f)(unsigned int param0,float param1,float param2,float param3);

static void APIENTRY init_ov_glMultiTexCoord3f(unsigned int param0,float param1,float param2,float param3)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord3f");
	ov_glMultiTexCoord3f=f;
	ov_glMultiTexCoord3f(param0,param1,param2,param3);
}
void (APIENTRY *ov_glMultiTexCoord3f)(unsigned int param0,float param1,float param2,float param3)=init_ov_glMultiTexCoord3f;

void __gl_ml_glMultiTexCoord3f(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	float l3=Double_val(v3);
	ov_glMultiTexCoord3f(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glMultiTexCoord3dv)(unsigned int param0,double* param1);

static void APIENTRY init_ov_glMultiTexCoord3dv(unsigned int param0,double* param1)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord3dv");
	ov_glMultiTexCoord3dv=f;
	ov_glMultiTexCoord3dv(param0,param1);
}
void (APIENTRY *ov_glMultiTexCoord3dv)(unsigned int param0,double* param1)=init_ov_glMultiTexCoord3dv;

void __gl_ml_glMultiTexCoord3dv(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	double* l1=(double*)(gl__ba__(v1));
	ov_glMultiTexCoord3dv(l0,l1);
}
extern void (APIENTRY *ov_glMultiTexCoord3d)(unsigned int param0,double param1,double param2,double param3);

static void APIENTRY init_ov_glMultiTexCoord3d(unsigned int param0,double param1,double param2,double param3)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord3d");
	ov_glMultiTexCoord3d=f;
	ov_glMultiTexCoord3d(param0,param1,param2,param3);
}
void (APIENTRY *ov_glMultiTexCoord3d)(unsigned int param0,double param1,double param2,double param3)=init_ov_glMultiTexCoord3d;

void __gl_ml_glMultiTexCoord3d(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	double l1=Double_val(v1);
	double l2=Double_val(v2);
	double l3=Double_val(v3);
	ov_glMultiTexCoord3d(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glMultiTexCoord2sv)(unsigned int param0,signed short* param1);

static void APIENTRY init_ov_glMultiTexCoord2sv(unsigned int param0,signed short* param1)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord2sv");
	ov_glMultiTexCoord2sv=f;
	ov_glMultiTexCoord2sv(param0,param1);
}
void (APIENTRY *ov_glMultiTexCoord2sv)(unsigned int param0,signed short* param1)=init_ov_glMultiTexCoord2sv;

void __gl_ml_glMultiTexCoord2sv(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed short* l1=(signed short*)(gl__ba__(v1));
	ov_glMultiTexCoord2sv(l0,l1);
}
extern void (APIENTRY *ov_glMultiTexCoord2s)(unsigned int param0,signed short param1,signed short param2);

static void APIENTRY init_ov_glMultiTexCoord2s(unsigned int param0,signed short param1,signed short param2)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord2s");
	ov_glMultiTexCoord2s=f;
	ov_glMultiTexCoord2s(param0,param1,param2);
}
void (APIENTRY *ov_glMultiTexCoord2s)(unsigned int param0,signed short param1,signed short param2)=init_ov_glMultiTexCoord2s;

void __gl_ml_glMultiTexCoord2s(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	signed short l1=Int_val(v1);
	signed short l2=Int_val(v2);
	ov_glMultiTexCoord2s(l0,l1,l2);
}
extern void (APIENTRY *ov_glMultiTexCoord2iv)(unsigned int param0,signed int* param1);

static void APIENTRY init_ov_glMultiTexCoord2iv(unsigned int param0,signed int* param1)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord2iv");
	ov_glMultiTexCoord2iv=f;
	ov_glMultiTexCoord2iv(param0,param1);
}
void (APIENTRY *ov_glMultiTexCoord2iv)(unsigned int param0,signed int* param1)=init_ov_glMultiTexCoord2iv;

void __gl_ml_glMultiTexCoord2iv(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed int* l1=(signed int*)(gl__ba__(v1));
	ov_glMultiTexCoord2iv(l0,l1);
}
extern void (APIENTRY *ov_glMultiTexCoord2i)(unsigned int param0,signed int param1,signed int param2);

static void APIENTRY init_ov_glMultiTexCoord2i(unsigned int param0,signed int param1,signed int param2)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord2i");
	ov_glMultiTexCoord2i=f;
	ov_glMultiTexCoord2i(param0,param1,param2);
}
void (APIENTRY *ov_glMultiTexCoord2i)(unsigned int param0,signed int param1,signed int param2)=init_ov_glMultiTexCoord2i;

void __gl_ml_glMultiTexCoord2i(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	ov_glMultiTexCoord2i(l0,l1,l2);
}
extern void (APIENTRY *ov_glMultiTexCoord2fv)(unsigned int param0,float* param1);

static void APIENTRY init_ov_glMultiTexCoord2fv(unsigned int param0,float* param1)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord2fv");
	ov_glMultiTexCoord2fv=f;
	ov_glMultiTexCoord2fv(param0,param1);
}
void (APIENTRY *ov_glMultiTexCoord2fv)(unsigned int param0,float* param1)=init_ov_glMultiTexCoord2fv;

void __gl_ml_glMultiTexCoord2fv(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float* l1=(float*)(gl__ba__(v1));
	ov_glMultiTexCoord2fv(l0,l1);
}
extern void (APIENTRY *ov_glMultiTexCoord2f)(unsigned int param0,float param1,float param2);

static void APIENTRY init_ov_glMultiTexCoord2f(unsigned int param0,float param1,float param2)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord2f");
	ov_glMultiTexCoord2f=f;
	ov_glMultiTexCoord2f(param0,param1,param2);
}
void (APIENTRY *ov_glMultiTexCoord2f)(unsigned int param0,float param1,float param2)=init_ov_glMultiTexCoord2f;

void __gl_ml_glMultiTexCoord2f(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	ov_glMultiTexCoord2f(l0,l1,l2);
}
extern void (APIENTRY *ov_glMultiTexCoord2dv)(unsigned int param0,double* param1);

static void APIENTRY init_ov_glMultiTexCoord2dv(unsigned int param0,double* param1)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord2dv");
	ov_glMultiTexCoord2dv=f;
	ov_glMultiTexCoord2dv(param0,param1);
}
void (APIENTRY *ov_glMultiTexCoord2dv)(unsigned int param0,double* param1)=init_ov_glMultiTexCoord2dv;

void __gl_ml_glMultiTexCoord2dv(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	double* l1=(double*)(gl__ba__(v1));
	ov_glMultiTexCoord2dv(l0,l1);
}
extern void (APIENTRY *ov_glMultiTexCoord2d)(unsigned int param0,double param1,double param2);

static void APIENTRY init_ov_glMultiTexCoord2d(unsigned int param0,double param1,double param2)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord2d");
	ov_glMultiTexCoord2d=f;
	ov_glMultiTexCoord2d(param0,param1,param2);
}
void (APIENTRY *ov_glMultiTexCoord2d)(unsigned int param0,double param1,double param2)=init_ov_glMultiTexCoord2d;

void __gl_ml_glMultiTexCoord2d(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	double l1=Double_val(v1);
	double l2=Double_val(v2);
	ov_glMultiTexCoord2d(l0,l1,l2);
}
extern void (APIENTRY *ov_glMultiTexCoord1sv)(unsigned int param0,signed short* param1);

static void APIENTRY init_ov_glMultiTexCoord1sv(unsigned int param0,signed short* param1)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord1sv");
	ov_glMultiTexCoord1sv=f;
	ov_glMultiTexCoord1sv(param0,param1);
}
void (APIENTRY *ov_glMultiTexCoord1sv)(unsigned int param0,signed short* param1)=init_ov_glMultiTexCoord1sv;

void __gl_ml_glMultiTexCoord1sv(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed short* l1=(signed short*)(gl__ba__(v1));
	ov_glMultiTexCoord1sv(l0,l1);
}
extern void (APIENTRY *ov_glMultiTexCoord1s)(unsigned int param0,signed short param1);

static void APIENTRY init_ov_glMultiTexCoord1s(unsigned int param0,signed short param1)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord1s");
	ov_glMultiTexCoord1s=f;
	ov_glMultiTexCoord1s(param0,param1);
}
void (APIENTRY *ov_glMultiTexCoord1s)(unsigned int param0,signed short param1)=init_ov_glMultiTexCoord1s;

void __gl_ml_glMultiTexCoord1s(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed short l1=Int_val(v1);
	ov_glMultiTexCoord1s(l0,l1);
}
extern void (APIENTRY *ov_glMultiTexCoord1iv)(unsigned int param0,signed int* param1);

static void APIENTRY init_ov_glMultiTexCoord1iv(unsigned int param0,signed int* param1)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord1iv");
	ov_glMultiTexCoord1iv=f;
	ov_glMultiTexCoord1iv(param0,param1);
}
void (APIENTRY *ov_glMultiTexCoord1iv)(unsigned int param0,signed int* param1)=init_ov_glMultiTexCoord1iv;

void __gl_ml_glMultiTexCoord1iv(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed int* l1=(signed int*)(gl__ba__(v1));
	ov_glMultiTexCoord1iv(l0,l1);
}
extern void (APIENTRY *ov_glMultiTexCoord1i)(unsigned int param0,signed int param1);

static void APIENTRY init_ov_glMultiTexCoord1i(unsigned int param0,signed int param1)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord1i");
	ov_glMultiTexCoord1i=f;
	ov_glMultiTexCoord1i(param0,param1);
}
void (APIENTRY *ov_glMultiTexCoord1i)(unsigned int param0,signed int param1)=init_ov_glMultiTexCoord1i;

void __gl_ml_glMultiTexCoord1i(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	ov_glMultiTexCoord1i(l0,l1);
}
extern void (APIENTRY *ov_glMultiTexCoord1fv)(unsigned int param0,float* param1);

static void APIENTRY init_ov_glMultiTexCoord1fv(unsigned int param0,float* param1)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord1fv");
	ov_glMultiTexCoord1fv=f;
	ov_glMultiTexCoord1fv(param0,param1);
}
void (APIENTRY *ov_glMultiTexCoord1fv)(unsigned int param0,float* param1)=init_ov_glMultiTexCoord1fv;

void __gl_ml_glMultiTexCoord1fv(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float* l1=(float*)(gl__ba__(v1));
	ov_glMultiTexCoord1fv(l0,l1);
}
extern void (APIENTRY *ov_glMultiTexCoord1f)(unsigned int param0,float param1);

static void APIENTRY init_ov_glMultiTexCoord1f(unsigned int param0,float param1)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord1f");
	ov_glMultiTexCoord1f=f;
	ov_glMultiTexCoord1f(param0,param1);
}
void (APIENTRY *ov_glMultiTexCoord1f)(unsigned int param0,float param1)=init_ov_glMultiTexCoord1f;

void __gl_ml_glMultiTexCoord1f(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float l1=Double_val(v1);
	ov_glMultiTexCoord1f(l0,l1);
}
extern void (APIENTRY *ov_glMultiTexCoord1dv)(unsigned int param0,double* param1);

static void APIENTRY init_ov_glMultiTexCoord1dv(unsigned int param0,double* param1)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord1dv");
	ov_glMultiTexCoord1dv=f;
	ov_glMultiTexCoord1dv(param0,param1);
}
void (APIENTRY *ov_glMultiTexCoord1dv)(unsigned int param0,double* param1)=init_ov_glMultiTexCoord1dv;

void __gl_ml_glMultiTexCoord1dv(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	double* l1=(double*)(gl__ba__(v1));
	ov_glMultiTexCoord1dv(l0,l1);
}
extern void (APIENTRY *ov_glMultiTexCoord1d)(unsigned int param0,double param1);

static void APIENTRY init_ov_glMultiTexCoord1d(unsigned int param0,double param1)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord1d");
	ov_glMultiTexCoord1d=f;
	ov_glMultiTexCoord1d(param0,param1);
}
void (APIENTRY *ov_glMultiTexCoord1d)(unsigned int param0,double param1)=init_ov_glMultiTexCoord1d;

void __gl_ml_glMultiTexCoord1d(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	double l1=Double_val(v1);
	ov_glMultiTexCoord1d(l0,l1);
}
extern void (APIENTRY *ov_glClientActiveTexture)(unsigned int param0);

static void APIENTRY init_ov_glClientActiveTexture(unsigned int param0)
{
	void *f;
	f=(void*)getfunc("glClientActiveTexture");
	ov_glClientActiveTexture=f;
	ov_glClientActiveTexture(param0);
}
void (APIENTRY *ov_glClientActiveTexture)(unsigned int param0)=init_ov_glClientActiveTexture;

void __gl_ml_glClientActiveTexture(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glClientActiveTexture(l0);
}
extern void (APIENTRY *ov_glActiveTexture)(unsigned int param0);

static void APIENTRY init_ov_glActiveTexture(unsigned int param0)
{
	void *f;
	f=(void*)getfunc("glActiveTexture");
	ov_glActiveTexture=f;
	ov_glActiveTexture(param0);
}
void (APIENTRY *ov_glActiveTexture)(unsigned int param0)=init_ov_glActiveTexture;

void __gl_ml_glActiveTexture(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glActiveTexture(l0);
}
extern void (APIENTRY *ov_glWindowPos3sv)(signed short* param0);

static void APIENTRY init_ov_glWindowPos3sv(signed short* param0)
{
	void *f;
	f=(void*)getfunc("glWindowPos3sv");
	ov_glWindowPos3sv=f;
	ov_glWindowPos3sv(param0);
}
void (APIENTRY *ov_glWindowPos3sv)(signed short* param0)=init_ov_glWindowPos3sv;

void __gl_ml_glWindowPos3sv(value v0)
{
	signed short* l0=(signed short*)(gl__ba__(v0));
	ov_glWindowPos3sv(l0);
}
extern void (APIENTRY *ov_glWindowPos3s)(signed short param0,signed short param1,signed short param2);

static void APIENTRY init_ov_glWindowPos3s(signed short param0,signed short param1,signed short param2)
{
	void *f;
	f=(void*)getfunc("glWindowPos3s");
	ov_glWindowPos3s=f;
	ov_glWindowPos3s(param0,param1,param2);
}
void (APIENTRY *ov_glWindowPos3s)(signed short param0,signed short param1,signed short param2)=init_ov_glWindowPos3s;

void __gl_ml_glWindowPos3s(value v0,value v1,value v2)
{
	signed short l0=Int_val(v0);
	signed short l1=Int_val(v1);
	signed short l2=Int_val(v2);
	ov_glWindowPos3s(l0,l1,l2);
}
extern void (APIENTRY *ov_glWindowPos3iv)(signed int* param0);

static void APIENTRY init_ov_glWindowPos3iv(signed int* param0)
{
	void *f;
	f=(void*)getfunc("glWindowPos3iv");
	ov_glWindowPos3iv=f;
	ov_glWindowPos3iv(param0);
}
void (APIENTRY *ov_glWindowPos3iv)(signed int* param0)=init_ov_glWindowPos3iv;

void __gl_ml_glWindowPos3iv(value v0)
{
	signed int* l0=(signed int*)(gl__ba__(v0));
	ov_glWindowPos3iv(l0);
}
extern void (APIENTRY *ov_glWindowPos3i)(signed int param0,signed int param1,signed int param2);

static void APIENTRY init_ov_glWindowPos3i(signed int param0,signed int param1,signed int param2)
{
	void *f;
	f=(void*)getfunc("glWindowPos3i");
	ov_glWindowPos3i=f;
	ov_glWindowPos3i(param0,param1,param2);
}
void (APIENTRY *ov_glWindowPos3i)(signed int param0,signed int param1,signed int param2)=init_ov_glWindowPos3i;

void __gl_ml_glWindowPos3i(value v0,value v1,value v2)
{
	signed int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	ov_glWindowPos3i(l0,l1,l2);
}
extern void (APIENTRY *ov_glWindowPos3fv)(float* param0);

static void APIENTRY init_ov_glWindowPos3fv(float* param0)
{
	void *f;
	f=(void*)getfunc("glWindowPos3fv");
	ov_glWindowPos3fv=f;
	ov_glWindowPos3fv(param0);
}
void (APIENTRY *ov_glWindowPos3fv)(float* param0)=init_ov_glWindowPos3fv;

void __gl_ml_glWindowPos3fv(value v0)
{
	float* l0=(float*)(gl__ba__(v0));
	ov_glWindowPos3fv(l0);
}
extern void (APIENTRY *ov_glWindowPos3f)(float param0,float param1,float param2);

static void APIENTRY init_ov_glWindowPos3f(float param0,float param1,float param2)
{
	void *f;
	f=(void*)getfunc("glWindowPos3f");
	ov_glWindowPos3f=f;
	ov_glWindowPos3f(param0,param1,param2);
}
void (APIENTRY *ov_glWindowPos3f)(float param0,float param1,float param2)=init_ov_glWindowPos3f;

void __gl_ml_glWindowPos3f(value v0,value v1,value v2)
{
	float l0=Double_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	ov_glWindowPos3f(l0,l1,l2);
}
extern void (APIENTRY *ov_glWindowPos3dv)(double* param0);

static void APIENTRY init_ov_glWindowPos3dv(double* param0)
{
	void *f;
	f=(void*)getfunc("glWindowPos3dv");
	ov_glWindowPos3dv=f;
	ov_glWindowPos3dv(param0);
}
void (APIENTRY *ov_glWindowPos3dv)(double* param0)=init_ov_glWindowPos3dv;

void __gl_ml_glWindowPos3dv(value v0)
{
	double* l0=(double*)(gl__ba__(v0));
	ov_glWindowPos3dv(l0);
}
extern void (APIENTRY *ov_glWindowPos3d)(double param0,double param1,double param2);

static void APIENTRY init_ov_glWindowPos3d(double param0,double param1,double param2)
{
	void *f;
	f=(void*)getfunc("glWindowPos3d");
	ov_glWindowPos3d=f;
	ov_glWindowPos3d(param0,param1,param2);
}
void (APIENTRY *ov_glWindowPos3d)(double param0,double param1,double param2)=init_ov_glWindowPos3d;

void __gl_ml_glWindowPos3d(value v0,value v1,value v2)
{
	double l0=Double_val(v0);
	double l1=Double_val(v1);
	double l2=Double_val(v2);
	ov_glWindowPos3d(l0,l1,l2);
}
extern void (APIENTRY *ov_glWindowPos2sv)(signed short* param0);

static void APIENTRY init_ov_glWindowPos2sv(signed short* param0)
{
	void *f;
	f=(void*)getfunc("glWindowPos2sv");
	ov_glWindowPos2sv=f;
	ov_glWindowPos2sv(param0);
}
void (APIENTRY *ov_glWindowPos2sv)(signed short* param0)=init_ov_glWindowPos2sv;

void __gl_ml_glWindowPos2sv(value v0)
{
	signed short* l0=(signed short*)(gl__ba__(v0));
	ov_glWindowPos2sv(l0);
}
extern void (APIENTRY *ov_glWindowPos2s)(signed short param0,signed short param1);

static void APIENTRY init_ov_glWindowPos2s(signed short param0,signed short param1)
{
	void *f;
	f=(void*)getfunc("glWindowPos2s");
	ov_glWindowPos2s=f;
	ov_glWindowPos2s(param0,param1);
}
void (APIENTRY *ov_glWindowPos2s)(signed short param0,signed short param1)=init_ov_glWindowPos2s;

void __gl_ml_glWindowPos2s(value v0,value v1)
{
	signed short l0=Int_val(v0);
	signed short l1=Int_val(v1);
	ov_glWindowPos2s(l0,l1);
}
extern void (APIENTRY *ov_glWindowPos2iv)(signed int* param0);

static void APIENTRY init_ov_glWindowPos2iv(signed int* param0)
{
	void *f;
	f=(void*)getfunc("glWindowPos2iv");
	ov_glWindowPos2iv=f;
	ov_glWindowPos2iv(param0);
}
void (APIENTRY *ov_glWindowPos2iv)(signed int* param0)=init_ov_glWindowPos2iv;

void __gl_ml_glWindowPos2iv(value v0)
{
	signed int* l0=(signed int*)(gl__ba__(v0));
	ov_glWindowPos2iv(l0);
}
extern void (APIENTRY *ov_glWindowPos2i)(signed int param0,signed int param1);

static void APIENTRY init_ov_glWindowPos2i(signed int param0,signed int param1)
{
	void *f;
	f=(void*)getfunc("glWindowPos2i");
	ov_glWindowPos2i=f;
	ov_glWindowPos2i(param0,param1);
}
void (APIENTRY *ov_glWindowPos2i)(signed int param0,signed int param1)=init_ov_glWindowPos2i;

void __gl_ml_glWindowPos2i(value v0,value v1)
{
	signed int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	ov_glWindowPos2i(l0,l1);
}
extern void (APIENTRY *ov_glWindowPos2fv)(float* param0);

static void APIENTRY init_ov_glWindowPos2fv(float* param0)
{
	void *f;
	f=(void*)getfunc("glWindowPos2fv");
	ov_glWindowPos2fv=f;
	ov_glWindowPos2fv(param0);
}
void (APIENTRY *ov_glWindowPos2fv)(float* param0)=init_ov_glWindowPos2fv;

void __gl_ml_glWindowPos2fv(value v0)
{
	float* l0=(float*)(gl__ba__(v0));
	ov_glWindowPos2fv(l0);
}
extern void (APIENTRY *ov_glWindowPos2f)(float param0,float param1);

static void APIENTRY init_ov_glWindowPos2f(float param0,float param1)
{
	void *f;
	f=(void*)getfunc("glWindowPos2f");
	ov_glWindowPos2f=f;
	ov_glWindowPos2f(param0,param1);
}
void (APIENTRY *ov_glWindowPos2f)(float param0,float param1)=init_ov_glWindowPos2f;

void __gl_ml_glWindowPos2f(value v0,value v1)
{
	float l0=Double_val(v0);
	float l1=Double_val(v1);
	ov_glWindowPos2f(l0,l1);
}
extern void (APIENTRY *ov_glWindowPos2dv)(double* param0);

static void APIENTRY init_ov_glWindowPos2dv(double* param0)
{
	void *f;
	f=(void*)getfunc("glWindowPos2dv");
	ov_glWindowPos2dv=f;
	ov_glWindowPos2dv(param0);
}
void (APIENTRY *ov_glWindowPos2dv)(double* param0)=init_ov_glWindowPos2dv;

void __gl_ml_glWindowPos2dv(value v0)
{
	double* l0=(double*)(gl__ba__(v0));
	ov_glWindowPos2dv(l0);
}
extern void (APIENTRY *ov_glWindowPos2d)(double param0,double param1);

static void APIENTRY init_ov_glWindowPos2d(double param0,double param1)
{
	void *f;
	f=(void*)getfunc("glWindowPos2d");
	ov_glWindowPos2d=f;
	ov_glWindowPos2d(param0,param1);
}
void (APIENTRY *ov_glWindowPos2d)(double param0,double param1)=init_ov_glWindowPos2d;

void __gl_ml_glWindowPos2d(value v0,value v1)
{
	double l0=Double_val(v0);
	double l1=Double_val(v1);
	ov_glWindowPos2d(l0,l1);
}
extern void (APIENTRY *ov_glSecondaryColorPointer)(signed int param0,unsigned int param1,signed int param2,void* param3);

static void APIENTRY init_ov_glSecondaryColorPointer(signed int param0,unsigned int param1,signed int param2,void* param3)
{
	void *f;
	f=(void*)getfunc("glSecondaryColorPointer");
	ov_glSecondaryColorPointer=f;
	ov_glSecondaryColorPointer(param0,param1,param2,param3);
}
void (APIENTRY *ov_glSecondaryColorPointer)(signed int param0,unsigned int param1,signed int param2,void* param3)=init_ov_glSecondaryColorPointer;

void __gl_ml_glSecondaryColorPointer(value v0,value v1,value v2,value v3)
{
	signed int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	void* l3=gl__ba__(v3);
	ov_glSecondaryColorPointer(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glSecondaryColor3usv)(unsigned short* param0);

static void APIENTRY init_ov_glSecondaryColor3usv(unsigned short* param0)
{
	void *f;
	f=(void*)getfunc("glSecondaryColor3usv");
	ov_glSecondaryColor3usv=f;
	ov_glSecondaryColor3usv(param0);
}
void (APIENTRY *ov_glSecondaryColor3usv)(unsigned short* param0)=init_ov_glSecondaryColor3usv;

void __gl_ml_glSecondaryColor3usv(value v0)
{
	unsigned short* l0=(unsigned short*)(gl__ba__(v0));
	ov_glSecondaryColor3usv(l0);
}
extern void (APIENTRY *ov_glSecondaryColor3us)(unsigned short param0,unsigned short param1,unsigned short param2);

static void APIENTRY init_ov_glSecondaryColor3us(unsigned short param0,unsigned short param1,unsigned short param2)
{
	void *f;
	f=(void*)getfunc("glSecondaryColor3us");
	ov_glSecondaryColor3us=f;
	ov_glSecondaryColor3us(param0,param1,param2);
}
void (APIENTRY *ov_glSecondaryColor3us)(unsigned short param0,unsigned short param1,unsigned short param2)=init_ov_glSecondaryColor3us;

void __gl_ml_glSecondaryColor3us(value v0,value v1,value v2)
{
	unsigned short l0=Int_val(v0);
	unsigned short l1=Int_val(v1);
	unsigned short l2=Int_val(v2);
	ov_glSecondaryColor3us(l0,l1,l2);
}
extern void (APIENTRY *ov_glSecondaryColor3uiv)(unsigned int* param0);

static void APIENTRY init_ov_glSecondaryColor3uiv(unsigned int* param0)
{
	void *f;
	f=(void*)getfunc("glSecondaryColor3uiv");
	ov_glSecondaryColor3uiv=f;
	ov_glSecondaryColor3uiv(param0);
}
void (APIENTRY *ov_glSecondaryColor3uiv)(unsigned int* param0)=init_ov_glSecondaryColor3uiv;

void __gl_ml_glSecondaryColor3uiv(value v0)
{
	unsigned int* l0=(unsigned int*)(gl__ba__(v0));
	ov_glSecondaryColor3uiv(l0);
}
extern void (APIENTRY *ov_glSecondaryColor3ui)(unsigned int param0,unsigned int param1,unsigned int param2);

static void APIENTRY init_ov_glSecondaryColor3ui(unsigned int param0,unsigned int param1,unsigned int param2)
{
	void *f;
	f=(void*)getfunc("glSecondaryColor3ui");
	ov_glSecondaryColor3ui=f;
	ov_glSecondaryColor3ui(param0,param1,param2);
}
void (APIENTRY *ov_glSecondaryColor3ui)(unsigned int param0,unsigned int param1,unsigned int param2)=init_ov_glSecondaryColor3ui;

void __gl_ml_glSecondaryColor3ui(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	ov_glSecondaryColor3ui(l0,l1,l2);
}
extern void (APIENTRY *ov_glSecondaryColor3ubv)(unsigned char* param0);

static void APIENTRY init_ov_glSecondaryColor3ubv(unsigned char* param0)
{
	void *f;
	f=(void*)getfunc("glSecondaryColor3ubv");
	ov_glSecondaryColor3ubv=f;
	ov_glSecondaryColor3ubv(param0);
}
void (APIENTRY *ov_glSecondaryColor3ubv)(unsigned char* param0)=init_ov_glSecondaryColor3ubv;

void __gl_ml_glSecondaryColor3ubv(value v0)
{
	unsigned char* l0=(unsigned char*)(gl__ba__(v0));
	ov_glSecondaryColor3ubv(l0);
}
extern void (APIENTRY *ov_glSecondaryColor3ub)(unsigned char param0,unsigned char param1,unsigned char param2);

static void APIENTRY init_ov_glSecondaryColor3ub(unsigned char param0,unsigned char param1,unsigned char param2)
{
	void *f;
	f=(void*)getfunc("glSecondaryColor3ub");
	ov_glSecondaryColor3ub=f;
	ov_glSecondaryColor3ub(param0,param1,param2);
}
void (APIENTRY *ov_glSecondaryColor3ub)(unsigned char param0,unsigned char param1,unsigned char param2)=init_ov_glSecondaryColor3ub;

void __gl_ml_glSecondaryColor3ub(value v0,value v1,value v2)
{
	unsigned char l0=Int_val(v0);
	unsigned char l1=Int_val(v1);
	unsigned char l2=Int_val(v2);
	ov_glSecondaryColor3ub(l0,l1,l2);
}
extern void (APIENTRY *ov_glSecondaryColor3sv)(signed short* param0);

static void APIENTRY init_ov_glSecondaryColor3sv(signed short* param0)
{
	void *f;
	f=(void*)getfunc("glSecondaryColor3sv");
	ov_glSecondaryColor3sv=f;
	ov_glSecondaryColor3sv(param0);
}
void (APIENTRY *ov_glSecondaryColor3sv)(signed short* param0)=init_ov_glSecondaryColor3sv;

void __gl_ml_glSecondaryColor3sv(value v0)
{
	signed short* l0=(signed short*)(gl__ba__(v0));
	ov_glSecondaryColor3sv(l0);
}
extern void (APIENTRY *ov_glSecondaryColor3s)(signed short param0,signed short param1,signed short param2);

static void APIENTRY init_ov_glSecondaryColor3s(signed short param0,signed short param1,signed short param2)
{
	void *f;
	f=(void*)getfunc("glSecondaryColor3s");
	ov_glSecondaryColor3s=f;
	ov_glSecondaryColor3s(param0,param1,param2);
}
void (APIENTRY *ov_glSecondaryColor3s)(signed short param0,signed short param1,signed short param2)=init_ov_glSecondaryColor3s;

void __gl_ml_glSecondaryColor3s(value v0,value v1,value v2)
{
	signed short l0=Int_val(v0);
	signed short l1=Int_val(v1);
	signed short l2=Int_val(v2);
	ov_glSecondaryColor3s(l0,l1,l2);
}
extern void (APIENTRY *ov_glSecondaryColor3iv)(signed int* param0);

static void APIENTRY init_ov_glSecondaryColor3iv(signed int* param0)
{
	void *f;
	f=(void*)getfunc("glSecondaryColor3iv");
	ov_glSecondaryColor3iv=f;
	ov_glSecondaryColor3iv(param0);
}
void (APIENTRY *ov_glSecondaryColor3iv)(signed int* param0)=init_ov_glSecondaryColor3iv;

void __gl_ml_glSecondaryColor3iv(value v0)
{
	signed int* l0=(signed int*)(gl__ba__(v0));
	ov_glSecondaryColor3iv(l0);
}
extern void (APIENTRY *ov_glSecondaryColor3i)(signed int param0,signed int param1,signed int param2);

static void APIENTRY init_ov_glSecondaryColor3i(signed int param0,signed int param1,signed int param2)
{
	void *f;
	f=(void*)getfunc("glSecondaryColor3i");
	ov_glSecondaryColor3i=f;
	ov_glSecondaryColor3i(param0,param1,param2);
}
void (APIENTRY *ov_glSecondaryColor3i)(signed int param0,signed int param1,signed int param2)=init_ov_glSecondaryColor3i;

void __gl_ml_glSecondaryColor3i(value v0,value v1,value v2)
{
	signed int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	ov_glSecondaryColor3i(l0,l1,l2);
}
extern void (APIENTRY *ov_glSecondaryColor3fv)(float* param0);

static void APIENTRY init_ov_glSecondaryColor3fv(float* param0)
{
	void *f;
	f=(void*)getfunc("glSecondaryColor3fv");
	ov_glSecondaryColor3fv=f;
	ov_glSecondaryColor3fv(param0);
}
void (APIENTRY *ov_glSecondaryColor3fv)(float* param0)=init_ov_glSecondaryColor3fv;

void __gl_ml_glSecondaryColor3fv(value v0)
{
	float* l0=(float*)(gl__ba__(v0));
	ov_glSecondaryColor3fv(l0);
}
extern void (APIENTRY *ov_glSecondaryColor3f)(float param0,float param1,float param2);

static void APIENTRY init_ov_glSecondaryColor3f(float param0,float param1,float param2)
{
	void *f;
	f=(void*)getfunc("glSecondaryColor3f");
	ov_glSecondaryColor3f=f;
	ov_glSecondaryColor3f(param0,param1,param2);
}
void (APIENTRY *ov_glSecondaryColor3f)(float param0,float param1,float param2)=init_ov_glSecondaryColor3f;

void __gl_ml_glSecondaryColor3f(value v0,value v1,value v2)
{
	float l0=Double_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	ov_glSecondaryColor3f(l0,l1,l2);
}
extern void (APIENTRY *ov_glSecondaryColor3dv)(double* param0);

static void APIENTRY init_ov_glSecondaryColor3dv(double* param0)
{
	void *f;
	f=(void*)getfunc("glSecondaryColor3dv");
	ov_glSecondaryColor3dv=f;
	ov_glSecondaryColor3dv(param0);
}
void (APIENTRY *ov_glSecondaryColor3dv)(double* param0)=init_ov_glSecondaryColor3dv;

void __gl_ml_glSecondaryColor3dv(value v0)
{
	double* l0=(double*)(gl__ba__(v0));
	ov_glSecondaryColor3dv(l0);
}
extern void (APIENTRY *ov_glSecondaryColor3d)(double param0,double param1,double param2);

static void APIENTRY init_ov_glSecondaryColor3d(double param0,double param1,double param2)
{
	void *f;
	f=(void*)getfunc("glSecondaryColor3d");
	ov_glSecondaryColor3d=f;
	ov_glSecondaryColor3d(param0,param1,param2);
}
void (APIENTRY *ov_glSecondaryColor3d)(double param0,double param1,double param2)=init_ov_glSecondaryColor3d;

void __gl_ml_glSecondaryColor3d(value v0,value v1,value v2)
{
	double l0=Double_val(v0);
	double l1=Double_val(v1);
	double l2=Double_val(v2);
	ov_glSecondaryColor3d(l0,l1,l2);
}
extern void (APIENTRY *ov_glSecondaryColor3bv)(signed char* param0);

static void APIENTRY init_ov_glSecondaryColor3bv(signed char* param0)
{
	void *f;
	f=(void*)getfunc("glSecondaryColor3bv");
	ov_glSecondaryColor3bv=f;
	ov_glSecondaryColor3bv(param0);
}
void (APIENTRY *ov_glSecondaryColor3bv)(signed char* param0)=init_ov_glSecondaryColor3bv;

void __gl_ml_glSecondaryColor3bv(value v0)
{
	signed char* l0=(signed char*)(gl__ba__(v0));
	ov_glSecondaryColor3bv(l0);
}
extern void (APIENTRY *ov_glSecondaryColor3b)(signed char param0,signed char param1,signed char param2);

static void APIENTRY init_ov_glSecondaryColor3b(signed char param0,signed char param1,signed char param2)
{
	void *f;
	f=(void*)getfunc("glSecondaryColor3b");
	ov_glSecondaryColor3b=f;
	ov_glSecondaryColor3b(param0,param1,param2);
}
void (APIENTRY *ov_glSecondaryColor3b)(signed char param0,signed char param1,signed char param2)=init_ov_glSecondaryColor3b;

void __gl_ml_glSecondaryColor3b(value v0,value v1,value v2)
{
	signed char l0=Int_val(v0);
	signed char l1=Int_val(v1);
	signed char l2=Int_val(v2);
	ov_glSecondaryColor3b(l0,l1,l2);
}
extern void (APIENTRY *ov_glPointParameteriv)(unsigned int param0,signed int* param1);

static void APIENTRY init_ov_glPointParameteriv(unsigned int param0,signed int* param1)
{
	void *f;
	f=(void*)getfunc("glPointParameteriv");
	ov_glPointParameteriv=f;
	ov_glPointParameteriv(param0,param1);
}
void (APIENTRY *ov_glPointParameteriv)(unsigned int param0,signed int* param1)=init_ov_glPointParameteriv;

void __gl_ml_glPointParameteriv(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed int* l1=(signed int*)(gl__ba__(v1));
	ov_glPointParameteriv(l0,l1);
}
extern void (APIENTRY *ov_glPointParameteri)(unsigned int param0,signed int param1);

static void APIENTRY init_ov_glPointParameteri(unsigned int param0,signed int param1)
{
	void *f;
	f=(void*)getfunc("glPointParameteri");
	ov_glPointParameteri=f;
	ov_glPointParameteri(param0,param1);
}
void (APIENTRY *ov_glPointParameteri)(unsigned int param0,signed int param1)=init_ov_glPointParameteri;

void __gl_ml_glPointParameteri(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	ov_glPointParameteri(l0,l1);
}
extern void (APIENTRY *ov_glPointParameterfv)(unsigned int param0,float* param1);

static void APIENTRY init_ov_glPointParameterfv(unsigned int param0,float* param1)
{
	void *f;
	f=(void*)getfunc("glPointParameterfv");
	ov_glPointParameterfv=f;
	ov_glPointParameterfv(param0,param1);
}
void (APIENTRY *ov_glPointParameterfv)(unsigned int param0,float* param1)=init_ov_glPointParameterfv;

void __gl_ml_glPointParameterfv(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float* l1=(float*)(gl__ba__(v1));
	ov_glPointParameterfv(l0,l1);
}
extern void (APIENTRY *ov_glPointParameterf)(unsigned int param0,float param1);

static void APIENTRY init_ov_glPointParameterf(unsigned int param0,float param1)
{
	void *f;
	f=(void*)getfunc("glPointParameterf");
	ov_glPointParameterf=f;
	ov_glPointParameterf(param0,param1);
}
void (APIENTRY *ov_glPointParameterf)(unsigned int param0,float param1)=init_ov_glPointParameterf;

void __gl_ml_glPointParameterf(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float l1=Double_val(v1);
	ov_glPointParameterf(l0,l1);
}
extern void (APIENTRY *ov_glMultiDrawElements)(unsigned int param0,signed int* param1,unsigned int param2,void** param3,signed int param4);

static void APIENTRY init_ov_glMultiDrawElements(unsigned int param0,signed int* param1,unsigned int param2,void** param3,signed int param4)
{
	void *f;
	f=(void*)getfunc("glMultiDrawElements");
	ov_glMultiDrawElements=f;
	ov_glMultiDrawElements(param0,param1,param2,param3,param4);
}
void (APIENTRY *ov_glMultiDrawElements)(unsigned int param0,signed int* param1,unsigned int param2,void** param3,signed int param4)=init_ov_glMultiDrawElements;
extern void (APIENTRY *ov_glMultiDrawArrays)(unsigned int param0,signed int* param1,signed int* param2,signed int param3);

static void APIENTRY init_ov_glMultiDrawArrays(unsigned int param0,signed int* param1,signed int* param2,signed int param3)
{
	void *f;
	f=(void*)getfunc("glMultiDrawArrays");
	ov_glMultiDrawArrays=f;
	ov_glMultiDrawArrays(param0,param1,param2,param3);
}
void (APIENTRY *ov_glMultiDrawArrays)(unsigned int param0,signed int* param1,signed int* param2,signed int param3)=init_ov_glMultiDrawArrays;

void __gl_ml_glMultiDrawArrays(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	signed int* l1=(signed int*)(gl__ba__(v1));
	signed int* l2=(signed int*)(gl__ba__(v2));
	signed int l3=Int_val(v3);
	ov_glMultiDrawArrays(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glFogCoordPointer)(unsigned int param0,signed int param1,void* param2);

static void APIENTRY init_ov_glFogCoordPointer(unsigned int param0,signed int param1,void* param2)
{
	void *f;
	f=(void*)getfunc("glFogCoordPointer");
	ov_glFogCoordPointer=f;
	ov_glFogCoordPointer(param0,param1,param2);
}
void (APIENTRY *ov_glFogCoordPointer)(unsigned int param0,signed int param1,void* param2)=init_ov_glFogCoordPointer;

void __gl_ml_glFogCoordPointer(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	void* l2=gl__ba__(v2);
	ov_glFogCoordPointer(l0,l1,l2);
}
extern void (APIENTRY *ov_glFogCoorddv)(double* param0);

static void APIENTRY init_ov_glFogCoorddv(double* param0)
{
	void *f;
	f=(void*)getfunc("glFogCoorddv");
	ov_glFogCoorddv=f;
	ov_glFogCoorddv(param0);
}
void (APIENTRY *ov_glFogCoorddv)(double* param0)=init_ov_glFogCoorddv;

void __gl_ml_glFogCoorddv(value v0)
{
	double* l0=(double*)(gl__ba__(v0));
	ov_glFogCoorddv(l0);
}
extern void (APIENTRY *ov_glFogCoordd)(double param0);

static void APIENTRY init_ov_glFogCoordd(double param0)
{
	void *f;
	f=(void*)getfunc("glFogCoordd");
	ov_glFogCoordd=f;
	ov_glFogCoordd(param0);
}
void (APIENTRY *ov_glFogCoordd)(double param0)=init_ov_glFogCoordd;

void __gl_ml_glFogCoordd(value v0)
{
	double l0=Double_val(v0);
	ov_glFogCoordd(l0);
}
extern void (APIENTRY *ov_glFogCoordfv)(float* param0);

static void APIENTRY init_ov_glFogCoordfv(float* param0)
{
	void *f;
	f=(void*)getfunc("glFogCoordfv");
	ov_glFogCoordfv=f;
	ov_glFogCoordfv(param0);
}
void (APIENTRY *ov_glFogCoordfv)(float* param0)=init_ov_glFogCoordfv;

void __gl_ml_glFogCoordfv(value v0)
{
	float* l0=(float*)(gl__ba__(v0));
	ov_glFogCoordfv(l0);
}
extern void (APIENTRY *ov_glFogCoordf)(float param0);

static void APIENTRY init_ov_glFogCoordf(float param0)
{
	void *f;
	f=(void*)getfunc("glFogCoordf");
	ov_glFogCoordf=f;
	ov_glFogCoordf(param0);
}
void (APIENTRY *ov_glFogCoordf)(float param0)=init_ov_glFogCoordf;

void __gl_ml_glFogCoordf(value v0)
{
	float l0=Double_val(v0);
	ov_glFogCoordf(l0);
}
extern void (APIENTRY *ov_glBlendFuncSeparate)(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3);

static void APIENTRY init_ov_glBlendFuncSeparate(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3)
{
	void *f;
	f=(void*)getfunc("glBlendFuncSeparate");
	ov_glBlendFuncSeparate=f;
	ov_glBlendFuncSeparate(param0,param1,param2,param3);
}
void (APIENTRY *ov_glBlendFuncSeparate)(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3)=init_ov_glBlendFuncSeparate;

void __gl_ml_glBlendFuncSeparate(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	unsigned int l3=Int_val(v3);
	ov_glBlendFuncSeparate(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glMatrixIndexPointerARB)(signed int param0,unsigned int param1,signed int param2,void* param3);

static void APIENTRY init_ov_glMatrixIndexPointerARB(signed int param0,unsigned int param1,signed int param2,void* param3)
{
	void *f;
	f=(void*)getfunc("glMatrixIndexPointerARB");
	ov_glMatrixIndexPointerARB=f;
	ov_glMatrixIndexPointerARB(param0,param1,param2,param3);
}
void (APIENTRY *ov_glMatrixIndexPointerARB)(signed int param0,unsigned int param1,signed int param2,void* param3)=init_ov_glMatrixIndexPointerARB;

void __gl_ml_glMatrixIndexPointerARB(value v0,value v1,value v2,value v3)
{
	signed int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	void* l3=gl__ba__(v3);
	ov_glMatrixIndexPointerARB(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glMatrixIndexuivARB)(signed int param0,unsigned int* param1);

static void APIENTRY init_ov_glMatrixIndexuivARB(signed int param0,unsigned int* param1)
{
	void *f;
	f=(void*)getfunc("glMatrixIndexuivARB");
	ov_glMatrixIndexuivARB=f;
	ov_glMatrixIndexuivARB(param0,param1);
}
void (APIENTRY *ov_glMatrixIndexuivARB)(signed int param0,unsigned int* param1)=init_ov_glMatrixIndexuivARB;

void __gl_ml_glMatrixIndexuivARB(value v0,value v1)
{
	signed int l0=Int_val(v0);
	unsigned int* l1=(unsigned int*)(gl__ba__(v1));
	ov_glMatrixIndexuivARB(l0,l1);
}
extern void (APIENTRY *ov_glMatrixIndexusvARB)(signed int param0,unsigned short* param1);

static void APIENTRY init_ov_glMatrixIndexusvARB(signed int param0,unsigned short* param1)
{
	void *f;
	f=(void*)getfunc("glMatrixIndexusvARB");
	ov_glMatrixIndexusvARB=f;
	ov_glMatrixIndexusvARB(param0,param1);
}
void (APIENTRY *ov_glMatrixIndexusvARB)(signed int param0,unsigned short* param1)=init_ov_glMatrixIndexusvARB;

void __gl_ml_glMatrixIndexusvARB(value v0,value v1)
{
	signed int l0=Int_val(v0);
	unsigned short* l1=(unsigned short*)(gl__ba__(v1));
	ov_glMatrixIndexusvARB(l0,l1);
}
extern void (APIENTRY *ov_glMatrixIndexubvARB)(signed int param0,unsigned char* param1);

static void APIENTRY init_ov_glMatrixIndexubvARB(signed int param0,unsigned char* param1)
{
	void *f;
	f=(void*)getfunc("glMatrixIndexubvARB");
	ov_glMatrixIndexubvARB=f;
	ov_glMatrixIndexubvARB(param0,param1);
}
void (APIENTRY *ov_glMatrixIndexubvARB)(signed int param0,unsigned char* param1)=init_ov_glMatrixIndexubvARB;

void __gl_ml_glMatrixIndexubvARB(value v0,value v1)
{
	signed int l0=Int_val(v0);
	unsigned char* l1=(unsigned char*)(gl__ba__(v1));
	ov_glMatrixIndexubvARB(l0,l1);
}
extern void (APIENTRY *ov_glCurrentPaletteMatrixARB)(signed int param0);

static void APIENTRY init_ov_glCurrentPaletteMatrixARB(signed int param0)
{
	void *f;
	f=(void*)getfunc("glCurrentPaletteMatrixARB");
	ov_glCurrentPaletteMatrixARB=f;
	ov_glCurrentPaletteMatrixARB(param0);
}
void (APIENTRY *ov_glCurrentPaletteMatrixARB)(signed int param0)=init_ov_glCurrentPaletteMatrixARB;

void __gl_ml_glCurrentPaletteMatrixARB(value v0)
{
	signed int l0=Int_val(v0);
	ov_glCurrentPaletteMatrixARB(l0);
}
extern void (APIENTRY *ov_glSamplePassARB)(unsigned int param0);

static void APIENTRY init_ov_glSamplePassARB(unsigned int param0)
{
	void *f;
	f=(void*)getfunc("glSamplePassARB");
	ov_glSamplePassARB=f;
	ov_glSamplePassARB(param0);
}
void (APIENTRY *ov_glSamplePassARB)(unsigned int param0)=init_ov_glSamplePassARB;

void __gl_ml_glSamplePassARB(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glSamplePassARB(l0);
}
extern void (APIENTRY *ov_glSampleCoverageARB)(float param0,unsigned char param1);

static void APIENTRY init_ov_glSampleCoverageARB(float param0,unsigned char param1)
{
	void *f;
	f=(void*)getfunc("glSampleCoverageARB");
	ov_glSampleCoverageARB=f;
	ov_glSampleCoverageARB(param0,param1);
}
void (APIENTRY *ov_glSampleCoverageARB)(float param0,unsigned char param1)=init_ov_glSampleCoverageARB;

void __gl_ml_glSampleCoverageARB(value v0,value v1)
{
	float l0=Double_val(v0);
	unsigned char l1=Int_val(v1);
	ov_glSampleCoverageARB(l0,l1);
}
extern void (APIENTRY *ov_glMultiTexCoord4svARB)(unsigned int param0,signed short* param1);

static void APIENTRY init_ov_glMultiTexCoord4svARB(unsigned int param0,signed short* param1)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord4svARB");
	ov_glMultiTexCoord4svARB=f;
	ov_glMultiTexCoord4svARB(param0,param1);
}
void (APIENTRY *ov_glMultiTexCoord4svARB)(unsigned int param0,signed short* param1)=init_ov_glMultiTexCoord4svARB;

void __gl_ml_glMultiTexCoord4svARB(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed short* l1=(signed short*)(gl__ba__(v1));
	ov_glMultiTexCoord4svARB(l0,l1);
}
extern void (APIENTRY *ov_glMultiTexCoord4sARB)(unsigned int param0,signed short param1,signed short param2,signed short param3,signed short param4);

static void APIENTRY init_ov_glMultiTexCoord4sARB(unsigned int param0,signed short param1,signed short param2,signed short param3,signed short param4)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord4sARB");
	ov_glMultiTexCoord4sARB=f;
	ov_glMultiTexCoord4sARB(param0,param1,param2,param3,param4);
}
void (APIENTRY *ov_glMultiTexCoord4sARB)(unsigned int param0,signed short param1,signed short param2,signed short param3,signed short param4)=init_ov_glMultiTexCoord4sARB;

void __gl_ml_glMultiTexCoord4sARB(value v0,value v1,value v2,value v3,value v4)
{
	unsigned int l0=Int_val(v0);
	signed short l1=Int_val(v1);
	signed short l2=Int_val(v2);
	signed short l3=Int_val(v3);
	signed short l4=Int_val(v4);
	ov_glMultiTexCoord4sARB(l0,l1,l2,l3,l4);
}
extern void (APIENTRY *ov_glMultiTexCoord4ivARB)(unsigned int param0,signed int* param1);

static void APIENTRY init_ov_glMultiTexCoord4ivARB(unsigned int param0,signed int* param1)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord4ivARB");
	ov_glMultiTexCoord4ivARB=f;
	ov_glMultiTexCoord4ivARB(param0,param1);
}
void (APIENTRY *ov_glMultiTexCoord4ivARB)(unsigned int param0,signed int* param1)=init_ov_glMultiTexCoord4ivARB;

void __gl_ml_glMultiTexCoord4ivARB(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed int* l1=(signed int*)(gl__ba__(v1));
	ov_glMultiTexCoord4ivARB(l0,l1);
}
extern void (APIENTRY *ov_glMultiTexCoord4iARB)(unsigned int param0,signed int param1,signed int param2,signed int param3,signed int param4);

static void APIENTRY init_ov_glMultiTexCoord4iARB(unsigned int param0,signed int param1,signed int param2,signed int param3,signed int param4)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord4iARB");
	ov_glMultiTexCoord4iARB=f;
	ov_glMultiTexCoord4iARB(param0,param1,param2,param3,param4);
}
void (APIENTRY *ov_glMultiTexCoord4iARB)(unsigned int param0,signed int param1,signed int param2,signed int param3,signed int param4)=init_ov_glMultiTexCoord4iARB;

void __gl_ml_glMultiTexCoord4iARB(value v0,value v1,value v2,value v3,value v4)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	signed int l4=Int_val(v4);
	ov_glMultiTexCoord4iARB(l0,l1,l2,l3,l4);
}
extern void (APIENTRY *ov_glMultiTexCoord4fvARB)(unsigned int param0,float* param1);

static void APIENTRY init_ov_glMultiTexCoord4fvARB(unsigned int param0,float* param1)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord4fvARB");
	ov_glMultiTexCoord4fvARB=f;
	ov_glMultiTexCoord4fvARB(param0,param1);
}
void (APIENTRY *ov_glMultiTexCoord4fvARB)(unsigned int param0,float* param1)=init_ov_glMultiTexCoord4fvARB;

void __gl_ml_glMultiTexCoord4fvARB(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float* l1=(float*)(gl__ba__(v1));
	ov_glMultiTexCoord4fvARB(l0,l1);
}
extern void (APIENTRY *ov_glMultiTexCoord4fARB)(unsigned int param0,float param1,float param2,float param3,float param4);

static void APIENTRY init_ov_glMultiTexCoord4fARB(unsigned int param0,float param1,float param2,float param3,float param4)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord4fARB");
	ov_glMultiTexCoord4fARB=f;
	ov_glMultiTexCoord4fARB(param0,param1,param2,param3,param4);
}
void (APIENTRY *ov_glMultiTexCoord4fARB)(unsigned int param0,float param1,float param2,float param3,float param4)=init_ov_glMultiTexCoord4fARB;

void __gl_ml_glMultiTexCoord4fARB(value v0,value v1,value v2,value v3,value v4)
{
	unsigned int l0=Int_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	float l3=Double_val(v3);
	float l4=Double_val(v4);
	ov_glMultiTexCoord4fARB(l0,l1,l2,l3,l4);
}
extern void (APIENTRY *ov_glMultiTexCoord4dvARB)(unsigned int param0,double* param1);

static void APIENTRY init_ov_glMultiTexCoord4dvARB(unsigned int param0,double* param1)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord4dvARB");
	ov_glMultiTexCoord4dvARB=f;
	ov_glMultiTexCoord4dvARB(param0,param1);
}
void (APIENTRY *ov_glMultiTexCoord4dvARB)(unsigned int param0,double* param1)=init_ov_glMultiTexCoord4dvARB;

void __gl_ml_glMultiTexCoord4dvARB(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	double* l1=(double*)(gl__ba__(v1));
	ov_glMultiTexCoord4dvARB(l0,l1);
}
extern void (APIENTRY *ov_glMultiTexCoord4dARB)(unsigned int param0,double param1,double param2,double param3,double param4);

static void APIENTRY init_ov_glMultiTexCoord4dARB(unsigned int param0,double param1,double param2,double param3,double param4)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord4dARB");
	ov_glMultiTexCoord4dARB=f;
	ov_glMultiTexCoord4dARB(param0,param1,param2,param3,param4);
}
void (APIENTRY *ov_glMultiTexCoord4dARB)(unsigned int param0,double param1,double param2,double param3,double param4)=init_ov_glMultiTexCoord4dARB;

void __gl_ml_glMultiTexCoord4dARB(value v0,value v1,value v2,value v3,value v4)
{
	unsigned int l0=Int_val(v0);
	double l1=Double_val(v1);
	double l2=Double_val(v2);
	double l3=Double_val(v3);
	double l4=Double_val(v4);
	ov_glMultiTexCoord4dARB(l0,l1,l2,l3,l4);
}
extern void (APIENTRY *ov_glMultiTexCoord3svARB)(unsigned int param0,signed short* param1);

static void APIENTRY init_ov_glMultiTexCoord3svARB(unsigned int param0,signed short* param1)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord3svARB");
	ov_glMultiTexCoord3svARB=f;
	ov_glMultiTexCoord3svARB(param0,param1);
}
void (APIENTRY *ov_glMultiTexCoord3svARB)(unsigned int param0,signed short* param1)=init_ov_glMultiTexCoord3svARB;

void __gl_ml_glMultiTexCoord3svARB(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed short* l1=(signed short*)(gl__ba__(v1));
	ov_glMultiTexCoord3svARB(l0,l1);
}
extern void (APIENTRY *ov_glMultiTexCoord3sARB)(unsigned int param0,signed short param1,signed short param2,signed short param3);

static void APIENTRY init_ov_glMultiTexCoord3sARB(unsigned int param0,signed short param1,signed short param2,signed short param3)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord3sARB");
	ov_glMultiTexCoord3sARB=f;
	ov_glMultiTexCoord3sARB(param0,param1,param2,param3);
}
void (APIENTRY *ov_glMultiTexCoord3sARB)(unsigned int param0,signed short param1,signed short param2,signed short param3)=init_ov_glMultiTexCoord3sARB;

void __gl_ml_glMultiTexCoord3sARB(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	signed short l1=Int_val(v1);
	signed short l2=Int_val(v2);
	signed short l3=Int_val(v3);
	ov_glMultiTexCoord3sARB(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glMultiTexCoord3ivARB)(unsigned int param0,signed int* param1);

static void APIENTRY init_ov_glMultiTexCoord3ivARB(unsigned int param0,signed int* param1)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord3ivARB");
	ov_glMultiTexCoord3ivARB=f;
	ov_glMultiTexCoord3ivARB(param0,param1);
}
void (APIENTRY *ov_glMultiTexCoord3ivARB)(unsigned int param0,signed int* param1)=init_ov_glMultiTexCoord3ivARB;

void __gl_ml_glMultiTexCoord3ivARB(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed int* l1=(signed int*)(gl__ba__(v1));
	ov_glMultiTexCoord3ivARB(l0,l1);
}
extern void (APIENTRY *ov_glMultiTexCoord3iARB)(unsigned int param0,signed int param1,signed int param2,signed int param3);

static void APIENTRY init_ov_glMultiTexCoord3iARB(unsigned int param0,signed int param1,signed int param2,signed int param3)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord3iARB");
	ov_glMultiTexCoord3iARB=f;
	ov_glMultiTexCoord3iARB(param0,param1,param2,param3);
}
void (APIENTRY *ov_glMultiTexCoord3iARB)(unsigned int param0,signed int param1,signed int param2,signed int param3)=init_ov_glMultiTexCoord3iARB;

void __gl_ml_glMultiTexCoord3iARB(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	ov_glMultiTexCoord3iARB(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glMultiTexCoord3fvARB)(unsigned int param0,float* param1);

static void APIENTRY init_ov_glMultiTexCoord3fvARB(unsigned int param0,float* param1)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord3fvARB");
	ov_glMultiTexCoord3fvARB=f;
	ov_glMultiTexCoord3fvARB(param0,param1);
}
void (APIENTRY *ov_glMultiTexCoord3fvARB)(unsigned int param0,float* param1)=init_ov_glMultiTexCoord3fvARB;

void __gl_ml_glMultiTexCoord3fvARB(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float* l1=(float*)(gl__ba__(v1));
	ov_glMultiTexCoord3fvARB(l0,l1);
}
extern void (APIENTRY *ov_glMultiTexCoord3fARB)(unsigned int param0,float param1,float param2,float param3);

static void APIENTRY init_ov_glMultiTexCoord3fARB(unsigned int param0,float param1,float param2,float param3)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord3fARB");
	ov_glMultiTexCoord3fARB=f;
	ov_glMultiTexCoord3fARB(param0,param1,param2,param3);
}
void (APIENTRY *ov_glMultiTexCoord3fARB)(unsigned int param0,float param1,float param2,float param3)=init_ov_glMultiTexCoord3fARB;

void __gl_ml_glMultiTexCoord3fARB(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	float l3=Double_val(v3);
	ov_glMultiTexCoord3fARB(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glMultiTexCoord3dvARB)(unsigned int param0,double* param1);

static void APIENTRY init_ov_glMultiTexCoord3dvARB(unsigned int param0,double* param1)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord3dvARB");
	ov_glMultiTexCoord3dvARB=f;
	ov_glMultiTexCoord3dvARB(param0,param1);
}
void (APIENTRY *ov_glMultiTexCoord3dvARB)(unsigned int param0,double* param1)=init_ov_glMultiTexCoord3dvARB;

void __gl_ml_glMultiTexCoord3dvARB(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	double* l1=(double*)(gl__ba__(v1));
	ov_glMultiTexCoord3dvARB(l0,l1);
}
extern void (APIENTRY *ov_glMultiTexCoord3dARB)(unsigned int param0,double param1,double param2,double param3);

static void APIENTRY init_ov_glMultiTexCoord3dARB(unsigned int param0,double param1,double param2,double param3)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord3dARB");
	ov_glMultiTexCoord3dARB=f;
	ov_glMultiTexCoord3dARB(param0,param1,param2,param3);
}
void (APIENTRY *ov_glMultiTexCoord3dARB)(unsigned int param0,double param1,double param2,double param3)=init_ov_glMultiTexCoord3dARB;

void __gl_ml_glMultiTexCoord3dARB(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	double l1=Double_val(v1);
	double l2=Double_val(v2);
	double l3=Double_val(v3);
	ov_glMultiTexCoord3dARB(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glMultiTexCoord2svARB)(unsigned int param0,signed short* param1);

static void APIENTRY init_ov_glMultiTexCoord2svARB(unsigned int param0,signed short* param1)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord2svARB");
	ov_glMultiTexCoord2svARB=f;
	ov_glMultiTexCoord2svARB(param0,param1);
}
void (APIENTRY *ov_glMultiTexCoord2svARB)(unsigned int param0,signed short* param1)=init_ov_glMultiTexCoord2svARB;

void __gl_ml_glMultiTexCoord2svARB(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed short* l1=(signed short*)(gl__ba__(v1));
	ov_glMultiTexCoord2svARB(l0,l1);
}
extern void (APIENTRY *ov_glMultiTexCoord2sARB)(unsigned int param0,signed short param1,signed short param2);

static void APIENTRY init_ov_glMultiTexCoord2sARB(unsigned int param0,signed short param1,signed short param2)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord2sARB");
	ov_glMultiTexCoord2sARB=f;
	ov_glMultiTexCoord2sARB(param0,param1,param2);
}
void (APIENTRY *ov_glMultiTexCoord2sARB)(unsigned int param0,signed short param1,signed short param2)=init_ov_glMultiTexCoord2sARB;

void __gl_ml_glMultiTexCoord2sARB(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	signed short l1=Int_val(v1);
	signed short l2=Int_val(v2);
	ov_glMultiTexCoord2sARB(l0,l1,l2);
}
extern void (APIENTRY *ov_glMultiTexCoord2ivARB)(unsigned int param0,signed int* param1);

static void APIENTRY init_ov_glMultiTexCoord2ivARB(unsigned int param0,signed int* param1)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord2ivARB");
	ov_glMultiTexCoord2ivARB=f;
	ov_glMultiTexCoord2ivARB(param0,param1);
}
void (APIENTRY *ov_glMultiTexCoord2ivARB)(unsigned int param0,signed int* param1)=init_ov_glMultiTexCoord2ivARB;

void __gl_ml_glMultiTexCoord2ivARB(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed int* l1=(signed int*)(gl__ba__(v1));
	ov_glMultiTexCoord2ivARB(l0,l1);
}
extern void (APIENTRY *ov_glMultiTexCoord2iARB)(unsigned int param0,signed int param1,signed int param2);

static void APIENTRY init_ov_glMultiTexCoord2iARB(unsigned int param0,signed int param1,signed int param2)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord2iARB");
	ov_glMultiTexCoord2iARB=f;
	ov_glMultiTexCoord2iARB(param0,param1,param2);
}
void (APIENTRY *ov_glMultiTexCoord2iARB)(unsigned int param0,signed int param1,signed int param2)=init_ov_glMultiTexCoord2iARB;

void __gl_ml_glMultiTexCoord2iARB(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	ov_glMultiTexCoord2iARB(l0,l1,l2);
}
extern void (APIENTRY *ov_glMultiTexCoord2fvARB)(unsigned int param0,float* param1);

static void APIENTRY init_ov_glMultiTexCoord2fvARB(unsigned int param0,float* param1)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord2fvARB");
	ov_glMultiTexCoord2fvARB=f;
	ov_glMultiTexCoord2fvARB(param0,param1);
}
void (APIENTRY *ov_glMultiTexCoord2fvARB)(unsigned int param0,float* param1)=init_ov_glMultiTexCoord2fvARB;

void __gl_ml_glMultiTexCoord2fvARB(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float* l1=(float*)(gl__ba__(v1));
	ov_glMultiTexCoord2fvARB(l0,l1);
}
extern void (APIENTRY *ov_glMultiTexCoord2fARB)(unsigned int param0,float param1,float param2);

static void APIENTRY init_ov_glMultiTexCoord2fARB(unsigned int param0,float param1,float param2)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord2fARB");
	ov_glMultiTexCoord2fARB=f;
	ov_glMultiTexCoord2fARB(param0,param1,param2);
}
void (APIENTRY *ov_glMultiTexCoord2fARB)(unsigned int param0,float param1,float param2)=init_ov_glMultiTexCoord2fARB;

void __gl_ml_glMultiTexCoord2fARB(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	ov_glMultiTexCoord2fARB(l0,l1,l2);
}
extern void (APIENTRY *ov_glMultiTexCoord2dvARB)(unsigned int param0,double* param1);

static void APIENTRY init_ov_glMultiTexCoord2dvARB(unsigned int param0,double* param1)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord2dvARB");
	ov_glMultiTexCoord2dvARB=f;
	ov_glMultiTexCoord2dvARB(param0,param1);
}
void (APIENTRY *ov_glMultiTexCoord2dvARB)(unsigned int param0,double* param1)=init_ov_glMultiTexCoord2dvARB;

void __gl_ml_glMultiTexCoord2dvARB(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	double* l1=(double*)(gl__ba__(v1));
	ov_glMultiTexCoord2dvARB(l0,l1);
}
extern void (APIENTRY *ov_glMultiTexCoord2dARB)(unsigned int param0,double param1,double param2);

static void APIENTRY init_ov_glMultiTexCoord2dARB(unsigned int param0,double param1,double param2)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord2dARB");
	ov_glMultiTexCoord2dARB=f;
	ov_glMultiTexCoord2dARB(param0,param1,param2);
}
void (APIENTRY *ov_glMultiTexCoord2dARB)(unsigned int param0,double param1,double param2)=init_ov_glMultiTexCoord2dARB;

void __gl_ml_glMultiTexCoord2dARB(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	double l1=Double_val(v1);
	double l2=Double_val(v2);
	ov_glMultiTexCoord2dARB(l0,l1,l2);
}
extern void (APIENTRY *ov_glMultiTexCoord1svARB)(unsigned int param0,signed short* param1);

static void APIENTRY init_ov_glMultiTexCoord1svARB(unsigned int param0,signed short* param1)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord1svARB");
	ov_glMultiTexCoord1svARB=f;
	ov_glMultiTexCoord1svARB(param0,param1);
}
void (APIENTRY *ov_glMultiTexCoord1svARB)(unsigned int param0,signed short* param1)=init_ov_glMultiTexCoord1svARB;

void __gl_ml_glMultiTexCoord1svARB(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed short* l1=(signed short*)(gl__ba__(v1));
	ov_glMultiTexCoord1svARB(l0,l1);
}
extern void (APIENTRY *ov_glMultiTexCoord1sARB)(unsigned int param0,signed short param1);

static void APIENTRY init_ov_glMultiTexCoord1sARB(unsigned int param0,signed short param1)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord1sARB");
	ov_glMultiTexCoord1sARB=f;
	ov_glMultiTexCoord1sARB(param0,param1);
}
void (APIENTRY *ov_glMultiTexCoord1sARB)(unsigned int param0,signed short param1)=init_ov_glMultiTexCoord1sARB;

void __gl_ml_glMultiTexCoord1sARB(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed short l1=Int_val(v1);
	ov_glMultiTexCoord1sARB(l0,l1);
}
extern void (APIENTRY *ov_glMultiTexCoord1ivARB)(unsigned int param0,signed int* param1);

static void APIENTRY init_ov_glMultiTexCoord1ivARB(unsigned int param0,signed int* param1)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord1ivARB");
	ov_glMultiTexCoord1ivARB=f;
	ov_glMultiTexCoord1ivARB(param0,param1);
}
void (APIENTRY *ov_glMultiTexCoord1ivARB)(unsigned int param0,signed int* param1)=init_ov_glMultiTexCoord1ivARB;

void __gl_ml_glMultiTexCoord1ivARB(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed int* l1=(signed int*)(gl__ba__(v1));
	ov_glMultiTexCoord1ivARB(l0,l1);
}
extern void (APIENTRY *ov_glMultiTexCoord1iARB)(unsigned int param0,signed int param1);

static void APIENTRY init_ov_glMultiTexCoord1iARB(unsigned int param0,signed int param1)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord1iARB");
	ov_glMultiTexCoord1iARB=f;
	ov_glMultiTexCoord1iARB(param0,param1);
}
void (APIENTRY *ov_glMultiTexCoord1iARB)(unsigned int param0,signed int param1)=init_ov_glMultiTexCoord1iARB;

void __gl_ml_glMultiTexCoord1iARB(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	ov_glMultiTexCoord1iARB(l0,l1);
}
extern void (APIENTRY *ov_glMultiTexCoord1fvARB)(unsigned int param0,float* param1);

static void APIENTRY init_ov_glMultiTexCoord1fvARB(unsigned int param0,float* param1)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord1fvARB");
	ov_glMultiTexCoord1fvARB=f;
	ov_glMultiTexCoord1fvARB(param0,param1);
}
void (APIENTRY *ov_glMultiTexCoord1fvARB)(unsigned int param0,float* param1)=init_ov_glMultiTexCoord1fvARB;

void __gl_ml_glMultiTexCoord1fvARB(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float* l1=(float*)(gl__ba__(v1));
	ov_glMultiTexCoord1fvARB(l0,l1);
}
extern void (APIENTRY *ov_glMultiTexCoord1fARB)(unsigned int param0,float param1);

static void APIENTRY init_ov_glMultiTexCoord1fARB(unsigned int param0,float param1)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord1fARB");
	ov_glMultiTexCoord1fARB=f;
	ov_glMultiTexCoord1fARB(param0,param1);
}
void (APIENTRY *ov_glMultiTexCoord1fARB)(unsigned int param0,float param1)=init_ov_glMultiTexCoord1fARB;

void __gl_ml_glMultiTexCoord1fARB(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float l1=Double_val(v1);
	ov_glMultiTexCoord1fARB(l0,l1);
}
extern void (APIENTRY *ov_glMultiTexCoord1dvARB)(unsigned int param0,double* param1);

static void APIENTRY init_ov_glMultiTexCoord1dvARB(unsigned int param0,double* param1)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord1dvARB");
	ov_glMultiTexCoord1dvARB=f;
	ov_glMultiTexCoord1dvARB(param0,param1);
}
void (APIENTRY *ov_glMultiTexCoord1dvARB)(unsigned int param0,double* param1)=init_ov_glMultiTexCoord1dvARB;

void __gl_ml_glMultiTexCoord1dvARB(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	double* l1=(double*)(gl__ba__(v1));
	ov_glMultiTexCoord1dvARB(l0,l1);
}
extern void (APIENTRY *ov_glMultiTexCoord1dARB)(unsigned int param0,double param1);

static void APIENTRY init_ov_glMultiTexCoord1dARB(unsigned int param0,double param1)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord1dARB");
	ov_glMultiTexCoord1dARB=f;
	ov_glMultiTexCoord1dARB(param0,param1);
}
void (APIENTRY *ov_glMultiTexCoord1dARB)(unsigned int param0,double param1)=init_ov_glMultiTexCoord1dARB;

void __gl_ml_glMultiTexCoord1dARB(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	double l1=Double_val(v1);
	ov_glMultiTexCoord1dARB(l0,l1);
}
extern void (APIENTRY *ov_glClientActiveTextureARB)(unsigned int param0);

static void APIENTRY init_ov_glClientActiveTextureARB(unsigned int param0)
{
	void *f;
	f=(void*)getfunc("glClientActiveTextureARB");
	ov_glClientActiveTextureARB=f;
	ov_glClientActiveTextureARB(param0);
}
void (APIENTRY *ov_glClientActiveTextureARB)(unsigned int param0)=init_ov_glClientActiveTextureARB;

void __gl_ml_glClientActiveTextureARB(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glClientActiveTextureARB(l0);
}
extern void (APIENTRY *ov_glActiveTextureARB)(unsigned int param0);

static void APIENTRY init_ov_glActiveTextureARB(unsigned int param0)
{
	void *f;
	f=(void*)getfunc("glActiveTextureARB");
	ov_glActiveTextureARB=f;
	ov_glActiveTextureARB(param0);
}
void (APIENTRY *ov_glActiveTextureARB)(unsigned int param0)=init_ov_glActiveTextureARB;

void __gl_ml_glActiveTextureARB(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glActiveTextureARB(l0);
}
extern void (APIENTRY *ov_glPointParameterfvARB)(unsigned int param0,float* param1);

static void APIENTRY init_ov_glPointParameterfvARB(unsigned int param0,float* param1)
{
	void *f;
	f=(void*)getfunc("glPointParameterfvARB");
	ov_glPointParameterfvARB=f;
	ov_glPointParameterfvARB(param0,param1);
}
void (APIENTRY *ov_glPointParameterfvARB)(unsigned int param0,float* param1)=init_ov_glPointParameterfvARB;

void __gl_ml_glPointParameterfvARB(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float* l1=(float*)(gl__ba__(v1));
	ov_glPointParameterfvARB(l0,l1);
}
extern void (APIENTRY *ov_glPointParameterfARB)(unsigned int param0,float param1);

static void APIENTRY init_ov_glPointParameterfARB(unsigned int param0,float param1)
{
	void *f;
	f=(void*)getfunc("glPointParameterfARB");
	ov_glPointParameterfARB=f;
	ov_glPointParameterfARB(param0,param1);
}
void (APIENTRY *ov_glPointParameterfARB)(unsigned int param0,float param1)=init_ov_glPointParameterfARB;

void __gl_ml_glPointParameterfARB(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float l1=Double_val(v1);
	ov_glPointParameterfARB(l0,l1);
}
extern void (APIENTRY *ov_glGetCompressedTexImageARB)(unsigned int param0,signed int param1,void* param2);

static void APIENTRY init_ov_glGetCompressedTexImageARB(unsigned int param0,signed int param1,void* param2)
{
	void *f;
	f=(void*)getfunc("glGetCompressedTexImageARB");
	ov_glGetCompressedTexImageARB=f;
	ov_glGetCompressedTexImageARB(param0,param1,param2);
}
void (APIENTRY *ov_glGetCompressedTexImageARB)(unsigned int param0,signed int param1,void* param2)=init_ov_glGetCompressedTexImageARB;

void __gl_ml_glGetCompressedTexImageARB(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	void* l2=gl__ba__(v2);
	ov_glGetCompressedTexImageARB(l0,l1,l2);
}
extern void (APIENTRY *ov_glCompressedTexSubImage1DARB)(unsigned int param0,signed int param1,signed int param2,signed int param3,unsigned int param4,signed int param5,void* param6);

static void APIENTRY init_ov_glCompressedTexSubImage1DARB(unsigned int param0,signed int param1,signed int param2,signed int param3,unsigned int param4,signed int param5,void* param6)
{
	void *f;
	f=(void*)getfunc("glCompressedTexSubImage1DARB");
	ov_glCompressedTexSubImage1DARB=f;
	ov_glCompressedTexSubImage1DARB(param0,param1,param2,param3,param4,param5,param6);
}
void (APIENTRY *ov_glCompressedTexSubImage1DARB)(unsigned int param0,signed int param1,signed int param2,signed int param3,unsigned int param4,signed int param5,void* param6)=init_ov_glCompressedTexSubImage1DARB;

void __gl_ml_glCompressedTexSubImage1DARB(value v0,value v1,value v2,value v3,value v4,value v5,value v6)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	unsigned int l4=Int_val(v4);
	signed int l5=Int_val(v5);
	void* l6=gl__ba__(v6);
	ov_glCompressedTexSubImage1DARB(l0,l1,l2,l3,l4,l5,l6);
}

void byte___gl_ml_glCompressedTexSubImage1DARB(value * argv,int argn)
{
	__gl_ml_glCompressedTexSubImage1DARB(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6]);
}
extern void (APIENTRY *ov_glCompressedTexSubImage2DARB)(unsigned int param0,signed int param1,signed int param2,signed int param3,signed int param4,signed int param5,unsigned int param6,signed int param7,void* param8);

static void APIENTRY init_ov_glCompressedTexSubImage2DARB(unsigned int param0,signed int param1,signed int param2,signed int param3,signed int param4,signed int param5,unsigned int param6,signed int param7,void* param8)
{
	void *f;
	f=(void*)getfunc("glCompressedTexSubImage2DARB");
	ov_glCompressedTexSubImage2DARB=f;
	ov_glCompressedTexSubImage2DARB(param0,param1,param2,param3,param4,param5,param6,param7,param8);
}
void (APIENTRY *ov_glCompressedTexSubImage2DARB)(unsigned int param0,signed int param1,signed int param2,signed int param3,signed int param4,signed int param5,unsigned int param6,signed int param7,void* param8)=init_ov_glCompressedTexSubImage2DARB;

void __gl_ml_glCompressedTexSubImage2DARB(value v0,value v1,value v2,value v3,value v4,value v5,value v6,value v7,value v8)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	signed int l4=Int_val(v4);
	signed int l5=Int_val(v5);
	unsigned int l6=Int_val(v6);
	signed int l7=Int_val(v7);
	void* l8=gl__ba__(v8);
	ov_glCompressedTexSubImage2DARB(l0,l1,l2,l3,l4,l5,l6,l7,l8);
}

void byte___gl_ml_glCompressedTexSubImage2DARB(value * argv,int argn)
{
	__gl_ml_glCompressedTexSubImage2DARB(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6],argv[7],argv[8]);
}
extern void (APIENTRY *ov_glCompressedTexSubImage3DARB)(unsigned int param0,signed int param1,signed int param2,signed int param3,signed int param4,signed int param5,signed int param6,signed int param7,unsigned int param8,signed int param9,void* param10);

static void APIENTRY init_ov_glCompressedTexSubImage3DARB(unsigned int param0,signed int param1,signed int param2,signed int param3,signed int param4,signed int param5,signed int param6,signed int param7,unsigned int param8,signed int param9,void* param10)
{
	void *f;
	f=(void*)getfunc("glCompressedTexSubImage3DARB");
	ov_glCompressedTexSubImage3DARB=f;
	ov_glCompressedTexSubImage3DARB(param0,param1,param2,param3,param4,param5,param6,param7,param8,param9,param10);
}
void (APIENTRY *ov_glCompressedTexSubImage3DARB)(unsigned int param0,signed int param1,signed int param2,signed int param3,signed int param4,signed int param5,signed int param6,signed int param7,unsigned int param8,signed int param9,void* param10)=init_ov_glCompressedTexSubImage3DARB;

void __gl_ml_glCompressedTexSubImage3DARB(value v0,value v1,value v2,value v3,value v4,value v5,value v6,value v7,value v8,value v9,value v10)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	signed int l4=Int_val(v4);
	signed int l5=Int_val(v5);
	signed int l6=Int_val(v6);
	signed int l7=Int_val(v7);
	unsigned int l8=Int_val(v8);
	signed int l9=Int_val(v9);
	void* l10=gl__ba__(v10);
	ov_glCompressedTexSubImage3DARB(l0,l1,l2,l3,l4,l5,l6,l7,l8,l9,l10);
}

void byte___gl_ml_glCompressedTexSubImage3DARB(value * argv,int argn)
{
	__gl_ml_glCompressedTexSubImage3DARB(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6],argv[7],argv[8],argv[9],argv[10]);
}
extern void (APIENTRY *ov_glCompressedTexImage1DARB)(unsigned int param0,signed int param1,unsigned int param2,signed int param3,signed int param4,signed int param5,void* param6);

static void APIENTRY init_ov_glCompressedTexImage1DARB(unsigned int param0,signed int param1,unsigned int param2,signed int param3,signed int param4,signed int param5,void* param6)
{
	void *f;
	f=(void*)getfunc("glCompressedTexImage1DARB");
	ov_glCompressedTexImage1DARB=f;
	ov_glCompressedTexImage1DARB(param0,param1,param2,param3,param4,param5,param6);
}
void (APIENTRY *ov_glCompressedTexImage1DARB)(unsigned int param0,signed int param1,unsigned int param2,signed int param3,signed int param4,signed int param5,void* param6)=init_ov_glCompressedTexImage1DARB;

void __gl_ml_glCompressedTexImage1DARB(value v0,value v1,value v2,value v3,value v4,value v5,value v6)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	signed int l4=Int_val(v4);
	signed int l5=Int_val(v5);
	void* l6=gl__ba__(v6);
	ov_glCompressedTexImage1DARB(l0,l1,l2,l3,l4,l5,l6);
}

void byte___gl_ml_glCompressedTexImage1DARB(value * argv,int argn)
{
	__gl_ml_glCompressedTexImage1DARB(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6]);
}
extern void (APIENTRY *ov_glCompressedTexImage2DARB)(unsigned int param0,signed int param1,unsigned int param2,signed int param3,signed int param4,signed int param5,signed int param6,void* param7);

static void APIENTRY init_ov_glCompressedTexImage2DARB(unsigned int param0,signed int param1,unsigned int param2,signed int param3,signed int param4,signed int param5,signed int param6,void* param7)
{
	void *f;
	f=(void*)getfunc("glCompressedTexImage2DARB");
	ov_glCompressedTexImage2DARB=f;
	ov_glCompressedTexImage2DARB(param0,param1,param2,param3,param4,param5,param6,param7);
}
void (APIENTRY *ov_glCompressedTexImage2DARB)(unsigned int param0,signed int param1,unsigned int param2,signed int param3,signed int param4,signed int param5,signed int param6,void* param7)=init_ov_glCompressedTexImage2DARB;

void __gl_ml_glCompressedTexImage2DARB(value v0,value v1,value v2,value v3,value v4,value v5,value v6,value v7)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	signed int l4=Int_val(v4);
	signed int l5=Int_val(v5);
	signed int l6=Int_val(v6);
	void* l7=gl__ba__(v7);
	ov_glCompressedTexImage2DARB(l0,l1,l2,l3,l4,l5,l6,l7);
}

void byte___gl_ml_glCompressedTexImage2DARB(value * argv,int argn)
{
	__gl_ml_glCompressedTexImage2DARB(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6],argv[7]);
}
extern void (APIENTRY *ov_glCompressedTexImage3DARB)(unsigned int param0,signed int param1,unsigned int param2,signed int param3,signed int param4,signed int param5,signed int param6,signed int param7,void* param8);

static void APIENTRY init_ov_glCompressedTexImage3DARB(unsigned int param0,signed int param1,unsigned int param2,signed int param3,signed int param4,signed int param5,signed int param6,signed int param7,void* param8)
{
	void *f;
	f=(void*)getfunc("glCompressedTexImage3DARB");
	ov_glCompressedTexImage3DARB=f;
	ov_glCompressedTexImage3DARB(param0,param1,param2,param3,param4,param5,param6,param7,param8);
}
void (APIENTRY *ov_glCompressedTexImage3DARB)(unsigned int param0,signed int param1,unsigned int param2,signed int param3,signed int param4,signed int param5,signed int param6,signed int param7,void* param8)=init_ov_glCompressedTexImage3DARB;

void __gl_ml_glCompressedTexImage3DARB(value v0,value v1,value v2,value v3,value v4,value v5,value v6,value v7,value v8)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	signed int l4=Int_val(v4);
	signed int l5=Int_val(v5);
	signed int l6=Int_val(v6);
	signed int l7=Int_val(v7);
	void* l8=gl__ba__(v8);
	ov_glCompressedTexImage3DARB(l0,l1,l2,l3,l4,l5,l6,l7,l8);
}

void byte___gl_ml_glCompressedTexImage3DARB(value * argv,int argn)
{
	__gl_ml_glCompressedTexImage3DARB(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6],argv[7],argv[8]);
}
extern void (APIENTRY *ov_glMultTransposeMatrixdARB)(double* param0);

static void APIENTRY init_ov_glMultTransposeMatrixdARB(double* param0)
{
	void *f;
	f=(void*)getfunc("glMultTransposeMatrixdARB");
	ov_glMultTransposeMatrixdARB=f;
	ov_glMultTransposeMatrixdARB(param0);
}
void (APIENTRY *ov_glMultTransposeMatrixdARB)(double* param0)=init_ov_glMultTransposeMatrixdARB;

void __gl_ml_glMultTransposeMatrixdARB(value v0)
{
	double* l0=(double*)(gl__ba__(v0));
	ov_glMultTransposeMatrixdARB(l0);
}
extern void (APIENTRY *ov_glMultTransposeMatrixfARB)(float* param0);

static void APIENTRY init_ov_glMultTransposeMatrixfARB(float* param0)
{
	void *f;
	f=(void*)getfunc("glMultTransposeMatrixfARB");
	ov_glMultTransposeMatrixfARB=f;
	ov_glMultTransposeMatrixfARB(param0);
}
void (APIENTRY *ov_glMultTransposeMatrixfARB)(float* param0)=init_ov_glMultTransposeMatrixfARB;

void __gl_ml_glMultTransposeMatrixfARB(value v0)
{
	float* l0=(float*)(gl__ba__(v0));
	ov_glMultTransposeMatrixfARB(l0);
}
extern void (APIENTRY *ov_glLoadTransposeMatrixdARB)(double* param0);

static void APIENTRY init_ov_glLoadTransposeMatrixdARB(double* param0)
{
	void *f;
	f=(void*)getfunc("glLoadTransposeMatrixdARB");
	ov_glLoadTransposeMatrixdARB=f;
	ov_glLoadTransposeMatrixdARB(param0);
}
void (APIENTRY *ov_glLoadTransposeMatrixdARB)(double* param0)=init_ov_glLoadTransposeMatrixdARB;

void __gl_ml_glLoadTransposeMatrixdARB(value v0)
{
	double* l0=(double*)(gl__ba__(v0));
	ov_glLoadTransposeMatrixdARB(l0);
}
extern void (APIENTRY *ov_glLoadTransposeMatrixfARB)(float* param0);

static void APIENTRY init_ov_glLoadTransposeMatrixfARB(float* param0)
{
	void *f;
	f=(void*)getfunc("glLoadTransposeMatrixfARB");
	ov_glLoadTransposeMatrixfARB=f;
	ov_glLoadTransposeMatrixfARB(param0);
}
void (APIENTRY *ov_glLoadTransposeMatrixfARB)(float* param0)=init_ov_glLoadTransposeMatrixfARB;

void __gl_ml_glLoadTransposeMatrixfARB(value v0)
{
	float* l0=(float*)(gl__ba__(v0));
	ov_glLoadTransposeMatrixfARB(l0);
}
extern void (APIENTRY *ov_glVertexBlendARB)(signed int param0);

static void APIENTRY init_ov_glVertexBlendARB(signed int param0)
{
	void *f;
	f=(void*)getfunc("glVertexBlendARB");
	ov_glVertexBlendARB=f;
	ov_glVertexBlendARB(param0);
}
void (APIENTRY *ov_glVertexBlendARB)(signed int param0)=init_ov_glVertexBlendARB;

void __gl_ml_glVertexBlendARB(value v0)
{
	signed int l0=Int_val(v0);
	ov_glVertexBlendARB(l0);
}
extern void (APIENTRY *ov_glWeightPointerARB)(signed int param0,unsigned int param1,signed int param2,void* param3);

static void APIENTRY init_ov_glWeightPointerARB(signed int param0,unsigned int param1,signed int param2,void* param3)
{
	void *f;
	f=(void*)getfunc("glWeightPointerARB");
	ov_glWeightPointerARB=f;
	ov_glWeightPointerARB(param0,param1,param2,param3);
}
void (APIENTRY *ov_glWeightPointerARB)(signed int param0,unsigned int param1,signed int param2,void* param3)=init_ov_glWeightPointerARB;

void __gl_ml_glWeightPointerARB(value v0,value v1,value v2,value v3)
{
	signed int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	void* l3=gl__ba__(v3);
	ov_glWeightPointerARB(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glWeightuivARB)(signed int param0,unsigned int* param1);

static void APIENTRY init_ov_glWeightuivARB(signed int param0,unsigned int* param1)
{
	void *f;
	f=(void*)getfunc("glWeightuivARB");
	ov_glWeightuivARB=f;
	ov_glWeightuivARB(param0,param1);
}
void (APIENTRY *ov_glWeightuivARB)(signed int param0,unsigned int* param1)=init_ov_glWeightuivARB;

void __gl_ml_glWeightuivARB(value v0,value v1)
{
	signed int l0=Int_val(v0);
	unsigned int* l1=(unsigned int*)(gl__ba__(v1));
	ov_glWeightuivARB(l0,l1);
}
extern void (APIENTRY *ov_glWeightusvARB)(signed int param0,unsigned short* param1);

static void APIENTRY init_ov_glWeightusvARB(signed int param0,unsigned short* param1)
{
	void *f;
	f=(void*)getfunc("glWeightusvARB");
	ov_glWeightusvARB=f;
	ov_glWeightusvARB(param0,param1);
}
void (APIENTRY *ov_glWeightusvARB)(signed int param0,unsigned short* param1)=init_ov_glWeightusvARB;

void __gl_ml_glWeightusvARB(value v0,value v1)
{
	signed int l0=Int_val(v0);
	unsigned short* l1=(unsigned short*)(gl__ba__(v1));
	ov_glWeightusvARB(l0,l1);
}
extern void (APIENTRY *ov_glWeightubvARB)(signed int param0,unsigned char* param1);

static void APIENTRY init_ov_glWeightubvARB(signed int param0,unsigned char* param1)
{
	void *f;
	f=(void*)getfunc("glWeightubvARB");
	ov_glWeightubvARB=f;
	ov_glWeightubvARB(param0,param1);
}
void (APIENTRY *ov_glWeightubvARB)(signed int param0,unsigned char* param1)=init_ov_glWeightubvARB;

void __gl_ml_glWeightubvARB(value v0,value v1)
{
	signed int l0=Int_val(v0);
	unsigned char* l1=(unsigned char*)(gl__ba__(v1));
	ov_glWeightubvARB(l0,l1);
}
extern void (APIENTRY *ov_glWeightdvARB)(signed int param0,double* param1);

static void APIENTRY init_ov_glWeightdvARB(signed int param0,double* param1)
{
	void *f;
	f=(void*)getfunc("glWeightdvARB");
	ov_glWeightdvARB=f;
	ov_glWeightdvARB(param0,param1);
}
void (APIENTRY *ov_glWeightdvARB)(signed int param0,double* param1)=init_ov_glWeightdvARB;

void __gl_ml_glWeightdvARB(value v0,value v1)
{
	signed int l0=Int_val(v0);
	double* l1=(double*)(gl__ba__(v1));
	ov_glWeightdvARB(l0,l1);
}
extern void (APIENTRY *ov_glWeightfvARB)(signed int param0,float* param1);

static void APIENTRY init_ov_glWeightfvARB(signed int param0,float* param1)
{
	void *f;
	f=(void*)getfunc("glWeightfvARB");
	ov_glWeightfvARB=f;
	ov_glWeightfvARB(param0,param1);
}
void (APIENTRY *ov_glWeightfvARB)(signed int param0,float* param1)=init_ov_glWeightfvARB;

void __gl_ml_glWeightfvARB(value v0,value v1)
{
	signed int l0=Int_val(v0);
	float* l1=(float*)(gl__ba__(v1));
	ov_glWeightfvARB(l0,l1);
}
extern void (APIENTRY *ov_glWeightivARB)(signed int param0,signed int* param1);

static void APIENTRY init_ov_glWeightivARB(signed int param0,signed int* param1)
{
	void *f;
	f=(void*)getfunc("glWeightivARB");
	ov_glWeightivARB=f;
	ov_glWeightivARB(param0,param1);
}
void (APIENTRY *ov_glWeightivARB)(signed int param0,signed int* param1)=init_ov_glWeightivARB;

void __gl_ml_glWeightivARB(value v0,value v1)
{
	signed int l0=Int_val(v0);
	signed int* l1=(signed int*)(gl__ba__(v1));
	ov_glWeightivARB(l0,l1);
}
extern void (APIENTRY *ov_glWeightsvARB)(signed int param0,signed short* param1);

static void APIENTRY init_ov_glWeightsvARB(signed int param0,signed short* param1)
{
	void *f;
	f=(void*)getfunc("glWeightsvARB");
	ov_glWeightsvARB=f;
	ov_glWeightsvARB(param0,param1);
}
void (APIENTRY *ov_glWeightsvARB)(signed int param0,signed short* param1)=init_ov_glWeightsvARB;

void __gl_ml_glWeightsvARB(value v0,value v1)
{
	signed int l0=Int_val(v0);
	signed short* l1=(signed short*)(gl__ba__(v1));
	ov_glWeightsvARB(l0,l1);
}
extern void (APIENTRY *ov_glWeightbvARB)(signed int param0,signed char* param1);

static void APIENTRY init_ov_glWeightbvARB(signed int param0,signed char* param1)
{
	void *f;
	f=(void*)getfunc("glWeightbvARB");
	ov_glWeightbvARB=f;
	ov_glWeightbvARB(param0,param1);
}
void (APIENTRY *ov_glWeightbvARB)(signed int param0,signed char* param1)=init_ov_glWeightbvARB;

void __gl_ml_glWeightbvARB(value v0,value v1)
{
	signed int l0=Int_val(v0);
	signed char* l1=(signed char*)(gl__ba__(v1));
	ov_glWeightbvARB(l0,l1);
}
extern unsigned char (APIENTRY *ov_glIsProgramARB)(unsigned int param0);

static unsigned char APIENTRY init_ov_glIsProgramARB(unsigned int param0)
{
	void *f;
	f=(void*)getfunc("glIsProgramARB");
	ov_glIsProgramARB=f;
	return ov_glIsProgramARB(param0);
}
unsigned char (APIENTRY *ov_glIsProgramARB)(unsigned int param0)=init_ov_glIsProgramARB;

value __gl_ml_glIsProgramARB(value v0)
{
	unsigned int l0=Int_val(v0);
	unsigned char r=ov_glIsProgramARB(l0);
	return(Val_int(r));
}
extern void (APIENTRY *ov_glGetVertexAttribPointervARB)(unsigned int param0,unsigned int param1,void** param2);

static void APIENTRY init_ov_glGetVertexAttribPointervARB(unsigned int param0,unsigned int param1,void** param2)
{
	void *f;
	f=(void*)getfunc("glGetVertexAttribPointervARB");
	ov_glGetVertexAttribPointervARB=f;
	ov_glGetVertexAttribPointervARB(param0,param1,param2);
}
void (APIENTRY *ov_glGetVertexAttribPointervARB)(unsigned int param0,unsigned int param1,void** param2)=init_ov_glGetVertexAttribPointervARB;
extern void (APIENTRY *ov_glGetVertexAttribivARB)(unsigned int param0,unsigned int param1,signed int* param2);

static void APIENTRY init_ov_glGetVertexAttribivARB(unsigned int param0,unsigned int param1,signed int* param2)
{
	void *f;
	f=(void*)getfunc("glGetVertexAttribivARB");
	ov_glGetVertexAttribivARB=f;
	ov_glGetVertexAttribivARB(param0,param1,param2);
}
void (APIENTRY *ov_glGetVertexAttribivARB)(unsigned int param0,unsigned int param1,signed int* param2)=init_ov_glGetVertexAttribivARB;

void __gl_ml_glGetVertexAttribivARB(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int* l2=(signed int*)(gl__ba__(v2));
	ov_glGetVertexAttribivARB(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetVertexAttribfvARB)(unsigned int param0,unsigned int param1,float* param2);

static void APIENTRY init_ov_glGetVertexAttribfvARB(unsigned int param0,unsigned int param1,float* param2)
{
	void *f;
	f=(void*)getfunc("glGetVertexAttribfvARB");
	ov_glGetVertexAttribfvARB=f;
	ov_glGetVertexAttribfvARB(param0,param1,param2);
}
void (APIENTRY *ov_glGetVertexAttribfvARB)(unsigned int param0,unsigned int param1,float* param2)=init_ov_glGetVertexAttribfvARB;

void __gl_ml_glGetVertexAttribfvARB(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glGetVertexAttribfvARB(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetVertexAttribdvARB)(unsigned int param0,unsigned int param1,double* param2);

static void APIENTRY init_ov_glGetVertexAttribdvARB(unsigned int param0,unsigned int param1,double* param2)
{
	void *f;
	f=(void*)getfunc("glGetVertexAttribdvARB");
	ov_glGetVertexAttribdvARB=f;
	ov_glGetVertexAttribdvARB(param0,param1,param2);
}
void (APIENTRY *ov_glGetVertexAttribdvARB)(unsigned int param0,unsigned int param1,double* param2)=init_ov_glGetVertexAttribdvARB;

void __gl_ml_glGetVertexAttribdvARB(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	double* l2=(double*)(gl__ba__(v2));
	ov_glGetVertexAttribdvARB(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetProgramStringARB)(unsigned int param0,unsigned int param1,void* param2);

static void APIENTRY init_ov_glGetProgramStringARB(unsigned int param0,unsigned int param1,void* param2)
{
	void *f;
	f=(void*)getfunc("glGetProgramStringARB");
	ov_glGetProgramStringARB=f;
	ov_glGetProgramStringARB(param0,param1,param2);
}
void (APIENTRY *ov_glGetProgramStringARB)(unsigned int param0,unsigned int param1,void* param2)=init_ov_glGetProgramStringARB;

void __gl_ml_glGetProgramStringARB(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	void* l2=gl__ba__(v2);
	ov_glGetProgramStringARB(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetProgramivARB)(unsigned int param0,unsigned int param1,signed int* param2);

static void APIENTRY init_ov_glGetProgramivARB(unsigned int param0,unsigned int param1,signed int* param2)
{
	void *f;
	f=(void*)getfunc("glGetProgramivARB");
	ov_glGetProgramivARB=f;
	ov_glGetProgramivARB(param0,param1,param2);
}
void (APIENTRY *ov_glGetProgramivARB)(unsigned int param0,unsigned int param1,signed int* param2)=init_ov_glGetProgramivARB;

void __gl_ml_glGetProgramivARB(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int* l2=(signed int*)(gl__ba__(v2));
	ov_glGetProgramivARB(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetProgramLocalParameterfvARB)(unsigned int param0,unsigned int param1,float* param2);

static void APIENTRY init_ov_glGetProgramLocalParameterfvARB(unsigned int param0,unsigned int param1,float* param2)
{
	void *f;
	f=(void*)getfunc("glGetProgramLocalParameterfvARB");
	ov_glGetProgramLocalParameterfvARB=f;
	ov_glGetProgramLocalParameterfvARB(param0,param1,param2);
}
void (APIENTRY *ov_glGetProgramLocalParameterfvARB)(unsigned int param0,unsigned int param1,float* param2)=init_ov_glGetProgramLocalParameterfvARB;

void __gl_ml_glGetProgramLocalParameterfvARB(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glGetProgramLocalParameterfvARB(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetProgramLocalParameterdvARB)(unsigned int param0,unsigned int param1,double* param2);

static void APIENTRY init_ov_glGetProgramLocalParameterdvARB(unsigned int param0,unsigned int param1,double* param2)
{
	void *f;
	f=(void*)getfunc("glGetProgramLocalParameterdvARB");
	ov_glGetProgramLocalParameterdvARB=f;
	ov_glGetProgramLocalParameterdvARB(param0,param1,param2);
}
void (APIENTRY *ov_glGetProgramLocalParameterdvARB)(unsigned int param0,unsigned int param1,double* param2)=init_ov_glGetProgramLocalParameterdvARB;

void __gl_ml_glGetProgramLocalParameterdvARB(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	double* l2=(double*)(gl__ba__(v2));
	ov_glGetProgramLocalParameterdvARB(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetProgramEnvParameterfvARB)(unsigned int param0,unsigned int param1,float* param2);

static void APIENTRY init_ov_glGetProgramEnvParameterfvARB(unsigned int param0,unsigned int param1,float* param2)
{
	void *f;
	f=(void*)getfunc("glGetProgramEnvParameterfvARB");
	ov_glGetProgramEnvParameterfvARB=f;
	ov_glGetProgramEnvParameterfvARB(param0,param1,param2);
}
void (APIENTRY *ov_glGetProgramEnvParameterfvARB)(unsigned int param0,unsigned int param1,float* param2)=init_ov_glGetProgramEnvParameterfvARB;

void __gl_ml_glGetProgramEnvParameterfvARB(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glGetProgramEnvParameterfvARB(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetProgramEnvParameterdvARB)(unsigned int param0,unsigned int param1,double* param2);

static void APIENTRY init_ov_glGetProgramEnvParameterdvARB(unsigned int param0,unsigned int param1,double* param2)
{
	void *f;
	f=(void*)getfunc("glGetProgramEnvParameterdvARB");
	ov_glGetProgramEnvParameterdvARB=f;
	ov_glGetProgramEnvParameterdvARB(param0,param1,param2);
}
void (APIENTRY *ov_glGetProgramEnvParameterdvARB)(unsigned int param0,unsigned int param1,double* param2)=init_ov_glGetProgramEnvParameterdvARB;

void __gl_ml_glGetProgramEnvParameterdvARB(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	double* l2=(double*)(gl__ba__(v2));
	ov_glGetProgramEnvParameterdvARB(l0,l1,l2);
}
extern void (APIENTRY *ov_glProgramLocalParameter4fvARB)(unsigned int param0,unsigned int param1,float* param2);

static void APIENTRY init_ov_glProgramLocalParameter4fvARB(unsigned int param0,unsigned int param1,float* param2)
{
	void *f;
	f=(void*)getfunc("glProgramLocalParameter4fvARB");
	ov_glProgramLocalParameter4fvARB=f;
	ov_glProgramLocalParameter4fvARB(param0,param1,param2);
}
void (APIENTRY *ov_glProgramLocalParameter4fvARB)(unsigned int param0,unsigned int param1,float* param2)=init_ov_glProgramLocalParameter4fvARB;

void __gl_ml_glProgramLocalParameter4fvARB(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glProgramLocalParameter4fvARB(l0,l1,l2);
}
extern void (APIENTRY *ov_glProgramLocalParameter4fARB)(unsigned int param0,unsigned int param1,float param2,float param3,float param4,float param5);

static void APIENTRY init_ov_glProgramLocalParameter4fARB(unsigned int param0,unsigned int param1,float param2,float param3,float param4,float param5)
{
	void *f;
	f=(void*)getfunc("glProgramLocalParameter4fARB");
	ov_glProgramLocalParameter4fARB=f;
	ov_glProgramLocalParameter4fARB(param0,param1,param2,param3,param4,param5);
}
void (APIENTRY *ov_glProgramLocalParameter4fARB)(unsigned int param0,unsigned int param1,float param2,float param3,float param4,float param5)=init_ov_glProgramLocalParameter4fARB;

void __gl_ml_glProgramLocalParameter4fARB(value v0,value v1,value v2,value v3,value v4,value v5)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float l2=Double_val(v2);
	float l3=Double_val(v3);
	float l4=Double_val(v4);
	float l5=Double_val(v5);
	ov_glProgramLocalParameter4fARB(l0,l1,l2,l3,l4,l5);
}

void byte___gl_ml_glProgramLocalParameter4fARB(value * argv,int argn)
{
	__gl_ml_glProgramLocalParameter4fARB(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5]);
}
extern void (APIENTRY *ov_glProgramLocalParameter4dvARB)(unsigned int param0,unsigned int param1,double* param2);

static void APIENTRY init_ov_glProgramLocalParameter4dvARB(unsigned int param0,unsigned int param1,double* param2)
{
	void *f;
	f=(void*)getfunc("glProgramLocalParameter4dvARB");
	ov_glProgramLocalParameter4dvARB=f;
	ov_glProgramLocalParameter4dvARB(param0,param1,param2);
}
void (APIENTRY *ov_glProgramLocalParameter4dvARB)(unsigned int param0,unsigned int param1,double* param2)=init_ov_glProgramLocalParameter4dvARB;

void __gl_ml_glProgramLocalParameter4dvARB(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	double* l2=(double*)(gl__ba__(v2));
	ov_glProgramLocalParameter4dvARB(l0,l1,l2);
}
extern void (APIENTRY *ov_glProgramLocalParameter4dARB)(unsigned int param0,unsigned int param1,double param2,double param3,double param4,double param5);

static void APIENTRY init_ov_glProgramLocalParameter4dARB(unsigned int param0,unsigned int param1,double param2,double param3,double param4,double param5)
{
	void *f;
	f=(void*)getfunc("glProgramLocalParameter4dARB");
	ov_glProgramLocalParameter4dARB=f;
	ov_glProgramLocalParameter4dARB(param0,param1,param2,param3,param4,param5);
}
void (APIENTRY *ov_glProgramLocalParameter4dARB)(unsigned int param0,unsigned int param1,double param2,double param3,double param4,double param5)=init_ov_glProgramLocalParameter4dARB;

void __gl_ml_glProgramLocalParameter4dARB(value v0,value v1,value v2,value v3,value v4,value v5)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	double l2=Double_val(v2);
	double l3=Double_val(v3);
	double l4=Double_val(v4);
	double l5=Double_val(v5);
	ov_glProgramLocalParameter4dARB(l0,l1,l2,l3,l4,l5);
}

void byte___gl_ml_glProgramLocalParameter4dARB(value * argv,int argn)
{
	__gl_ml_glProgramLocalParameter4dARB(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5]);
}
extern void (APIENTRY *ov_glProgramEnvParameter4fvARB)(unsigned int param0,unsigned int param1,float* param2);

static void APIENTRY init_ov_glProgramEnvParameter4fvARB(unsigned int param0,unsigned int param1,float* param2)
{
	void *f;
	f=(void*)getfunc("glProgramEnvParameter4fvARB");
	ov_glProgramEnvParameter4fvARB=f;
	ov_glProgramEnvParameter4fvARB(param0,param1,param2);
}
void (APIENTRY *ov_glProgramEnvParameter4fvARB)(unsigned int param0,unsigned int param1,float* param2)=init_ov_glProgramEnvParameter4fvARB;

void __gl_ml_glProgramEnvParameter4fvARB(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glProgramEnvParameter4fvARB(l0,l1,l2);
}
extern void (APIENTRY *ov_glProgramEnvParameter4fARB)(unsigned int param0,unsigned int param1,float param2,float param3,float param4,float param5);

static void APIENTRY init_ov_glProgramEnvParameter4fARB(unsigned int param0,unsigned int param1,float param2,float param3,float param4,float param5)
{
	void *f;
	f=(void*)getfunc("glProgramEnvParameter4fARB");
	ov_glProgramEnvParameter4fARB=f;
	ov_glProgramEnvParameter4fARB(param0,param1,param2,param3,param4,param5);
}
void (APIENTRY *ov_glProgramEnvParameter4fARB)(unsigned int param0,unsigned int param1,float param2,float param3,float param4,float param5)=init_ov_glProgramEnvParameter4fARB;

void __gl_ml_glProgramEnvParameter4fARB(value v0,value v1,value v2,value v3,value v4,value v5)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float l2=Double_val(v2);
	float l3=Double_val(v3);
	float l4=Double_val(v4);
	float l5=Double_val(v5);
	ov_glProgramEnvParameter4fARB(l0,l1,l2,l3,l4,l5);
}

void byte___gl_ml_glProgramEnvParameter4fARB(value * argv,int argn)
{
	__gl_ml_glProgramEnvParameter4fARB(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5]);
}
extern void (APIENTRY *ov_glProgramEnvParameter4dvARB)(unsigned int param0,unsigned int param1,double* param2);

static void APIENTRY init_ov_glProgramEnvParameter4dvARB(unsigned int param0,unsigned int param1,double* param2)
{
	void *f;
	f=(void*)getfunc("glProgramEnvParameter4dvARB");
	ov_glProgramEnvParameter4dvARB=f;
	ov_glProgramEnvParameter4dvARB(param0,param1,param2);
}
void (APIENTRY *ov_glProgramEnvParameter4dvARB)(unsigned int param0,unsigned int param1,double* param2)=init_ov_glProgramEnvParameter4dvARB;

void __gl_ml_glProgramEnvParameter4dvARB(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	double* l2=(double*)(gl__ba__(v2));
	ov_glProgramEnvParameter4dvARB(l0,l1,l2);
}
extern void (APIENTRY *ov_glProgramEnvParameter4dARB)(unsigned int param0,unsigned int param1,double param2,double param3,double param4,double param5);

static void APIENTRY init_ov_glProgramEnvParameter4dARB(unsigned int param0,unsigned int param1,double param2,double param3,double param4,double param5)
{
	void *f;
	f=(void*)getfunc("glProgramEnvParameter4dARB");
	ov_glProgramEnvParameter4dARB=f;
	ov_glProgramEnvParameter4dARB(param0,param1,param2,param3,param4,param5);
}
void (APIENTRY *ov_glProgramEnvParameter4dARB)(unsigned int param0,unsigned int param1,double param2,double param3,double param4,double param5)=init_ov_glProgramEnvParameter4dARB;

void __gl_ml_glProgramEnvParameter4dARB(value v0,value v1,value v2,value v3,value v4,value v5)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	double l2=Double_val(v2);
	double l3=Double_val(v3);
	double l4=Double_val(v4);
	double l5=Double_val(v5);
	ov_glProgramEnvParameter4dARB(l0,l1,l2,l3,l4,l5);
}

void byte___gl_ml_glProgramEnvParameter4dARB(value * argv,int argn)
{
	__gl_ml_glProgramEnvParameter4dARB(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5]);
}
extern void (APIENTRY *ov_glGenProgramsARB)(signed int param0,unsigned int* param1);

static void APIENTRY init_ov_glGenProgramsARB(signed int param0,unsigned int* param1)
{
	void *f;
	f=(void*)getfunc("glGenProgramsARB");
	ov_glGenProgramsARB=f;
	ov_glGenProgramsARB(param0,param1);
}
void (APIENTRY *ov_glGenProgramsARB)(signed int param0,unsigned int* param1)=init_ov_glGenProgramsARB;

void __gl_ml_glGenProgramsARB(value v0,value v1)
{
	signed int l0=Int_val(v0);
	unsigned int* l1=(unsigned int*)(gl__ba__(v1));
	ov_glGenProgramsARB(l0,l1);
}
extern void (APIENTRY *ov_glDeleteProgramsARB)(signed int param0,unsigned int* param1);

static void APIENTRY init_ov_glDeleteProgramsARB(signed int param0,unsigned int* param1)
{
	void *f;
	f=(void*)getfunc("glDeleteProgramsARB");
	ov_glDeleteProgramsARB=f;
	ov_glDeleteProgramsARB(param0,param1);
}
void (APIENTRY *ov_glDeleteProgramsARB)(signed int param0,unsigned int* param1)=init_ov_glDeleteProgramsARB;

void __gl_ml_glDeleteProgramsARB(value v0,value v1)
{
	signed int l0=Int_val(v0);
	unsigned int* l1=(unsigned int*)(gl__ba__(v1));
	ov_glDeleteProgramsARB(l0,l1);
}
extern void (APIENTRY *ov_glBindProgramARB)(unsigned int param0,unsigned int param1);

static void APIENTRY init_ov_glBindProgramARB(unsigned int param0,unsigned int param1)
{
	void *f;
	f=(void*)getfunc("glBindProgramARB");
	ov_glBindProgramARB=f;
	ov_glBindProgramARB(param0,param1);
}
void (APIENTRY *ov_glBindProgramARB)(unsigned int param0,unsigned int param1)=init_ov_glBindProgramARB;

void __gl_ml_glBindProgramARB(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	ov_glBindProgramARB(l0,l1);
}
extern void (APIENTRY *ov_glProgramStringARB)(unsigned int param0,unsigned int param1,signed int param2,char* param3);

static void APIENTRY init_ov_glProgramStringARB(unsigned int param0,unsigned int param1,signed int param2,char* param3)
{
	void *f;
	f=(void*)getfunc("glProgramStringARB");
	ov_glProgramStringARB=f;
	ov_glProgramStringARB(param0,param1,param2,param3);
}
void (APIENTRY *ov_glProgramStringARB)(unsigned int param0,unsigned int param1,signed int param2,char* param3)=init_ov_glProgramStringARB;

void __gl_ml_glProgramStringARB(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	char* l3=String_val(v3);
	ov_glProgramStringARB(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glDisableVertexAttribArrayARB)(unsigned int param0);

static void APIENTRY init_ov_glDisableVertexAttribArrayARB(unsigned int param0)
{
	void *f;
	f=(void*)getfunc("glDisableVertexAttribArrayARB");
	ov_glDisableVertexAttribArrayARB=f;
	ov_glDisableVertexAttribArrayARB(param0);
}
void (APIENTRY *ov_glDisableVertexAttribArrayARB)(unsigned int param0)=init_ov_glDisableVertexAttribArrayARB;

void __gl_ml_glDisableVertexAttribArrayARB(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glDisableVertexAttribArrayARB(l0);
}
extern void (APIENTRY *ov_glEnableVertexAttribArrayARB)(unsigned int param0);

static void APIENTRY init_ov_glEnableVertexAttribArrayARB(unsigned int param0)
{
	void *f;
	f=(void*)getfunc("glEnableVertexAttribArrayARB");
	ov_glEnableVertexAttribArrayARB=f;
	ov_glEnableVertexAttribArrayARB(param0);
}
void (APIENTRY *ov_glEnableVertexAttribArrayARB)(unsigned int param0)=init_ov_glEnableVertexAttribArrayARB;

void __gl_ml_glEnableVertexAttribArrayARB(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glEnableVertexAttribArrayARB(l0);
}
extern void (APIENTRY *ov_glVertexAttribPointerARB)(unsigned int param0,signed int param1,unsigned int param2,unsigned char param3,signed int param4,void* param5);

static void APIENTRY init_ov_glVertexAttribPointerARB(unsigned int param0,signed int param1,unsigned int param2,unsigned char param3,signed int param4,void* param5)
{
	void *f;
	f=(void*)getfunc("glVertexAttribPointerARB");
	ov_glVertexAttribPointerARB=f;
	ov_glVertexAttribPointerARB(param0,param1,param2,param3,param4,param5);
}
void (APIENTRY *ov_glVertexAttribPointerARB)(unsigned int param0,signed int param1,unsigned int param2,unsigned char param3,signed int param4,void* param5)=init_ov_glVertexAttribPointerARB;

void __gl_ml_glVertexAttribPointerARB(value v0,value v1,value v2,value v3,value v4,value v5)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	unsigned char l3=Int_val(v3);
	signed int l4=Int_val(v4);
	void* l5=gl__ba__(v5);
	ov_glVertexAttribPointerARB(l0,l1,l2,l3,l4,l5);
}

void byte___gl_ml_glVertexAttribPointerARB(value * argv,int argn)
{
	__gl_ml_glVertexAttribPointerARB(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5]);
}
extern void (APIENTRY *ov_glVertexAttrib4usvARB)(unsigned int param0,unsigned short* param1);

static void APIENTRY init_ov_glVertexAttrib4usvARB(unsigned int param0,unsigned short* param1)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib4usvARB");
	ov_glVertexAttrib4usvARB=f;
	ov_glVertexAttrib4usvARB(param0,param1);
}
void (APIENTRY *ov_glVertexAttrib4usvARB)(unsigned int param0,unsigned short* param1)=init_ov_glVertexAttrib4usvARB;

void __gl_ml_glVertexAttrib4usvARB(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	unsigned short* l1=(unsigned short*)(gl__ba__(v1));
	ov_glVertexAttrib4usvARB(l0,l1);
}
extern void (APIENTRY *ov_glVertexAttrib4uivARB)(unsigned int param0,unsigned int* param1);

static void APIENTRY init_ov_glVertexAttrib4uivARB(unsigned int param0,unsigned int* param1)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib4uivARB");
	ov_glVertexAttrib4uivARB=f;
	ov_glVertexAttrib4uivARB(param0,param1);
}
void (APIENTRY *ov_glVertexAttrib4uivARB)(unsigned int param0,unsigned int* param1)=init_ov_glVertexAttrib4uivARB;

void __gl_ml_glVertexAttrib4uivARB(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	unsigned int* l1=(unsigned int*)(gl__ba__(v1));
	ov_glVertexAttrib4uivARB(l0,l1);
}
extern void (APIENTRY *ov_glVertexAttrib4ubvARB)(unsigned int param0,unsigned char* param1);

static void APIENTRY init_ov_glVertexAttrib4ubvARB(unsigned int param0,unsigned char* param1)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib4ubvARB");
	ov_glVertexAttrib4ubvARB=f;
	ov_glVertexAttrib4ubvARB(param0,param1);
}
void (APIENTRY *ov_glVertexAttrib4ubvARB)(unsigned int param0,unsigned char* param1)=init_ov_glVertexAttrib4ubvARB;

void __gl_ml_glVertexAttrib4ubvARB(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	unsigned char* l1=(unsigned char*)(gl__ba__(v1));
	ov_glVertexAttrib4ubvARB(l0,l1);
}
extern void (APIENTRY *ov_glVertexAttrib4svARB)(unsigned int param0,signed short* param1);

static void APIENTRY init_ov_glVertexAttrib4svARB(unsigned int param0,signed short* param1)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib4svARB");
	ov_glVertexAttrib4svARB=f;
	ov_glVertexAttrib4svARB(param0,param1);
}
void (APIENTRY *ov_glVertexAttrib4svARB)(unsigned int param0,signed short* param1)=init_ov_glVertexAttrib4svARB;

void __gl_ml_glVertexAttrib4svARB(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed short* l1=(signed short*)(gl__ba__(v1));
	ov_glVertexAttrib4svARB(l0,l1);
}
extern void (APIENTRY *ov_glVertexAttrib4sARB)(unsigned int param0,signed short param1,signed short param2,signed short param3,signed short param4);

static void APIENTRY init_ov_glVertexAttrib4sARB(unsigned int param0,signed short param1,signed short param2,signed short param3,signed short param4)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib4sARB");
	ov_glVertexAttrib4sARB=f;
	ov_glVertexAttrib4sARB(param0,param1,param2,param3,param4);
}
void (APIENTRY *ov_glVertexAttrib4sARB)(unsigned int param0,signed short param1,signed short param2,signed short param3,signed short param4)=init_ov_glVertexAttrib4sARB;

void __gl_ml_glVertexAttrib4sARB(value v0,value v1,value v2,value v3,value v4)
{
	unsigned int l0=Int_val(v0);
	signed short l1=Int_val(v1);
	signed short l2=Int_val(v2);
	signed short l3=Int_val(v3);
	signed short l4=Int_val(v4);
	ov_glVertexAttrib4sARB(l0,l1,l2,l3,l4);
}
extern void (APIENTRY *ov_glVertexAttrib4ivARB)(unsigned int param0,signed int* param1);

static void APIENTRY init_ov_glVertexAttrib4ivARB(unsigned int param0,signed int* param1)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib4ivARB");
	ov_glVertexAttrib4ivARB=f;
	ov_glVertexAttrib4ivARB(param0,param1);
}
void (APIENTRY *ov_glVertexAttrib4ivARB)(unsigned int param0,signed int* param1)=init_ov_glVertexAttrib4ivARB;

void __gl_ml_glVertexAttrib4ivARB(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed int* l1=(signed int*)(gl__ba__(v1));
	ov_glVertexAttrib4ivARB(l0,l1);
}
extern void (APIENTRY *ov_glVertexAttrib4fvARB)(unsigned int param0,float* param1);

static void APIENTRY init_ov_glVertexAttrib4fvARB(unsigned int param0,float* param1)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib4fvARB");
	ov_glVertexAttrib4fvARB=f;
	ov_glVertexAttrib4fvARB(param0,param1);
}
void (APIENTRY *ov_glVertexAttrib4fvARB)(unsigned int param0,float* param1)=init_ov_glVertexAttrib4fvARB;

void __gl_ml_glVertexAttrib4fvARB(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float* l1=(float*)(gl__ba__(v1));
	ov_glVertexAttrib4fvARB(l0,l1);
}
extern void (APIENTRY *ov_glVertexAttrib4fARB)(unsigned int param0,float param1,float param2,float param3,float param4);

static void APIENTRY init_ov_glVertexAttrib4fARB(unsigned int param0,float param1,float param2,float param3,float param4)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib4fARB");
	ov_glVertexAttrib4fARB=f;
	ov_glVertexAttrib4fARB(param0,param1,param2,param3,param4);
}
void (APIENTRY *ov_glVertexAttrib4fARB)(unsigned int param0,float param1,float param2,float param3,float param4)=init_ov_glVertexAttrib4fARB;

void __gl_ml_glVertexAttrib4fARB(value v0,value v1,value v2,value v3,value v4)
{
	unsigned int l0=Int_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	float l3=Double_val(v3);
	float l4=Double_val(v4);
	ov_glVertexAttrib4fARB(l0,l1,l2,l3,l4);
}
extern void (APIENTRY *ov_glVertexAttrib4dvARB)(unsigned int param0,double* param1);

static void APIENTRY init_ov_glVertexAttrib4dvARB(unsigned int param0,double* param1)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib4dvARB");
	ov_glVertexAttrib4dvARB=f;
	ov_glVertexAttrib4dvARB(param0,param1);
}
void (APIENTRY *ov_glVertexAttrib4dvARB)(unsigned int param0,double* param1)=init_ov_glVertexAttrib4dvARB;

void __gl_ml_glVertexAttrib4dvARB(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	double* l1=(double*)(gl__ba__(v1));
	ov_glVertexAttrib4dvARB(l0,l1);
}
extern void (APIENTRY *ov_glVertexAttrib4dARB)(unsigned int param0,double param1,double param2,double param3,double param4);

static void APIENTRY init_ov_glVertexAttrib4dARB(unsigned int param0,double param1,double param2,double param3,double param4)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib4dARB");
	ov_glVertexAttrib4dARB=f;
	ov_glVertexAttrib4dARB(param0,param1,param2,param3,param4);
}
void (APIENTRY *ov_glVertexAttrib4dARB)(unsigned int param0,double param1,double param2,double param3,double param4)=init_ov_glVertexAttrib4dARB;

void __gl_ml_glVertexAttrib4dARB(value v0,value v1,value v2,value v3,value v4)
{
	unsigned int l0=Int_val(v0);
	double l1=Double_val(v1);
	double l2=Double_val(v2);
	double l3=Double_val(v3);
	double l4=Double_val(v4);
	ov_glVertexAttrib4dARB(l0,l1,l2,l3,l4);
}
extern void (APIENTRY *ov_glVertexAttrib4bvARB)(unsigned int param0,signed char* param1);

static void APIENTRY init_ov_glVertexAttrib4bvARB(unsigned int param0,signed char* param1)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib4bvARB");
	ov_glVertexAttrib4bvARB=f;
	ov_glVertexAttrib4bvARB(param0,param1);
}
void (APIENTRY *ov_glVertexAttrib4bvARB)(unsigned int param0,signed char* param1)=init_ov_glVertexAttrib4bvARB;

void __gl_ml_glVertexAttrib4bvARB(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed char* l1=(signed char*)(gl__ba__(v1));
	ov_glVertexAttrib4bvARB(l0,l1);
}
extern void (APIENTRY *ov_glVertexAttrib4NusvARB)(unsigned int param0,unsigned short* param1);

static void APIENTRY init_ov_glVertexAttrib4NusvARB(unsigned int param0,unsigned short* param1)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib4NusvARB");
	ov_glVertexAttrib4NusvARB=f;
	ov_glVertexAttrib4NusvARB(param0,param1);
}
void (APIENTRY *ov_glVertexAttrib4NusvARB)(unsigned int param0,unsigned short* param1)=init_ov_glVertexAttrib4NusvARB;

void __gl_ml_glVertexAttrib4NusvARB(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	unsigned short* l1=(unsigned short*)(gl__ba__(v1));
	ov_glVertexAttrib4NusvARB(l0,l1);
}
extern void (APIENTRY *ov_glVertexAttrib4NuivARB)(unsigned int param0,unsigned int* param1);

static void APIENTRY init_ov_glVertexAttrib4NuivARB(unsigned int param0,unsigned int* param1)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib4NuivARB");
	ov_glVertexAttrib4NuivARB=f;
	ov_glVertexAttrib4NuivARB(param0,param1);
}
void (APIENTRY *ov_glVertexAttrib4NuivARB)(unsigned int param0,unsigned int* param1)=init_ov_glVertexAttrib4NuivARB;

void __gl_ml_glVertexAttrib4NuivARB(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	unsigned int* l1=(unsigned int*)(gl__ba__(v1));
	ov_glVertexAttrib4NuivARB(l0,l1);
}
extern void (APIENTRY *ov_glVertexAttrib4NubvARB)(unsigned int param0,unsigned char* param1);

static void APIENTRY init_ov_glVertexAttrib4NubvARB(unsigned int param0,unsigned char* param1)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib4NubvARB");
	ov_glVertexAttrib4NubvARB=f;
	ov_glVertexAttrib4NubvARB(param0,param1);
}
void (APIENTRY *ov_glVertexAttrib4NubvARB)(unsigned int param0,unsigned char* param1)=init_ov_glVertexAttrib4NubvARB;

void __gl_ml_glVertexAttrib4NubvARB(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	unsigned char* l1=(unsigned char*)(gl__ba__(v1));
	ov_glVertexAttrib4NubvARB(l0,l1);
}
extern void (APIENTRY *ov_glVertexAttrib4NubARB)(unsigned int param0,unsigned char param1,unsigned char param2,unsigned char param3,unsigned char param4);

static void APIENTRY init_ov_glVertexAttrib4NubARB(unsigned int param0,unsigned char param1,unsigned char param2,unsigned char param3,unsigned char param4)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib4NubARB");
	ov_glVertexAttrib4NubARB=f;
	ov_glVertexAttrib4NubARB(param0,param1,param2,param3,param4);
}
void (APIENTRY *ov_glVertexAttrib4NubARB)(unsigned int param0,unsigned char param1,unsigned char param2,unsigned char param3,unsigned char param4)=init_ov_glVertexAttrib4NubARB;

void __gl_ml_glVertexAttrib4NubARB(value v0,value v1,value v2,value v3,value v4)
{
	unsigned int l0=Int_val(v0);
	unsigned char l1=Int_val(v1);
	unsigned char l2=Int_val(v2);
	unsigned char l3=Int_val(v3);
	unsigned char l4=Int_val(v4);
	ov_glVertexAttrib4NubARB(l0,l1,l2,l3,l4);
}
extern void (APIENTRY *ov_glVertexAttrib4NsvARB)(unsigned int param0,signed short* param1);

static void APIENTRY init_ov_glVertexAttrib4NsvARB(unsigned int param0,signed short* param1)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib4NsvARB");
	ov_glVertexAttrib4NsvARB=f;
	ov_glVertexAttrib4NsvARB(param0,param1);
}
void (APIENTRY *ov_glVertexAttrib4NsvARB)(unsigned int param0,signed short* param1)=init_ov_glVertexAttrib4NsvARB;

void __gl_ml_glVertexAttrib4NsvARB(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed short* l1=(signed short*)(gl__ba__(v1));
	ov_glVertexAttrib4NsvARB(l0,l1);
}
extern void (APIENTRY *ov_glVertexAttrib4NivARB)(unsigned int param0,signed int* param1);

static void APIENTRY init_ov_glVertexAttrib4NivARB(unsigned int param0,signed int* param1)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib4NivARB");
	ov_glVertexAttrib4NivARB=f;
	ov_glVertexAttrib4NivARB(param0,param1);
}
void (APIENTRY *ov_glVertexAttrib4NivARB)(unsigned int param0,signed int* param1)=init_ov_glVertexAttrib4NivARB;

void __gl_ml_glVertexAttrib4NivARB(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed int* l1=(signed int*)(gl__ba__(v1));
	ov_glVertexAttrib4NivARB(l0,l1);
}
extern void (APIENTRY *ov_glVertexAttrib4NbvARB)(unsigned int param0,signed char* param1);

static void APIENTRY init_ov_glVertexAttrib4NbvARB(unsigned int param0,signed char* param1)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib4NbvARB");
	ov_glVertexAttrib4NbvARB=f;
	ov_glVertexAttrib4NbvARB(param0,param1);
}
void (APIENTRY *ov_glVertexAttrib4NbvARB)(unsigned int param0,signed char* param1)=init_ov_glVertexAttrib4NbvARB;

void __gl_ml_glVertexAttrib4NbvARB(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed char* l1=(signed char*)(gl__ba__(v1));
	ov_glVertexAttrib4NbvARB(l0,l1);
}
extern void (APIENTRY *ov_glVertexAttrib3svARB)(unsigned int param0,signed short* param1);

static void APIENTRY init_ov_glVertexAttrib3svARB(unsigned int param0,signed short* param1)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib3svARB");
	ov_glVertexAttrib3svARB=f;
	ov_glVertexAttrib3svARB(param0,param1);
}
void (APIENTRY *ov_glVertexAttrib3svARB)(unsigned int param0,signed short* param1)=init_ov_glVertexAttrib3svARB;

void __gl_ml_glVertexAttrib3svARB(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed short* l1=(signed short*)(gl__ba__(v1));
	ov_glVertexAttrib3svARB(l0,l1);
}
extern void (APIENTRY *ov_glVertexAttrib3sARB)(unsigned int param0,signed short param1,signed short param2,signed short param3);

static void APIENTRY init_ov_glVertexAttrib3sARB(unsigned int param0,signed short param1,signed short param2,signed short param3)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib3sARB");
	ov_glVertexAttrib3sARB=f;
	ov_glVertexAttrib3sARB(param0,param1,param2,param3);
}
void (APIENTRY *ov_glVertexAttrib3sARB)(unsigned int param0,signed short param1,signed short param2,signed short param3)=init_ov_glVertexAttrib3sARB;

void __gl_ml_glVertexAttrib3sARB(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	signed short l1=Int_val(v1);
	signed short l2=Int_val(v2);
	signed short l3=Int_val(v3);
	ov_glVertexAttrib3sARB(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glVertexAttrib3fvARB)(unsigned int param0,float* param1);

static void APIENTRY init_ov_glVertexAttrib3fvARB(unsigned int param0,float* param1)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib3fvARB");
	ov_glVertexAttrib3fvARB=f;
	ov_glVertexAttrib3fvARB(param0,param1);
}
void (APIENTRY *ov_glVertexAttrib3fvARB)(unsigned int param0,float* param1)=init_ov_glVertexAttrib3fvARB;

void __gl_ml_glVertexAttrib3fvARB(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float* l1=(float*)(gl__ba__(v1));
	ov_glVertexAttrib3fvARB(l0,l1);
}
extern void (APIENTRY *ov_glVertexAttrib3fARB)(unsigned int param0,float param1,float param2,float param3);

static void APIENTRY init_ov_glVertexAttrib3fARB(unsigned int param0,float param1,float param2,float param3)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib3fARB");
	ov_glVertexAttrib3fARB=f;
	ov_glVertexAttrib3fARB(param0,param1,param2,param3);
}
void (APIENTRY *ov_glVertexAttrib3fARB)(unsigned int param0,float param1,float param2,float param3)=init_ov_glVertexAttrib3fARB;

void __gl_ml_glVertexAttrib3fARB(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	float l3=Double_val(v3);
	ov_glVertexAttrib3fARB(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glVertexAttrib3dvARB)(unsigned int param0,double* param1);

static void APIENTRY init_ov_glVertexAttrib3dvARB(unsigned int param0,double* param1)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib3dvARB");
	ov_glVertexAttrib3dvARB=f;
	ov_glVertexAttrib3dvARB(param0,param1);
}
void (APIENTRY *ov_glVertexAttrib3dvARB)(unsigned int param0,double* param1)=init_ov_glVertexAttrib3dvARB;

void __gl_ml_glVertexAttrib3dvARB(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	double* l1=(double*)(gl__ba__(v1));
	ov_glVertexAttrib3dvARB(l0,l1);
}
extern void (APIENTRY *ov_glVertexAttrib3dARB)(unsigned int param0,double param1,double param2,double param3);

static void APIENTRY init_ov_glVertexAttrib3dARB(unsigned int param0,double param1,double param2,double param3)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib3dARB");
	ov_glVertexAttrib3dARB=f;
	ov_glVertexAttrib3dARB(param0,param1,param2,param3);
}
void (APIENTRY *ov_glVertexAttrib3dARB)(unsigned int param0,double param1,double param2,double param3)=init_ov_glVertexAttrib3dARB;

void __gl_ml_glVertexAttrib3dARB(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	double l1=Double_val(v1);
	double l2=Double_val(v2);
	double l3=Double_val(v3);
	ov_glVertexAttrib3dARB(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glVertexAttrib2svARB)(unsigned int param0,signed short* param1);

static void APIENTRY init_ov_glVertexAttrib2svARB(unsigned int param0,signed short* param1)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib2svARB");
	ov_glVertexAttrib2svARB=f;
	ov_glVertexAttrib2svARB(param0,param1);
}
void (APIENTRY *ov_glVertexAttrib2svARB)(unsigned int param0,signed short* param1)=init_ov_glVertexAttrib2svARB;

void __gl_ml_glVertexAttrib2svARB(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed short* l1=(signed short*)(gl__ba__(v1));
	ov_glVertexAttrib2svARB(l0,l1);
}
extern void (APIENTRY *ov_glVertexAttrib2sARB)(unsigned int param0,signed short param1,signed short param2);

static void APIENTRY init_ov_glVertexAttrib2sARB(unsigned int param0,signed short param1,signed short param2)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib2sARB");
	ov_glVertexAttrib2sARB=f;
	ov_glVertexAttrib2sARB(param0,param1,param2);
}
void (APIENTRY *ov_glVertexAttrib2sARB)(unsigned int param0,signed short param1,signed short param2)=init_ov_glVertexAttrib2sARB;

void __gl_ml_glVertexAttrib2sARB(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	signed short l1=Int_val(v1);
	signed short l2=Int_val(v2);
	ov_glVertexAttrib2sARB(l0,l1,l2);
}
extern void (APIENTRY *ov_glVertexAttrib2fvARB)(unsigned int param0,float* param1);

static void APIENTRY init_ov_glVertexAttrib2fvARB(unsigned int param0,float* param1)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib2fvARB");
	ov_glVertexAttrib2fvARB=f;
	ov_glVertexAttrib2fvARB(param0,param1);
}
void (APIENTRY *ov_glVertexAttrib2fvARB)(unsigned int param0,float* param1)=init_ov_glVertexAttrib2fvARB;

void __gl_ml_glVertexAttrib2fvARB(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float* l1=(float*)(gl__ba__(v1));
	ov_glVertexAttrib2fvARB(l0,l1);
}
extern void (APIENTRY *ov_glVertexAttrib2fARB)(unsigned int param0,float param1,float param2);

static void APIENTRY init_ov_glVertexAttrib2fARB(unsigned int param0,float param1,float param2)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib2fARB");
	ov_glVertexAttrib2fARB=f;
	ov_glVertexAttrib2fARB(param0,param1,param2);
}
void (APIENTRY *ov_glVertexAttrib2fARB)(unsigned int param0,float param1,float param2)=init_ov_glVertexAttrib2fARB;

void __gl_ml_glVertexAttrib2fARB(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	ov_glVertexAttrib2fARB(l0,l1,l2);
}
extern void (APIENTRY *ov_glVertexAttrib2dvARB)(unsigned int param0,double* param1);

static void APIENTRY init_ov_glVertexAttrib2dvARB(unsigned int param0,double* param1)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib2dvARB");
	ov_glVertexAttrib2dvARB=f;
	ov_glVertexAttrib2dvARB(param0,param1);
}
void (APIENTRY *ov_glVertexAttrib2dvARB)(unsigned int param0,double* param1)=init_ov_glVertexAttrib2dvARB;

void __gl_ml_glVertexAttrib2dvARB(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	double* l1=(double*)(gl__ba__(v1));
	ov_glVertexAttrib2dvARB(l0,l1);
}
extern void (APIENTRY *ov_glVertexAttrib2dARB)(unsigned int param0,double param1,double param2);

static void APIENTRY init_ov_glVertexAttrib2dARB(unsigned int param0,double param1,double param2)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib2dARB");
	ov_glVertexAttrib2dARB=f;
	ov_glVertexAttrib2dARB(param0,param1,param2);
}
void (APIENTRY *ov_glVertexAttrib2dARB)(unsigned int param0,double param1,double param2)=init_ov_glVertexAttrib2dARB;

void __gl_ml_glVertexAttrib2dARB(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	double l1=Double_val(v1);
	double l2=Double_val(v2);
	ov_glVertexAttrib2dARB(l0,l1,l2);
}
extern void (APIENTRY *ov_glVertexAttrib1svARB)(unsigned int param0,signed short* param1);

static void APIENTRY init_ov_glVertexAttrib1svARB(unsigned int param0,signed short* param1)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib1svARB");
	ov_glVertexAttrib1svARB=f;
	ov_glVertexAttrib1svARB(param0,param1);
}
void (APIENTRY *ov_glVertexAttrib1svARB)(unsigned int param0,signed short* param1)=init_ov_glVertexAttrib1svARB;

void __gl_ml_glVertexAttrib1svARB(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed short* l1=(signed short*)(gl__ba__(v1));
	ov_glVertexAttrib1svARB(l0,l1);
}
extern void (APIENTRY *ov_glVertexAttrib1sARB)(unsigned int param0,signed short param1);

static void APIENTRY init_ov_glVertexAttrib1sARB(unsigned int param0,signed short param1)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib1sARB");
	ov_glVertexAttrib1sARB=f;
	ov_glVertexAttrib1sARB(param0,param1);
}
void (APIENTRY *ov_glVertexAttrib1sARB)(unsigned int param0,signed short param1)=init_ov_glVertexAttrib1sARB;

void __gl_ml_glVertexAttrib1sARB(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed short l1=Int_val(v1);
	ov_glVertexAttrib1sARB(l0,l1);
}
extern void (APIENTRY *ov_glVertexAttrib1fvARB)(unsigned int param0,float* param1);

static void APIENTRY init_ov_glVertexAttrib1fvARB(unsigned int param0,float* param1)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib1fvARB");
	ov_glVertexAttrib1fvARB=f;
	ov_glVertexAttrib1fvARB(param0,param1);
}
void (APIENTRY *ov_glVertexAttrib1fvARB)(unsigned int param0,float* param1)=init_ov_glVertexAttrib1fvARB;

void __gl_ml_glVertexAttrib1fvARB(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float* l1=(float*)(gl__ba__(v1));
	ov_glVertexAttrib1fvARB(l0,l1);
}
extern void (APIENTRY *ov_glVertexAttrib1fARB)(unsigned int param0,float param1);

static void APIENTRY init_ov_glVertexAttrib1fARB(unsigned int param0,float param1)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib1fARB");
	ov_glVertexAttrib1fARB=f;
	ov_glVertexAttrib1fARB(param0,param1);
}
void (APIENTRY *ov_glVertexAttrib1fARB)(unsigned int param0,float param1)=init_ov_glVertexAttrib1fARB;

void __gl_ml_glVertexAttrib1fARB(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float l1=Double_val(v1);
	ov_glVertexAttrib1fARB(l0,l1);
}
extern void (APIENTRY *ov_glVertexAttrib1dvARB)(unsigned int param0,double* param1);

static void APIENTRY init_ov_glVertexAttrib1dvARB(unsigned int param0,double* param1)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib1dvARB");
	ov_glVertexAttrib1dvARB=f;
	ov_glVertexAttrib1dvARB(param0,param1);
}
void (APIENTRY *ov_glVertexAttrib1dvARB)(unsigned int param0,double* param1)=init_ov_glVertexAttrib1dvARB;

void __gl_ml_glVertexAttrib1dvARB(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	double* l1=(double*)(gl__ba__(v1));
	ov_glVertexAttrib1dvARB(l0,l1);
}
extern void (APIENTRY *ov_glVertexAttrib1dARB)(unsigned int param0,double param1);

static void APIENTRY init_ov_glVertexAttrib1dARB(unsigned int param0,double param1)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib1dARB");
	ov_glVertexAttrib1dARB=f;
	ov_glVertexAttrib1dARB(param0,param1);
}
void (APIENTRY *ov_glVertexAttrib1dARB)(unsigned int param0,double param1)=init_ov_glVertexAttrib1dARB;

void __gl_ml_glVertexAttrib1dARB(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	double l1=Double_val(v1);
	ov_glVertexAttrib1dARB(l0,l1);
}
extern void (APIENTRY *ov_glWindowPos3svARB)(signed short* param0);

static void APIENTRY init_ov_glWindowPos3svARB(signed short* param0)
{
	void *f;
	f=(void*)getfunc("glWindowPos3svARB");
	ov_glWindowPos3svARB=f;
	ov_glWindowPos3svARB(param0);
}
void (APIENTRY *ov_glWindowPos3svARB)(signed short* param0)=init_ov_glWindowPos3svARB;

void __gl_ml_glWindowPos3svARB(value v0)
{
	signed short* l0=(signed short*)(gl__ba__(v0));
	ov_glWindowPos3svARB(l0);
}
extern void (APIENTRY *ov_glWindowPos3sARB)(signed short param0,signed short param1,signed short param2);

static void APIENTRY init_ov_glWindowPos3sARB(signed short param0,signed short param1,signed short param2)
{
	void *f;
	f=(void*)getfunc("glWindowPos3sARB");
	ov_glWindowPos3sARB=f;
	ov_glWindowPos3sARB(param0,param1,param2);
}
void (APIENTRY *ov_glWindowPos3sARB)(signed short param0,signed short param1,signed short param2)=init_ov_glWindowPos3sARB;

void __gl_ml_glWindowPos3sARB(value v0,value v1,value v2)
{
	signed short l0=Int_val(v0);
	signed short l1=Int_val(v1);
	signed short l2=Int_val(v2);
	ov_glWindowPos3sARB(l0,l1,l2);
}
extern void (APIENTRY *ov_glWindowPos3ivARB)(signed int* param0);

static void APIENTRY init_ov_glWindowPos3ivARB(signed int* param0)
{
	void *f;
	f=(void*)getfunc("glWindowPos3ivARB");
	ov_glWindowPos3ivARB=f;
	ov_glWindowPos3ivARB(param0);
}
void (APIENTRY *ov_glWindowPos3ivARB)(signed int* param0)=init_ov_glWindowPos3ivARB;

void __gl_ml_glWindowPos3ivARB(value v0)
{
	signed int* l0=(signed int*)(gl__ba__(v0));
	ov_glWindowPos3ivARB(l0);
}
extern void (APIENTRY *ov_glWindowPos3iARB)(signed int param0,signed int param1,signed int param2);

static void APIENTRY init_ov_glWindowPos3iARB(signed int param0,signed int param1,signed int param2)
{
	void *f;
	f=(void*)getfunc("glWindowPos3iARB");
	ov_glWindowPos3iARB=f;
	ov_glWindowPos3iARB(param0,param1,param2);
}
void (APIENTRY *ov_glWindowPos3iARB)(signed int param0,signed int param1,signed int param2)=init_ov_glWindowPos3iARB;

void __gl_ml_glWindowPos3iARB(value v0,value v1,value v2)
{
	signed int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	ov_glWindowPos3iARB(l0,l1,l2);
}
extern void (APIENTRY *ov_glWindowPos3fvARB)(float* param0);

static void APIENTRY init_ov_glWindowPos3fvARB(float* param0)
{
	void *f;
	f=(void*)getfunc("glWindowPos3fvARB");
	ov_glWindowPos3fvARB=f;
	ov_glWindowPos3fvARB(param0);
}
void (APIENTRY *ov_glWindowPos3fvARB)(float* param0)=init_ov_glWindowPos3fvARB;

void __gl_ml_glWindowPos3fvARB(value v0)
{
	float* l0=(float*)(gl__ba__(v0));
	ov_glWindowPos3fvARB(l0);
}
extern void (APIENTRY *ov_glWindowPos3fARB)(float param0,float param1,float param2);

static void APIENTRY init_ov_glWindowPos3fARB(float param0,float param1,float param2)
{
	void *f;
	f=(void*)getfunc("glWindowPos3fARB");
	ov_glWindowPos3fARB=f;
	ov_glWindowPos3fARB(param0,param1,param2);
}
void (APIENTRY *ov_glWindowPos3fARB)(float param0,float param1,float param2)=init_ov_glWindowPos3fARB;

void __gl_ml_glWindowPos3fARB(value v0,value v1,value v2)
{
	float l0=Double_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	ov_glWindowPos3fARB(l0,l1,l2);
}
extern void (APIENTRY *ov_glWindowPos3dvARB)(double* param0);

static void APIENTRY init_ov_glWindowPos3dvARB(double* param0)
{
	void *f;
	f=(void*)getfunc("glWindowPos3dvARB");
	ov_glWindowPos3dvARB=f;
	ov_glWindowPos3dvARB(param0);
}
void (APIENTRY *ov_glWindowPos3dvARB)(double* param0)=init_ov_glWindowPos3dvARB;

void __gl_ml_glWindowPos3dvARB(value v0)
{
	double* l0=(double*)(gl__ba__(v0));
	ov_glWindowPos3dvARB(l0);
}
extern void (APIENTRY *ov_glWindowPos3dARB)(double param0,double param1,double param2);

static void APIENTRY init_ov_glWindowPos3dARB(double param0,double param1,double param2)
{
	void *f;
	f=(void*)getfunc("glWindowPos3dARB");
	ov_glWindowPos3dARB=f;
	ov_glWindowPos3dARB(param0,param1,param2);
}
void (APIENTRY *ov_glWindowPos3dARB)(double param0,double param1,double param2)=init_ov_glWindowPos3dARB;

void __gl_ml_glWindowPos3dARB(value v0,value v1,value v2)
{
	double l0=Double_val(v0);
	double l1=Double_val(v1);
	double l2=Double_val(v2);
	ov_glWindowPos3dARB(l0,l1,l2);
}
extern void (APIENTRY *ov_glWindowPos2svARB)(signed short* param0);

static void APIENTRY init_ov_glWindowPos2svARB(signed short* param0)
{
	void *f;
	f=(void*)getfunc("glWindowPos2svARB");
	ov_glWindowPos2svARB=f;
	ov_glWindowPos2svARB(param0);
}
void (APIENTRY *ov_glWindowPos2svARB)(signed short* param0)=init_ov_glWindowPos2svARB;

void __gl_ml_glWindowPos2svARB(value v0)
{
	signed short* l0=(signed short*)(gl__ba__(v0));
	ov_glWindowPos2svARB(l0);
}
extern void (APIENTRY *ov_glWindowPos2sARB)(signed short param0,signed short param1);

static void APIENTRY init_ov_glWindowPos2sARB(signed short param0,signed short param1)
{
	void *f;
	f=(void*)getfunc("glWindowPos2sARB");
	ov_glWindowPos2sARB=f;
	ov_glWindowPos2sARB(param0,param1);
}
void (APIENTRY *ov_glWindowPos2sARB)(signed short param0,signed short param1)=init_ov_glWindowPos2sARB;

void __gl_ml_glWindowPos2sARB(value v0,value v1)
{
	signed short l0=Int_val(v0);
	signed short l1=Int_val(v1);
	ov_glWindowPos2sARB(l0,l1);
}
extern void (APIENTRY *ov_glWindowPos2ivARB)(signed int* param0);

static void APIENTRY init_ov_glWindowPos2ivARB(signed int* param0)
{
	void *f;
	f=(void*)getfunc("glWindowPos2ivARB");
	ov_glWindowPos2ivARB=f;
	ov_glWindowPos2ivARB(param0);
}
void (APIENTRY *ov_glWindowPos2ivARB)(signed int* param0)=init_ov_glWindowPos2ivARB;

void __gl_ml_glWindowPos2ivARB(value v0)
{
	signed int* l0=(signed int*)(gl__ba__(v0));
	ov_glWindowPos2ivARB(l0);
}
extern void (APIENTRY *ov_glWindowPos2iARB)(signed int param0,signed int param1);

static void APIENTRY init_ov_glWindowPos2iARB(signed int param0,signed int param1)
{
	void *f;
	f=(void*)getfunc("glWindowPos2iARB");
	ov_glWindowPos2iARB=f;
	ov_glWindowPos2iARB(param0,param1);
}
void (APIENTRY *ov_glWindowPos2iARB)(signed int param0,signed int param1)=init_ov_glWindowPos2iARB;

void __gl_ml_glWindowPos2iARB(value v0,value v1)
{
	signed int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	ov_glWindowPos2iARB(l0,l1);
}
extern void (APIENTRY *ov_glWindowPos2fvARB)(float* param0);

static void APIENTRY init_ov_glWindowPos2fvARB(float* param0)
{
	void *f;
	f=(void*)getfunc("glWindowPos2fvARB");
	ov_glWindowPos2fvARB=f;
	ov_glWindowPos2fvARB(param0);
}
void (APIENTRY *ov_glWindowPos2fvARB)(float* param0)=init_ov_glWindowPos2fvARB;

void __gl_ml_glWindowPos2fvARB(value v0)
{
	float* l0=(float*)(gl__ba__(v0));
	ov_glWindowPos2fvARB(l0);
}
extern void (APIENTRY *ov_glWindowPos2fARB)(float param0,float param1);

static void APIENTRY init_ov_glWindowPos2fARB(float param0,float param1)
{
	void *f;
	f=(void*)getfunc("glWindowPos2fARB");
	ov_glWindowPos2fARB=f;
	ov_glWindowPos2fARB(param0,param1);
}
void (APIENTRY *ov_glWindowPos2fARB)(float param0,float param1)=init_ov_glWindowPos2fARB;

void __gl_ml_glWindowPos2fARB(value v0,value v1)
{
	float l0=Double_val(v0);
	float l1=Double_val(v1);
	ov_glWindowPos2fARB(l0,l1);
}
extern void (APIENTRY *ov_glWindowPos2dvARB)(double* param0);

static void APIENTRY init_ov_glWindowPos2dvARB(double* param0)
{
	void *f;
	f=(void*)getfunc("glWindowPos2dvARB");
	ov_glWindowPos2dvARB=f;
	ov_glWindowPos2dvARB(param0);
}
void (APIENTRY *ov_glWindowPos2dvARB)(double* param0)=init_ov_glWindowPos2dvARB;

void __gl_ml_glWindowPos2dvARB(value v0)
{
	double* l0=(double*)(gl__ba__(v0));
	ov_glWindowPos2dvARB(l0);
}
extern void (APIENTRY *ov_glWindowPos2dARB)(double param0,double param1);

static void APIENTRY init_ov_glWindowPos2dARB(double param0,double param1)
{
	void *f;
	f=(void*)getfunc("glWindowPos2dARB");
	ov_glWindowPos2dARB=f;
	ov_glWindowPos2dARB(param0,param1);
}
void (APIENTRY *ov_glWindowPos2dARB)(double param0,double param1)=init_ov_glWindowPos2dARB;

void __gl_ml_glWindowPos2dARB(value v0,value v1)
{
	double l0=Double_val(v0);
	double l1=Double_val(v1);
	ov_glWindowPos2dARB(l0,l1);
}
extern void (APIENTRY *ov_glDrawRangeElementArrayATI)(unsigned int param0,unsigned int param1,unsigned int param2,signed int param3);

static void APIENTRY init_ov_glDrawRangeElementArrayATI(unsigned int param0,unsigned int param1,unsigned int param2,signed int param3)
{
	void *f;
	f=(void*)getfunc("glDrawRangeElementArrayATI");
	ov_glDrawRangeElementArrayATI=f;
	ov_glDrawRangeElementArrayATI(param0,param1,param2,param3);
}
void (APIENTRY *ov_glDrawRangeElementArrayATI)(unsigned int param0,unsigned int param1,unsigned int param2,signed int param3)=init_ov_glDrawRangeElementArrayATI;

void __gl_ml_glDrawRangeElementArrayATI(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	ov_glDrawRangeElementArrayATI(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glDrawElementArrayATI)(unsigned int param0,signed int param1);

static void APIENTRY init_ov_glDrawElementArrayATI(unsigned int param0,signed int param1)
{
	void *f;
	f=(void*)getfunc("glDrawElementArrayATI");
	ov_glDrawElementArrayATI=f;
	ov_glDrawElementArrayATI(param0,param1);
}
void (APIENTRY *ov_glDrawElementArrayATI)(unsigned int param0,signed int param1)=init_ov_glDrawElementArrayATI;

void __gl_ml_glDrawElementArrayATI(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	ov_glDrawElementArrayATI(l0,l1);
}
extern void (APIENTRY *ov_glElementPointerATI)(unsigned int param0,void* param1);

static void APIENTRY init_ov_glElementPointerATI(unsigned int param0,void* param1)
{
	void *f;
	f=(void*)getfunc("glElementPointerATI");
	ov_glElementPointerATI=f;
	ov_glElementPointerATI(param0,param1);
}
void (APIENTRY *ov_glElementPointerATI)(unsigned int param0,void* param1)=init_ov_glElementPointerATI;

void __gl_ml_glElementPointerATI(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	void* l1=gl__ba__(v1);
	ov_glElementPointerATI(l0,l1);
}
extern void (APIENTRY *ov_glGetTexBumpParameterfvATI)(unsigned int param0,float* param1);

static void APIENTRY init_ov_glGetTexBumpParameterfvATI(unsigned int param0,float* param1)
{
	void *f;
	f=(void*)getfunc("glGetTexBumpParameterfvATI");
	ov_glGetTexBumpParameterfvATI=f;
	ov_glGetTexBumpParameterfvATI(param0,param1);
}
void (APIENTRY *ov_glGetTexBumpParameterfvATI)(unsigned int param0,float* param1)=init_ov_glGetTexBumpParameterfvATI;

void __gl_ml_glGetTexBumpParameterfvATI(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float* l1=(float*)(gl__ba__(v1));
	ov_glGetTexBumpParameterfvATI(l0,l1);
}
extern void (APIENTRY *ov_glGetTexBumpParameterivATI)(unsigned int param0,signed int* param1);

static void APIENTRY init_ov_glGetTexBumpParameterivATI(unsigned int param0,signed int* param1)
{
	void *f;
	f=(void*)getfunc("glGetTexBumpParameterivATI");
	ov_glGetTexBumpParameterivATI=f;
	ov_glGetTexBumpParameterivATI(param0,param1);
}
void (APIENTRY *ov_glGetTexBumpParameterivATI)(unsigned int param0,signed int* param1)=init_ov_glGetTexBumpParameterivATI;

void __gl_ml_glGetTexBumpParameterivATI(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed int* l1=(signed int*)(gl__ba__(v1));
	ov_glGetTexBumpParameterivATI(l0,l1);
}
extern void (APIENTRY *ov_glTexBumpParameterfvATI)(unsigned int param0,float* param1);

static void APIENTRY init_ov_glTexBumpParameterfvATI(unsigned int param0,float* param1)
{
	void *f;
	f=(void*)getfunc("glTexBumpParameterfvATI");
	ov_glTexBumpParameterfvATI=f;
	ov_glTexBumpParameterfvATI(param0,param1);
}
void (APIENTRY *ov_glTexBumpParameterfvATI)(unsigned int param0,float* param1)=init_ov_glTexBumpParameterfvATI;

void __gl_ml_glTexBumpParameterfvATI(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float* l1=(float*)(gl__ba__(v1));
	ov_glTexBumpParameterfvATI(l0,l1);
}
extern void (APIENTRY *ov_glTexBumpParameterivATI)(unsigned int param0,signed int* param1);

static void APIENTRY init_ov_glTexBumpParameterivATI(unsigned int param0,signed int* param1)
{
	void *f;
	f=(void*)getfunc("glTexBumpParameterivATI");
	ov_glTexBumpParameterivATI=f;
	ov_glTexBumpParameterivATI(param0,param1);
}
void (APIENTRY *ov_glTexBumpParameterivATI)(unsigned int param0,signed int* param1)=init_ov_glTexBumpParameterivATI;

void __gl_ml_glTexBumpParameterivATI(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed int* l1=(signed int*)(gl__ba__(v1));
	ov_glTexBumpParameterivATI(l0,l1);
}
extern void (APIENTRY *ov_glSetFragmentShaderConstantATI)(unsigned int param0,float* param1);

static void APIENTRY init_ov_glSetFragmentShaderConstantATI(unsigned int param0,float* param1)
{
	void *f;
	f=(void*)getfunc("glSetFragmentShaderConstantATI");
	ov_glSetFragmentShaderConstantATI=f;
	ov_glSetFragmentShaderConstantATI(param0,param1);
}
void (APIENTRY *ov_glSetFragmentShaderConstantATI)(unsigned int param0,float* param1)=init_ov_glSetFragmentShaderConstantATI;

void __gl_ml_glSetFragmentShaderConstantATI(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float* l1=(float*)(gl__ba__(v1));
	ov_glSetFragmentShaderConstantATI(l0,l1);
}
extern void (APIENTRY *ov_glAlphaFragmentOp3ATI)(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3,unsigned int param4,unsigned int param5,unsigned int param6,unsigned int param7,unsigned int param8,unsigned int param9,unsigned int param10,unsigned int param11);

static void APIENTRY init_ov_glAlphaFragmentOp3ATI(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3,unsigned int param4,unsigned int param5,unsigned int param6,unsigned int param7,unsigned int param8,unsigned int param9,unsigned int param10,unsigned int param11)
{
	void *f;
	f=(void*)getfunc("glAlphaFragmentOp3ATI");
	ov_glAlphaFragmentOp3ATI=f;
	ov_glAlphaFragmentOp3ATI(param0,param1,param2,param3,param4,param5,param6,param7,param8,param9,param10,param11);
}
void (APIENTRY *ov_glAlphaFragmentOp3ATI)(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3,unsigned int param4,unsigned int param5,unsigned int param6,unsigned int param7,unsigned int param8,unsigned int param9,unsigned int param10,unsigned int param11)=init_ov_glAlphaFragmentOp3ATI;

void __gl_ml_glAlphaFragmentOp3ATI(value v0,value v1,value v2,value v3,value v4,value v5,value v6,value v7,value v8,value v9,value v10,value v11)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	unsigned int l3=Int_val(v3);
	unsigned int l4=Int_val(v4);
	unsigned int l5=Int_val(v5);
	unsigned int l6=Int_val(v6);
	unsigned int l7=Int_val(v7);
	unsigned int l8=Int_val(v8);
	unsigned int l9=Int_val(v9);
	unsigned int l10=Int_val(v10);
	unsigned int l11=Int_val(v11);
	ov_glAlphaFragmentOp3ATI(l0,l1,l2,l3,l4,l5,l6,l7,l8,l9,l10,l11);
}

void byte___gl_ml_glAlphaFragmentOp3ATI(value * argv,int argn)
{
	__gl_ml_glAlphaFragmentOp3ATI(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6],argv[7],argv[8],argv[9],argv[10],argv[11]);
}
extern void (APIENTRY *ov_glAlphaFragmentOp2ATI)(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3,unsigned int param4,unsigned int param5,unsigned int param6,unsigned int param7,unsigned int param8);

static void APIENTRY init_ov_glAlphaFragmentOp2ATI(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3,unsigned int param4,unsigned int param5,unsigned int param6,unsigned int param7,unsigned int param8)
{
	void *f;
	f=(void*)getfunc("glAlphaFragmentOp2ATI");
	ov_glAlphaFragmentOp2ATI=f;
	ov_glAlphaFragmentOp2ATI(param0,param1,param2,param3,param4,param5,param6,param7,param8);
}
void (APIENTRY *ov_glAlphaFragmentOp2ATI)(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3,unsigned int param4,unsigned int param5,unsigned int param6,unsigned int param7,unsigned int param8)=init_ov_glAlphaFragmentOp2ATI;

void __gl_ml_glAlphaFragmentOp2ATI(value v0,value v1,value v2,value v3,value v4,value v5,value v6,value v7,value v8)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	unsigned int l3=Int_val(v3);
	unsigned int l4=Int_val(v4);
	unsigned int l5=Int_val(v5);
	unsigned int l6=Int_val(v6);
	unsigned int l7=Int_val(v7);
	unsigned int l8=Int_val(v8);
	ov_glAlphaFragmentOp2ATI(l0,l1,l2,l3,l4,l5,l6,l7,l8);
}

void byte___gl_ml_glAlphaFragmentOp2ATI(value * argv,int argn)
{
	__gl_ml_glAlphaFragmentOp2ATI(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6],argv[7],argv[8]);
}
extern void (APIENTRY *ov_glAlphaFragmentOp1ATI)(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3,unsigned int param4,unsigned int param5);

static void APIENTRY init_ov_glAlphaFragmentOp1ATI(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3,unsigned int param4,unsigned int param5)
{
	void *f;
	f=(void*)getfunc("glAlphaFragmentOp1ATI");
	ov_glAlphaFragmentOp1ATI=f;
	ov_glAlphaFragmentOp1ATI(param0,param1,param2,param3,param4,param5);
}
void (APIENTRY *ov_glAlphaFragmentOp1ATI)(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3,unsigned int param4,unsigned int param5)=init_ov_glAlphaFragmentOp1ATI;

void __gl_ml_glAlphaFragmentOp1ATI(value v0,value v1,value v2,value v3,value v4,value v5)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	unsigned int l3=Int_val(v3);
	unsigned int l4=Int_val(v4);
	unsigned int l5=Int_val(v5);
	ov_glAlphaFragmentOp1ATI(l0,l1,l2,l3,l4,l5);
}

void byte___gl_ml_glAlphaFragmentOp1ATI(value * argv,int argn)
{
	__gl_ml_glAlphaFragmentOp1ATI(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5]);
}
extern void (APIENTRY *ov_glColorFragmentOp3ATI)(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3,unsigned int param4,unsigned int param5,unsigned int param6,unsigned int param7,unsigned int param8,unsigned int param9,unsigned int param10,unsigned int param11,unsigned int param12);

static void APIENTRY init_ov_glColorFragmentOp3ATI(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3,unsigned int param4,unsigned int param5,unsigned int param6,unsigned int param7,unsigned int param8,unsigned int param9,unsigned int param10,unsigned int param11,unsigned int param12)
{
	void *f;
	f=(void*)getfunc("glColorFragmentOp3ATI");
	ov_glColorFragmentOp3ATI=f;
	ov_glColorFragmentOp3ATI(param0,param1,param2,param3,param4,param5,param6,param7,param8,param9,param10,param11,param12);
}
void (APIENTRY *ov_glColorFragmentOp3ATI)(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3,unsigned int param4,unsigned int param5,unsigned int param6,unsigned int param7,unsigned int param8,unsigned int param9,unsigned int param10,unsigned int param11,unsigned int param12)=init_ov_glColorFragmentOp3ATI;

void __gl_ml_glColorFragmentOp3ATI(value v0,value v1,value v2,value v3,value v4,value v5,value v6,value v7,value v8,value v9,value v10,value v11,value v12)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	unsigned int l3=Int_val(v3);
	unsigned int l4=Int_val(v4);
	unsigned int l5=Int_val(v5);
	unsigned int l6=Int_val(v6);
	unsigned int l7=Int_val(v7);
	unsigned int l8=Int_val(v8);
	unsigned int l9=Int_val(v9);
	unsigned int l10=Int_val(v10);
	unsigned int l11=Int_val(v11);
	unsigned int l12=Int_val(v12);
	ov_glColorFragmentOp3ATI(l0,l1,l2,l3,l4,l5,l6,l7,l8,l9,l10,l11,l12);
}

void byte___gl_ml_glColorFragmentOp3ATI(value * argv,int argn)
{
	__gl_ml_glColorFragmentOp3ATI(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6],argv[7],argv[8],argv[9],argv[10],argv[11],argv[12]);
}
extern void (APIENTRY *ov_glColorFragmentOp2ATI)(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3,unsigned int param4,unsigned int param5,unsigned int param6,unsigned int param7,unsigned int param8,unsigned int param9);

static void APIENTRY init_ov_glColorFragmentOp2ATI(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3,unsigned int param4,unsigned int param5,unsigned int param6,unsigned int param7,unsigned int param8,unsigned int param9)
{
	void *f;
	f=(void*)getfunc("glColorFragmentOp2ATI");
	ov_glColorFragmentOp2ATI=f;
	ov_glColorFragmentOp2ATI(param0,param1,param2,param3,param4,param5,param6,param7,param8,param9);
}
void (APIENTRY *ov_glColorFragmentOp2ATI)(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3,unsigned int param4,unsigned int param5,unsigned int param6,unsigned int param7,unsigned int param8,unsigned int param9)=init_ov_glColorFragmentOp2ATI;

void __gl_ml_glColorFragmentOp2ATI(value v0,value v1,value v2,value v3,value v4,value v5,value v6,value v7,value v8,value v9)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	unsigned int l3=Int_val(v3);
	unsigned int l4=Int_val(v4);
	unsigned int l5=Int_val(v5);
	unsigned int l6=Int_val(v6);
	unsigned int l7=Int_val(v7);
	unsigned int l8=Int_val(v8);
	unsigned int l9=Int_val(v9);
	ov_glColorFragmentOp2ATI(l0,l1,l2,l3,l4,l5,l6,l7,l8,l9);
}

void byte___gl_ml_glColorFragmentOp2ATI(value * argv,int argn)
{
	__gl_ml_glColorFragmentOp2ATI(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6],argv[7],argv[8],argv[9]);
}
extern void (APIENTRY *ov_glColorFragmentOp1ATI)(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3,unsigned int param4,unsigned int param5,unsigned int param6);

static void APIENTRY init_ov_glColorFragmentOp1ATI(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3,unsigned int param4,unsigned int param5,unsigned int param6)
{
	void *f;
	f=(void*)getfunc("glColorFragmentOp1ATI");
	ov_glColorFragmentOp1ATI=f;
	ov_glColorFragmentOp1ATI(param0,param1,param2,param3,param4,param5,param6);
}
void (APIENTRY *ov_glColorFragmentOp1ATI)(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3,unsigned int param4,unsigned int param5,unsigned int param6)=init_ov_glColorFragmentOp1ATI;

void __gl_ml_glColorFragmentOp1ATI(value v0,value v1,value v2,value v3,value v4,value v5,value v6)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	unsigned int l3=Int_val(v3);
	unsigned int l4=Int_val(v4);
	unsigned int l5=Int_val(v5);
	unsigned int l6=Int_val(v6);
	ov_glColorFragmentOp1ATI(l0,l1,l2,l3,l4,l5,l6);
}

void byte___gl_ml_glColorFragmentOp1ATI(value * argv,int argn)
{
	__gl_ml_glColorFragmentOp1ATI(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6]);
}
extern void (APIENTRY *ov_glSampleMapATI)(unsigned int param0,unsigned int param1,unsigned int param2);

static void APIENTRY init_ov_glSampleMapATI(unsigned int param0,unsigned int param1,unsigned int param2)
{
	void *f;
	f=(void*)getfunc("glSampleMapATI");
	ov_glSampleMapATI=f;
	ov_glSampleMapATI(param0,param1,param2);
}
void (APIENTRY *ov_glSampleMapATI)(unsigned int param0,unsigned int param1,unsigned int param2)=init_ov_glSampleMapATI;

void __gl_ml_glSampleMapATI(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	ov_glSampleMapATI(l0,l1,l2);
}
extern void (APIENTRY *ov_glPassTexCoordATI)(unsigned int param0,unsigned int param1,unsigned int param2);

static void APIENTRY init_ov_glPassTexCoordATI(unsigned int param0,unsigned int param1,unsigned int param2)
{
	void *f;
	f=(void*)getfunc("glPassTexCoordATI");
	ov_glPassTexCoordATI=f;
	ov_glPassTexCoordATI(param0,param1,param2);
}
void (APIENTRY *ov_glPassTexCoordATI)(unsigned int param0,unsigned int param1,unsigned int param2)=init_ov_glPassTexCoordATI;

void __gl_ml_glPassTexCoordATI(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	ov_glPassTexCoordATI(l0,l1,l2);
}
extern void (APIENTRY *ov_glEndFragmentShaderATI)();

static void APIENTRY init_ov_glEndFragmentShaderATI()
{
	void *f;
	f=(void*)getfunc("glEndFragmentShaderATI");
	ov_glEndFragmentShaderATI=f;
	ov_glEndFragmentShaderATI();
}
void (APIENTRY *ov_glEndFragmentShaderATI)()=init_ov_glEndFragmentShaderATI;

void __gl_ml_glEndFragmentShaderATI()
{
	ov_glEndFragmentShaderATI();
}
extern void (APIENTRY *ov_glBeginFragmentShaderATI)();

static void APIENTRY init_ov_glBeginFragmentShaderATI()
{
	void *f;
	f=(void*)getfunc("glBeginFragmentShaderATI");
	ov_glBeginFragmentShaderATI=f;
	ov_glBeginFragmentShaderATI();
}
void (APIENTRY *ov_glBeginFragmentShaderATI)()=init_ov_glBeginFragmentShaderATI;

void __gl_ml_glBeginFragmentShaderATI()
{
	ov_glBeginFragmentShaderATI();
}
extern void (APIENTRY *ov_glDeleteFragmentShaderATI)(unsigned int param0);

static void APIENTRY init_ov_glDeleteFragmentShaderATI(unsigned int param0)
{
	void *f;
	f=(void*)getfunc("glDeleteFragmentShaderATI");
	ov_glDeleteFragmentShaderATI=f;
	ov_glDeleteFragmentShaderATI(param0);
}
void (APIENTRY *ov_glDeleteFragmentShaderATI)(unsigned int param0)=init_ov_glDeleteFragmentShaderATI;

void __gl_ml_glDeleteFragmentShaderATI(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glDeleteFragmentShaderATI(l0);
}
extern void (APIENTRY *ov_glBindFragmentShaderATI)(unsigned int param0);

static void APIENTRY init_ov_glBindFragmentShaderATI(unsigned int param0)
{
	void *f;
	f=(void*)getfunc("glBindFragmentShaderATI");
	ov_glBindFragmentShaderATI=f;
	ov_glBindFragmentShaderATI(param0);
}
void (APIENTRY *ov_glBindFragmentShaderATI)(unsigned int param0)=init_ov_glBindFragmentShaderATI;

void __gl_ml_glBindFragmentShaderATI(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glBindFragmentShaderATI(l0);
}
extern unsigned int (APIENTRY *ov_glGenFragmentShadersATI)(unsigned int param0);

static unsigned int APIENTRY init_ov_glGenFragmentShadersATI(unsigned int param0)
{
	void *f;
	f=(void*)getfunc("glGenFragmentShadersATI");
	ov_glGenFragmentShadersATI=f;
	return ov_glGenFragmentShadersATI(param0);
}
unsigned int (APIENTRY *ov_glGenFragmentShadersATI)(unsigned int param0)=init_ov_glGenFragmentShadersATI;

value __gl_ml_glGenFragmentShadersATI(value v0)
{
	unsigned int l0=Int_val(v0);
	unsigned int r=ov_glGenFragmentShadersATI(l0);
	if(((unsigned int)(r)) & 0x40000000) failwith("glGenFragmentShadersATI int out of range");
	return(Val_int(r));
}
extern void (APIENTRY *ov_glPNTrianglesfATI)(unsigned int param0,float param1);

static void APIENTRY init_ov_glPNTrianglesfATI(unsigned int param0,float param1)
{
	void *f;
	f=(void*)getfunc("glPNTrianglesfATI");
	ov_glPNTrianglesfATI=f;
	ov_glPNTrianglesfATI(param0,param1);
}
void (APIENTRY *ov_glPNTrianglesfATI)(unsigned int param0,float param1)=init_ov_glPNTrianglesfATI;

void __gl_ml_glPNTrianglesfATI(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float l1=Double_val(v1);
	ov_glPNTrianglesfATI(l0,l1);
}
extern void (APIENTRY *ov_glPNTrianglesiATI)(unsigned int param0,signed int param1);

static void APIENTRY init_ov_glPNTrianglesiATI(unsigned int param0,signed int param1)
{
	void *f;
	f=(void*)getfunc("glPNTrianglesiATI");
	ov_glPNTrianglesiATI=f;
	ov_glPNTrianglesiATI(param0,param1);
}
void (APIENTRY *ov_glPNTrianglesiATI)(unsigned int param0,signed int param1)=init_ov_glPNTrianglesiATI;

void __gl_ml_glPNTrianglesiATI(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	ov_glPNTrianglesiATI(l0,l1);
}
extern void (APIENTRY *ov_glGetVariantArrayObjectivATI)(unsigned int param0,unsigned int param1,signed int* param2);

static void APIENTRY init_ov_glGetVariantArrayObjectivATI(unsigned int param0,unsigned int param1,signed int* param2)
{
	void *f;
	f=(void*)getfunc("glGetVariantArrayObjectivATI");
	ov_glGetVariantArrayObjectivATI=f;
	ov_glGetVariantArrayObjectivATI(param0,param1,param2);
}
void (APIENTRY *ov_glGetVariantArrayObjectivATI)(unsigned int param0,unsigned int param1,signed int* param2)=init_ov_glGetVariantArrayObjectivATI;

void __gl_ml_glGetVariantArrayObjectivATI(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int* l2=(signed int*)(gl__ba__(v2));
	ov_glGetVariantArrayObjectivATI(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetVariantArrayObjectfvATI)(unsigned int param0,unsigned int param1,float* param2);

static void APIENTRY init_ov_glGetVariantArrayObjectfvATI(unsigned int param0,unsigned int param1,float* param2)
{
	void *f;
	f=(void*)getfunc("glGetVariantArrayObjectfvATI");
	ov_glGetVariantArrayObjectfvATI=f;
	ov_glGetVariantArrayObjectfvATI(param0,param1,param2);
}
void (APIENTRY *ov_glGetVariantArrayObjectfvATI)(unsigned int param0,unsigned int param1,float* param2)=init_ov_glGetVariantArrayObjectfvATI;

void __gl_ml_glGetVariantArrayObjectfvATI(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glGetVariantArrayObjectfvATI(l0,l1,l2);
}
extern void (APIENTRY *ov_glVariantArrayObjectATI)(unsigned int param0,unsigned int param1,signed int param2,unsigned int param3,unsigned int param4);

static void APIENTRY init_ov_glVariantArrayObjectATI(unsigned int param0,unsigned int param1,signed int param2,unsigned int param3,unsigned int param4)
{
	void *f;
	f=(void*)getfunc("glVariantArrayObjectATI");
	ov_glVariantArrayObjectATI=f;
	ov_glVariantArrayObjectATI(param0,param1,param2,param3,param4);
}
void (APIENTRY *ov_glVariantArrayObjectATI)(unsigned int param0,unsigned int param1,signed int param2,unsigned int param3,unsigned int param4)=init_ov_glVariantArrayObjectATI;

void __gl_ml_glVariantArrayObjectATI(value v0,value v1,value v2,value v3,value v4)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	unsigned int l3=Int_val(v3);
	unsigned int l4=Int_val(v4);
	ov_glVariantArrayObjectATI(l0,l1,l2,l3,l4);
}
extern void (APIENTRY *ov_glGetArrayObjectivATI)(unsigned int param0,unsigned int param1,signed int* param2);

static void APIENTRY init_ov_glGetArrayObjectivATI(unsigned int param0,unsigned int param1,signed int* param2)
{
	void *f;
	f=(void*)getfunc("glGetArrayObjectivATI");
	ov_glGetArrayObjectivATI=f;
	ov_glGetArrayObjectivATI(param0,param1,param2);
}
void (APIENTRY *ov_glGetArrayObjectivATI)(unsigned int param0,unsigned int param1,signed int* param2)=init_ov_glGetArrayObjectivATI;

void __gl_ml_glGetArrayObjectivATI(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int* l2=(signed int*)(gl__ba__(v2));
	ov_glGetArrayObjectivATI(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetArrayObjectfvATI)(unsigned int param0,unsigned int param1,float* param2);

static void APIENTRY init_ov_glGetArrayObjectfvATI(unsigned int param0,unsigned int param1,float* param2)
{
	void *f;
	f=(void*)getfunc("glGetArrayObjectfvATI");
	ov_glGetArrayObjectfvATI=f;
	ov_glGetArrayObjectfvATI(param0,param1,param2);
}
void (APIENTRY *ov_glGetArrayObjectfvATI)(unsigned int param0,unsigned int param1,float* param2)=init_ov_glGetArrayObjectfvATI;

void __gl_ml_glGetArrayObjectfvATI(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glGetArrayObjectfvATI(l0,l1,l2);
}
extern void (APIENTRY *ov_glArrayObjectATI)(unsigned int param0,signed int param1,unsigned int param2,signed int param3,unsigned int param4,unsigned int param5);

static void APIENTRY init_ov_glArrayObjectATI(unsigned int param0,signed int param1,unsigned int param2,signed int param3,unsigned int param4,unsigned int param5)
{
	void *f;
	f=(void*)getfunc("glArrayObjectATI");
	ov_glArrayObjectATI=f;
	ov_glArrayObjectATI(param0,param1,param2,param3,param4,param5);
}
void (APIENTRY *ov_glArrayObjectATI)(unsigned int param0,signed int param1,unsigned int param2,signed int param3,unsigned int param4,unsigned int param5)=init_ov_glArrayObjectATI;

void __gl_ml_glArrayObjectATI(value v0,value v1,value v2,value v3,value v4,value v5)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	unsigned int l4=Int_val(v4);
	unsigned int l5=Int_val(v5);
	ov_glArrayObjectATI(l0,l1,l2,l3,l4,l5);
}

void byte___gl_ml_glArrayObjectATI(value * argv,int argn)
{
	__gl_ml_glArrayObjectATI(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5]);
}
extern void (APIENTRY *ov_glDeleteObjectBufferATI)(unsigned int param0);

static void APIENTRY init_ov_glDeleteObjectBufferATI(unsigned int param0)
{
	void *f;
	f=(void*)getfunc("glDeleteObjectBufferATI");
	ov_glDeleteObjectBufferATI=f;
	ov_glDeleteObjectBufferATI(param0);
}
void (APIENTRY *ov_glDeleteObjectBufferATI)(unsigned int param0)=init_ov_glDeleteObjectBufferATI;

void __gl_ml_glDeleteObjectBufferATI(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glDeleteObjectBufferATI(l0);
}
extern void (APIENTRY *ov_glGetObjectBufferivATI)(unsigned int param0,unsigned int param1,signed int* param2);

static void APIENTRY init_ov_glGetObjectBufferivATI(unsigned int param0,unsigned int param1,signed int* param2)
{
	void *f;
	f=(void*)getfunc("glGetObjectBufferivATI");
	ov_glGetObjectBufferivATI=f;
	ov_glGetObjectBufferivATI(param0,param1,param2);
}
void (APIENTRY *ov_glGetObjectBufferivATI)(unsigned int param0,unsigned int param1,signed int* param2)=init_ov_glGetObjectBufferivATI;

void __gl_ml_glGetObjectBufferivATI(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int* l2=(signed int*)(gl__ba__(v2));
	ov_glGetObjectBufferivATI(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetObjectBufferfvATI)(unsigned int param0,unsigned int param1,float* param2);

static void APIENTRY init_ov_glGetObjectBufferfvATI(unsigned int param0,unsigned int param1,float* param2)
{
	void *f;
	f=(void*)getfunc("glGetObjectBufferfvATI");
	ov_glGetObjectBufferfvATI=f;
	ov_glGetObjectBufferfvATI(param0,param1,param2);
}
void (APIENTRY *ov_glGetObjectBufferfvATI)(unsigned int param0,unsigned int param1,float* param2)=init_ov_glGetObjectBufferfvATI;

void __gl_ml_glGetObjectBufferfvATI(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glGetObjectBufferfvATI(l0,l1,l2);
}
extern void (APIENTRY *ov_glUpdateObjectBufferATI)(unsigned int param0,unsigned int param1,signed int param2,void* param3,unsigned int param4);

static void APIENTRY init_ov_glUpdateObjectBufferATI(unsigned int param0,unsigned int param1,signed int param2,void* param3,unsigned int param4)
{
	void *f;
	f=(void*)getfunc("glUpdateObjectBufferATI");
	ov_glUpdateObjectBufferATI=f;
	ov_glUpdateObjectBufferATI(param0,param1,param2,param3,param4);
}
void (APIENTRY *ov_glUpdateObjectBufferATI)(unsigned int param0,unsigned int param1,signed int param2,void* param3,unsigned int param4)=init_ov_glUpdateObjectBufferATI;

void __gl_ml_glUpdateObjectBufferATI(value v0,value v1,value v2,value v3,value v4)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	void* l3=gl__ba__(v3);
	unsigned int l4=Int_val(v4);
	ov_glUpdateObjectBufferATI(l0,l1,l2,l3,l4);
}
extern unsigned char (APIENTRY *ov_glIsObjectBufferATI)(unsigned int param0);

static unsigned char APIENTRY init_ov_glIsObjectBufferATI(unsigned int param0)
{
	void *f;
	f=(void*)getfunc("glIsObjectBufferATI");
	ov_glIsObjectBufferATI=f;
	return ov_glIsObjectBufferATI(param0);
}
unsigned char (APIENTRY *ov_glIsObjectBufferATI)(unsigned int param0)=init_ov_glIsObjectBufferATI;

value __gl_ml_glIsObjectBufferATI(value v0)
{
	unsigned int l0=Int_val(v0);
	unsigned char r=ov_glIsObjectBufferATI(l0);
	return(Val_int(r));
}
extern unsigned int (APIENTRY *ov_glNewObjectBufferATI)(signed int param0,void* param1,unsigned int param2);

static unsigned int APIENTRY init_ov_glNewObjectBufferATI(signed int param0,void* param1,unsigned int param2)
{
	void *f;
	f=(void*)getfunc("glNewObjectBufferATI");
	ov_glNewObjectBufferATI=f;
	return ov_glNewObjectBufferATI(param0,param1,param2);
}
unsigned int (APIENTRY *ov_glNewObjectBufferATI)(signed int param0,void* param1,unsigned int param2)=init_ov_glNewObjectBufferATI;

value __gl_ml_glNewObjectBufferATI(value v0,value v1,value v2)
{
	signed int l0=Int_val(v0);
	void* l1=gl__ba__(v1);
	unsigned int l2=Int_val(v2);
	unsigned int r=ov_glNewObjectBufferATI(l0,l1,l2);
	if(((unsigned int)(r)) & 0x40000000) failwith("glNewObjectBufferATI int out of range");
	return(Val_int(r));
}
extern void (APIENTRY *ov_glVertexBlendEnvfATI)(unsigned int param0,float param1);

static void APIENTRY init_ov_glVertexBlendEnvfATI(unsigned int param0,float param1)
{
	void *f;
	f=(void*)getfunc("glVertexBlendEnvfATI");
	ov_glVertexBlendEnvfATI=f;
	ov_glVertexBlendEnvfATI(param0,param1);
}
void (APIENTRY *ov_glVertexBlendEnvfATI)(unsigned int param0,float param1)=init_ov_glVertexBlendEnvfATI;

void __gl_ml_glVertexBlendEnvfATI(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float l1=Double_val(v1);
	ov_glVertexBlendEnvfATI(l0,l1);
}
extern void (APIENTRY *ov_glVertexBlendEnviATI)(unsigned int param0,signed int param1);

static void APIENTRY init_ov_glVertexBlendEnviATI(unsigned int param0,signed int param1)
{
	void *f;
	f=(void*)getfunc("glVertexBlendEnviATI");
	ov_glVertexBlendEnviATI=f;
	ov_glVertexBlendEnviATI(param0,param1);
}
void (APIENTRY *ov_glVertexBlendEnviATI)(unsigned int param0,signed int param1)=init_ov_glVertexBlendEnviATI;

void __gl_ml_glVertexBlendEnviATI(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	ov_glVertexBlendEnviATI(l0,l1);
}
extern void (APIENTRY *ov_glClientActiveVertexStreamATI)(unsigned int param0);

static void APIENTRY init_ov_glClientActiveVertexStreamATI(unsigned int param0)
{
	void *f;
	f=(void*)getfunc("glClientActiveVertexStreamATI");
	ov_glClientActiveVertexStreamATI=f;
	ov_glClientActiveVertexStreamATI(param0);
}
void (APIENTRY *ov_glClientActiveVertexStreamATI)(unsigned int param0)=init_ov_glClientActiveVertexStreamATI;

void __gl_ml_glClientActiveVertexStreamATI(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glClientActiveVertexStreamATI(l0);
}
extern void (APIENTRY *ov_glNormalStream3dvATI)(unsigned int param0,double* param1);

static void APIENTRY init_ov_glNormalStream3dvATI(unsigned int param0,double* param1)
{
	void *f;
	f=(void*)getfunc("glNormalStream3dvATI");
	ov_glNormalStream3dvATI=f;
	ov_glNormalStream3dvATI(param0,param1);
}
void (APIENTRY *ov_glNormalStream3dvATI)(unsigned int param0,double* param1)=init_ov_glNormalStream3dvATI;

void __gl_ml_glNormalStream3dvATI(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	double* l1=(double*)(gl__ba__(v1));
	ov_glNormalStream3dvATI(l0,l1);
}
extern void (APIENTRY *ov_glNormalStream3dATI)(unsigned int param0,double param1,double param2,double param3);

static void APIENTRY init_ov_glNormalStream3dATI(unsigned int param0,double param1,double param2,double param3)
{
	void *f;
	f=(void*)getfunc("glNormalStream3dATI");
	ov_glNormalStream3dATI=f;
	ov_glNormalStream3dATI(param0,param1,param2,param3);
}
void (APIENTRY *ov_glNormalStream3dATI)(unsigned int param0,double param1,double param2,double param3)=init_ov_glNormalStream3dATI;

void __gl_ml_glNormalStream3dATI(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	double l1=Double_val(v1);
	double l2=Double_val(v2);
	double l3=Double_val(v3);
	ov_glNormalStream3dATI(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glNormalStream3fvATI)(unsigned int param0,float* param1);

static void APIENTRY init_ov_glNormalStream3fvATI(unsigned int param0,float* param1)
{
	void *f;
	f=(void*)getfunc("glNormalStream3fvATI");
	ov_glNormalStream3fvATI=f;
	ov_glNormalStream3fvATI(param0,param1);
}
void (APIENTRY *ov_glNormalStream3fvATI)(unsigned int param0,float* param1)=init_ov_glNormalStream3fvATI;

void __gl_ml_glNormalStream3fvATI(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float* l1=(float*)(gl__ba__(v1));
	ov_glNormalStream3fvATI(l0,l1);
}
extern void (APIENTRY *ov_glNormalStream3fATI)(unsigned int param0,float param1,float param2,float param3);

static void APIENTRY init_ov_glNormalStream3fATI(unsigned int param0,float param1,float param2,float param3)
{
	void *f;
	f=(void*)getfunc("glNormalStream3fATI");
	ov_glNormalStream3fATI=f;
	ov_glNormalStream3fATI(param0,param1,param2,param3);
}
void (APIENTRY *ov_glNormalStream3fATI)(unsigned int param0,float param1,float param2,float param3)=init_ov_glNormalStream3fATI;

void __gl_ml_glNormalStream3fATI(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	float l3=Double_val(v3);
	ov_glNormalStream3fATI(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glNormalStream3ivATI)(unsigned int param0,signed int* param1);

static void APIENTRY init_ov_glNormalStream3ivATI(unsigned int param0,signed int* param1)
{
	void *f;
	f=(void*)getfunc("glNormalStream3ivATI");
	ov_glNormalStream3ivATI=f;
	ov_glNormalStream3ivATI(param0,param1);
}
void (APIENTRY *ov_glNormalStream3ivATI)(unsigned int param0,signed int* param1)=init_ov_glNormalStream3ivATI;

void __gl_ml_glNormalStream3ivATI(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed int* l1=(signed int*)(gl__ba__(v1));
	ov_glNormalStream3ivATI(l0,l1);
}
extern void (APIENTRY *ov_glNormalStream3iATI)(unsigned int param0,signed int param1,signed int param2,signed int param3);

static void APIENTRY init_ov_glNormalStream3iATI(unsigned int param0,signed int param1,signed int param2,signed int param3)
{
	void *f;
	f=(void*)getfunc("glNormalStream3iATI");
	ov_glNormalStream3iATI=f;
	ov_glNormalStream3iATI(param0,param1,param2,param3);
}
void (APIENTRY *ov_glNormalStream3iATI)(unsigned int param0,signed int param1,signed int param2,signed int param3)=init_ov_glNormalStream3iATI;

void __gl_ml_glNormalStream3iATI(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	ov_glNormalStream3iATI(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glNormalStream3svATI)(unsigned int param0,signed short* param1);

static void APIENTRY init_ov_glNormalStream3svATI(unsigned int param0,signed short* param1)
{
	void *f;
	f=(void*)getfunc("glNormalStream3svATI");
	ov_glNormalStream3svATI=f;
	ov_glNormalStream3svATI(param0,param1);
}
void (APIENTRY *ov_glNormalStream3svATI)(unsigned int param0,signed short* param1)=init_ov_glNormalStream3svATI;

void __gl_ml_glNormalStream3svATI(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed short* l1=(signed short*)(gl__ba__(v1));
	ov_glNormalStream3svATI(l0,l1);
}
extern void (APIENTRY *ov_glNormalStream3sATI)(unsigned int param0,signed short param1,signed short param2,signed short param3);

static void APIENTRY init_ov_glNormalStream3sATI(unsigned int param0,signed short param1,signed short param2,signed short param3)
{
	void *f;
	f=(void*)getfunc("glNormalStream3sATI");
	ov_glNormalStream3sATI=f;
	ov_glNormalStream3sATI(param0,param1,param2,param3);
}
void (APIENTRY *ov_glNormalStream3sATI)(unsigned int param0,signed short param1,signed short param2,signed short param3)=init_ov_glNormalStream3sATI;

void __gl_ml_glNormalStream3sATI(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	signed short l1=Int_val(v1);
	signed short l2=Int_val(v2);
	signed short l3=Int_val(v3);
	ov_glNormalStream3sATI(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glNormalStream3bvATI)(unsigned int param0,signed char* param1);

static void APIENTRY init_ov_glNormalStream3bvATI(unsigned int param0,signed char* param1)
{
	void *f;
	f=(void*)getfunc("glNormalStream3bvATI");
	ov_glNormalStream3bvATI=f;
	ov_glNormalStream3bvATI(param0,param1);
}
void (APIENTRY *ov_glNormalStream3bvATI)(unsigned int param0,signed char* param1)=init_ov_glNormalStream3bvATI;

void __gl_ml_glNormalStream3bvATI(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed char* l1=(signed char*)(gl__ba__(v1));
	ov_glNormalStream3bvATI(l0,l1);
}
extern void (APIENTRY *ov_glNormalStream3bATI)(unsigned int param0,signed char param1,signed char param2,signed char param3);

static void APIENTRY init_ov_glNormalStream3bATI(unsigned int param0,signed char param1,signed char param2,signed char param3)
{
	void *f;
	f=(void*)getfunc("glNormalStream3bATI");
	ov_glNormalStream3bATI=f;
	ov_glNormalStream3bATI(param0,param1,param2,param3);
}
void (APIENTRY *ov_glNormalStream3bATI)(unsigned int param0,signed char param1,signed char param2,signed char param3)=init_ov_glNormalStream3bATI;

void __gl_ml_glNormalStream3bATI(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	signed char l1=Int_val(v1);
	signed char l2=Int_val(v2);
	signed char l3=Int_val(v3);
	ov_glNormalStream3bATI(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glVertexStream4dvATI)(unsigned int param0,double* param1);

static void APIENTRY init_ov_glVertexStream4dvATI(unsigned int param0,double* param1)
{
	void *f;
	f=(void*)getfunc("glVertexStream4dvATI");
	ov_glVertexStream4dvATI=f;
	ov_glVertexStream4dvATI(param0,param1);
}
void (APIENTRY *ov_glVertexStream4dvATI)(unsigned int param0,double* param1)=init_ov_glVertexStream4dvATI;

void __gl_ml_glVertexStream4dvATI(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	double* l1=(double*)(gl__ba__(v1));
	ov_glVertexStream4dvATI(l0,l1);
}
extern void (APIENTRY *ov_glVertexStream4dATI)(unsigned int param0,double param1,double param2,double param3,double param4);

static void APIENTRY init_ov_glVertexStream4dATI(unsigned int param0,double param1,double param2,double param3,double param4)
{
	void *f;
	f=(void*)getfunc("glVertexStream4dATI");
	ov_glVertexStream4dATI=f;
	ov_glVertexStream4dATI(param0,param1,param2,param3,param4);
}
void (APIENTRY *ov_glVertexStream4dATI)(unsigned int param0,double param1,double param2,double param3,double param4)=init_ov_glVertexStream4dATI;

void __gl_ml_glVertexStream4dATI(value v0,value v1,value v2,value v3,value v4)
{
	unsigned int l0=Int_val(v0);
	double l1=Double_val(v1);
	double l2=Double_val(v2);
	double l3=Double_val(v3);
	double l4=Double_val(v4);
	ov_glVertexStream4dATI(l0,l1,l2,l3,l4);
}
extern void (APIENTRY *ov_glVertexStream4fvATI)(unsigned int param0,float* param1);

static void APIENTRY init_ov_glVertexStream4fvATI(unsigned int param0,float* param1)
{
	void *f;
	f=(void*)getfunc("glVertexStream4fvATI");
	ov_glVertexStream4fvATI=f;
	ov_glVertexStream4fvATI(param0,param1);
}
void (APIENTRY *ov_glVertexStream4fvATI)(unsigned int param0,float* param1)=init_ov_glVertexStream4fvATI;

void __gl_ml_glVertexStream4fvATI(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float* l1=(float*)(gl__ba__(v1));
	ov_glVertexStream4fvATI(l0,l1);
}
extern void (APIENTRY *ov_glVertexStream4fATI)(unsigned int param0,float param1,float param2,float param3,float param4);

static void APIENTRY init_ov_glVertexStream4fATI(unsigned int param0,float param1,float param2,float param3,float param4)
{
	void *f;
	f=(void*)getfunc("glVertexStream4fATI");
	ov_glVertexStream4fATI=f;
	ov_glVertexStream4fATI(param0,param1,param2,param3,param4);
}
void (APIENTRY *ov_glVertexStream4fATI)(unsigned int param0,float param1,float param2,float param3,float param4)=init_ov_glVertexStream4fATI;

void __gl_ml_glVertexStream4fATI(value v0,value v1,value v2,value v3,value v4)
{
	unsigned int l0=Int_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	float l3=Double_val(v3);
	float l4=Double_val(v4);
	ov_glVertexStream4fATI(l0,l1,l2,l3,l4);
}
extern void (APIENTRY *ov_glVertexStream4ivATI)(unsigned int param0,signed int* param1);

static void APIENTRY init_ov_glVertexStream4ivATI(unsigned int param0,signed int* param1)
{
	void *f;
	f=(void*)getfunc("glVertexStream4ivATI");
	ov_glVertexStream4ivATI=f;
	ov_glVertexStream4ivATI(param0,param1);
}
void (APIENTRY *ov_glVertexStream4ivATI)(unsigned int param0,signed int* param1)=init_ov_glVertexStream4ivATI;

void __gl_ml_glVertexStream4ivATI(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed int* l1=(signed int*)(gl__ba__(v1));
	ov_glVertexStream4ivATI(l0,l1);
}
extern void (APIENTRY *ov_glVertexStream4iATI)(unsigned int param0,signed int param1,signed int param2,signed int param3,signed int param4);

static void APIENTRY init_ov_glVertexStream4iATI(unsigned int param0,signed int param1,signed int param2,signed int param3,signed int param4)
{
	void *f;
	f=(void*)getfunc("glVertexStream4iATI");
	ov_glVertexStream4iATI=f;
	ov_glVertexStream4iATI(param0,param1,param2,param3,param4);
}
void (APIENTRY *ov_glVertexStream4iATI)(unsigned int param0,signed int param1,signed int param2,signed int param3,signed int param4)=init_ov_glVertexStream4iATI;

void __gl_ml_glVertexStream4iATI(value v0,value v1,value v2,value v3,value v4)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	signed int l4=Int_val(v4);
	ov_glVertexStream4iATI(l0,l1,l2,l3,l4);
}
extern void (APIENTRY *ov_glVertexStream4svATI)(unsigned int param0,signed short* param1);

static void APIENTRY init_ov_glVertexStream4svATI(unsigned int param0,signed short* param1)
{
	void *f;
	f=(void*)getfunc("glVertexStream4svATI");
	ov_glVertexStream4svATI=f;
	ov_glVertexStream4svATI(param0,param1);
}
void (APIENTRY *ov_glVertexStream4svATI)(unsigned int param0,signed short* param1)=init_ov_glVertexStream4svATI;

void __gl_ml_glVertexStream4svATI(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed short* l1=(signed short*)(gl__ba__(v1));
	ov_glVertexStream4svATI(l0,l1);
}
extern void (APIENTRY *ov_glVertexStream4sATI)(unsigned int param0,signed short param1,signed short param2,signed short param3,signed short param4);

static void APIENTRY init_ov_glVertexStream4sATI(unsigned int param0,signed short param1,signed short param2,signed short param3,signed short param4)
{
	void *f;
	f=(void*)getfunc("glVertexStream4sATI");
	ov_glVertexStream4sATI=f;
	ov_glVertexStream4sATI(param0,param1,param2,param3,param4);
}
void (APIENTRY *ov_glVertexStream4sATI)(unsigned int param0,signed short param1,signed short param2,signed short param3,signed short param4)=init_ov_glVertexStream4sATI;

void __gl_ml_glVertexStream4sATI(value v0,value v1,value v2,value v3,value v4)
{
	unsigned int l0=Int_val(v0);
	signed short l1=Int_val(v1);
	signed short l2=Int_val(v2);
	signed short l3=Int_val(v3);
	signed short l4=Int_val(v4);
	ov_glVertexStream4sATI(l0,l1,l2,l3,l4);
}
extern void (APIENTRY *ov_glVertexStream3dvATI)(unsigned int param0,double* param1);

static void APIENTRY init_ov_glVertexStream3dvATI(unsigned int param0,double* param1)
{
	void *f;
	f=(void*)getfunc("glVertexStream3dvATI");
	ov_glVertexStream3dvATI=f;
	ov_glVertexStream3dvATI(param0,param1);
}
void (APIENTRY *ov_glVertexStream3dvATI)(unsigned int param0,double* param1)=init_ov_glVertexStream3dvATI;

void __gl_ml_glVertexStream3dvATI(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	double* l1=(double*)(gl__ba__(v1));
	ov_glVertexStream3dvATI(l0,l1);
}
extern void (APIENTRY *ov_glVertexStream3dATI)(unsigned int param0,double param1,double param2,double param3);

static void APIENTRY init_ov_glVertexStream3dATI(unsigned int param0,double param1,double param2,double param3)
{
	void *f;
	f=(void*)getfunc("glVertexStream3dATI");
	ov_glVertexStream3dATI=f;
	ov_glVertexStream3dATI(param0,param1,param2,param3);
}
void (APIENTRY *ov_glVertexStream3dATI)(unsigned int param0,double param1,double param2,double param3)=init_ov_glVertexStream3dATI;

void __gl_ml_glVertexStream3dATI(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	double l1=Double_val(v1);
	double l2=Double_val(v2);
	double l3=Double_val(v3);
	ov_glVertexStream3dATI(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glVertexStream3fvATI)(unsigned int param0,float* param1);

static void APIENTRY init_ov_glVertexStream3fvATI(unsigned int param0,float* param1)
{
	void *f;
	f=(void*)getfunc("glVertexStream3fvATI");
	ov_glVertexStream3fvATI=f;
	ov_glVertexStream3fvATI(param0,param1);
}
void (APIENTRY *ov_glVertexStream3fvATI)(unsigned int param0,float* param1)=init_ov_glVertexStream3fvATI;

void __gl_ml_glVertexStream3fvATI(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float* l1=(float*)(gl__ba__(v1));
	ov_glVertexStream3fvATI(l0,l1);
}
extern void (APIENTRY *ov_glVertexStream3fATI)(unsigned int param0,float param1,float param2,float param3);

static void APIENTRY init_ov_glVertexStream3fATI(unsigned int param0,float param1,float param2,float param3)
{
	void *f;
	f=(void*)getfunc("glVertexStream3fATI");
	ov_glVertexStream3fATI=f;
	ov_glVertexStream3fATI(param0,param1,param2,param3);
}
void (APIENTRY *ov_glVertexStream3fATI)(unsigned int param0,float param1,float param2,float param3)=init_ov_glVertexStream3fATI;

void __gl_ml_glVertexStream3fATI(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	float l3=Double_val(v3);
	ov_glVertexStream3fATI(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glVertexStream3ivATI)(unsigned int param0,signed int* param1);

static void APIENTRY init_ov_glVertexStream3ivATI(unsigned int param0,signed int* param1)
{
	void *f;
	f=(void*)getfunc("glVertexStream3ivATI");
	ov_glVertexStream3ivATI=f;
	ov_glVertexStream3ivATI(param0,param1);
}
void (APIENTRY *ov_glVertexStream3ivATI)(unsigned int param0,signed int* param1)=init_ov_glVertexStream3ivATI;

void __gl_ml_glVertexStream3ivATI(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed int* l1=(signed int*)(gl__ba__(v1));
	ov_glVertexStream3ivATI(l0,l1);
}
extern void (APIENTRY *ov_glVertexStream3iATI)(unsigned int param0,signed int param1,signed int param2,signed int param3);

static void APIENTRY init_ov_glVertexStream3iATI(unsigned int param0,signed int param1,signed int param2,signed int param3)
{
	void *f;
	f=(void*)getfunc("glVertexStream3iATI");
	ov_glVertexStream3iATI=f;
	ov_glVertexStream3iATI(param0,param1,param2,param3);
}
void (APIENTRY *ov_glVertexStream3iATI)(unsigned int param0,signed int param1,signed int param2,signed int param3)=init_ov_glVertexStream3iATI;

void __gl_ml_glVertexStream3iATI(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	ov_glVertexStream3iATI(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glVertexStream3svATI)(unsigned int param0,signed short* param1);

static void APIENTRY init_ov_glVertexStream3svATI(unsigned int param0,signed short* param1)
{
	void *f;
	f=(void*)getfunc("glVertexStream3svATI");
	ov_glVertexStream3svATI=f;
	ov_glVertexStream3svATI(param0,param1);
}
void (APIENTRY *ov_glVertexStream3svATI)(unsigned int param0,signed short* param1)=init_ov_glVertexStream3svATI;

void __gl_ml_glVertexStream3svATI(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed short* l1=(signed short*)(gl__ba__(v1));
	ov_glVertexStream3svATI(l0,l1);
}
extern void (APIENTRY *ov_glVertexStream3sATI)(unsigned int param0,signed short param1,signed short param2,signed short param3);

static void APIENTRY init_ov_glVertexStream3sATI(unsigned int param0,signed short param1,signed short param2,signed short param3)
{
	void *f;
	f=(void*)getfunc("glVertexStream3sATI");
	ov_glVertexStream3sATI=f;
	ov_glVertexStream3sATI(param0,param1,param2,param3);
}
void (APIENTRY *ov_glVertexStream3sATI)(unsigned int param0,signed short param1,signed short param2,signed short param3)=init_ov_glVertexStream3sATI;

void __gl_ml_glVertexStream3sATI(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	signed short l1=Int_val(v1);
	signed short l2=Int_val(v2);
	signed short l3=Int_val(v3);
	ov_glVertexStream3sATI(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glVertexStream2dvATI)(unsigned int param0,double* param1);

static void APIENTRY init_ov_glVertexStream2dvATI(unsigned int param0,double* param1)
{
	void *f;
	f=(void*)getfunc("glVertexStream2dvATI");
	ov_glVertexStream2dvATI=f;
	ov_glVertexStream2dvATI(param0,param1);
}
void (APIENTRY *ov_glVertexStream2dvATI)(unsigned int param0,double* param1)=init_ov_glVertexStream2dvATI;

void __gl_ml_glVertexStream2dvATI(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	double* l1=(double*)(gl__ba__(v1));
	ov_glVertexStream2dvATI(l0,l1);
}
extern void (APIENTRY *ov_glVertexStream2dATI)(unsigned int param0,double param1,double param2);

static void APIENTRY init_ov_glVertexStream2dATI(unsigned int param0,double param1,double param2)
{
	void *f;
	f=(void*)getfunc("glVertexStream2dATI");
	ov_glVertexStream2dATI=f;
	ov_glVertexStream2dATI(param0,param1,param2);
}
void (APIENTRY *ov_glVertexStream2dATI)(unsigned int param0,double param1,double param2)=init_ov_glVertexStream2dATI;

void __gl_ml_glVertexStream2dATI(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	double l1=Double_val(v1);
	double l2=Double_val(v2);
	ov_glVertexStream2dATI(l0,l1,l2);
}
extern void (APIENTRY *ov_glVertexStream2fvATI)(unsigned int param0,float* param1);

static void APIENTRY init_ov_glVertexStream2fvATI(unsigned int param0,float* param1)
{
	void *f;
	f=(void*)getfunc("glVertexStream2fvATI");
	ov_glVertexStream2fvATI=f;
	ov_glVertexStream2fvATI(param0,param1);
}
void (APIENTRY *ov_glVertexStream2fvATI)(unsigned int param0,float* param1)=init_ov_glVertexStream2fvATI;

void __gl_ml_glVertexStream2fvATI(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float* l1=(float*)(gl__ba__(v1));
	ov_glVertexStream2fvATI(l0,l1);
}
extern void (APIENTRY *ov_glVertexStream2fATI)(unsigned int param0,float param1,float param2);

static void APIENTRY init_ov_glVertexStream2fATI(unsigned int param0,float param1,float param2)
{
	void *f;
	f=(void*)getfunc("glVertexStream2fATI");
	ov_glVertexStream2fATI=f;
	ov_glVertexStream2fATI(param0,param1,param2);
}
void (APIENTRY *ov_glVertexStream2fATI)(unsigned int param0,float param1,float param2)=init_ov_glVertexStream2fATI;

void __gl_ml_glVertexStream2fATI(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	ov_glVertexStream2fATI(l0,l1,l2);
}
extern void (APIENTRY *ov_glVertexStream2ivATI)(unsigned int param0,signed int* param1);

static void APIENTRY init_ov_glVertexStream2ivATI(unsigned int param0,signed int* param1)
{
	void *f;
	f=(void*)getfunc("glVertexStream2ivATI");
	ov_glVertexStream2ivATI=f;
	ov_glVertexStream2ivATI(param0,param1);
}
void (APIENTRY *ov_glVertexStream2ivATI)(unsigned int param0,signed int* param1)=init_ov_glVertexStream2ivATI;

void __gl_ml_glVertexStream2ivATI(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed int* l1=(signed int*)(gl__ba__(v1));
	ov_glVertexStream2ivATI(l0,l1);
}
extern void (APIENTRY *ov_glVertexStream2iATI)(unsigned int param0,signed int param1,signed int param2);

static void APIENTRY init_ov_glVertexStream2iATI(unsigned int param0,signed int param1,signed int param2)
{
	void *f;
	f=(void*)getfunc("glVertexStream2iATI");
	ov_glVertexStream2iATI=f;
	ov_glVertexStream2iATI(param0,param1,param2);
}
void (APIENTRY *ov_glVertexStream2iATI)(unsigned int param0,signed int param1,signed int param2)=init_ov_glVertexStream2iATI;

void __gl_ml_glVertexStream2iATI(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	ov_glVertexStream2iATI(l0,l1,l2);
}
extern void (APIENTRY *ov_glVertexStream2svATI)(unsigned int param0,signed short* param1);

static void APIENTRY init_ov_glVertexStream2svATI(unsigned int param0,signed short* param1)
{
	void *f;
	f=(void*)getfunc("glVertexStream2svATI");
	ov_glVertexStream2svATI=f;
	ov_glVertexStream2svATI(param0,param1);
}
void (APIENTRY *ov_glVertexStream2svATI)(unsigned int param0,signed short* param1)=init_ov_glVertexStream2svATI;

void __gl_ml_glVertexStream2svATI(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed short* l1=(signed short*)(gl__ba__(v1));
	ov_glVertexStream2svATI(l0,l1);
}
extern void (APIENTRY *ov_glVertexStream2sATI)(unsigned int param0,signed short param1,signed short param2);

static void APIENTRY init_ov_glVertexStream2sATI(unsigned int param0,signed short param1,signed short param2)
{
	void *f;
	f=(void*)getfunc("glVertexStream2sATI");
	ov_glVertexStream2sATI=f;
	ov_glVertexStream2sATI(param0,param1,param2);
}
void (APIENTRY *ov_glVertexStream2sATI)(unsigned int param0,signed short param1,signed short param2)=init_ov_glVertexStream2sATI;

void __gl_ml_glVertexStream2sATI(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	signed short l1=Int_val(v1);
	signed short l2=Int_val(v2);
	ov_glVertexStream2sATI(l0,l1,l2);
}
extern void (APIENTRY *ov_glVertexStream1dvATI)(unsigned int param0,double* param1);

static void APIENTRY init_ov_glVertexStream1dvATI(unsigned int param0,double* param1)
{
	void *f;
	f=(void*)getfunc("glVertexStream1dvATI");
	ov_glVertexStream1dvATI=f;
	ov_glVertexStream1dvATI(param0,param1);
}
void (APIENTRY *ov_glVertexStream1dvATI)(unsigned int param0,double* param1)=init_ov_glVertexStream1dvATI;

void __gl_ml_glVertexStream1dvATI(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	double* l1=(double*)(gl__ba__(v1));
	ov_glVertexStream1dvATI(l0,l1);
}
extern void (APIENTRY *ov_glVertexStream1dATI)(unsigned int param0,double param1);

static void APIENTRY init_ov_glVertexStream1dATI(unsigned int param0,double param1)
{
	void *f;
	f=(void*)getfunc("glVertexStream1dATI");
	ov_glVertexStream1dATI=f;
	ov_glVertexStream1dATI(param0,param1);
}
void (APIENTRY *ov_glVertexStream1dATI)(unsigned int param0,double param1)=init_ov_glVertexStream1dATI;

void __gl_ml_glVertexStream1dATI(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	double l1=Double_val(v1);
	ov_glVertexStream1dATI(l0,l1);
}
extern void (APIENTRY *ov_glVertexStream1fvATI)(unsigned int param0,float* param1);

static void APIENTRY init_ov_glVertexStream1fvATI(unsigned int param0,float* param1)
{
	void *f;
	f=(void*)getfunc("glVertexStream1fvATI");
	ov_glVertexStream1fvATI=f;
	ov_glVertexStream1fvATI(param0,param1);
}
void (APIENTRY *ov_glVertexStream1fvATI)(unsigned int param0,float* param1)=init_ov_glVertexStream1fvATI;

void __gl_ml_glVertexStream1fvATI(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float* l1=(float*)(gl__ba__(v1));
	ov_glVertexStream1fvATI(l0,l1);
}
extern void (APIENTRY *ov_glVertexStream1fATI)(unsigned int param0,float param1);

static void APIENTRY init_ov_glVertexStream1fATI(unsigned int param0,float param1)
{
	void *f;
	f=(void*)getfunc("glVertexStream1fATI");
	ov_glVertexStream1fATI=f;
	ov_glVertexStream1fATI(param0,param1);
}
void (APIENTRY *ov_glVertexStream1fATI)(unsigned int param0,float param1)=init_ov_glVertexStream1fATI;

void __gl_ml_glVertexStream1fATI(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float l1=Double_val(v1);
	ov_glVertexStream1fATI(l0,l1);
}
extern void (APIENTRY *ov_glVertexStream1ivATI)(unsigned int param0,signed int* param1);

static void APIENTRY init_ov_glVertexStream1ivATI(unsigned int param0,signed int* param1)
{
	void *f;
	f=(void*)getfunc("glVertexStream1ivATI");
	ov_glVertexStream1ivATI=f;
	ov_glVertexStream1ivATI(param0,param1);
}
void (APIENTRY *ov_glVertexStream1ivATI)(unsigned int param0,signed int* param1)=init_ov_glVertexStream1ivATI;

void __gl_ml_glVertexStream1ivATI(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed int* l1=(signed int*)(gl__ba__(v1));
	ov_glVertexStream1ivATI(l0,l1);
}
extern void (APIENTRY *ov_glVertexStream1iATI)(unsigned int param0,signed int param1);

static void APIENTRY init_ov_glVertexStream1iATI(unsigned int param0,signed int param1)
{
	void *f;
	f=(void*)getfunc("glVertexStream1iATI");
	ov_glVertexStream1iATI=f;
	ov_glVertexStream1iATI(param0,param1);
}
void (APIENTRY *ov_glVertexStream1iATI)(unsigned int param0,signed int param1)=init_ov_glVertexStream1iATI;

void __gl_ml_glVertexStream1iATI(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	ov_glVertexStream1iATI(l0,l1);
}
extern void (APIENTRY *ov_glVertexStream1svATI)(unsigned int param0,signed short* param1);

static void APIENTRY init_ov_glVertexStream1svATI(unsigned int param0,signed short* param1)
{
	void *f;
	f=(void*)getfunc("glVertexStream1svATI");
	ov_glVertexStream1svATI=f;
	ov_glVertexStream1svATI(param0,param1);
}
void (APIENTRY *ov_glVertexStream1svATI)(unsigned int param0,signed short* param1)=init_ov_glVertexStream1svATI;

void __gl_ml_glVertexStream1svATI(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed short* l1=(signed short*)(gl__ba__(v1));
	ov_glVertexStream1svATI(l0,l1);
}
extern void (APIENTRY *ov_glVertexStream1sATI)(unsigned int param0,signed short param1);

static void APIENTRY init_ov_glVertexStream1sATI(unsigned int param0,signed short param1)
{
	void *f;
	f=(void*)getfunc("glVertexStream1sATI");
	ov_glVertexStream1sATI=f;
	ov_glVertexStream1sATI(param0,param1);
}
void (APIENTRY *ov_glVertexStream1sATI)(unsigned int param0,signed short param1)=init_ov_glVertexStream1sATI;

void __gl_ml_glVertexStream1sATI(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed short l1=Int_val(v1);
	ov_glVertexStream1sATI(l0,l1);
}
extern void (APIENTRY *ov_glBlendColorEXT)(float param0,float param1,float param2,float param3);

static void APIENTRY init_ov_glBlendColorEXT(float param0,float param1,float param2,float param3)
{
	void *f;
	f=(void*)getfunc("glBlendColorEXT");
	ov_glBlendColorEXT=f;
	ov_glBlendColorEXT(param0,param1,param2,param3);
}
void (APIENTRY *ov_glBlendColorEXT)(float param0,float param1,float param2,float param3)=init_ov_glBlendColorEXT;

void __gl_ml_glBlendColorEXT(value v0,value v1,value v2,value v3)
{
	float l0=Double_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	float l3=Double_val(v3);
	ov_glBlendColorEXT(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glBlendFuncSeparateEXT)(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3);

static void APIENTRY init_ov_glBlendFuncSeparateEXT(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3)
{
	void *f;
	f=(void*)getfunc("glBlendFuncSeparateEXT");
	ov_glBlendFuncSeparateEXT=f;
	ov_glBlendFuncSeparateEXT(param0,param1,param2,param3);
}
void (APIENTRY *ov_glBlendFuncSeparateEXT)(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3)=init_ov_glBlendFuncSeparateEXT;

void __gl_ml_glBlendFuncSeparateEXT(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	unsigned int l3=Int_val(v3);
	ov_glBlendFuncSeparateEXT(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glBlendEquationEXT)(unsigned int param0);

static void APIENTRY init_ov_glBlendEquationEXT(unsigned int param0)
{
	void *f;
	f=(void*)getfunc("glBlendEquationEXT");
	ov_glBlendEquationEXT=f;
	ov_glBlendEquationEXT(param0);
}
void (APIENTRY *ov_glBlendEquationEXT)(unsigned int param0)=init_ov_glBlendEquationEXT;

void __gl_ml_glBlendEquationEXT(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glBlendEquationEXT(l0);
}
extern void (APIENTRY *ov_glCopyColorSubTableEXT)(unsigned int param0,signed int param1,signed int param2,signed int param3,signed int param4);

static void APIENTRY init_ov_glCopyColorSubTableEXT(unsigned int param0,signed int param1,signed int param2,signed int param3,signed int param4)
{
	void *f;
	f=(void*)getfunc("glCopyColorSubTableEXT");
	ov_glCopyColorSubTableEXT=f;
	ov_glCopyColorSubTableEXT(param0,param1,param2,param3,param4);
}
void (APIENTRY *ov_glCopyColorSubTableEXT)(unsigned int param0,signed int param1,signed int param2,signed int param3,signed int param4)=init_ov_glCopyColorSubTableEXT;

void __gl_ml_glCopyColorSubTableEXT(value v0,value v1,value v2,value v3,value v4)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	signed int l4=Int_val(v4);
	ov_glCopyColorSubTableEXT(l0,l1,l2,l3,l4);
}
extern void (APIENTRY *ov_glColorSubTableEXT)(unsigned int param0,signed int param1,signed int param2,unsigned int param3,unsigned int param4,void* param5);

static void APIENTRY init_ov_glColorSubTableEXT(unsigned int param0,signed int param1,signed int param2,unsigned int param3,unsigned int param4,void* param5)
{
	void *f;
	f=(void*)getfunc("glColorSubTableEXT");
	ov_glColorSubTableEXT=f;
	ov_glColorSubTableEXT(param0,param1,param2,param3,param4,param5);
}
void (APIENTRY *ov_glColorSubTableEXT)(unsigned int param0,signed int param1,signed int param2,unsigned int param3,unsigned int param4,void* param5)=init_ov_glColorSubTableEXT;

void __gl_ml_glColorSubTableEXT(value v0,value v1,value v2,value v3,value v4,value v5)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	unsigned int l3=Int_val(v3);
	unsigned int l4=Int_val(v4);
	void* l5=gl__ba__(v5);
	ov_glColorSubTableEXT(l0,l1,l2,l3,l4,l5);
}

void byte___gl_ml_glColorSubTableEXT(value * argv,int argn)
{
	__gl_ml_glColorSubTableEXT(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5]);
}
extern void (APIENTRY *ov_glUnlockArraysEXT)();

static void APIENTRY init_ov_glUnlockArraysEXT()
{
	void *f;
	f=(void*)getfunc("glUnlockArraysEXT");
	ov_glUnlockArraysEXT=f;
	ov_glUnlockArraysEXT();
}
void (APIENTRY *ov_glUnlockArraysEXT)()=init_ov_glUnlockArraysEXT;

void __gl_ml_glUnlockArraysEXT()
{
	ov_glUnlockArraysEXT();
}
extern void (APIENTRY *ov_glLockArraysEXT)(signed int param0,signed int param1);

static void APIENTRY init_ov_glLockArraysEXT(signed int param0,signed int param1)
{
	void *f;
	f=(void*)getfunc("glLockArraysEXT");
	ov_glLockArraysEXT=f;
	ov_glLockArraysEXT(param0,param1);
}
void (APIENTRY *ov_glLockArraysEXT)(signed int param0,signed int param1)=init_ov_glLockArraysEXT;

void __gl_ml_glLockArraysEXT(value v0,value v1)
{
	signed int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	ov_glLockArraysEXT(l0,l1);
}
extern void (APIENTRY *ov_glSeparableFilter2DEXT)(unsigned int param0,unsigned int param1,signed int param2,signed int param3,unsigned int param4,unsigned int param5,void* param6,void* param7);

static void APIENTRY init_ov_glSeparableFilter2DEXT(unsigned int param0,unsigned int param1,signed int param2,signed int param3,unsigned int param4,unsigned int param5,void* param6,void* param7)
{
	void *f;
	f=(void*)getfunc("glSeparableFilter2DEXT");
	ov_glSeparableFilter2DEXT=f;
	ov_glSeparableFilter2DEXT(param0,param1,param2,param3,param4,param5,param6,param7);
}
void (APIENTRY *ov_glSeparableFilter2DEXT)(unsigned int param0,unsigned int param1,signed int param2,signed int param3,unsigned int param4,unsigned int param5,void* param6,void* param7)=init_ov_glSeparableFilter2DEXT;

void __gl_ml_glSeparableFilter2DEXT(value v0,value v1,value v2,value v3,value v4,value v5,value v6,value v7)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	unsigned int l4=Int_val(v4);
	unsigned int l5=Int_val(v5);
	void* l6=gl__ba__(v6);
	void* l7=gl__ba__(v7);
	ov_glSeparableFilter2DEXT(l0,l1,l2,l3,l4,l5,l6,l7);
}

void byte___gl_ml_glSeparableFilter2DEXT(value * argv,int argn)
{
	__gl_ml_glSeparableFilter2DEXT(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6],argv[7]);
}
extern void (APIENTRY *ov_glGetSeparableFilterEXT)(unsigned int param0,unsigned int param1,unsigned int param2,void* param3,void* param4,void* param5);

static void APIENTRY init_ov_glGetSeparableFilterEXT(unsigned int param0,unsigned int param1,unsigned int param2,void* param3,void* param4,void* param5)
{
	void *f;
	f=(void*)getfunc("glGetSeparableFilterEXT");
	ov_glGetSeparableFilterEXT=f;
	ov_glGetSeparableFilterEXT(param0,param1,param2,param3,param4,param5);
}
void (APIENTRY *ov_glGetSeparableFilterEXT)(unsigned int param0,unsigned int param1,unsigned int param2,void* param3,void* param4,void* param5)=init_ov_glGetSeparableFilterEXT;

void __gl_ml_glGetSeparableFilterEXT(value v0,value v1,value v2,value v3,value v4,value v5)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	void* l3=gl__ba__(v3);
	void* l4=gl__ba__(v4);
	void* l5=gl__ba__(v5);
	ov_glGetSeparableFilterEXT(l0,l1,l2,l3,l4,l5);
}

void byte___gl_ml_glGetSeparableFilterEXT(value * argv,int argn)
{
	__gl_ml_glGetSeparableFilterEXT(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5]);
}
extern void (APIENTRY *ov_glGetConvolutionParameterivEXT)(unsigned int param0,unsigned int param1,signed int* param2);

static void APIENTRY init_ov_glGetConvolutionParameterivEXT(unsigned int param0,unsigned int param1,signed int* param2)
{
	void *f;
	f=(void*)getfunc("glGetConvolutionParameterivEXT");
	ov_glGetConvolutionParameterivEXT=f;
	ov_glGetConvolutionParameterivEXT(param0,param1,param2);
}
void (APIENTRY *ov_glGetConvolutionParameterivEXT)(unsigned int param0,unsigned int param1,signed int* param2)=init_ov_glGetConvolutionParameterivEXT;

void __gl_ml_glGetConvolutionParameterivEXT(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int* l2=(signed int*)(gl__ba__(v2));
	ov_glGetConvolutionParameterivEXT(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetConvolutionParameterfvEXT)(unsigned int param0,unsigned int param1,float* param2);

static void APIENTRY init_ov_glGetConvolutionParameterfvEXT(unsigned int param0,unsigned int param1,float* param2)
{
	void *f;
	f=(void*)getfunc("glGetConvolutionParameterfvEXT");
	ov_glGetConvolutionParameterfvEXT=f;
	ov_glGetConvolutionParameterfvEXT(param0,param1,param2);
}
void (APIENTRY *ov_glGetConvolutionParameterfvEXT)(unsigned int param0,unsigned int param1,float* param2)=init_ov_glGetConvolutionParameterfvEXT;

void __gl_ml_glGetConvolutionParameterfvEXT(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glGetConvolutionParameterfvEXT(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetConvolutionFilterEXT)(unsigned int param0,unsigned int param1,unsigned int param2,void* param3);

static void APIENTRY init_ov_glGetConvolutionFilterEXT(unsigned int param0,unsigned int param1,unsigned int param2,void* param3)
{
	void *f;
	f=(void*)getfunc("glGetConvolutionFilterEXT");
	ov_glGetConvolutionFilterEXT=f;
	ov_glGetConvolutionFilterEXT(param0,param1,param2,param3);
}
void (APIENTRY *ov_glGetConvolutionFilterEXT)(unsigned int param0,unsigned int param1,unsigned int param2,void* param3)=init_ov_glGetConvolutionFilterEXT;

void __gl_ml_glGetConvolutionFilterEXT(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	void* l3=gl__ba__(v3);
	ov_glGetConvolutionFilterEXT(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glCopyConvolutionFilter2DEXT)(unsigned int param0,unsigned int param1,signed int param2,signed int param3,signed int param4,signed int param5);

static void APIENTRY init_ov_glCopyConvolutionFilter2DEXT(unsigned int param0,unsigned int param1,signed int param2,signed int param3,signed int param4,signed int param5)
{
	void *f;
	f=(void*)getfunc("glCopyConvolutionFilter2DEXT");
	ov_glCopyConvolutionFilter2DEXT=f;
	ov_glCopyConvolutionFilter2DEXT(param0,param1,param2,param3,param4,param5);
}
void (APIENTRY *ov_glCopyConvolutionFilter2DEXT)(unsigned int param0,unsigned int param1,signed int param2,signed int param3,signed int param4,signed int param5)=init_ov_glCopyConvolutionFilter2DEXT;

void __gl_ml_glCopyConvolutionFilter2DEXT(value v0,value v1,value v2,value v3,value v4,value v5)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	signed int l4=Int_val(v4);
	signed int l5=Int_val(v5);
	ov_glCopyConvolutionFilter2DEXT(l0,l1,l2,l3,l4,l5);
}

void byte___gl_ml_glCopyConvolutionFilter2DEXT(value * argv,int argn)
{
	__gl_ml_glCopyConvolutionFilter2DEXT(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5]);
}
extern void (APIENTRY *ov_glCopyConvolutionFilter1DEXT)(unsigned int param0,unsigned int param1,signed int param2,signed int param3,signed int param4);

static void APIENTRY init_ov_glCopyConvolutionFilter1DEXT(unsigned int param0,unsigned int param1,signed int param2,signed int param3,signed int param4)
{
	void *f;
	f=(void*)getfunc("glCopyConvolutionFilter1DEXT");
	ov_glCopyConvolutionFilter1DEXT=f;
	ov_glCopyConvolutionFilter1DEXT(param0,param1,param2,param3,param4);
}
void (APIENTRY *ov_glCopyConvolutionFilter1DEXT)(unsigned int param0,unsigned int param1,signed int param2,signed int param3,signed int param4)=init_ov_glCopyConvolutionFilter1DEXT;

void __gl_ml_glCopyConvolutionFilter1DEXT(value v0,value v1,value v2,value v3,value v4)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	signed int l4=Int_val(v4);
	ov_glCopyConvolutionFilter1DEXT(l0,l1,l2,l3,l4);
}
extern void (APIENTRY *ov_glConvolutionParameterivEXT)(unsigned int param0,unsigned int param1,signed int* param2);

static void APIENTRY init_ov_glConvolutionParameterivEXT(unsigned int param0,unsigned int param1,signed int* param2)
{
	void *f;
	f=(void*)getfunc("glConvolutionParameterivEXT");
	ov_glConvolutionParameterivEXT=f;
	ov_glConvolutionParameterivEXT(param0,param1,param2);
}
void (APIENTRY *ov_glConvolutionParameterivEXT)(unsigned int param0,unsigned int param1,signed int* param2)=init_ov_glConvolutionParameterivEXT;

void __gl_ml_glConvolutionParameterivEXT(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int* l2=(signed int*)(gl__ba__(v2));
	ov_glConvolutionParameterivEXT(l0,l1,l2);
}
extern void (APIENTRY *ov_glConvolutionParameteriEXT)(unsigned int param0,unsigned int param1,signed int param2);

static void APIENTRY init_ov_glConvolutionParameteriEXT(unsigned int param0,unsigned int param1,signed int param2)
{
	void *f;
	f=(void*)getfunc("glConvolutionParameteriEXT");
	ov_glConvolutionParameteriEXT=f;
	ov_glConvolutionParameteriEXT(param0,param1,param2);
}
void (APIENTRY *ov_glConvolutionParameteriEXT)(unsigned int param0,unsigned int param1,signed int param2)=init_ov_glConvolutionParameteriEXT;

void __gl_ml_glConvolutionParameteriEXT(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	ov_glConvolutionParameteriEXT(l0,l1,l2);
}
extern void (APIENTRY *ov_glConvolutionParameterfvEXT)(unsigned int param0,unsigned int param1,float* param2);

static void APIENTRY init_ov_glConvolutionParameterfvEXT(unsigned int param0,unsigned int param1,float* param2)
{
	void *f;
	f=(void*)getfunc("glConvolutionParameterfvEXT");
	ov_glConvolutionParameterfvEXT=f;
	ov_glConvolutionParameterfvEXT(param0,param1,param2);
}
void (APIENTRY *ov_glConvolutionParameterfvEXT)(unsigned int param0,unsigned int param1,float* param2)=init_ov_glConvolutionParameterfvEXT;

void __gl_ml_glConvolutionParameterfvEXT(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glConvolutionParameterfvEXT(l0,l1,l2);
}
extern void (APIENTRY *ov_glConvolutionParameterfEXT)(unsigned int param0,unsigned int param1,float param2);

static void APIENTRY init_ov_glConvolutionParameterfEXT(unsigned int param0,unsigned int param1,float param2)
{
	void *f;
	f=(void*)getfunc("glConvolutionParameterfEXT");
	ov_glConvolutionParameterfEXT=f;
	ov_glConvolutionParameterfEXT(param0,param1,param2);
}
void (APIENTRY *ov_glConvolutionParameterfEXT)(unsigned int param0,unsigned int param1,float param2)=init_ov_glConvolutionParameterfEXT;

void __gl_ml_glConvolutionParameterfEXT(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float l2=Double_val(v2);
	ov_glConvolutionParameterfEXT(l0,l1,l2);
}
extern void (APIENTRY *ov_glConvolutionFilter2DEXT)(unsigned int param0,unsigned int param1,signed int param2,signed int param3,unsigned int param4,unsigned int param5,void* param6);

static void APIENTRY init_ov_glConvolutionFilter2DEXT(unsigned int param0,unsigned int param1,signed int param2,signed int param3,unsigned int param4,unsigned int param5,void* param6)
{
	void *f;
	f=(void*)getfunc("glConvolutionFilter2DEXT");
	ov_glConvolutionFilter2DEXT=f;
	ov_glConvolutionFilter2DEXT(param0,param1,param2,param3,param4,param5,param6);
}
void (APIENTRY *ov_glConvolutionFilter2DEXT)(unsigned int param0,unsigned int param1,signed int param2,signed int param3,unsigned int param4,unsigned int param5,void* param6)=init_ov_glConvolutionFilter2DEXT;

void __gl_ml_glConvolutionFilter2DEXT(value v0,value v1,value v2,value v3,value v4,value v5,value v6)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	unsigned int l4=Int_val(v4);
	unsigned int l5=Int_val(v5);
	void* l6=gl__ba__(v6);
	ov_glConvolutionFilter2DEXT(l0,l1,l2,l3,l4,l5,l6);
}

void byte___gl_ml_glConvolutionFilter2DEXT(value * argv,int argn)
{
	__gl_ml_glConvolutionFilter2DEXT(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6]);
}
extern void (APIENTRY *ov_glConvolutionFilter1DEXT)(unsigned int param0,unsigned int param1,signed int param2,unsigned int param3,unsigned int param4,void* param5);

static void APIENTRY init_ov_glConvolutionFilter1DEXT(unsigned int param0,unsigned int param1,signed int param2,unsigned int param3,unsigned int param4,void* param5)
{
	void *f;
	f=(void*)getfunc("glConvolutionFilter1DEXT");
	ov_glConvolutionFilter1DEXT=f;
	ov_glConvolutionFilter1DEXT(param0,param1,param2,param3,param4,param5);
}
void (APIENTRY *ov_glConvolutionFilter1DEXT)(unsigned int param0,unsigned int param1,signed int param2,unsigned int param3,unsigned int param4,void* param5)=init_ov_glConvolutionFilter1DEXT;

void __gl_ml_glConvolutionFilter1DEXT(value v0,value v1,value v2,value v3,value v4,value v5)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	unsigned int l3=Int_val(v3);
	unsigned int l4=Int_val(v4);
	void* l5=gl__ba__(v5);
	ov_glConvolutionFilter1DEXT(l0,l1,l2,l3,l4,l5);
}

void byte___gl_ml_glConvolutionFilter1DEXT(value * argv,int argn)
{
	__gl_ml_glConvolutionFilter1DEXT(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5]);
}
extern void (APIENTRY *ov_glBinormalPointerEXT)(unsigned int param0,signed int param1,void* param2);

static void APIENTRY init_ov_glBinormalPointerEXT(unsigned int param0,signed int param1,void* param2)
{
	void *f;
	f=(void*)getfunc("glBinormalPointerEXT");
	ov_glBinormalPointerEXT=f;
	ov_glBinormalPointerEXT(param0,param1,param2);
}
void (APIENTRY *ov_glBinormalPointerEXT)(unsigned int param0,signed int param1,void* param2)=init_ov_glBinormalPointerEXT;

void __gl_ml_glBinormalPointerEXT(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	void* l2=gl__ba__(v2);
	ov_glBinormalPointerEXT(l0,l1,l2);
}
extern void (APIENTRY *ov_glTangentPointerEXT)(unsigned int param0,signed int param1,void* param2);

static void APIENTRY init_ov_glTangentPointerEXT(unsigned int param0,signed int param1,void* param2)
{
	void *f;
	f=(void*)getfunc("glTangentPointerEXT");
	ov_glTangentPointerEXT=f;
	ov_glTangentPointerEXT(param0,param1,param2);
}
void (APIENTRY *ov_glTangentPointerEXT)(unsigned int param0,signed int param1,void* param2)=init_ov_glTangentPointerEXT;

void __gl_ml_glTangentPointerEXT(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	void* l2=gl__ba__(v2);
	ov_glTangentPointerEXT(l0,l1,l2);
}
extern void (APIENTRY *ov_glBinormal3svEXT)(signed short* param0);

static void APIENTRY init_ov_glBinormal3svEXT(signed short* param0)
{
	void *f;
	f=(void*)getfunc("glBinormal3svEXT");
	ov_glBinormal3svEXT=f;
	ov_glBinormal3svEXT(param0);
}
void (APIENTRY *ov_glBinormal3svEXT)(signed short* param0)=init_ov_glBinormal3svEXT;

void __gl_ml_glBinormal3svEXT(value v0)
{
	signed short* l0=(signed short*)(gl__ba__(v0));
	ov_glBinormal3svEXT(l0);
}
extern void (APIENTRY *ov_glBinormal3sEXT)(signed short param0,signed short param1,signed short param2);

static void APIENTRY init_ov_glBinormal3sEXT(signed short param0,signed short param1,signed short param2)
{
	void *f;
	f=(void*)getfunc("glBinormal3sEXT");
	ov_glBinormal3sEXT=f;
	ov_glBinormal3sEXT(param0,param1,param2);
}
void (APIENTRY *ov_glBinormal3sEXT)(signed short param0,signed short param1,signed short param2)=init_ov_glBinormal3sEXT;

void __gl_ml_glBinormal3sEXT(value v0,value v1,value v2)
{
	signed short l0=Int_val(v0);
	signed short l1=Int_val(v1);
	signed short l2=Int_val(v2);
	ov_glBinormal3sEXT(l0,l1,l2);
}
extern void (APIENTRY *ov_glBinormal3ivEXT)(signed int* param0);

static void APIENTRY init_ov_glBinormal3ivEXT(signed int* param0)
{
	void *f;
	f=(void*)getfunc("glBinormal3ivEXT");
	ov_glBinormal3ivEXT=f;
	ov_glBinormal3ivEXT(param0);
}
void (APIENTRY *ov_glBinormal3ivEXT)(signed int* param0)=init_ov_glBinormal3ivEXT;

void __gl_ml_glBinormal3ivEXT(value v0)
{
	signed int* l0=(signed int*)(gl__ba__(v0));
	ov_glBinormal3ivEXT(l0);
}
extern void (APIENTRY *ov_glBinormal3iEXT)(signed int param0,signed int param1,signed int param2);

static void APIENTRY init_ov_glBinormal3iEXT(signed int param0,signed int param1,signed int param2)
{
	void *f;
	f=(void*)getfunc("glBinormal3iEXT");
	ov_glBinormal3iEXT=f;
	ov_glBinormal3iEXT(param0,param1,param2);
}
void (APIENTRY *ov_glBinormal3iEXT)(signed int param0,signed int param1,signed int param2)=init_ov_glBinormal3iEXT;

void __gl_ml_glBinormal3iEXT(value v0,value v1,value v2)
{
	signed int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	ov_glBinormal3iEXT(l0,l1,l2);
}
extern void (APIENTRY *ov_glBinormal3fvEXT)(float* param0);

static void APIENTRY init_ov_glBinormal3fvEXT(float* param0)
{
	void *f;
	f=(void*)getfunc("glBinormal3fvEXT");
	ov_glBinormal3fvEXT=f;
	ov_glBinormal3fvEXT(param0);
}
void (APIENTRY *ov_glBinormal3fvEXT)(float* param0)=init_ov_glBinormal3fvEXT;

void __gl_ml_glBinormal3fvEXT(value v0)
{
	float* l0=(float*)(gl__ba__(v0));
	ov_glBinormal3fvEXT(l0);
}
extern void (APIENTRY *ov_glBinormal3fEXT)(float param0,float param1,float param2);

static void APIENTRY init_ov_glBinormal3fEXT(float param0,float param1,float param2)
{
	void *f;
	f=(void*)getfunc("glBinormal3fEXT");
	ov_glBinormal3fEXT=f;
	ov_glBinormal3fEXT(param0,param1,param2);
}
void (APIENTRY *ov_glBinormal3fEXT)(float param0,float param1,float param2)=init_ov_glBinormal3fEXT;

void __gl_ml_glBinormal3fEXT(value v0,value v1,value v2)
{
	float l0=Double_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	ov_glBinormal3fEXT(l0,l1,l2);
}
extern void (APIENTRY *ov_glBinormal3dvEXT)(double* param0);

static void APIENTRY init_ov_glBinormal3dvEXT(double* param0)
{
	void *f;
	f=(void*)getfunc("glBinormal3dvEXT");
	ov_glBinormal3dvEXT=f;
	ov_glBinormal3dvEXT(param0);
}
void (APIENTRY *ov_glBinormal3dvEXT)(double* param0)=init_ov_glBinormal3dvEXT;

void __gl_ml_glBinormal3dvEXT(value v0)
{
	double* l0=(double*)(gl__ba__(v0));
	ov_glBinormal3dvEXT(l0);
}
extern void (APIENTRY *ov_glBinormal3dEXT)(double param0,double param1,double param2);

static void APIENTRY init_ov_glBinormal3dEXT(double param0,double param1,double param2)
{
	void *f;
	f=(void*)getfunc("glBinormal3dEXT");
	ov_glBinormal3dEXT=f;
	ov_glBinormal3dEXT(param0,param1,param2);
}
void (APIENTRY *ov_glBinormal3dEXT)(double param0,double param1,double param2)=init_ov_glBinormal3dEXT;

void __gl_ml_glBinormal3dEXT(value v0,value v1,value v2)
{
	double l0=Double_val(v0);
	double l1=Double_val(v1);
	double l2=Double_val(v2);
	ov_glBinormal3dEXT(l0,l1,l2);
}
extern void (APIENTRY *ov_glBinormal3bvEXT)(signed char* param0);

static void APIENTRY init_ov_glBinormal3bvEXT(signed char* param0)
{
	void *f;
	f=(void*)getfunc("glBinormal3bvEXT");
	ov_glBinormal3bvEXT=f;
	ov_glBinormal3bvEXT(param0);
}
void (APIENTRY *ov_glBinormal3bvEXT)(signed char* param0)=init_ov_glBinormal3bvEXT;

void __gl_ml_glBinormal3bvEXT(value v0)
{
	signed char* l0=(signed char*)(gl__ba__(v0));
	ov_glBinormal3bvEXT(l0);
}
extern void (APIENTRY *ov_glBinormal3bEXT)(signed char param0,signed char param1,signed char param2);

static void APIENTRY init_ov_glBinormal3bEXT(signed char param0,signed char param1,signed char param2)
{
	void *f;
	f=(void*)getfunc("glBinormal3bEXT");
	ov_glBinormal3bEXT=f;
	ov_glBinormal3bEXT(param0,param1,param2);
}
void (APIENTRY *ov_glBinormal3bEXT)(signed char param0,signed char param1,signed char param2)=init_ov_glBinormal3bEXT;

void __gl_ml_glBinormal3bEXT(value v0,value v1,value v2)
{
	signed char l0=Int_val(v0);
	signed char l1=Int_val(v1);
	signed char l2=Int_val(v2);
	ov_glBinormal3bEXT(l0,l1,l2);
}
extern void (APIENTRY *ov_glTangent3svEXT)(signed short* param0);

static void APIENTRY init_ov_glTangent3svEXT(signed short* param0)
{
	void *f;
	f=(void*)getfunc("glTangent3svEXT");
	ov_glTangent3svEXT=f;
	ov_glTangent3svEXT(param0);
}
void (APIENTRY *ov_glTangent3svEXT)(signed short* param0)=init_ov_glTangent3svEXT;

void __gl_ml_glTangent3svEXT(value v0)
{
	signed short* l0=(signed short*)(gl__ba__(v0));
	ov_glTangent3svEXT(l0);
}
extern void (APIENTRY *ov_glTangent3sEXT)(signed short param0,signed short param1,signed short param2);

static void APIENTRY init_ov_glTangent3sEXT(signed short param0,signed short param1,signed short param2)
{
	void *f;
	f=(void*)getfunc("glTangent3sEXT");
	ov_glTangent3sEXT=f;
	ov_glTangent3sEXT(param0,param1,param2);
}
void (APIENTRY *ov_glTangent3sEXT)(signed short param0,signed short param1,signed short param2)=init_ov_glTangent3sEXT;

void __gl_ml_glTangent3sEXT(value v0,value v1,value v2)
{
	signed short l0=Int_val(v0);
	signed short l1=Int_val(v1);
	signed short l2=Int_val(v2);
	ov_glTangent3sEXT(l0,l1,l2);
}
extern void (APIENTRY *ov_glTangent3ivEXT)(signed int* param0);

static void APIENTRY init_ov_glTangent3ivEXT(signed int* param0)
{
	void *f;
	f=(void*)getfunc("glTangent3ivEXT");
	ov_glTangent3ivEXT=f;
	ov_glTangent3ivEXT(param0);
}
void (APIENTRY *ov_glTangent3ivEXT)(signed int* param0)=init_ov_glTangent3ivEXT;

void __gl_ml_glTangent3ivEXT(value v0)
{
	signed int* l0=(signed int*)(gl__ba__(v0));
	ov_glTangent3ivEXT(l0);
}
extern void (APIENTRY *ov_glTangent3iEXT)(signed int param0,signed int param1,signed int param2);

static void APIENTRY init_ov_glTangent3iEXT(signed int param0,signed int param1,signed int param2)
{
	void *f;
	f=(void*)getfunc("glTangent3iEXT");
	ov_glTangent3iEXT=f;
	ov_glTangent3iEXT(param0,param1,param2);
}
void (APIENTRY *ov_glTangent3iEXT)(signed int param0,signed int param1,signed int param2)=init_ov_glTangent3iEXT;

void __gl_ml_glTangent3iEXT(value v0,value v1,value v2)
{
	signed int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	ov_glTangent3iEXT(l0,l1,l2);
}
extern void (APIENTRY *ov_glTangent3fvEXT)(float* param0);

static void APIENTRY init_ov_glTangent3fvEXT(float* param0)
{
	void *f;
	f=(void*)getfunc("glTangent3fvEXT");
	ov_glTangent3fvEXT=f;
	ov_glTangent3fvEXT(param0);
}
void (APIENTRY *ov_glTangent3fvEXT)(float* param0)=init_ov_glTangent3fvEXT;

void __gl_ml_glTangent3fvEXT(value v0)
{
	float* l0=(float*)(gl__ba__(v0));
	ov_glTangent3fvEXT(l0);
}
extern void (APIENTRY *ov_glTangent3fEXT)(float param0,float param1,float param2);

static void APIENTRY init_ov_glTangent3fEXT(float param0,float param1,float param2)
{
	void *f;
	f=(void*)getfunc("glTangent3fEXT");
	ov_glTangent3fEXT=f;
	ov_glTangent3fEXT(param0,param1,param2);
}
void (APIENTRY *ov_glTangent3fEXT)(float param0,float param1,float param2)=init_ov_glTangent3fEXT;

void __gl_ml_glTangent3fEXT(value v0,value v1,value v2)
{
	float l0=Double_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	ov_glTangent3fEXT(l0,l1,l2);
}
extern void (APIENTRY *ov_glTangent3dvEXT)(double* param0);

static void APIENTRY init_ov_glTangent3dvEXT(double* param0)
{
	void *f;
	f=(void*)getfunc("glTangent3dvEXT");
	ov_glTangent3dvEXT=f;
	ov_glTangent3dvEXT(param0);
}
void (APIENTRY *ov_glTangent3dvEXT)(double* param0)=init_ov_glTangent3dvEXT;

void __gl_ml_glTangent3dvEXT(value v0)
{
	double* l0=(double*)(gl__ba__(v0));
	ov_glTangent3dvEXT(l0);
}
extern void (APIENTRY *ov_glTangent3dEXT)(double param0,double param1,double param2);

static void APIENTRY init_ov_glTangent3dEXT(double param0,double param1,double param2)
{
	void *f;
	f=(void*)getfunc("glTangent3dEXT");
	ov_glTangent3dEXT=f;
	ov_glTangent3dEXT(param0,param1,param2);
}
void (APIENTRY *ov_glTangent3dEXT)(double param0,double param1,double param2)=init_ov_glTangent3dEXT;

void __gl_ml_glTangent3dEXT(value v0,value v1,value v2)
{
	double l0=Double_val(v0);
	double l1=Double_val(v1);
	double l2=Double_val(v2);
	ov_glTangent3dEXT(l0,l1,l2);
}
extern void (APIENTRY *ov_glTangent3bvEXT)(signed char* param0);

static void APIENTRY init_ov_glTangent3bvEXT(signed char* param0)
{
	void *f;
	f=(void*)getfunc("glTangent3bvEXT");
	ov_glTangent3bvEXT=f;
	ov_glTangent3bvEXT(param0);
}
void (APIENTRY *ov_glTangent3bvEXT)(signed char* param0)=init_ov_glTangent3bvEXT;

void __gl_ml_glTangent3bvEXT(value v0)
{
	signed char* l0=(signed char*)(gl__ba__(v0));
	ov_glTangent3bvEXT(l0);
}
extern void (APIENTRY *ov_glTangent3bEXT)(signed char param0,signed char param1,signed char param2);

static void APIENTRY init_ov_glTangent3bEXT(signed char param0,signed char param1,signed char param2)
{
	void *f;
	f=(void*)getfunc("glTangent3bEXT");
	ov_glTangent3bEXT=f;
	ov_glTangent3bEXT(param0,param1,param2);
}
void (APIENTRY *ov_glTangent3bEXT)(signed char param0,signed char param1,signed char param2)=init_ov_glTangent3bEXT;

void __gl_ml_glTangent3bEXT(value v0,value v1,value v2)
{
	signed char l0=Int_val(v0);
	signed char l1=Int_val(v1);
	signed char l2=Int_val(v2);
	ov_glTangent3bEXT(l0,l1,l2);
}
extern void (APIENTRY *ov_glCopyTexSubImage3DEXT)(unsigned int param0,signed int param1,signed int param2,signed int param3,signed int param4,signed int param5,signed int param6,signed int param7,signed int param8);

static void APIENTRY init_ov_glCopyTexSubImage3DEXT(unsigned int param0,signed int param1,signed int param2,signed int param3,signed int param4,signed int param5,signed int param6,signed int param7,signed int param8)
{
	void *f;
	f=(void*)getfunc("glCopyTexSubImage3DEXT");
	ov_glCopyTexSubImage3DEXT=f;
	ov_glCopyTexSubImage3DEXT(param0,param1,param2,param3,param4,param5,param6,param7,param8);
}
void (APIENTRY *ov_glCopyTexSubImage3DEXT)(unsigned int param0,signed int param1,signed int param2,signed int param3,signed int param4,signed int param5,signed int param6,signed int param7,signed int param8)=init_ov_glCopyTexSubImage3DEXT;

void __gl_ml_glCopyTexSubImage3DEXT(value v0,value v1,value v2,value v3,value v4,value v5,value v6,value v7,value v8)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	signed int l4=Int_val(v4);
	signed int l5=Int_val(v5);
	signed int l6=Int_val(v6);
	signed int l7=Int_val(v7);
	signed int l8=Int_val(v8);
	ov_glCopyTexSubImage3DEXT(l0,l1,l2,l3,l4,l5,l6,l7,l8);
}

void byte___gl_ml_glCopyTexSubImage3DEXT(value * argv,int argn)
{
	__gl_ml_glCopyTexSubImage3DEXT(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6],argv[7],argv[8]);
}
extern void (APIENTRY *ov_glCopyTexSubImage2DEXT)(unsigned int param0,signed int param1,signed int param2,signed int param3,signed int param4,signed int param5,signed int param6,signed int param7);

static void APIENTRY init_ov_glCopyTexSubImage2DEXT(unsigned int param0,signed int param1,signed int param2,signed int param3,signed int param4,signed int param5,signed int param6,signed int param7)
{
	void *f;
	f=(void*)getfunc("glCopyTexSubImage2DEXT");
	ov_glCopyTexSubImage2DEXT=f;
	ov_glCopyTexSubImage2DEXT(param0,param1,param2,param3,param4,param5,param6,param7);
}
void (APIENTRY *ov_glCopyTexSubImage2DEXT)(unsigned int param0,signed int param1,signed int param2,signed int param3,signed int param4,signed int param5,signed int param6,signed int param7)=init_ov_glCopyTexSubImage2DEXT;

void __gl_ml_glCopyTexSubImage2DEXT(value v0,value v1,value v2,value v3,value v4,value v5,value v6,value v7)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	signed int l4=Int_val(v4);
	signed int l5=Int_val(v5);
	signed int l6=Int_val(v6);
	signed int l7=Int_val(v7);
	ov_glCopyTexSubImage2DEXT(l0,l1,l2,l3,l4,l5,l6,l7);
}

void byte___gl_ml_glCopyTexSubImage2DEXT(value * argv,int argn)
{
	__gl_ml_glCopyTexSubImage2DEXT(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6],argv[7]);
}
extern void (APIENTRY *ov_glCopyTexSubImage1DEXT)(unsigned int param0,signed int param1,signed int param2,signed int param3,signed int param4,signed int param5);

static void APIENTRY init_ov_glCopyTexSubImage1DEXT(unsigned int param0,signed int param1,signed int param2,signed int param3,signed int param4,signed int param5)
{
	void *f;
	f=(void*)getfunc("glCopyTexSubImage1DEXT");
	ov_glCopyTexSubImage1DEXT=f;
	ov_glCopyTexSubImage1DEXT(param0,param1,param2,param3,param4,param5);
}
void (APIENTRY *ov_glCopyTexSubImage1DEXT)(unsigned int param0,signed int param1,signed int param2,signed int param3,signed int param4,signed int param5)=init_ov_glCopyTexSubImage1DEXT;

void __gl_ml_glCopyTexSubImage1DEXT(value v0,value v1,value v2,value v3,value v4,value v5)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	signed int l4=Int_val(v4);
	signed int l5=Int_val(v5);
	ov_glCopyTexSubImage1DEXT(l0,l1,l2,l3,l4,l5);
}

void byte___gl_ml_glCopyTexSubImage1DEXT(value * argv,int argn)
{
	__gl_ml_glCopyTexSubImage1DEXT(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5]);
}
extern void (APIENTRY *ov_glCopyTexImage2DEXT)(unsigned int param0,signed int param1,unsigned int param2,signed int param3,signed int param4,signed int param5,signed int param6,signed int param7);

static void APIENTRY init_ov_glCopyTexImage2DEXT(unsigned int param0,signed int param1,unsigned int param2,signed int param3,signed int param4,signed int param5,signed int param6,signed int param7)
{
	void *f;
	f=(void*)getfunc("glCopyTexImage2DEXT");
	ov_glCopyTexImage2DEXT=f;
	ov_glCopyTexImage2DEXT(param0,param1,param2,param3,param4,param5,param6,param7);
}
void (APIENTRY *ov_glCopyTexImage2DEXT)(unsigned int param0,signed int param1,unsigned int param2,signed int param3,signed int param4,signed int param5,signed int param6,signed int param7)=init_ov_glCopyTexImage2DEXT;

void __gl_ml_glCopyTexImage2DEXT(value v0,value v1,value v2,value v3,value v4,value v5,value v6,value v7)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	signed int l4=Int_val(v4);
	signed int l5=Int_val(v5);
	signed int l6=Int_val(v6);
	signed int l7=Int_val(v7);
	ov_glCopyTexImage2DEXT(l0,l1,l2,l3,l4,l5,l6,l7);
}

void byte___gl_ml_glCopyTexImage2DEXT(value * argv,int argn)
{
	__gl_ml_glCopyTexImage2DEXT(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6],argv[7]);
}
extern void (APIENTRY *ov_glCopyTexImage1DEXT)(unsigned int param0,signed int param1,unsigned int param2,signed int param3,signed int param4,signed int param5,signed int param6);

static void APIENTRY init_ov_glCopyTexImage1DEXT(unsigned int param0,signed int param1,unsigned int param2,signed int param3,signed int param4,signed int param5,signed int param6)
{
	void *f;
	f=(void*)getfunc("glCopyTexImage1DEXT");
	ov_glCopyTexImage1DEXT=f;
	ov_glCopyTexImage1DEXT(param0,param1,param2,param3,param4,param5,param6);
}
void (APIENTRY *ov_glCopyTexImage1DEXT)(unsigned int param0,signed int param1,unsigned int param2,signed int param3,signed int param4,signed int param5,signed int param6)=init_ov_glCopyTexImage1DEXT;

void __gl_ml_glCopyTexImage1DEXT(value v0,value v1,value v2,value v3,value v4,value v5,value v6)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	signed int l4=Int_val(v4);
	signed int l5=Int_val(v5);
	signed int l6=Int_val(v6);
	ov_glCopyTexImage1DEXT(l0,l1,l2,l3,l4,l5,l6);
}

void byte___gl_ml_glCopyTexImage1DEXT(value * argv,int argn)
{
	__gl_ml_glCopyTexImage1DEXT(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6]);
}
extern void (APIENTRY *ov_glCullParameterfvEXT)(unsigned int param0,float* param1);

static void APIENTRY init_ov_glCullParameterfvEXT(unsigned int param0,float* param1)
{
	void *f;
	f=(void*)getfunc("glCullParameterfvEXT");
	ov_glCullParameterfvEXT=f;
	ov_glCullParameterfvEXT(param0,param1);
}
void (APIENTRY *ov_glCullParameterfvEXT)(unsigned int param0,float* param1)=init_ov_glCullParameterfvEXT;

void __gl_ml_glCullParameterfvEXT(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float* l1=(float*)(gl__ba__(v1));
	ov_glCullParameterfvEXT(l0,l1);
}
extern void (APIENTRY *ov_glCullParameterdvEXT)(unsigned int param0,double* param1);

static void APIENTRY init_ov_glCullParameterdvEXT(unsigned int param0,double* param1)
{
	void *f;
	f=(void*)getfunc("glCullParameterdvEXT");
	ov_glCullParameterdvEXT=f;
	ov_glCullParameterdvEXT(param0,param1);
}
void (APIENTRY *ov_glCullParameterdvEXT)(unsigned int param0,double* param1)=init_ov_glCullParameterdvEXT;

void __gl_ml_glCullParameterdvEXT(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	double* l1=(double*)(gl__ba__(v1));
	ov_glCullParameterdvEXT(l0,l1);
}
extern void (APIENTRY *ov_glDrawRangeElementsEXT)(unsigned int param0,unsigned int param1,unsigned int param2,signed int param3,unsigned int param4,void* param5);

static void APIENTRY init_ov_glDrawRangeElementsEXT(unsigned int param0,unsigned int param1,unsigned int param2,signed int param3,unsigned int param4,void* param5)
{
	void *f;
	f=(void*)getfunc("glDrawRangeElementsEXT");
	ov_glDrawRangeElementsEXT=f;
	ov_glDrawRangeElementsEXT(param0,param1,param2,param3,param4,param5);
}
void (APIENTRY *ov_glDrawRangeElementsEXT)(unsigned int param0,unsigned int param1,unsigned int param2,signed int param3,unsigned int param4,void* param5)=init_ov_glDrawRangeElementsEXT;

void __gl_ml_glDrawRangeElementsEXT(value v0,value v1,value v2,value v3,value v4,value v5)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	unsigned int l4=Int_val(v4);
	void* l5=gl__ba__(v5);
	ov_glDrawRangeElementsEXT(l0,l1,l2,l3,l4,l5);
}

void byte___gl_ml_glDrawRangeElementsEXT(value * argv,int argn)
{
	__gl_ml_glDrawRangeElementsEXT(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5]);
}
extern void (APIENTRY *ov_glFogCoordPointerEXT)(unsigned int param0,signed int param1,void* param2);

static void APIENTRY init_ov_glFogCoordPointerEXT(unsigned int param0,signed int param1,void* param2)
{
	void *f;
	f=(void*)getfunc("glFogCoordPointerEXT");
	ov_glFogCoordPointerEXT=f;
	ov_glFogCoordPointerEXT(param0,param1,param2);
}
void (APIENTRY *ov_glFogCoordPointerEXT)(unsigned int param0,signed int param1,void* param2)=init_ov_glFogCoordPointerEXT;

void __gl_ml_glFogCoordPointerEXT(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	void* l2=gl__ba__(v2);
	ov_glFogCoordPointerEXT(l0,l1,l2);
}
extern void (APIENTRY *ov_glFogCoorddvEXT)(double* param0);

static void APIENTRY init_ov_glFogCoorddvEXT(double* param0)
{
	void *f;
	f=(void*)getfunc("glFogCoorddvEXT");
	ov_glFogCoorddvEXT=f;
	ov_glFogCoorddvEXT(param0);
}
void (APIENTRY *ov_glFogCoorddvEXT)(double* param0)=init_ov_glFogCoorddvEXT;

void __gl_ml_glFogCoorddvEXT(value v0)
{
	double* l0=(double*)(gl__ba__(v0));
	ov_glFogCoorddvEXT(l0);
}
extern void (APIENTRY *ov_glFogCoorddEXT)(double param0);

static void APIENTRY init_ov_glFogCoorddEXT(double param0)
{
	void *f;
	f=(void*)getfunc("glFogCoorddEXT");
	ov_glFogCoorddEXT=f;
	ov_glFogCoorddEXT(param0);
}
void (APIENTRY *ov_glFogCoorddEXT)(double param0)=init_ov_glFogCoorddEXT;

void __gl_ml_glFogCoorddEXT(value v0)
{
	double l0=Double_val(v0);
	ov_glFogCoorddEXT(l0);
}
extern void (APIENTRY *ov_glFogCoordfvEXT)(float* param0);

static void APIENTRY init_ov_glFogCoordfvEXT(float* param0)
{
	void *f;
	f=(void*)getfunc("glFogCoordfvEXT");
	ov_glFogCoordfvEXT=f;
	ov_glFogCoordfvEXT(param0);
}
void (APIENTRY *ov_glFogCoordfvEXT)(float* param0)=init_ov_glFogCoordfvEXT;

void __gl_ml_glFogCoordfvEXT(value v0)
{
	float* l0=(float*)(gl__ba__(v0));
	ov_glFogCoordfvEXT(l0);
}
extern void (APIENTRY *ov_glFogCoordfEXT)(float param0);

static void APIENTRY init_ov_glFogCoordfEXT(float param0)
{
	void *f;
	f=(void*)getfunc("glFogCoordfEXT");
	ov_glFogCoordfEXT=f;
	ov_glFogCoordfEXT(param0);
}
void (APIENTRY *ov_glFogCoordfEXT)(float param0)=init_ov_glFogCoordfEXT;

void __gl_ml_glFogCoordfEXT(value v0)
{
	float l0=Double_val(v0);
	ov_glFogCoordfEXT(l0);
}
extern void (APIENTRY *ov_glResetMinmaxEXT)(unsigned int param0);

static void APIENTRY init_ov_glResetMinmaxEXT(unsigned int param0)
{
	void *f;
	f=(void*)getfunc("glResetMinmaxEXT");
	ov_glResetMinmaxEXT=f;
	ov_glResetMinmaxEXT(param0);
}
void (APIENTRY *ov_glResetMinmaxEXT)(unsigned int param0)=init_ov_glResetMinmaxEXT;

void __gl_ml_glResetMinmaxEXT(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glResetMinmaxEXT(l0);
}
extern void (APIENTRY *ov_glResetHistogramEXT)(unsigned int param0);

static void APIENTRY init_ov_glResetHistogramEXT(unsigned int param0)
{
	void *f;
	f=(void*)getfunc("glResetHistogramEXT");
	ov_glResetHistogramEXT=f;
	ov_glResetHistogramEXT(param0);
}
void (APIENTRY *ov_glResetHistogramEXT)(unsigned int param0)=init_ov_glResetHistogramEXT;

void __gl_ml_glResetHistogramEXT(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glResetHistogramEXT(l0);
}
extern void (APIENTRY *ov_glMinmaxEXT)(unsigned int param0,unsigned int param1,unsigned char param2);

static void APIENTRY init_ov_glMinmaxEXT(unsigned int param0,unsigned int param1,unsigned char param2)
{
	void *f;
	f=(void*)getfunc("glMinmaxEXT");
	ov_glMinmaxEXT=f;
	ov_glMinmaxEXT(param0,param1,param2);
}
void (APIENTRY *ov_glMinmaxEXT)(unsigned int param0,unsigned int param1,unsigned char param2)=init_ov_glMinmaxEXT;

void __gl_ml_glMinmaxEXT(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned char l2=Int_val(v2);
	ov_glMinmaxEXT(l0,l1,l2);
}
extern void (APIENTRY *ov_glHistogramEXT)(unsigned int param0,signed int param1,unsigned int param2,unsigned char param3);

static void APIENTRY init_ov_glHistogramEXT(unsigned int param0,signed int param1,unsigned int param2,unsigned char param3)
{
	void *f;
	f=(void*)getfunc("glHistogramEXT");
	ov_glHistogramEXT=f;
	ov_glHistogramEXT(param0,param1,param2,param3);
}
void (APIENTRY *ov_glHistogramEXT)(unsigned int param0,signed int param1,unsigned int param2,unsigned char param3)=init_ov_glHistogramEXT;

void __gl_ml_glHistogramEXT(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	unsigned char l3=Int_val(v3);
	ov_glHistogramEXT(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glGetMinmaxParameterivEXT)(unsigned int param0,unsigned int param1,signed int* param2);

static void APIENTRY init_ov_glGetMinmaxParameterivEXT(unsigned int param0,unsigned int param1,signed int* param2)
{
	void *f;
	f=(void*)getfunc("glGetMinmaxParameterivEXT");
	ov_glGetMinmaxParameterivEXT=f;
	ov_glGetMinmaxParameterivEXT(param0,param1,param2);
}
void (APIENTRY *ov_glGetMinmaxParameterivEXT)(unsigned int param0,unsigned int param1,signed int* param2)=init_ov_glGetMinmaxParameterivEXT;

void __gl_ml_glGetMinmaxParameterivEXT(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int* l2=(signed int*)(gl__ba__(v2));
	ov_glGetMinmaxParameterivEXT(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetMinmaxParameterfvEXT)(unsigned int param0,unsigned int param1,float* param2);

static void APIENTRY init_ov_glGetMinmaxParameterfvEXT(unsigned int param0,unsigned int param1,float* param2)
{
	void *f;
	f=(void*)getfunc("glGetMinmaxParameterfvEXT");
	ov_glGetMinmaxParameterfvEXT=f;
	ov_glGetMinmaxParameterfvEXT(param0,param1,param2);
}
void (APIENTRY *ov_glGetMinmaxParameterfvEXT)(unsigned int param0,unsigned int param1,float* param2)=init_ov_glGetMinmaxParameterfvEXT;

void __gl_ml_glGetMinmaxParameterfvEXT(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glGetMinmaxParameterfvEXT(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetMinmaxEXT)(unsigned int param0,unsigned char param1,unsigned int param2,unsigned int param3,void* param4);

static void APIENTRY init_ov_glGetMinmaxEXT(unsigned int param0,unsigned char param1,unsigned int param2,unsigned int param3,void* param4)
{
	void *f;
	f=(void*)getfunc("glGetMinmaxEXT");
	ov_glGetMinmaxEXT=f;
	ov_glGetMinmaxEXT(param0,param1,param2,param3,param4);
}
void (APIENTRY *ov_glGetMinmaxEXT)(unsigned int param0,unsigned char param1,unsigned int param2,unsigned int param3,void* param4)=init_ov_glGetMinmaxEXT;

void __gl_ml_glGetMinmaxEXT(value v0,value v1,value v2,value v3,value v4)
{
	unsigned int l0=Int_val(v0);
	unsigned char l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	unsigned int l3=Int_val(v3);
	void* l4=gl__ba__(v4);
	ov_glGetMinmaxEXT(l0,l1,l2,l3,l4);
}
extern void (APIENTRY *ov_glGetHistogramParameterivEXT)(unsigned int param0,unsigned int param1,signed int* param2);

static void APIENTRY init_ov_glGetHistogramParameterivEXT(unsigned int param0,unsigned int param1,signed int* param2)
{
	void *f;
	f=(void*)getfunc("glGetHistogramParameterivEXT");
	ov_glGetHistogramParameterivEXT=f;
	ov_glGetHistogramParameterivEXT(param0,param1,param2);
}
void (APIENTRY *ov_glGetHistogramParameterivEXT)(unsigned int param0,unsigned int param1,signed int* param2)=init_ov_glGetHistogramParameterivEXT;

void __gl_ml_glGetHistogramParameterivEXT(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int* l2=(signed int*)(gl__ba__(v2));
	ov_glGetHistogramParameterivEXT(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetHistogramParameterfvEXT)(unsigned int param0,unsigned int param1,float* param2);

static void APIENTRY init_ov_glGetHistogramParameterfvEXT(unsigned int param0,unsigned int param1,float* param2)
{
	void *f;
	f=(void*)getfunc("glGetHistogramParameterfvEXT");
	ov_glGetHistogramParameterfvEXT=f;
	ov_glGetHistogramParameterfvEXT(param0,param1,param2);
}
void (APIENTRY *ov_glGetHistogramParameterfvEXT)(unsigned int param0,unsigned int param1,float* param2)=init_ov_glGetHistogramParameterfvEXT;

void __gl_ml_glGetHistogramParameterfvEXT(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glGetHistogramParameterfvEXT(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetHistogramEXT)(unsigned int param0,unsigned char param1,unsigned int param2,unsigned int param3,void* param4);

static void APIENTRY init_ov_glGetHistogramEXT(unsigned int param0,unsigned char param1,unsigned int param2,unsigned int param3,void* param4)
{
	void *f;
	f=(void*)getfunc("glGetHistogramEXT");
	ov_glGetHistogramEXT=f;
	ov_glGetHistogramEXT(param0,param1,param2,param3,param4);
}
void (APIENTRY *ov_glGetHistogramEXT)(unsigned int param0,unsigned char param1,unsigned int param2,unsigned int param3,void* param4)=init_ov_glGetHistogramEXT;

void __gl_ml_glGetHistogramEXT(value v0,value v1,value v2,value v3,value v4)
{
	unsigned int l0=Int_val(v0);
	unsigned char l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	unsigned int l3=Int_val(v3);
	void* l4=gl__ba__(v4);
	ov_glGetHistogramEXT(l0,l1,l2,l3,l4);
}
extern void (APIENTRY *ov_glIndexFuncEXT)(unsigned int param0,float param1);

static void APIENTRY init_ov_glIndexFuncEXT(unsigned int param0,float param1)
{
	void *f;
	f=(void*)getfunc("glIndexFuncEXT");
	ov_glIndexFuncEXT=f;
	ov_glIndexFuncEXT(param0,param1);
}
void (APIENTRY *ov_glIndexFuncEXT)(unsigned int param0,float param1)=init_ov_glIndexFuncEXT;

void __gl_ml_glIndexFuncEXT(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float l1=Double_val(v1);
	ov_glIndexFuncEXT(l0,l1);
}
extern void (APIENTRY *ov_glIndexMaterialEXT)(unsigned int param0,unsigned int param1);

static void APIENTRY init_ov_glIndexMaterialEXT(unsigned int param0,unsigned int param1)
{
	void *f;
	f=(void*)getfunc("glIndexMaterialEXT");
	ov_glIndexMaterialEXT=f;
	ov_glIndexMaterialEXT(param0,param1);
}
void (APIENTRY *ov_glIndexMaterialEXT)(unsigned int param0,unsigned int param1)=init_ov_glIndexMaterialEXT;

void __gl_ml_glIndexMaterialEXT(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	ov_glIndexMaterialEXT(l0,l1);
}
extern void (APIENTRY *ov_glTextureMaterialEXT)(unsigned int param0,unsigned int param1);

static void APIENTRY init_ov_glTextureMaterialEXT(unsigned int param0,unsigned int param1)
{
	void *f;
	f=(void*)getfunc("glTextureMaterialEXT");
	ov_glTextureMaterialEXT=f;
	ov_glTextureMaterialEXT(param0,param1);
}
void (APIENTRY *ov_glTextureMaterialEXT)(unsigned int param0,unsigned int param1)=init_ov_glTextureMaterialEXT;

void __gl_ml_glTextureMaterialEXT(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	ov_glTextureMaterialEXT(l0,l1);
}
extern void (APIENTRY *ov_glTextureLightEXT)(unsigned int param0);

static void APIENTRY init_ov_glTextureLightEXT(unsigned int param0)
{
	void *f;
	f=(void*)getfunc("glTextureLightEXT");
	ov_glTextureLightEXT=f;
	ov_glTextureLightEXT(param0);
}
void (APIENTRY *ov_glTextureLightEXT)(unsigned int param0)=init_ov_glTextureLightEXT;

void __gl_ml_glTextureLightEXT(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glTextureLightEXT(l0);
}
extern void (APIENTRY *ov_glApplyTextureEXT)(unsigned int param0);

static void APIENTRY init_ov_glApplyTextureEXT(unsigned int param0)
{
	void *f;
	f=(void*)getfunc("glApplyTextureEXT");
	ov_glApplyTextureEXT=f;
	ov_glApplyTextureEXT(param0);
}
void (APIENTRY *ov_glApplyTextureEXT)(unsigned int param0)=init_ov_glApplyTextureEXT;

void __gl_ml_glApplyTextureEXT(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glApplyTextureEXT(l0);
}
extern void (APIENTRY *ov_glMultiDrawElementsEXT)(unsigned int param0,signed int* param1,unsigned int param2,void** param3,signed int param4);

static void APIENTRY init_ov_glMultiDrawElementsEXT(unsigned int param0,signed int* param1,unsigned int param2,void** param3,signed int param4)
{
	void *f;
	f=(void*)getfunc("glMultiDrawElementsEXT");
	ov_glMultiDrawElementsEXT=f;
	ov_glMultiDrawElementsEXT(param0,param1,param2,param3,param4);
}
void (APIENTRY *ov_glMultiDrawElementsEXT)(unsigned int param0,signed int* param1,unsigned int param2,void** param3,signed int param4)=init_ov_glMultiDrawElementsEXT;
extern void (APIENTRY *ov_glMultiDrawArraysEXT)(unsigned int param0,signed int* param1,signed int* param2,signed int param3);

static void APIENTRY init_ov_glMultiDrawArraysEXT(unsigned int param0,signed int* param1,signed int* param2,signed int param3)
{
	void *f;
	f=(void*)getfunc("glMultiDrawArraysEXT");
	ov_glMultiDrawArraysEXT=f;
	ov_glMultiDrawArraysEXT(param0,param1,param2,param3);
}
void (APIENTRY *ov_glMultiDrawArraysEXT)(unsigned int param0,signed int* param1,signed int* param2,signed int param3)=init_ov_glMultiDrawArraysEXT;

void __gl_ml_glMultiDrawArraysEXT(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	signed int* l1=(signed int*)(gl__ba__(v1));
	signed int* l2=(signed int*)(gl__ba__(v2));
	signed int l3=Int_val(v3);
	ov_glMultiDrawArraysEXT(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glSamplePatternEXT)(unsigned int param0);

static void APIENTRY init_ov_glSamplePatternEXT(unsigned int param0)
{
	void *f;
	f=(void*)getfunc("glSamplePatternEXT");
	ov_glSamplePatternEXT=f;
	ov_glSamplePatternEXT(param0);
}
void (APIENTRY *ov_glSamplePatternEXT)(unsigned int param0)=init_ov_glSamplePatternEXT;

void __gl_ml_glSamplePatternEXT(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glSamplePatternEXT(l0);
}
extern void (APIENTRY *ov_glSampleMaskEXT)(float param0,unsigned char param1);

static void APIENTRY init_ov_glSampleMaskEXT(float param0,unsigned char param1)
{
	void *f;
	f=(void*)getfunc("glSampleMaskEXT");
	ov_glSampleMaskEXT=f;
	ov_glSampleMaskEXT(param0,param1);
}
void (APIENTRY *ov_glSampleMaskEXT)(float param0,unsigned char param1)=init_ov_glSampleMaskEXT;

void __gl_ml_glSampleMaskEXT(value v0,value v1)
{
	float l0=Double_val(v0);
	unsigned char l1=Int_val(v1);
	ov_glSampleMaskEXT(l0,l1);
}
extern void (APIENTRY *ov_glGetColorTableParameterfvEXT)(unsigned int param0,unsigned int param1,float* param2);

static void APIENTRY init_ov_glGetColorTableParameterfvEXT(unsigned int param0,unsigned int param1,float* param2)
{
	void *f;
	f=(void*)getfunc("glGetColorTableParameterfvEXT");
	ov_glGetColorTableParameterfvEXT=f;
	ov_glGetColorTableParameterfvEXT(param0,param1,param2);
}
void (APIENTRY *ov_glGetColorTableParameterfvEXT)(unsigned int param0,unsigned int param1,float* param2)=init_ov_glGetColorTableParameterfvEXT;

void __gl_ml_glGetColorTableParameterfvEXT(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glGetColorTableParameterfvEXT(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetColorTableParameterivEXT)(unsigned int param0,unsigned int param1,signed int* param2);

static void APIENTRY init_ov_glGetColorTableParameterivEXT(unsigned int param0,unsigned int param1,signed int* param2)
{
	void *f;
	f=(void*)getfunc("glGetColorTableParameterivEXT");
	ov_glGetColorTableParameterivEXT=f;
	ov_glGetColorTableParameterivEXT(param0,param1,param2);
}
void (APIENTRY *ov_glGetColorTableParameterivEXT)(unsigned int param0,unsigned int param1,signed int* param2)=init_ov_glGetColorTableParameterivEXT;

void __gl_ml_glGetColorTableParameterivEXT(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int* l2=(signed int*)(gl__ba__(v2));
	ov_glGetColorTableParameterivEXT(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetColorTableEXT)(unsigned int param0,unsigned int param1,unsigned int param2,void* param3);

static void APIENTRY init_ov_glGetColorTableEXT(unsigned int param0,unsigned int param1,unsigned int param2,void* param3)
{
	void *f;
	f=(void*)getfunc("glGetColorTableEXT");
	ov_glGetColorTableEXT=f;
	ov_glGetColorTableEXT(param0,param1,param2,param3);
}
void (APIENTRY *ov_glGetColorTableEXT)(unsigned int param0,unsigned int param1,unsigned int param2,void* param3)=init_ov_glGetColorTableEXT;

void __gl_ml_glGetColorTableEXT(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	void* l3=gl__ba__(v3);
	ov_glGetColorTableEXT(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glColorTableEXT)(unsigned int param0,unsigned int param1,signed int param2,unsigned int param3,unsigned int param4,void* param5);

static void APIENTRY init_ov_glColorTableEXT(unsigned int param0,unsigned int param1,signed int param2,unsigned int param3,unsigned int param4,void* param5)
{
	void *f;
	f=(void*)getfunc("glColorTableEXT");
	ov_glColorTableEXT=f;
	ov_glColorTableEXT(param0,param1,param2,param3,param4,param5);
}
void (APIENTRY *ov_glColorTableEXT)(unsigned int param0,unsigned int param1,signed int param2,unsigned int param3,unsigned int param4,void* param5)=init_ov_glColorTableEXT;

void __gl_ml_glColorTableEXT(value v0,value v1,value v2,value v3,value v4,value v5)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	unsigned int l3=Int_val(v3);
	unsigned int l4=Int_val(v4);
	void* l5=gl__ba__(v5);
	ov_glColorTableEXT(l0,l1,l2,l3,l4,l5);
}

void byte___gl_ml_glColorTableEXT(value * argv,int argn)
{
	__gl_ml_glColorTableEXT(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5]);
}
extern void (APIENTRY *ov_glPixelTransformParameterfvEXT)(unsigned int param0,unsigned int param1,float* param2);

static void APIENTRY init_ov_glPixelTransformParameterfvEXT(unsigned int param0,unsigned int param1,float* param2)
{
	void *f;
	f=(void*)getfunc("glPixelTransformParameterfvEXT");
	ov_glPixelTransformParameterfvEXT=f;
	ov_glPixelTransformParameterfvEXT(param0,param1,param2);
}
void (APIENTRY *ov_glPixelTransformParameterfvEXT)(unsigned int param0,unsigned int param1,float* param2)=init_ov_glPixelTransformParameterfvEXT;

void __gl_ml_glPixelTransformParameterfvEXT(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glPixelTransformParameterfvEXT(l0,l1,l2);
}
extern void (APIENTRY *ov_glPixelTransformParameterivEXT)(unsigned int param0,unsigned int param1,signed int* param2);

static void APIENTRY init_ov_glPixelTransformParameterivEXT(unsigned int param0,unsigned int param1,signed int* param2)
{
	void *f;
	f=(void*)getfunc("glPixelTransformParameterivEXT");
	ov_glPixelTransformParameterivEXT=f;
	ov_glPixelTransformParameterivEXT(param0,param1,param2);
}
void (APIENTRY *ov_glPixelTransformParameterivEXT)(unsigned int param0,unsigned int param1,signed int* param2)=init_ov_glPixelTransformParameterivEXT;

void __gl_ml_glPixelTransformParameterivEXT(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int* l2=(signed int*)(gl__ba__(v2));
	ov_glPixelTransformParameterivEXT(l0,l1,l2);
}
extern void (APIENTRY *ov_glPixelTransformParameterfEXT)(unsigned int param0,unsigned int param1,float param2);

static void APIENTRY init_ov_glPixelTransformParameterfEXT(unsigned int param0,unsigned int param1,float param2)
{
	void *f;
	f=(void*)getfunc("glPixelTransformParameterfEXT");
	ov_glPixelTransformParameterfEXT=f;
	ov_glPixelTransformParameterfEXT(param0,param1,param2);
}
void (APIENTRY *ov_glPixelTransformParameterfEXT)(unsigned int param0,unsigned int param1,float param2)=init_ov_glPixelTransformParameterfEXT;

void __gl_ml_glPixelTransformParameterfEXT(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float l2=Double_val(v2);
	ov_glPixelTransformParameterfEXT(l0,l1,l2);
}
extern void (APIENTRY *ov_glPixelTransformParameteriEXT)(unsigned int param0,unsigned int param1,signed int param2);

static void APIENTRY init_ov_glPixelTransformParameteriEXT(unsigned int param0,unsigned int param1,signed int param2)
{
	void *f;
	f=(void*)getfunc("glPixelTransformParameteriEXT");
	ov_glPixelTransformParameteriEXT=f;
	ov_glPixelTransformParameteriEXT(param0,param1,param2);
}
void (APIENTRY *ov_glPixelTransformParameteriEXT)(unsigned int param0,unsigned int param1,signed int param2)=init_ov_glPixelTransformParameteriEXT;

void __gl_ml_glPixelTransformParameteriEXT(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	ov_glPixelTransformParameteriEXT(l0,l1,l2);
}
extern void (APIENTRY *ov_glPointParameterfvEXT)(unsigned int param0,float* param1);

static void APIENTRY init_ov_glPointParameterfvEXT(unsigned int param0,float* param1)
{
	void *f;
	f=(void*)getfunc("glPointParameterfvEXT");
	ov_glPointParameterfvEXT=f;
	ov_glPointParameterfvEXT(param0,param1);
}
void (APIENTRY *ov_glPointParameterfvEXT)(unsigned int param0,float* param1)=init_ov_glPointParameterfvEXT;

void __gl_ml_glPointParameterfvEXT(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float* l1=(float*)(gl__ba__(v1));
	ov_glPointParameterfvEXT(l0,l1);
}
extern void (APIENTRY *ov_glPointParameterfEXT)(unsigned int param0,float param1);

static void APIENTRY init_ov_glPointParameterfEXT(unsigned int param0,float param1)
{
	void *f;
	f=(void*)getfunc("glPointParameterfEXT");
	ov_glPointParameterfEXT=f;
	ov_glPointParameterfEXT(param0,param1);
}
void (APIENTRY *ov_glPointParameterfEXT)(unsigned int param0,float param1)=init_ov_glPointParameterfEXT;

void __gl_ml_glPointParameterfEXT(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float l1=Double_val(v1);
	ov_glPointParameterfEXT(l0,l1);
}
extern void (APIENTRY *ov_glPolygonOffsetEXT)(float param0,float param1);

static void APIENTRY init_ov_glPolygonOffsetEXT(float param0,float param1)
{
	void *f;
	f=(void*)getfunc("glPolygonOffsetEXT");
	ov_glPolygonOffsetEXT=f;
	ov_glPolygonOffsetEXT(param0,param1);
}
void (APIENTRY *ov_glPolygonOffsetEXT)(float param0,float param1)=init_ov_glPolygonOffsetEXT;

void __gl_ml_glPolygonOffsetEXT(value v0,value v1)
{
	float l0=Double_val(v0);
	float l1=Double_val(v1);
	ov_glPolygonOffsetEXT(l0,l1);
}
extern void (APIENTRY *ov_glSecondaryColorPointerEXT)(signed int param0,unsigned int param1,signed int param2,void* param3);

static void APIENTRY init_ov_glSecondaryColorPointerEXT(signed int param0,unsigned int param1,signed int param2,void* param3)
{
	void *f;
	f=(void*)getfunc("glSecondaryColorPointerEXT");
	ov_glSecondaryColorPointerEXT=f;
	ov_glSecondaryColorPointerEXT(param0,param1,param2,param3);
}
void (APIENTRY *ov_glSecondaryColorPointerEXT)(signed int param0,unsigned int param1,signed int param2,void* param3)=init_ov_glSecondaryColorPointerEXT;

void __gl_ml_glSecondaryColorPointerEXT(value v0,value v1,value v2,value v3)
{
	signed int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	void* l3=gl__ba__(v3);
	ov_glSecondaryColorPointerEXT(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glSecondaryColor3usvEXT)(unsigned short* param0);

static void APIENTRY init_ov_glSecondaryColor3usvEXT(unsigned short* param0)
{
	void *f;
	f=(void*)getfunc("glSecondaryColor3usvEXT");
	ov_glSecondaryColor3usvEXT=f;
	ov_glSecondaryColor3usvEXT(param0);
}
void (APIENTRY *ov_glSecondaryColor3usvEXT)(unsigned short* param0)=init_ov_glSecondaryColor3usvEXT;

void __gl_ml_glSecondaryColor3usvEXT(value v0)
{
	unsigned short* l0=(unsigned short*)(gl__ba__(v0));
	ov_glSecondaryColor3usvEXT(l0);
}
extern void (APIENTRY *ov_glSecondaryColor3usEXT)(unsigned short param0,unsigned short param1,unsigned short param2);

static void APIENTRY init_ov_glSecondaryColor3usEXT(unsigned short param0,unsigned short param1,unsigned short param2)
{
	void *f;
	f=(void*)getfunc("glSecondaryColor3usEXT");
	ov_glSecondaryColor3usEXT=f;
	ov_glSecondaryColor3usEXT(param0,param1,param2);
}
void (APIENTRY *ov_glSecondaryColor3usEXT)(unsigned short param0,unsigned short param1,unsigned short param2)=init_ov_glSecondaryColor3usEXT;

void __gl_ml_glSecondaryColor3usEXT(value v0,value v1,value v2)
{
	unsigned short l0=Int_val(v0);
	unsigned short l1=Int_val(v1);
	unsigned short l2=Int_val(v2);
	ov_glSecondaryColor3usEXT(l0,l1,l2);
}
extern void (APIENTRY *ov_glSecondaryColor3uivEXT)(unsigned int* param0);

static void APIENTRY init_ov_glSecondaryColor3uivEXT(unsigned int* param0)
{
	void *f;
	f=(void*)getfunc("glSecondaryColor3uivEXT");
	ov_glSecondaryColor3uivEXT=f;
	ov_glSecondaryColor3uivEXT(param0);
}
void (APIENTRY *ov_glSecondaryColor3uivEXT)(unsigned int* param0)=init_ov_glSecondaryColor3uivEXT;

void __gl_ml_glSecondaryColor3uivEXT(value v0)
{
	unsigned int* l0=(unsigned int*)(gl__ba__(v0));
	ov_glSecondaryColor3uivEXT(l0);
}
extern void (APIENTRY *ov_glSecondaryColor3uiEXT)(unsigned int param0,unsigned int param1,unsigned int param2);

static void APIENTRY init_ov_glSecondaryColor3uiEXT(unsigned int param0,unsigned int param1,unsigned int param2)
{
	void *f;
	f=(void*)getfunc("glSecondaryColor3uiEXT");
	ov_glSecondaryColor3uiEXT=f;
	ov_glSecondaryColor3uiEXT(param0,param1,param2);
}
void (APIENTRY *ov_glSecondaryColor3uiEXT)(unsigned int param0,unsigned int param1,unsigned int param2)=init_ov_glSecondaryColor3uiEXT;

void __gl_ml_glSecondaryColor3uiEXT(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	ov_glSecondaryColor3uiEXT(l0,l1,l2);
}
extern void (APIENTRY *ov_glSecondaryColor3ubvEXT)(unsigned char* param0);

static void APIENTRY init_ov_glSecondaryColor3ubvEXT(unsigned char* param0)
{
	void *f;
	f=(void*)getfunc("glSecondaryColor3ubvEXT");
	ov_glSecondaryColor3ubvEXT=f;
	ov_glSecondaryColor3ubvEXT(param0);
}
void (APIENTRY *ov_glSecondaryColor3ubvEXT)(unsigned char* param0)=init_ov_glSecondaryColor3ubvEXT;

void __gl_ml_glSecondaryColor3ubvEXT(value v0)
{
	unsigned char* l0=(unsigned char*)(gl__ba__(v0));
	ov_glSecondaryColor3ubvEXT(l0);
}
extern void (APIENTRY *ov_glSecondaryColor3ubEXT)(unsigned char param0,unsigned char param1,unsigned char param2);

static void APIENTRY init_ov_glSecondaryColor3ubEXT(unsigned char param0,unsigned char param1,unsigned char param2)
{
	void *f;
	f=(void*)getfunc("glSecondaryColor3ubEXT");
	ov_glSecondaryColor3ubEXT=f;
	ov_glSecondaryColor3ubEXT(param0,param1,param2);
}
void (APIENTRY *ov_glSecondaryColor3ubEXT)(unsigned char param0,unsigned char param1,unsigned char param2)=init_ov_glSecondaryColor3ubEXT;

void __gl_ml_glSecondaryColor3ubEXT(value v0,value v1,value v2)
{
	unsigned char l0=Int_val(v0);
	unsigned char l1=Int_val(v1);
	unsigned char l2=Int_val(v2);
	ov_glSecondaryColor3ubEXT(l0,l1,l2);
}
extern void (APIENTRY *ov_glSecondaryColor3svEXT)(signed short* param0);

static void APIENTRY init_ov_glSecondaryColor3svEXT(signed short* param0)
{
	void *f;
	f=(void*)getfunc("glSecondaryColor3svEXT");
	ov_glSecondaryColor3svEXT=f;
	ov_glSecondaryColor3svEXT(param0);
}
void (APIENTRY *ov_glSecondaryColor3svEXT)(signed short* param0)=init_ov_glSecondaryColor3svEXT;

void __gl_ml_glSecondaryColor3svEXT(value v0)
{
	signed short* l0=(signed short*)(gl__ba__(v0));
	ov_glSecondaryColor3svEXT(l0);
}
extern void (APIENTRY *ov_glSecondaryColor3sEXT)(signed short param0,signed short param1,signed short param2);

static void APIENTRY init_ov_glSecondaryColor3sEXT(signed short param0,signed short param1,signed short param2)
{
	void *f;
	f=(void*)getfunc("glSecondaryColor3sEXT");
	ov_glSecondaryColor3sEXT=f;
	ov_glSecondaryColor3sEXT(param0,param1,param2);
}
void (APIENTRY *ov_glSecondaryColor3sEXT)(signed short param0,signed short param1,signed short param2)=init_ov_glSecondaryColor3sEXT;

void __gl_ml_glSecondaryColor3sEXT(value v0,value v1,value v2)
{
	signed short l0=Int_val(v0);
	signed short l1=Int_val(v1);
	signed short l2=Int_val(v2);
	ov_glSecondaryColor3sEXT(l0,l1,l2);
}
extern void (APIENTRY *ov_glSecondaryColor3ivEXT)(signed int* param0);

static void APIENTRY init_ov_glSecondaryColor3ivEXT(signed int* param0)
{
	void *f;
	f=(void*)getfunc("glSecondaryColor3ivEXT");
	ov_glSecondaryColor3ivEXT=f;
	ov_glSecondaryColor3ivEXT(param0);
}
void (APIENTRY *ov_glSecondaryColor3ivEXT)(signed int* param0)=init_ov_glSecondaryColor3ivEXT;

void __gl_ml_glSecondaryColor3ivEXT(value v0)
{
	signed int* l0=(signed int*)(gl__ba__(v0));
	ov_glSecondaryColor3ivEXT(l0);
}
extern void (APIENTRY *ov_glSecondaryColor3iEXT)(signed int param0,signed int param1,signed int param2);

static void APIENTRY init_ov_glSecondaryColor3iEXT(signed int param0,signed int param1,signed int param2)
{
	void *f;
	f=(void*)getfunc("glSecondaryColor3iEXT");
	ov_glSecondaryColor3iEXT=f;
	ov_glSecondaryColor3iEXT(param0,param1,param2);
}
void (APIENTRY *ov_glSecondaryColor3iEXT)(signed int param0,signed int param1,signed int param2)=init_ov_glSecondaryColor3iEXT;

void __gl_ml_glSecondaryColor3iEXT(value v0,value v1,value v2)
{
	signed int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	ov_glSecondaryColor3iEXT(l0,l1,l2);
}
extern void (APIENTRY *ov_glSecondaryColor3fvEXT)(float* param0);

static void APIENTRY init_ov_glSecondaryColor3fvEXT(float* param0)
{
	void *f;
	f=(void*)getfunc("glSecondaryColor3fvEXT");
	ov_glSecondaryColor3fvEXT=f;
	ov_glSecondaryColor3fvEXT(param0);
}
void (APIENTRY *ov_glSecondaryColor3fvEXT)(float* param0)=init_ov_glSecondaryColor3fvEXT;

void __gl_ml_glSecondaryColor3fvEXT(value v0)
{
	float* l0=(float*)(gl__ba__(v0));
	ov_glSecondaryColor3fvEXT(l0);
}
extern void (APIENTRY *ov_glSecondaryColor3fEXT)(float param0,float param1,float param2);

static void APIENTRY init_ov_glSecondaryColor3fEXT(float param0,float param1,float param2)
{
	void *f;
	f=(void*)getfunc("glSecondaryColor3fEXT");
	ov_glSecondaryColor3fEXT=f;
	ov_glSecondaryColor3fEXT(param0,param1,param2);
}
void (APIENTRY *ov_glSecondaryColor3fEXT)(float param0,float param1,float param2)=init_ov_glSecondaryColor3fEXT;

void __gl_ml_glSecondaryColor3fEXT(value v0,value v1,value v2)
{
	float l0=Double_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	ov_glSecondaryColor3fEXT(l0,l1,l2);
}
extern void (APIENTRY *ov_glSecondaryColor3dvEXT)(double* param0);

static void APIENTRY init_ov_glSecondaryColor3dvEXT(double* param0)
{
	void *f;
	f=(void*)getfunc("glSecondaryColor3dvEXT");
	ov_glSecondaryColor3dvEXT=f;
	ov_glSecondaryColor3dvEXT(param0);
}
void (APIENTRY *ov_glSecondaryColor3dvEXT)(double* param0)=init_ov_glSecondaryColor3dvEXT;

void __gl_ml_glSecondaryColor3dvEXT(value v0)
{
	double* l0=(double*)(gl__ba__(v0));
	ov_glSecondaryColor3dvEXT(l0);
}
extern void (APIENTRY *ov_glSecondaryColor3dEXT)(double param0,double param1,double param2);

static void APIENTRY init_ov_glSecondaryColor3dEXT(double param0,double param1,double param2)
{
	void *f;
	f=(void*)getfunc("glSecondaryColor3dEXT");
	ov_glSecondaryColor3dEXT=f;
	ov_glSecondaryColor3dEXT(param0,param1,param2);
}
void (APIENTRY *ov_glSecondaryColor3dEXT)(double param0,double param1,double param2)=init_ov_glSecondaryColor3dEXT;

void __gl_ml_glSecondaryColor3dEXT(value v0,value v1,value v2)
{
	double l0=Double_val(v0);
	double l1=Double_val(v1);
	double l2=Double_val(v2);
	ov_glSecondaryColor3dEXT(l0,l1,l2);
}
extern void (APIENTRY *ov_glSecondaryColor3bvEXT)(signed char* param0);

static void APIENTRY init_ov_glSecondaryColor3bvEXT(signed char* param0)
{
	void *f;
	f=(void*)getfunc("glSecondaryColor3bvEXT");
	ov_glSecondaryColor3bvEXT=f;
	ov_glSecondaryColor3bvEXT(param0);
}
void (APIENTRY *ov_glSecondaryColor3bvEXT)(signed char* param0)=init_ov_glSecondaryColor3bvEXT;

void __gl_ml_glSecondaryColor3bvEXT(value v0)
{
	signed char* l0=(signed char*)(gl__ba__(v0));
	ov_glSecondaryColor3bvEXT(l0);
}
extern void (APIENTRY *ov_glSecondaryColor3bEXT)(signed char param0,signed char param1,signed char param2);

static void APIENTRY init_ov_glSecondaryColor3bEXT(signed char param0,signed char param1,signed char param2)
{
	void *f;
	f=(void*)getfunc("glSecondaryColor3bEXT");
	ov_glSecondaryColor3bEXT=f;
	ov_glSecondaryColor3bEXT(param0,param1,param2);
}
void (APIENTRY *ov_glSecondaryColor3bEXT)(signed char param0,signed char param1,signed char param2)=init_ov_glSecondaryColor3bEXT;

void __gl_ml_glSecondaryColor3bEXT(value v0,value v1,value v2)
{
	signed char l0=Int_val(v0);
	signed char l1=Int_val(v1);
	signed char l2=Int_val(v2);
	ov_glSecondaryColor3bEXT(l0,l1,l2);
}
extern void (APIENTRY *ov_glActiveStencilFaceEXT)(unsigned int param0);

static void APIENTRY init_ov_glActiveStencilFaceEXT(unsigned int param0)
{
	void *f;
	f=(void*)getfunc("glActiveStencilFaceEXT");
	ov_glActiveStencilFaceEXT=f;
	ov_glActiveStencilFaceEXT(param0);
}
void (APIENTRY *ov_glActiveStencilFaceEXT)(unsigned int param0)=init_ov_glActiveStencilFaceEXT;

void __gl_ml_glActiveStencilFaceEXT(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glActiveStencilFaceEXT(l0);
}
extern void (APIENTRY *ov_glTexSubImage2DEXT)(unsigned int param0,signed int param1,signed int param2,signed int param3,signed int param4,signed int param5,unsigned int param6,unsigned int param7,void* param8);

static void APIENTRY init_ov_glTexSubImage2DEXT(unsigned int param0,signed int param1,signed int param2,signed int param3,signed int param4,signed int param5,unsigned int param6,unsigned int param7,void* param8)
{
	void *f;
	f=(void*)getfunc("glTexSubImage2DEXT");
	ov_glTexSubImage2DEXT=f;
	ov_glTexSubImage2DEXT(param0,param1,param2,param3,param4,param5,param6,param7,param8);
}
void (APIENTRY *ov_glTexSubImage2DEXT)(unsigned int param0,signed int param1,signed int param2,signed int param3,signed int param4,signed int param5,unsigned int param6,unsigned int param7,void* param8)=init_ov_glTexSubImage2DEXT;

void __gl_ml_glTexSubImage2DEXT(value v0,value v1,value v2,value v3,value v4,value v5,value v6,value v7,value v8)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	signed int l4=Int_val(v4);
	signed int l5=Int_val(v5);
	unsigned int l6=Int_val(v6);
	unsigned int l7=Int_val(v7);
	void* l8=gl__ba__(v8);
	ov_glTexSubImage2DEXT(l0,l1,l2,l3,l4,l5,l6,l7,l8);
}

void byte___gl_ml_glTexSubImage2DEXT(value * argv,int argn)
{
	__gl_ml_glTexSubImage2DEXT(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6],argv[7],argv[8]);
}
extern void (APIENTRY *ov_glTexSubImage1DEXT)(unsigned int param0,signed int param1,signed int param2,signed int param3,unsigned int param4,unsigned int param5,void* param6);

static void APIENTRY init_ov_glTexSubImage1DEXT(unsigned int param0,signed int param1,signed int param2,signed int param3,unsigned int param4,unsigned int param5,void* param6)
{
	void *f;
	f=(void*)getfunc("glTexSubImage1DEXT");
	ov_glTexSubImage1DEXT=f;
	ov_glTexSubImage1DEXT(param0,param1,param2,param3,param4,param5,param6);
}
void (APIENTRY *ov_glTexSubImage1DEXT)(unsigned int param0,signed int param1,signed int param2,signed int param3,unsigned int param4,unsigned int param5,void* param6)=init_ov_glTexSubImage1DEXT;

void __gl_ml_glTexSubImage1DEXT(value v0,value v1,value v2,value v3,value v4,value v5,value v6)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	unsigned int l4=Int_val(v4);
	unsigned int l5=Int_val(v5);
	void* l6=gl__ba__(v6);
	ov_glTexSubImage1DEXT(l0,l1,l2,l3,l4,l5,l6);
}

void byte___gl_ml_glTexSubImage1DEXT(value * argv,int argn)
{
	__gl_ml_glTexSubImage1DEXT(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6]);
}
extern void (APIENTRY *ov_glTexSubImage3DEXT)(unsigned int param0,signed int param1,signed int param2,signed int param3,signed int param4,signed int param5,signed int param6,signed int param7,unsigned int param8,unsigned int param9,void* param10);

static void APIENTRY init_ov_glTexSubImage3DEXT(unsigned int param0,signed int param1,signed int param2,signed int param3,signed int param4,signed int param5,signed int param6,signed int param7,unsigned int param8,unsigned int param9,void* param10)
{
	void *f;
	f=(void*)getfunc("glTexSubImage3DEXT");
	ov_glTexSubImage3DEXT=f;
	ov_glTexSubImage3DEXT(param0,param1,param2,param3,param4,param5,param6,param7,param8,param9,param10);
}
void (APIENTRY *ov_glTexSubImage3DEXT)(unsigned int param0,signed int param1,signed int param2,signed int param3,signed int param4,signed int param5,signed int param6,signed int param7,unsigned int param8,unsigned int param9,void* param10)=init_ov_glTexSubImage3DEXT;

void __gl_ml_glTexSubImage3DEXT(value v0,value v1,value v2,value v3,value v4,value v5,value v6,value v7,value v8,value v9,value v10)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	signed int l4=Int_val(v4);
	signed int l5=Int_val(v5);
	signed int l6=Int_val(v6);
	signed int l7=Int_val(v7);
	unsigned int l8=Int_val(v8);
	unsigned int l9=Int_val(v9);
	void* l10=gl__ba__(v10);
	ov_glTexSubImage3DEXT(l0,l1,l2,l3,l4,l5,l6,l7,l8,l9,l10);
}

void byte___gl_ml_glTexSubImage3DEXT(value * argv,int argn)
{
	__gl_ml_glTexSubImage3DEXT(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6],argv[7],argv[8],argv[9],argv[10]);
}
extern void (APIENTRY *ov_glTexImage3DEXT)(unsigned int param0,signed int param1,unsigned int param2,signed int param3,signed int param4,signed int param5,signed int param6,unsigned int param7,unsigned int param8,void* param9);

static void APIENTRY init_ov_glTexImage3DEXT(unsigned int param0,signed int param1,unsigned int param2,signed int param3,signed int param4,signed int param5,signed int param6,unsigned int param7,unsigned int param8,void* param9)
{
	void *f;
	f=(void*)getfunc("glTexImage3DEXT");
	ov_glTexImage3DEXT=f;
	ov_glTexImage3DEXT(param0,param1,param2,param3,param4,param5,param6,param7,param8,param9);
}
void (APIENTRY *ov_glTexImage3DEXT)(unsigned int param0,signed int param1,unsigned int param2,signed int param3,signed int param4,signed int param5,signed int param6,unsigned int param7,unsigned int param8,void* param9)=init_ov_glTexImage3DEXT;

void __gl_ml_glTexImage3DEXT(value v0,value v1,value v2,value v3,value v4,value v5,value v6,value v7,value v8,value v9)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	signed int l4=Int_val(v4);
	signed int l5=Int_val(v5);
	signed int l6=Int_val(v6);
	unsigned int l7=Int_val(v7);
	unsigned int l8=Int_val(v8);
	void* l9=gl__ba__(v9);
	ov_glTexImage3DEXT(l0,l1,l2,l3,l4,l5,l6,l7,l8,l9);
}

void byte___gl_ml_glTexImage3DEXT(value * argv,int argn)
{
	__gl_ml_glTexImage3DEXT(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6],argv[7],argv[8],argv[9]);
}
extern void (APIENTRY *ov_glPrioritizeTexturesEXT)(signed int param0,unsigned int* param1,float* param2);

static void APIENTRY init_ov_glPrioritizeTexturesEXT(signed int param0,unsigned int* param1,float* param2)
{
	void *f;
	f=(void*)getfunc("glPrioritizeTexturesEXT");
	ov_glPrioritizeTexturesEXT=f;
	ov_glPrioritizeTexturesEXT(param0,param1,param2);
}
void (APIENTRY *ov_glPrioritizeTexturesEXT)(signed int param0,unsigned int* param1,float* param2)=init_ov_glPrioritizeTexturesEXT;

void __gl_ml_glPrioritizeTexturesEXT(value v0,value v1,value v2)
{
	signed int l0=Int_val(v0);
	unsigned int* l1=(unsigned int*)(gl__ba__(v1));
	float* l2=(float*)(gl__ba__(v2));
	ov_glPrioritizeTexturesEXT(l0,l1,l2);
}
extern unsigned char (APIENTRY *ov_glIsTextureEXT)(unsigned int param0);

static unsigned char APIENTRY init_ov_glIsTextureEXT(unsigned int param0)
{
	void *f;
	f=(void*)getfunc("glIsTextureEXT");
	ov_glIsTextureEXT=f;
	return ov_glIsTextureEXT(param0);
}
unsigned char (APIENTRY *ov_glIsTextureEXT)(unsigned int param0)=init_ov_glIsTextureEXT;

value __gl_ml_glIsTextureEXT(value v0)
{
	unsigned int l0=Int_val(v0);
	unsigned char r=ov_glIsTextureEXT(l0);
	return(Val_int(r));
}
extern void (APIENTRY *ov_glGenTexturesEXT)(signed int param0,unsigned int* param1);

static void APIENTRY init_ov_glGenTexturesEXT(signed int param0,unsigned int* param1)
{
	void *f;
	f=(void*)getfunc("glGenTexturesEXT");
	ov_glGenTexturesEXT=f;
	ov_glGenTexturesEXT(param0,param1);
}
void (APIENTRY *ov_glGenTexturesEXT)(signed int param0,unsigned int* param1)=init_ov_glGenTexturesEXT;

void __gl_ml_glGenTexturesEXT(value v0,value v1)
{
	signed int l0=Int_val(v0);
	unsigned int* l1=(unsigned int*)(gl__ba__(v1));
	ov_glGenTexturesEXT(l0,l1);
}
extern void (APIENTRY *ov_glDeleteTexturesEXT)(signed int param0,unsigned int* param1);

static void APIENTRY init_ov_glDeleteTexturesEXT(signed int param0,unsigned int* param1)
{
	void *f;
	f=(void*)getfunc("glDeleteTexturesEXT");
	ov_glDeleteTexturesEXT=f;
	ov_glDeleteTexturesEXT(param0,param1);
}
void (APIENTRY *ov_glDeleteTexturesEXT)(signed int param0,unsigned int* param1)=init_ov_glDeleteTexturesEXT;

void __gl_ml_glDeleteTexturesEXT(value v0,value v1)
{
	signed int l0=Int_val(v0);
	unsigned int* l1=(unsigned int*)(gl__ba__(v1));
	ov_glDeleteTexturesEXT(l0,l1);
}
extern void (APIENTRY *ov_glBindTextureEXT)(unsigned int param0,unsigned int param1);

static void APIENTRY init_ov_glBindTextureEXT(unsigned int param0,unsigned int param1)
{
	void *f;
	f=(void*)getfunc("glBindTextureEXT");
	ov_glBindTextureEXT=f;
	ov_glBindTextureEXT(param0,param1);
}
void (APIENTRY *ov_glBindTextureEXT)(unsigned int param0,unsigned int param1)=init_ov_glBindTextureEXT;

void __gl_ml_glBindTextureEXT(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	ov_glBindTextureEXT(l0,l1);
}
extern unsigned char (APIENTRY *ov_glAreTexturesResidentEXT)(signed int param0,unsigned int* param1,unsigned char* param2);

static unsigned char APIENTRY init_ov_glAreTexturesResidentEXT(signed int param0,unsigned int* param1,unsigned char* param2)
{
	void *f;
	f=(void*)getfunc("glAreTexturesResidentEXT");
	ov_glAreTexturesResidentEXT=f;
	return ov_glAreTexturesResidentEXT(param0,param1,param2);
}
unsigned char (APIENTRY *ov_glAreTexturesResidentEXT)(signed int param0,unsigned int* param1,unsigned char* param2)=init_ov_glAreTexturesResidentEXT;

value __gl_ml_glAreTexturesResidentEXT(value v0,value v1,value v2)
{
	signed int l0=Int_val(v0);
	unsigned int* l1=(unsigned int*)(gl__ba__(v1));
	unsigned char* l2=(unsigned char*)(gl__ba__(v2));
	unsigned char r=ov_glAreTexturesResidentEXT(l0,l1,l2);
	return(Val_int(r));
}
extern void (APIENTRY *ov_glTextureNormalEXT)(unsigned int param0);

static void APIENTRY init_ov_glTextureNormalEXT(unsigned int param0)
{
	void *f;
	f=(void*)getfunc("glTextureNormalEXT");
	ov_glTextureNormalEXT=f;
	ov_glTextureNormalEXT(param0);
}
void (APIENTRY *ov_glTextureNormalEXT)(unsigned int param0)=init_ov_glTextureNormalEXT;

void __gl_ml_glTextureNormalEXT(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glTextureNormalEXT(l0);
}
extern void (APIENTRY *ov_glVertexPointerEXT)(signed int param0,unsigned int param1,signed int param2,signed int param3,void* param4);

static void APIENTRY init_ov_glVertexPointerEXT(signed int param0,unsigned int param1,signed int param2,signed int param3,void* param4)
{
	void *f;
	f=(void*)getfunc("glVertexPointerEXT");
	ov_glVertexPointerEXT=f;
	ov_glVertexPointerEXT(param0,param1,param2,param3,param4);
}
void (APIENTRY *ov_glVertexPointerEXT)(signed int param0,unsigned int param1,signed int param2,signed int param3,void* param4)=init_ov_glVertexPointerEXT;

void __gl_ml_glVertexPointerEXT(value v0,value v1,value v2,value v3,value v4)
{
	signed int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	void* l4=gl__ba__(v4);
	ov_glVertexPointerEXT(l0,l1,l2,l3,l4);
}
extern void (APIENTRY *ov_glTexCoordPointerEXT)(signed int param0,unsigned int param1,signed int param2,signed int param3,void* param4);

static void APIENTRY init_ov_glTexCoordPointerEXT(signed int param0,unsigned int param1,signed int param2,signed int param3,void* param4)
{
	void *f;
	f=(void*)getfunc("glTexCoordPointerEXT");
	ov_glTexCoordPointerEXT=f;
	ov_glTexCoordPointerEXT(param0,param1,param2,param3,param4);
}
void (APIENTRY *ov_glTexCoordPointerEXT)(signed int param0,unsigned int param1,signed int param2,signed int param3,void* param4)=init_ov_glTexCoordPointerEXT;

void __gl_ml_glTexCoordPointerEXT(value v0,value v1,value v2,value v3,value v4)
{
	signed int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	void* l4=gl__ba__(v4);
	ov_glTexCoordPointerEXT(l0,l1,l2,l3,l4);
}
extern void (APIENTRY *ov_glNormalPointerEXT)(unsigned int param0,signed int param1,signed int param2,void* param3);

static void APIENTRY init_ov_glNormalPointerEXT(unsigned int param0,signed int param1,signed int param2,void* param3)
{
	void *f;
	f=(void*)getfunc("glNormalPointerEXT");
	ov_glNormalPointerEXT=f;
	ov_glNormalPointerEXT(param0,param1,param2,param3);
}
void (APIENTRY *ov_glNormalPointerEXT)(unsigned int param0,signed int param1,signed int param2,void* param3)=init_ov_glNormalPointerEXT;

void __gl_ml_glNormalPointerEXT(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	void* l3=gl__ba__(v3);
	ov_glNormalPointerEXT(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glIndexPointerEXT)(unsigned int param0,signed int param1,signed int param2,void* param3);

static void APIENTRY init_ov_glIndexPointerEXT(unsigned int param0,signed int param1,signed int param2,void* param3)
{
	void *f;
	f=(void*)getfunc("glIndexPointerEXT");
	ov_glIndexPointerEXT=f;
	ov_glIndexPointerEXT(param0,param1,param2,param3);
}
void (APIENTRY *ov_glIndexPointerEXT)(unsigned int param0,signed int param1,signed int param2,void* param3)=init_ov_glIndexPointerEXT;

void __gl_ml_glIndexPointerEXT(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	void* l3=gl__ba__(v3);
	ov_glIndexPointerEXT(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glGetPointervEXT)(unsigned int param0,void** param1);

static void APIENTRY init_ov_glGetPointervEXT(unsigned int param0,void** param1)
{
	void *f;
	f=(void*)getfunc("glGetPointervEXT");
	ov_glGetPointervEXT=f;
	ov_glGetPointervEXT(param0,param1);
}
void (APIENTRY *ov_glGetPointervEXT)(unsigned int param0,void** param1)=init_ov_glGetPointervEXT;
extern void (APIENTRY *ov_glEdgeFlagPointerEXT)(signed int param0,signed int param1,unsigned char* param2);

static void APIENTRY init_ov_glEdgeFlagPointerEXT(signed int param0,signed int param1,unsigned char* param2)
{
	void *f;
	f=(void*)getfunc("glEdgeFlagPointerEXT");
	ov_glEdgeFlagPointerEXT=f;
	ov_glEdgeFlagPointerEXT(param0,param1,param2);
}
void (APIENTRY *ov_glEdgeFlagPointerEXT)(signed int param0,signed int param1,unsigned char* param2)=init_ov_glEdgeFlagPointerEXT;

void __gl_ml_glEdgeFlagPointerEXT(value v0,value v1,value v2)
{
	signed int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	unsigned char* l2=(unsigned char*)(gl__ba__(v2));
	ov_glEdgeFlagPointerEXT(l0,l1,l2);
}
extern void (APIENTRY *ov_glDrawArraysEXT)(unsigned int param0,signed int param1,signed int param2);

static void APIENTRY init_ov_glDrawArraysEXT(unsigned int param0,signed int param1,signed int param2)
{
	void *f;
	f=(void*)getfunc("glDrawArraysEXT");
	ov_glDrawArraysEXT=f;
	ov_glDrawArraysEXT(param0,param1,param2);
}
void (APIENTRY *ov_glDrawArraysEXT)(unsigned int param0,signed int param1,signed int param2)=init_ov_glDrawArraysEXT;

void __gl_ml_glDrawArraysEXT(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	ov_glDrawArraysEXT(l0,l1,l2);
}
extern void (APIENTRY *ov_glColorPointerEXT)(signed int param0,unsigned int param1,signed int param2,signed int param3,void* param4);

static void APIENTRY init_ov_glColorPointerEXT(signed int param0,unsigned int param1,signed int param2,signed int param3,void* param4)
{
	void *f;
	f=(void*)getfunc("glColorPointerEXT");
	ov_glColorPointerEXT=f;
	ov_glColorPointerEXT(param0,param1,param2,param3,param4);
}
void (APIENTRY *ov_glColorPointerEXT)(signed int param0,unsigned int param1,signed int param2,signed int param3,void* param4)=init_ov_glColorPointerEXT;

void __gl_ml_glColorPointerEXT(value v0,value v1,value v2,value v3,value v4)
{
	signed int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	void* l4=gl__ba__(v4);
	ov_glColorPointerEXT(l0,l1,l2,l3,l4);
}
extern void (APIENTRY *ov_glArrayElementEXT)(signed int param0);

static void APIENTRY init_ov_glArrayElementEXT(signed int param0)
{
	void *f;
	f=(void*)getfunc("glArrayElementEXT");
	ov_glArrayElementEXT=f;
	ov_glArrayElementEXT(param0);
}
void (APIENTRY *ov_glArrayElementEXT)(signed int param0)=init_ov_glArrayElementEXT;

void __gl_ml_glArrayElementEXT(value v0)
{
	signed int l0=Int_val(v0);
	ov_glArrayElementEXT(l0);
}
extern void (APIENTRY *ov_glGetLocalConstantFloatvEXT)(unsigned int param0,unsigned int param1,float* param2);

static void APIENTRY init_ov_glGetLocalConstantFloatvEXT(unsigned int param0,unsigned int param1,float* param2)
{
	void *f;
	f=(void*)getfunc("glGetLocalConstantFloatvEXT");
	ov_glGetLocalConstantFloatvEXT=f;
	ov_glGetLocalConstantFloatvEXT(param0,param1,param2);
}
void (APIENTRY *ov_glGetLocalConstantFloatvEXT)(unsigned int param0,unsigned int param1,float* param2)=init_ov_glGetLocalConstantFloatvEXT;

void __gl_ml_glGetLocalConstantFloatvEXT(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glGetLocalConstantFloatvEXT(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetLocalConstantIntegervEXT)(unsigned int param0,unsigned int param1,signed int* param2);

static void APIENTRY init_ov_glGetLocalConstantIntegervEXT(unsigned int param0,unsigned int param1,signed int* param2)
{
	void *f;
	f=(void*)getfunc("glGetLocalConstantIntegervEXT");
	ov_glGetLocalConstantIntegervEXT=f;
	ov_glGetLocalConstantIntegervEXT(param0,param1,param2);
}
void (APIENTRY *ov_glGetLocalConstantIntegervEXT)(unsigned int param0,unsigned int param1,signed int* param2)=init_ov_glGetLocalConstantIntegervEXT;

void __gl_ml_glGetLocalConstantIntegervEXT(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int* l2=(signed int*)(gl__ba__(v2));
	ov_glGetLocalConstantIntegervEXT(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetLocalConstantBooleanvEXT)(unsigned int param0,unsigned int param1,unsigned char* param2);

static void APIENTRY init_ov_glGetLocalConstantBooleanvEXT(unsigned int param0,unsigned int param1,unsigned char* param2)
{
	void *f;
	f=(void*)getfunc("glGetLocalConstantBooleanvEXT");
	ov_glGetLocalConstantBooleanvEXT=f;
	ov_glGetLocalConstantBooleanvEXT(param0,param1,param2);
}
void (APIENTRY *ov_glGetLocalConstantBooleanvEXT)(unsigned int param0,unsigned int param1,unsigned char* param2)=init_ov_glGetLocalConstantBooleanvEXT;

void __gl_ml_glGetLocalConstantBooleanvEXT(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned char* l2=(unsigned char*)(gl__ba__(v2));
	ov_glGetLocalConstantBooleanvEXT(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetInvariantFloatvEXT)(unsigned int param0,unsigned int param1,float* param2);

static void APIENTRY init_ov_glGetInvariantFloatvEXT(unsigned int param0,unsigned int param1,float* param2)
{
	void *f;
	f=(void*)getfunc("glGetInvariantFloatvEXT");
	ov_glGetInvariantFloatvEXT=f;
	ov_glGetInvariantFloatvEXT(param0,param1,param2);
}
void (APIENTRY *ov_glGetInvariantFloatvEXT)(unsigned int param0,unsigned int param1,float* param2)=init_ov_glGetInvariantFloatvEXT;

void __gl_ml_glGetInvariantFloatvEXT(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glGetInvariantFloatvEXT(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetInvariantIntegervEXT)(unsigned int param0,unsigned int param1,signed int* param2);

static void APIENTRY init_ov_glGetInvariantIntegervEXT(unsigned int param0,unsigned int param1,signed int* param2)
{
	void *f;
	f=(void*)getfunc("glGetInvariantIntegervEXT");
	ov_glGetInvariantIntegervEXT=f;
	ov_glGetInvariantIntegervEXT(param0,param1,param2);
}
void (APIENTRY *ov_glGetInvariantIntegervEXT)(unsigned int param0,unsigned int param1,signed int* param2)=init_ov_glGetInvariantIntegervEXT;

void __gl_ml_glGetInvariantIntegervEXT(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int* l2=(signed int*)(gl__ba__(v2));
	ov_glGetInvariantIntegervEXT(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetInvariantBooleanvEXT)(unsigned int param0,unsigned int param1,unsigned char* param2);

static void APIENTRY init_ov_glGetInvariantBooleanvEXT(unsigned int param0,unsigned int param1,unsigned char* param2)
{
	void *f;
	f=(void*)getfunc("glGetInvariantBooleanvEXT");
	ov_glGetInvariantBooleanvEXT=f;
	ov_glGetInvariantBooleanvEXT(param0,param1,param2);
}
void (APIENTRY *ov_glGetInvariantBooleanvEXT)(unsigned int param0,unsigned int param1,unsigned char* param2)=init_ov_glGetInvariantBooleanvEXT;

void __gl_ml_glGetInvariantBooleanvEXT(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned char* l2=(unsigned char*)(gl__ba__(v2));
	ov_glGetInvariantBooleanvEXT(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetVariantPointervEXT)(unsigned int param0,unsigned int param1,void** param2);

static void APIENTRY init_ov_glGetVariantPointervEXT(unsigned int param0,unsigned int param1,void** param2)
{
	void *f;
	f=(void*)getfunc("glGetVariantPointervEXT");
	ov_glGetVariantPointervEXT=f;
	ov_glGetVariantPointervEXT(param0,param1,param2);
}
void (APIENTRY *ov_glGetVariantPointervEXT)(unsigned int param0,unsigned int param1,void** param2)=init_ov_glGetVariantPointervEXT;
extern void (APIENTRY *ov_glGetVariantFloatvEXT)(unsigned int param0,unsigned int param1,float* param2);

static void APIENTRY init_ov_glGetVariantFloatvEXT(unsigned int param0,unsigned int param1,float* param2)
{
	void *f;
	f=(void*)getfunc("glGetVariantFloatvEXT");
	ov_glGetVariantFloatvEXT=f;
	ov_glGetVariantFloatvEXT(param0,param1,param2);
}
void (APIENTRY *ov_glGetVariantFloatvEXT)(unsigned int param0,unsigned int param1,float* param2)=init_ov_glGetVariantFloatvEXT;

void __gl_ml_glGetVariantFloatvEXT(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glGetVariantFloatvEXT(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetVariantIntegervEXT)(unsigned int param0,unsigned int param1,signed int* param2);

static void APIENTRY init_ov_glGetVariantIntegervEXT(unsigned int param0,unsigned int param1,signed int* param2)
{
	void *f;
	f=(void*)getfunc("glGetVariantIntegervEXT");
	ov_glGetVariantIntegervEXT=f;
	ov_glGetVariantIntegervEXT(param0,param1,param2);
}
void (APIENTRY *ov_glGetVariantIntegervEXT)(unsigned int param0,unsigned int param1,signed int* param2)=init_ov_glGetVariantIntegervEXT;

void __gl_ml_glGetVariantIntegervEXT(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int* l2=(signed int*)(gl__ba__(v2));
	ov_glGetVariantIntegervEXT(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetVariantBooleanvEXT)(unsigned int param0,unsigned int param1,unsigned char* param2);

static void APIENTRY init_ov_glGetVariantBooleanvEXT(unsigned int param0,unsigned int param1,unsigned char* param2)
{
	void *f;
	f=(void*)getfunc("glGetVariantBooleanvEXT");
	ov_glGetVariantBooleanvEXT=f;
	ov_glGetVariantBooleanvEXT(param0,param1,param2);
}
void (APIENTRY *ov_glGetVariantBooleanvEXT)(unsigned int param0,unsigned int param1,unsigned char* param2)=init_ov_glGetVariantBooleanvEXT;

void __gl_ml_glGetVariantBooleanvEXT(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned char* l2=(unsigned char*)(gl__ba__(v2));
	ov_glGetVariantBooleanvEXT(l0,l1,l2);
}
extern unsigned char (APIENTRY *ov_glIsVariantEnabledEXT)(unsigned int param0,unsigned int param1);

static unsigned char APIENTRY init_ov_glIsVariantEnabledEXT(unsigned int param0,unsigned int param1)
{
	void *f;
	f=(void*)getfunc("glIsVariantEnabledEXT");
	ov_glIsVariantEnabledEXT=f;
	return ov_glIsVariantEnabledEXT(param0,param1);
}
unsigned char (APIENTRY *ov_glIsVariantEnabledEXT)(unsigned int param0,unsigned int param1)=init_ov_glIsVariantEnabledEXT;

value __gl_ml_glIsVariantEnabledEXT(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned char r=ov_glIsVariantEnabledEXT(l0,l1);
	return(Val_int(r));
}
extern unsigned int (APIENTRY *ov_glBindParameterEXT)(unsigned int param0);

static unsigned int APIENTRY init_ov_glBindParameterEXT(unsigned int param0)
{
	void *f;
	f=(void*)getfunc("glBindParameterEXT");
	ov_glBindParameterEXT=f;
	return ov_glBindParameterEXT(param0);
}
unsigned int (APIENTRY *ov_glBindParameterEXT)(unsigned int param0)=init_ov_glBindParameterEXT;

value __gl_ml_glBindParameterEXT(value v0)
{
	unsigned int l0=Int_val(v0);
	unsigned int r=ov_glBindParameterEXT(l0);
	if(((unsigned int)(r)) & 0x40000000) failwith("glBindParameterEXT int out of range");
	return(Val_int(r));
}
extern unsigned int (APIENTRY *ov_glBindTextureUnitParameterEXT)(unsigned int param0,unsigned int param1);

static unsigned int APIENTRY init_ov_glBindTextureUnitParameterEXT(unsigned int param0,unsigned int param1)
{
	void *f;
	f=(void*)getfunc("glBindTextureUnitParameterEXT");
	ov_glBindTextureUnitParameterEXT=f;
	return ov_glBindTextureUnitParameterEXT(param0,param1);
}
unsigned int (APIENTRY *ov_glBindTextureUnitParameterEXT)(unsigned int param0,unsigned int param1)=init_ov_glBindTextureUnitParameterEXT;

value __gl_ml_glBindTextureUnitParameterEXT(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int r=ov_glBindTextureUnitParameterEXT(l0,l1);
	if(((unsigned int)(r)) & 0x40000000) failwith("glBindTextureUnitParameterEXT int out of range");
	return(Val_int(r));
}
extern unsigned int (APIENTRY *ov_glBindTexGenParameterEXT)(unsigned int param0,unsigned int param1,unsigned int param2);

static unsigned int APIENTRY init_ov_glBindTexGenParameterEXT(unsigned int param0,unsigned int param1,unsigned int param2)
{
	void *f;
	f=(void*)getfunc("glBindTexGenParameterEXT");
	ov_glBindTexGenParameterEXT=f;
	return ov_glBindTexGenParameterEXT(param0,param1,param2);
}
unsigned int (APIENTRY *ov_glBindTexGenParameterEXT)(unsigned int param0,unsigned int param1,unsigned int param2)=init_ov_glBindTexGenParameterEXT;

value __gl_ml_glBindTexGenParameterEXT(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	unsigned int r=ov_glBindTexGenParameterEXT(l0,l1,l2);
	if(((unsigned int)(r)) & 0x40000000) failwith("glBindTexGenParameterEXT int out of range");
	return(Val_int(r));
}
extern unsigned int (APIENTRY *ov_glBindMaterialParameterEXT)(unsigned int param0,unsigned int param1);

static unsigned int APIENTRY init_ov_glBindMaterialParameterEXT(unsigned int param0,unsigned int param1)
{
	void *f;
	f=(void*)getfunc("glBindMaterialParameterEXT");
	ov_glBindMaterialParameterEXT=f;
	return ov_glBindMaterialParameterEXT(param0,param1);
}
unsigned int (APIENTRY *ov_glBindMaterialParameterEXT)(unsigned int param0,unsigned int param1)=init_ov_glBindMaterialParameterEXT;

value __gl_ml_glBindMaterialParameterEXT(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int r=ov_glBindMaterialParameterEXT(l0,l1);
	if(((unsigned int)(r)) & 0x40000000) failwith("glBindMaterialParameterEXT int out of range");
	return(Val_int(r));
}
extern unsigned int (APIENTRY *ov_glBindLightParameterEXT)(unsigned int param0,unsigned int param1);

static unsigned int APIENTRY init_ov_glBindLightParameterEXT(unsigned int param0,unsigned int param1)
{
	void *f;
	f=(void*)getfunc("glBindLightParameterEXT");
	ov_glBindLightParameterEXT=f;
	return ov_glBindLightParameterEXT(param0,param1);
}
unsigned int (APIENTRY *ov_glBindLightParameterEXT)(unsigned int param0,unsigned int param1)=init_ov_glBindLightParameterEXT;

value __gl_ml_glBindLightParameterEXT(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int r=ov_glBindLightParameterEXT(l0,l1);
	if(((unsigned int)(r)) & 0x40000000) failwith("glBindLightParameterEXT int out of range");
	return(Val_int(r));
}
extern void (APIENTRY *ov_glDisableVariantClientStateEXT)(unsigned int param0);

static void APIENTRY init_ov_glDisableVariantClientStateEXT(unsigned int param0)
{
	void *f;
	f=(void*)getfunc("glDisableVariantClientStateEXT");
	ov_glDisableVariantClientStateEXT=f;
	ov_glDisableVariantClientStateEXT(param0);
}
void (APIENTRY *ov_glDisableVariantClientStateEXT)(unsigned int param0)=init_ov_glDisableVariantClientStateEXT;

void __gl_ml_glDisableVariantClientStateEXT(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glDisableVariantClientStateEXT(l0);
}
extern void (APIENTRY *ov_glEnableVariantClientStateEXT)(unsigned int param0);

static void APIENTRY init_ov_glEnableVariantClientStateEXT(unsigned int param0)
{
	void *f;
	f=(void*)getfunc("glEnableVariantClientStateEXT");
	ov_glEnableVariantClientStateEXT=f;
	ov_glEnableVariantClientStateEXT(param0);
}
void (APIENTRY *ov_glEnableVariantClientStateEXT)(unsigned int param0)=init_ov_glEnableVariantClientStateEXT;

void __gl_ml_glEnableVariantClientStateEXT(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glEnableVariantClientStateEXT(l0);
}
extern void (APIENTRY *ov_glVariantPointerEXT)(unsigned int param0,unsigned int param1,unsigned int param2,void* param3);

static void APIENTRY init_ov_glVariantPointerEXT(unsigned int param0,unsigned int param1,unsigned int param2,void* param3)
{
	void *f;
	f=(void*)getfunc("glVariantPointerEXT");
	ov_glVariantPointerEXT=f;
	ov_glVariantPointerEXT(param0,param1,param2,param3);
}
void (APIENTRY *ov_glVariantPointerEXT)(unsigned int param0,unsigned int param1,unsigned int param2,void* param3)=init_ov_glVariantPointerEXT;

void __gl_ml_glVariantPointerEXT(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	void* l3=gl__ba__(v3);
	ov_glVariantPointerEXT(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glVariantuivEXT)(unsigned int param0,unsigned int* param1);

static void APIENTRY init_ov_glVariantuivEXT(unsigned int param0,unsigned int* param1)
{
	void *f;
	f=(void*)getfunc("glVariantuivEXT");
	ov_glVariantuivEXT=f;
	ov_glVariantuivEXT(param0,param1);
}
void (APIENTRY *ov_glVariantuivEXT)(unsigned int param0,unsigned int* param1)=init_ov_glVariantuivEXT;

void __gl_ml_glVariantuivEXT(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	unsigned int* l1=(unsigned int*)(gl__ba__(v1));
	ov_glVariantuivEXT(l0,l1);
}
extern void (APIENTRY *ov_glVariantusvEXT)(unsigned int param0,unsigned short* param1);

static void APIENTRY init_ov_glVariantusvEXT(unsigned int param0,unsigned short* param1)
{
	void *f;
	f=(void*)getfunc("glVariantusvEXT");
	ov_glVariantusvEXT=f;
	ov_glVariantusvEXT(param0,param1);
}
void (APIENTRY *ov_glVariantusvEXT)(unsigned int param0,unsigned short* param1)=init_ov_glVariantusvEXT;

void __gl_ml_glVariantusvEXT(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	unsigned short* l1=(unsigned short*)(gl__ba__(v1));
	ov_glVariantusvEXT(l0,l1);
}
extern void (APIENTRY *ov_glVariantubvEXT)(unsigned int param0,unsigned char* param1);

static void APIENTRY init_ov_glVariantubvEXT(unsigned int param0,unsigned char* param1)
{
	void *f;
	f=(void*)getfunc("glVariantubvEXT");
	ov_glVariantubvEXT=f;
	ov_glVariantubvEXT(param0,param1);
}
void (APIENTRY *ov_glVariantubvEXT)(unsigned int param0,unsigned char* param1)=init_ov_glVariantubvEXT;

void __gl_ml_glVariantubvEXT(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	unsigned char* l1=(unsigned char*)(gl__ba__(v1));
	ov_glVariantubvEXT(l0,l1);
}
extern void (APIENTRY *ov_glVariantdvEXT)(unsigned int param0,double* param1);

static void APIENTRY init_ov_glVariantdvEXT(unsigned int param0,double* param1)
{
	void *f;
	f=(void*)getfunc("glVariantdvEXT");
	ov_glVariantdvEXT=f;
	ov_glVariantdvEXT(param0,param1);
}
void (APIENTRY *ov_glVariantdvEXT)(unsigned int param0,double* param1)=init_ov_glVariantdvEXT;

void __gl_ml_glVariantdvEXT(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	double* l1=(double*)(gl__ba__(v1));
	ov_glVariantdvEXT(l0,l1);
}
extern void (APIENTRY *ov_glVariantfvEXT)(unsigned int param0,float* param1);

static void APIENTRY init_ov_glVariantfvEXT(unsigned int param0,float* param1)
{
	void *f;
	f=(void*)getfunc("glVariantfvEXT");
	ov_glVariantfvEXT=f;
	ov_glVariantfvEXT(param0,param1);
}
void (APIENTRY *ov_glVariantfvEXT)(unsigned int param0,float* param1)=init_ov_glVariantfvEXT;

void __gl_ml_glVariantfvEXT(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float* l1=(float*)(gl__ba__(v1));
	ov_glVariantfvEXT(l0,l1);
}
extern void (APIENTRY *ov_glVariantivEXT)(unsigned int param0,signed int* param1);

static void APIENTRY init_ov_glVariantivEXT(unsigned int param0,signed int* param1)
{
	void *f;
	f=(void*)getfunc("glVariantivEXT");
	ov_glVariantivEXT=f;
	ov_glVariantivEXT(param0,param1);
}
void (APIENTRY *ov_glVariantivEXT)(unsigned int param0,signed int* param1)=init_ov_glVariantivEXT;

void __gl_ml_glVariantivEXT(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed int* l1=(signed int*)(gl__ba__(v1));
	ov_glVariantivEXT(l0,l1);
}
extern void (APIENTRY *ov_glVariantsvEXT)(unsigned int param0,signed short* param1);

static void APIENTRY init_ov_glVariantsvEXT(unsigned int param0,signed short* param1)
{
	void *f;
	f=(void*)getfunc("glVariantsvEXT");
	ov_glVariantsvEXT=f;
	ov_glVariantsvEXT(param0,param1);
}
void (APIENTRY *ov_glVariantsvEXT)(unsigned int param0,signed short* param1)=init_ov_glVariantsvEXT;

void __gl_ml_glVariantsvEXT(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed short* l1=(signed short*)(gl__ba__(v1));
	ov_glVariantsvEXT(l0,l1);
}
extern void (APIENTRY *ov_glVariantbvEXT)(unsigned int param0,signed char* param1);

static void APIENTRY init_ov_glVariantbvEXT(unsigned int param0,signed char* param1)
{
	void *f;
	f=(void*)getfunc("glVariantbvEXT");
	ov_glVariantbvEXT=f;
	ov_glVariantbvEXT(param0,param1);
}
void (APIENTRY *ov_glVariantbvEXT)(unsigned int param0,signed char* param1)=init_ov_glVariantbvEXT;

void __gl_ml_glVariantbvEXT(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed char* l1=(signed char*)(gl__ba__(v1));
	ov_glVariantbvEXT(l0,l1);
}
extern void (APIENTRY *ov_glSetLocalConstantEXT)(unsigned int param0,unsigned int param1,void* param2);

static void APIENTRY init_ov_glSetLocalConstantEXT(unsigned int param0,unsigned int param1,void* param2)
{
	void *f;
	f=(void*)getfunc("glSetLocalConstantEXT");
	ov_glSetLocalConstantEXT=f;
	ov_glSetLocalConstantEXT(param0,param1,param2);
}
void (APIENTRY *ov_glSetLocalConstantEXT)(unsigned int param0,unsigned int param1,void* param2)=init_ov_glSetLocalConstantEXT;

void __gl_ml_glSetLocalConstantEXT(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	void* l2=gl__ba__(v2);
	ov_glSetLocalConstantEXT(l0,l1,l2);
}
extern void (APIENTRY *ov_glSetInvariantEXT)(unsigned int param0,unsigned int param1,void* param2);

static void APIENTRY init_ov_glSetInvariantEXT(unsigned int param0,unsigned int param1,void* param2)
{
	void *f;
	f=(void*)getfunc("glSetInvariantEXT");
	ov_glSetInvariantEXT=f;
	ov_glSetInvariantEXT(param0,param1,param2);
}
void (APIENTRY *ov_glSetInvariantEXT)(unsigned int param0,unsigned int param1,void* param2)=init_ov_glSetInvariantEXT;

void __gl_ml_glSetInvariantEXT(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	void* l2=gl__ba__(v2);
	ov_glSetInvariantEXT(l0,l1,l2);
}
extern unsigned int (APIENTRY *ov_glGenSymbolsEXT)(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3);

static unsigned int APIENTRY init_ov_glGenSymbolsEXT(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3)
{
	void *f;
	f=(void*)getfunc("glGenSymbolsEXT");
	ov_glGenSymbolsEXT=f;
	return ov_glGenSymbolsEXT(param0,param1,param2,param3);
}
unsigned int (APIENTRY *ov_glGenSymbolsEXT)(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3)=init_ov_glGenSymbolsEXT;

value __gl_ml_glGenSymbolsEXT(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	unsigned int l3=Int_val(v3);
	unsigned int r=ov_glGenSymbolsEXT(l0,l1,l2,l3);
	if(((unsigned int)(r)) & 0x40000000) failwith("glGenSymbolsEXT int out of range");
	return(Val_int(r));
}
extern void (APIENTRY *ov_glExtractComponentEXT)(unsigned int param0,unsigned int param1,unsigned int param2);

static void APIENTRY init_ov_glExtractComponentEXT(unsigned int param0,unsigned int param1,unsigned int param2)
{
	void *f;
	f=(void*)getfunc("glExtractComponentEXT");
	ov_glExtractComponentEXT=f;
	ov_glExtractComponentEXT(param0,param1,param2);
}
void (APIENTRY *ov_glExtractComponentEXT)(unsigned int param0,unsigned int param1,unsigned int param2)=init_ov_glExtractComponentEXT;

void __gl_ml_glExtractComponentEXT(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	ov_glExtractComponentEXT(l0,l1,l2);
}
extern void (APIENTRY *ov_glInsertComponentEXT)(unsigned int param0,unsigned int param1,unsigned int param2);

static void APIENTRY init_ov_glInsertComponentEXT(unsigned int param0,unsigned int param1,unsigned int param2)
{
	void *f;
	f=(void*)getfunc("glInsertComponentEXT");
	ov_glInsertComponentEXT=f;
	ov_glInsertComponentEXT(param0,param1,param2);
}
void (APIENTRY *ov_glInsertComponentEXT)(unsigned int param0,unsigned int param1,unsigned int param2)=init_ov_glInsertComponentEXT;

void __gl_ml_glInsertComponentEXT(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	ov_glInsertComponentEXT(l0,l1,l2);
}
extern void (APIENTRY *ov_glWriteMaskEXT)(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3,unsigned int param4,unsigned int param5);

static void APIENTRY init_ov_glWriteMaskEXT(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3,unsigned int param4,unsigned int param5)
{
	void *f;
	f=(void*)getfunc("glWriteMaskEXT");
	ov_glWriteMaskEXT=f;
	ov_glWriteMaskEXT(param0,param1,param2,param3,param4,param5);
}
void (APIENTRY *ov_glWriteMaskEXT)(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3,unsigned int param4,unsigned int param5)=init_ov_glWriteMaskEXT;

void __gl_ml_glWriteMaskEXT(value v0,value v1,value v2,value v3,value v4,value v5)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	unsigned int l3=Int_val(v3);
	unsigned int l4=Int_val(v4);
	unsigned int l5=Int_val(v5);
	ov_glWriteMaskEXT(l0,l1,l2,l3,l4,l5);
}

void byte___gl_ml_glWriteMaskEXT(value * argv,int argn)
{
	__gl_ml_glWriteMaskEXT(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5]);
}
extern void (APIENTRY *ov_glSwizzleEXT)(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3,unsigned int param4,unsigned int param5);

static void APIENTRY init_ov_glSwizzleEXT(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3,unsigned int param4,unsigned int param5)
{
	void *f;
	f=(void*)getfunc("glSwizzleEXT");
	ov_glSwizzleEXT=f;
	ov_glSwizzleEXT(param0,param1,param2,param3,param4,param5);
}
void (APIENTRY *ov_glSwizzleEXT)(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3,unsigned int param4,unsigned int param5)=init_ov_glSwizzleEXT;

void __gl_ml_glSwizzleEXT(value v0,value v1,value v2,value v3,value v4,value v5)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	unsigned int l3=Int_val(v3);
	unsigned int l4=Int_val(v4);
	unsigned int l5=Int_val(v5);
	ov_glSwizzleEXT(l0,l1,l2,l3,l4,l5);
}

void byte___gl_ml_glSwizzleEXT(value * argv,int argn)
{
	__gl_ml_glSwizzleEXT(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5]);
}
extern void (APIENTRY *ov_glShaderOp3EXT)(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3,unsigned int param4);

static void APIENTRY init_ov_glShaderOp3EXT(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3,unsigned int param4)
{
	void *f;
	f=(void*)getfunc("glShaderOp3EXT");
	ov_glShaderOp3EXT=f;
	ov_glShaderOp3EXT(param0,param1,param2,param3,param4);
}
void (APIENTRY *ov_glShaderOp3EXT)(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3,unsigned int param4)=init_ov_glShaderOp3EXT;

void __gl_ml_glShaderOp3EXT(value v0,value v1,value v2,value v3,value v4)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	unsigned int l3=Int_val(v3);
	unsigned int l4=Int_val(v4);
	ov_glShaderOp3EXT(l0,l1,l2,l3,l4);
}
extern void (APIENTRY *ov_glShaderOp2EXT)(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3);

static void APIENTRY init_ov_glShaderOp2EXT(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3)
{
	void *f;
	f=(void*)getfunc("glShaderOp2EXT");
	ov_glShaderOp2EXT=f;
	ov_glShaderOp2EXT(param0,param1,param2,param3);
}
void (APIENTRY *ov_glShaderOp2EXT)(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3)=init_ov_glShaderOp2EXT;

void __gl_ml_glShaderOp2EXT(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	unsigned int l3=Int_val(v3);
	ov_glShaderOp2EXT(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glShaderOp1EXT)(unsigned int param0,unsigned int param1,unsigned int param2);

static void APIENTRY init_ov_glShaderOp1EXT(unsigned int param0,unsigned int param1,unsigned int param2)
{
	void *f;
	f=(void*)getfunc("glShaderOp1EXT");
	ov_glShaderOp1EXT=f;
	ov_glShaderOp1EXT(param0,param1,param2);
}
void (APIENTRY *ov_glShaderOp1EXT)(unsigned int param0,unsigned int param1,unsigned int param2)=init_ov_glShaderOp1EXT;

void __gl_ml_glShaderOp1EXT(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	ov_glShaderOp1EXT(l0,l1,l2);
}
extern void (APIENTRY *ov_glDeleteVertexShaderEXT)(unsigned int param0);

static void APIENTRY init_ov_glDeleteVertexShaderEXT(unsigned int param0)
{
	void *f;
	f=(void*)getfunc("glDeleteVertexShaderEXT");
	ov_glDeleteVertexShaderEXT=f;
	ov_glDeleteVertexShaderEXT(param0);
}
void (APIENTRY *ov_glDeleteVertexShaderEXT)(unsigned int param0)=init_ov_glDeleteVertexShaderEXT;

void __gl_ml_glDeleteVertexShaderEXT(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glDeleteVertexShaderEXT(l0);
}
extern unsigned int (APIENTRY *ov_glGenVertexShadersEXT)(unsigned int param0);

static unsigned int APIENTRY init_ov_glGenVertexShadersEXT(unsigned int param0)
{
	void *f;
	f=(void*)getfunc("glGenVertexShadersEXT");
	ov_glGenVertexShadersEXT=f;
	return ov_glGenVertexShadersEXT(param0);
}
unsigned int (APIENTRY *ov_glGenVertexShadersEXT)(unsigned int param0)=init_ov_glGenVertexShadersEXT;

value __gl_ml_glGenVertexShadersEXT(value v0)
{
	unsigned int l0=Int_val(v0);
	unsigned int r=ov_glGenVertexShadersEXT(l0);
	if(((unsigned int)(r)) & 0x40000000) failwith("glGenVertexShadersEXT int out of range");
	return(Val_int(r));
}
extern void (APIENTRY *ov_glBindVertexShaderEXT)(unsigned int param0);

static void APIENTRY init_ov_glBindVertexShaderEXT(unsigned int param0)
{
	void *f;
	f=(void*)getfunc("glBindVertexShaderEXT");
	ov_glBindVertexShaderEXT=f;
	ov_glBindVertexShaderEXT(param0);
}
void (APIENTRY *ov_glBindVertexShaderEXT)(unsigned int param0)=init_ov_glBindVertexShaderEXT;

void __gl_ml_glBindVertexShaderEXT(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glBindVertexShaderEXT(l0);
}
extern void (APIENTRY *ov_glEndVertexShaderEXT)();

static void APIENTRY init_ov_glEndVertexShaderEXT()
{
	void *f;
	f=(void*)getfunc("glEndVertexShaderEXT");
	ov_glEndVertexShaderEXT=f;
	ov_glEndVertexShaderEXT();
}
void (APIENTRY *ov_glEndVertexShaderEXT)()=init_ov_glEndVertexShaderEXT;

void __gl_ml_glEndVertexShaderEXT()
{
	ov_glEndVertexShaderEXT();
}
extern void (APIENTRY *ov_glBeginVertexShaderEXT)();

static void APIENTRY init_ov_glBeginVertexShaderEXT()
{
	void *f;
	f=(void*)getfunc("glBeginVertexShaderEXT");
	ov_glBeginVertexShaderEXT=f;
	ov_glBeginVertexShaderEXT();
}
void (APIENTRY *ov_glBeginVertexShaderEXT)()=init_ov_glBeginVertexShaderEXT;

void __gl_ml_glBeginVertexShaderEXT()
{
	ov_glBeginVertexShaderEXT();
}
extern void (APIENTRY *ov_glVertexWeightPointerEXT)(signed int param0,unsigned int param1,signed int param2,void* param3);

static void APIENTRY init_ov_glVertexWeightPointerEXT(signed int param0,unsigned int param1,signed int param2,void* param3)
{
	void *f;
	f=(void*)getfunc("glVertexWeightPointerEXT");
	ov_glVertexWeightPointerEXT=f;
	ov_glVertexWeightPointerEXT(param0,param1,param2,param3);
}
void (APIENTRY *ov_glVertexWeightPointerEXT)(signed int param0,unsigned int param1,signed int param2,void* param3)=init_ov_glVertexWeightPointerEXT;

void __gl_ml_glVertexWeightPointerEXT(value v0,value v1,value v2,value v3)
{
	signed int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	void* l3=gl__ba__(v3);
	ov_glVertexWeightPointerEXT(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glVertexWeightfvEXT)(float* param0);

static void APIENTRY init_ov_glVertexWeightfvEXT(float* param0)
{
	void *f;
	f=(void*)getfunc("glVertexWeightfvEXT");
	ov_glVertexWeightfvEXT=f;
	ov_glVertexWeightfvEXT(param0);
}
void (APIENTRY *ov_glVertexWeightfvEXT)(float* param0)=init_ov_glVertexWeightfvEXT;

void __gl_ml_glVertexWeightfvEXT(value v0)
{
	float* l0=(float*)(gl__ba__(v0));
	ov_glVertexWeightfvEXT(l0);
}
extern void (APIENTRY *ov_glVertexWeightfEXT)(float param0);

static void APIENTRY init_ov_glVertexWeightfEXT(float param0)
{
	void *f;
	f=(void*)getfunc("glVertexWeightfEXT");
	ov_glVertexWeightfEXT=f;
	ov_glVertexWeightfEXT(param0);
}
void (APIENTRY *ov_glVertexWeightfEXT)(float param0)=init_ov_glVertexWeightfEXT;

void __gl_ml_glVertexWeightfEXT(value v0)
{
	float l0=Double_val(v0);
	ov_glVertexWeightfEXT(l0);
}
extern void (APIENTRY *ov_glEvalMapsNV)(unsigned int param0,unsigned int param1);

static void APIENTRY init_ov_glEvalMapsNV(unsigned int param0,unsigned int param1)
{
	void *f;
	f=(void*)getfunc("glEvalMapsNV");
	ov_glEvalMapsNV=f;
	ov_glEvalMapsNV(param0,param1);
}
void (APIENTRY *ov_glEvalMapsNV)(unsigned int param0,unsigned int param1)=init_ov_glEvalMapsNV;

void __gl_ml_glEvalMapsNV(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	ov_glEvalMapsNV(l0,l1);
}
extern void (APIENTRY *ov_glGetMapAttribParameterfvNV)(unsigned int param0,unsigned int param1,unsigned int param2,float* param3);

static void APIENTRY init_ov_glGetMapAttribParameterfvNV(unsigned int param0,unsigned int param1,unsigned int param2,float* param3)
{
	void *f;
	f=(void*)getfunc("glGetMapAttribParameterfvNV");
	ov_glGetMapAttribParameterfvNV=f;
	ov_glGetMapAttribParameterfvNV(param0,param1,param2,param3);
}
void (APIENTRY *ov_glGetMapAttribParameterfvNV)(unsigned int param0,unsigned int param1,unsigned int param2,float* param3)=init_ov_glGetMapAttribParameterfvNV;

void __gl_ml_glGetMapAttribParameterfvNV(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	float* l3=(float*)(gl__ba__(v3));
	ov_glGetMapAttribParameterfvNV(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glGetMapAttribParameterivNV)(unsigned int param0,unsigned int param1,unsigned int param2,signed int* param3);

static void APIENTRY init_ov_glGetMapAttribParameterivNV(unsigned int param0,unsigned int param1,unsigned int param2,signed int* param3)
{
	void *f;
	f=(void*)getfunc("glGetMapAttribParameterivNV");
	ov_glGetMapAttribParameterivNV=f;
	ov_glGetMapAttribParameterivNV(param0,param1,param2,param3);
}
void (APIENTRY *ov_glGetMapAttribParameterivNV)(unsigned int param0,unsigned int param1,unsigned int param2,signed int* param3)=init_ov_glGetMapAttribParameterivNV;

void __gl_ml_glGetMapAttribParameterivNV(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	signed int* l3=(signed int*)(gl__ba__(v3));
	ov_glGetMapAttribParameterivNV(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glGetMapParameterfvNV)(unsigned int param0,unsigned int param1,float* param2);

static void APIENTRY init_ov_glGetMapParameterfvNV(unsigned int param0,unsigned int param1,float* param2)
{
	void *f;
	f=(void*)getfunc("glGetMapParameterfvNV");
	ov_glGetMapParameterfvNV=f;
	ov_glGetMapParameterfvNV(param0,param1,param2);
}
void (APIENTRY *ov_glGetMapParameterfvNV)(unsigned int param0,unsigned int param1,float* param2)=init_ov_glGetMapParameterfvNV;

void __gl_ml_glGetMapParameterfvNV(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glGetMapParameterfvNV(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetMapParameterivNV)(unsigned int param0,unsigned int param1,signed int* param2);

static void APIENTRY init_ov_glGetMapParameterivNV(unsigned int param0,unsigned int param1,signed int* param2)
{
	void *f;
	f=(void*)getfunc("glGetMapParameterivNV");
	ov_glGetMapParameterivNV=f;
	ov_glGetMapParameterivNV(param0,param1,param2);
}
void (APIENTRY *ov_glGetMapParameterivNV)(unsigned int param0,unsigned int param1,signed int* param2)=init_ov_glGetMapParameterivNV;

void __gl_ml_glGetMapParameterivNV(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int* l2=(signed int*)(gl__ba__(v2));
	ov_glGetMapParameterivNV(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetMapControlPointsNV)(unsigned int param0,unsigned int param1,unsigned int param2,signed int param3,signed int param4,unsigned char param5,void* param6);

static void APIENTRY init_ov_glGetMapControlPointsNV(unsigned int param0,unsigned int param1,unsigned int param2,signed int param3,signed int param4,unsigned char param5,void* param6)
{
	void *f;
	f=(void*)getfunc("glGetMapControlPointsNV");
	ov_glGetMapControlPointsNV=f;
	ov_glGetMapControlPointsNV(param0,param1,param2,param3,param4,param5,param6);
}
void (APIENTRY *ov_glGetMapControlPointsNV)(unsigned int param0,unsigned int param1,unsigned int param2,signed int param3,signed int param4,unsigned char param5,void* param6)=init_ov_glGetMapControlPointsNV;

void __gl_ml_glGetMapControlPointsNV(value v0,value v1,value v2,value v3,value v4,value v5,value v6)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	signed int l4=Int_val(v4);
	unsigned char l5=Int_val(v5);
	void* l6=gl__ba__(v6);
	ov_glGetMapControlPointsNV(l0,l1,l2,l3,l4,l5,l6);
}

void byte___gl_ml_glGetMapControlPointsNV(value * argv,int argn)
{
	__gl_ml_glGetMapControlPointsNV(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6]);
}
extern void (APIENTRY *ov_glMapParameterfvNV)(unsigned int param0,unsigned int param1,float* param2);

static void APIENTRY init_ov_glMapParameterfvNV(unsigned int param0,unsigned int param1,float* param2)
{
	void *f;
	f=(void*)getfunc("glMapParameterfvNV");
	ov_glMapParameterfvNV=f;
	ov_glMapParameterfvNV(param0,param1,param2);
}
void (APIENTRY *ov_glMapParameterfvNV)(unsigned int param0,unsigned int param1,float* param2)=init_ov_glMapParameterfvNV;

void __gl_ml_glMapParameterfvNV(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glMapParameterfvNV(l0,l1,l2);
}
extern void (APIENTRY *ov_glMapParameterivNV)(unsigned int param0,unsigned int param1,signed int* param2);

static void APIENTRY init_ov_glMapParameterivNV(unsigned int param0,unsigned int param1,signed int* param2)
{
	void *f;
	f=(void*)getfunc("glMapParameterivNV");
	ov_glMapParameterivNV=f;
	ov_glMapParameterivNV(param0,param1,param2);
}
void (APIENTRY *ov_glMapParameterivNV)(unsigned int param0,unsigned int param1,signed int* param2)=init_ov_glMapParameterivNV;

void __gl_ml_glMapParameterivNV(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int* l2=(signed int*)(gl__ba__(v2));
	ov_glMapParameterivNV(l0,l1,l2);
}
extern void (APIENTRY *ov_glMapControlPointsNV)(unsigned int param0,unsigned int param1,unsigned int param2,signed int param3,signed int param4,signed int param5,signed int param6,unsigned char param7,void* param8);

static void APIENTRY init_ov_glMapControlPointsNV(unsigned int param0,unsigned int param1,unsigned int param2,signed int param3,signed int param4,signed int param5,signed int param6,unsigned char param7,void* param8)
{
	void *f;
	f=(void*)getfunc("glMapControlPointsNV");
	ov_glMapControlPointsNV=f;
	ov_glMapControlPointsNV(param0,param1,param2,param3,param4,param5,param6,param7,param8);
}
void (APIENTRY *ov_glMapControlPointsNV)(unsigned int param0,unsigned int param1,unsigned int param2,signed int param3,signed int param4,signed int param5,signed int param6,unsigned char param7,void* param8)=init_ov_glMapControlPointsNV;

void __gl_ml_glMapControlPointsNV(value v0,value v1,value v2,value v3,value v4,value v5,value v6,value v7,value v8)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	signed int l4=Int_val(v4);
	signed int l5=Int_val(v5);
	signed int l6=Int_val(v6);
	unsigned char l7=Int_val(v7);
	void* l8=gl__ba__(v8);
	ov_glMapControlPointsNV(l0,l1,l2,l3,l4,l5,l6,l7,l8);
}

void byte___gl_ml_glMapControlPointsNV(value * argv,int argn)
{
	__gl_ml_glMapControlPointsNV(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6],argv[7],argv[8]);
}
extern void (APIENTRY *ov_glSetFenceNV)(unsigned int param0,unsigned int param1);

static void APIENTRY init_ov_glSetFenceNV(unsigned int param0,unsigned int param1)
{
	void *f;
	f=(void*)getfunc("glSetFenceNV");
	ov_glSetFenceNV=f;
	ov_glSetFenceNV(param0,param1);
}
void (APIENTRY *ov_glSetFenceNV)(unsigned int param0,unsigned int param1)=init_ov_glSetFenceNV;

void __gl_ml_glSetFenceNV(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	ov_glSetFenceNV(l0,l1);
}
extern void (APIENTRY *ov_glFinishFenceNV)(unsigned int param0);

static void APIENTRY init_ov_glFinishFenceNV(unsigned int param0)
{
	void *f;
	f=(void*)getfunc("glFinishFenceNV");
	ov_glFinishFenceNV=f;
	ov_glFinishFenceNV(param0);
}
void (APIENTRY *ov_glFinishFenceNV)(unsigned int param0)=init_ov_glFinishFenceNV;

void __gl_ml_glFinishFenceNV(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glFinishFenceNV(l0);
}
extern void (APIENTRY *ov_glGetFenceivNV)(unsigned int param0,unsigned int param1,signed int* param2);

static void APIENTRY init_ov_glGetFenceivNV(unsigned int param0,unsigned int param1,signed int* param2)
{
	void *f;
	f=(void*)getfunc("glGetFenceivNV");
	ov_glGetFenceivNV=f;
	ov_glGetFenceivNV(param0,param1,param2);
}
void (APIENTRY *ov_glGetFenceivNV)(unsigned int param0,unsigned int param1,signed int* param2)=init_ov_glGetFenceivNV;

void __gl_ml_glGetFenceivNV(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int* l2=(signed int*)(gl__ba__(v2));
	ov_glGetFenceivNV(l0,l1,l2);
}
extern unsigned char (APIENTRY *ov_glTestFenceNV)(unsigned int param0);

static unsigned char APIENTRY init_ov_glTestFenceNV(unsigned int param0)
{
	void *f;
	f=(void*)getfunc("glTestFenceNV");
	ov_glTestFenceNV=f;
	return ov_glTestFenceNV(param0);
}
unsigned char (APIENTRY *ov_glTestFenceNV)(unsigned int param0)=init_ov_glTestFenceNV;

value __gl_ml_glTestFenceNV(value v0)
{
	unsigned int l0=Int_val(v0);
	unsigned char r=ov_glTestFenceNV(l0);
	return(Val_int(r));
}
extern unsigned char (APIENTRY *ov_glIsFenceNV)(unsigned int param0);

static unsigned char APIENTRY init_ov_glIsFenceNV(unsigned int param0)
{
	void *f;
	f=(void*)getfunc("glIsFenceNV");
	ov_glIsFenceNV=f;
	return ov_glIsFenceNV(param0);
}
unsigned char (APIENTRY *ov_glIsFenceNV)(unsigned int param0)=init_ov_glIsFenceNV;

value __gl_ml_glIsFenceNV(value v0)
{
	unsigned int l0=Int_val(v0);
	unsigned char r=ov_glIsFenceNV(l0);
	return(Val_int(r));
}
extern void (APIENTRY *ov_glGenFencesNV)(signed int param0,unsigned int* param1);

static void APIENTRY init_ov_glGenFencesNV(signed int param0,unsigned int* param1)
{
	void *f;
	f=(void*)getfunc("glGenFencesNV");
	ov_glGenFencesNV=f;
	ov_glGenFencesNV(param0,param1);
}
void (APIENTRY *ov_glGenFencesNV)(signed int param0,unsigned int* param1)=init_ov_glGenFencesNV;

void __gl_ml_glGenFencesNV(value v0,value v1)
{
	signed int l0=Int_val(v0);
	unsigned int* l1=(unsigned int*)(gl__ba__(v1));
	ov_glGenFencesNV(l0,l1);
}
extern void (APIENTRY *ov_glDeleteFencesNV)(signed int param0,unsigned int* param1);

static void APIENTRY init_ov_glDeleteFencesNV(signed int param0,unsigned int* param1)
{
	void *f;
	f=(void*)getfunc("glDeleteFencesNV");
	ov_glDeleteFencesNV=f;
	ov_glDeleteFencesNV(param0,param1);
}
void (APIENTRY *ov_glDeleteFencesNV)(signed int param0,unsigned int* param1)=init_ov_glDeleteFencesNV;

void __gl_ml_glDeleteFencesNV(value v0,value v1)
{
	signed int l0=Int_val(v0);
	unsigned int* l1=(unsigned int*)(gl__ba__(v1));
	ov_glDeleteFencesNV(l0,l1);
}
extern void (APIENTRY *ov_glGetOcclusionQueryuivNV)(unsigned int param0,unsigned int param1,unsigned int* param2);

static void APIENTRY init_ov_glGetOcclusionQueryuivNV(unsigned int param0,unsigned int param1,unsigned int* param2)
{
	void *f;
	f=(void*)getfunc("glGetOcclusionQueryuivNV");
	ov_glGetOcclusionQueryuivNV=f;
	ov_glGetOcclusionQueryuivNV(param0,param1,param2);
}
void (APIENTRY *ov_glGetOcclusionQueryuivNV)(unsigned int param0,unsigned int param1,unsigned int* param2)=init_ov_glGetOcclusionQueryuivNV;

void __gl_ml_glGetOcclusionQueryuivNV(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int* l2=(unsigned int*)(gl__ba__(v2));
	ov_glGetOcclusionQueryuivNV(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetOcclusionQueryivNV)(unsigned int param0,unsigned int param1,signed int* param2);

static void APIENTRY init_ov_glGetOcclusionQueryivNV(unsigned int param0,unsigned int param1,signed int* param2)
{
	void *f;
	f=(void*)getfunc("glGetOcclusionQueryivNV");
	ov_glGetOcclusionQueryivNV=f;
	ov_glGetOcclusionQueryivNV(param0,param1,param2);
}
void (APIENTRY *ov_glGetOcclusionQueryivNV)(unsigned int param0,unsigned int param1,signed int* param2)=init_ov_glGetOcclusionQueryivNV;

void __gl_ml_glGetOcclusionQueryivNV(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int* l2=(signed int*)(gl__ba__(v2));
	ov_glGetOcclusionQueryivNV(l0,l1,l2);
}
extern void (APIENTRY *ov_glEndOcclusionQueryNV)();

static void APIENTRY init_ov_glEndOcclusionQueryNV()
{
	void *f;
	f=(void*)getfunc("glEndOcclusionQueryNV");
	ov_glEndOcclusionQueryNV=f;
	ov_glEndOcclusionQueryNV();
}
void (APIENTRY *ov_glEndOcclusionQueryNV)()=init_ov_glEndOcclusionQueryNV;

void __gl_ml_glEndOcclusionQueryNV()
{
	ov_glEndOcclusionQueryNV();
}
extern void (APIENTRY *ov_glBeginOcclusionQueryNV)(unsigned int param0);

static void APIENTRY init_ov_glBeginOcclusionQueryNV(unsigned int param0)
{
	void *f;
	f=(void*)getfunc("glBeginOcclusionQueryNV");
	ov_glBeginOcclusionQueryNV=f;
	ov_glBeginOcclusionQueryNV(param0);
}
void (APIENTRY *ov_glBeginOcclusionQueryNV)(unsigned int param0)=init_ov_glBeginOcclusionQueryNV;

void __gl_ml_glBeginOcclusionQueryNV(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glBeginOcclusionQueryNV(l0);
}
extern unsigned char (APIENTRY *ov_glIsOcclusionQueryNV)(unsigned int param0);

static unsigned char APIENTRY init_ov_glIsOcclusionQueryNV(unsigned int param0)
{
	void *f;
	f=(void*)getfunc("glIsOcclusionQueryNV");
	ov_glIsOcclusionQueryNV=f;
	return ov_glIsOcclusionQueryNV(param0);
}
unsigned char (APIENTRY *ov_glIsOcclusionQueryNV)(unsigned int param0)=init_ov_glIsOcclusionQueryNV;

value __gl_ml_glIsOcclusionQueryNV(value v0)
{
	unsigned int l0=Int_val(v0);
	unsigned char r=ov_glIsOcclusionQueryNV(l0);
	return(Val_int(r));
}
extern void (APIENTRY *ov_glDeleteOcclusionQueriesNV)(signed int param0,unsigned int* param1);

static void APIENTRY init_ov_glDeleteOcclusionQueriesNV(signed int param0,unsigned int* param1)
{
	void *f;
	f=(void*)getfunc("glDeleteOcclusionQueriesNV");
	ov_glDeleteOcclusionQueriesNV=f;
	ov_glDeleteOcclusionQueriesNV(param0,param1);
}
void (APIENTRY *ov_glDeleteOcclusionQueriesNV)(signed int param0,unsigned int* param1)=init_ov_glDeleteOcclusionQueriesNV;

void __gl_ml_glDeleteOcclusionQueriesNV(value v0,value v1)
{
	signed int l0=Int_val(v0);
	unsigned int* l1=(unsigned int*)(gl__ba__(v1));
	ov_glDeleteOcclusionQueriesNV(l0,l1);
}
extern void (APIENTRY *ov_glGenOcclusionQueriesNV)(signed int param0,unsigned int* param1);

static void APIENTRY init_ov_glGenOcclusionQueriesNV(signed int param0,unsigned int* param1)
{
	void *f;
	f=(void*)getfunc("glGenOcclusionQueriesNV");
	ov_glGenOcclusionQueriesNV=f;
	ov_glGenOcclusionQueriesNV(param0,param1);
}
void (APIENTRY *ov_glGenOcclusionQueriesNV)(signed int param0,unsigned int* param1)=init_ov_glGenOcclusionQueriesNV;

void __gl_ml_glGenOcclusionQueriesNV(value v0,value v1)
{
	signed int l0=Int_val(v0);
	unsigned int* l1=(unsigned int*)(gl__ba__(v1));
	ov_glGenOcclusionQueriesNV(l0,l1);
}
extern void (APIENTRY *ov_glPointParameterivNV)(unsigned int param0,signed int* param1);

static void APIENTRY init_ov_glPointParameterivNV(unsigned int param0,signed int* param1)
{
	void *f;
	f=(void*)getfunc("glPointParameterivNV");
	ov_glPointParameterivNV=f;
	ov_glPointParameterivNV(param0,param1);
}
void (APIENTRY *ov_glPointParameterivNV)(unsigned int param0,signed int* param1)=init_ov_glPointParameterivNV;

void __gl_ml_glPointParameterivNV(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed int* l1=(signed int*)(gl__ba__(v1));
	ov_glPointParameterivNV(l0,l1);
}
extern void (APIENTRY *ov_glPointParameteriNV)(unsigned int param0,signed int param1);

static void APIENTRY init_ov_glPointParameteriNV(unsigned int param0,signed int param1)
{
	void *f;
	f=(void*)getfunc("glPointParameteriNV");
	ov_glPointParameteriNV=f;
	ov_glPointParameteriNV(param0,param1);
}
void (APIENTRY *ov_glPointParameteriNV)(unsigned int param0,signed int param1)=init_ov_glPointParameteriNV;

void __gl_ml_glPointParameteriNV(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	ov_glPointParameteriNV(l0,l1);
}
extern void (APIENTRY *ov_glGetFinalCombinerInputParameterivNV)(unsigned int param0,unsigned int param1,signed int* param2);

static void APIENTRY init_ov_glGetFinalCombinerInputParameterivNV(unsigned int param0,unsigned int param1,signed int* param2)
{
	void *f;
	f=(void*)getfunc("glGetFinalCombinerInputParameterivNV");
	ov_glGetFinalCombinerInputParameterivNV=f;
	ov_glGetFinalCombinerInputParameterivNV(param0,param1,param2);
}
void (APIENTRY *ov_glGetFinalCombinerInputParameterivNV)(unsigned int param0,unsigned int param1,signed int* param2)=init_ov_glGetFinalCombinerInputParameterivNV;

void __gl_ml_glGetFinalCombinerInputParameterivNV(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int* l2=(signed int*)(gl__ba__(v2));
	ov_glGetFinalCombinerInputParameterivNV(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetFinalCombinerInputParameterfvNV)(unsigned int param0,unsigned int param1,float* param2);

static void APIENTRY init_ov_glGetFinalCombinerInputParameterfvNV(unsigned int param0,unsigned int param1,float* param2)
{
	void *f;
	f=(void*)getfunc("glGetFinalCombinerInputParameterfvNV");
	ov_glGetFinalCombinerInputParameterfvNV=f;
	ov_glGetFinalCombinerInputParameterfvNV(param0,param1,param2);
}
void (APIENTRY *ov_glGetFinalCombinerInputParameterfvNV)(unsigned int param0,unsigned int param1,float* param2)=init_ov_glGetFinalCombinerInputParameterfvNV;

void __gl_ml_glGetFinalCombinerInputParameterfvNV(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glGetFinalCombinerInputParameterfvNV(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetCombinerOutputParameterivNV)(unsigned int param0,unsigned int param1,unsigned int param2,signed int* param3);

static void APIENTRY init_ov_glGetCombinerOutputParameterivNV(unsigned int param0,unsigned int param1,unsigned int param2,signed int* param3)
{
	void *f;
	f=(void*)getfunc("glGetCombinerOutputParameterivNV");
	ov_glGetCombinerOutputParameterivNV=f;
	ov_glGetCombinerOutputParameterivNV(param0,param1,param2,param3);
}
void (APIENTRY *ov_glGetCombinerOutputParameterivNV)(unsigned int param0,unsigned int param1,unsigned int param2,signed int* param3)=init_ov_glGetCombinerOutputParameterivNV;

void __gl_ml_glGetCombinerOutputParameterivNV(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	signed int* l3=(signed int*)(gl__ba__(v3));
	ov_glGetCombinerOutputParameterivNV(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glGetCombinerOutputParameterfvNV)(unsigned int param0,unsigned int param1,unsigned int param2,float* param3);

static void APIENTRY init_ov_glGetCombinerOutputParameterfvNV(unsigned int param0,unsigned int param1,unsigned int param2,float* param3)
{
	void *f;
	f=(void*)getfunc("glGetCombinerOutputParameterfvNV");
	ov_glGetCombinerOutputParameterfvNV=f;
	ov_glGetCombinerOutputParameterfvNV(param0,param1,param2,param3);
}
void (APIENTRY *ov_glGetCombinerOutputParameterfvNV)(unsigned int param0,unsigned int param1,unsigned int param2,float* param3)=init_ov_glGetCombinerOutputParameterfvNV;

void __gl_ml_glGetCombinerOutputParameterfvNV(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	float* l3=(float*)(gl__ba__(v3));
	ov_glGetCombinerOutputParameterfvNV(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glGetCombinerInputParameterivNV)(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3,signed int* param4);

static void APIENTRY init_ov_glGetCombinerInputParameterivNV(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3,signed int* param4)
{
	void *f;
	f=(void*)getfunc("glGetCombinerInputParameterivNV");
	ov_glGetCombinerInputParameterivNV=f;
	ov_glGetCombinerInputParameterivNV(param0,param1,param2,param3,param4);
}
void (APIENTRY *ov_glGetCombinerInputParameterivNV)(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3,signed int* param4)=init_ov_glGetCombinerInputParameterivNV;

void __gl_ml_glGetCombinerInputParameterivNV(value v0,value v1,value v2,value v3,value v4)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	unsigned int l3=Int_val(v3);
	signed int* l4=(signed int*)(gl__ba__(v4));
	ov_glGetCombinerInputParameterivNV(l0,l1,l2,l3,l4);
}
extern void (APIENTRY *ov_glGetCombinerInputParameterfvNV)(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3,float* param4);

static void APIENTRY init_ov_glGetCombinerInputParameterfvNV(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3,float* param4)
{
	void *f;
	f=(void*)getfunc("glGetCombinerInputParameterfvNV");
	ov_glGetCombinerInputParameterfvNV=f;
	ov_glGetCombinerInputParameterfvNV(param0,param1,param2,param3,param4);
}
void (APIENTRY *ov_glGetCombinerInputParameterfvNV)(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3,float* param4)=init_ov_glGetCombinerInputParameterfvNV;

void __gl_ml_glGetCombinerInputParameterfvNV(value v0,value v1,value v2,value v3,value v4)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	unsigned int l3=Int_val(v3);
	float* l4=(float*)(gl__ba__(v4));
	ov_glGetCombinerInputParameterfvNV(l0,l1,l2,l3,l4);
}
extern void (APIENTRY *ov_glFinalCombinerInputNV)(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3);

static void APIENTRY init_ov_glFinalCombinerInputNV(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3)
{
	void *f;
	f=(void*)getfunc("glFinalCombinerInputNV");
	ov_glFinalCombinerInputNV=f;
	ov_glFinalCombinerInputNV(param0,param1,param2,param3);
}
void (APIENTRY *ov_glFinalCombinerInputNV)(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3)=init_ov_glFinalCombinerInputNV;

void __gl_ml_glFinalCombinerInputNV(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	unsigned int l3=Int_val(v3);
	ov_glFinalCombinerInputNV(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glCombinerOutputNV)(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3,unsigned int param4,unsigned int param5,unsigned int param6,unsigned char param7,unsigned char param8,unsigned char param9);

static void APIENTRY init_ov_glCombinerOutputNV(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3,unsigned int param4,unsigned int param5,unsigned int param6,unsigned char param7,unsigned char param8,unsigned char param9)
{
	void *f;
	f=(void*)getfunc("glCombinerOutputNV");
	ov_glCombinerOutputNV=f;
	ov_glCombinerOutputNV(param0,param1,param2,param3,param4,param5,param6,param7,param8,param9);
}
void (APIENTRY *ov_glCombinerOutputNV)(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3,unsigned int param4,unsigned int param5,unsigned int param6,unsigned char param7,unsigned char param8,unsigned char param9)=init_ov_glCombinerOutputNV;

void __gl_ml_glCombinerOutputNV(value v0,value v1,value v2,value v3,value v4,value v5,value v6,value v7,value v8,value v9)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	unsigned int l3=Int_val(v3);
	unsigned int l4=Int_val(v4);
	unsigned int l5=Int_val(v5);
	unsigned int l6=Int_val(v6);
	unsigned char l7=Int_val(v7);
	unsigned char l8=Int_val(v8);
	unsigned char l9=Int_val(v9);
	ov_glCombinerOutputNV(l0,l1,l2,l3,l4,l5,l6,l7,l8,l9);
}

void byte___gl_ml_glCombinerOutputNV(value * argv,int argn)
{
	__gl_ml_glCombinerOutputNV(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6],argv[7],argv[8],argv[9]);
}
extern void (APIENTRY *ov_glCombinerInputNV)(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3,unsigned int param4,unsigned int param5);

static void APIENTRY init_ov_glCombinerInputNV(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3,unsigned int param4,unsigned int param5)
{
	void *f;
	f=(void*)getfunc("glCombinerInputNV");
	ov_glCombinerInputNV=f;
	ov_glCombinerInputNV(param0,param1,param2,param3,param4,param5);
}
void (APIENTRY *ov_glCombinerInputNV)(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3,unsigned int param4,unsigned int param5)=init_ov_glCombinerInputNV;

void __gl_ml_glCombinerInputNV(value v0,value v1,value v2,value v3,value v4,value v5)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	unsigned int l3=Int_val(v3);
	unsigned int l4=Int_val(v4);
	unsigned int l5=Int_val(v5);
	ov_glCombinerInputNV(l0,l1,l2,l3,l4,l5);
}

void byte___gl_ml_glCombinerInputNV(value * argv,int argn)
{
	__gl_ml_glCombinerInputNV(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5]);
}
extern void (APIENTRY *ov_glCombinerParameteriNV)(unsigned int param0,signed int param1);

static void APIENTRY init_ov_glCombinerParameteriNV(unsigned int param0,signed int param1)
{
	void *f;
	f=(void*)getfunc("glCombinerParameteriNV");
	ov_glCombinerParameteriNV=f;
	ov_glCombinerParameteriNV(param0,param1);
}
void (APIENTRY *ov_glCombinerParameteriNV)(unsigned int param0,signed int param1)=init_ov_glCombinerParameteriNV;

void __gl_ml_glCombinerParameteriNV(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	ov_glCombinerParameteriNV(l0,l1);
}
extern void (APIENTRY *ov_glCombinerParameterivNV)(unsigned int param0,signed int* param1);

static void APIENTRY init_ov_glCombinerParameterivNV(unsigned int param0,signed int* param1)
{
	void *f;
	f=(void*)getfunc("glCombinerParameterivNV");
	ov_glCombinerParameterivNV=f;
	ov_glCombinerParameterivNV(param0,param1);
}
void (APIENTRY *ov_glCombinerParameterivNV)(unsigned int param0,signed int* param1)=init_ov_glCombinerParameterivNV;

void __gl_ml_glCombinerParameterivNV(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed int* l1=(signed int*)(gl__ba__(v1));
	ov_glCombinerParameterivNV(l0,l1);
}
extern void (APIENTRY *ov_glCombinerParameterfNV)(unsigned int param0,float param1);

static void APIENTRY init_ov_glCombinerParameterfNV(unsigned int param0,float param1)
{
	void *f;
	f=(void*)getfunc("glCombinerParameterfNV");
	ov_glCombinerParameterfNV=f;
	ov_glCombinerParameterfNV(param0,param1);
}
void (APIENTRY *ov_glCombinerParameterfNV)(unsigned int param0,float param1)=init_ov_glCombinerParameterfNV;

void __gl_ml_glCombinerParameterfNV(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float l1=Double_val(v1);
	ov_glCombinerParameterfNV(l0,l1);
}
extern void (APIENTRY *ov_glCombinerParameterfvNV)(unsigned int param0,float* param1);

static void APIENTRY init_ov_glCombinerParameterfvNV(unsigned int param0,float* param1)
{
	void *f;
	f=(void*)getfunc("glCombinerParameterfvNV");
	ov_glCombinerParameterfvNV=f;
	ov_glCombinerParameterfvNV(param0,param1);
}
void (APIENTRY *ov_glCombinerParameterfvNV)(unsigned int param0,float* param1)=init_ov_glCombinerParameterfvNV;

void __gl_ml_glCombinerParameterfvNV(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float* l1=(float*)(gl__ba__(v1));
	ov_glCombinerParameterfvNV(l0,l1);
}
extern void (APIENTRY *ov_glGetCombinerStageParameterfvNV)(unsigned int param0,unsigned int param1,float* param2);

static void APIENTRY init_ov_glGetCombinerStageParameterfvNV(unsigned int param0,unsigned int param1,float* param2)
{
	void *f;
	f=(void*)getfunc("glGetCombinerStageParameterfvNV");
	ov_glGetCombinerStageParameterfvNV=f;
	ov_glGetCombinerStageParameterfvNV(param0,param1,param2);
}
void (APIENTRY *ov_glGetCombinerStageParameterfvNV)(unsigned int param0,unsigned int param1,float* param2)=init_ov_glGetCombinerStageParameterfvNV;

void __gl_ml_glGetCombinerStageParameterfvNV(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glGetCombinerStageParameterfvNV(l0,l1,l2);
}
extern void (APIENTRY *ov_glCombinerStageParameterfvNV)(unsigned int param0,unsigned int param1,float* param2);

static void APIENTRY init_ov_glCombinerStageParameterfvNV(unsigned int param0,unsigned int param1,float* param2)
{
	void *f;
	f=(void*)getfunc("glCombinerStageParameterfvNV");
	ov_glCombinerStageParameterfvNV=f;
	ov_glCombinerStageParameterfvNV(param0,param1,param2);
}
void (APIENTRY *ov_glCombinerStageParameterfvNV)(unsigned int param0,unsigned int param1,float* param2)=init_ov_glCombinerStageParameterfvNV;

void __gl_ml_glCombinerStageParameterfvNV(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glCombinerStageParameterfvNV(l0,l1,l2);
}
extern void (APIENTRY *ov_glVertexArrayRangeNV)(signed int param0,void* param1);

static void APIENTRY init_ov_glVertexArrayRangeNV(signed int param0,void* param1)
{
	void *f;
	f=(void*)getfunc("glVertexArrayRangeNV");
	ov_glVertexArrayRangeNV=f;
	ov_glVertexArrayRangeNV(param0,param1);
}
void (APIENTRY *ov_glVertexArrayRangeNV)(signed int param0,void* param1)=init_ov_glVertexArrayRangeNV;

void __gl_ml_glVertexArrayRangeNV(value v0,value v1)
{
	signed int l0=Int_val(v0);
	void* l1=gl__ba__(v1);
	ov_glVertexArrayRangeNV(l0,l1);
}
extern void (APIENTRY *ov_glFlushVertexArrayRangeNV)();

static void APIENTRY init_ov_glFlushVertexArrayRangeNV()
{
	void *f;
	f=(void*)getfunc("glFlushVertexArrayRangeNV");
	ov_glFlushVertexArrayRangeNV=f;
	ov_glFlushVertexArrayRangeNV();
}
void (APIENTRY *ov_glFlushVertexArrayRangeNV)()=init_ov_glFlushVertexArrayRangeNV;

void __gl_ml_glFlushVertexArrayRangeNV()
{
	ov_glFlushVertexArrayRangeNV();
}
extern void (APIENTRY *ov_glVertexAttribs4ubvNV)(unsigned int param0,signed int param1,unsigned char* param2);

static void APIENTRY init_ov_glVertexAttribs4ubvNV(unsigned int param0,signed int param1,unsigned char* param2)
{
	void *f;
	f=(void*)getfunc("glVertexAttribs4ubvNV");
	ov_glVertexAttribs4ubvNV=f;
	ov_glVertexAttribs4ubvNV(param0,param1,param2);
}
void (APIENTRY *ov_glVertexAttribs4ubvNV)(unsigned int param0,signed int param1,unsigned char* param2)=init_ov_glVertexAttribs4ubvNV;

void __gl_ml_glVertexAttribs4ubvNV(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	unsigned char* l2=(unsigned char*)(gl__ba__(v2));
	ov_glVertexAttribs4ubvNV(l0,l1,l2);
}
extern void (APIENTRY *ov_glVertexAttribs4svNV)(unsigned int param0,signed int param1,signed short* param2);

static void APIENTRY init_ov_glVertexAttribs4svNV(unsigned int param0,signed int param1,signed short* param2)
{
	void *f;
	f=(void*)getfunc("glVertexAttribs4svNV");
	ov_glVertexAttribs4svNV=f;
	ov_glVertexAttribs4svNV(param0,param1,param2);
}
void (APIENTRY *ov_glVertexAttribs4svNV)(unsigned int param0,signed int param1,signed short* param2)=init_ov_glVertexAttribs4svNV;

void __gl_ml_glVertexAttribs4svNV(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed short* l2=(signed short*)(gl__ba__(v2));
	ov_glVertexAttribs4svNV(l0,l1,l2);
}
extern void (APIENTRY *ov_glVertexAttribs4fvNV)(unsigned int param0,signed int param1,float* param2);

static void APIENTRY init_ov_glVertexAttribs4fvNV(unsigned int param0,signed int param1,float* param2)
{
	void *f;
	f=(void*)getfunc("glVertexAttribs4fvNV");
	ov_glVertexAttribs4fvNV=f;
	ov_glVertexAttribs4fvNV(param0,param1,param2);
}
void (APIENTRY *ov_glVertexAttribs4fvNV)(unsigned int param0,signed int param1,float* param2)=init_ov_glVertexAttribs4fvNV;

void __gl_ml_glVertexAttribs4fvNV(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glVertexAttribs4fvNV(l0,l1,l2);
}
extern void (APIENTRY *ov_glVertexAttribs4dvNV)(unsigned int param0,signed int param1,double* param2);

static void APIENTRY init_ov_glVertexAttribs4dvNV(unsigned int param0,signed int param1,double* param2)
{
	void *f;
	f=(void*)getfunc("glVertexAttribs4dvNV");
	ov_glVertexAttribs4dvNV=f;
	ov_glVertexAttribs4dvNV(param0,param1,param2);
}
void (APIENTRY *ov_glVertexAttribs4dvNV)(unsigned int param0,signed int param1,double* param2)=init_ov_glVertexAttribs4dvNV;

void __gl_ml_glVertexAttribs4dvNV(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	double* l2=(double*)(gl__ba__(v2));
	ov_glVertexAttribs4dvNV(l0,l1,l2);
}
extern void (APIENTRY *ov_glVertexAttribs3svNV)(unsigned int param0,signed int param1,signed short* param2);

static void APIENTRY init_ov_glVertexAttribs3svNV(unsigned int param0,signed int param1,signed short* param2)
{
	void *f;
	f=(void*)getfunc("glVertexAttribs3svNV");
	ov_glVertexAttribs3svNV=f;
	ov_glVertexAttribs3svNV(param0,param1,param2);
}
void (APIENTRY *ov_glVertexAttribs3svNV)(unsigned int param0,signed int param1,signed short* param2)=init_ov_glVertexAttribs3svNV;

void __gl_ml_glVertexAttribs3svNV(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed short* l2=(signed short*)(gl__ba__(v2));
	ov_glVertexAttribs3svNV(l0,l1,l2);
}
extern void (APIENTRY *ov_glVertexAttribs3fvNV)(unsigned int param0,signed int param1,float* param2);

static void APIENTRY init_ov_glVertexAttribs3fvNV(unsigned int param0,signed int param1,float* param2)
{
	void *f;
	f=(void*)getfunc("glVertexAttribs3fvNV");
	ov_glVertexAttribs3fvNV=f;
	ov_glVertexAttribs3fvNV(param0,param1,param2);
}
void (APIENTRY *ov_glVertexAttribs3fvNV)(unsigned int param0,signed int param1,float* param2)=init_ov_glVertexAttribs3fvNV;

void __gl_ml_glVertexAttribs3fvNV(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glVertexAttribs3fvNV(l0,l1,l2);
}
extern void (APIENTRY *ov_glVertexAttribs3dvNV)(unsigned int param0,signed int param1,double* param2);

static void APIENTRY init_ov_glVertexAttribs3dvNV(unsigned int param0,signed int param1,double* param2)
{
	void *f;
	f=(void*)getfunc("glVertexAttribs3dvNV");
	ov_glVertexAttribs3dvNV=f;
	ov_glVertexAttribs3dvNV(param0,param1,param2);
}
void (APIENTRY *ov_glVertexAttribs3dvNV)(unsigned int param0,signed int param1,double* param2)=init_ov_glVertexAttribs3dvNV;

void __gl_ml_glVertexAttribs3dvNV(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	double* l2=(double*)(gl__ba__(v2));
	ov_glVertexAttribs3dvNV(l0,l1,l2);
}
extern void (APIENTRY *ov_glVertexAttribs2svNV)(unsigned int param0,signed int param1,signed short* param2);

static void APIENTRY init_ov_glVertexAttribs2svNV(unsigned int param0,signed int param1,signed short* param2)
{
	void *f;
	f=(void*)getfunc("glVertexAttribs2svNV");
	ov_glVertexAttribs2svNV=f;
	ov_glVertexAttribs2svNV(param0,param1,param2);
}
void (APIENTRY *ov_glVertexAttribs2svNV)(unsigned int param0,signed int param1,signed short* param2)=init_ov_glVertexAttribs2svNV;

void __gl_ml_glVertexAttribs2svNV(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed short* l2=(signed short*)(gl__ba__(v2));
	ov_glVertexAttribs2svNV(l0,l1,l2);
}
extern void (APIENTRY *ov_glVertexAttribs2fvNV)(unsigned int param0,signed int param1,float* param2);

static void APIENTRY init_ov_glVertexAttribs2fvNV(unsigned int param0,signed int param1,float* param2)
{
	void *f;
	f=(void*)getfunc("glVertexAttribs2fvNV");
	ov_glVertexAttribs2fvNV=f;
	ov_glVertexAttribs2fvNV(param0,param1,param2);
}
void (APIENTRY *ov_glVertexAttribs2fvNV)(unsigned int param0,signed int param1,float* param2)=init_ov_glVertexAttribs2fvNV;

void __gl_ml_glVertexAttribs2fvNV(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glVertexAttribs2fvNV(l0,l1,l2);
}
extern void (APIENTRY *ov_glVertexAttribs2dvNV)(unsigned int param0,signed int param1,double* param2);

static void APIENTRY init_ov_glVertexAttribs2dvNV(unsigned int param0,signed int param1,double* param2)
{
	void *f;
	f=(void*)getfunc("glVertexAttribs2dvNV");
	ov_glVertexAttribs2dvNV=f;
	ov_glVertexAttribs2dvNV(param0,param1,param2);
}
void (APIENTRY *ov_glVertexAttribs2dvNV)(unsigned int param0,signed int param1,double* param2)=init_ov_glVertexAttribs2dvNV;

void __gl_ml_glVertexAttribs2dvNV(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	double* l2=(double*)(gl__ba__(v2));
	ov_glVertexAttribs2dvNV(l0,l1,l2);
}
extern void (APIENTRY *ov_glVertexAttribs1svNV)(unsigned int param0,signed int param1,signed short* param2);

static void APIENTRY init_ov_glVertexAttribs1svNV(unsigned int param0,signed int param1,signed short* param2)
{
	void *f;
	f=(void*)getfunc("glVertexAttribs1svNV");
	ov_glVertexAttribs1svNV=f;
	ov_glVertexAttribs1svNV(param0,param1,param2);
}
void (APIENTRY *ov_glVertexAttribs1svNV)(unsigned int param0,signed int param1,signed short* param2)=init_ov_glVertexAttribs1svNV;

void __gl_ml_glVertexAttribs1svNV(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed short* l2=(signed short*)(gl__ba__(v2));
	ov_glVertexAttribs1svNV(l0,l1,l2);
}
extern void (APIENTRY *ov_glVertexAttribs1fvNV)(unsigned int param0,signed int param1,float* param2);

static void APIENTRY init_ov_glVertexAttribs1fvNV(unsigned int param0,signed int param1,float* param2)
{
	void *f;
	f=(void*)getfunc("glVertexAttribs1fvNV");
	ov_glVertexAttribs1fvNV=f;
	ov_glVertexAttribs1fvNV(param0,param1,param2);
}
void (APIENTRY *ov_glVertexAttribs1fvNV)(unsigned int param0,signed int param1,float* param2)=init_ov_glVertexAttribs1fvNV;

void __gl_ml_glVertexAttribs1fvNV(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glVertexAttribs1fvNV(l0,l1,l2);
}
extern void (APIENTRY *ov_glVertexAttribs1dvNV)(unsigned int param0,signed int param1,double* param2);

static void APIENTRY init_ov_glVertexAttribs1dvNV(unsigned int param0,signed int param1,double* param2)
{
	void *f;
	f=(void*)getfunc("glVertexAttribs1dvNV");
	ov_glVertexAttribs1dvNV=f;
	ov_glVertexAttribs1dvNV(param0,param1,param2);
}
void (APIENTRY *ov_glVertexAttribs1dvNV)(unsigned int param0,signed int param1,double* param2)=init_ov_glVertexAttribs1dvNV;

void __gl_ml_glVertexAttribs1dvNV(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	double* l2=(double*)(gl__ba__(v2));
	ov_glVertexAttribs1dvNV(l0,l1,l2);
}
extern void (APIENTRY *ov_glVertexAttrib4ubvNV)(unsigned int param0,unsigned char* param1);

static void APIENTRY init_ov_glVertexAttrib4ubvNV(unsigned int param0,unsigned char* param1)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib4ubvNV");
	ov_glVertexAttrib4ubvNV=f;
	ov_glVertexAttrib4ubvNV(param0,param1);
}
void (APIENTRY *ov_glVertexAttrib4ubvNV)(unsigned int param0,unsigned char* param1)=init_ov_glVertexAttrib4ubvNV;

void __gl_ml_glVertexAttrib4ubvNV(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	unsigned char* l1=(unsigned char*)(gl__ba__(v1));
	ov_glVertexAttrib4ubvNV(l0,l1);
}
extern void (APIENTRY *ov_glVertexAttrib4ubNV)(unsigned int param0,unsigned char param1,unsigned char param2,unsigned char param3,unsigned char param4);

static void APIENTRY init_ov_glVertexAttrib4ubNV(unsigned int param0,unsigned char param1,unsigned char param2,unsigned char param3,unsigned char param4)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib4ubNV");
	ov_glVertexAttrib4ubNV=f;
	ov_glVertexAttrib4ubNV(param0,param1,param2,param3,param4);
}
void (APIENTRY *ov_glVertexAttrib4ubNV)(unsigned int param0,unsigned char param1,unsigned char param2,unsigned char param3,unsigned char param4)=init_ov_glVertexAttrib4ubNV;

void __gl_ml_glVertexAttrib4ubNV(value v0,value v1,value v2,value v3,value v4)
{
	unsigned int l0=Int_val(v0);
	unsigned char l1=Int_val(v1);
	unsigned char l2=Int_val(v2);
	unsigned char l3=Int_val(v3);
	unsigned char l4=Int_val(v4);
	ov_glVertexAttrib4ubNV(l0,l1,l2,l3,l4);
}
extern void (APIENTRY *ov_glVertexAttrib4svNV)(unsigned int param0,signed short* param1);

static void APIENTRY init_ov_glVertexAttrib4svNV(unsigned int param0,signed short* param1)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib4svNV");
	ov_glVertexAttrib4svNV=f;
	ov_glVertexAttrib4svNV(param0,param1);
}
void (APIENTRY *ov_glVertexAttrib4svNV)(unsigned int param0,signed short* param1)=init_ov_glVertexAttrib4svNV;

void __gl_ml_glVertexAttrib4svNV(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed short* l1=(signed short*)(gl__ba__(v1));
	ov_glVertexAttrib4svNV(l0,l1);
}
extern void (APIENTRY *ov_glVertexAttrib4sNV)(unsigned int param0,signed short param1,signed short param2,signed short param3,signed short param4);

static void APIENTRY init_ov_glVertexAttrib4sNV(unsigned int param0,signed short param1,signed short param2,signed short param3,signed short param4)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib4sNV");
	ov_glVertexAttrib4sNV=f;
	ov_glVertexAttrib4sNV(param0,param1,param2,param3,param4);
}
void (APIENTRY *ov_glVertexAttrib4sNV)(unsigned int param0,signed short param1,signed short param2,signed short param3,signed short param4)=init_ov_glVertexAttrib4sNV;

void __gl_ml_glVertexAttrib4sNV(value v0,value v1,value v2,value v3,value v4)
{
	unsigned int l0=Int_val(v0);
	signed short l1=Int_val(v1);
	signed short l2=Int_val(v2);
	signed short l3=Int_val(v3);
	signed short l4=Int_val(v4);
	ov_glVertexAttrib4sNV(l0,l1,l2,l3,l4);
}
extern void (APIENTRY *ov_glVertexAttrib4fvNV)(unsigned int param0,float* param1);

static void APIENTRY init_ov_glVertexAttrib4fvNV(unsigned int param0,float* param1)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib4fvNV");
	ov_glVertexAttrib4fvNV=f;
	ov_glVertexAttrib4fvNV(param0,param1);
}
void (APIENTRY *ov_glVertexAttrib4fvNV)(unsigned int param0,float* param1)=init_ov_glVertexAttrib4fvNV;

void __gl_ml_glVertexAttrib4fvNV(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float* l1=(float*)(gl__ba__(v1));
	ov_glVertexAttrib4fvNV(l0,l1);
}
extern void (APIENTRY *ov_glVertexAttrib4fNV)(unsigned int param0,float param1,float param2,float param3,float param4);

static void APIENTRY init_ov_glVertexAttrib4fNV(unsigned int param0,float param1,float param2,float param3,float param4)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib4fNV");
	ov_glVertexAttrib4fNV=f;
	ov_glVertexAttrib4fNV(param0,param1,param2,param3,param4);
}
void (APIENTRY *ov_glVertexAttrib4fNV)(unsigned int param0,float param1,float param2,float param3,float param4)=init_ov_glVertexAttrib4fNV;

void __gl_ml_glVertexAttrib4fNV(value v0,value v1,value v2,value v3,value v4)
{
	unsigned int l0=Int_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	float l3=Double_val(v3);
	float l4=Double_val(v4);
	ov_glVertexAttrib4fNV(l0,l1,l2,l3,l4);
}
extern void (APIENTRY *ov_glVertexAttrib4dvNV)(unsigned int param0,double* param1);

static void APIENTRY init_ov_glVertexAttrib4dvNV(unsigned int param0,double* param1)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib4dvNV");
	ov_glVertexAttrib4dvNV=f;
	ov_glVertexAttrib4dvNV(param0,param1);
}
void (APIENTRY *ov_glVertexAttrib4dvNV)(unsigned int param0,double* param1)=init_ov_glVertexAttrib4dvNV;

void __gl_ml_glVertexAttrib4dvNV(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	double* l1=(double*)(gl__ba__(v1));
	ov_glVertexAttrib4dvNV(l0,l1);
}
extern void (APIENTRY *ov_glVertexAttrib4dNV)(unsigned int param0,double param1,double param2,double param3,double param4);

static void APIENTRY init_ov_glVertexAttrib4dNV(unsigned int param0,double param1,double param2,double param3,double param4)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib4dNV");
	ov_glVertexAttrib4dNV=f;
	ov_glVertexAttrib4dNV(param0,param1,param2,param3,param4);
}
void (APIENTRY *ov_glVertexAttrib4dNV)(unsigned int param0,double param1,double param2,double param3,double param4)=init_ov_glVertexAttrib4dNV;

void __gl_ml_glVertexAttrib4dNV(value v0,value v1,value v2,value v3,value v4)
{
	unsigned int l0=Int_val(v0);
	double l1=Double_val(v1);
	double l2=Double_val(v2);
	double l3=Double_val(v3);
	double l4=Double_val(v4);
	ov_glVertexAttrib4dNV(l0,l1,l2,l3,l4);
}
extern void (APIENTRY *ov_glVertexAttrib3svNV)(unsigned int param0,signed short* param1);

static void APIENTRY init_ov_glVertexAttrib3svNV(unsigned int param0,signed short* param1)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib3svNV");
	ov_glVertexAttrib3svNV=f;
	ov_glVertexAttrib3svNV(param0,param1);
}
void (APIENTRY *ov_glVertexAttrib3svNV)(unsigned int param0,signed short* param1)=init_ov_glVertexAttrib3svNV;

void __gl_ml_glVertexAttrib3svNV(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed short* l1=(signed short*)(gl__ba__(v1));
	ov_glVertexAttrib3svNV(l0,l1);
}
extern void (APIENTRY *ov_glVertexAttrib3sNV)(unsigned int param0,signed short param1,signed short param2,signed short param3);

static void APIENTRY init_ov_glVertexAttrib3sNV(unsigned int param0,signed short param1,signed short param2,signed short param3)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib3sNV");
	ov_glVertexAttrib3sNV=f;
	ov_glVertexAttrib3sNV(param0,param1,param2,param3);
}
void (APIENTRY *ov_glVertexAttrib3sNV)(unsigned int param0,signed short param1,signed short param2,signed short param3)=init_ov_glVertexAttrib3sNV;

void __gl_ml_glVertexAttrib3sNV(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	signed short l1=Int_val(v1);
	signed short l2=Int_val(v2);
	signed short l3=Int_val(v3);
	ov_glVertexAttrib3sNV(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glVertexAttrib3fvNV)(unsigned int param0,float* param1);

static void APIENTRY init_ov_glVertexAttrib3fvNV(unsigned int param0,float* param1)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib3fvNV");
	ov_glVertexAttrib3fvNV=f;
	ov_glVertexAttrib3fvNV(param0,param1);
}
void (APIENTRY *ov_glVertexAttrib3fvNV)(unsigned int param0,float* param1)=init_ov_glVertexAttrib3fvNV;

void __gl_ml_glVertexAttrib3fvNV(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float* l1=(float*)(gl__ba__(v1));
	ov_glVertexAttrib3fvNV(l0,l1);
}
extern void (APIENTRY *ov_glVertexAttrib3fNV)(unsigned int param0,float param1,float param2,float param3);

static void APIENTRY init_ov_glVertexAttrib3fNV(unsigned int param0,float param1,float param2,float param3)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib3fNV");
	ov_glVertexAttrib3fNV=f;
	ov_glVertexAttrib3fNV(param0,param1,param2,param3);
}
void (APIENTRY *ov_glVertexAttrib3fNV)(unsigned int param0,float param1,float param2,float param3)=init_ov_glVertexAttrib3fNV;

void __gl_ml_glVertexAttrib3fNV(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	float l3=Double_val(v3);
	ov_glVertexAttrib3fNV(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glVertexAttrib3dvNV)(unsigned int param0,double* param1);

static void APIENTRY init_ov_glVertexAttrib3dvNV(unsigned int param0,double* param1)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib3dvNV");
	ov_glVertexAttrib3dvNV=f;
	ov_glVertexAttrib3dvNV(param0,param1);
}
void (APIENTRY *ov_glVertexAttrib3dvNV)(unsigned int param0,double* param1)=init_ov_glVertexAttrib3dvNV;

void __gl_ml_glVertexAttrib3dvNV(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	double* l1=(double*)(gl__ba__(v1));
	ov_glVertexAttrib3dvNV(l0,l1);
}
extern void (APIENTRY *ov_glVertexAttrib3dNV)(unsigned int param0,double param1,double param2,double param3);

static void APIENTRY init_ov_glVertexAttrib3dNV(unsigned int param0,double param1,double param2,double param3)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib3dNV");
	ov_glVertexAttrib3dNV=f;
	ov_glVertexAttrib3dNV(param0,param1,param2,param3);
}
void (APIENTRY *ov_glVertexAttrib3dNV)(unsigned int param0,double param1,double param2,double param3)=init_ov_glVertexAttrib3dNV;

void __gl_ml_glVertexAttrib3dNV(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	double l1=Double_val(v1);
	double l2=Double_val(v2);
	double l3=Double_val(v3);
	ov_glVertexAttrib3dNV(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glVertexAttrib2svNV)(unsigned int param0,signed short* param1);

static void APIENTRY init_ov_glVertexAttrib2svNV(unsigned int param0,signed short* param1)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib2svNV");
	ov_glVertexAttrib2svNV=f;
	ov_glVertexAttrib2svNV(param0,param1);
}
void (APIENTRY *ov_glVertexAttrib2svNV)(unsigned int param0,signed short* param1)=init_ov_glVertexAttrib2svNV;

void __gl_ml_glVertexAttrib2svNV(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed short* l1=(signed short*)(gl__ba__(v1));
	ov_glVertexAttrib2svNV(l0,l1);
}
extern void (APIENTRY *ov_glVertexAttrib2sNV)(unsigned int param0,signed short param1,signed short param2);

static void APIENTRY init_ov_glVertexAttrib2sNV(unsigned int param0,signed short param1,signed short param2)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib2sNV");
	ov_glVertexAttrib2sNV=f;
	ov_glVertexAttrib2sNV(param0,param1,param2);
}
void (APIENTRY *ov_glVertexAttrib2sNV)(unsigned int param0,signed short param1,signed short param2)=init_ov_glVertexAttrib2sNV;

void __gl_ml_glVertexAttrib2sNV(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	signed short l1=Int_val(v1);
	signed short l2=Int_val(v2);
	ov_glVertexAttrib2sNV(l0,l1,l2);
}
extern void (APIENTRY *ov_glVertexAttrib2fvNV)(unsigned int param0,float* param1);

static void APIENTRY init_ov_glVertexAttrib2fvNV(unsigned int param0,float* param1)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib2fvNV");
	ov_glVertexAttrib2fvNV=f;
	ov_glVertexAttrib2fvNV(param0,param1);
}
void (APIENTRY *ov_glVertexAttrib2fvNV)(unsigned int param0,float* param1)=init_ov_glVertexAttrib2fvNV;

void __gl_ml_glVertexAttrib2fvNV(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float* l1=(float*)(gl__ba__(v1));
	ov_glVertexAttrib2fvNV(l0,l1);
}
extern void (APIENTRY *ov_glVertexAttrib2fNV)(unsigned int param0,float param1,float param2);

static void APIENTRY init_ov_glVertexAttrib2fNV(unsigned int param0,float param1,float param2)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib2fNV");
	ov_glVertexAttrib2fNV=f;
	ov_glVertexAttrib2fNV(param0,param1,param2);
}
void (APIENTRY *ov_glVertexAttrib2fNV)(unsigned int param0,float param1,float param2)=init_ov_glVertexAttrib2fNV;

void __gl_ml_glVertexAttrib2fNV(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	ov_glVertexAttrib2fNV(l0,l1,l2);
}
extern void (APIENTRY *ov_glVertexAttrib2dvNV)(unsigned int param0,double* param1);

static void APIENTRY init_ov_glVertexAttrib2dvNV(unsigned int param0,double* param1)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib2dvNV");
	ov_glVertexAttrib2dvNV=f;
	ov_glVertexAttrib2dvNV(param0,param1);
}
void (APIENTRY *ov_glVertexAttrib2dvNV)(unsigned int param0,double* param1)=init_ov_glVertexAttrib2dvNV;

void __gl_ml_glVertexAttrib2dvNV(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	double* l1=(double*)(gl__ba__(v1));
	ov_glVertexAttrib2dvNV(l0,l1);
}
extern void (APIENTRY *ov_glVertexAttrib2dNV)(unsigned int param0,double param1,double param2);

static void APIENTRY init_ov_glVertexAttrib2dNV(unsigned int param0,double param1,double param2)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib2dNV");
	ov_glVertexAttrib2dNV=f;
	ov_glVertexAttrib2dNV(param0,param1,param2);
}
void (APIENTRY *ov_glVertexAttrib2dNV)(unsigned int param0,double param1,double param2)=init_ov_glVertexAttrib2dNV;

void __gl_ml_glVertexAttrib2dNV(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	double l1=Double_val(v1);
	double l2=Double_val(v2);
	ov_glVertexAttrib2dNV(l0,l1,l2);
}
extern void (APIENTRY *ov_glVertexAttrib1svNV)(unsigned int param0,signed short* param1);

static void APIENTRY init_ov_glVertexAttrib1svNV(unsigned int param0,signed short* param1)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib1svNV");
	ov_glVertexAttrib1svNV=f;
	ov_glVertexAttrib1svNV(param0,param1);
}
void (APIENTRY *ov_glVertexAttrib1svNV)(unsigned int param0,signed short* param1)=init_ov_glVertexAttrib1svNV;

void __gl_ml_glVertexAttrib1svNV(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed short* l1=(signed short*)(gl__ba__(v1));
	ov_glVertexAttrib1svNV(l0,l1);
}
extern void (APIENTRY *ov_glVertexAttrib1sNV)(unsigned int param0,signed short param1);

static void APIENTRY init_ov_glVertexAttrib1sNV(unsigned int param0,signed short param1)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib1sNV");
	ov_glVertexAttrib1sNV=f;
	ov_glVertexAttrib1sNV(param0,param1);
}
void (APIENTRY *ov_glVertexAttrib1sNV)(unsigned int param0,signed short param1)=init_ov_glVertexAttrib1sNV;

void __gl_ml_glVertexAttrib1sNV(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed short l1=Int_val(v1);
	ov_glVertexAttrib1sNV(l0,l1);
}
extern void (APIENTRY *ov_glVertexAttrib1fvNV)(unsigned int param0,float* param1);

static void APIENTRY init_ov_glVertexAttrib1fvNV(unsigned int param0,float* param1)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib1fvNV");
	ov_glVertexAttrib1fvNV=f;
	ov_glVertexAttrib1fvNV(param0,param1);
}
void (APIENTRY *ov_glVertexAttrib1fvNV)(unsigned int param0,float* param1)=init_ov_glVertexAttrib1fvNV;

void __gl_ml_glVertexAttrib1fvNV(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float* l1=(float*)(gl__ba__(v1));
	ov_glVertexAttrib1fvNV(l0,l1);
}
extern void (APIENTRY *ov_glVertexAttrib1fNV)(unsigned int param0,float param1);

static void APIENTRY init_ov_glVertexAttrib1fNV(unsigned int param0,float param1)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib1fNV");
	ov_glVertexAttrib1fNV=f;
	ov_glVertexAttrib1fNV(param0,param1);
}
void (APIENTRY *ov_glVertexAttrib1fNV)(unsigned int param0,float param1)=init_ov_glVertexAttrib1fNV;

void __gl_ml_glVertexAttrib1fNV(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float l1=Double_val(v1);
	ov_glVertexAttrib1fNV(l0,l1);
}
extern void (APIENTRY *ov_glVertexAttrib1dvNV)(unsigned int param0,double* param1);

static void APIENTRY init_ov_glVertexAttrib1dvNV(unsigned int param0,double* param1)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib1dvNV");
	ov_glVertexAttrib1dvNV=f;
	ov_glVertexAttrib1dvNV(param0,param1);
}
void (APIENTRY *ov_glVertexAttrib1dvNV)(unsigned int param0,double* param1)=init_ov_glVertexAttrib1dvNV;

void __gl_ml_glVertexAttrib1dvNV(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	double* l1=(double*)(gl__ba__(v1));
	ov_glVertexAttrib1dvNV(l0,l1);
}
extern void (APIENTRY *ov_glVertexAttrib1dNV)(unsigned int param0,double param1);

static void APIENTRY init_ov_glVertexAttrib1dNV(unsigned int param0,double param1)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib1dNV");
	ov_glVertexAttrib1dNV=f;
	ov_glVertexAttrib1dNV(param0,param1);
}
void (APIENTRY *ov_glVertexAttrib1dNV)(unsigned int param0,double param1)=init_ov_glVertexAttrib1dNV;

void __gl_ml_glVertexAttrib1dNV(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	double l1=Double_val(v1);
	ov_glVertexAttrib1dNV(l0,l1);
}
extern void (APIENTRY *ov_glVertexAttribPointerNV)(unsigned int param0,signed int param1,unsigned int param2,signed int param3,void* param4);

static void APIENTRY init_ov_glVertexAttribPointerNV(unsigned int param0,signed int param1,unsigned int param2,signed int param3,void* param4)
{
	void *f;
	f=(void*)getfunc("glVertexAttribPointerNV");
	ov_glVertexAttribPointerNV=f;
	ov_glVertexAttribPointerNV(param0,param1,param2,param3,param4);
}
void (APIENTRY *ov_glVertexAttribPointerNV)(unsigned int param0,signed int param1,unsigned int param2,signed int param3,void* param4)=init_ov_glVertexAttribPointerNV;

void __gl_ml_glVertexAttribPointerNV(value v0,value v1,value v2,value v3,value v4)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	void* l4=gl__ba__(v4);
	ov_glVertexAttribPointerNV(l0,l1,l2,l3,l4);
}
extern void (APIENTRY *ov_glTrackMatrixNV)(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3);

static void APIENTRY init_ov_glTrackMatrixNV(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3)
{
	void *f;
	f=(void*)getfunc("glTrackMatrixNV");
	ov_glTrackMatrixNV=f;
	ov_glTrackMatrixNV(param0,param1,param2,param3);
}
void (APIENTRY *ov_glTrackMatrixNV)(unsigned int param0,unsigned int param1,unsigned int param2,unsigned int param3)=init_ov_glTrackMatrixNV;

void __gl_ml_glTrackMatrixNV(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	unsigned int l3=Int_val(v3);
	ov_glTrackMatrixNV(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glRequestResidentProgramsNV)(signed int param0,unsigned int* param1);

static void APIENTRY init_ov_glRequestResidentProgramsNV(signed int param0,unsigned int* param1)
{
	void *f;
	f=(void*)getfunc("glRequestResidentProgramsNV");
	ov_glRequestResidentProgramsNV=f;
	ov_glRequestResidentProgramsNV(param0,param1);
}
void (APIENTRY *ov_glRequestResidentProgramsNV)(signed int param0,unsigned int* param1)=init_ov_glRequestResidentProgramsNV;

void __gl_ml_glRequestResidentProgramsNV(value v0,value v1)
{
	signed int l0=Int_val(v0);
	unsigned int* l1=(unsigned int*)(gl__ba__(v1));
	ov_glRequestResidentProgramsNV(l0,l1);
}
extern void (APIENTRY *ov_glProgramParameters4fvNV)(unsigned int param0,unsigned int param1,unsigned int param2,float* param3);

static void APIENTRY init_ov_glProgramParameters4fvNV(unsigned int param0,unsigned int param1,unsigned int param2,float* param3)
{
	void *f;
	f=(void*)getfunc("glProgramParameters4fvNV");
	ov_glProgramParameters4fvNV=f;
	ov_glProgramParameters4fvNV(param0,param1,param2,param3);
}
void (APIENTRY *ov_glProgramParameters4fvNV)(unsigned int param0,unsigned int param1,unsigned int param2,float* param3)=init_ov_glProgramParameters4fvNV;

void __gl_ml_glProgramParameters4fvNV(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	float* l3=(float*)(gl__ba__(v3));
	ov_glProgramParameters4fvNV(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glProgramParameters4dvNV)(unsigned int param0,unsigned int param1,unsigned int param2,double* param3);

static void APIENTRY init_ov_glProgramParameters4dvNV(unsigned int param0,unsigned int param1,unsigned int param2,double* param3)
{
	void *f;
	f=(void*)getfunc("glProgramParameters4dvNV");
	ov_glProgramParameters4dvNV=f;
	ov_glProgramParameters4dvNV(param0,param1,param2,param3);
}
void (APIENTRY *ov_glProgramParameters4dvNV)(unsigned int param0,unsigned int param1,unsigned int param2,double* param3)=init_ov_glProgramParameters4dvNV;

void __gl_ml_glProgramParameters4dvNV(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	double* l3=(double*)(gl__ba__(v3));
	ov_glProgramParameters4dvNV(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glProgramParameter4fvNV)(unsigned int param0,unsigned int param1,float* param2);

static void APIENTRY init_ov_glProgramParameter4fvNV(unsigned int param0,unsigned int param1,float* param2)
{
	void *f;
	f=(void*)getfunc("glProgramParameter4fvNV");
	ov_glProgramParameter4fvNV=f;
	ov_glProgramParameter4fvNV(param0,param1,param2);
}
void (APIENTRY *ov_glProgramParameter4fvNV)(unsigned int param0,unsigned int param1,float* param2)=init_ov_glProgramParameter4fvNV;

void __gl_ml_glProgramParameter4fvNV(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glProgramParameter4fvNV(l0,l1,l2);
}
extern void (APIENTRY *ov_glProgramParameter4fNV)(unsigned int param0,unsigned int param1,float param2,float param3,float param4,float param5);

static void APIENTRY init_ov_glProgramParameter4fNV(unsigned int param0,unsigned int param1,float param2,float param3,float param4,float param5)
{
	void *f;
	f=(void*)getfunc("glProgramParameter4fNV");
	ov_glProgramParameter4fNV=f;
	ov_glProgramParameter4fNV(param0,param1,param2,param3,param4,param5);
}
void (APIENTRY *ov_glProgramParameter4fNV)(unsigned int param0,unsigned int param1,float param2,float param3,float param4,float param5)=init_ov_glProgramParameter4fNV;

void __gl_ml_glProgramParameter4fNV(value v0,value v1,value v2,value v3,value v4,value v5)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float l2=Double_val(v2);
	float l3=Double_val(v3);
	float l4=Double_val(v4);
	float l5=Double_val(v5);
	ov_glProgramParameter4fNV(l0,l1,l2,l3,l4,l5);
}

void byte___gl_ml_glProgramParameter4fNV(value * argv,int argn)
{
	__gl_ml_glProgramParameter4fNV(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5]);
}
extern void (APIENTRY *ov_glProgramParameter4dvNV)(unsigned int param0,unsigned int param1,double* param2);

static void APIENTRY init_ov_glProgramParameter4dvNV(unsigned int param0,unsigned int param1,double* param2)
{
	void *f;
	f=(void*)getfunc("glProgramParameter4dvNV");
	ov_glProgramParameter4dvNV=f;
	ov_glProgramParameter4dvNV(param0,param1,param2);
}
void (APIENTRY *ov_glProgramParameter4dvNV)(unsigned int param0,unsigned int param1,double* param2)=init_ov_glProgramParameter4dvNV;

void __gl_ml_glProgramParameter4dvNV(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	double* l2=(double*)(gl__ba__(v2));
	ov_glProgramParameter4dvNV(l0,l1,l2);
}
extern void (APIENTRY *ov_glProgramParameter4dNV)(unsigned int param0,unsigned int param1,double param2,double param3,double param4,double param5);

static void APIENTRY init_ov_glProgramParameter4dNV(unsigned int param0,unsigned int param1,double param2,double param3,double param4,double param5)
{
	void *f;
	f=(void*)getfunc("glProgramParameter4dNV");
	ov_glProgramParameter4dNV=f;
	ov_glProgramParameter4dNV(param0,param1,param2,param3,param4,param5);
}
void (APIENTRY *ov_glProgramParameter4dNV)(unsigned int param0,unsigned int param1,double param2,double param3,double param4,double param5)=init_ov_glProgramParameter4dNV;

void __gl_ml_glProgramParameter4dNV(value v0,value v1,value v2,value v3,value v4,value v5)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	double l2=Double_val(v2);
	double l3=Double_val(v3);
	double l4=Double_val(v4);
	double l5=Double_val(v5);
	ov_glProgramParameter4dNV(l0,l1,l2,l3,l4,l5);
}

void byte___gl_ml_glProgramParameter4dNV(value * argv,int argn)
{
	__gl_ml_glProgramParameter4dNV(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5]);
}
extern void (APIENTRY *ov_glLoadProgramNV)(unsigned int param0,unsigned int param1,signed int param2,char* param3);

static void APIENTRY init_ov_glLoadProgramNV(unsigned int param0,unsigned int param1,signed int param2,char* param3)
{
	void *f;
	f=(void*)getfunc("glLoadProgramNV");
	ov_glLoadProgramNV=f;
	ov_glLoadProgramNV(param0,param1,param2,param3);
}
void (APIENTRY *ov_glLoadProgramNV)(unsigned int param0,unsigned int param1,signed int param2,char* param3)=init_ov_glLoadProgramNV;

void __gl_ml_glLoadProgramNV(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	char* l3=String_val(v3);
	ov_glLoadProgramNV(l0,l1,l2,l3);
}
extern unsigned char (APIENTRY *ov_glIsProgramNV)(unsigned int param0);

static unsigned char APIENTRY init_ov_glIsProgramNV(unsigned int param0)
{
	void *f;
	f=(void*)getfunc("glIsProgramNV");
	ov_glIsProgramNV=f;
	return ov_glIsProgramNV(param0);
}
unsigned char (APIENTRY *ov_glIsProgramNV)(unsigned int param0)=init_ov_glIsProgramNV;

value __gl_ml_glIsProgramNV(value v0)
{
	unsigned int l0=Int_val(v0);
	unsigned char r=ov_glIsProgramNV(l0);
	return(Val_int(r));
}
extern void (APIENTRY *ov_glGetVertexAttribPointervNV)(unsigned int param0,unsigned int param1,void** param2);

static void APIENTRY init_ov_glGetVertexAttribPointervNV(unsigned int param0,unsigned int param1,void** param2)
{
	void *f;
	f=(void*)getfunc("glGetVertexAttribPointervNV");
	ov_glGetVertexAttribPointervNV=f;
	ov_glGetVertexAttribPointervNV(param0,param1,param2);
}
void (APIENTRY *ov_glGetVertexAttribPointervNV)(unsigned int param0,unsigned int param1,void** param2)=init_ov_glGetVertexAttribPointervNV;
extern void (APIENTRY *ov_glGetVertexAttribivNV)(unsigned int param0,unsigned int param1,signed int* param2);

static void APIENTRY init_ov_glGetVertexAttribivNV(unsigned int param0,unsigned int param1,signed int* param2)
{
	void *f;
	f=(void*)getfunc("glGetVertexAttribivNV");
	ov_glGetVertexAttribivNV=f;
	ov_glGetVertexAttribivNV(param0,param1,param2);
}
void (APIENTRY *ov_glGetVertexAttribivNV)(unsigned int param0,unsigned int param1,signed int* param2)=init_ov_glGetVertexAttribivNV;

void __gl_ml_glGetVertexAttribivNV(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int* l2=(signed int*)(gl__ba__(v2));
	ov_glGetVertexAttribivNV(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetVertexAttribfvNV)(unsigned int param0,unsigned int param1,float* param2);

static void APIENTRY init_ov_glGetVertexAttribfvNV(unsigned int param0,unsigned int param1,float* param2)
{
	void *f;
	f=(void*)getfunc("glGetVertexAttribfvNV");
	ov_glGetVertexAttribfvNV=f;
	ov_glGetVertexAttribfvNV(param0,param1,param2);
}
void (APIENTRY *ov_glGetVertexAttribfvNV)(unsigned int param0,unsigned int param1,float* param2)=init_ov_glGetVertexAttribfvNV;

void __gl_ml_glGetVertexAttribfvNV(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glGetVertexAttribfvNV(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetVertexAttribdvNV)(unsigned int param0,unsigned int param1,double* param2);

static void APIENTRY init_ov_glGetVertexAttribdvNV(unsigned int param0,unsigned int param1,double* param2)
{
	void *f;
	f=(void*)getfunc("glGetVertexAttribdvNV");
	ov_glGetVertexAttribdvNV=f;
	ov_glGetVertexAttribdvNV(param0,param1,param2);
}
void (APIENTRY *ov_glGetVertexAttribdvNV)(unsigned int param0,unsigned int param1,double* param2)=init_ov_glGetVertexAttribdvNV;

void __gl_ml_glGetVertexAttribdvNV(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	double* l2=(double*)(gl__ba__(v2));
	ov_glGetVertexAttribdvNV(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetTrackMatrixivNV)(unsigned int param0,unsigned int param1,unsigned int param2,signed int* param3);

static void APIENTRY init_ov_glGetTrackMatrixivNV(unsigned int param0,unsigned int param1,unsigned int param2,signed int* param3)
{
	void *f;
	f=(void*)getfunc("glGetTrackMatrixivNV");
	ov_glGetTrackMatrixivNV=f;
	ov_glGetTrackMatrixivNV(param0,param1,param2,param3);
}
void (APIENTRY *ov_glGetTrackMatrixivNV)(unsigned int param0,unsigned int param1,unsigned int param2,signed int* param3)=init_ov_glGetTrackMatrixivNV;

void __gl_ml_glGetTrackMatrixivNV(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	signed int* l3=(signed int*)(gl__ba__(v3));
	ov_glGetTrackMatrixivNV(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glGetProgramStringNV)(unsigned int param0,unsigned int param1,unsigned char* param2);

static void APIENTRY init_ov_glGetProgramStringNV(unsigned int param0,unsigned int param1,unsigned char* param2)
{
	void *f;
	f=(void*)getfunc("glGetProgramStringNV");
	ov_glGetProgramStringNV=f;
	ov_glGetProgramStringNV(param0,param1,param2);
}
void (APIENTRY *ov_glGetProgramStringNV)(unsigned int param0,unsigned int param1,unsigned char* param2)=init_ov_glGetProgramStringNV;

void __gl_ml_glGetProgramStringNV(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned char* l2=(unsigned char*)(gl__ba__(v2));
	ov_glGetProgramStringNV(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetProgramivNV)(unsigned int param0,unsigned int param1,signed int* param2);

static void APIENTRY init_ov_glGetProgramivNV(unsigned int param0,unsigned int param1,signed int* param2)
{
	void *f;
	f=(void*)getfunc("glGetProgramivNV");
	ov_glGetProgramivNV=f;
	ov_glGetProgramivNV(param0,param1,param2);
}
void (APIENTRY *ov_glGetProgramivNV)(unsigned int param0,unsigned int param1,signed int* param2)=init_ov_glGetProgramivNV;

void __gl_ml_glGetProgramivNV(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int* l2=(signed int*)(gl__ba__(v2));
	ov_glGetProgramivNV(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetProgramParameterfvNV)(unsigned int param0,unsigned int param1,unsigned int param2,float* param3);

static void APIENTRY init_ov_glGetProgramParameterfvNV(unsigned int param0,unsigned int param1,unsigned int param2,float* param3)
{
	void *f;
	f=(void*)getfunc("glGetProgramParameterfvNV");
	ov_glGetProgramParameterfvNV=f;
	ov_glGetProgramParameterfvNV(param0,param1,param2,param3);
}
void (APIENTRY *ov_glGetProgramParameterfvNV)(unsigned int param0,unsigned int param1,unsigned int param2,float* param3)=init_ov_glGetProgramParameterfvNV;

void __gl_ml_glGetProgramParameterfvNV(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	float* l3=(float*)(gl__ba__(v3));
	ov_glGetProgramParameterfvNV(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glGetProgramParameterdvNV)(unsigned int param0,unsigned int param1,unsigned int param2,double* param3);

static void APIENTRY init_ov_glGetProgramParameterdvNV(unsigned int param0,unsigned int param1,unsigned int param2,double* param3)
{
	void *f;
	f=(void*)getfunc("glGetProgramParameterdvNV");
	ov_glGetProgramParameterdvNV=f;
	ov_glGetProgramParameterdvNV(param0,param1,param2,param3);
}
void (APIENTRY *ov_glGetProgramParameterdvNV)(unsigned int param0,unsigned int param1,unsigned int param2,double* param3)=init_ov_glGetProgramParameterdvNV;

void __gl_ml_glGetProgramParameterdvNV(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	double* l3=(double*)(gl__ba__(v3));
	ov_glGetProgramParameterdvNV(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glGenProgramsNV)(signed int param0,unsigned int* param1);

static void APIENTRY init_ov_glGenProgramsNV(signed int param0,unsigned int* param1)
{
	void *f;
	f=(void*)getfunc("glGenProgramsNV");
	ov_glGenProgramsNV=f;
	ov_glGenProgramsNV(param0,param1);
}
void (APIENTRY *ov_glGenProgramsNV)(signed int param0,unsigned int* param1)=init_ov_glGenProgramsNV;

void __gl_ml_glGenProgramsNV(value v0,value v1)
{
	signed int l0=Int_val(v0);
	unsigned int* l1=(unsigned int*)(gl__ba__(v1));
	ov_glGenProgramsNV(l0,l1);
}
extern void (APIENTRY *ov_glExecuteProgramNV)(unsigned int param0,unsigned int param1,float* param2);

static void APIENTRY init_ov_glExecuteProgramNV(unsigned int param0,unsigned int param1,float* param2)
{
	void *f;
	f=(void*)getfunc("glExecuteProgramNV");
	ov_glExecuteProgramNV=f;
	ov_glExecuteProgramNV(param0,param1,param2);
}
void (APIENTRY *ov_glExecuteProgramNV)(unsigned int param0,unsigned int param1,float* param2)=init_ov_glExecuteProgramNV;

void __gl_ml_glExecuteProgramNV(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glExecuteProgramNV(l0,l1,l2);
}
extern void (APIENTRY *ov_glDeleteProgramsNV)(signed int param0,unsigned int* param1);

static void APIENTRY init_ov_glDeleteProgramsNV(signed int param0,unsigned int* param1)
{
	void *f;
	f=(void*)getfunc("glDeleteProgramsNV");
	ov_glDeleteProgramsNV=f;
	ov_glDeleteProgramsNV(param0,param1);
}
void (APIENTRY *ov_glDeleteProgramsNV)(signed int param0,unsigned int* param1)=init_ov_glDeleteProgramsNV;

void __gl_ml_glDeleteProgramsNV(value v0,value v1)
{
	signed int l0=Int_val(v0);
	unsigned int* l1=(unsigned int*)(gl__ba__(v1));
	ov_glDeleteProgramsNV(l0,l1);
}
extern void (APIENTRY *ov_glBindProgramNV)(unsigned int param0,unsigned int param1);

static void APIENTRY init_ov_glBindProgramNV(unsigned int param0,unsigned int param1)
{
	void *f;
	f=(void*)getfunc("glBindProgramNV");
	ov_glBindProgramNV=f;
	ov_glBindProgramNV(param0,param1);
}
void (APIENTRY *ov_glBindProgramNV)(unsigned int param0,unsigned int param1)=init_ov_glBindProgramNV;

void __gl_ml_glBindProgramNV(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	ov_glBindProgramNV(l0,l1);
}
extern unsigned char (APIENTRY *ov_glAreProgramsResidentNV)(signed int param0,unsigned int* param1,unsigned char* param2);

static unsigned char APIENTRY init_ov_glAreProgramsResidentNV(signed int param0,unsigned int* param1,unsigned char* param2)
{
	void *f;
	f=(void*)getfunc("glAreProgramsResidentNV");
	ov_glAreProgramsResidentNV=f;
	return ov_glAreProgramsResidentNV(param0,param1,param2);
}
unsigned char (APIENTRY *ov_glAreProgramsResidentNV)(signed int param0,unsigned int* param1,unsigned char* param2)=init_ov_glAreProgramsResidentNV;

value __gl_ml_glAreProgramsResidentNV(value v0,value v1,value v2)
{
	signed int l0=Int_val(v0);
	unsigned int* l1=(unsigned int*)(gl__ba__(v1));
	unsigned char* l2=(unsigned char*)(gl__ba__(v2));
	unsigned char r=ov_glAreProgramsResidentNV(l0,l1,l2);
	return(Val_int(r));
}
extern void (APIENTRY *ov_glMultiDrawRangeElementArrayNV)(unsigned int mode,unsigned int start,unsigned int end,signed int* first,signed int* count,signed int primcount);

static void APIENTRY init_ov_glMultiDrawRangeElementArrayNV(unsigned int mode,unsigned int start,unsigned int end,signed int* first,signed int* count,signed int primcount)
{
	void *f;
	f=(void*)getfunc("glMultiDrawRangeElementArrayNV");
	ov_glMultiDrawRangeElementArrayNV=f;
	ov_glMultiDrawRangeElementArrayNV(mode,start,end,first,count,primcount);
}
void (APIENTRY *ov_glMultiDrawRangeElementArrayNV)(unsigned int mode,unsigned int start,unsigned int end,signed int* first,signed int* count,signed int primcount)=init_ov_glMultiDrawRangeElementArrayNV;

void __gl_ml_glMultiDrawRangeElementArrayNV(value v0,value v1,value v2,value v3,value v4,value v5)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	signed int* l3=(signed int*)(gl__ba__(v3));
	signed int* l4=(signed int*)(gl__ba__(v4));
	signed int l5=Int_val(v5);
	ov_glMultiDrawRangeElementArrayNV(l0,l1,l2,l3,l4,l5);
}

void byte___gl_ml_glMultiDrawRangeElementArrayNV(value * argv,int argn)
{
	__gl_ml_glMultiDrawRangeElementArrayNV(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5]);
}
extern void (APIENTRY *ov_glMultiDrawElementArrayNV)(unsigned int mode,signed int* first,signed int* count,signed int primcount);

static void APIENTRY init_ov_glMultiDrawElementArrayNV(unsigned int mode,signed int* first,signed int* count,signed int primcount)
{
	void *f;
	f=(void*)getfunc("glMultiDrawElementArrayNV");
	ov_glMultiDrawElementArrayNV=f;
	ov_glMultiDrawElementArrayNV(mode,first,count,primcount);
}
void (APIENTRY *ov_glMultiDrawElementArrayNV)(unsigned int mode,signed int* first,signed int* count,signed int primcount)=init_ov_glMultiDrawElementArrayNV;

void __gl_ml_glMultiDrawElementArrayNV(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	signed int* l1=(signed int*)(gl__ba__(v1));
	signed int* l2=(signed int*)(gl__ba__(v2));
	signed int l3=Int_val(v3);
	ov_glMultiDrawElementArrayNV(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glDrawRangeElementArrayNV)(unsigned int mode,unsigned int start,unsigned int end,signed int first,signed int count);

static void APIENTRY init_ov_glDrawRangeElementArrayNV(unsigned int mode,unsigned int start,unsigned int end,signed int first,signed int count)
{
	void *f;
	f=(void*)getfunc("glDrawRangeElementArrayNV");
	ov_glDrawRangeElementArrayNV=f;
	ov_glDrawRangeElementArrayNV(mode,start,end,first,count);
}
void (APIENTRY *ov_glDrawRangeElementArrayNV)(unsigned int mode,unsigned int start,unsigned int end,signed int first,signed int count)=init_ov_glDrawRangeElementArrayNV;

void __gl_ml_glDrawRangeElementArrayNV(value v0,value v1,value v2,value v3,value v4)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	signed int l4=Int_val(v4);
	ov_glDrawRangeElementArrayNV(l0,l1,l2,l3,l4);
}
extern void (APIENTRY *ov_glDrawElementArrayNV)(unsigned int mode,signed int first,signed int count);

static void APIENTRY init_ov_glDrawElementArrayNV(unsigned int mode,signed int first,signed int count)
{
	void *f;
	f=(void*)getfunc("glDrawElementArrayNV");
	ov_glDrawElementArrayNV=f;
	ov_glDrawElementArrayNV(mode,first,count);
}
void (APIENTRY *ov_glDrawElementArrayNV)(unsigned int mode,signed int first,signed int count)=init_ov_glDrawElementArrayNV;

void __gl_ml_glDrawElementArrayNV(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	ov_glDrawElementArrayNV(l0,l1,l2);
}
extern void (APIENTRY *ov_glElementPointerNV)(unsigned int type,void* pointer);

static void APIENTRY init_ov_glElementPointerNV(unsigned int type,void* pointer)
{
	void *f;
	f=(void*)getfunc("glElementPointerNV");
	ov_glElementPointerNV=f;
	ov_glElementPointerNV(type,pointer);
}
void (APIENTRY *ov_glElementPointerNV)(unsigned int type,void* pointer)=init_ov_glElementPointerNV;

void __gl_ml_glElementPointerNV(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	void* l1=gl__ba__(v1);
	ov_glElementPointerNV(l0,l1);
}
extern void (APIENTRY *ov_glGetProgramNamedParameterdvNV)(unsigned int id,signed int len,char* name,double* params);

static void APIENTRY init_ov_glGetProgramNamedParameterdvNV(unsigned int id,signed int len,char* name,double* params)
{
	void *f;
	f=(void*)getfunc("glGetProgramNamedParameterdvNV");
	ov_glGetProgramNamedParameterdvNV=f;
	ov_glGetProgramNamedParameterdvNV(id,len,name,params);
}
void (APIENTRY *ov_glGetProgramNamedParameterdvNV)(unsigned int id,signed int len,char* name,double* params)=init_ov_glGetProgramNamedParameterdvNV;

void __gl_ml_glGetProgramNamedParameterdvNV(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	char* l2=String_val(v2);
	double* l3=(double*)(gl__ba__(v3));
	ov_glGetProgramNamedParameterdvNV(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glGetProgramNamedParameterfvNV)(unsigned int id,signed int len,char* name,float* params);

static void APIENTRY init_ov_glGetProgramNamedParameterfvNV(unsigned int id,signed int len,char* name,float* params)
{
	void *f;
	f=(void*)getfunc("glGetProgramNamedParameterfvNV");
	ov_glGetProgramNamedParameterfvNV=f;
	ov_glGetProgramNamedParameterfvNV(id,len,name,params);
}
void (APIENTRY *ov_glGetProgramNamedParameterfvNV)(unsigned int id,signed int len,char* name,float* params)=init_ov_glGetProgramNamedParameterfvNV;

void __gl_ml_glGetProgramNamedParameterfvNV(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	char* l2=String_val(v2);
	float* l3=(float*)(gl__ba__(v3));
	ov_glGetProgramNamedParameterfvNV(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glProgramNamedParameter4dvNV)(unsigned int id,signed int len,char* name,double* v);

static void APIENTRY init_ov_glProgramNamedParameter4dvNV(unsigned int id,signed int len,char* name,double* v)
{
	void *f;
	f=(void*)getfunc("glProgramNamedParameter4dvNV");
	ov_glProgramNamedParameter4dvNV=f;
	ov_glProgramNamedParameter4dvNV(id,len,name,v);
}
void (APIENTRY *ov_glProgramNamedParameter4dvNV)(unsigned int id,signed int len,char* name,double* v)=init_ov_glProgramNamedParameter4dvNV;

void __gl_ml_glProgramNamedParameter4dvNV(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	char* l2=String_val(v2);
	double* l3=(double*)(gl__ba__(v3));
	ov_glProgramNamedParameter4dvNV(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glProgramNamedParameter4fvNV)(unsigned int id,signed int len,char* name,float* v);

static void APIENTRY init_ov_glProgramNamedParameter4fvNV(unsigned int id,signed int len,char* name,float* v)
{
	void *f;
	f=(void*)getfunc("glProgramNamedParameter4fvNV");
	ov_glProgramNamedParameter4fvNV=f;
	ov_glProgramNamedParameter4fvNV(id,len,name,v);
}
void (APIENTRY *ov_glProgramNamedParameter4fvNV)(unsigned int id,signed int len,char* name,float* v)=init_ov_glProgramNamedParameter4fvNV;

void __gl_ml_glProgramNamedParameter4fvNV(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	char* l2=String_val(v2);
	float* l3=(float*)(gl__ba__(v3));
	ov_glProgramNamedParameter4fvNV(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glProgramNamedParameter4dNV)(unsigned int id,signed int len,char* name,double x,double y,double z,double w);

static void APIENTRY init_ov_glProgramNamedParameter4dNV(unsigned int id,signed int len,char* name,double x,double y,double z,double w)
{
	void *f;
	f=(void*)getfunc("glProgramNamedParameter4dNV");
	ov_glProgramNamedParameter4dNV=f;
	ov_glProgramNamedParameter4dNV(id,len,name,x,y,z,w);
}
void (APIENTRY *ov_glProgramNamedParameter4dNV)(unsigned int id,signed int len,char* name,double x,double y,double z,double w)=init_ov_glProgramNamedParameter4dNV;

void __gl_ml_glProgramNamedParameter4dNV(value v0,value v1,value v2,value v3,value v4,value v5,value v6)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	char* l2=String_val(v2);
	double l3=Double_val(v3);
	double l4=Double_val(v4);
	double l5=Double_val(v5);
	double l6=Double_val(v6);
	ov_glProgramNamedParameter4dNV(l0,l1,l2,l3,l4,l5,l6);
}

void byte___gl_ml_glProgramNamedParameter4dNV(value * argv,int argn)
{
	__gl_ml_glProgramNamedParameter4dNV(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6]);
}
extern void (APIENTRY *ov_glProgramNamedParameter4fNV)(unsigned int id,signed int len,char* name,float x,float y,float z,float w);

static void APIENTRY init_ov_glProgramNamedParameter4fNV(unsigned int id,signed int len,char* name,float x,float y,float z,float w)
{
	void *f;
	f=(void*)getfunc("glProgramNamedParameter4fNV");
	ov_glProgramNamedParameter4fNV=f;
	ov_glProgramNamedParameter4fNV(id,len,name,x,y,z,w);
}
void (APIENTRY *ov_glProgramNamedParameter4fNV)(unsigned int id,signed int len,char* name,float x,float y,float z,float w)=init_ov_glProgramNamedParameter4fNV;

void __gl_ml_glProgramNamedParameter4fNV(value v0,value v1,value v2,value v3,value v4,value v5,value v6)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	char* l2=String_val(v2);
	float l3=Double_val(v3);
	float l4=Double_val(v4);
	float l5=Double_val(v5);
	float l6=Double_val(v6);
	ov_glProgramNamedParameter4fNV(l0,l1,l2,l3,l4,l5,l6);
}

void byte___gl_ml_glProgramNamedParameter4fNV(value * argv,int argn)
{
	__gl_ml_glProgramNamedParameter4fNV(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6]);
}
extern void (APIENTRY *ov_glVertexAttribs4hvNV)(unsigned int index,signed int n,unsigned short* v);

static void APIENTRY init_ov_glVertexAttribs4hvNV(unsigned int index,signed int n,unsigned short* v)
{
	void *f;
	f=(void*)getfunc("glVertexAttribs4hvNV");
	ov_glVertexAttribs4hvNV=f;
	ov_glVertexAttribs4hvNV(index,n,v);
}
void (APIENTRY *ov_glVertexAttribs4hvNV)(unsigned int index,signed int n,unsigned short* v)=init_ov_glVertexAttribs4hvNV;

void __gl_ml_glVertexAttribs4hvNV(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	unsigned short* l2=(unsigned short*)(gl__ba__(v2));
	ov_glVertexAttribs4hvNV(l0,l1,l2);
}
extern void (APIENTRY *ov_glVertexAttribs3hvNV)(unsigned int index,signed int n,unsigned short* v);

static void APIENTRY init_ov_glVertexAttribs3hvNV(unsigned int index,signed int n,unsigned short* v)
{
	void *f;
	f=(void*)getfunc("glVertexAttribs3hvNV");
	ov_glVertexAttribs3hvNV=f;
	ov_glVertexAttribs3hvNV(index,n,v);
}
void (APIENTRY *ov_glVertexAttribs3hvNV)(unsigned int index,signed int n,unsigned short* v)=init_ov_glVertexAttribs3hvNV;

void __gl_ml_glVertexAttribs3hvNV(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	unsigned short* l2=(unsigned short*)(gl__ba__(v2));
	ov_glVertexAttribs3hvNV(l0,l1,l2);
}
extern void (APIENTRY *ov_glVertexAttribs2hvNV)(unsigned int index,signed int n,unsigned short* v);

static void APIENTRY init_ov_glVertexAttribs2hvNV(unsigned int index,signed int n,unsigned short* v)
{
	void *f;
	f=(void*)getfunc("glVertexAttribs2hvNV");
	ov_glVertexAttribs2hvNV=f;
	ov_glVertexAttribs2hvNV(index,n,v);
}
void (APIENTRY *ov_glVertexAttribs2hvNV)(unsigned int index,signed int n,unsigned short* v)=init_ov_glVertexAttribs2hvNV;

void __gl_ml_glVertexAttribs2hvNV(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	unsigned short* l2=(unsigned short*)(gl__ba__(v2));
	ov_glVertexAttribs2hvNV(l0,l1,l2);
}
extern void (APIENTRY *ov_glVertexAttribs1hvNV)(unsigned int index,signed int n,unsigned short* v);

static void APIENTRY init_ov_glVertexAttribs1hvNV(unsigned int index,signed int n,unsigned short* v)
{
	void *f;
	f=(void*)getfunc("glVertexAttribs1hvNV");
	ov_glVertexAttribs1hvNV=f;
	ov_glVertexAttribs1hvNV(index,n,v);
}
void (APIENTRY *ov_glVertexAttribs1hvNV)(unsigned int index,signed int n,unsigned short* v)=init_ov_glVertexAttribs1hvNV;

void __gl_ml_glVertexAttribs1hvNV(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	unsigned short* l2=(unsigned short*)(gl__ba__(v2));
	ov_glVertexAttribs1hvNV(l0,l1,l2);
}
extern void (APIENTRY *ov_glVertexAttrib4hvNV)(unsigned int index,unsigned short* v);

static void APIENTRY init_ov_glVertexAttrib4hvNV(unsigned int index,unsigned short* v)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib4hvNV");
	ov_glVertexAttrib4hvNV=f;
	ov_glVertexAttrib4hvNV(index,v);
}
void (APIENTRY *ov_glVertexAttrib4hvNV)(unsigned int index,unsigned short* v)=init_ov_glVertexAttrib4hvNV;

void __gl_ml_glVertexAttrib4hvNV(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	unsigned short* l1=(unsigned short*)(gl__ba__(v1));
	ov_glVertexAttrib4hvNV(l0,l1);
}
extern void (APIENTRY *ov_glVertexAttrib4hNV)(unsigned int index,unsigned short x,unsigned short y,unsigned short z,unsigned short w);

static void APIENTRY init_ov_glVertexAttrib4hNV(unsigned int index,unsigned short x,unsigned short y,unsigned short z,unsigned short w)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib4hNV");
	ov_glVertexAttrib4hNV=f;
	ov_glVertexAttrib4hNV(index,x,y,z,w);
}
void (APIENTRY *ov_glVertexAttrib4hNV)(unsigned int index,unsigned short x,unsigned short y,unsigned short z,unsigned short w)=init_ov_glVertexAttrib4hNV;

void __gl_ml_glVertexAttrib4hNV(value v0,value v1,value v2,value v3,value v4)
{
	unsigned int l0=Int_val(v0);
	unsigned short l1=Int_val(v1);
	unsigned short l2=Int_val(v2);
	unsigned short l3=Int_val(v3);
	unsigned short l4=Int_val(v4);
	ov_glVertexAttrib4hNV(l0,l1,l2,l3,l4);
}
extern void (APIENTRY *ov_glVertexAttrib3hvNV)(unsigned int index,unsigned short* v);

static void APIENTRY init_ov_glVertexAttrib3hvNV(unsigned int index,unsigned short* v)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib3hvNV");
	ov_glVertexAttrib3hvNV=f;
	ov_glVertexAttrib3hvNV(index,v);
}
void (APIENTRY *ov_glVertexAttrib3hvNV)(unsigned int index,unsigned short* v)=init_ov_glVertexAttrib3hvNV;

void __gl_ml_glVertexAttrib3hvNV(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	unsigned short* l1=(unsigned short*)(gl__ba__(v1));
	ov_glVertexAttrib3hvNV(l0,l1);
}
extern void (APIENTRY *ov_glVertexAttrib3hNV)(unsigned int index,unsigned short x,unsigned short y,unsigned short z);

static void APIENTRY init_ov_glVertexAttrib3hNV(unsigned int index,unsigned short x,unsigned short y,unsigned short z)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib3hNV");
	ov_glVertexAttrib3hNV=f;
	ov_glVertexAttrib3hNV(index,x,y,z);
}
void (APIENTRY *ov_glVertexAttrib3hNV)(unsigned int index,unsigned short x,unsigned short y,unsigned short z)=init_ov_glVertexAttrib3hNV;

void __gl_ml_glVertexAttrib3hNV(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	unsigned short l1=Int_val(v1);
	unsigned short l2=Int_val(v2);
	unsigned short l3=Int_val(v3);
	ov_glVertexAttrib3hNV(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glVertexAttrib2hvNV)(unsigned int index,unsigned short* v);

static void APIENTRY init_ov_glVertexAttrib2hvNV(unsigned int index,unsigned short* v)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib2hvNV");
	ov_glVertexAttrib2hvNV=f;
	ov_glVertexAttrib2hvNV(index,v);
}
void (APIENTRY *ov_glVertexAttrib2hvNV)(unsigned int index,unsigned short* v)=init_ov_glVertexAttrib2hvNV;

void __gl_ml_glVertexAttrib2hvNV(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	unsigned short* l1=(unsigned short*)(gl__ba__(v1));
	ov_glVertexAttrib2hvNV(l0,l1);
}
extern void (APIENTRY *ov_glVertexAttrib2hNV)(unsigned int index,unsigned short x,unsigned short y);

static void APIENTRY init_ov_glVertexAttrib2hNV(unsigned int index,unsigned short x,unsigned short y)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib2hNV");
	ov_glVertexAttrib2hNV=f;
	ov_glVertexAttrib2hNV(index,x,y);
}
void (APIENTRY *ov_glVertexAttrib2hNV)(unsigned int index,unsigned short x,unsigned short y)=init_ov_glVertexAttrib2hNV;

void __gl_ml_glVertexAttrib2hNV(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned short l1=Int_val(v1);
	unsigned short l2=Int_val(v2);
	ov_glVertexAttrib2hNV(l0,l1,l2);
}
extern void (APIENTRY *ov_glVertexAttrib1hvNV)(unsigned int index,unsigned short* v);

static void APIENTRY init_ov_glVertexAttrib1hvNV(unsigned int index,unsigned short* v)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib1hvNV");
	ov_glVertexAttrib1hvNV=f;
	ov_glVertexAttrib1hvNV(index,v);
}
void (APIENTRY *ov_glVertexAttrib1hvNV)(unsigned int index,unsigned short* v)=init_ov_glVertexAttrib1hvNV;

void __gl_ml_glVertexAttrib1hvNV(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	unsigned short* l1=(unsigned short*)(gl__ba__(v1));
	ov_glVertexAttrib1hvNV(l0,l1);
}
extern void (APIENTRY *ov_glVertexAttrib1hNV)(unsigned int index,unsigned short x);

static void APIENTRY init_ov_glVertexAttrib1hNV(unsigned int index,unsigned short x)
{
	void *f;
	f=(void*)getfunc("glVertexAttrib1hNV");
	ov_glVertexAttrib1hNV=f;
	ov_glVertexAttrib1hNV(index,x);
}
void (APIENTRY *ov_glVertexAttrib1hNV)(unsigned int index,unsigned short x)=init_ov_glVertexAttrib1hNV;

void __gl_ml_glVertexAttrib1hNV(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	unsigned short l1=Int_val(v1);
	ov_glVertexAttrib1hNV(l0,l1);
}
extern void (APIENTRY *ov_glVertexWeighthvNV)(unsigned short* weight);

static void APIENTRY init_ov_glVertexWeighthvNV(unsigned short* weight)
{
	void *f;
	f=(void*)getfunc("glVertexWeighthvNV");
	ov_glVertexWeighthvNV=f;
	ov_glVertexWeighthvNV(weight);
}
void (APIENTRY *ov_glVertexWeighthvNV)(unsigned short* weight)=init_ov_glVertexWeighthvNV;

void __gl_ml_glVertexWeighthvNV(value v0)
{
	unsigned short* l0=(unsigned short*)(gl__ba__(v0));
	ov_glVertexWeighthvNV(l0);
}
extern void (APIENTRY *ov_glVertexWeighthNV)(unsigned short weight);

static void APIENTRY init_ov_glVertexWeighthNV(unsigned short weight)
{
	void *f;
	f=(void*)getfunc("glVertexWeighthNV");
	ov_glVertexWeighthNV=f;
	ov_glVertexWeighthNV(weight);
}
void (APIENTRY *ov_glVertexWeighthNV)(unsigned short weight)=init_ov_glVertexWeighthNV;

void __gl_ml_glVertexWeighthNV(value v0)
{
	unsigned short l0=Int_val(v0);
	ov_glVertexWeighthNV(l0);
}
extern void (APIENTRY *ov_glSecondaryColor3hvNV)(unsigned short* v);

static void APIENTRY init_ov_glSecondaryColor3hvNV(unsigned short* v)
{
	void *f;
	f=(void*)getfunc("glSecondaryColor3hvNV");
	ov_glSecondaryColor3hvNV=f;
	ov_glSecondaryColor3hvNV(v);
}
void (APIENTRY *ov_glSecondaryColor3hvNV)(unsigned short* v)=init_ov_glSecondaryColor3hvNV;

void __gl_ml_glSecondaryColor3hvNV(value v0)
{
	unsigned short* l0=(unsigned short*)(gl__ba__(v0));
	ov_glSecondaryColor3hvNV(l0);
}
extern void (APIENTRY *ov_glSecondaryColor3hNV)(unsigned short red,unsigned short green,unsigned short blue);

static void APIENTRY init_ov_glSecondaryColor3hNV(unsigned short red,unsigned short green,unsigned short blue)
{
	void *f;
	f=(void*)getfunc("glSecondaryColor3hNV");
	ov_glSecondaryColor3hNV=f;
	ov_glSecondaryColor3hNV(red,green,blue);
}
void (APIENTRY *ov_glSecondaryColor3hNV)(unsigned short red,unsigned short green,unsigned short blue)=init_ov_glSecondaryColor3hNV;

void __gl_ml_glSecondaryColor3hNV(value v0,value v1,value v2)
{
	unsigned short l0=Int_val(v0);
	unsigned short l1=Int_val(v1);
	unsigned short l2=Int_val(v2);
	ov_glSecondaryColor3hNV(l0,l1,l2);
}
extern void (APIENTRY *ov_glFogCoordhvNV)(unsigned short* fog);

static void APIENTRY init_ov_glFogCoordhvNV(unsigned short* fog)
{
	void *f;
	f=(void*)getfunc("glFogCoordhvNV");
	ov_glFogCoordhvNV=f;
	ov_glFogCoordhvNV(fog);
}
void (APIENTRY *ov_glFogCoordhvNV)(unsigned short* fog)=init_ov_glFogCoordhvNV;

void __gl_ml_glFogCoordhvNV(value v0)
{
	unsigned short* l0=(unsigned short*)(gl__ba__(v0));
	ov_glFogCoordhvNV(l0);
}
extern void (APIENTRY *ov_glFogCoordhNV)(unsigned short fog);

static void APIENTRY init_ov_glFogCoordhNV(unsigned short fog)
{
	void *f;
	f=(void*)getfunc("glFogCoordhNV");
	ov_glFogCoordhNV=f;
	ov_glFogCoordhNV(fog);
}
void (APIENTRY *ov_glFogCoordhNV)(unsigned short fog)=init_ov_glFogCoordhNV;

void __gl_ml_glFogCoordhNV(value v0)
{
	unsigned short l0=Int_val(v0);
	ov_glFogCoordhNV(l0);
}
extern void (APIENTRY *ov_glMultiTexCoord4hvNV)(unsigned int target,unsigned short* v);

static void APIENTRY init_ov_glMultiTexCoord4hvNV(unsigned int target,unsigned short* v)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord4hvNV");
	ov_glMultiTexCoord4hvNV=f;
	ov_glMultiTexCoord4hvNV(target,v);
}
void (APIENTRY *ov_glMultiTexCoord4hvNV)(unsigned int target,unsigned short* v)=init_ov_glMultiTexCoord4hvNV;

void __gl_ml_glMultiTexCoord4hvNV(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	unsigned short* l1=(unsigned short*)(gl__ba__(v1));
	ov_glMultiTexCoord4hvNV(l0,l1);
}
extern void (APIENTRY *ov_glMultiTexCoord4hNV)(unsigned int target,unsigned short s,unsigned short t,unsigned short r,unsigned short q);

static void APIENTRY init_ov_glMultiTexCoord4hNV(unsigned int target,unsigned short s,unsigned short t,unsigned short r,unsigned short q)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord4hNV");
	ov_glMultiTexCoord4hNV=f;
	ov_glMultiTexCoord4hNV(target,s,t,r,q);
}
void (APIENTRY *ov_glMultiTexCoord4hNV)(unsigned int target,unsigned short s,unsigned short t,unsigned short r,unsigned short q)=init_ov_glMultiTexCoord4hNV;

void __gl_ml_glMultiTexCoord4hNV(value v0,value v1,value v2,value v3,value v4)
{
	unsigned int l0=Int_val(v0);
	unsigned short l1=Int_val(v1);
	unsigned short l2=Int_val(v2);
	unsigned short l3=Int_val(v3);
	unsigned short l4=Int_val(v4);
	ov_glMultiTexCoord4hNV(l0,l1,l2,l3,l4);
}
extern void (APIENTRY *ov_glMultiTexCoord3hvNV)(unsigned int target,unsigned short* v);

static void APIENTRY init_ov_glMultiTexCoord3hvNV(unsigned int target,unsigned short* v)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord3hvNV");
	ov_glMultiTexCoord3hvNV=f;
	ov_glMultiTexCoord3hvNV(target,v);
}
void (APIENTRY *ov_glMultiTexCoord3hvNV)(unsigned int target,unsigned short* v)=init_ov_glMultiTexCoord3hvNV;

void __gl_ml_glMultiTexCoord3hvNV(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	unsigned short* l1=(unsigned short*)(gl__ba__(v1));
	ov_glMultiTexCoord3hvNV(l0,l1);
}
extern void (APIENTRY *ov_glMultiTexCoord3hNV)(unsigned int target,unsigned short s,unsigned short t,unsigned short r);

static void APIENTRY init_ov_glMultiTexCoord3hNV(unsigned int target,unsigned short s,unsigned short t,unsigned short r)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord3hNV");
	ov_glMultiTexCoord3hNV=f;
	ov_glMultiTexCoord3hNV(target,s,t,r);
}
void (APIENTRY *ov_glMultiTexCoord3hNV)(unsigned int target,unsigned short s,unsigned short t,unsigned short r)=init_ov_glMultiTexCoord3hNV;

void __gl_ml_glMultiTexCoord3hNV(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	unsigned short l1=Int_val(v1);
	unsigned short l2=Int_val(v2);
	unsigned short l3=Int_val(v3);
	ov_glMultiTexCoord3hNV(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glMultiTexCoord2hvNV)(unsigned int target,unsigned short* v);

static void APIENTRY init_ov_glMultiTexCoord2hvNV(unsigned int target,unsigned short* v)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord2hvNV");
	ov_glMultiTexCoord2hvNV=f;
	ov_glMultiTexCoord2hvNV(target,v);
}
void (APIENTRY *ov_glMultiTexCoord2hvNV)(unsigned int target,unsigned short* v)=init_ov_glMultiTexCoord2hvNV;

void __gl_ml_glMultiTexCoord2hvNV(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	unsigned short* l1=(unsigned short*)(gl__ba__(v1));
	ov_glMultiTexCoord2hvNV(l0,l1);
}
extern void (APIENTRY *ov_glMultiTexCoord2hNV)(unsigned int target,unsigned short s,unsigned short t);

static void APIENTRY init_ov_glMultiTexCoord2hNV(unsigned int target,unsigned short s,unsigned short t)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord2hNV");
	ov_glMultiTexCoord2hNV=f;
	ov_glMultiTexCoord2hNV(target,s,t);
}
void (APIENTRY *ov_glMultiTexCoord2hNV)(unsigned int target,unsigned short s,unsigned short t)=init_ov_glMultiTexCoord2hNV;

void __gl_ml_glMultiTexCoord2hNV(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned short l1=Int_val(v1);
	unsigned short l2=Int_val(v2);
	ov_glMultiTexCoord2hNV(l0,l1,l2);
}
extern void (APIENTRY *ov_glMultiTexCoord1hvNV)(unsigned int target,unsigned short* v);

static void APIENTRY init_ov_glMultiTexCoord1hvNV(unsigned int target,unsigned short* v)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord1hvNV");
	ov_glMultiTexCoord1hvNV=f;
	ov_glMultiTexCoord1hvNV(target,v);
}
void (APIENTRY *ov_glMultiTexCoord1hvNV)(unsigned int target,unsigned short* v)=init_ov_glMultiTexCoord1hvNV;

void __gl_ml_glMultiTexCoord1hvNV(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	unsigned short* l1=(unsigned short*)(gl__ba__(v1));
	ov_glMultiTexCoord1hvNV(l0,l1);
}
extern void (APIENTRY *ov_glMultiTexCoord1hNV)(unsigned int target,unsigned short s);

static void APIENTRY init_ov_glMultiTexCoord1hNV(unsigned int target,unsigned short s)
{
	void *f;
	f=(void*)getfunc("glMultiTexCoord1hNV");
	ov_glMultiTexCoord1hNV=f;
	ov_glMultiTexCoord1hNV(target,s);
}
void (APIENTRY *ov_glMultiTexCoord1hNV)(unsigned int target,unsigned short s)=init_ov_glMultiTexCoord1hNV;

void __gl_ml_glMultiTexCoord1hNV(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	unsigned short l1=Int_val(v1);
	ov_glMultiTexCoord1hNV(l0,l1);
}
extern void (APIENTRY *ov_glTexCoord4hvNV)(unsigned short* v);

static void APIENTRY init_ov_glTexCoord4hvNV(unsigned short* v)
{
	void *f;
	f=(void*)getfunc("glTexCoord4hvNV");
	ov_glTexCoord4hvNV=f;
	ov_glTexCoord4hvNV(v);
}
void (APIENTRY *ov_glTexCoord4hvNV)(unsigned short* v)=init_ov_glTexCoord4hvNV;

void __gl_ml_glTexCoord4hvNV(value v0)
{
	unsigned short* l0=(unsigned short*)(gl__ba__(v0));
	ov_glTexCoord4hvNV(l0);
}
extern void (APIENTRY *ov_glTexCoord4hNV)(unsigned short s,unsigned short t,unsigned short r,unsigned short q);

static void APIENTRY init_ov_glTexCoord4hNV(unsigned short s,unsigned short t,unsigned short r,unsigned short q)
{
	void *f;
	f=(void*)getfunc("glTexCoord4hNV");
	ov_glTexCoord4hNV=f;
	ov_glTexCoord4hNV(s,t,r,q);
}
void (APIENTRY *ov_glTexCoord4hNV)(unsigned short s,unsigned short t,unsigned short r,unsigned short q)=init_ov_glTexCoord4hNV;

void __gl_ml_glTexCoord4hNV(value v0,value v1,value v2,value v3)
{
	unsigned short l0=Int_val(v0);
	unsigned short l1=Int_val(v1);
	unsigned short l2=Int_val(v2);
	unsigned short l3=Int_val(v3);
	ov_glTexCoord4hNV(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glTexCoord3hvNV)(unsigned short* v);

static void APIENTRY init_ov_glTexCoord3hvNV(unsigned short* v)
{
	void *f;
	f=(void*)getfunc("glTexCoord3hvNV");
	ov_glTexCoord3hvNV=f;
	ov_glTexCoord3hvNV(v);
}
void (APIENTRY *ov_glTexCoord3hvNV)(unsigned short* v)=init_ov_glTexCoord3hvNV;

void __gl_ml_glTexCoord3hvNV(value v0)
{
	unsigned short* l0=(unsigned short*)(gl__ba__(v0));
	ov_glTexCoord3hvNV(l0);
}
extern void (APIENTRY *ov_glTexCoord3hNV)(unsigned short s,unsigned short t,unsigned short r);

static void APIENTRY init_ov_glTexCoord3hNV(unsigned short s,unsigned short t,unsigned short r)
{
	void *f;
	f=(void*)getfunc("glTexCoord3hNV");
	ov_glTexCoord3hNV=f;
	ov_glTexCoord3hNV(s,t,r);
}
void (APIENTRY *ov_glTexCoord3hNV)(unsigned short s,unsigned short t,unsigned short r)=init_ov_glTexCoord3hNV;

void __gl_ml_glTexCoord3hNV(value v0,value v1,value v2)
{
	unsigned short l0=Int_val(v0);
	unsigned short l1=Int_val(v1);
	unsigned short l2=Int_val(v2);
	ov_glTexCoord3hNV(l0,l1,l2);
}
extern void (APIENTRY *ov_glTexCoord2hvNV)(unsigned short* v);

static void APIENTRY init_ov_glTexCoord2hvNV(unsigned short* v)
{
	void *f;
	f=(void*)getfunc("glTexCoord2hvNV");
	ov_glTexCoord2hvNV=f;
	ov_glTexCoord2hvNV(v);
}
void (APIENTRY *ov_glTexCoord2hvNV)(unsigned short* v)=init_ov_glTexCoord2hvNV;

void __gl_ml_glTexCoord2hvNV(value v0)
{
	unsigned short* l0=(unsigned short*)(gl__ba__(v0));
	ov_glTexCoord2hvNV(l0);
}
extern void (APIENTRY *ov_glTexCoord2hNV)(unsigned short s,unsigned short t);

static void APIENTRY init_ov_glTexCoord2hNV(unsigned short s,unsigned short t)
{
	void *f;
	f=(void*)getfunc("glTexCoord2hNV");
	ov_glTexCoord2hNV=f;
	ov_glTexCoord2hNV(s,t);
}
void (APIENTRY *ov_glTexCoord2hNV)(unsigned short s,unsigned short t)=init_ov_glTexCoord2hNV;

void __gl_ml_glTexCoord2hNV(value v0,value v1)
{
	unsigned short l0=Int_val(v0);
	unsigned short l1=Int_val(v1);
	ov_glTexCoord2hNV(l0,l1);
}
extern void (APIENTRY *ov_glTexCoord1hvNV)(unsigned short* v);

static void APIENTRY init_ov_glTexCoord1hvNV(unsigned short* v)
{
	void *f;
	f=(void*)getfunc("glTexCoord1hvNV");
	ov_glTexCoord1hvNV=f;
	ov_glTexCoord1hvNV(v);
}
void (APIENTRY *ov_glTexCoord1hvNV)(unsigned short* v)=init_ov_glTexCoord1hvNV;

void __gl_ml_glTexCoord1hvNV(value v0)
{
	unsigned short* l0=(unsigned short*)(gl__ba__(v0));
	ov_glTexCoord1hvNV(l0);
}
extern void (APIENTRY *ov_glTexCoord1hNV)(unsigned short s);

static void APIENTRY init_ov_glTexCoord1hNV(unsigned short s)
{
	void *f;
	f=(void*)getfunc("glTexCoord1hNV");
	ov_glTexCoord1hNV=f;
	ov_glTexCoord1hNV(s);
}
void (APIENTRY *ov_glTexCoord1hNV)(unsigned short s)=init_ov_glTexCoord1hNV;

void __gl_ml_glTexCoord1hNV(value v0)
{
	unsigned short l0=Int_val(v0);
	ov_glTexCoord1hNV(l0);
}
extern void (APIENTRY *ov_glColor4hvNV)(unsigned short* v);

static void APIENTRY init_ov_glColor4hvNV(unsigned short* v)
{
	void *f;
	f=(void*)getfunc("glColor4hvNV");
	ov_glColor4hvNV=f;
	ov_glColor4hvNV(v);
}
void (APIENTRY *ov_glColor4hvNV)(unsigned short* v)=init_ov_glColor4hvNV;

void __gl_ml_glColor4hvNV(value v0)
{
	unsigned short* l0=(unsigned short*)(gl__ba__(v0));
	ov_glColor4hvNV(l0);
}
extern void (APIENTRY *ov_glColor4hNV)(unsigned short red,unsigned short green,unsigned short blue,unsigned short alpha);

static void APIENTRY init_ov_glColor4hNV(unsigned short red,unsigned short green,unsigned short blue,unsigned short alpha)
{
	void *f;
	f=(void*)getfunc("glColor4hNV");
	ov_glColor4hNV=f;
	ov_glColor4hNV(red,green,blue,alpha);
}
void (APIENTRY *ov_glColor4hNV)(unsigned short red,unsigned short green,unsigned short blue,unsigned short alpha)=init_ov_glColor4hNV;

void __gl_ml_glColor4hNV(value v0,value v1,value v2,value v3)
{
	unsigned short l0=Int_val(v0);
	unsigned short l1=Int_val(v1);
	unsigned short l2=Int_val(v2);
	unsigned short l3=Int_val(v3);
	ov_glColor4hNV(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glColor3hvNV)(unsigned short* v);

static void APIENTRY init_ov_glColor3hvNV(unsigned short* v)
{
	void *f;
	f=(void*)getfunc("glColor3hvNV");
	ov_glColor3hvNV=f;
	ov_glColor3hvNV(v);
}
void (APIENTRY *ov_glColor3hvNV)(unsigned short* v)=init_ov_glColor3hvNV;

void __gl_ml_glColor3hvNV(value v0)
{
	unsigned short* l0=(unsigned short*)(gl__ba__(v0));
	ov_glColor3hvNV(l0);
}
extern void (APIENTRY *ov_glColor3hNV)(unsigned short red,unsigned short green,unsigned short blue);

static void APIENTRY init_ov_glColor3hNV(unsigned short red,unsigned short green,unsigned short blue)
{
	void *f;
	f=(void*)getfunc("glColor3hNV");
	ov_glColor3hNV=f;
	ov_glColor3hNV(red,green,blue);
}
void (APIENTRY *ov_glColor3hNV)(unsigned short red,unsigned short green,unsigned short blue)=init_ov_glColor3hNV;

void __gl_ml_glColor3hNV(value v0,value v1,value v2)
{
	unsigned short l0=Int_val(v0);
	unsigned short l1=Int_val(v1);
	unsigned short l2=Int_val(v2);
	ov_glColor3hNV(l0,l1,l2);
}
extern void (APIENTRY *ov_glNormal3hvNV)(unsigned short* v);

static void APIENTRY init_ov_glNormal3hvNV(unsigned short* v)
{
	void *f;
	f=(void*)getfunc("glNormal3hvNV");
	ov_glNormal3hvNV=f;
	ov_glNormal3hvNV(v);
}
void (APIENTRY *ov_glNormal3hvNV)(unsigned short* v)=init_ov_glNormal3hvNV;

void __gl_ml_glNormal3hvNV(value v0)
{
	unsigned short* l0=(unsigned short*)(gl__ba__(v0));
	ov_glNormal3hvNV(l0);
}
extern void (APIENTRY *ov_glNormal3hNV)(unsigned short nx,unsigned short ny,unsigned short nz);

static void APIENTRY init_ov_glNormal3hNV(unsigned short nx,unsigned short ny,unsigned short nz)
{
	void *f;
	f=(void*)getfunc("glNormal3hNV");
	ov_glNormal3hNV=f;
	ov_glNormal3hNV(nx,ny,nz);
}
void (APIENTRY *ov_glNormal3hNV)(unsigned short nx,unsigned short ny,unsigned short nz)=init_ov_glNormal3hNV;

void __gl_ml_glNormal3hNV(value v0,value v1,value v2)
{
	unsigned short l0=Int_val(v0);
	unsigned short l1=Int_val(v1);
	unsigned short l2=Int_val(v2);
	ov_glNormal3hNV(l0,l1,l2);
}
extern void (APIENTRY *ov_glVertex4hvNV)(unsigned short* v);

static void APIENTRY init_ov_glVertex4hvNV(unsigned short* v)
{
	void *f;
	f=(void*)getfunc("glVertex4hvNV");
	ov_glVertex4hvNV=f;
	ov_glVertex4hvNV(v);
}
void (APIENTRY *ov_glVertex4hvNV)(unsigned short* v)=init_ov_glVertex4hvNV;

void __gl_ml_glVertex4hvNV(value v0)
{
	unsigned short* l0=(unsigned short*)(gl__ba__(v0));
	ov_glVertex4hvNV(l0);
}
extern void (APIENTRY *ov_glVertex4hNV)(unsigned short x,unsigned short y,unsigned short z,unsigned short w);

static void APIENTRY init_ov_glVertex4hNV(unsigned short x,unsigned short y,unsigned short z,unsigned short w)
{
	void *f;
	f=(void*)getfunc("glVertex4hNV");
	ov_glVertex4hNV=f;
	ov_glVertex4hNV(x,y,z,w);
}
void (APIENTRY *ov_glVertex4hNV)(unsigned short x,unsigned short y,unsigned short z,unsigned short w)=init_ov_glVertex4hNV;

void __gl_ml_glVertex4hNV(value v0,value v1,value v2,value v3)
{
	unsigned short l0=Int_val(v0);
	unsigned short l1=Int_val(v1);
	unsigned short l2=Int_val(v2);
	unsigned short l3=Int_val(v3);
	ov_glVertex4hNV(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glVertex3hvNV)(unsigned short* v);

static void APIENTRY init_ov_glVertex3hvNV(unsigned short* v)
{
	void *f;
	f=(void*)getfunc("glVertex3hvNV");
	ov_glVertex3hvNV=f;
	ov_glVertex3hvNV(v);
}
void (APIENTRY *ov_glVertex3hvNV)(unsigned short* v)=init_ov_glVertex3hvNV;

void __gl_ml_glVertex3hvNV(value v0)
{
	unsigned short* l0=(unsigned short*)(gl__ba__(v0));
	ov_glVertex3hvNV(l0);
}
extern void (APIENTRY *ov_glVertex3hNV)(unsigned short x,unsigned short y,unsigned short z);

static void APIENTRY init_ov_glVertex3hNV(unsigned short x,unsigned short y,unsigned short z)
{
	void *f;
	f=(void*)getfunc("glVertex3hNV");
	ov_glVertex3hNV=f;
	ov_glVertex3hNV(x,y,z);
}
void (APIENTRY *ov_glVertex3hNV)(unsigned short x,unsigned short y,unsigned short z)=init_ov_glVertex3hNV;

void __gl_ml_glVertex3hNV(value v0,value v1,value v2)
{
	unsigned short l0=Int_val(v0);
	unsigned short l1=Int_val(v1);
	unsigned short l2=Int_val(v2);
	ov_glVertex3hNV(l0,l1,l2);
}
extern void (APIENTRY *ov_glVertex2hvNV)(unsigned short* v);

static void APIENTRY init_ov_glVertex2hvNV(unsigned short* v)
{
	void *f;
	f=(void*)getfunc("glVertex2hvNV");
	ov_glVertex2hvNV=f;
	ov_glVertex2hvNV(v);
}
void (APIENTRY *ov_glVertex2hvNV)(unsigned short* v)=init_ov_glVertex2hvNV;

void __gl_ml_glVertex2hvNV(value v0)
{
	unsigned short* l0=(unsigned short*)(gl__ba__(v0));
	ov_glVertex2hvNV(l0);
}
extern void (APIENTRY *ov_glVertex2hNV)(unsigned short x,unsigned short y);

static void APIENTRY init_ov_glVertex2hNV(unsigned short x,unsigned short y)
{
	void *f;
	f=(void*)getfunc("glVertex2hNV");
	ov_glVertex2hNV=f;
	ov_glVertex2hNV(x,y);
}
void (APIENTRY *ov_glVertex2hNV)(unsigned short x,unsigned short y)=init_ov_glVertex2hNV;

void __gl_ml_glVertex2hNV(value v0,value v1)
{
	unsigned short l0=Int_val(v0);
	unsigned short l1=Int_val(v1);
	ov_glVertex2hNV(l0,l1);
}
extern void (APIENTRY *ov_glFlushPixelDataRangeNV)(unsigned int target);

static void APIENTRY init_ov_glFlushPixelDataRangeNV(unsigned int target)
{
	void *f;
	f=(void*)getfunc("glFlushPixelDataRangeNV");
	ov_glFlushPixelDataRangeNV=f;
	ov_glFlushPixelDataRangeNV(target);
}
void (APIENTRY *ov_glFlushPixelDataRangeNV)(unsigned int target)=init_ov_glFlushPixelDataRangeNV;

void __gl_ml_glFlushPixelDataRangeNV(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glFlushPixelDataRangeNV(l0);
}
extern void (APIENTRY *ov_glPixelDataRangeNV)(unsigned int target,signed int length,void* pointer);

static void APIENTRY init_ov_glPixelDataRangeNV(unsigned int target,signed int length,void* pointer)
{
	void *f;
	f=(void*)getfunc("glPixelDataRangeNV");
	ov_glPixelDataRangeNV=f;
	ov_glPixelDataRangeNV(target,length,pointer);
}
void (APIENTRY *ov_glPixelDataRangeNV)(unsigned int target,signed int length,void* pointer)=init_ov_glPixelDataRangeNV;

void __gl_ml_glPixelDataRangeNV(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	void* l2=gl__ba__(v2);
	ov_glPixelDataRangeNV(l0,l1,l2);
}
extern void (APIENTRY *ov_glPrimitiveRestartIndexNV)(unsigned int index);

static void APIENTRY init_ov_glPrimitiveRestartIndexNV(unsigned int index)
{
	void *f;
	f=(void*)getfunc("glPrimitiveRestartIndexNV");
	ov_glPrimitiveRestartIndexNV=f;
	ov_glPrimitiveRestartIndexNV(index);
}
void (APIENTRY *ov_glPrimitiveRestartIndexNV)(unsigned int index)=init_ov_glPrimitiveRestartIndexNV;

void __gl_ml_glPrimitiveRestartIndexNV(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glPrimitiveRestartIndexNV(l0);
}
extern void (APIENTRY *ov_glPrimitiveRestartNV)();

static void APIENTRY init_ov_glPrimitiveRestartNV()
{
	void *f;
	f=(void*)getfunc("glPrimitiveRestartNV");
	ov_glPrimitiveRestartNV=f;
	ov_glPrimitiveRestartNV();
}
void (APIENTRY *ov_glPrimitiveRestartNV)()=init_ov_glPrimitiveRestartNV;

void __gl_ml_glPrimitiveRestartNV()
{
	ov_glPrimitiveRestartNV();
}
extern void (APIENTRY *ov_glActiveStencilFaceNV)(unsigned int face);

static void APIENTRY init_ov_glActiveStencilFaceNV(unsigned int face)
{
	void *f;
	f=(void*)getfunc("glActiveStencilFaceNV");
	ov_glActiveStencilFaceNV=f;
	ov_glActiveStencilFaceNV(face);
}
void (APIENTRY *ov_glActiveStencilFaceNV)(unsigned int face)=init_ov_glActiveStencilFaceNV;

void __gl_ml_glActiveStencilFaceNV(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glActiveStencilFaceNV(l0);
}
extern void (APIENTRY *ov_glGetDetailTexFuncSGIS)(unsigned int param0,float* param1);

static void APIENTRY init_ov_glGetDetailTexFuncSGIS(unsigned int param0,float* param1)
{
	void *f;
	f=(void*)getfunc("glGetDetailTexFuncSGIS");
	ov_glGetDetailTexFuncSGIS=f;
	ov_glGetDetailTexFuncSGIS(param0,param1);
}
void (APIENTRY *ov_glGetDetailTexFuncSGIS)(unsigned int param0,float* param1)=init_ov_glGetDetailTexFuncSGIS;

void __gl_ml_glGetDetailTexFuncSGIS(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float* l1=(float*)(gl__ba__(v1));
	ov_glGetDetailTexFuncSGIS(l0,l1);
}
extern void (APIENTRY *ov_glDetailTexFuncSGIS)(unsigned int param0,signed int param1,float* param2);

static void APIENTRY init_ov_glDetailTexFuncSGIS(unsigned int param0,signed int param1,float* param2)
{
	void *f;
	f=(void*)getfunc("glDetailTexFuncSGIS");
	ov_glDetailTexFuncSGIS=f;
	ov_glDetailTexFuncSGIS(param0,param1,param2);
}
void (APIENTRY *ov_glDetailTexFuncSGIS)(unsigned int param0,signed int param1,float* param2)=init_ov_glDetailTexFuncSGIS;

void __gl_ml_glDetailTexFuncSGIS(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glDetailTexFuncSGIS(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetFogFuncSGIS)(float* param0);

static void APIENTRY init_ov_glGetFogFuncSGIS(float* param0)
{
	void *f;
	f=(void*)getfunc("glGetFogFuncSGIS");
	ov_glGetFogFuncSGIS=f;
	ov_glGetFogFuncSGIS(param0);
}
void (APIENTRY *ov_glGetFogFuncSGIS)(float* param0)=init_ov_glGetFogFuncSGIS;

void __gl_ml_glGetFogFuncSGIS(value v0)
{
	float* l0=(float*)(gl__ba__(v0));
	ov_glGetFogFuncSGIS(l0);
}
extern void (APIENTRY *ov_glFogFuncSGIS)(signed int param0,float* param1);

static void APIENTRY init_ov_glFogFuncSGIS(signed int param0,float* param1)
{
	void *f;
	f=(void*)getfunc("glFogFuncSGIS");
	ov_glFogFuncSGIS=f;
	ov_glFogFuncSGIS(param0,param1);
}
void (APIENTRY *ov_glFogFuncSGIS)(signed int param0,float* param1)=init_ov_glFogFuncSGIS;

void __gl_ml_glFogFuncSGIS(value v0,value v1)
{
	signed int l0=Int_val(v0);
	float* l1=(float*)(gl__ba__(v1));
	ov_glFogFuncSGIS(l0,l1);
}
extern void (APIENTRY *ov_glSamplePatternSGIS)(unsigned int param0);

static void APIENTRY init_ov_glSamplePatternSGIS(unsigned int param0)
{
	void *f;
	f=(void*)getfunc("glSamplePatternSGIS");
	ov_glSamplePatternSGIS=f;
	ov_glSamplePatternSGIS(param0);
}
void (APIENTRY *ov_glSamplePatternSGIS)(unsigned int param0)=init_ov_glSamplePatternSGIS;

void __gl_ml_glSamplePatternSGIS(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glSamplePatternSGIS(l0);
}
extern void (APIENTRY *ov_glSampleMaskSGIS)(float param0,unsigned char param1);

static void APIENTRY init_ov_glSampleMaskSGIS(float param0,unsigned char param1)
{
	void *f;
	f=(void*)getfunc("glSampleMaskSGIS");
	ov_glSampleMaskSGIS=f;
	ov_glSampleMaskSGIS(param0,param1);
}
void (APIENTRY *ov_glSampleMaskSGIS)(float param0,unsigned char param1)=init_ov_glSampleMaskSGIS;

void __gl_ml_glSampleMaskSGIS(value v0,value v1)
{
	float l0=Double_val(v0);
	unsigned char l1=Int_val(v1);
	ov_glSampleMaskSGIS(l0,l1);
}
extern void (APIENTRY *ov_glGetPixelTexGenParameterfvSGIS)(unsigned int param0,float* param1);

static void APIENTRY init_ov_glGetPixelTexGenParameterfvSGIS(unsigned int param0,float* param1)
{
	void *f;
	f=(void*)getfunc("glGetPixelTexGenParameterfvSGIS");
	ov_glGetPixelTexGenParameterfvSGIS=f;
	ov_glGetPixelTexGenParameterfvSGIS(param0,param1);
}
void (APIENTRY *ov_glGetPixelTexGenParameterfvSGIS)(unsigned int param0,float* param1)=init_ov_glGetPixelTexGenParameterfvSGIS;

void __gl_ml_glGetPixelTexGenParameterfvSGIS(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float* l1=(float*)(gl__ba__(v1));
	ov_glGetPixelTexGenParameterfvSGIS(l0,l1);
}
extern void (APIENTRY *ov_glGetPixelTexGenParameterivSGIS)(unsigned int param0,signed int* param1);

static void APIENTRY init_ov_glGetPixelTexGenParameterivSGIS(unsigned int param0,signed int* param1)
{
	void *f;
	f=(void*)getfunc("glGetPixelTexGenParameterivSGIS");
	ov_glGetPixelTexGenParameterivSGIS=f;
	ov_glGetPixelTexGenParameterivSGIS(param0,param1);
}
void (APIENTRY *ov_glGetPixelTexGenParameterivSGIS)(unsigned int param0,signed int* param1)=init_ov_glGetPixelTexGenParameterivSGIS;

void __gl_ml_glGetPixelTexGenParameterivSGIS(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed int* l1=(signed int*)(gl__ba__(v1));
	ov_glGetPixelTexGenParameterivSGIS(l0,l1);
}
extern void (APIENTRY *ov_glPixelTexGenParameterfvSGIS)(unsigned int param0,float* param1);

static void APIENTRY init_ov_glPixelTexGenParameterfvSGIS(unsigned int param0,float* param1)
{
	void *f;
	f=(void*)getfunc("glPixelTexGenParameterfvSGIS");
	ov_glPixelTexGenParameterfvSGIS=f;
	ov_glPixelTexGenParameterfvSGIS(param0,param1);
}
void (APIENTRY *ov_glPixelTexGenParameterfvSGIS)(unsigned int param0,float* param1)=init_ov_glPixelTexGenParameterfvSGIS;

void __gl_ml_glPixelTexGenParameterfvSGIS(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float* l1=(float*)(gl__ba__(v1));
	ov_glPixelTexGenParameterfvSGIS(l0,l1);
}
extern void (APIENTRY *ov_glPixelTexGenParameterfSGIS)(unsigned int param0,float param1);

static void APIENTRY init_ov_glPixelTexGenParameterfSGIS(unsigned int param0,float param1)
{
	void *f;
	f=(void*)getfunc("glPixelTexGenParameterfSGIS");
	ov_glPixelTexGenParameterfSGIS=f;
	ov_glPixelTexGenParameterfSGIS(param0,param1);
}
void (APIENTRY *ov_glPixelTexGenParameterfSGIS)(unsigned int param0,float param1)=init_ov_glPixelTexGenParameterfSGIS;

void __gl_ml_glPixelTexGenParameterfSGIS(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float l1=Double_val(v1);
	ov_glPixelTexGenParameterfSGIS(l0,l1);
}
extern void (APIENTRY *ov_glPixelTexGenParameterivSGIS)(unsigned int param0,signed int* param1);

static void APIENTRY init_ov_glPixelTexGenParameterivSGIS(unsigned int param0,signed int* param1)
{
	void *f;
	f=(void*)getfunc("glPixelTexGenParameterivSGIS");
	ov_glPixelTexGenParameterivSGIS=f;
	ov_glPixelTexGenParameterivSGIS(param0,param1);
}
void (APIENTRY *ov_glPixelTexGenParameterivSGIS)(unsigned int param0,signed int* param1)=init_ov_glPixelTexGenParameterivSGIS;

void __gl_ml_glPixelTexGenParameterivSGIS(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed int* l1=(signed int*)(gl__ba__(v1));
	ov_glPixelTexGenParameterivSGIS(l0,l1);
}
extern void (APIENTRY *ov_glPixelTexGenParameteriSGIS)(unsigned int param0,signed int param1);

static void APIENTRY init_ov_glPixelTexGenParameteriSGIS(unsigned int param0,signed int param1)
{
	void *f;
	f=(void*)getfunc("glPixelTexGenParameteriSGIS");
	ov_glPixelTexGenParameteriSGIS=f;
	ov_glPixelTexGenParameteriSGIS(param0,param1);
}
void (APIENTRY *ov_glPixelTexGenParameteriSGIS)(unsigned int param0,signed int param1)=init_ov_glPixelTexGenParameteriSGIS;

void __gl_ml_glPixelTexGenParameteriSGIS(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	ov_glPixelTexGenParameteriSGIS(l0,l1);
}
extern void (APIENTRY *ov_glPointParameterfvSGIS)(unsigned int param0,float* param1);

static void APIENTRY init_ov_glPointParameterfvSGIS(unsigned int param0,float* param1)
{
	void *f;
	f=(void*)getfunc("glPointParameterfvSGIS");
	ov_glPointParameterfvSGIS=f;
	ov_glPointParameterfvSGIS(param0,param1);
}
void (APIENTRY *ov_glPointParameterfvSGIS)(unsigned int param0,float* param1)=init_ov_glPointParameterfvSGIS;

void __gl_ml_glPointParameterfvSGIS(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float* l1=(float*)(gl__ba__(v1));
	ov_glPointParameterfvSGIS(l0,l1);
}
extern void (APIENTRY *ov_glPointParameterfSGIS)(unsigned int param0,float param1);

static void APIENTRY init_ov_glPointParameterfSGIS(unsigned int param0,float param1)
{
	void *f;
	f=(void*)getfunc("glPointParameterfSGIS");
	ov_glPointParameterfSGIS=f;
	ov_glPointParameterfSGIS(param0,param1);
}
void (APIENTRY *ov_glPointParameterfSGIS)(unsigned int param0,float param1)=init_ov_glPointParameterfSGIS;

void __gl_ml_glPointParameterfSGIS(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float l1=Double_val(v1);
	ov_glPointParameterfSGIS(l0,l1);
}
extern void (APIENTRY *ov_glGetSharpenTexFuncSGIS)(unsigned int param0,float* param1);

static void APIENTRY init_ov_glGetSharpenTexFuncSGIS(unsigned int param0,float* param1)
{
	void *f;
	f=(void*)getfunc("glGetSharpenTexFuncSGIS");
	ov_glGetSharpenTexFuncSGIS=f;
	ov_glGetSharpenTexFuncSGIS(param0,param1);
}
void (APIENTRY *ov_glGetSharpenTexFuncSGIS)(unsigned int param0,float* param1)=init_ov_glGetSharpenTexFuncSGIS;

void __gl_ml_glGetSharpenTexFuncSGIS(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float* l1=(float*)(gl__ba__(v1));
	ov_glGetSharpenTexFuncSGIS(l0,l1);
}
extern void (APIENTRY *ov_glSharpenTexFuncSGIS)(unsigned int param0,signed int param1,float* param2);

static void APIENTRY init_ov_glSharpenTexFuncSGIS(unsigned int param0,signed int param1,float* param2)
{
	void *f;
	f=(void*)getfunc("glSharpenTexFuncSGIS");
	ov_glSharpenTexFuncSGIS=f;
	ov_glSharpenTexFuncSGIS(param0,param1,param2);
}
void (APIENTRY *ov_glSharpenTexFuncSGIS)(unsigned int param0,signed int param1,float* param2)=init_ov_glSharpenTexFuncSGIS;

void __gl_ml_glSharpenTexFuncSGIS(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glSharpenTexFuncSGIS(l0,l1,l2);
}
extern void (APIENTRY *ov_glTexSubImage4DSGIS)(unsigned int param0,signed int param1,signed int param2,signed int param3,signed int param4,signed int param5,signed int param6,signed int param7,signed int param8,signed int param9,unsigned int param10,unsigned int param11,void* param12);

static void APIENTRY init_ov_glTexSubImage4DSGIS(unsigned int param0,signed int param1,signed int param2,signed int param3,signed int param4,signed int param5,signed int param6,signed int param7,signed int param8,signed int param9,unsigned int param10,unsigned int param11,void* param12)
{
	void *f;
	f=(void*)getfunc("glTexSubImage4DSGIS");
	ov_glTexSubImage4DSGIS=f;
	ov_glTexSubImage4DSGIS(param0,param1,param2,param3,param4,param5,param6,param7,param8,param9,param10,param11,param12);
}
void (APIENTRY *ov_glTexSubImage4DSGIS)(unsigned int param0,signed int param1,signed int param2,signed int param3,signed int param4,signed int param5,signed int param6,signed int param7,signed int param8,signed int param9,unsigned int param10,unsigned int param11,void* param12)=init_ov_glTexSubImage4DSGIS;

void __gl_ml_glTexSubImage4DSGIS(value v0,value v1,value v2,value v3,value v4,value v5,value v6,value v7,value v8,value v9,value v10,value v11,value v12)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	signed int l4=Int_val(v4);
	signed int l5=Int_val(v5);
	signed int l6=Int_val(v6);
	signed int l7=Int_val(v7);
	signed int l8=Int_val(v8);
	signed int l9=Int_val(v9);
	unsigned int l10=Int_val(v10);
	unsigned int l11=Int_val(v11);
	void* l12=gl__ba__(v12);
	ov_glTexSubImage4DSGIS(l0,l1,l2,l3,l4,l5,l6,l7,l8,l9,l10,l11,l12);
}

void byte___gl_ml_glTexSubImage4DSGIS(value * argv,int argn)
{
	__gl_ml_glTexSubImage4DSGIS(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6],argv[7],argv[8],argv[9],argv[10],argv[11],argv[12]);
}
extern void (APIENTRY *ov_glTexImage4DSGIS)(unsigned int param0,signed int param1,unsigned int param2,signed int param3,signed int param4,signed int param5,signed int param6,signed int param7,unsigned int param8,unsigned int param9,void* param10);

static void APIENTRY init_ov_glTexImage4DSGIS(unsigned int param0,signed int param1,unsigned int param2,signed int param3,signed int param4,signed int param5,signed int param6,signed int param7,unsigned int param8,unsigned int param9,void* param10)
{
	void *f;
	f=(void*)getfunc("glTexImage4DSGIS");
	ov_glTexImage4DSGIS=f;
	ov_glTexImage4DSGIS(param0,param1,param2,param3,param4,param5,param6,param7,param8,param9,param10);
}
void (APIENTRY *ov_glTexImage4DSGIS)(unsigned int param0,signed int param1,unsigned int param2,signed int param3,signed int param4,signed int param5,signed int param6,signed int param7,unsigned int param8,unsigned int param9,void* param10)=init_ov_glTexImage4DSGIS;

void __gl_ml_glTexImage4DSGIS(value v0,value v1,value v2,value v3,value v4,value v5,value v6,value v7,value v8,value v9,value v10)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	signed int l4=Int_val(v4);
	signed int l5=Int_val(v5);
	signed int l6=Int_val(v6);
	signed int l7=Int_val(v7);
	unsigned int l8=Int_val(v8);
	unsigned int l9=Int_val(v9);
	void* l10=gl__ba__(v10);
	ov_glTexImage4DSGIS(l0,l1,l2,l3,l4,l5,l6,l7,l8,l9,l10);
}

void byte___gl_ml_glTexImage4DSGIS(value * argv,int argn)
{
	__gl_ml_glTexImage4DSGIS(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6],argv[7],argv[8],argv[9],argv[10]);
}
extern void (APIENTRY *ov_glTextureColorMaskSGIS)(unsigned char param0,unsigned char param1,unsigned char param2,unsigned char param3);

static void APIENTRY init_ov_glTextureColorMaskSGIS(unsigned char param0,unsigned char param1,unsigned char param2,unsigned char param3)
{
	void *f;
	f=(void*)getfunc("glTextureColorMaskSGIS");
	ov_glTextureColorMaskSGIS=f;
	ov_glTextureColorMaskSGIS(param0,param1,param2,param3);
}
void (APIENTRY *ov_glTextureColorMaskSGIS)(unsigned char param0,unsigned char param1,unsigned char param2,unsigned char param3)=init_ov_glTextureColorMaskSGIS;

void __gl_ml_glTextureColorMaskSGIS(value v0,value v1,value v2,value v3)
{
	unsigned char l0=Int_val(v0);
	unsigned char l1=Int_val(v1);
	unsigned char l2=Int_val(v2);
	unsigned char l3=Int_val(v3);
	ov_glTextureColorMaskSGIS(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glTexFilterFuncSGIS)(unsigned int param0,unsigned int param1,signed int param2,float* param3);

static void APIENTRY init_ov_glTexFilterFuncSGIS(unsigned int param0,unsigned int param1,signed int param2,float* param3)
{
	void *f;
	f=(void*)getfunc("glTexFilterFuncSGIS");
	ov_glTexFilterFuncSGIS=f;
	ov_glTexFilterFuncSGIS(param0,param1,param2,param3);
}
void (APIENTRY *ov_glTexFilterFuncSGIS)(unsigned int param0,unsigned int param1,signed int param2,float* param3)=init_ov_glTexFilterFuncSGIS;

void __gl_ml_glTexFilterFuncSGIS(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	float* l3=(float*)(gl__ba__(v3));
	ov_glTexFilterFuncSGIS(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glGetTexFilterFuncSGIS)(unsigned int param0,unsigned int param1,float* param2);

static void APIENTRY init_ov_glGetTexFilterFuncSGIS(unsigned int param0,unsigned int param1,float* param2)
{
	void *f;
	f=(void*)getfunc("glGetTexFilterFuncSGIS");
	ov_glGetTexFilterFuncSGIS=f;
	ov_glGetTexFilterFuncSGIS(param0,param1,param2);
}
void (APIENTRY *ov_glGetTexFilterFuncSGIS)(unsigned int param0,unsigned int param1,float* param2)=init_ov_glGetTexFilterFuncSGIS;

void __gl_ml_glGetTexFilterFuncSGIS(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glGetTexFilterFuncSGIS(l0,l1,l2);
}
extern unsigned char (APIENTRY *ov_glIsAsyncMarkerSGIX)(unsigned int param0);

static unsigned char APIENTRY init_ov_glIsAsyncMarkerSGIX(unsigned int param0)
{
	void *f;
	f=(void*)getfunc("glIsAsyncMarkerSGIX");
	ov_glIsAsyncMarkerSGIX=f;
	return ov_glIsAsyncMarkerSGIX(param0);
}
unsigned char (APIENTRY *ov_glIsAsyncMarkerSGIX)(unsigned int param0)=init_ov_glIsAsyncMarkerSGIX;

value __gl_ml_glIsAsyncMarkerSGIX(value v0)
{
	unsigned int l0=Int_val(v0);
	unsigned char r=ov_glIsAsyncMarkerSGIX(l0);
	return(Val_int(r));
}
extern void (APIENTRY *ov_glDeleteAsyncMarkersSGIX)(unsigned int param0,signed int param1);

static void APIENTRY init_ov_glDeleteAsyncMarkersSGIX(unsigned int param0,signed int param1)
{
	void *f;
	f=(void*)getfunc("glDeleteAsyncMarkersSGIX");
	ov_glDeleteAsyncMarkersSGIX=f;
	ov_glDeleteAsyncMarkersSGIX(param0,param1);
}
void (APIENTRY *ov_glDeleteAsyncMarkersSGIX)(unsigned int param0,signed int param1)=init_ov_glDeleteAsyncMarkersSGIX;

void __gl_ml_glDeleteAsyncMarkersSGIX(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	ov_glDeleteAsyncMarkersSGIX(l0,l1);
}
extern unsigned int (APIENTRY *ov_glGenAsyncMarkersSGIX)(signed int param0);

static unsigned int APIENTRY init_ov_glGenAsyncMarkersSGIX(signed int param0)
{
	void *f;
	f=(void*)getfunc("glGenAsyncMarkersSGIX");
	ov_glGenAsyncMarkersSGIX=f;
	return ov_glGenAsyncMarkersSGIX(param0);
}
unsigned int (APIENTRY *ov_glGenAsyncMarkersSGIX)(signed int param0)=init_ov_glGenAsyncMarkersSGIX;

value __gl_ml_glGenAsyncMarkersSGIX(value v0)
{
	signed int l0=Int_val(v0);
	unsigned int r=ov_glGenAsyncMarkersSGIX(l0);
	if(((unsigned int)(r)) & 0x40000000) failwith("glGenAsyncMarkersSGIX int out of range");
	return(Val_int(r));
}
extern signed int (APIENTRY *ov_glPollAsyncSGIX)(unsigned int* param0);

static signed int APIENTRY init_ov_glPollAsyncSGIX(unsigned int* param0)
{
	void *f;
	f=(void*)getfunc("glPollAsyncSGIX");
	ov_glPollAsyncSGIX=f;
	return ov_glPollAsyncSGIX(param0);
}
signed int (APIENTRY *ov_glPollAsyncSGIX)(unsigned int* param0)=init_ov_glPollAsyncSGIX;

value __gl_ml_glPollAsyncSGIX(value v0)
{
	unsigned int* l0=(unsigned int*)(gl__ba__(v0));
	signed int r=ov_glPollAsyncSGIX(l0);
	if(((unsigned int)(r)) & 0x40000000) failwith("glPollAsyncSGIX int out of range");
	return(Val_int(r));
}
extern signed int (APIENTRY *ov_glFinishAsyncSGIX)(unsigned int* param0);

static signed int APIENTRY init_ov_glFinishAsyncSGIX(unsigned int* param0)
{
	void *f;
	f=(void*)getfunc("glFinishAsyncSGIX");
	ov_glFinishAsyncSGIX=f;
	return ov_glFinishAsyncSGIX(param0);
}
signed int (APIENTRY *ov_glFinishAsyncSGIX)(unsigned int* param0)=init_ov_glFinishAsyncSGIX;

value __gl_ml_glFinishAsyncSGIX(value v0)
{
	unsigned int* l0=(unsigned int*)(gl__ba__(v0));
	signed int r=ov_glFinishAsyncSGIX(l0);
	if(((unsigned int)(r)) & 0x40000000) failwith("glFinishAsyncSGIX int out of range");
	return(Val_int(r));
}
extern void (APIENTRY *ov_glAsyncMarkerSGIX)(unsigned int param0);

static void APIENTRY init_ov_glAsyncMarkerSGIX(unsigned int param0)
{
	void *f;
	f=(void*)getfunc("glAsyncMarkerSGIX");
	ov_glAsyncMarkerSGIX=f;
	ov_glAsyncMarkerSGIX(param0);
}
void (APIENTRY *ov_glAsyncMarkerSGIX)(unsigned int param0)=init_ov_glAsyncMarkerSGIX;

void __gl_ml_glAsyncMarkerSGIX(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glAsyncMarkerSGIX(l0);
}
extern void (APIENTRY *ov_glFlushRasterSGIX)();

static void APIENTRY init_ov_glFlushRasterSGIX()
{
	void *f;
	f=(void*)getfunc("glFlushRasterSGIX");
	ov_glFlushRasterSGIX=f;
	ov_glFlushRasterSGIX();
}
void (APIENTRY *ov_glFlushRasterSGIX)()=init_ov_glFlushRasterSGIX;

void __gl_ml_glFlushRasterSGIX()
{
	ov_glFlushRasterSGIX();
}
extern void (APIENTRY *ov_glLightEnviSGIX)(unsigned int param0,signed int param1);

static void APIENTRY init_ov_glLightEnviSGIX(unsigned int param0,signed int param1)
{
	void *f;
	f=(void*)getfunc("glLightEnviSGIX");
	ov_glLightEnviSGIX=f;
	ov_glLightEnviSGIX(param0,param1);
}
void (APIENTRY *ov_glLightEnviSGIX)(unsigned int param0,signed int param1)=init_ov_glLightEnviSGIX;

void __gl_ml_glLightEnviSGIX(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	ov_glLightEnviSGIX(l0,l1);
}
extern void (APIENTRY *ov_glGetFragmentMaterialivSGIX)(unsigned int param0,unsigned int param1,signed int* param2);

static void APIENTRY init_ov_glGetFragmentMaterialivSGIX(unsigned int param0,unsigned int param1,signed int* param2)
{
	void *f;
	f=(void*)getfunc("glGetFragmentMaterialivSGIX");
	ov_glGetFragmentMaterialivSGIX=f;
	ov_glGetFragmentMaterialivSGIX(param0,param1,param2);
}
void (APIENTRY *ov_glGetFragmentMaterialivSGIX)(unsigned int param0,unsigned int param1,signed int* param2)=init_ov_glGetFragmentMaterialivSGIX;

void __gl_ml_glGetFragmentMaterialivSGIX(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int* l2=(signed int*)(gl__ba__(v2));
	ov_glGetFragmentMaterialivSGIX(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetFragmentMaterialfvSGIX)(unsigned int param0,unsigned int param1,float* param2);

static void APIENTRY init_ov_glGetFragmentMaterialfvSGIX(unsigned int param0,unsigned int param1,float* param2)
{
	void *f;
	f=(void*)getfunc("glGetFragmentMaterialfvSGIX");
	ov_glGetFragmentMaterialfvSGIX=f;
	ov_glGetFragmentMaterialfvSGIX(param0,param1,param2);
}
void (APIENTRY *ov_glGetFragmentMaterialfvSGIX)(unsigned int param0,unsigned int param1,float* param2)=init_ov_glGetFragmentMaterialfvSGIX;

void __gl_ml_glGetFragmentMaterialfvSGIX(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glGetFragmentMaterialfvSGIX(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetFragmentLightivSGIX)(unsigned int param0,unsigned int param1,signed int* param2);

static void APIENTRY init_ov_glGetFragmentLightivSGIX(unsigned int param0,unsigned int param1,signed int* param2)
{
	void *f;
	f=(void*)getfunc("glGetFragmentLightivSGIX");
	ov_glGetFragmentLightivSGIX=f;
	ov_glGetFragmentLightivSGIX(param0,param1,param2);
}
void (APIENTRY *ov_glGetFragmentLightivSGIX)(unsigned int param0,unsigned int param1,signed int* param2)=init_ov_glGetFragmentLightivSGIX;

void __gl_ml_glGetFragmentLightivSGIX(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int* l2=(signed int*)(gl__ba__(v2));
	ov_glGetFragmentLightivSGIX(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetFragmentLightfvSGIX)(unsigned int param0,unsigned int param1,float* param2);

static void APIENTRY init_ov_glGetFragmentLightfvSGIX(unsigned int param0,unsigned int param1,float* param2)
{
	void *f;
	f=(void*)getfunc("glGetFragmentLightfvSGIX");
	ov_glGetFragmentLightfvSGIX=f;
	ov_glGetFragmentLightfvSGIX(param0,param1,param2);
}
void (APIENTRY *ov_glGetFragmentLightfvSGIX)(unsigned int param0,unsigned int param1,float* param2)=init_ov_glGetFragmentLightfvSGIX;

void __gl_ml_glGetFragmentLightfvSGIX(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glGetFragmentLightfvSGIX(l0,l1,l2);
}
extern void (APIENTRY *ov_glFragmentMaterialivSGIX)(unsigned int param0,unsigned int param1,signed int* param2);

static void APIENTRY init_ov_glFragmentMaterialivSGIX(unsigned int param0,unsigned int param1,signed int* param2)
{
	void *f;
	f=(void*)getfunc("glFragmentMaterialivSGIX");
	ov_glFragmentMaterialivSGIX=f;
	ov_glFragmentMaterialivSGIX(param0,param1,param2);
}
void (APIENTRY *ov_glFragmentMaterialivSGIX)(unsigned int param0,unsigned int param1,signed int* param2)=init_ov_glFragmentMaterialivSGIX;

void __gl_ml_glFragmentMaterialivSGIX(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int* l2=(signed int*)(gl__ba__(v2));
	ov_glFragmentMaterialivSGIX(l0,l1,l2);
}
extern void (APIENTRY *ov_glFragmentMaterialiSGIX)(unsigned int param0,unsigned int param1,signed int param2);

static void APIENTRY init_ov_glFragmentMaterialiSGIX(unsigned int param0,unsigned int param1,signed int param2)
{
	void *f;
	f=(void*)getfunc("glFragmentMaterialiSGIX");
	ov_glFragmentMaterialiSGIX=f;
	ov_glFragmentMaterialiSGIX(param0,param1,param2);
}
void (APIENTRY *ov_glFragmentMaterialiSGIX)(unsigned int param0,unsigned int param1,signed int param2)=init_ov_glFragmentMaterialiSGIX;

void __gl_ml_glFragmentMaterialiSGIX(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	ov_glFragmentMaterialiSGIX(l0,l1,l2);
}
extern void (APIENTRY *ov_glFragmentMaterialfvSGIX)(unsigned int param0,unsigned int param1,float* param2);

static void APIENTRY init_ov_glFragmentMaterialfvSGIX(unsigned int param0,unsigned int param1,float* param2)
{
	void *f;
	f=(void*)getfunc("glFragmentMaterialfvSGIX");
	ov_glFragmentMaterialfvSGIX=f;
	ov_glFragmentMaterialfvSGIX(param0,param1,param2);
}
void (APIENTRY *ov_glFragmentMaterialfvSGIX)(unsigned int param0,unsigned int param1,float* param2)=init_ov_glFragmentMaterialfvSGIX;

void __gl_ml_glFragmentMaterialfvSGIX(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glFragmentMaterialfvSGIX(l0,l1,l2);
}
extern void (APIENTRY *ov_glFragmentMaterialfSGIX)(unsigned int param0,unsigned int param1,float param2);

static void APIENTRY init_ov_glFragmentMaterialfSGIX(unsigned int param0,unsigned int param1,float param2)
{
	void *f;
	f=(void*)getfunc("glFragmentMaterialfSGIX");
	ov_glFragmentMaterialfSGIX=f;
	ov_glFragmentMaterialfSGIX(param0,param1,param2);
}
void (APIENTRY *ov_glFragmentMaterialfSGIX)(unsigned int param0,unsigned int param1,float param2)=init_ov_glFragmentMaterialfSGIX;

void __gl_ml_glFragmentMaterialfSGIX(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float l2=Double_val(v2);
	ov_glFragmentMaterialfSGIX(l0,l1,l2);
}
extern void (APIENTRY *ov_glFragmentLightModelivSGIX)(unsigned int param0,signed int* param1);

static void APIENTRY init_ov_glFragmentLightModelivSGIX(unsigned int param0,signed int* param1)
{
	void *f;
	f=(void*)getfunc("glFragmentLightModelivSGIX");
	ov_glFragmentLightModelivSGIX=f;
	ov_glFragmentLightModelivSGIX(param0,param1);
}
void (APIENTRY *ov_glFragmentLightModelivSGIX)(unsigned int param0,signed int* param1)=init_ov_glFragmentLightModelivSGIX;

void __gl_ml_glFragmentLightModelivSGIX(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed int* l1=(signed int*)(gl__ba__(v1));
	ov_glFragmentLightModelivSGIX(l0,l1);
}
extern void (APIENTRY *ov_glFragmentLightModeliSGIX)(unsigned int param0,signed int param1);

static void APIENTRY init_ov_glFragmentLightModeliSGIX(unsigned int param0,signed int param1)
{
	void *f;
	f=(void*)getfunc("glFragmentLightModeliSGIX");
	ov_glFragmentLightModeliSGIX=f;
	ov_glFragmentLightModeliSGIX(param0,param1);
}
void (APIENTRY *ov_glFragmentLightModeliSGIX)(unsigned int param0,signed int param1)=init_ov_glFragmentLightModeliSGIX;

void __gl_ml_glFragmentLightModeliSGIX(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	ov_glFragmentLightModeliSGIX(l0,l1);
}
extern void (APIENTRY *ov_glFragmentLightModelfvSGIX)(unsigned int param0,float* param1);

static void APIENTRY init_ov_glFragmentLightModelfvSGIX(unsigned int param0,float* param1)
{
	void *f;
	f=(void*)getfunc("glFragmentLightModelfvSGIX");
	ov_glFragmentLightModelfvSGIX=f;
	ov_glFragmentLightModelfvSGIX(param0,param1);
}
void (APIENTRY *ov_glFragmentLightModelfvSGIX)(unsigned int param0,float* param1)=init_ov_glFragmentLightModelfvSGIX;

void __gl_ml_glFragmentLightModelfvSGIX(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float* l1=(float*)(gl__ba__(v1));
	ov_glFragmentLightModelfvSGIX(l0,l1);
}
extern void (APIENTRY *ov_glFragmentLightModelfSGIX)(unsigned int param0,float param1);

static void APIENTRY init_ov_glFragmentLightModelfSGIX(unsigned int param0,float param1)
{
	void *f;
	f=(void*)getfunc("glFragmentLightModelfSGIX");
	ov_glFragmentLightModelfSGIX=f;
	ov_glFragmentLightModelfSGIX(param0,param1);
}
void (APIENTRY *ov_glFragmentLightModelfSGIX)(unsigned int param0,float param1)=init_ov_glFragmentLightModelfSGIX;

void __gl_ml_glFragmentLightModelfSGIX(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float l1=Double_val(v1);
	ov_glFragmentLightModelfSGIX(l0,l1);
}
extern void (APIENTRY *ov_glFragmentLightivSGIX)(unsigned int param0,unsigned int param1,signed int* param2);

static void APIENTRY init_ov_glFragmentLightivSGIX(unsigned int param0,unsigned int param1,signed int* param2)
{
	void *f;
	f=(void*)getfunc("glFragmentLightivSGIX");
	ov_glFragmentLightivSGIX=f;
	ov_glFragmentLightivSGIX(param0,param1,param2);
}
void (APIENTRY *ov_glFragmentLightivSGIX)(unsigned int param0,unsigned int param1,signed int* param2)=init_ov_glFragmentLightivSGIX;

void __gl_ml_glFragmentLightivSGIX(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int* l2=(signed int*)(gl__ba__(v2));
	ov_glFragmentLightivSGIX(l0,l1,l2);
}
extern void (APIENTRY *ov_glFragmentLightiSGIX)(unsigned int param0,unsigned int param1,signed int param2);

static void APIENTRY init_ov_glFragmentLightiSGIX(unsigned int param0,unsigned int param1,signed int param2)
{
	void *f;
	f=(void*)getfunc("glFragmentLightiSGIX");
	ov_glFragmentLightiSGIX=f;
	ov_glFragmentLightiSGIX(param0,param1,param2);
}
void (APIENTRY *ov_glFragmentLightiSGIX)(unsigned int param0,unsigned int param1,signed int param2)=init_ov_glFragmentLightiSGIX;

void __gl_ml_glFragmentLightiSGIX(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	ov_glFragmentLightiSGIX(l0,l1,l2);
}
extern void (APIENTRY *ov_glFragmentLightfvSGIX)(unsigned int param0,unsigned int param1,float* param2);

static void APIENTRY init_ov_glFragmentLightfvSGIX(unsigned int param0,unsigned int param1,float* param2)
{
	void *f;
	f=(void*)getfunc("glFragmentLightfvSGIX");
	ov_glFragmentLightfvSGIX=f;
	ov_glFragmentLightfvSGIX(param0,param1,param2);
}
void (APIENTRY *ov_glFragmentLightfvSGIX)(unsigned int param0,unsigned int param1,float* param2)=init_ov_glFragmentLightfvSGIX;

void __gl_ml_glFragmentLightfvSGIX(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glFragmentLightfvSGIX(l0,l1,l2);
}
extern void (APIENTRY *ov_glFragmentLightfSGIX)(unsigned int param0,unsigned int param1,float param2);

static void APIENTRY init_ov_glFragmentLightfSGIX(unsigned int param0,unsigned int param1,float param2)
{
	void *f;
	f=(void*)getfunc("glFragmentLightfSGIX");
	ov_glFragmentLightfSGIX=f;
	ov_glFragmentLightfSGIX(param0,param1,param2);
}
void (APIENTRY *ov_glFragmentLightfSGIX)(unsigned int param0,unsigned int param1,float param2)=init_ov_glFragmentLightfSGIX;

void __gl_ml_glFragmentLightfSGIX(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float l2=Double_val(v2);
	ov_glFragmentLightfSGIX(l0,l1,l2);
}
extern void (APIENTRY *ov_glFragmentColorMaterialSGIX)(unsigned int param0,unsigned int param1);

static void APIENTRY init_ov_glFragmentColorMaterialSGIX(unsigned int param0,unsigned int param1)
{
	void *f;
	f=(void*)getfunc("glFragmentColorMaterialSGIX");
	ov_glFragmentColorMaterialSGIX=f;
	ov_glFragmentColorMaterialSGIX(param0,param1);
}
void (APIENTRY *ov_glFragmentColorMaterialSGIX)(unsigned int param0,unsigned int param1)=init_ov_glFragmentColorMaterialSGIX;

void __gl_ml_glFragmentColorMaterialSGIX(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	ov_glFragmentColorMaterialSGIX(l0,l1);
}
extern void (APIENTRY *ov_glFrameZoomSGIX)(signed int param0);

static void APIENTRY init_ov_glFrameZoomSGIX(signed int param0)
{
	void *f;
	f=(void*)getfunc("glFrameZoomSGIX");
	ov_glFrameZoomSGIX=f;
	ov_glFrameZoomSGIX(param0);
}
void (APIENTRY *ov_glFrameZoomSGIX)(signed int param0)=init_ov_glFrameZoomSGIX;

void __gl_ml_glFrameZoomSGIX(value v0)
{
	signed int l0=Int_val(v0);
	ov_glFrameZoomSGIX(l0);
}
extern void (APIENTRY *ov_glIglooInterfaceSGIX)(unsigned int param0,void* param1);

static void APIENTRY init_ov_glIglooInterfaceSGIX(unsigned int param0,void* param1)
{
	void *f;
	f=(void*)getfunc("glIglooInterfaceSGIX");
	ov_glIglooInterfaceSGIX=f;
	ov_glIglooInterfaceSGIX(param0,param1);
}
void (APIENTRY *ov_glIglooInterfaceSGIX)(unsigned int param0,void* param1)=init_ov_glIglooInterfaceSGIX;

void __gl_ml_glIglooInterfaceSGIX(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	void* l1=gl__ba__(v1);
	ov_glIglooInterfaceSGIX(l0,l1);
}
extern void (APIENTRY *ov_glStopInstrumentsSGIX)(signed int param0);

static void APIENTRY init_ov_glStopInstrumentsSGIX(signed int param0)
{
	void *f;
	f=(void*)getfunc("glStopInstrumentsSGIX");
	ov_glStopInstrumentsSGIX=f;
	ov_glStopInstrumentsSGIX(param0);
}
void (APIENTRY *ov_glStopInstrumentsSGIX)(signed int param0)=init_ov_glStopInstrumentsSGIX;

void __gl_ml_glStopInstrumentsSGIX(value v0)
{
	signed int l0=Int_val(v0);
	ov_glStopInstrumentsSGIX(l0);
}
extern void (APIENTRY *ov_glStartInstrumentsSGIX)();

static void APIENTRY init_ov_glStartInstrumentsSGIX()
{
	void *f;
	f=(void*)getfunc("glStartInstrumentsSGIX");
	ov_glStartInstrumentsSGIX=f;
	ov_glStartInstrumentsSGIX();
}
void (APIENTRY *ov_glStartInstrumentsSGIX)()=init_ov_glStartInstrumentsSGIX;

void __gl_ml_glStartInstrumentsSGIX()
{
	ov_glStartInstrumentsSGIX();
}
extern void (APIENTRY *ov_glReadInstrumentsSGIX)(signed int param0);

static void APIENTRY init_ov_glReadInstrumentsSGIX(signed int param0)
{
	void *f;
	f=(void*)getfunc("glReadInstrumentsSGIX");
	ov_glReadInstrumentsSGIX=f;
	ov_glReadInstrumentsSGIX(param0);
}
void (APIENTRY *ov_glReadInstrumentsSGIX)(signed int param0)=init_ov_glReadInstrumentsSGIX;

void __gl_ml_glReadInstrumentsSGIX(value v0)
{
	signed int l0=Int_val(v0);
	ov_glReadInstrumentsSGIX(l0);
}
extern signed int (APIENTRY *ov_glPollInstrumentsSGIX)(signed int* param0);

static signed int APIENTRY init_ov_glPollInstrumentsSGIX(signed int* param0)
{
	void *f;
	f=(void*)getfunc("glPollInstrumentsSGIX");
	ov_glPollInstrumentsSGIX=f;
	return ov_glPollInstrumentsSGIX(param0);
}
signed int (APIENTRY *ov_glPollInstrumentsSGIX)(signed int* param0)=init_ov_glPollInstrumentsSGIX;

value __gl_ml_glPollInstrumentsSGIX(value v0)
{
	signed int* l0=(signed int*)(gl__ba__(v0));
	signed int r=ov_glPollInstrumentsSGIX(l0);
	if(((unsigned int)(r)) & 0x40000000) failwith("glPollInstrumentsSGIX int out of range");
	return(Val_int(r));
}
extern void (APIENTRY *ov_glInstrumentsBufferSGIX)(signed int param0,signed int* param1);

static void APIENTRY init_ov_glInstrumentsBufferSGIX(signed int param0,signed int* param1)
{
	void *f;
	f=(void*)getfunc("glInstrumentsBufferSGIX");
	ov_glInstrumentsBufferSGIX=f;
	ov_glInstrumentsBufferSGIX(param0,param1);
}
void (APIENTRY *ov_glInstrumentsBufferSGIX)(signed int param0,signed int* param1)=init_ov_glInstrumentsBufferSGIX;

void __gl_ml_glInstrumentsBufferSGIX(value v0,value v1)
{
	signed int l0=Int_val(v0);
	signed int* l1=(signed int*)(gl__ba__(v1));
	ov_glInstrumentsBufferSGIX(l0,l1);
}
extern signed int (APIENTRY *ov_glGetInstrumentsSGIX)();

static signed int APIENTRY init_ov_glGetInstrumentsSGIX()
{
	void *f;
	f=(void*)getfunc("glGetInstrumentsSGIX");
	ov_glGetInstrumentsSGIX=f;
	return ov_glGetInstrumentsSGIX();
}
signed int (APIENTRY *ov_glGetInstrumentsSGIX)()=init_ov_glGetInstrumentsSGIX;

value __gl_ml_glGetInstrumentsSGIX()
{
	signed int r=ov_glGetInstrumentsSGIX();
	if(((unsigned int)(r)) & 0x40000000) failwith("glGetInstrumentsSGIX int out of range");
	return(Val_int(r));
}
extern void (APIENTRY *ov_glListParameterivSGIX)(unsigned int param0,unsigned int param1,signed int* param2);

static void APIENTRY init_ov_glListParameterivSGIX(unsigned int param0,unsigned int param1,signed int* param2)
{
	void *f;
	f=(void*)getfunc("glListParameterivSGIX");
	ov_glListParameterivSGIX=f;
	ov_glListParameterivSGIX(param0,param1,param2);
}
void (APIENTRY *ov_glListParameterivSGIX)(unsigned int param0,unsigned int param1,signed int* param2)=init_ov_glListParameterivSGIX;

void __gl_ml_glListParameterivSGIX(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int* l2=(signed int*)(gl__ba__(v2));
	ov_glListParameterivSGIX(l0,l1,l2);
}
extern void (APIENTRY *ov_glListParameteriSGIX)(unsigned int param0,unsigned int param1,signed int param2);

static void APIENTRY init_ov_glListParameteriSGIX(unsigned int param0,unsigned int param1,signed int param2)
{
	void *f;
	f=(void*)getfunc("glListParameteriSGIX");
	ov_glListParameteriSGIX=f;
	ov_glListParameteriSGIX(param0,param1,param2);
}
void (APIENTRY *ov_glListParameteriSGIX)(unsigned int param0,unsigned int param1,signed int param2)=init_ov_glListParameteriSGIX;

void __gl_ml_glListParameteriSGIX(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	ov_glListParameteriSGIX(l0,l1,l2);
}
extern void (APIENTRY *ov_glListParameterfvSGIX)(unsigned int param0,unsigned int param1,float* param2);

static void APIENTRY init_ov_glListParameterfvSGIX(unsigned int param0,unsigned int param1,float* param2)
{
	void *f;
	f=(void*)getfunc("glListParameterfvSGIX");
	ov_glListParameterfvSGIX=f;
	ov_glListParameterfvSGIX(param0,param1,param2);
}
void (APIENTRY *ov_glListParameterfvSGIX)(unsigned int param0,unsigned int param1,float* param2)=init_ov_glListParameterfvSGIX;

void __gl_ml_glListParameterfvSGIX(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glListParameterfvSGIX(l0,l1,l2);
}
extern void (APIENTRY *ov_glListParameterfSGIX)(unsigned int param0,unsigned int param1,float param2);

static void APIENTRY init_ov_glListParameterfSGIX(unsigned int param0,unsigned int param1,float param2)
{
	void *f;
	f=(void*)getfunc("glListParameterfSGIX");
	ov_glListParameterfSGIX=f;
	ov_glListParameterfSGIX(param0,param1,param2);
}
void (APIENTRY *ov_glListParameterfSGIX)(unsigned int param0,unsigned int param1,float param2)=init_ov_glListParameterfSGIX;

void __gl_ml_glListParameterfSGIX(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float l2=Double_val(v2);
	ov_glListParameterfSGIX(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetListParameterivSGIX)(unsigned int param0,unsigned int param1,signed int* param2);

static void APIENTRY init_ov_glGetListParameterivSGIX(unsigned int param0,unsigned int param1,signed int* param2)
{
	void *f;
	f=(void*)getfunc("glGetListParameterivSGIX");
	ov_glGetListParameterivSGIX=f;
	ov_glGetListParameterivSGIX(param0,param1,param2);
}
void (APIENTRY *ov_glGetListParameterivSGIX)(unsigned int param0,unsigned int param1,signed int* param2)=init_ov_glGetListParameterivSGIX;

void __gl_ml_glGetListParameterivSGIX(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int* l2=(signed int*)(gl__ba__(v2));
	ov_glGetListParameterivSGIX(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetListParameterfvSGIX)(unsigned int param0,unsigned int param1,float* param2);

static void APIENTRY init_ov_glGetListParameterfvSGIX(unsigned int param0,unsigned int param1,float* param2)
{
	void *f;
	f=(void*)getfunc("glGetListParameterfvSGIX");
	ov_glGetListParameterfvSGIX=f;
	ov_glGetListParameterfvSGIX(param0,param1,param2);
}
void (APIENTRY *ov_glGetListParameterfvSGIX)(unsigned int param0,unsigned int param1,float* param2)=init_ov_glGetListParameterfvSGIX;

void __gl_ml_glGetListParameterfvSGIX(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glGetListParameterfvSGIX(l0,l1,l2);
}
extern void (APIENTRY *ov_glPixelTexGenSGIX)(unsigned int param0);

static void APIENTRY init_ov_glPixelTexGenSGIX(unsigned int param0)
{
	void *f;
	f=(void*)getfunc("glPixelTexGenSGIX");
	ov_glPixelTexGenSGIX=f;
	ov_glPixelTexGenSGIX(param0);
}
void (APIENTRY *ov_glPixelTexGenSGIX)(unsigned int param0)=init_ov_glPixelTexGenSGIX;

void __gl_ml_glPixelTexGenSGIX(value v0)
{
	unsigned int l0=Int_val(v0);
	ov_glPixelTexGenSGIX(l0);
}
extern void (APIENTRY *ov_glLoadIdentityDeformationMapSGIX)(unsigned int param0);

static void APIENTRY init_ov_glLoadIdentityDeformationMapSGIX(unsigned int param0)
{
	void *f;
	f=(void*)getfunc("glLoadIdentityDeformationMapSGIX");
	ov_glLoadIdentityDeformationMapSGIX=f;
	ov_glLoadIdentityDeformationMapSGIX(param0);
}
void (APIENTRY *ov_glLoadIdentityDeformationMapSGIX)(unsigned int param0)=init_ov_glLoadIdentityDeformationMapSGIX;
extern void (APIENTRY *ov_glDeformSGIX)(unsigned int param0);

static void APIENTRY init_ov_glDeformSGIX(unsigned int param0)
{
	void *f;
	f=(void*)getfunc("glDeformSGIX");
	ov_glDeformSGIX=f;
	ov_glDeformSGIX(param0);
}
void (APIENTRY *ov_glDeformSGIX)(unsigned int param0)=init_ov_glDeformSGIX;
extern void (APIENTRY *ov_glDeformationMap3fSGIX)(unsigned int param0,float param1,float param2,signed int param3,signed int param4,float param5,float param6,signed int param7,signed int param8,float param9,float param10,signed int param11,signed int param12,float* param13);

static void APIENTRY init_ov_glDeformationMap3fSGIX(unsigned int param0,float param1,float param2,signed int param3,signed int param4,float param5,float param6,signed int param7,signed int param8,float param9,float param10,signed int param11,signed int param12,float* param13)
{
	void *f;
	f=(void*)getfunc("glDeformationMap3fSGIX");
	ov_glDeformationMap3fSGIX=f;
	ov_glDeformationMap3fSGIX(param0,param1,param2,param3,param4,param5,param6,param7,param8,param9,param10,param11,param12,param13);
}
void (APIENTRY *ov_glDeformationMap3fSGIX)(unsigned int param0,float param1,float param2,signed int param3,signed int param4,float param5,float param6,signed int param7,signed int param8,float param9,float param10,signed int param11,signed int param12,float* param13)=init_ov_glDeformationMap3fSGIX;

void __gl_ml_glDeformationMap3fSGIX(value v0,value v1,value v2,value v3,value v4,value v5,value v6,value v7,value v8,value v9,value v10,value v11,value v12,value v13)
{
	unsigned int l0=Int_val(v0);
	float l1=Double_val(v1);
	float l2=Double_val(v2);
	signed int l3=Int_val(v3);
	signed int l4=Int_val(v4);
	float l5=Double_val(v5);
	float l6=Double_val(v6);
	signed int l7=Int_val(v7);
	signed int l8=Int_val(v8);
	float l9=Double_val(v9);
	float l10=Double_val(v10);
	signed int l11=Int_val(v11);
	signed int l12=Int_val(v12);
	float* l13=(float*)(gl__ba__(v13));
	ov_glDeformationMap3fSGIX(l0,l1,l2,l3,l4,l5,l6,l7,l8,l9,l10,l11,l12,l13);
}

void byte___gl_ml_glDeformationMap3fSGIX(value * argv,int argn)
{
	__gl_ml_glDeformationMap3fSGIX(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6],argv[7],argv[8],argv[9],argv[10],argv[11],argv[12],argv[13]);
}
extern void (APIENTRY *ov_glDeformationMap3dSGIX)(unsigned int param0,double param1,double param2,signed int param3,signed int param4,double param5,double param6,signed int param7,signed int param8,double param9,double param10,signed int param11,signed int param12,double* param13);

static void APIENTRY init_ov_glDeformationMap3dSGIX(unsigned int param0,double param1,double param2,signed int param3,signed int param4,double param5,double param6,signed int param7,signed int param8,double param9,double param10,signed int param11,signed int param12,double* param13)
{
	void *f;
	f=(void*)getfunc("glDeformationMap3dSGIX");
	ov_glDeformationMap3dSGIX=f;
	ov_glDeformationMap3dSGIX(param0,param1,param2,param3,param4,param5,param6,param7,param8,param9,param10,param11,param12,param13);
}
void (APIENTRY *ov_glDeformationMap3dSGIX)(unsigned int param0,double param1,double param2,signed int param3,signed int param4,double param5,double param6,signed int param7,signed int param8,double param9,double param10,signed int param11,signed int param12,double* param13)=init_ov_glDeformationMap3dSGIX;

void __gl_ml_glDeformationMap3dSGIX(value v0,value v1,value v2,value v3,value v4,value v5,value v6,value v7,value v8,value v9,value v10,value v11,value v12,value v13)
{
	unsigned int l0=Int_val(v0);
	double l1=Double_val(v1);
	double l2=Double_val(v2);
	signed int l3=Int_val(v3);
	signed int l4=Int_val(v4);
	double l5=Double_val(v5);
	double l6=Double_val(v6);
	signed int l7=Int_val(v7);
	signed int l8=Int_val(v8);
	double l9=Double_val(v9);
	double l10=Double_val(v10);
	signed int l11=Int_val(v11);
	signed int l12=Int_val(v12);
	double* l13=(double*)(gl__ba__(v13));
	ov_glDeformationMap3dSGIX(l0,l1,l2,l3,l4,l5,l6,l7,l8,l9,l10,l11,l12,l13);
}

void byte___gl_ml_glDeformationMap3dSGIX(value * argv,int argn)
{
	__gl_ml_glDeformationMap3dSGIX(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6],argv[7],argv[8],argv[9],argv[10],argv[11],argv[12],argv[13]);
}
extern void (APIENTRY *ov_glReferencePlaneSGIX)(double* param0);

static void APIENTRY init_ov_glReferencePlaneSGIX(double* param0)
{
	void *f;
	f=(void*)getfunc("glReferencePlaneSGIX");
	ov_glReferencePlaneSGIX=f;
	ov_glReferencePlaneSGIX(param0);
}
void (APIENTRY *ov_glReferencePlaneSGIX)(double* param0)=init_ov_glReferencePlaneSGIX;

void __gl_ml_glReferencePlaneSGIX(value v0)
{
	double* l0=(double*)(gl__ba__(v0));
	ov_glReferencePlaneSGIX(l0);
}
extern void (APIENTRY *ov_glSpriteParameterivSGIX)(unsigned int param0,signed int* param1);

static void APIENTRY init_ov_glSpriteParameterivSGIX(unsigned int param0,signed int* param1)
{
	void *f;
	f=(void*)getfunc("glSpriteParameterivSGIX");
	ov_glSpriteParameterivSGIX=f;
	ov_glSpriteParameterivSGIX(param0,param1);
}
void (APIENTRY *ov_glSpriteParameterivSGIX)(unsigned int param0,signed int* param1)=init_ov_glSpriteParameterivSGIX;

void __gl_ml_glSpriteParameterivSGIX(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed int* l1=(signed int*)(gl__ba__(v1));
	ov_glSpriteParameterivSGIX(l0,l1);
}
extern void (APIENTRY *ov_glSpriteParameteriSGIX)(unsigned int param0,signed int param1);

static void APIENTRY init_ov_glSpriteParameteriSGIX(unsigned int param0,signed int param1)
{
	void *f;
	f=(void*)getfunc("glSpriteParameteriSGIX");
	ov_glSpriteParameteriSGIX=f;
	ov_glSpriteParameteriSGIX(param0,param1);
}
void (APIENTRY *ov_glSpriteParameteriSGIX)(unsigned int param0,signed int param1)=init_ov_glSpriteParameteriSGIX;

void __gl_ml_glSpriteParameteriSGIX(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	signed int l1=Int_val(v1);
	ov_glSpriteParameteriSGIX(l0,l1);
}
extern void (APIENTRY *ov_glSpriteParameterfvSGIX)(unsigned int param0,float* param1);

static void APIENTRY init_ov_glSpriteParameterfvSGIX(unsigned int param0,float* param1)
{
	void *f;
	f=(void*)getfunc("glSpriteParameterfvSGIX");
	ov_glSpriteParameterfvSGIX=f;
	ov_glSpriteParameterfvSGIX(param0,param1);
}
void (APIENTRY *ov_glSpriteParameterfvSGIX)(unsigned int param0,float* param1)=init_ov_glSpriteParameterfvSGIX;

void __gl_ml_glSpriteParameterfvSGIX(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float* l1=(float*)(gl__ba__(v1));
	ov_glSpriteParameterfvSGIX(l0,l1);
}
extern void (APIENTRY *ov_glSpriteParameterfSGIX)(unsigned int param0,float param1);

static void APIENTRY init_ov_glSpriteParameterfSGIX(unsigned int param0,float param1)
{
	void *f;
	f=(void*)getfunc("glSpriteParameterfSGIX");
	ov_glSpriteParameterfSGIX=f;
	ov_glSpriteParameterfSGIX(param0,param1);
}
void (APIENTRY *ov_glSpriteParameterfSGIX)(unsigned int param0,float param1)=init_ov_glSpriteParameterfSGIX;

void __gl_ml_glSpriteParameterfSGIX(value v0,value v1)
{
	unsigned int l0=Int_val(v0);
	float l1=Double_val(v1);
	ov_glSpriteParameterfSGIX(l0,l1);
}
extern void (APIENTRY *ov_glTagSampleBufferSGIX)();

static void APIENTRY init_ov_glTagSampleBufferSGIX()
{
	void *f;
	f=(void*)getfunc("glTagSampleBufferSGIX");
	ov_glTagSampleBufferSGIX=f;
	ov_glTagSampleBufferSGIX();
}
void (APIENTRY *ov_glTagSampleBufferSGIX)()=init_ov_glTagSampleBufferSGIX;

void __gl_ml_glTagSampleBufferSGIX()
{
	ov_glTagSampleBufferSGIX();
}
extern void (APIENTRY *ov_glGetColorTableParameterivSGI)(unsigned int param0,unsigned int param1,signed int* param2);

static void APIENTRY init_ov_glGetColorTableParameterivSGI(unsigned int param0,unsigned int param1,signed int* param2)
{
	void *f;
	f=(void*)getfunc("glGetColorTableParameterivSGI");
	ov_glGetColorTableParameterivSGI=f;
	ov_glGetColorTableParameterivSGI(param0,param1,param2);
}
void (APIENTRY *ov_glGetColorTableParameterivSGI)(unsigned int param0,unsigned int param1,signed int* param2)=init_ov_glGetColorTableParameterivSGI;

void __gl_ml_glGetColorTableParameterivSGI(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int* l2=(signed int*)(gl__ba__(v2));
	ov_glGetColorTableParameterivSGI(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetColorTableParameterfvSGI)(unsigned int param0,unsigned int param1,float* param2);

static void APIENTRY init_ov_glGetColorTableParameterfvSGI(unsigned int param0,unsigned int param1,float* param2)
{
	void *f;
	f=(void*)getfunc("glGetColorTableParameterfvSGI");
	ov_glGetColorTableParameterfvSGI=f;
	ov_glGetColorTableParameterfvSGI(param0,param1,param2);
}
void (APIENTRY *ov_glGetColorTableParameterfvSGI)(unsigned int param0,unsigned int param1,float* param2)=init_ov_glGetColorTableParameterfvSGI;

void __gl_ml_glGetColorTableParameterfvSGI(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glGetColorTableParameterfvSGI(l0,l1,l2);
}
extern void (APIENTRY *ov_glGetColorTableSGI)(unsigned int param0,unsigned int param1,unsigned int param2,void* param3);

static void APIENTRY init_ov_glGetColorTableSGI(unsigned int param0,unsigned int param1,unsigned int param2,void* param3)
{
	void *f;
	f=(void*)getfunc("glGetColorTableSGI");
	ov_glGetColorTableSGI=f;
	ov_glGetColorTableSGI(param0,param1,param2,param3);
}
void (APIENTRY *ov_glGetColorTableSGI)(unsigned int param0,unsigned int param1,unsigned int param2,void* param3)=init_ov_glGetColorTableSGI;

void __gl_ml_glGetColorTableSGI(value v0,value v1,value v2,value v3)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	unsigned int l2=Int_val(v2);
	void* l3=gl__ba__(v3);
	ov_glGetColorTableSGI(l0,l1,l2,l3);
}
extern void (APIENTRY *ov_glCopyColorTableSGI)(unsigned int param0,unsigned int param1,signed int param2,signed int param3,signed int param4);

static void APIENTRY init_ov_glCopyColorTableSGI(unsigned int param0,unsigned int param1,signed int param2,signed int param3,signed int param4)
{
	void *f;
	f=(void*)getfunc("glCopyColorTableSGI");
	ov_glCopyColorTableSGI=f;
	ov_glCopyColorTableSGI(param0,param1,param2,param3,param4);
}
void (APIENTRY *ov_glCopyColorTableSGI)(unsigned int param0,unsigned int param1,signed int param2,signed int param3,signed int param4)=init_ov_glCopyColorTableSGI;

void __gl_ml_glCopyColorTableSGI(value v0,value v1,value v2,value v3,value v4)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	signed int l3=Int_val(v3);
	signed int l4=Int_val(v4);
	ov_glCopyColorTableSGI(l0,l1,l2,l3,l4);
}
extern void (APIENTRY *ov_glColorTableParameterivSGI)(unsigned int param0,unsigned int param1,signed int* param2);

static void APIENTRY init_ov_glColorTableParameterivSGI(unsigned int param0,unsigned int param1,signed int* param2)
{
	void *f;
	f=(void*)getfunc("glColorTableParameterivSGI");
	ov_glColorTableParameterivSGI=f;
	ov_glColorTableParameterivSGI(param0,param1,param2);
}
void (APIENTRY *ov_glColorTableParameterivSGI)(unsigned int param0,unsigned int param1,signed int* param2)=init_ov_glColorTableParameterivSGI;

void __gl_ml_glColorTableParameterivSGI(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int* l2=(signed int*)(gl__ba__(v2));
	ov_glColorTableParameterivSGI(l0,l1,l2);
}
extern void (APIENTRY *ov_glColorTableParameterfvSGI)(unsigned int param0,unsigned int param1,float* param2);

static void APIENTRY init_ov_glColorTableParameterfvSGI(unsigned int param0,unsigned int param1,float* param2)
{
	void *f;
	f=(void*)getfunc("glColorTableParameterfvSGI");
	ov_glColorTableParameterfvSGI=f;
	ov_glColorTableParameterfvSGI(param0,param1,param2);
}
void (APIENTRY *ov_glColorTableParameterfvSGI)(unsigned int param0,unsigned int param1,float* param2)=init_ov_glColorTableParameterfvSGI;

void __gl_ml_glColorTableParameterfvSGI(value v0,value v1,value v2)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	float* l2=(float*)(gl__ba__(v2));
	ov_glColorTableParameterfvSGI(l0,l1,l2);
}
extern void (APIENTRY *ov_glColorTableSGI)(unsigned int param0,unsigned int param1,signed int param2,unsigned int param3,unsigned int param4,void* param5);

static void APIENTRY init_ov_glColorTableSGI(unsigned int param0,unsigned int param1,signed int param2,unsigned int param3,unsigned int param4,void* param5)
{
	void *f;
	f=(void*)getfunc("glColorTableSGI");
	ov_glColorTableSGI=f;
	ov_glColorTableSGI(param0,param1,param2,param3,param4,param5);
}
void (APIENTRY *ov_glColorTableSGI)(unsigned int param0,unsigned int param1,signed int param2,unsigned int param3,unsigned int param4,void* param5)=init_ov_glColorTableSGI;

void __gl_ml_glColorTableSGI(value v0,value v1,value v2,value v3,value v4,value v5)
{
	unsigned int l0=Int_val(v0);
	unsigned int l1=Int_val(v1);
	signed int l2=Int_val(v2);
	unsigned int l3=Int_val(v3);
	unsigned int l4=Int_val(v4);
	void* l5=gl__ba__(v5);
	ov_glColorTableSGI(l0,l1,l2,l3,l4,l5);
}

void byte___gl_ml_glColorTableSGI(value * argv,int argn)
{
	__gl_ml_glColorTableSGI(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5]);
}
extern void* (APIENTRY *ov_AllocateMemoryNV)(int,float,float,float);
static void* APIENTRY init_ov_AllocateMemoryNV(int a,float b,float c,float d)
{
    void* f;
#ifdef _WIN32    
    f=(void*)getfunc("wglAllocateMemoryNV");
#elif __linux__
    f=(void*)getfunc("glXAllocateMemoryNV");
#endif
    ov_AllocateMemoryNV=f;
    return ov_AllocateMemoryNV(a,b,c,d);
}
void* (APIENTRY *ov_AllocateMemoryNV)(int,float,float,float)=init_ov_AllocateMemoryNV;

extern void (APIENTRY *ov_FreeMemoryNV)(void*);
static void APIENTRY init_ov_FreeMemoryNV(void* a)
{
    void* f;
#ifdef _WIN32    
    f=(void*)getfunc("wglFreeMemoryNV");
#elif __linux__
    f=(void*)getfunc("glXFreeMemoryNV");
#endif
    ov_FreeMemoryNV=f;
    return ov_FreeMemoryNV(a);
}
void (APIENTRY *ov_FreeMemoryNV)(void*)=init_ov_FreeMemoryNV;

value __gl_ml_AllocateMemoryNV(value a,value b,value c,value d)
{
    CAMLparam4(a,b,c,d);
    CAMLlocal1(r);
    void* v;
    v=ov_AllocateMemoryNV(Int_val(a),Double_val(b),Double_val(c),Double_val(d));
    r=alloc_bigarray_dims(BIGARRAY_UINT8|BIGARRAY_C_LAYOUT,1,v,v?Int_val(a):0);
    CAMLreturn(r);
}

void __gl_ml_FreeMemoryNV(value a)
{
    CAMLparam1(a);
    ov_FreeMemoryNV(Data_bigarray_val(a));
    CAMLreturn0;
}
